@@ -1,49 +1,10 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Calendar views code
- *
- * The Initial Developer of the Original Code is
- *   the Mozilla Calendar Squad
- * Portions created by the Initial Developer are Copyright (C) 2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir.vukicevic@oracle.com>
- *   Joey Minta <jminta@gmail.com>
- *   Michael Buettner <michael.buettner@sun.com>
- *   gekacheka@yahoo.com
- *   Matthew Willis <lilmatt@mozilla.com>
- *   Philipp Kewisch <mozilla@kewis.ch>
- *   Martin Schroeder <mschroeder@mozilla.x-home.org>
- *   Berend Cornelius <berend.cornelius@sun.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://calendar/modules/calAlarmUtils.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 /**
  * Controller for the views
@@ -66,6 +27,7 @@ var calendarViewController = {
     createNewEvent: function (aCalendar, aStartTime, aEndTime, aForceAllday) {
         aCalendar = aCalendar || getSelectedCalendar();
 
+
         // if we're given both times, skip the dialog
         if (aStartTime && aEndTime && !aStartTime.isDate && !aEndTime.isDate) {
             let item = cal.createEvent();
@@ -150,13 +112,18 @@ var calendarViewController = {
      * @see calICalendarViewController
      */
     modifyOccurrence: function (aOccurrence, aNewStartTime, aNewEndTime, aNewTitle) {
+        let dlg = cal.findItemWindow(aOccurrence);
+        if (dlg) {
+            dlg.focus();
+            return;
+        }
 
         aOccurrence = this.finalizePendingModification(aOccurrence);
 
         // if modifying this item directly (e.g. just dragged to new time),
         // then do so; otherwise pop up the dialog
         if (aNewStartTime || aNewEndTime || aNewTitle) {
-            var instance = aOccurrence.clone();
+            let instance = aOccurrence.clone();
 
             if (aNewTitle) {
                 instance.title = aNewTitle;
@@ -320,10 +287,12 @@ function switchToView(aViewType) {
      */
     function setupViewNode(id, attr) {
         let node = document.getElementById(id);
-        if (node.hasAttribute(attr + "-" + aViewType)) {
-            node.setAttribute(attr, node.getAttribute(attr + "-" + aViewType));
-        } else {
-            node.setAttribute(attr, node.getAttribute(attr + "-all"));
+        if (node) {
+            if (node.hasAttribute(attr + "-" + aViewType)) {
+                node.setAttribute(attr, node.getAttribute(attr + "-" + aViewType));
+            } else {
+                node.setAttribute(attr, node.getAttribute(attr + "-all"));
+            }
         }
     }
 
@@ -331,7 +300,9 @@ function switchToView(aViewType) {
     ["calendar-view-context-menu-next",
      "calendar-view-context-menu-previous",
      "calendar-go-menu-next",
-     "calendar-go-menu-previous"].forEach(function(x) {
+     "calendar-go-menu-previous",
+     "appmenu_calendar-go-menu-next",
+     "appmenu_calendar-go-menu-previous"].forEach(function(x) {
             setupViewNode(x, "label");
             setupViewNode(x, "accesskey")
      });
@@ -446,14 +417,12 @@ function scheduleMidnightUpdate(aRefreshCallback) {
         };
 
         // Add observer
-        var observerService = Components.classes["@mozilla.org/observer-service;1"]
-                                        .getService(Components.interfaces.nsIObserverService);
-        observerService.addObserver(wakeObserver, "wake_notification", false);
+        Services.obs.addObserver(wakeObserver, "wake_notification", false);
 
         // Remove observer on unload
         window.addEventListener("unload",
                                 function() {
-                                    observerService.removeObserver(wakeObserver, "wake_notification");
+                                    Services.obs.removeObserver(wakeObserver, "wake_notification");
                                 }, false);
         gMidnightTimer = Components.classes["@mozilla.org/timer;1"]
                                    .createInstance(Components.interfaces.nsITimer);
@@ -526,10 +495,7 @@ var categoryManagement = {
     },
 
     initCategories: function cM_initCategories() {
-      let prefService = Components.classes["@mozilla.org/preferences-service;1"]
-                                  .getService(Components.interfaces.nsIPrefService);
-      categoryPrefBranch = prefService.getBranch("calendar.category.color.")
-                                      .QueryInterface(Components.interfaces.nsIPrefBranch2);
+      categoryPrefBranch = Services.prefs.getBranch("calendar.category.color.");
       let categories = categoryPrefBranch.getChildList("");
 
       // Fix illegally formatted category prefs.
@@ -555,10 +521,7 @@ var categoryManagement = {
     },
 
     cleanupCategories: function cM_cleanupCategories() {
-      let prefService = Components.classes["@mozilla.org/preferences-service;1"]
-                                  .getService(Components.interfaces.nsIPrefService);
-      categoryPrefBranch = prefService.getBranch("calendar.category.color.")
-                                      .QueryInterface(Components.interfaces.nsIPrefBranch2);
+      categoryPrefBranch = Services.prefs.getBranch("calendar.category.color.");
       categoryPrefBranch.removeObserver("", categoryManagement);
     },
 
@@ -833,3 +796,21 @@ cal.navigationBar = {
         viewTabs.selectedIndex = getViewDeck().selectedIndex;
     }
 };
+
+/*
+ * Timer for the time indicator in day and week view.
+ */
+var timeIndicator = {
+    timer: null,
+    start: function(aInterval, aThis) {
+        timeIndicator.timer = setInterval(function() {aThis.updateTimeIndicatorPosition(false);},
+                                          aInterval * 1000);
+    },
+    cancel: function() {
+        if (timeIndicator.timer) {
+            clearTimeout(timeIndicator.timer);
+            timeIndicator.timer = null;
+        }
+    },
+    lastView: null
+}
