@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+
 var gStartDate = null;
 var gEndDate = null;
 var gStartTimezone = null;
@@ -25,11 +27,15 @@ function onLoad() {
     window.addEventListener("resize", onResize, true);
     window.addEventListener("modify", onModify, true);
     window.addEventListener("rowchange", onRowChange, true);
-    window.addEventListener("DOMMouseScroll", onMouseScroll, true);
     window.addEventListener("DOMAttrModified", onAttrModified, true);
     window.addEventListener("timebar", onTimebar, true);
     window.addEventListener("timechange", onTimeChange, true);
 
+    // As long as DOMMouseScroll is still implemented, we need to keep it
+    // around to make sure scrolling is blocked.
+    window.addEventListener("wheel", onMouseScroll, true);
+    window.addEventListener("DOMMouseScroll", onMouseScroll, true);
+
     var args = window.arguments[0];
     var startTime = args.startTime;
     var endTime = args.endTime;
@@ -39,24 +45,6 @@ function onLoad() {
 
     onChangeCalendar(calendar);
 
-    // we need to enforce several layout constraints which can't be modelled
-    // with plain xul and css, at least as far as i know.
-    const kStylesheet = "chrome://calendar/skin/calendar-event-dialog.css";
-    for each (var stylesheet in document.styleSheets) {
-        if (stylesheet.href == kStylesheet) {
-            // make the dummy-spacer #1 [top] the same height as the timebar
-            var timebar = document.getElementById("timebar");
-            stylesheet.insertRule(
-                ".attendee-spacer-top { height: "
-                    + timebar.boxObject.height+"px; }", 0);
-            // make the dummy-spacer #2 [bottom] the same height as the scrollbar
-            var scrollbar = document.getElementById("horizontal-scrollbar");
-            stylesheet.insertRule(
-                ".attendee-spacer-bottom { height: "
-                    + scrollbar.boxObject.height+"px; }", 0);
-            break;
-        }
-    }
 
     let zoom = document.getElementById("zoom-menulist");
     let zoomOut = document.getElementById("zoom-out-button");
@@ -86,9 +74,29 @@ function onLoad() {
 
     loadDateTime(startTime, endTime);
     propagateDateTime();
-
+    // Set the scroll bar at where the event is
+    scrollToCurrentTime();
     updateButtons();
 
+    // we need to enforce several layout constraints which can't be modelled
+    // with plain xul and css, at least as far as i know.
+    const kStylesheet = "chrome://calendar/skin/calendar-event-dialog.css";
+    for each (var stylesheet in document.styleSheets) {
+        if (stylesheet.href == kStylesheet) {
+            // make the dummy-spacer #1 [top] the same height as the timebar
+            var timebar = document.getElementById("timebar");
+            stylesheet.insertRule(
+                ".attendee-spacer-top { height: "
+                    + timebar.boxObject.height+"px; }", 0);
+            // make the dummy-spacer #2 [bottom] the same height as the scrollbar
+            var scrollbar = document.getElementById("horizontal-scrollbar");
+            stylesheet.insertRule(
+                ".attendee-spacer-bottom { height: "
+                    + scrollbar.boxObject.height+"px; }", 0);
+            break;
+        }
+    }
+
     // attach an observer to get notified of changes
     // that are relevant to this dialog.
     var prefObserver = {
@@ -102,12 +110,10 @@ function onLoad() {
             }
         }
     }
-    var pb2 = Components.classes["@mozilla.org/preferences-service;1"].
-              getService(Components.interfaces.nsIPrefBranch);
-    pb2.addObserver("calendar.", prefObserver, false);
+    Services.prefs.addObserver("calendar.", prefObserver, false);
     window.addEventListener("unload",
         function() {
-            pb2.removeObserver("calendar.", prefObserver);
+            Services.prefs.removeObserver("calendar.", prefObserver);
         },
         false);
 
@@ -330,23 +336,24 @@ function updateStartTime() {
         return;
     }
 
-    var startWidgetId = "event-starttime";
-    var endWidgetId = "event-endtime";
+    let startWidgetId = "event-starttime";
+    let endWidgetId = "event-endtime";
 
-    var startWidget = document.getElementById(startWidgetId);
-    var endWidget = document.getElementById(endWidgetId);
+    let startWidget = document.getElementById(startWidgetId);
+    let endWidget = document.getElementById(endWidgetId);
 
     // jsDate is always in OS timezone, thus we create a calIDateTime
     // object from the jsDate representation and simply set the new
     // timezone instead of converting.
-    var start = jsDateToDateTime(startWidget.value,
-                                 gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
+    let timezone = gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone();
+    let start = cal.jsDateToDateTime(startWidget.value, timezone);
+                                    
     gStartDate = start.clone();
     start.addDuration(gDuration);
     gEndDate = start.getInTimezone(gEndTimezone);
 
-    var allDayElement = document.getElementById("all-day");
-    var allDay = allDayElement.getAttribute("checked") == "true";
+    let allDayElement = document.getElementById("all-day");
+    let allDay = allDayElement.getAttribute("checked") == "true";
     if (allDay) {
         gStartDate.isDate = true;
         gEndDate.isDate = true;
@@ -373,7 +380,7 @@ function updateEndTime() {
     var saveEndTime = gEndDate;
     var kDefaultTimezone = calendarDefaultTimezone();
 
-    gStartDate = jsDateToDateTime(startWidget.value,
+    gStartDate = cal.jsDateToDateTime(startWidget.value,
                                   gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
 
     var timezone = gEndTimezone;
@@ -382,8 +389,8 @@ function updateEndTime() {
         !compareObjects(gStartTimezone, gEndTimezone)) {
         timezone = gStartTimezone;
     }
-    gEndDate = jsDateToDateTime(endWidget.value,
-                                gDisplayTimezone ? timezone : kDefaultTimezone);
+    gEndDate = cal.jsDateToDateTime(endWidget.value,
+                                    gDisplayTimezone ? timezone : kDefaultTimezone);
 
     var allDayElement = document.getElementById("all-day");
     var allDay = allDayElement.getAttribute("checked") == "true";
@@ -407,12 +414,7 @@ function updateEndTime() {
 
     if (warning) {
         var callback = function() {
-            var promptService =
-                Components.classes[
-                    "@mozilla.org/embedcomp/prompt-service;1"]
-                    .getService(
-                        Components.interfaces.nsIPromptService);
-            promptService.alert(
+            Services.prompt.alert(
                 null,
                 document.title,
                 calGetString("calendar", "warningEndBeforeStart"));
@@ -718,7 +720,7 @@ function onPreviousSlot() {
  */
 function scrollToCurrentTime() {
     var timebar = document.getElementById("timebar");
-    var ratio = (gStartDate.hour - gStartHour) * timebar.step;
+    var ratio = (gStartDate.hour - gStartHour - 1) * timebar.step;
     if (ratio <= 0.0) {
         ratio = 0.0;
     }
@@ -855,7 +857,7 @@ function onRowChange(event) {
 /**
  * Handler function to take care of mouse scrolling on the window
  *
- * @param event     The DOMMouseScroll event caused by scrolling.
+ * @param event     The wheel event caused by scrolling.
  */
 function onMouseScroll(event) {
     // ignore mouse scrolling for now...
