@@ -7,55 +7,40 @@
  * that loading this file twice in the same scope will throw errors.
  */
 
-Components.utils.import("resource:///modules/Services.jsm");
+Components.utils.import("resource:///modules/mailServices.js");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
-/**
- * Returns a clean new calIEvent
- *
- * @param aIcalString       (optional) The icalString to read event details from.
- */
-function createEvent(aIcalString) {
-    let event = Components.classes["@mozilla.org/calendar/event;1"]
-                          .createInstance(Components.interfaces.calIEvent);
-    if (aIcalString) {
-        event.icalString = aIcalString;
-    }
-    return event;
+function _calIcalCreator(cid, iid) {
+    return function(icalString) {
+        let thing = Components.classes[cid].createInstance(iid);
+        if (icalString) {
+            thing.icalString = icalString;
+        }
+        return thing;
+    };
 }
 
-/**
- * Returns a clean new calITodo
- *
- * @param aIcalString       (optional) The icalString to read task details from.
- */
-function createTodo(aIcalString) {
-    let todo = Components.classes["@mozilla.org/calendar/todo;1"]
-                         .createInstance(Components.interfaces.calITodo);
-    if (aIcalString) {
-        todo.icalString = aIcalString;
-    }
-    return todo;
-}
+let createEvent = _calIcalCreator("@mozilla.org/calendar/event;1",
+                                  Components.interfaces.calIEvent);
+let createTodo = _calIcalCreator("@mozilla.org/calendar/todo;1",
+                                 Components.interfaces.calITodo);
+let createDateTime  = _calIcalCreator("@mozilla.org/calendar/datetime;1",
+                                      Components.interfaces.calIDateTime);
+let createDuration = _calIcalCreator("@mozilla.org/calendar/duration;1",
+                                     Components.interfaces.calIDuration);
+let createAttendee = _calIcalCreator("@mozilla.org/calendar/attendee;1",
+                                     Components.interfaces.calIAttendee);
+let createAttachment = _calIcalCreator("@mozilla.org/calendar/attachment;1",
+                                       Components.interfaces.calIAttachment);
+let createAlarm = _calIcalCreator("@mozilla.org/calendar/alarm;1",
+                                  Components.interfaces.calIAlarm);
+let createRelation = _calIcalCreator("@mozilla.org/calendar/relation;1",
+                                     Components.interfaces.calIRelation);
+let createRecurrenceDate = _calIcalCreator("@mozilla.org/calendar/recurrence-date;1",
+                                           Components.interfaces.calIRecurrenceDate);
+let createRecurrenceRule = _calIcalCreator("@mozilla.org/calendar/recurrence-rule;1",
+                                           Components.interfaces.calIRecurrenceRule);
 
-/* Returns a clean new calIDateTime */
-function createDateTime(aIcalString) {
-    var dt = Components.classes["@mozilla.org/calendar/datetime;1"]
-                       .createInstance(Components.interfaces.calIDateTime);
-    if (aIcalString) {
-        dt.icalString = aIcalString;
-    }
-    return dt;
-}
-
-/* Returns a clean new calIDuration */
-function createDuration(aIcalString) {
-    var dur = Components.classes["@mozilla.org/calendar/duration;1"]
-                        .createInstance(Components.interfaces.calIDuration);
-    if (aIcalString) {
-        dur.icalString = aIcalString;
-    }
-    return dur;
-}
 /* Returns a clean new calIRecurrenceInfo */
 function createRecurrenceInfo(aItem) {
     var recInfo = Components.classes["@mozilla.org/calendar/recurrence-info;1"].
@@ -64,60 +49,6 @@ function createRecurrenceInfo(aItem) {
     return recInfo;
 }
 
-/* Returns a clean new calIRecurrenceRule */
-function createRecurrenceRule() {
-    return Components.classes["@mozilla.org/calendar/recurrence-rule;1"].
-           createInstance(Components.interfaces.calIRecurrenceRule);
-}
-
-/* Returns a clean new calIRecurrenceRule */
-function createRecurrenceDate() {
-    return Components.classes["@mozilla.org/calendar/recurrence-date;1"].
-           createInstance(Components.interfaces.calIRecurrenceDate);
-}
-
-/* Returns a clean new calIAttendee */
-function createAttendee() {
-    return Components.classes["@mozilla.org/calendar/attendee;1"].
-           createInstance(Components.interfaces.calIAttendee);
-}
-
-/* Returns a clean new calIAttachment */
-function createAttachment() {
-    return Components.classes["@mozilla.org/calendar/attachment;1"].
-           createInstance(Components.interfaces.calIAttachment);
-}
-
-/* Returns a clean new calIAlarm*/
-function createAlarm() {
-    return Components.classes["@mozilla.org/calendar/alarm;1"].
-           createInstance(Components.interfaces.calIAlarm);
-}
-
-/* Returns a clean new calIRelation */
-function createRelation() {
-    return Components.classes["@mozilla.org/calendar/relation;1"].
-           createInstance(Components.interfaces.calIRelation);
-}
-
-/* Shortcut to the console service */
-function getConsoleService() {
-    return Components.classes["@mozilla.org/consoleservice;1"]
-                     .getService(Components.interfaces.nsIConsoleService);
-}
-
-/* Shortcut to the account manager service */
-function getAccountManager() {
-    return Components.classes["@mozilla.org/messenger/account-manager;1"]
-                     .getService(Components.interfaces.nsIMsgAccountManager);
-}
-
-/* Shortcut to the IO service */
-function getIOService() {
-    return Components.classes["@mozilla.org/network/io-service;1"]
-                     .getService(Components.interfaces.nsIIOService);
-}
-
 /* Shortcut to the calendar-manager service */
 function getCalendarManager() {
     return Components.classes["@mozilla.org/calendar/manager;1"]
@@ -284,9 +215,7 @@ function formatStringForCSSRule(aString) {
  */
 function getCalendarDirectory() {
     if (getCalendarDirectory.mDir === undefined) {
-        var dirSvc = Components.classes["@mozilla.org/file/directory_service;1"]
-                               .getService(Components.interfaces.nsIProperties);
-        var dir = dirSvc.get("ProfD", Components.interfaces.nsILocalFile);
+        var dir = Services.dirsvc.get("ProfD", Components.interfaces.nsILocalFile);
         dir.append("calendar-data");
         if (!dir.exists()) {
             try {
@@ -313,8 +242,9 @@ function getCalendarDirectory() {
 function isCalendarWritable(aCalendar) {
     return (!aCalendar.getProperty("disabled") &&
             !aCalendar.readOnly &&
-            (!getIOService().offline ||
+            (!Services.io.offline ||
              aCalendar.getProperty("cache.enabled") ||
+             aCalendar.getProperty("cache.always") ||
              aCalendar.getProperty("requiresNetwork") === false));
 }
 
@@ -362,7 +292,17 @@ function userCanModifyItem(aItem) {
  * @return              True if there is a match
  */
 function attendeeMatchesAddresses(anAttendee, addresses) {
-    let attId = anAttendee.id.toLowerCase().replace(/^mailto:/, "");
+    let attId = anAttendee.id;
+    if (!attId.match(/^mailto:/i)) {
+        // Looks like its not a normal attendee, possibly urn:uuid:...
+        // Try getting the email through the EMAIL property.
+        let emailProp = anAttendee.getProperty("EMAIL");
+        if (emailProp) {
+            attId = emailProp;
+        }
+    }
+
+    attId = attId.toLowerCase().replace(/^mailto:/, "");
     for each (let address in addresses) {
         if (attId == address.toLowerCase().replace(/^mailto:/, "")) {
             return true;
@@ -427,7 +367,7 @@ function calPrint() {
  * @returns  an nsIURI whose spec is aUriString
  */
 function makeURL(aUriString) {
-    return getIOService().newURI(aUriString, null, null);
+    return Services.io.newURI(aUriString, null, null);
 }
 
 /**
@@ -440,34 +380,6 @@ function now() {
     return d.getInTimezone(calendarDefaultTimezone());
 }
 
-/**
- * Returns a calIDateTime corresponding to a javascript Date.
- *
- * @param aDate     a javascript date
- * @param aTimezone (optional) a timezone that should be enforced
- * @returns         a calIDateTime
- *
- * @warning  Use of this function is strongly discouraged.  calIDateTime should
- *           be used directly whenever possible.
- *           If you pass a timezone, then the passed jsDate's timezone will be ignored,
- *           but only its local time portions are be taken.
- */
-function jsDateToDateTime(aDate, aTimezone) {
-    var newDate = createDateTime();
-    if (aTimezone) {
-        newDate.resetTo(aDate.getFullYear(),
-                        aDate.getMonth(),
-                        aDate.getDate(),
-                        aDate.getHours(),
-                        aDate.getMinutes(),
-                        aDate.getSeconds(),
-                        aTimezone);
-    } else {
-        newDate.jsDate = aDate;
-    }
-    return newDate;
-}
-
 /**
  * Selects an item with id aItemId in the radio group with id aRadioGroupId
  *
@@ -553,8 +465,7 @@ function isToDo(aObject) {
  */
 function getPrefSafe(aPrefName, aDefault) {
     const nsIPrefBranch = Components.interfaces.nsIPrefBranch;
-    const prefB = Components.classes["@mozilla.org/preferences-service;1"]
-                            .getService(nsIPrefBranch);
+    const prefB = Services.prefs;
     // Since bug 193332 does not fix the current branch, calling get*Pref will
     // throw NS_ERROR_UNEXPECTED if clearUserPref() was called and there is no
     // default value. To work around that, catch the exception.
@@ -596,17 +507,15 @@ function setPref(aPrefName, aPrefValue, aPrefType) {
                 break;
         }
     }
-    let prefB = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefBranch);
     switch (aPrefType) {
         case "BOOL":
-            prefB.setBoolPref(aPrefName, aPrefValue);
+            Services.prefs.setBoolPref(aPrefName, aPrefValue);
             break;
         case "INT":
-            prefB.setIntPref(aPrefName, aPrefValue);
+            Services.prefs.setIntPref(aPrefName, aPrefValue);
             break;
         case "CHAR":
-            prefB.setCharPref(aPrefName, aPrefValue);
+            Services.prefs.setCharPref(aPrefName, aPrefValue);
             break;
     }
 }
@@ -618,12 +527,10 @@ function setPref(aPrefName, aPrefValue, aPrefType) {
  * @param aString     the string to which the preference value should be set
  */
 function setLocalizedPref(aPrefName, aString) {
-    const prefB = Components.classes["@mozilla.org/preferences-service;1"].
-                  getService(Components.interfaces.nsIPrefBranch);
     var str = Components.classes["@mozilla.org/supports-string;1"].
               createInstance(Components.interfaces.nsISupportsString);
     str.data = aString;
-    prefB.setComplexValue(aPrefName, Components.interfaces.nsISupportsString, str);
+    Services.prefs.setComplexValue(aPrefName, Components.interfaces.nsISupportsString, str);
 }
 
 /**
@@ -633,11 +540,9 @@ function setLocalizedPref(aPrefName, aString) {
  * @param aDefault    (optional) the value to return if the pref is undefined
  */
 function getLocalizedPref(aPrefName, aDefault) {
-    const pb2 = Components.classes["@mozilla.org/preferences-service;1"].
-                getService(Components.interfaces.nsIPrefBranch);
     var result;
     try {
-        result = pb2.getComplexValue(aPrefName, Components.interfaces.nsISupportsString).data;
+        result = Services.prefs.getComplexValue(aPrefName, Components.interfaces.nsISupportsString).data;
     } catch(ex) {
         return aDefault;
     }
@@ -754,18 +659,6 @@ function categoriesArrayToString(aSortedCategoriesArray) {
     return aSortedCategoriesArray.map(escapeComma).join(",");
 }
 
-/**
- * Creates a string bundle.
- *
- * @param bundleURL The bundle URL
- * @return string bundle
- */
-function calGetStringBundle(bundleURL) {
-    let service = Components.classes["@mozilla.org/intl/stringbundle;1"]
-                            .getService(Components.interfaces.nsIStringBundleService);
-    return service.createBundle(bundleURL);
-}
-
 /**
  * Gets the value of a string in a .properties file from the calendar bundle
  *
@@ -781,7 +674,7 @@ function calGetString(aBundleName, aStringName, aParams, aComponent) {
             aComponent = "calendar";
         }
         var propName = "chrome://" + aComponent + "/locale/" + aBundleName + ".properties";
-        var props = calGetStringBundle(propName);
+        var props = Services.strings.createBundle(propName);
 
         if (aParams && aParams.length) {
             return props.formatStringFromName(aStringName, aParams, aParams.length);
@@ -1020,11 +913,9 @@ function setDefaultStartEndHour(aItem, aReferenceDate) {
  *              properties should be logged.
  */
 function LOG(aArg) {
-    var prefB = Components.classes["@mozilla.org/preferences-service;1"].
-                getService(Components.interfaces.nsIPrefBranch);
     var shouldLog = false;
     try {
-        shouldLog = prefB.getBoolPref("calendar.debug.log");
+        shouldLog = Services.prefs.getBoolPref("calendar.debug.log");
     } catch(ex) {}
 
     if (!shouldLog) {
@@ -1045,7 +936,7 @@ function LOG(aArg) {
 
     // xxx todo consider using function debug()
     dump(string + '\n');
-    getConsoleService().logStringMessage(string);
+    Services.console.logStringMessage(string);
 }
 
 /**
@@ -1060,7 +951,7 @@ function WARN(aMessage) {
     scriptError.init(aMessage, null, null, 0, 0,
                      Components.interfaces.nsIScriptError.warningFlag,
                      "component javascript");
-    getConsoleService().logMessage(scriptError);
+    Services.console.logMessage(scriptError);
 }
 
 /**
@@ -1075,7 +966,7 @@ function ERROR(aMessage) {
     scriptError.init(aMessage, null, null, 0, 0,
                      Components.interfaces.nsIScriptError.errorFlag,
                      "component javascript");
-    getConsoleService().logMessage(scriptError);
+    Services.console.logMessage(scriptError);
 }
 
 /**
@@ -1132,9 +1023,7 @@ function ASSERT(aCondition, aMessage, aCritical) {
 function showError(aMsg) {
     let window = window || null;
     if (window) {
-        let promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                      .getService(Components.interfaces.nsIPromptService);
-        promptService.alert(window, calGetString("calendar", "genericErrorTitle"), aMsg);
+        Services.prompt.alert(window, calGetString("calendar", "genericErrorTitle"), aMsg);
     }
 }
 
@@ -1288,12 +1177,14 @@ function getProgressAtom(aTask) {
 /**
  * Returns true if we are Sunbird (according to our UUID), false otherwise.
  */
-function isSunbird()
-{
+function isSunbird() {
     if (isSunbird.mIsSunbird === undefined) {
-        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
-                                .getService(Components.interfaces.nsIXULAppInfo);
-        isSunbird.mIsSunbird = (appInfo.ID == "{718e30fb-e89b-41dd-9da7-e25a45638b28}");
+        try {
+            isSunbird.mIsSunbird = (Services.appinfo.ID == "{718e30fb-e89b-41dd-9da7-e25a45638b28}");
+        } catch (e) {
+            dump("### Warning: Could not access appinfo, using unreliable check for Lightning\n");
+            isSunbird.mIsSunbird = !("@mozilla.org/lightning/mime-converter;1" in Components.classes);
+        }
     }
     return isSunbird.mIsSunbird;
 }
@@ -1374,9 +1265,8 @@ calListenerBag.prototype = {
         function notifyFunc(iface) {
             try {
                 iface[func].apply(iface, args ? args : []);
-            }
-            catch (exc) {
-                Components.utils.reportError(exc + " STACK: " + STACK());
+            } catch (exc) {
+                Components.utils.reportError(exc + "\nSTACK: " + exc.stack);
             }
         }
         this.mInterfaces.forEach(notifyFunc);
@@ -1384,51 +1274,20 @@ calListenerBag.prototype = {
 };
 
 function sendMailTo(aRecipient, aSubject, aBody) {
+    let msgParams = Components.classes["@mozilla.org/messengercompose/composeparams;1"]
+                              .createInstance(Components.interfaces.nsIMsgComposeParams);
+    let composeFields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
+                                  .createInstance(Components.interfaces.nsIMsgCompFields);
 
-    if (Components.classes["@mozilla.org/messengercompose;1"]) {
-        // We are in Thunderbird, we can use the compose interface directly
-        var msgComposeService = Components.classes["@mozilla.org/messengercompose;1"]
-                                .getService(Components.interfaces.nsIMsgComposeService);
-        var msgParams = Components.classes["@mozilla.org/messengercompose/composeparams;1"]
-                        .createInstance(Components.interfaces.nsIMsgComposeParams);
-        var composeFields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
-                            .createInstance(Components.interfaces.nsIMsgCompFields);
+    composeFields.to = aRecipient;
+    composeFields.subject = aSubject;
+    composeFields.body = aBody;
 
-        composeFields.to = aRecipient;
-        composeFields.subject = aSubject;
-        composeFields.body = aBody;
+    msgParams.type = Components.interfaces.nsIMsgCompType.New;
+    msgParams.format = Components.interfaces.nsIMsgCompFormat.Default;
+    msgParams.composeFields = composeFields;
 
-        msgParams.type = Components.interfaces.nsIMsgCompType.New;
-        msgParams.format = Components.interfaces.nsIMsgCompFormat.Default;
-        msgParams.composeFields = composeFields;
-
-        msgComposeService.OpenComposeWindowWithParams(null, msgParams);
-    } else {
-        // We are in a place without a composer. Use the external protocol
-        // service.
-        var protoSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
-                       .getService(Components.interfaces.nsIExternalProtocolService);
-
-        var uriString = "mailto:";
-        var uriParams = [];
-        if (aRecipient) {
-            uriString += aRecipient;
-        }
-
-        if (aSubject) {
-            uriParams.push("subject=" + encodeURIComponent(aSubject));
-        }
-
-        if (aBody) {
-            uriParams.push("body=" + encodeURIComponent(aSubject));
-        }
-
-        if (uriParams.length > 0) {
-            uriString += "?" + uriParams.join("&");
-        }
-
-        protoSvc.loadUrl(makeURL(uriString));
-    }
+    MailServices.compose.OpenComposeWindowWithParams(null, msgParams);
 }
 
 /**
@@ -1878,12 +1737,12 @@ calPropertyBagEnumerator.prototype = {
  * If the called function returns false, iteration is stopped.
  */
 function calIterateEmailIdentities(func) {
-    var accounts = getAccountManager().accounts;
-    for (var i = 0; i < accounts.Count(); ++i) {
-        var account = accounts.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgAccount);
+    var accounts = MailServices.accounts.accounts;
+    for (var i = 0; i < accounts.length; ++i) {
+        var account = accounts.queryElementAt(i, Components.interfaces.nsIMsgAccount);
         var identities = account.identities;
-        for (var j = 0; j < identities.Count(); ++j) {
-            var identity = identities.GetElementAt(j).QueryInterface(Components.interfaces.nsIMsgIdentity);
+        for (var j = 0; j < identities.length; ++j) {
+            var identity = identities.queryElementAt(j, Components.interfaces.nsIMsgIdentity);
             if (!func(identity, account)) {
                 break;
             }
