@@ -14,9 +14,7 @@ function getDBConnection()
   let dbFile = Services.dirsvc.get(NS_APP_USER_PROFILE_50_DIR, Ci.nsIFile);
   dbFile.append("blist.sqlite");
 
-  let conn =
-    Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService)
-                                        .openDatabase(dbFile);
+  let conn = Services.storage.openDatabase(dbFile);
   if (!conn.connectionReady)
     throw Cr.NS_ERROR_UNEXPECTED;
 
@@ -434,12 +432,7 @@ Contact.prototype = {
     if (!this.hasTag(aTag) || this._isTagInherited(aTag))
       return;
 
-    let statement = DBConn.createStatement("DELETE FROM contact_tag " +
-                                           "WHERE contact_id = :contactId " +
-                                           "AND tag_id = :tagId");
-    statement.params.contactId = this.id;
-    statement.params.tagId = aTag.id;
-    statement.executeAsync();
+    this._removeContactTagRow(aTag);
 
     this._tags = this._tags.filter(function(tag) tag.id != aTag.id);
     aTag = TagsById[aTag.id];
@@ -450,6 +443,14 @@ Contact.prototype = {
       observer.observe(aTag, "contact-moved-out", null);
     Services.obs.notifyObservers(this, "contact-tag-removed", aTag.id);
   },
+  _removeContactTagRow: function(aTag) {
+    let statement = DBConn.createStatement("DELETE FROM contact_tag " +
+                                           "WHERE contact_id = :contactId " +
+                                           "AND tag_id = :tagId");
+    statement.params.contactId = this.id;
+    statement.params.tagId = aTag.id;
+    statement.executeAsync();
+  },
   hasTag: function(aTag) this._tags.some(function (t) t.id == aTag.id),
   _massMove: false,
   removeTag: function(aTag) {
@@ -1346,8 +1347,14 @@ ContactsService.prototype = {
     statement.params.newTagId = aNewTag.id;
     statement.execute();
 
+    let contact = ContactsById[buddy.contact.id];
+
+    // aNewTag is now inherited by the contact from an account buddy, so avoid
+    // keeping direct tag <-> contact links in the contact_tag table.
+    contact._removeContactTagRow(aNewTag);
+
     buddy.observe(aAccountBuddy, "account-buddy-moved");
-    ContactsById[buddy.contact.id]._moved(aOldTag, aNewTag);
+    contact._moved(aOldTag, aNewTag);
   },
 
   storeAccount: function(aId, aUserName, aPrplId) {
