@@ -30,7 +30,6 @@ Cu.import("resource:///modules/socket.jsm");
  *              address (e.g. 3ffe:1900:4545:3:200:f8ff:fe21:67cf).
  */
 function ircMessage(aData) {
-  LOG(aData);
   let message = {rawMessage: aData};
   let temp, prefix;
 
@@ -42,58 +41,61 @@ function ircMessage(aData) {
   //     <parameter>: /[^ ]+/
   //     <last parameter>: /.+/
   // See http://joshualuckers.nl/2010/01/10/regular-expression-to-match-raw-irc-messages/
-  if ((temp = aData.match(/^(?::([^ ]+) )?([^ ]+)(?: ((?:[^: ][^ ]* ?)*))?(?: ?:(.*))?$/))) {
-    // Assume message is from the server if not specified
-    prefix = temp[1];
-    message.command = temp[2];
-    // Space separated parameters
-    message.params = temp[3] ? temp[3].trim().split(/ +/) : [];
-    // Last parameter can contain spaces or be an empty string.
-    if (temp[4] != undefined)
-      message.params.push(temp[4]);
-
-    // The source string can be split into multiple parts as:
-    //   :(server|nickname[[!user]@host]
-    // If the source contains a . or a :, assume it's a server name. See RFC
-    // 2812 Section 2.3 definition of servername vs. nickname.
-    if (prefix &&
-        (temp = prefix.match(/^([^ !@\.:]+)(?:!([^ @]+))?(?:@([^ ]+))?$/))) {
-      message.nickname = temp[1];
-      message.user = temp[2] || null; // Optional
-      message.host = temp[3] || null; // Optional
-      if (message.user)
-        message.source = message.user + "@" + message.host;
-      else
-        message.source = message.host; // Note: this can be null!
-    }
-    else if (prefix)
-      message.servername = prefix;
+  // Note that this expression is slightly more aggressive in matching than RFC
+  // 2812 would allow. It allows for empty parameters (besides the last
+  // parameter, which can always be empty), by allowing two spaces in a row.
+  // (This is for compatibility with Unreal's 432 response, which returns an
+  // empty first parameter.) It also allows a trailing space after the
+  // <parameter>s when no <last parameter> is present (also occurs with Unreal).
+  if (!(temp = aData.match(/^(?::([^ ]+) )?([^ ]+)((?: +[^: ][^ ]*)*)? ?(?::([\s\S]*))?$/))) {
+    ERROR("Couldn't parse message: \"" + aData + "\"");
+    return message;
   }
 
+  // Assume message is from the server if not specified
+  prefix = temp[1];
+  message.command = temp[2];
+  // Space separated parameters. Since we expect a space as the first thing
+  // here, we want to ignore the first value (which is empty).
+  message.params = temp[3] ? temp[3].split(" ").slice(1) : [];
+  // Last parameter can contain spaces or be an empty string.
+  if (temp[4] != undefined)
+    message.params.push(temp[4]);
+
+  // The source string can be split into multiple parts as:
+  //   :(server|nickname[[!user]@host]
+  // If the source contains a . or a :, assume it's a server name. See RFC
+  // 2812 Section 2.3 definition of servername vs. nickname.
+  if (prefix &&
+      (temp = prefix.match(/^([^ !@\.:]+)(?:!([^ @]+))?(?:@([^ ]+))?$/))) {
+    message.nickname = temp[1];
+    message.user = temp[2] || null; // Optional
+    message.host = temp[3] || null; // Optional
+    if (message.user)
+      message.source = message.user + "@" + message.host;
+    else
+      message.source = message.host; // Note: this can be null!
+  }
+  else if (prefix)
+    message.servername = prefix;
+
   return message;
 }
 
 // This handles a mode change string for both channels and participants. A mode
 // change string is of the form:
-//   ("+" | "-")<mode key>[<mode key>]*
-// e.g. +iaw or -i
-function _setMode(aNewMode) {
-  // Are we going to add or remove the modes?
-  if (aNewMode[0] != "+" && aNewMode[0] != "-") {
-    WARN("Invalid mode string: " + aNewMode);
-    return;
-  }
-  let addNewMode = aNewMode[0] == "+";
-
-  // Check each mode being added and update the user
-  for (let i = 1; i < aNewMode.length; ++i) {
-    let index = this._modes.indexOf(aNewMode[i]);
+//   aAddNewMode is true if modes are being added, false otherwise.
+//   aNewModes is an array of mode characters.
+function _setMode(aAddNewMode, aNewModes) {
+  // Check each mode being added/removed.
+  for each (let newMode in aNewModes) {
+    let index = this._modes.indexOf(newMode);
     // If the mode is in the list of modes and we want to remove it.
-    if (index != -1 && !addNewMode)
+    if (index != -1 && !aAddNewMode)
       this._modes.splice(index, 1);
     // If the mode is not in the list of modes and we want to add it.
-    else if (index == -1 && addNewMode)
-      this._modes.push(aNewMode[i]);
+    else if (index == -1 && aAddNewMode)
+      this._modes.push(newMode);
   }
 }
 
@@ -115,13 +117,48 @@ const GenericIRCConversation = {
   _waitingForNick: false,
 
   sendMsg: function(aMessage) {
-    this._account.sendMessage("PRIVMSG", [this.name, aMessage]);
+    // Split the message by line breaks and send each one individually.
+    let messages = aMessage.split(/[\r\n]+/);
 
-    // Since the server doesn't send us a message back, just assume the message
-    // was received and immediately show it.
-    this.writeMessage(this._account._nickname, aMessage, {outgoing: true});
+    // Build the shortest possible message that could be sent.
+    let baseMessage = this._account._nickname + this._account.prefix +
+                      " " + this._account.buildMessage("PRIVMSG", this.name) +
+                      " :\r\n";
+    let maxLength =
+      this._account.maxMessageLength - this._account.countBytes(baseMessage);
 
-    this._pendingMessage = true;
+    // Attempt to smartly split a string into multiple lines (based on the
+    // maximum number of characters the message can contain).
+    for (let i = 0; i < messages.length; ++i) {
+      let message = messages[i];
+      let length = this._account.countBytes(message);
+      // The message is short enough.
+      if (length <= maxLength)
+        continue;
+
+      // Find the location of a space before the maximum length.
+      let index = message.lastIndexOf(" ", maxLength);
+
+      // Remove the current message and insert the two new ones. If no space was
+      // found, cut the first message to the maximum length and start the second
+      // message one character after that. If a space was found, exclude it.
+      messages.splice(i, 1, message.substr(0, index == -1 ? maxLength : index),
+                      message.substr((index + 1) || maxLength));
+    }
+
+    // Send each message and display it in the conversation.
+    messages.forEach(function (aMessage) {
+      if (!aMessage.length)
+        return;
+
+      this._account.sendMessage("PRIVMSG", [this.name, aMessage]);
+
+      // Since the server doesn't send us a message back, just assume the
+      // message was received and immediately show it.
+      this.writeMessage(this._account._nickname, aMessage, {outgoing: true});
+
+      this._pendingMessage = true;
+    }, this);
   },
 
   requestBuddyInfo: function(aNick) {
@@ -167,11 +204,16 @@ function ircChannel(aAccount, aName, aNick) {
   this._init(aAccount, aName, aNick);
   this._modes = [];
   this._observedNicks = [];
+  this.banMasks = [];
 }
 ircChannel.prototype = {
   __proto__: GenericConvChatPrototype,
   _modes: [],
   _receivedInitialMode: false,
+  // For IRC you're not in a channel until the JOIN command is received, open
+  // all channels (initially) as left.
+  _left: true,
+  banMasks: [],
 
   // Overwrite the writeMessage function to apply CTCP formatting before
   // display.
@@ -181,6 +223,11 @@ ircChannel.prototype = {
                                                aProperties);
   },
 
+  // Stores the prplIChatRoomFieldValues required to join this channel
+  // to enable later reconnections. If absent, the MUC will not be reconnected
+  // automatically after disconnections.
+  _chatRoomFields: null,
+
   // Section 3.2.2 of RFC 2812.
   part: function(aMessage) {
     let params = [this.name];
@@ -192,6 +239,9 @@ ircChannel.prototype = {
       params.push(msg);
 
     this._account.sendMessage("PART", params);
+
+    // Remove reconnection information.
+    delete this._chatRoomFields;
   },
 
   close: function() {
@@ -282,14 +332,117 @@ ircChannel.prototype = {
     this._participants = {};
   },
 
-  setMode: function(aNewMode, aMessage) {
-    _setMode.call(this, aNewMode);
+  setMode: function(aNewMode, aModeParams, aSetter) {
+    const hostMaskExp = /^.+!.+@.+$/;
+    function getNextParam() {
+      // If there's no next parameter, throw a warning.
+      if (!aModeParams.length) {
+        WARN("Mode parameter expected!");
+        return undefined;
+      }
+      return aModeParams.pop();
+    }
+    function peekNextParam() {
+      // Non-destructively gets the next param.
+      if (!aModeParams.length)
+        return undefined;
+      return aModeParams.slice(-1)[0];
+    }
 
-    // Notify the UI of changes, this message can come from the server or
-    // from a user.
-    let source = aMessage.nickname || aMessage.servername;
-    let msg = _("message.mode", this.name, aNewMode, source);
-    this.writeMessage(source, msg, {system: true});
+    // Are modes being added or removed?
+    if (aNewMode[0] != "+" && aNewMode[0] != "-") {
+      WARN("Invalid mode string: " + aNewMode);
+      return;
+    }
+    let addNewMode = aNewMode[0] == "+";
+
+    // Check each mode being added and update the user.
+    let channelModes = [];
+    let userModes = {};
+    let msg;
+
+    for (let i = aNewMode.length - 1; i > 0; --i) {
+      // Since some modes are conflicted between different server
+      // implementations, check if a participant with that name exists. If this
+      // is true, then update the mode of the ConvChatBuddy.
+      if (this._account.memberStatuses.indexOf(aNewMode[i]) != -1 &&
+          aModeParams.length && this.hasParticipant(peekNextParam())) {
+        // Store the new modes for this nick (so each participant's mode is only
+        // updated once).
+        let nick = this._account.normalize(getNextParam());
+        if (!hasOwnProperty(userModes, nick))
+          userModes[nick] = [];
+        userModes[nick].push(aNewMode[i]);
+
+        // Don't use this mode as a channel mode.
+        continue;
+      }
+      else if (aNewMode[i] == "k") {
+        // Channel key.
+        let newFields = this.name;
+        if (addNewMode) {
+          let key = getNextParam();
+          // A new channel key was set.
+          msg = _("message.channelKeyAdded", aSetter, key);
+          newFields += " " + key;
+        }
+        else
+          msg = _("message.channelKeyRemoved", aSetter);
+
+        this.writeMessage(aSetter, msg, {system: true});
+        // Store the new fields for reconnect.
+        this._chatRoomFields =
+          this._account.getChatRoomDefaultFieldValues(newFields);
+      }
+      else if (aNewMode[i] == "b") {
+        // A banmask was added or removed.
+        let banMask = getNextParam();
+        let msgKey = "message.banMask";
+        if (addNewMode) {
+          this.banMasks.push(banMask);
+          msgKey += "Added";
+        }
+        else {
+          this.banMasks =
+            this.banMasks.filter(function (aBanMask) banMask != aBanMask);
+          msgKey += "Removed";
+        }
+        this.writeMessage(aSetter, _(msgKey, banMask, aSetter), {system: true});
+      }
+      else if (["e", "I", "l"].indexOf(aNewMode[i]) != -1) {
+        // TODO The following have parameters that must be accounted for.
+        getNextParam();
+      }
+      else if (aNewMode[i] == "R" && aModeParams.length &&
+               peekNextParam().match(hostMaskExp)) {
+        // REOP_LIST takes a mask as a parameter, since R is a conflicted mode,
+        // try to match the parameter. Implemented by IRCNet.
+        // TODO The parameter must be acounted for.
+        getNextParam();
+      }
+      // TODO From RFC 2811: a, i, m, n, q, p, s, r, t, l, e, I.
+
+      // Keep track of the channel modes in the order they were received.
+      channelModes.unshift(aNewMode[i]);
+    }
+
+    if (aModeParams.length)
+      WARN("Unused mode parameters: " + aModeParams.join(", "));
+
+    // Update the mode of each participant.
+    for (let nick in userModes)
+      this.getParticipant(nick).setMode(addNewMode, userModes[nick], aSetter);
+
+    if (!channelModes.length)
+      return;
+
+    // Store the channel modes.
+    _setMode.call(this, addNewMode, channelModes);
+
+    // Notify the UI of changes.
+    msg = _("message.channelmode", aNewMode[0] + channelModes.join(""),
+            aSetter);
+    this.writeMessage(aSetter, msg, {system: true});
     this.checkTopicSettable();
 
     this._receivedInitialMode = true;
@@ -352,11 +505,12 @@ function ircParticipant(aName, aConv) {
 ircParticipant.prototype = {
   __proto__: GenericConvChatBuddyPrototype,
 
-  setMode: function(aNewMode, aSetter) {
-    _setMode.call(this, aNewMode);
+  setMode: function(aAddNewMode, aNewModes, aSetter) {
+    _setMode.call(this, aAddNewMode, aNewModes);
 
     // Notify the UI of changes.
-    let msg = _("message.mode", this.name, aNewMode, aSetter);
+    let msg = _("message.usermode", (aAddNewMode ? "+" : "-") + aNewModes.join(""),
+                this.name, aSetter);
     this._conv.writeMessage(aSetter, msg, {system: true});
     this._conv.notifyObservers(this, "chat-buddy-update");
 
@@ -369,7 +523,8 @@ ircParticipant.prototype = {
   get voiced() this._modes.indexOf("v") != -1,
   get halfOp() this._modes.indexOf("h") != -1,
   get op() this._modes.indexOf("o") != -1,
-  get founder() this._modes.indexOf("n") != -1,
+  get founder()
+    this._modes.indexOf("O") != -1 || this._modes.indexOf("q") != -1,
   get typing() false
 };
 
@@ -450,6 +605,13 @@ ircSocket.prototype = {
       }
     }
 
+    // Low level dequote: replace quote character \020 followed by 0, n, r or
+    // \020 with a \0, \n, \r or \020, respectively. Any other character is
+    // replaced with itself.
+    const lowDequote = {"0": "\0", "n": "\n", "r": "\r", "\x10": "\x10"};
+    aRawMessage = aRawMessage.replace(/\x10./g,
+      function(aStr) lowDequote[aStr[1]] || aStr[1]);
+
     // If nothing handled the message, throw an error.
     if (!ircHandlers.handleMessage(this._account, new ircMessage(aRawMessage)))
       WARN("Unhandled IRC message: " + aRawMessage);
@@ -478,10 +640,10 @@ ircSocket.prototype = {
     this._account.gotDisconnected(Ci.prplIAccount.ERROR_NETWORK_ERROR,
                                   _("connection.error.timeOut"));
   },
-  onCertificationError: function(aSocketInfo, aStatus, aTargetSite) {
-    ERROR("Certification error.");
+  onBadCertificate: function(aNSSErrorMessage) {
+    ERROR("bad certificate: " + aNSSErrorMessage);
     this._account.gotDisconnected(Ci.prplIAccount.ERROR_CERT_OTHER_ERROR,
-                                  _("connection.error.certError"));
+                                  aNSSErrorMessage);
   },
   log: LOG
 };
@@ -498,7 +660,9 @@ ircAccountBuddy.prototype = {
 
   get normalizedName() this._account.normalize(this.userName),
 
-  // Can not send messages to buddies who appear offline.
+  // Allow sending of messages to buddies even if they are not online since IRC
+  // does not always provide status information in a timely fashion. (Note that
+  // this is OK since the server will throw an error if the user is not online.)
   get canSendMessage() this.account.connected,
 
   // Called when the user wants to chat with the buddy.
@@ -522,6 +686,8 @@ function ircAccount(aProtocol, aImAccount) {
   this._isOnQueue = [];
   this.pendingIsOnQueue = [];
   this.whoisInformation = {};
+  this._chatRoomFieldsList = {};
+  this._caps = [];
 }
 ircAccount.prototype = {
   __proto__: GenericAccountPrototype,
@@ -530,12 +696,15 @@ ircAccount.prototype = {
   _MODE_INVISIBLE: 1 << 3, // mode 'i'
   get _mode() 0,
 
+  // Whether the user has successfully authenticated with NickServ.
+  isAuthenticated: false,
   // The nickname stored in the account name.
   _accountNickname: null,
   // The nickname that will be used when connecting.
   _requestedNickname: null,
-
-  get noNewlines() true,
+  // The prefix minus the nick (!user@host) as returned by the server, this is
+  // necessary for guessing message lengths.
+  prefix: null,
 
   get normalizedName() this.normalize(this.name),
 
@@ -550,6 +719,9 @@ ircAccount.prototype = {
   userPrefixes: ["@", "!", "%", "+"],
   // The default prefixes to modes.
   userPrefixToModeMap: {"@": "o", "!": "n", "%": "h", "+": "v"},
+  // Modes that have a nickname parameter and affect a participant. See 4.1
+  // Member Status of RFC 2811.
+  memberStatuses: ["a", "h", "o", "O", "q", "v", "!"],
   channelPrefixes: ["&", "#", "+", "!"], // 1.3 Channels
   channelRestrictionToModeMap: {"@": "s", "*": "p", "=": null}, // 353 RPL_NAMREPLY
 
@@ -630,10 +802,36 @@ ircAccount.prototype = {
       return EmptyEnumerator;
 
     let whoisInformation = this.whoisInformation[nick];
+    if (whoisInformation.serverName && whoisInformation.serverInfo) {
+      whoisInformation.server =
+        _("tooltip.serverValue", whoisInformation.serverName,
+          whoisInformation.serverInfo);
+    }
+
+    // List of the names of the info to actually show in the tooltip and
+    // optionally a transform function to apply to the value. Each field here
+    // maps to tooltip.<fieldname> in irc.properties.
+    // See the various RPL_WHOIS* results for the options.
+    let normalizeBool = function(aBool) _(aBool ? "yes" : "no");
+    const kFields = {
+      realname: null,
+      server: null,
+      connectedFrom: null,
+      registered: normalizeBool,
+      registeredAs: null,
+      secure: normalizeBool,
+      away: null,
+      ircOp: normalizeBool,
+      idleTime: null,
+      channels: null
+    };
+
     let tooltipInfo = [];
-    for (let field in whoisInformation) {
-      if (field != "nick" && field != "offline") {
+    for (let field in kFields) {
+      if (whoisInformation.hasOwnProperty(field) && whoisInformation[field]) {
         let value = whoisInformation[field];
+        if (kFields[field])
+          value = kFields[field](value);
         tooltipInfo.push(new TooltipInfo(_("tooltip." + field), value));
       }
     }
@@ -653,6 +851,28 @@ ircAccount.prototype = {
     if (!hasOwnProperty(this.whoisInformation, nick))
       this.whoisInformation[nick] = {"nick": aNick};
   },
+  // Copies the fields of aFields into the whois table. If the field already
+  // exists, that field is ignored (it is assumed that the first server response
+  // is the most up to date information, as is the case for 312/314). Note that
+  // the whois info for a nick is reset whenever whois information is requested,
+  // so the first response from each whois is recorded.
+  setWhois: function(aNick, aFields) {
+    let nick = this.normalize(aNick, this.userPrefixes);
+    // If the nickname isn't in the list yet, add it.
+    if (!hasOwnProperty(this.whoisInformation, nick))
+      this.whoisInformation[nick] = {};
+
+    // Set non-normalized nickname field.
+    this.whoisInformation[nick]["nick"] = aNick;
+
+    // Set the WHOIS fields, but only the first time a field is set.
+    for (let field in aFields) {
+      if (!this.whoisInformation[nick].hasOwnProperty(field))
+        this.whoisInformation[nick][field] = aFields[field];
+    }
+
+    return true;
+  },
   // Write WHOIS information to a conversation.
   writeWhois: function(aConv, aNick, aTooltipInfo) {
     let nick = this.normalize(aNick);
@@ -855,6 +1075,24 @@ ircAccount.prototype = {
     this._socket.connect(this._server, this._port, this._ssl ? ["ssl"] : []);
   },
 
+  // Functions for keeping track of whether the Client Capabilities is done.
+  // If a cap is to be handled, it should be registered with addCAP, where aCAP
+  // is a "unique" string defining what is being handled. When the cap is done
+  // being handled removeCAP should be called with the same string.
+  _caps: [],
+  _capTimeout: null,
+  addCAP: function(aCAP) {
+    this._caps.push(aCAP);
+  },
+  removeCAP: function(aDoneCAP) {
+    // Remove any reference to the given capability.
+    this._caps = this._caps.filter(function(aCAP) aCAP != aDoneCAP);
+
+    // If no more CAP messages are being handled, notify the server.
+    if (!this._caps.length)
+      this.sendMessage("CAP", "END");
+  },
+
   // Used to wait for a response from the server.
   _quitTimer: null,
   // RFC 2812 Section 3.1.7.
@@ -890,16 +1128,27 @@ ircAccount.prototype = {
 
   createConversation: function(aName) this.getConversation(aName),
 
+  // Temporarily stores the prplIChatRoomFieldValues passed to joinChat for
+  // each channel to enable later reconnections.
+  _chatRoomFieldsList: {},
+
   // aComponents implements prplIChatRoomFieldValues.
   joinChat: function(aComponents) {
     let channel = aComponents.getValue("channel");
+    if (!channel) {
+      ERROR("joinChat called without a channel name.");
+      return;
+    }
     let params = [channel];
-    let password = aComponents.getValue("password");
-    if (password)
-      params.push(password);
+    let key = aComponents.getValue("password");
+    if (key)
+      params.push(key);
+    let defaultName = key ? channel + " " + key : channel;
+    this._chatRoomFieldsList[this.normalize(channel)] =
+      this.getChatRoomDefaultFieldValues(defaultName);
     // Send the join command, but don't log the channel key.
     this.sendMessage("JOIN", params,
-                     "JOIN " + channel + (password ? " <key not logged>" : ""));
+                     "JOIN " + channel + (key ? " <key not logged>" : ""));
   },
 
   chatRoomFields: {
@@ -908,7 +1157,7 @@ ircAccount.prototype = {
   },
 
   parseDefaultChatName: function(aDefaultName) {
-    let params = aDefaultName.split(" ");
+    let params = aDefaultName.trim().split(/\s+/);
     let chatFields = {channel: params[0]};
     if (params.length > 1)
       chatFields.password = params[1];
@@ -981,8 +1230,11 @@ ircAccount.prototype = {
   // This sends a message over the socket and catches any errors. Use
   // aLoggedData to log something different than what is actually sent.
   sendRawMessage: function(aMessage, aLoggedData) {
-    // TODO This should escape any characters that can't be used in IRC (e.g.
-    // \001, \r\n).
+    // Low level quoting, replace \0, \n, \r or \020 with \0200, \020n, \020r or
+    // \020\020, respectively.
+    const lowQuote = {"\0": "0", "\n": "n", "\r": "r", "\x10": "\x10"};
+    const lowRegex = new RegExp("[" + Object.keys(lowQuote).join("") + "]", "g");
+    aMessage = aMessage.replace(lowRegex, function(aChar) "\x10" + lowQuote[aChar]);
 
     if (!this._socket || !this._socket.isConnected) {
       this.gotDisconnected(Ci.prplIAccount.ERROR_NETWORK_ERROR,
@@ -1014,13 +1266,21 @@ ircAccount.prototype = {
   // CTCP messages are \001<COMMAND> [<parameters>]*\001.
   sendCTCPMessage: function(aCommand, aParams, aTarget, aIsNotice) {
     // Combine the CTCP command and parameters into the single IRC param.
-    let ircParam = "\x01" + aCommand;
+    let ircParam = aCommand;
     // If aParams is empty, then use an empty array. If aParams is not an array,
     // consider it to be a single parameter and put it into an array.
     let params = !aParams ? [] : Array.isArray(aParams) ? aParams : [aParams];
     if (params.length)
       ircParam += " " + params.join(" ");
-    ircParam += "\x01";
+
+    // High/CTCP level quoting, replace \134 or \001 with \134\134 or \134a,
+    // respectively. This is only done inside the extended data message.
+    const highQuote = {"\x5C": "\x5C", "\x01": "a"};
+    const highRegex = new RegExp("[" + Object.keys(highQuote).join("") + "]", "g");
+    ircParam = ircParam.replace(highRegex, function(aChar) "\x5C" + highQuote[aChar]);
+
+    // Add the CTCP tagging.
+    ircParam = "\x01" + ircParam + "\x01";
 
     // Send the IRC message as a NOTICE or PRIVMSG.
     this.sendMessage(aIsNotice ? "NOTICE" : "PRIVMSG", [aTarget, ircParam]);
@@ -1028,18 +1288,25 @@ ircAccount.prototype = {
 
   // Implement section 3.1 of RFC 2812
   _connectionRegistration: function() {
-    // Send the password message, if provided (section 3.1.1).
-    if (this.imAccount.password) {
-      this.sendMessage("PASS", this.imAccount.password,
+    // Send the Client Capabilities list command.
+    this.sendMessage("CAP", "LS");
+
+    if (this.prefs.prefHasUserValue("serverPassword")) {
+      this.sendMessage("PASS", this.getString("serverPassword"),
                        "PASS <password not logged>");
     }
+
     // Send the nick message (section 3.1.2).
     this.sendMessage("NICK", this._requestedNickname);
 
     // Send the user message (section 3.1.3).
-    // Use brandShortName as the username.
-    let username =
-      l10nHelper("chrome://branding/locale/brand.properties")("brandShortName");
+    let username;
+    // Use a custom username in a hidden preference.
+    if (this.prefs.prefHasUserValue("username"))
+      username = this.getString("username");
+    // But fallback to brandShortName if no username is provided (or is empty).
+    if (!username)
+      username = Services.appinfo.name;
     this.sendMessage("USER", [username, this._mode.toString(), "*",
                               this._realname || this._requestedNickname]);
   },
@@ -1061,9 +1328,12 @@ ircAccount.prototype = {
     clearTimeout(this._isOnTimer);
     delete this._isOnTimer;
 
+    // We must authenticate if we reconnect.
+    delete this.isAuthenticated;
+
     // Clean up each conversation: mark as left and remove participant.
     for each (let conversation in this._conversations) {
-      if (conversation.isChat) {
+      if (conversation.isChat && !conversation.left) {
         // Remove the user's nick and mark the conversation as left as that's
         // the final known state of the room.
         conversation.removeParticipant(this._nickname, true);
@@ -1113,16 +1383,20 @@ function ircProtocol() {
   let tempScope = {};
   Cu.import("resource:///modules/ircBase.jsm", tempScope);
   Cu.import("resource:///modules/ircISUPPORT.jsm", tempScope);
+  Cu.import("resource:///modules/ircCAP.jsm", tempScope);
   Cu.import("resource:///modules/ircCTCP.jsm", tempScope);
   Cu.import("resource:///modules/ircDCC.jsm", tempScope);
   Cu.import("resource:///modules/ircServices.jsm", tempScope);
 
   // Extra features.
+  Cu.import("resource:///modules/ircNonStandard.jsm", tempScope);
+  Cu.import("resource:///modules/ircSASL.jsm", tempScope);
   Cu.import("resource:///modules/ircWatchMonitor.jsm", tempScope);
 
   // Register default IRC handlers (IRC base, CTCP).
   ircHandlers.registerHandler(tempScope.ircBase);
   ircHandlers.registerHandler(tempScope.ircISUPPORT);
+  ircHandlers.registerHandler(tempScope.ircCAP);
   ircHandlers.registerHandler(tempScope.ircCTCP);
   ircHandlers.registerHandler(tempScope.ircServices);
   // Register default ISUPPORT handler (ISUPPORT base).
@@ -1134,10 +1408,13 @@ function ircProtocol() {
   ircHandlers.registerServicesHandler(tempScope.servicesBase);
 
   // Register extra features.
+  ircHandlers.registerHandler(tempScope.ircNonStandard);
   ircHandlers.registerHandler(tempScope.ircWATCH);
   ircHandlers.registerISUPPORTHandler(tempScope.isupportWATCH);
   ircHandlers.registerHandler(tempScope.ircMONITOR);
   ircHandlers.registerISUPPORTHandler(tempScope.isupportMONITOR);
+  ircHandlers.registerHandler(tempScope.ircSASL);
+  ircHandlers.registerCAPHandler(tempScope.capSASL);
 }
 ircProtocol.prototype = {
   __proto__: GenericProtocolPrototype,
