@@ -1,43 +1,11 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mail folder tree code.
- *
- * The Initial Developer of the Original Code is
- *   Joey Minta <jminta@gmail.com>
- * Portions created by the Initial Developer are Copyright (C) 2008
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Siddharth Agarwal <sid.bugzilla@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource:///modules/folderUtils.jsm");
 Components.utils.import("resource:///modules/iteratorUtils.jsm");
 Components.utils.import("resource:///modules/MailUtils.js");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 const kDefaultMode = "all";
 
@@ -152,11 +120,9 @@ let gFolderTreeView = {
       // Normally our tree takes care of keeping the last selected by itself.
       // However older versions of TB stored this in a preference, which we need
       // to migrate
-      let prefB = Cc["@mozilla.org/preferences-service;1"]
-                     .getService(Ci.nsIPrefBranch);
-      let modeIndex = prefB.getIntPref("mail.ui.folderpane.view");
+      let modeIndex = Services.prefs.getIntPref("mail.ui.folderpane.view");
       this._mode = this._modeNames[modeIndex];
-      prefB.deleteBranch("mail.ui.folderpane");
+      Services.prefs.deleteBranch("mail.ui.folderpane");
     } catch(ex) {
       // This is ok.  If we've already migrated we'll end up here
     }
@@ -174,8 +140,7 @@ let gFolderTreeView = {
 
     if (aJSONFile) {
       // Parse our persistent-open-state json file
-      let file = Cc["@mozilla.org/file/directory_service;1"]
-                    .getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
+      let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
       file.append(aJSONFile);
 
       if (file.exists()) {
@@ -229,13 +194,12 @@ let gFolderTreeView = {
     if (aJSONFile) {
       // Write out our json file...
       let data = JSON.stringify(this._persistOpenMap);
-      let file = Cc["@mozilla.org/file/directory_service;1"]
-                    .getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
+      let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
       file.append(aJSONFile);
       let foStream = Cc["@mozilla.org/network/safe-file-output-stream;1"]
                         .createInstance(Ci.nsIFileOutputStream);
 
-      foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
+      foStream.init(file, 0x02 | 0x08 | 0x20, parseInt("0666", 8), 0);
       // safe-file-output-stream appears to throw an error if it doesn't write everything at once
       // so we won't worry about looping to deal with partial writes
       foStream.write(data, data.length);
@@ -608,10 +572,9 @@ let gFolderTreeView = {
       }
       return true;
     }
-    // allow subscribing to feeds by dragging an url to a feed account
-    else if (Array.indexOf(types, "text/x-moz-url") != -1 &&
-             targetFolder.server.type == "rss")
-      return true;
+    // Allow subscribing to feeds by dragging an url to a feed account.
+    else if (targetFolder.server.type == "rss" && dt.mozItemCount == 1)
+      return FeedUtils.getFeedUriFromDataTransfer(dt) ? true : false;
     else if (Array.indexOf(types, "application/x-moz-file") != -1) {
       if (aOrientation != Ci.nsITreeView.DROP_ON)
         return false;
@@ -685,8 +648,7 @@ let gFolderTreeView = {
           sourceFolder = msgHdr.folder;
         array.appendElement(msgHdr, false);
       }
-      let prefBranch = Cc["@mozilla.org/preferences-service;1"]
-                          .getService(Ci.nsIPrefService).getBranch("mail.");
+      let prefBranch = Services.prefs.getBranch("mail.");
       let isMove = Cc["@mozilla.org/widget/dragservice;1"]
                       .getService(Ci.nsIDragService).getCurrentSession()
                       .dragAction == Ci.nsIDragService.DRAGDROP_ACTION_MOVE;
@@ -711,19 +673,16 @@ let gFolderTreeView = {
         }
       }
     }
-    else if (Array.indexOf(types, "text/x-moz-url") != -1) {
-      // This is a potential rss feed to subscribe to
-      // and there's only one, so just get the 0th element.
-      let url = dt.mozGetDataAt("text/x-moz-url", 0);
-      let uri = Cc["@mozilla.org/network/io-service;1"]
-                   .getService(Ci.nsIIOService).newURI(url, null, null);
-      if (!(uri.schemeIs("http") || uri.schemeIs("https")) ||
-             targetFolder.server.type != "rss")
-        return;
 
-      Cc["@mozilla.org/newsblog-feed-downloader;1"]
-         .getService(Ci.nsINewsBlogFeedDownloader)
-         .subscribeToFeed(url, targetFolder, msgWindow);
+    if (targetFolder.server.type == "rss" && count == 1) {
+      // This is a potential rss feed.  A link image as well as link text url
+      // should be handled; try to extract a url from non moz apps as well.
+      let validUri = FeedUtils.getFeedUriFromDataTransfer(dt);
+
+      if (validUri)
+        Cc["@mozilla.org/newsblog-feed-downloader;1"]
+           .getService(Ci.nsINewsBlogFeedDownloader)
+           .subscribeToFeed(validUri.spec, targetFolder, msgWindow);
     }
   },
 
@@ -764,6 +723,7 @@ let gFolderTreeView = {
   getCellText: function ftv_getCellText(aRow, aCol) {
     if (aCol.id == "folderNameCol")
       return this._rowMap[aRow].text;
+    return "";
   },
 
   /**
@@ -847,6 +807,25 @@ let gFolderTreeView = {
     this._toggleRow(aIndex, true);
   },
 
+  recursivelyAddToMap: function ftv_recursivelyAddToMap(aChild, aNewIndex) {
+    // When we add sub-children, we're going to need to increase our index
+    // for the next add item at our own level.
+    let count = 0;
+    if (aChild.children.length && aChild.open) {
+      for (let [i, child] in Iterator(this._rowMap[aNewIndex].children)) {
+        count++;
+        let index = Number(aNewIndex) + Number(i) + 1;
+        this._rowMap.splice(index, 0, child);
+
+        let kidsAdded = this.recursivelyAddToMap(child, index);
+        count += kidsAdded;
+        // Somehow the aNewIndex turns into a string without this.
+        aNewIndex = Number(aNewIndex) + kidsAdded;
+      }
+    }
+    return count;
+  },
+
   _toggleRow: function toggleRow(aIndex, aExpandServer)
   {
     // Ok, this is a bit tricky.
@@ -883,26 +862,7 @@ let gFolderTreeView = {
       // Note that these children may have been open when we were last closed,
       // and if they are, we also have to add those grandchildren to the map
       let oldCount = this._rowMap.length;
-      function recursivelyAddToMap(aChild, aNewIndex, tree) {
-        // When we add sub-children, we're going to need to increase our index
-        // for the next add item at our own level
-        let count = 0;
-        if (aChild.children.length && aChild.open) {
-          for (let [i, child] in Iterator(tree._rowMap[aNewIndex].children)) {
-            count++;
-            var index = Number(aNewIndex) + Number(i) + 1;
-            tree._rowMap.splice(index, 0, child);
-
-            let kidsAdded = recursivelyAddToMap(child, index, tree);
-            count += kidsAdded;
-            // Somehow the aNewIndex turns into a string without this
-            aNewIndex = Number(aNewIndex) + kidsAdded;
-          }
-        }
-        return count;
-      }
-      // work around bug 658534 by passing in "this" instead of let tree = this;
-      recursivelyAddToMap(this._rowMap[aIndex], aIndex, this);
+      this.recursivelyAddToMap(this._rowMap[aIndex], aIndex);
 
       // Add this folder to the persist map
       if (!this._persistOpenMap[this.mode])
@@ -1027,7 +987,7 @@ let gFolderTreeView = {
         searchFolderURIs +=  searchFolder.URI;
       }
       if (!searchFolderURIs.length)
-        return;
+        return null;
       smartFolder = gFolderTreeView._createVFFolder(folderName, smartRootFolder,
                                                     searchFolderURIs, flag);
     }
@@ -1187,9 +1147,7 @@ let gFolderTreeView = {
     try {
       newRowMap = this._modes[this.mode].generateMap(this);
     } catch(ex) {
-      Components.classes["@mozilla.org/consoleservice;1"]
-                .getService(Components.interfaces.nsIConsoleService)
-                .logStringMessage("generator " + this.mode + " failed with exception: " + ex);
+      Services.console.logStringMessage("generator " + this.mode + " failed with exception: " + ex);
       this.mode = "all";
       newRowMap = this._modes[this.mode].generateMap(this);
     }
@@ -1222,46 +1180,17 @@ let gFolderTreeView = {
     }
   },
 
-  _sortedAccounts: function ftv_getSortedAccounts()
-  {
-      const Cc = Components.classes;
-      const Ci = Components.interfaces;
-      let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
-                       .getService(Ci.nsIMsgAccountManager);
-      let accounts = [a for each
-                      (a in fixIterator(acctMgr.accounts, Ci.nsIMsgAccount))];
-      // Bug 41133 workaround
-      accounts = accounts.filter(function fix(a) { return a.incomingServer; });
+  _sortedAccounts: function ftv_getSortedAccounts() {
+    let accounts = allAccountsSorted(true);
 
-      // Don't show deferred pop accounts
-      accounts = accounts.filter(function isNotDeferred(a) {
-        let server = a.incomingServer;
-        return !(server instanceof Ci.nsIPop3IncomingServer &&
-                 server.deferredToAccount);
-      });
+    // Don't show deferred pop accounts.
+    accounts = accounts.filter(function isNotDeferred(a) {
+      let server = a.incomingServer;
+      return !(server instanceof Components.interfaces.nsIPop3IncomingServer &&
+               server.deferredToAccount);
+    });
 
-      function sortAccounts(a, b) {
-        if (a.key == acctMgr.defaultAccount.key)
-          return -1;
-        if (b.key == acctMgr.defaultAccount.key)
-          return 1;
-        let aIsNews = a.incomingServer.type == "nntp";
-        let bIsNews = b.incomingServer.type == "nntp";
-        if (aIsNews && !bIsNews)
-          return 1;
-        if (bIsNews && !aIsNews)
-          return -1;
-
-        let aIsLocal = a.incomingServer.type == "none";
-        let bIsLocal = b.incomingServer.type == "none";
-        if (aIsLocal && !bIsLocal)
-          return 1;
-        if (bIsLocal && !aIsLocal)
-          return -1;
-        return 0;
-      }
-      accounts.sort(sortAccounts);
-      return accounts;
+    return accounts;
   },
 
   /**
@@ -1712,13 +1641,15 @@ let gFolderTreeView = {
 
     let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
                      .getService(Ci.nsIMsgAccountManager);
-    for each (let acct in fixIterator(acctMgr.accounts, Ci.nsIMsgAccount)) {
+    for each (let server in fixIterator(acctMgr.allServers, Ci.nsIMsgIncomingServer)) {
       // Skip deferred accounts
-      if (acct.incomingServer instanceof Ci.nsIPop3IncomingServer &&
-          acct.incomingServer.deferredToAccount)
+      if (server instanceof Ci.nsIPop3IncomingServer &&
+          server.deferredToAccount)
         continue;
-      folders.push(acct.incomingServer.rootFolder);
-      this.addSubFolders(acct.incomingServer.rootFolder, folders);
+
+      let rootFolder = server.rootFolder;
+      folders.push(rootFolder);
+      this.addSubFolders(rootFolder, folders);
     }
     return folders;
   },
@@ -2009,9 +1940,7 @@ ftvItem.prototype = {
   },
 
   command: function fti_command() {
-    let pref = Components.classes["@mozilla.org/preferences-service;1"]
-                         .getService(Components.interfaces.nsIPrefBranch);
-    if (!pref.getBoolPref("mailnews.reuse_thread_window2"))
+    if (!Services.prefs.getBoolPref("mailnews.reuse_thread_window2"))
       MsgOpenNewWindowForFolder(this._folder.URI, -1 /* key */);
   },
 
@@ -2024,9 +1953,8 @@ ftvItem.prototype = {
       try {
         iter = fixIterator(this._folder.subFolders, Ci.nsIMsgFolder);
       } catch (ex) {
-        Components.classes["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService)
-          .logStringMessage("Discovering children for " + this._folder.URI + " failed with " +
-                            "exception: " + ex);
+        Services.console.logStringMessage("Discovering children for " + this._folder.URI +
+                                          " failed with " + "exception: " + ex);
         iter = [];
       }
       this._children = [new ftvItem(f) for each (f in iter)];
@@ -2215,10 +2143,9 @@ let gFolderTreeController = {
                                  .getString("confirmSavedSearchDeleteMessage");
       let title = document.getElementById("bundle_messenger")
                           .getString("confirmSavedSearchTitle");
-      let IPS = Components.interfaces.nsIPromptService;
-      if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-            .getService(IPS)
-            .confirmEx(window, title, confirmation, IPS.STD_YES_NO_BUTTONS + IPS.BUTTON_POS_1_DEFAULT,
+      if (Services.prompt
+            .confirmEx(window, title, confirmation,
+                       Services.prompt.STD_YES_NO_BUTTONS + Services.prompt.BUTTON_POS_1_DEFAULT,
                        "", "", "", "", {}) != 0) /* the yes button is in position 0 */
         return;
     }
@@ -2374,25 +2301,21 @@ let gFolderTreeController = {
     const Ci = Components.interfaces;
     let showPrompt = true;
     try {
-      let pref = Cc["@mozilla.org/preferences-service;1"]
-                    .getService(Ci.nsIPrefBranch);
-      showPrompt = !pref.getBoolPref("mail." + aCommand + ".dontAskAgain");
+      showPrompt = !Services.prefs.getBoolPref("mail." + aCommand + ".dontAskAgain");
     } catch (ex) {}
 
     if (showPrompt) {
       let checkbox = {value:false};
-      let promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"]
-                             .getService(Ci.nsIPromptService);
       let bundle = document.getElementById("bundle_messenger");
-      let ok = promptService.confirmEx(window,
-                                       bundle.getString(aCommand + "Title"),
-                                       bundle.getString(aCommand + "Message"),
-                                       promptService.STD_YES_NO_BUTTONS,
-                                       null, null, null,
-                                       bundle.getString(aCommand + "DontAsk"),
-                                       checkbox) == 0;
+      let ok = Services.prompt.confirmEx(window,
+                                         bundle.getString(aCommand + "Title"),
+                                         bundle.getString(aCommand + "Message"),
+                                         Services.prompt.STD_YES_NO_BUTTONS,
+                                         null, null, null,
+                                         bundle.getString(aCommand + "DontAsk"),
+                                         checkbox) == 0;
       if (checkbox.value)
-        pref.setBoolPref("mail." + aCommand + ".dontAskAgain", true);
+        Services.prefs.setBoolPref("mail." + aCommand + ".dontAskAgain", true);
       if (!ok)
         return false;
     }
