@@ -1,45 +1,10 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jan Varga <varga@nixcorp.com>
- *   Hakan Waara <hwaara@chello.se>
- *   Markus Hossner <markushossner@gmx.de>
- *   Magnus Melin <mkmelin+mozilla@iki.fi>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource://gre/modules/PluralForm.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
+Components.utils.import("resource:///modules/mailServices.js");
 
 const mailtolength = 7;
 
@@ -149,8 +114,8 @@ function GetMessageIdFromNode(messageIdNode, cleanMessageId)
  */
 function OpenBrowserWithMessageId(messageId)
 {
-  var browserURL = pref.getComplexValue("mailnews.messageid_browser.url",
-                                        Components.interfaces.nsIPrefLocalizedString).data;
+  var browserURL = Services.prefs.getComplexValue("mailnews.messageid_browser.url",
+                                                  Components.interfaces.nsIPrefLocalizedString).data;
   browserURL = browserURL.replace(/%mid/, messageId);
   try
   {
@@ -188,12 +153,12 @@ function OpenMessageForMessageId(messageId)
 
     messageHeader = SearchForMessageIdInSubFolder(startServer.rootFolder, messageId);
 
-    for (var i = 0; i < allServers.Count() && !messageHeader; i++)
+    for (var i = 0; i < allServers.length && !messageHeader; i++)
     {
-      var currentServer = allServers.GetElementAt(i);
-      if ((currentServer instanceof Components.interfaces.nsIMsgIncomingServer) &&
-          startServer != currentServer && currentServer.canSearchMessages &&
-          !currentServer.isDeferredTo)
+      var currentServer =
+        allServers.queryElementAt(i, Components.interfaces.nsIMsgIncomingServer);
+      if (currentServer && startServer != currentServer &&
+	  currentServer.canSearchMessages && !currentServer.isDeferredTo)
       {
         messageHeader = SearchForMessageIdInSubFolder(currentServer.rootFolder, messageId);
       }
@@ -204,7 +169,7 @@ function OpenMessageForMessageId(messageId)
   // if message id was found open corresponding message
   // else show error message
   if (messageHeader)
-    OpenMessageByHeader(messageHeader, pref.getBoolPref("mailnews.messageid.openInNewWindow"));
+    OpenMessageByHeader(messageHeader, Services.prefs.getBoolPref("mailnews.messageid.openInNewWindow"));
   else
   {
     var messageIdStr = "<" + messageId + ">";
@@ -212,10 +177,8 @@ function OpenMessageForMessageId(messageId)
     var errorTitle = bundle.getString("errorOpenMessageForMessageIdTitle");
     var errorMessage = bundle.getFormattedString("errorOpenMessageForMessageIdMessage",
                                                  [messageIdStr]);
-    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                  .getService(Components.interfaces.nsIPromptService);
 
-    promptService.alert(window, errorTitle, errorMessage);
+    Services.prompt.alert(window, errorTitle, errorMessage);
   }
 }
 
@@ -609,7 +572,7 @@ function composeEmailTo()
                         gFolderDisplay.displayedFolder.server);
   }
   params.composeFields = fields;
-  msgComposeService.OpenComposeWindowWithParams(null, params);
+  MailServices.compose.OpenComposeWindowWithParams(null, params);
 }
 
 // Extracts email address from url string
