@@ -227,7 +227,7 @@ function MailToolboxCustomizeDone(aEvent, customizePopupId)
   }
 }
 
-function onViewToolbarsPopupShowing(aEvent, toolboxIds)
+function onViewToolbarsPopupShowing(aEvent, toolboxIds, aInsertPoint)
 {
   if (!Array.isArray(toolboxIds))
     toolboxIds = [toolboxIds];
@@ -244,8 +244,9 @@ function onViewToolbarsPopupShowing(aEvent, toolboxIds)
       popup.removeChild(deadItem);
   }
 
-  // We'll insert the menuitems before the first item in the list.
-  let firstMenuItem = popup.firstChild;
+  // We'll insert the menuitems before the first item in the list if no insert
+  // point is defined.
+  let firstMenuItem = aInsertPoint || popup.firstChild;
 
   for (let [, toolboxId] in Iterator(toolboxIds)) {
     let toolbox = document.getElementById(toolboxId);
@@ -350,29 +351,39 @@ function toImport()
                     "chrome, modal, titlebar, centerscreen");
 }
 
-// aPaneID
-function openOptionsDialog(aPaneID, aTabID)
+function toSanitize()
 {
-  var instantApply = Services.prefs.getBoolPref("browser.preferences.instantApply");
-  var features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
+   Components.classes["@mozilla.org/mail/mailglue;1"]
+             .getService(Components.interfaces.nsIMailGlue)
+             .sanitize(window);
+}
 
-  var win = Services.wm.getMostRecentWindow("Mail:Preferences");
-  if (win)
-  {
+/**
+ * Opens the Preferences (Options) dialog.
+ *
+ * @param aPaneID     ID of prefpane to select automatically.
+ * @param aTabID      ID of tab to select on the prefpane.
+ * @param aOtherArgs  other prefpane specific arguments
+ */
+function openOptionsDialog(aPaneID, aTabID, aOtherArgs)
+{
+  let win = Services.wm.getMostRecentWindow("Mail:Preferences");
+  if (win) {
+    // the dialog is already open
     win.focus();
-    if (aPaneID)
-    {
-      var pane = win.document.getElementById(aPaneID);
-      win.document.documentElement.showPane(pane);
-
-      // I don't know how to support aTabID for an arbitrary panel when the dialog is already open
-      // This is complicated because showPane is asynchronous (it could trigger a dynamic overlay)
-      // so our tab element may not be accessible right away...
+    if (aPaneID) {
+      let prefWindow = win.document.getElementById("MailPreferences");
+      win.selectPaneAndTab(prefWindow, aPaneID, aTabID);
     }
-  }
-  else {
+  } else {
+    // the dialog must be created
+    let instantApply = Services.prefs
+                               .getBoolPref("browser.preferences.instantApply");
+    let features = "chrome,titlebar,toolbar,centerscreen" +
+                   (instantApply ? ",dialog=no" : ",modal");
+
     openDialog("chrome://messenger/content/preferences/preferences.xul",
-               "Preferences", features, aPaneID, aTabID);
+               "Preferences", features, aPaneID, aTabID, aOtherArgs);
   }
 }
 
@@ -382,7 +393,7 @@ function openAddonsMgr(aView)
     let emWindow;
     let browserWindow;
 
-    function receivePong(aSubject, aTopic, aData) {
+    let receivePong = function receivePong(aSubject, aTopic, aData) {
       let browserWin = aSubject.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
         .getInterface(Components.interfaces.nsIWebNavigation)
         .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
