@@ -1,47 +1,7 @@
 /** ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jan Varga (varga@nixcorp.com)
- *   Håkan Waara (hwaara@chello.se)
- *   Neil Rashbrook (neil@parkwaycc.co.uk)
- *   Seth Spitzer <sspitzer@netscape.com>
- *   David Bienvenu <bienvenu@nventure.com>
- *   Jeremy Morton <bugzilla@game-point.net>
- *   Steffen Wilberg <steffen.wilberg@web.de>
- *   Joachim Herb <herb@leo.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource:///modules/folderUtils.jsm");
 Components.utils.import("resource:///modules/activity/activityModules.js");
@@ -53,6 +13,8 @@ Components.utils.import("resource:///modules/mailnewsMigrator.js");
 Components.utils.import("resource:///modules/sessionStoreManager.js");
 Components.utils.import("resource:///modules/summaryFrameManager.js");
 Components.utils.import("resource:///modules/mailInstrumentation.js");
+Components.utils.import("resource:///modules/msgDBCacheManager.js");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 /* This is where functions related to the 3 pane window are kept */
 
@@ -205,7 +167,7 @@ var gThreePaneIncomingServerListener = {
 function UpdateMailPaneConfig(aMsgWindowInitialized) {
   const dynamicIds = ["messagesBox", "mailContent", "threadPaneBox"];
   const layouts = ["standard", "wide", "vertical"];
-  var layoutView = gPrefBranch.getIntPref("mail.pane_config.dynamic");
+  var layoutView = Services.prefs.getIntPref("mail.pane_config.dynamic");
   // Ensure valid value; hard fail if not.
   layoutView = dynamicIds[layoutView] ? layoutView : kStandardPaneConfig;
   var desiredId = dynamicIds[layoutView];
@@ -314,10 +276,10 @@ const MailPrefObserver = {
         var threadTree = document.getElementById("threadTree");
 
         currentDisplayNameVersion =
-            gPrefBranch.getIntPref("mail.displayname.version");
+            Services.prefs.getIntPref("mail.displayname.version");
 
-        gPrefBranch.setIntPref("mail.displayname.version",
-                               ++currentDisplayNameVersion);
+        Services.prefs.setIntPref("mail.displayname.version",
+                                  ++currentDisplayNameVersion);
 
         //refresh the thread pane
         threadTree.treeBoxObject.invalid();
@@ -331,14 +293,33 @@ const MailPrefObserver = {
  */
 function AutoConfigWizard(okCallback)
 {
-  if (gPrefBranch.getBoolPref("mail.provider.enabled")) {
-    // We need to let the event loop pump a little so that the 3pane finishes
-    // opening - so we use setTimeout. The 100ms is a bit arbitrary, but seems
-    // to be enough time to let the 3pane do it's thing, and not pull focus
-    // when the Account Provisioner modal window closes.
-    setTimeout(function() {
-      NewMailAccountProvisioner(msgWindow, { okCallback: okCallback });
-    }, 100);
+  let suppressDialogs = false;
+
+  // Try to get the suppression pref that we stashed away in accountProvisionerTab.js.
+  // If it doesn't exist, nsIPrefBranch throws, so we eat it silently and move along.
+  try {
+    suppressDialogs = Services.prefs.getBoolPref("mail.provider.suppress_dialog_on_startup");
+  } catch(e) {};
+
+  if (suppressDialogs) {
+    // Looks like we were in the middle of filling out an account form. We
+    // won't display the dialogs in that case.
+    Services.prefs.clearUserPref("mail.provider.suppress_dialog_on_startup");
+    okCallback();
+    return;
+  }
+
+  if (Services.prefs.getBoolPref("mail.provider.enabled")) {
+    Services.obs.addObserver({
+      observe: function(aSubject, aTopic, aData) {
+        if (aTopic == "mail-tabs-session-restored" && aSubject === window) {
+          // We're done here, unregister this observer.
+          Services.obs.removeObserver(this, "mail-tabs-session-restored");
+          NewMailAccountProvisioner(msgWindow, { okCallback: null });
+        }
+      }
+    }, "mail-tabs-session-restored", false);
+    okCallback();
   }
   else
     NewMailAccount(msgWindow, okCallback);
@@ -350,12 +331,12 @@ function AutoConfigWizard(okCallback)
 function OnLoadMessenger()
 {
   migrateMailnews();
-
+  // Rig up our TabsInTitlebar early so that we can catch any resize events.
+  TabsInTitlebar.init();
   // update the pane config before we exit onload otherwise the user may see a flicker if we poke the document
   // in delayedOnLoadMessenger...
   UpdateMailPaneConfig(false);
   document.loadBindingDocument('chrome://global/content/bindings/textbox.xml');
-
   // Set a sane starting width/height for all resolutions on new profiles.
   // Do this before the window loads.
   if (!document.documentElement.hasAttribute("width"))
@@ -375,14 +356,14 @@ function OnLoadMessenger()
     document.documentElement.setAttribute("screenY", screen.availTop);
   }
 
-  gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
-  gPrefBranch.addObserver("mail.pane_config.dynamic", MailPrefObserver, false);
-  gPrefBranch.addObserver("mail.showCondensedAddresses", MailPrefObserver,
-                          false);
+  Services.prefs.addObserver("mail.pane_config.dynamic", MailPrefObserver, false);
+  Services.prefs.addObserver("mail.showCondensedAddresses", MailPrefObserver,
+                             false);
 
   MailOfflineMgr.init();
   CreateMailWindowGlobals();
   GetMessagePaneWrapper().collapsed = true;
+  msgDBCacheManager.init();
 
   // This needs to be before we throw up the account wizard on first run.
   try {
@@ -408,11 +389,6 @@ function OnLoadMessenger()
     tabmail.openFirstTab();
   }
 
-  // verifyAccounts returns true if the callback won't be called
-  // We also don't want the account wizard to open if any sort of account exists
-  if (verifyAccounts(LoadPostAccountWizard, false, AutoConfigWizard))
-    LoadPostAccountWizard();
-
   // Install the light-weight theme handlers
   let panelcontainer = document.getElementById("tabpanelcontainer");
   if (panelcontainer) {
@@ -431,6 +407,11 @@ function OnLoadMessenger()
   webSearchTabType.initialize();
   tabmail.registerTabType(accountProvisionerTabType);
 
+  // verifyAccounts returns true if the callback won't be called
+  // We also don't want the account wizard to open if any sort of account exists
+  if (verifyAccounts(LoadPostAccountWizard, false, AutoConfigWizard))
+    LoadPostAccountWizard();
+
   // Set up the summary frame manager to handle loading pages in the
   // multi-message pane
   gSummaryFrameManager = new SummaryFrameManager(
@@ -453,6 +434,7 @@ function LoadPostAccountWizard()
   MailMigrator.migratePostAccountWizard();
 
   accountManager.setSpecialFolders();
+
   try {
     accountManager.loadVirtualFolders();
   } catch (e) {Components.utils.reportError(e);}
@@ -497,8 +479,6 @@ function LoadPostAccountWizard()
   function completeStartup() {
     // Check whether we need to show the default client dialog
     // First, check the shell service
-    let obs = Components.classes["@mozilla.org/observer-service;1"]
-                        .getService(Components.interfaces.nsIObserverService);
     var nsIShellService = Components.interfaces.nsIShellService;
     if (nsIShellService) {
       var shellService;
@@ -528,11 +508,11 @@ function LoadPostAccountWizard()
         // On windows, there seems to be a delay between setting TB as the
         // default client, and the isDefaultClient check succeeding.
         if (shellService.isDefaultClient(true, nsIShellService.MAIL))
-          obs.notifyObservers(window, "mail:setAsDefault", null);
+          Services.obs.notifyObservers(window, "mail:setAsDefault", null);
       }
     }
     // All core modal dialogs are done, the user can now interact with the 3-pane window
-    obs.notifyObservers(window, "mail-startup-done", null);
+    Services.obs.notifyObservers(window, "mail-startup-done", null);
   }
 
   setTimeout(completeStartup, 0);
@@ -593,12 +573,9 @@ function HandleAppCommandEvent(evt)
  */
 function FindOther3PaneWindow()
 {
-  let windowMediator =
-    Components.classes["@mozilla.org/appshell/window-mediator;1"]
-        .getService(Components.interfaces.nsIWindowMediator);
   // XXX We'd like to use getZOrderDOMWindowEnumerator here, but it doesn't work
   // on Linux
-  let enumerator = windowMediator.getEnumerator("mail:3pane");
+  let enumerator = Services.wm.getEnumerator("mail:3pane");
   while (enumerator.hasMoreElements()) {
     let win = enumerator.getNext();
     if (win != window)
@@ -613,16 +590,20 @@ function FindOther3PaneWindow()
  */
 function OnUnloadMessenger()
 {
+  Services.obs.notifyObservers(window, "mail-unloading-messenger", null);
   accountManager.removeIncomingServerListener(gThreePaneIncomingServerListener);
-  gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
-  gPrefBranch.removeObserver("mail.pane_config.dynamic", MailPrefObserver);
-  gPrefBranch.removeObserver("mail.showCondensedAddresses", MailPrefObserver);
+  Services.prefs.removeObserver("mail.pane_config.dynamic", MailPrefObserver);
+  Services.prefs.removeObserver("mail.showCondensedAddresses", MailPrefObserver);
 
   sessionStoreManager.unloadingWindow(window);
 
+  TabsInTitlebar.uninit();
+
   let tabmail = document.getElementById("tabmail");
   tabmail._teardown();
 
+  webSearchTabType.shutdown();
+
   var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
                               .getService(Components.interfaces.nsIMsgMailSession);
   mailSession.RemoveFolderListener(folderListener);
@@ -676,7 +657,7 @@ function atStartupRestoreTabs(aDontRestoreFirstTab) {
 
   // it's now safe to load extra Tabs.
   setTimeout(loadExtraTabs, 0);
-
+  Services.obs.notifyObservers(window, "mail-tabs-session-restored", null);
   return state ? true : false;
 }
 
@@ -767,17 +748,23 @@ function loadStartFolder(initialUri)
 
     // If a URI was explicitly specified, we'll just clobber the default tab
     let loadFolder = !atStartupRestoreTabs(!!initialUri);
+
     if (initialUri)
       loadFolder = true;
 
     //First get default account
     try
     {
+
         if(initialUri)
             startFolder = GetMsgFolderFromUri(initialUri);
         else
         {
-            var defaultAccount = accountManager.defaultAccount;
+            try {
+                var defaultAccount = accountManager.defaultAccount;
+            } catch (x) {
+                return; // exception caused by no default account, ignore it.
+            }
 
             defaultServer = defaultAccount.incomingServer;
             var rootMsgFolder = defaultServer.rootMsgFolder;
@@ -787,9 +774,9 @@ function loadStartFolder(initialUri)
             // Enable check new mail once by turning checkmail pref 'on' to bring
             // all users to one plane. This allows all users to go to Inbox. User can
             // always go to server settings panel and turn off "Check for new mail at startup"
-            if (!gPrefBranch.getBoolPref(kMailCheckOncePrefName))
+            if (!Services.prefs.getBoolPref(kMailCheckOncePrefName))
             {
-                gPrefBranch.setBoolPref(kMailCheckOncePrefName, true);
+                Services.prefs.setBoolPref(kMailCheckOncePrefName, true);
                 defaultServer.loginAtStartUp = true;
             }
 
@@ -842,8 +829,7 @@ function loadStartFolder(initialUri)
         return;
       }
 
-      dump(ex);
-      dump('Exception in LoadStartFolder caused by no default account.  We know about this\n');
+      Components.utils.reportError(ex);
     }
 
     MsgGetMessagesForAllServers(defaultServer);
@@ -854,22 +840,21 @@ function loadStartFolder(initialUri)
       // the user should set, it's not in mailnews.js, so we need a try catch.
       let playbackOfflineEvents = false;
       try {
-        playbackOfflineEvents = gPrefBranch.getBoolPref("mailnews.playback_offline");
+        playbackOfflineEvents = Services.prefs.getBoolPref("mailnews.playback_offline");
       }
       catch(ex) {}
       if (playbackOfflineEvents)
       {
-        gPrefBranch.setBoolPref("mailnews.playback_offline", false);
+        Services.prefs.setBoolPref("mailnews.playback_offline", false);
         MailOfflineMgr.offlineManager.goOnline(false, true, msgWindow);
       }
 
       // If appropriate, send unsent messages. This may end up prompting the user,
       // so we need to get it out of the flow of the normal load sequence.
-      function checkUnsent() {
+      setTimeout(function checkUnsent() {
         if (MailOfflineMgr.shouldSendUnsentMessages())
           SendUnsentMessages();
-      }
-      setTimeout(checkUnsent, 0);
+      }, 0);
     }
 }
 
@@ -924,18 +909,10 @@ function UpgradeProfileAndBeUglyAboutIt()
   var threadPaneUIVersion;
 
   try {
-
-    threadPaneUIVersion = gPrefBranch.getIntPref("mailnews.ui.threadpane.version");
-
+    threadPaneUIVersion = Services.prefs.getIntPref("mailnews.ui.threadpane.version");
     if (threadPaneUIVersion < 7)
     {
-      // Open a dialog explaining the major changes from version 2.
-      if (gPrefBranch.getBoolPref("mail.ui.show.migration.on.upgrade"))
-        // But let the main window finish opening first.
-        setTimeout(openFeatureConfigurator, 0, [true,]);
-
-      gPrefBranch.setIntPref("mailnews.ui.threadpane.version", 7);
-
+      Services.prefs.setIntPref("mailnews.ui.threadpane.version", 7);
     } // version 7 upgrades
   }
   catch (ex) {
@@ -945,22 +922,16 @@ function UpgradeProfileAndBeUglyAboutIt()
 
 function OnLoadThreadPane()
 {
-  // Register a listener on the columns element so that we get a notification
-  //  whenever attributes on the columns change.  Because of the XBL bindings
-  //  I think we also get the column picker too, but our filtering to only the
-  //  attributes we care about takes care of that.
-  document.getElementById("threadCols").addEventListener(
-    "DOMAttrModified",
-    function(aEvent) {
-      // we only care about hidden status and ordinal
-      if (aEvent.attrName != "hidden" &&
-          aEvent.attrName != "ordinal")
-        return;
-      if (gFolderDisplay)
-        gFolderDisplay.hintColumnsChanged();
-    },
-    true);
-
+  // Use an observer to watch the columns element so that we get a notification
+  // whenever attributes on the columns change.
+  let observer = new MutationObserver(function handleMutations(mutations) {
+    gFolderDisplay.hintColumnsChanged();
+  });
+  observer.observe(document.getElementById("threadCols"), {
+    attributes: true,
+    subtree: true,
+    attributeFilter: ["hidden", "ordinal"]
+  });
   UpgradeProfileAndBeUglyAboutIt();
 }
 
@@ -1129,7 +1100,7 @@ function TreeOnMouseDown(event)
 
 function FolderPaneContextMenuNewTab(event)
 {
-  var bgLoad = gPrefBranch.getBoolPref("mail.tabs.loadInBackground");
+  var bgLoad = Services.prefs.getBoolPref("mail.tabs.loadInBackground");
   if (event.shiftKey)
     bgLoad = !bgLoad;
   MsgOpenNewTabForFolder(bgLoad);
@@ -1170,7 +1141,7 @@ function OpenMessageInNewTab(event)
 {
   if (!gFolderDisplay.selectedMessage)
     return;
-  var bgLoad = gPrefBranch.getBoolPref("mail.tabs.loadInBackground");
+  var bgLoad = Services.prefs.getBoolPref("mail.tabs.loadInBackground");
   if (event.shiftKey)
     bgLoad = !bgLoad;
 
@@ -1213,7 +1184,7 @@ function ReloadMessage()
 // of those settings from the default account.
 function MigrateJunkMailSettings()
 {
-  var junkMailSettingsVersion = gPrefBranch.getIntPref("mail.spam.version");
+  var junkMailSettingsVersion = Services.prefs.getIntPref("mail.spam.version");
   if (!junkMailSettingsVersion)
   {
     // Get the default account, check to see if we have values for our
@@ -1226,17 +1197,29 @@ function MigrateJunkMailSettings()
     {
       // we only care about
       var prefix = "mail.server." + defaultAccount.incomingServer.key + ".";
-      if (gPrefBranch.prefHasUserValue(prefix + "manualMark"))
-        gPrefBranch.setBoolPref("mail.spam.manualMark", pref.getBoolPref(prefix + "manualMark"));
-      if (gPrefBranch.prefHasUserValue(prefix + "manualMarkMode"))
-        gPrefBranch.setIntPref("mail.spam.manualMarkMode", pref.getIntPref(prefix + "manualMarkMode"));
-      if (gPrefBranch.prefHasUserValue(prefix + "spamLoggingEnabled"))
-        gPrefBranch.setBoolPref("mail.spam.logging.enabled", pref.getBoolPref(prefix + "spamLoggingEnabled"));
-      if (gPrefBranch.prefHasUserValue(prefix + "markAsReadOnSpam"))
-        gPrefBranch.setBoolPref("mail.spam.markAsReadOnSpam", pref.getBoolPref(prefix + "markAsReadOnSpam"));
+      if (Services.prefs.prefHasUserValue(prefix + "manualMark"))
+      {
+        Services.prefs.setBoolPref("mail.spam.manualMark",
+          Services.prefs.getBoolPref(prefix + "manualMark"));
+      }
+      if (Services.prefs.prefHasUserValue(prefix + "manualMarkMode"))
+      {
+        Services.prefs.setIntPref("mail.spam.manualMarkMode",
+          Services.prefs.getIntPref(prefix + "manualMarkMode"));
+      }
+      if (Services.prefs.prefHasUserValue(prefix + "spamLoggingEnabled"))
+      {
+        Services.prefs.setBoolPref("mail.spam.logging.enabled",
+          Services.prefs.getBoolPref(prefix + "spamLoggingEnabled"));
+      }
+      if (Services.prefs.prefHasUserValue(prefix + "markAsReadOnSpam"))
+      {
+        Services.prefs.setBoolPref("mail.spam.markAsReadOnSpam",
+          Services.prefs.getBoolPref(prefix + "markAsReadOnSpam"));
+      }
     }
     // bump the version so we don't bother doing this again.
-    gPrefBranch.setIntPref("mail.spam.version", 1);
+    Services.prefs.setIntPref("mail.spam.version", 1);
   }
 }
 
@@ -1244,15 +1227,15 @@ function MigrateJunkMailSettings()
 // with the existing INBOX folders.
 function MigrateFolderViews()
 {
-  var folderViewsVersion = gPrefBranch.getIntPref("mail.folder.views.version");
+  var folderViewsVersion = Services.prefs.getIntPref("mail.folder.views.version");
   if (!folderViewsVersion)
   {
      var servers = accountManager.allServers;
      var server;
      var inbox;
-     for (var index = 0; index < servers.Count(); index++)
+     for (var index = 0; index < servers.length; index++)
      {
-       server = servers.QueryElementAt(index, Components.interfaces.nsIMsgIncomingServer);
+       server = servers.queryElementAt(index, Components.interfaces.nsIMsgIncomingServer);
        if (server)
        {
          inbox = GetInboxFolder(server);
@@ -1260,7 +1243,7 @@ function MigrateFolderViews()
            inbox.setFlag(Components.interfaces.nsMsgFolderFlags.Favorite);
        }
      }
-    gPrefBranch.setIntPref("mail.folder.views.version", 1);
+    Services.prefs.setIntPref("mail.folder.views.version", 1);
   }
 }
 
@@ -1270,17 +1253,15 @@ function MigrateFolderViews()
 // To migrate to the new download manager, remove downloads.rdf.
 function MigrateAttachmentDownloadStore()
 {
-  var attachmentStoreVersion = gPrefBranch.getIntPref("mail.attachment.store.version");
+  var attachmentStoreVersion = Services.prefs.getIntPref("mail.attachment.store.version");
   if (!attachmentStoreVersion)
   {
-    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
-                     .getService(Components.interfaces.nsIProperties);
-    var downloadsFile = dirService.get("DLoads", Components.interfaces.nsIFile);
+    var downloadsFile = Services.dirsvc.get("DLoads", Components.interfaces.nsIFile);
     if (downloadsFile && downloadsFile.exists())
       downloadsFile.remove(false);
 
     // bump the version so we don't bother doing this again.
-    gPrefBranch.setIntPref("mail.attachment.store.version", 1);
+    Services.prefs.setIntPref("mail.attachment.store.version", 1);
   }
 }
 
@@ -1289,26 +1270,26 @@ function MigrateAttachmentDownloadStore()
 // is defined.
 function MigrateOpenMessageBehavior()
 {
-  let openMessageBehaviorVersion = gPrefBranch.getIntPref(
+  let openMessageBehaviorVersion = Services.prefs.getIntPref(
                                      "mail.openMessageBehavior.version");
   if (!openMessageBehaviorVersion)
   {
     let reuseMessageWindow;
     try {
-      reuseMessageWindow = gPrefBranch.getBoolPref(
+      reuseMessageWindow = Services.prefs.getBoolPref(
                              "mailnews.reuse_message_window");
     }
     catch (e) {}
 
     // Don't touch this if it isn't defined
     if (reuseMessageWindow === true)
-      gPrefBranch.setIntPref("mail.openMessageBehavior",
+      Services.prefs.setIntPref("mail.openMessageBehavior",
           MailConsts.OpenMessageBehavior.EXISTING_WINDOW);
     else if (reuseMessageWindow === false)
-      gPrefBranch.setIntPref("mail.openMessageBehavior",
+      Services.prefs.setIntPref("mail.openMessageBehavior",
           MailConsts.OpenMessageBehavior.NEW_TAB);
 
-    gPrefBranch.setIntPref("mail.openMessageBehavior.version", 1);
+    Services.prefs.setIntPref("mail.openMessageBehavior.version", 1);
   }
 }
 
@@ -1321,8 +1302,6 @@ function ThreadPaneOnDragStart(aEvent) {
     return;
 
   gFolderDisplay.hintAboutToDeleteMessages();
-  let ios = Components.classes["@mozilla.org/network/io-service;1"]
-                      .getService(Components.interfaces.nsIIOService);
   let fileNames = [];
   let msgUrls = {};
 
@@ -1569,26 +1548,8 @@ var LightWeightThemeWebInstaller = {
   },
 
   _isAllowed: function (node) {
-    let pm = Components.classes["@mozilla.org/permissionmanager;1"]
-      .getService(Components.interfaces.nsIPermissionManager);
-
-    let prefs = [["xpinstall.whitelist.add", pm.ALLOW_ACTION],
-                 ["xpinstall.whitelist.add.36", pm.ALLOW_ACTION],
-                 ["xpinstall.blacklist.add", pm.DENY_ACTION]];
-
-    prefs.forEach(function ([pref, permission]) {
-      let hosts = Application.prefs.getValue(pref, "");
-      if (hosts) {
-        hosts.split(",").forEach(function (host) {
-          pm.add(makeURI("http://" + host.trim()), "install", permission);
-        });
-
-        Application.prefs.setValue(pref, "");
-      }
-    });
-
     let uri = node.ownerDocument.documentURIObject;
-    return pm.testPermission(uri, "install") == pm.ALLOW_ACTION;
+    return Services.perms.testPermission(uri, "install") == Services.perms.ALLOW_ACTION;
   },
 
   _getNotificationBox: function () {
@@ -1624,3 +1585,107 @@ function InitPageMenu(menuPopup, event) {
   if (menuPopup.children.length == 0)
     event.preventDefault();
 }
+
+let TabsInTitlebar = {
+  init: function () {
+#ifdef CAN_DRAW_IN_TITLEBAR
+    // Don't trust the initial value of the sizemode attribute; wait for the resize event.
+    this._readPref();
+    Services.prefs.addObserver(this._prefName, this, false);
+
+    this.allowedBy("sizemode", false);
+    window.addEventListener("resize", function (event) {
+      if (event.target != window)
+        return;
+      TabsInTitlebar.allowedBy("sizemode", true);
+    }, false);
+
+    this._initialized = true;
+#endif
+  },
+
+  allowedBy: function (condition, allow) {
+#ifdef CAN_DRAW_IN_TITLEBAR
+    if (allow) {
+      if (condition in this._disallowed) {
+        delete this._disallowed[condition];
+        this._update();
+      }
+    } else {
+      if (!(condition in this._disallowed)) {
+        this._disallowed[condition] = null;
+        this._update();
+      }
+    }
+#endif
+  },
+
+  _initialized: false,
+  _disallowed: {},
+  _prefName: 'mail.tabs.drawInTitlebar',
+
+  get enabled() {
+    return document.documentElement.getAttribute('tabsintitlebar') == 'true';
+  },
+
+  _readPref: function() {
+    this.allowedBy('pref', Services.prefs.getBoolPref(this._prefName));
+  },
+
+  observe: function(aSubject, aTopic, aData) {
+    if (aTopic == 'nsPref:changed')
+      this._readPref();
+  },
+
+  _update: function() {
+#ifdef CAN_DRAW_IN_TITLEBAR
+    if (!this._initialized || window.fullScreen)
+      return;
+
+    let allowed = Object.keys(this._disallowed).length == 0;
+    if (allowed == this.enabled)
+      return;
+
+    function $(id) document.getElementById(id);
+    let titlebar = $("titlebar");
+
+    if (allowed) {
+      document.documentElement.setAttribute('tabsintitlebar', 'true');
+      document.documentElement.setAttribute('chromemargin', '0,2,2,2');
+      function rect(ele) ele.getBoundingClientRect();
+
+      let captionButtonsBox = $("titlebar-buttonbox");
+      this._sizePlaceholder("caption-buttons", rect(captionButtonsBox).width);
+
+      let titlebarRect = rect(titlebar);
+      titlebar.style.marginBottom = - (titlebarRect.height - 16) + "px";
+    } else {
+      document.documentElement.removeAttribute('tabsintitlebar');
+      document.documentElement.removeAttribute('chromemargin');
+      titlebar.style.marginBottom = "";
+    }
+#endif
+  },
+
+  _sizePlaceholder: function (type, width) {
+#ifdef CAN_DRAW_IN_TITLEBAR
+    Array.forEach(document.querySelectorAll(".titlebar-placeholder[type='"+ type +"']"),
+                  function (node) { node.width = width; });
+#endif
+  },
+
+  uninit: function () {
+#ifdef CAN_DRAW_IN_TITLEBAR
+    this._initialized = false;
+    Services.prefs.removeObserver(this._prefName, this);
+#endif
+  }
+};
+
+/* Draw */
+function onTitlebarMaxClick() {
+  if (window.windowState == window.STATE_MAXIMIZED)
+    window.restore();
+  else
+    window.maximize();
+}
