@@ -22,7 +22,7 @@ let stringBundle = new StringBundle("chrome://messenger/locale/newmailaccount/ac
 let isOSX = (Services.appinfo.OS == 'Darwin');
 
 const RETRY_TIMEOUT = 5000; // 5 seconds
-const CONNECTION_TIMEOUT = 10000; // 10 seconds
+const CONNECTION_TIMEOUT = 15000; // 15 seconds
 
 function isAccel (event) (isOSX && event.metaKey || event.ctrlKey)
 
@@ -37,14 +37,10 @@ function isAccel (event) (isOSX && event.metaKey || event.ctrlKey)
  */
 function getLocalStorage(page) {
   var url = "chrome://content/messenger/accountProvisionerStorage/" + page;
-  var ssm = Cc["@mozilla.org/scriptsecuritymanager;1"]
-    .getService(Ci.nsIScriptSecurityManager);
-  var dsm = Cc["@mozilla.org/dom/storagemanager;1"]
-    .getService(Ci.nsIDOMStorageManager);
 
   var uri = Services.io.newURI(url, "", null);
-  var principal = ssm.getCodebasePrincipal(uri);
-  return dsm.getLocalStorageForPrincipal(principal, url);
+  var principal = Services.scriptSecurityManager.getNoAppCodebasePrincipal(uri);
+  return Services.domStorageManager.getLocalStorageForPrincipal(principal, url);
 }
 
 const MAX_SMALL_ADDRESSES = 2;
@@ -78,7 +74,6 @@ var EmailAccountProvisioner = {
   NewComposeMessage: window.arguments[0].NewComposeMessage,
   openAddonsMgr: window.arguments[0].openAddonsMgr,
   msgWindow: window.arguments[0].msgWindow,
-  okCallback: window.arguments[0].okCallback,
 
   get someProvidersChecked() {
     return this._someProvidersChecked;
@@ -267,18 +262,11 @@ var EmailAccountProvisioner = {
     $("button.existing").click(function() {
       EmailAccountProvisioner.saveName();
       EmailAccountProvisioner.NewMailAccount(EmailAccountProvisioner.msgWindow,
-                                             EmailAccountProvisioner.okCallback,
+                                             null,
                                              window.arguments[0]);
-      // Set the callback to null, so that we don't call it.
-      EmailAccountProvisioner.okCallback = null;
       window.close();
     });
 
-    $(window).unload(function() {
-      if (EmailAccountProvisioner.okCallback)
-        EmailAccountProvisioner.okCallback();
-    });
-
     // Handle Ctrl-W and Esc
     $(window).keypress(function(event) {
       if ((event.which == "119" && isAccel(event))
@@ -415,9 +403,7 @@ var EmailAccountProvisioner = {
 
     gLog.info("Opening up a contentTab with the order form.");
     // Then open a content tab.
-    let mail3Pane = Cc["@mozilla.org/appshell/window-mediator;1"]
-          .getService(Ci.nsIWindowMediator)
-          .getMostRecentWindow("mail:3pane");
+    let mail3Pane = Services.wm.getMostRecentWindow("mail:3pane");
 
     let tabmail = mail3Pane.document.getElementById("tabmail");
     tabmail.openTab("accountProvisionerTab", {
