@@ -13,7 +13,7 @@
 #include "nsIStringBundle.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
-#include "nsNetError.h"
+#include "nsError.h"
 #include "nsMemory.h"
 #include "nsILDAPErrors.h"
 #include "nsMsgUtils.h"
@@ -53,7 +53,7 @@ nsAbLDAPAutoCompFormatter::Format(nsILDAPMessage *aMsg,
 
     // generate the appropriate string
     //
-    nsCAutoString name;
+    nsAutoCString name;
     rv = ProcessFormat(mNameFormat, aMsg, &name, 0);
     if (NS_FAILED(rv)) {
         // Something went wrong lower down the stack; a message should
@@ -63,7 +63,7 @@ nsAbLDAPAutoCompFormatter::Format(nsILDAPMessage *aMsg,
         return rv;
     }
 
-    nsCAutoString address;
+    nsAutoCString address;
     rv = ProcessFormat(mAddressFormat, aMsg, &address, 0);
     if (NS_FAILED(rv)) {
         // Something went wrong lower down the stack; a message should have 
@@ -104,7 +104,7 @@ nsAbLDAPAutoCompFormatter::Format(nsILDAPMessage *aMsg,
 
     // generate the appropriate string to appear as a comment off to the side
     //
-    nsCAutoString comment;
+    nsAutoCString comment;
     rv = ProcessFormat(mCommentFormat, aMsg, &comment, 0);
     if (NS_SUCCEEDED(rv)) {
         rv = item->SetComment(NS_ConvertUTF8toUTF16(comment).get());
@@ -126,11 +126,11 @@ nsAbLDAPAutoCompFormatter::Format(nsILDAPMessage *aMsg,
 }
 
 NS_IMETHODIMP
-nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState, 
+nsAbLDAPAutoCompFormatter::FormatException(int32_t aState, 
                                            nsresult aErrorCode, 
                                            nsIAutoCompleteItem **aItem) 
 {
-    PRInt32 errorKey;
+    int32_t errorKey;
     nsresult rv;
 
     // create an nsIAutoCompleteItem to hold the returned value
@@ -219,12 +219,12 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
         // put the entire nsresult in string form
         //
         errCodeNum.AppendLiteral("0x");
-        errCodeNum.AppendInt(aErrorCode, 16);    
+        errCodeNum.AppendInt(static_cast<uint32_t>(aErrorCode), 16);
 
         // figure out the key to index into the string bundle
         //
-        const PRInt32 HOST_NOT_FOUND_ERROR=5000;
-        const PRInt32 GENERIC_ERROR=9999;
+        const int32_t HOST_NOT_FOUND_ERROR=5000;
+        const int32_t GENERIC_ERROR=9999;
         errorKey = ( (aErrorCode == NS_ERROR_UNKNOWN_HOST) ? 
                      HOST_NOT_FOUND_ERROR : GENERIC_ERROR );
 
@@ -242,8 +242,8 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
 
     // and try to find a hint about what the user should do next
     //
-    const PRInt32 HINT_BASE=10000;
-    const PRInt32 GENERIC_HINT_CODE = 9999;
+    const int32_t HINT_BASE=10000;
+    const int32_t GENERIC_HINT_CODE = 9999;
     rv = ldapACBundle->GetStringFromID(HINT_BASE + errorKey, 
                                        getter_Copies(ldapHint));
     if (NS_FAILED(rv)) {
@@ -327,7 +327,7 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
 NS_IMETHODIMP
 nsAbLDAPAutoCompFormatter::GetAttributes(nsACString &aResult)
 {
-    nsCAutoString searchAttrs;
+    nsAutoCString searchAttrs;
     nsresult rv = ProcessFormat(mNameFormat, 0, 0, &searchAttrs);
     if (NS_FAILED(rv)) {
         NS_WARNING("nsAbLDAPAutoCompFormatter::SetNameFormat(): "
@@ -381,7 +381,7 @@ nsAbLDAPAutoCompFormatter::ProcessFormat(const nsAString & aFormat,
     }
 
     bool attrRequired = false;     // is this attr required or optional?
-    nsCAutoString attrName;             // current attr to get
+    nsAutoCString attrName;             // current attr to get
 
     // parse until we hit the end of the string
     //
@@ -546,7 +546,7 @@ nsAbLDAPAutoCompFormatter::AppendFirstAttrValue(
     // get the attribute values for the field which will be used 
     // to fill in nsIAutoCompleteItem::value
     //
-    PRUint32 numVals;
+    uint32_t numVals;
     PRUnichar **values;
 
     nsresult rv;
