@@ -89,14 +89,14 @@ NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryAuthDN(const nsACString & aValueUsedT
   nsresult rv = mDirectory->GetAttributeMap(getter_AddRefs(attrMap));
   NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString filter;
+    nsAutoCString filter;
     rv = attrMap->GetFirstAttribute(NS_LITERAL_CSTRING("PrimaryEmail"), filter);
     NS_ENSURE_SUCCESS(rv, rv);
 
     filter += '=';
     filter += aValueUsedToFindDn;
 
-    nsCAutoString dn;
+    nsAutoCString dn;
     rv = mURL->GetDn(dn);
     if(NS_FAILED(rv)) 
         return rv;
@@ -107,7 +107,7 @@ NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryAuthDN(const nsACString & aValueUsedT
     // XXX We really should be using LDAP_NO_ATTRS here once its exposed via
     // the XPCOM layer of the directory code.
     return mOperation->SearchExt(dn, nsILDAPURL::SCOPE_SUBTREE, filter, 
-                               0, nsnull,
+                               0, nullptr,
                                0, 0);
 }
 
@@ -124,21 +124,21 @@ NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryRootDSE()
                                  sChangeLogRootDSEAttribs, 0, 0);
 }
 
-NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryChangeLog(const nsACString & aChangeLogDN, PRInt32 aLastChangeNo)
+NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryChangeLog(const nsACString & aChangeLogDN, int32_t aLastChangeNo)
 {
   if (!mInitialized)
     return NS_ERROR_NOT_INITIALIZED;
   if (aChangeLogDN.IsEmpty()) 
     return NS_ERROR_UNEXPECTED;
 
-  PRInt32 lastChangeNumber;
+  int32_t lastChangeNumber;
   nsresult rv = mDirectory->GetLastChangeNumber(&lastChangeNumber);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // make sure that the filter here just have one condition 
   // and should not be enclosed in enclosing brackets.
   // also condition '>' doesnot work, it should be '>='/
-  nsCAutoString filter (NS_LITERAL_CSTRING("changenumber>="));
+  nsAutoCString filter (NS_LITERAL_CSTRING("changenumber>="));
   filter.AppendInt(lastChangeNumber + 1);
 
   rv = CreateNewLDAPOperation();
@@ -156,12 +156,12 @@ NS_IMETHODIMP nsAbLDAPChangeLogQuery::QueryChangedEntries(const nsACString & aCh
     if(aChangedEntryDN.IsEmpty()) 
         return NS_ERROR_UNEXPECTED;
 
-    nsCAutoString urlFilter;
+    nsAutoCString urlFilter;
     nsresult rv = mURL->GetFilter(urlFilter);
     if(NS_FAILED(rv)) 
         return rv;
 
-    PRInt32 scope;
+    int32_t scope;
     rv = mURL->GetScope(&scope);
     if(NS_FAILED(rv)) 
         return rv;
