@@ -25,7 +25,7 @@
 #include "nsVCardObj.h"
 #include "nsIAbLDAPAttributeMap.h"
 #include "nsICommandLine.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIMutableArray.h"
 #include "nsArrayUtils.h"
 #include "nsDirectoryServiceUtils.h"
@@ -37,11 +37,13 @@
 #include "nsIIOService.h"
 #include "nsAbQueryStringToExpression.h"
 #include "mozilla/Services.h"
+#include "mozilla/Util.h"
+using namespace mozilla;
 
 struct ExportAttributesTableStruct
 {
   const char* abPropertyName;
-  PRUint32 plainTextStringID;
+  uint32_t plainTextStringID;
 };
 
 // our schema is not fixed yet, but we still want some sort of objectclass
@@ -248,15 +250,15 @@ NS_IMETHODIMP nsAbManager::GetDirectory(const nsACString &aURI,
     // The directory wasn't in our look-up table, so we need to instantiate
     // it. First, extract the scheme from the URI...
 
-    nsCAutoString scheme;
+    nsAutoCString scheme;
 
-    PRInt32 colon = aURI.FindChar(':');
+    int32_t colon = aURI.FindChar(':');
     if (colon <= 0)
       return NS_ERROR_MALFORMED_URI;
     scheme = Substring(aURI, 0, colon);
 
     // Construct the appropriate nsIAbDirectory...
-    nsCAutoString contractID;
+    nsAutoCString contractID;
     contractID.AssignLiteral(NS_AB_DIRECTORY_TYPE_CONTRACTID_PREFIX);
     contractID.Append(scheme);
     directory = do_CreateInstance(contractID.get(), &rv);
@@ -282,7 +284,7 @@ NS_IMETHODIMP nsAbManager::GetDirectory(const nsACString &aURI,
 
 NS_IMETHODIMP nsAbManager::NewAddressBook(const nsAString &aDirName,
                                             const nsACString &aURI,
-                                            const PRUint32 aType,
+                                            const uint32_t aType,
                                             const nsACString &aPrefName,
                                             nsACString &aResult)
 {
@@ -346,7 +348,7 @@ NS_IMETHODIMP nsAbManager::DeleteAddressBook(const nsACString &aURI)
 
   nsCString parentUri;
   parentUri.Append(aURI);
-  PRInt32 pos = parentUri.RFindChar('/');
+  int32_t pos = parentUri.RFindChar('/');
 
   // If we didn't find a /, we're in trouble.
   if (pos == -1)
@@ -421,14 +423,14 @@ NS_IMETHODIMP nsAbManager::NotifyDirectoryDeleted(nsIAbDirectory *aParentDirecto
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAbManager::GetUserProfileDirectory(nsILocalFile **userDir)
+NS_IMETHODIMP nsAbManager::GetUserProfileDirectory(nsIFile **userDir)
 {
   NS_ENSURE_ARG_POINTER(userDir);
-  *userDir = nsnull;
+  *userDir = nullptr;
 
   nsresult rv;
   nsCOMPtr<nsIFile> profileDir;
-  nsCAutoString pathBuf;
+  nsAutoCString pathBuf;
 
   rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(profileDir));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -537,13 +539,13 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt16 dialogResult;
+  int16_t dialogResult;
   filePicker->Show(&dialogResult);
 
   if (dialogResult == nsIFilePicker::returnCancel)
     return rv;
 
-  nsCOMPtr<nsILocalFile> localFile;
+  nsCOMPtr<nsIFile> localFile;
   rv = filePicker->GetFile(getter_AddRefs(localFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -559,7 +561,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
 
   // The type of export is determined by the drop-down in
   // the file picker dialog.
-  PRInt32 exportType;
+  int32_t exportType;
   rv = filePicker->GetFilterIndex(&exportType);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -610,7 +612,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
 }
 
 nsresult
-nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const char *aDelim, PRUint32 aDelimLen, nsILocalFile *aLocalFile)
+nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const char *aDelim, uint32_t aDelimLen, nsIFile *aLocalFile)
 {
   nsCOMPtr <nsISimpleEnumerator> cardsEnumerator;
   nsCOMPtr <nsIAbCard> card;
@@ -627,9 +629,9 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
   if (NS_FAILED(rv))
     return rv;
 
-  PRUint32 i;
-  PRUint32 writeCount;
-  PRUint32 length;
+  uint32_t i;
+  uint32_t writeCount;
+  uint32_t length;
 
   nsCOMPtr<nsIStringBundleService> bundleService =
     mozilla::services::GetStringBundleService();
@@ -642,7 +644,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
   nsCString revisedName;
   nsString columnName;
 
-  for (i = 0; i < NS_ARRAY_LENGTH(EXPORT_ATTRIBUTES_TABLE); i++) {
+  for (i = 0; i < ArrayLength(EXPORT_ATTRIBUTES_TABLE); i++) {
     if (EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID != 0) {
 
       // We don't need to truncate the string here as getter_Copies will
@@ -662,7 +664,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
       if (revisedName.Length() != writeCount)
         return NS_ERROR_FAILURE;
 
-      if (i < NS_ARRAY_LENGTH(EXPORT_ATTRIBUTES_TABLE) - 1) {
+      if (i < ArrayLength(EXPORT_ATTRIBUTES_TABLE) - 1) {
         rv = outputStream->Write(aDelim, aDelimLen, &writeCount);
         NS_ENSURE_SUCCESS(rv,rv);
 
@@ -699,7 +701,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
           nsString value;
           nsCString valueCStr;
 
-          for (i = 0; i < NS_ARRAY_LENGTH(EXPORT_ATTRIBUTES_TABLE); i++) {
+          for (i = 0; i < ArrayLength(EXPORT_ATTRIBUTES_TABLE); i++) {
             if (EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID != 0) {
               rv = card->GetPropertyAsAString(EXPORT_ATTRIBUTES_TABLE[i].abPropertyName, value);
               if (NS_FAILED(rv))
@@ -714,8 +716,8 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
               {
                 needsQuotes = true;
                 
-                PRInt32 match = 0;
-                PRUint32 offset = 0;
+                int32_t match = 0;
+                uint32_t offset = 0;
                 nsString oldSubstr = NS_LITERAL_STRING("\"");
                 nsString newSubstr = NS_LITERAL_STRING("\"\""); 
                 while (offset < newValue.Length()) {
@@ -765,7 +767,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
               continue; // go to next field
             }
 
-            if (i < NS_ARRAY_LENGTH(EXPORT_ATTRIBUTES_TABLE) - 1) {
+            if (i < ArrayLength(EXPORT_ATTRIBUTES_TABLE) - 1) {
               rv = outputStream->Write(aDelim, aDelimLen, &writeCount);
               NS_ENSURE_SUCCESS(rv,rv);
               if (aDelimLen != writeCount)
@@ -792,7 +794,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
 }
 
 nsresult
-nsAbManager::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *aLocalFile)
+nsAbManager::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsIFile *aLocalFile)
 {
   nsCOMPtr <nsISimpleEnumerator> cardsEnumerator;
   nsCOMPtr <nsIAbCard> card;
@@ -821,9 +823,9 @@ nsAbManager::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *aLo
                                    getter_AddRefs(attrMap));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 i;
-  PRUint32 writeCount;
-  PRUint32 length;
+  uint32_t i;
+  uint32_t writeCount;
+  uint32_t length;
 
   rv = aDirectory->GetChildCards(getter_AddRefs(cardsEnumerator));
   if (NS_SUCCEEDED(rv) && cardsEnumerator) {
@@ -866,9 +868,9 @@ nsAbManager::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *aLo
 
           valueCStr.Truncate();
 
-          nsCAutoString ldapAttribute;
+          nsAutoCString ldapAttribute;
 
-          for (i = 0; i < NS_ARRAY_LENGTH(EXPORT_ATTRIBUTES_TABLE); i++) {
+          for (i = 0; i < ArrayLength(EXPORT_ATTRIBUTES_TABLE); i++) {
             if (NS_SUCCEEDED(attrMap->GetFirstAttribute(nsDependentCString(EXPORT_ATTRIBUTES_TABLE[i].abPropertyName),
                                                         ldapAttribute)) &&
                 !ldapAttribute.IsEmpty()) {
@@ -943,7 +945,7 @@ nsresult nsAbManager::AppendLDIFForMailList(nsIAbCard *aCard, nsIAbLDAPAttribute
   rv = aCard->GetDisplayName(attrValue);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCAutoString ldapAttributeName;
+  nsAutoCString ldapAttributeName;
 
   rv = aAttrMap->GetFirstAttribute(NS_LITERAL_CSTRING(kDisplayNameProperty),
                                   ldapAttributeName);
@@ -986,10 +988,10 @@ nsresult nsAbManager::AppendLDIFForMailList(nsIAbCard *aCard, nsIAbLDAPAttribute
   nsCOMPtr<nsIMutableArray> addresses;
   rv = mailList->GetAddressLists(getter_AddRefs(addresses));
   if (addresses) {
-    PRUint32 total = 0;
+    uint32_t total = 0;
     addresses->GetLength(&total);
     if (total) {
-      PRUint32 i;
+      uint32_t i;
       for (i = 0; i < total; i++) {
         nsCOMPtr <nsIAbCard> listCard = do_QueryElementAt(addresses, i, &rv);
         NS_ENSURE_SUCCESS(rv,rv);
@@ -1010,7 +1012,7 @@ nsresult nsAbManager::AppendDNForCard(const char *aProperty, nsIAbCard *aCard, n
 {
   nsString email;
   nsString displayName;
-  nsCAutoString ldapAttributeName;
+  nsAutoCString ldapAttributeName;
 
   nsresult rv = aCard->GetPrimaryEmail(email);
   NS_ENSURE_SUCCESS(rv,rv);
@@ -1069,8 +1071,8 @@ bool nsAbManager::IsSafeLDIFString(const PRUnichar *aStr)
       aStr[0] == PRUnichar('<'))
     return false;
 
-  PRUint32 i;
-  PRUint32 len = NS_strlen(aStr);
+  uint32_t i;
+  uint32_t len = NS_strlen(aStr);
   for (i=0; i<len; i++) {
     // If string contains CR or LF, it is not safe for LDIF
     // and MUST be base64 encoded
@@ -1094,7 +1096,7 @@ nsresult nsAbManager::AppendProperty(const char *aProperty, const PRUnichar *aVa
     aResult.Append(NS_LossyConvertUTF16toASCII(aValue));
   }
   else {
-    char *base64Str = PL_Base64Encode(NS_ConvertUTF16toUTF8(aValue).get(), 0, nsnull);
+    char *base64Str = PL_Base64Encode(NS_ConvertUTF16toUTF8(aValue).get(), 0, nullptr);
     if (!base64Str)
       return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1244,8 +1246,8 @@ nsAbManager::Handle(nsICommandLine* aCmdLine)
   NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDOMWindow> opened;
-  wwatch->OpenWindow(nsnull, "chrome://messenger/content/addressbook/addressbook.xul",
-                     "_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar", nsnull, getter_AddRefs(opened));
+  wwatch->OpenWindow(nullptr, "chrome://messenger/content/addressbook/addressbook.xul",
+                     "_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar", nullptr, getter_AddRefs(opened));
   aCmdLine->SetPreventDefault(true);
   return NS_OK;
 }
