@@ -25,6 +25,7 @@
 #include "nsServiceManagerUtils.h"
 #include "nsIMsgFilterService.h"
 #include "nsMemory.h"
+#include "nsIMutableArray.h"
 #include "prmem.h"
 #include "mozilla/Services.h"
 
@@ -100,7 +101,7 @@ nsMsgRuleAction::GetTargetFolderUri(nsACString &aResult)
 }
 
 NS_IMETHODIMP
-nsMsgRuleAction::SetJunkScore(PRInt32 aJunkScore)
+nsMsgRuleAction::SetJunkScore(int32_t aJunkScore)
 {
   NS_ENSURE_TRUE(m_type == nsMsgFilterAction::JunkScore && aJunkScore >= 0 && aJunkScore <= 100,
                  NS_ERROR_ILLEGAL_VALUE);
@@ -109,7 +110,7 @@ nsMsgRuleAction::SetJunkScore(PRInt32 aJunkScore)
 }
 
 NS_IMETHODIMP
-nsMsgRuleAction::GetJunkScore(PRInt32 *aResult)
+nsMsgRuleAction::GetJunkScore(int32_t *aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
   NS_ENSURE_TRUE(m_type == nsMsgFilterAction::JunkScore, NS_ERROR_ILLEGAL_VALUE);
@@ -169,11 +170,10 @@ NS_IMETHODIMP nsMsgRuleAction::GetCustomAction(nsIMsgFilterCustomAction **aCusto
 nsMsgFilter::nsMsgFilter():
     m_temporary(false),
     m_unparseable(false),
-    m_filterList(nsnull),
-    m_expressionTree(nsnull)
+    m_filterList(nullptr),
+    m_expressionTree(nullptr)
 {
   NS_NewISupportsArray(getter_AddRefs(m_termList));
-  NS_NewISupportsArray(getter_AddRefs(m_actionList));
 
   m_type = nsMsgFilterType::InboxRule | nsMsgFilterType::Manual;
 }
@@ -243,7 +243,7 @@ NS_IMETHODIMP nsMsgFilter::AppendTerm(nsIMsgSearchTerm * aTerm)
     NS_ENSURE_TRUE(aTerm, NS_ERROR_NULL_POINTER);
     // invalidate expression tree if we're changing the terms
     delete m_expressionTree;
-    m_expressionTree = nsnull;
+    m_expressionTree = nullptr;
     return m_termList->AppendElement(static_cast<nsISupports*>(aTerm));
 }
 
@@ -289,21 +289,24 @@ nsMsgFilter::CreateAction(nsIMsgRuleAction **aAction)
 //    m+1     MoveToFolder or Delete
 //    m+2     StopExecution
 NS_IMETHODIMP
-nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
+nsMsgFilter::GetSortedActionList(nsIArray **aActionList)
 {
-  NS_ENSURE_ARG_POINTER(actionList);
-  PRUint32 numActions;
-  nsresult rv = m_actionList->Count(&numActions);
+  NS_ENSURE_ARG_POINTER(aActionList);
+
+  uint32_t numActions;
+  nsresult rv = GetActionCount(&numActions);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMutableArray> orderedActions(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // hold separate pointers into the action list
-  PRUint32 nextIndexForNormal = 0, nextIndexForCopy = 0, nextIndexForMove = 0;
-  for (PRUint32 index = 0; index < numActions; ++index)
+  uint32_t nextIndexForNormal = 0, nextIndexForCopy = 0, nextIndexForMove = 0;
+  for (uint32_t index = 0; index < numActions; ++index)
   {
     nsCOMPtr<nsIMsgRuleAction> action;
-    rv = m_actionList->QueryElementAt(index, NS_GET_IID(nsIMsgRuleAction),
-                                      (void **)getter_AddRefs(action));
-    if (!action)
+    rv = GetActionAt(index, getter_AddRefs(action));
+    if (NS_FAILED(rv) || !action)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -313,7 +316,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::FetchBodyFromPop3Server:
       {
         // always insert in front
-        rv = actionList->InsertElementAt(action, 0);
+        rv = orderedActions->InsertElementAt(action, 0, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForNormal;
         ++nextIndexForCopy;
@@ -324,7 +327,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::CopyToFolder:
       {
         // insert into copy actions block, in order of appearance
-        rv = actionList->InsertElementAt(action, nextIndexForCopy);
+        rv = orderedActions->InsertElementAt(action, nextIndexForCopy, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForCopy;
         ++nextIndexForMove;
@@ -335,7 +338,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::Delete:
       {
         // insert into move/delete action block
-        rv = actionList->InsertElementAt(action, nextIndexForMove);
+        rv = orderedActions->InsertElementAt(action, nextIndexForMove, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForMove;
         break;
@@ -344,7 +347,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::StopExecution:
       {
         // insert into stop action block
-        rv = actionList->AppendElement(action);
+        rv = orderedActions->AppendElement(action, false);
         NS_ENSURE_SUCCESS(rv, rv);
         break;
       }
@@ -352,7 +355,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       default:
       {
         // insert into normal action block, in order of appearance
-        rv = actionList->InsertElementAt(action, nextIndexForNormal);
+        rv = orderedActions->InsertElementAt(action, nextIndexForNormal, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForNormal;
         ++nextIndexForCopy;
@@ -361,40 +364,48 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       }
     }
   }
-  return rv;
+
+  orderedActions.forget(aActionList);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgFilter::AppendAction(nsIMsgRuleAction *aAction)
 {
   NS_ENSURE_ARG_POINTER(aAction);
-  return m_actionList->AppendElement(static_cast<nsISupports*>(aAction));
+
+  m_actionList.AppendElement(aAction);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgFilter::GetActionAt(PRInt32 aIndex, nsIMsgRuleAction **aAction)
+nsMsgFilter::GetActionAt(uint32_t aIndex, nsIMsgRuleAction **aAction)
 {
   NS_ENSURE_ARG_POINTER(aAction);
-  return m_actionList->QueryElementAt(aIndex, NS_GET_IID(nsIMsgRuleAction),
-                                      (void **) aAction);
+  NS_ENSURE_ARG(aIndex < m_actionList.Length());
+
+  NS_ENSURE_TRUE(*aAction = m_actionList[aIndex], NS_ERROR_ILLEGAL_VALUE);
+  NS_IF_ADDREF(*aAction);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgFilter::GetActionList(nsISupportsArray **actionList)
+nsMsgFilter::GetActionCount(uint32_t *aCount)
 {
-  NS_ENSURE_ARG_POINTER(actionList);
+  NS_ENSURE_ARG_POINTER(aCount);
 
-  NS_IF_ADDREF(*actionList = m_actionList);
+  *aCount = m_actionList.Length();
   return NS_OK;
 }
 
 NS_IMETHODIMP  //for editing a filter
 nsMsgFilter::ClearActionList()
 {
-  return m_actionList->Clear();
+  m_actionList.Clear();
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilter::GetTerm(PRInt32 termIndex,
+NS_IMETHODIMP nsMsgFilter::GetTerm(int32_t termIndex,
                                    nsMsgSearchAttribValue *attrib,    /* attribute for this term          */
                                    nsMsgSearchOpValue *op,         /* operator e.g. opContains           */
                                    nsIMsgSearchValue **value,         /* value e.g. "Dogbert"               */
@@ -426,7 +437,7 @@ NS_IMETHODIMP nsMsgFilter::GetSearchTerms(nsISupportsArray **aResult)
     NS_ENSURE_ARG_POINTER(aResult);
     // caller can change m_termList, which can invalidate m_expressionTree.
     delete m_expressionTree;
-    m_expressionTree = nsnull;
+    m_expressionTree = nullptr;
     NS_IF_ADDREF(*aResult = m_termList);
     return NS_OK;
 }
@@ -434,7 +445,7 @@ NS_IMETHODIMP nsMsgFilter::GetSearchTerms(nsISupportsArray **aResult)
 NS_IMETHODIMP nsMsgFilter::SetSearchTerms(nsISupportsArray *aSearchList)
 {
     delete m_expressionTree;
-    m_expressionTree = nsnull;
+    m_expressionTree = nullptr;
     m_termList = aSearchList;
     return NS_OK;
 }
@@ -490,7 +501,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
         return NS_ERROR_FAILURE;
       }
     }
-    mDateFormatter->FormatPRExplodedTime(nsnull, kDateFormatShort,
+    mDateFormatter->FormatPRExplodedTime(nullptr, kDateFormatShort,
                                          kTimeFormatSeconds, &exploded,
                                          dateValue);
 
@@ -573,7 +584,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
     }
     buffer += "\n";
 
-    PRUint32 writeCount;
+    uint32_t writeCount;
 
     rv = logStream->Write(LOG_ENTRY_START_TAG, LOG_ENTRY_START_TAG_LEN, &writeCount);
     NS_ENSURE_SUCCESS(rv,rv);
@@ -586,7 +597,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
     if (!escapedBuffer)
       return NS_ERROR_OUT_OF_MEMORY;
 
-    PRUint32 escapedBufferLen = strlen(escapedBuffer);
+    uint32_t escapedBufferLen = strlen(escapedBuffer);
     rv = logStream->Write(escapedBuffer, escapedBufferLen, &writeCount);
     PR_Free(escapedBuffer);
     NS_ENSURE_SUCCESS(rv,rv);
@@ -601,7 +612,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
 NS_IMETHODIMP
 nsMsgFilter::MatchHdr(nsIMsgDBHdr *msgHdr, nsIMsgFolder *folder,
                       nsIMsgDatabase *db, const char *headers,
-                      PRUint32 headersSize, bool *pResult)
+                      uint32_t headersSize, bool *pResult)
 {
   NS_ENSURE_ARG_POINTER(folder);
   NS_ENSURE_ARG_POINTER(msgHdr);
@@ -613,19 +624,20 @@ nsMsgFilter::MatchHdr(nsIMsgDBHdr *msgHdr, nsIMsgFolder *folder,
   return rv;
 }
 
-void
+NS_IMETHODIMP
 nsMsgFilter::SetFilterList(nsIMsgFilterList *filterList)
 {
   // doesn't hold a ref.
   m_filterList = filterList;
+  return NS_OK;
 }
 
-nsresult
+NS_IMETHODIMP
 nsMsgFilter::GetFilterList(nsIMsgFilterList **aResult)
 {
-    NS_ENSURE_ARG_POINTER(aResult);
-    NS_IF_ADDREF(*aResult = m_filterList);
-    return NS_OK;
+  NS_ENSURE_ARG_POINTER(aResult);
+  NS_IF_ADDREF(*aResult = m_filterList);
+  return NS_OK;
 }
 
 void nsMsgFilter::SetFilterScript(nsCString *fileName)
@@ -636,7 +648,7 @@ void nsMsgFilter::SetFilterScript(nsCString *fileName)
 nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(nsIMsgRuleAction *filterAction, nsCString &moveValue)
 {
   NS_ENSURE_ARG_POINTER(filterAction);
-  PRInt16 filterVersion = kFileVersion;
+  int16_t filterVersion = kFileVersion;
   if (m_filterList)
       m_filterList->GetVersion(&filterVersion);
   if (filterVersion <= k60Beta1Version)
@@ -648,8 +660,8 @@ nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(nsIMsgRuleAction *filterAct
     // if relative path starts with kImap, this is a move to folder on the same server
     if (moveValue.Find(kImapPrefix) == 0)
     {
-      PRInt32 prefixLen = PL_strlen(kImapPrefix);
-      nsCAutoString originalServerPath(Substring(moveValue, prefixLen));
+      int32_t prefixLen = PL_strlen(kImapPrefix);
+      nsAutoCString originalServerPath(Substring(moveValue, prefixLen));
       if (filterVersion == k45Version)
       {
         nsAutoString unicodeStr;
@@ -704,7 +716,7 @@ nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(nsIMsgRuleAction *filterAct
         nsCString destFolderUri;
         destFolderUri.Assign( localRootURI);
         // need to remove ".sbd" from moveValue, and perhaps escape it.
-        PRInt32 offset = moveValue.Find(".sbd/");
+        int32_t offset = moveValue.Find(".sbd/");
         if (offset != -1)
           moveValue.Cut(offset, 4);
 
@@ -741,12 +753,13 @@ nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(nsIMsgRuleAction *filterAct
   // set m_action.m_value.m_folderUri
 }
 
-nsresult nsMsgFilter::SaveToTextFile(nsIOutputStream *aStream)
+NS_IMETHODIMP
+nsMsgFilter::SaveToTextFile(nsIOutputStream *aStream)
 {
   NS_ENSURE_ARG_POINTER(aStream);
   if (m_unparseable)
   {
-    PRUint32 bytesWritten;
+    uint32_t bytesWritten;
     //we need to trim leading whitespaces before filing out
     m_unparsedBuffer.Trim(kWhitespace, true /*leadingCharacters*/, false /*trailingCharacters*/);
     return aStream->Write(m_unparsedBuffer.get(), m_unparsedBuffer.Length(), &bytesWritten);
@@ -767,18 +780,17 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
   nsresult err = NS_OK;
   nsCOMPtr<nsIMsgFilterList> filterList;
   GetFilterList(getter_AddRefs(filterList));
-  nsCAutoString  actionFilingStr;
+  nsAutoCString  actionFilingStr;
 
-  PRUint32 numActions;
-  err = m_actionList->Count(&numActions);
+  uint32_t numActions;
+  err = GetActionCount(&numActions);
   NS_ENSURE_SUCCESS(err, err);
 
-  for (PRUint32 index =0; index < numActions; index++)
+  for (uint32_t index = 0; index < numActions; index++)
   {
     nsCOMPtr<nsIMsgRuleAction> action;
-    err = m_actionList->QueryElementAt(index, NS_GET_IID(nsIMsgRuleAction),
-                                       (void **)getter_AddRefs(action));
-    if (!action)
+    err = GetActionAt(index, getter_AddRefs(action));
+    if (NS_FAILED(err) || !action)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -802,7 +814,7 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
       {
         nsMsgPriorityValue priorityValue;
         action->GetPriority(&priorityValue);
-        nsCAutoString priority;
+        nsAutoCString priority;
         NS_MsgGetUntranslatedPriorityName(priorityValue, priority);
         err = filterList->WriteStrAttr(
                 nsIMsgFilterList::attribActionValue, priority.get(), aStream);
@@ -817,7 +829,7 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
       break;
       case nsMsgFilterAction::JunkScore:
       {
-        PRInt32 junkScore;
+        int32_t junkScore;
         action->GetJunkScore(&junkScore);
         err = filterList->WriteIntAttr(nsIMsgFilterList::attribActionValue, junkScore, aStream);
       }
@@ -834,10 +846,10 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
       break;
       case nsMsgFilterAction::Custom:
       {
-        nsCAutoString id;
+        nsAutoCString id;
         action->GetCustomId(id);
         err = filterList->WriteStrAttr(nsIMsgFilterList::attribCustomId, id.get(), aStream);
-        nsCAutoString strValue;
+        nsAutoCString strValue;
         action->GetStrValue(strValue);
         if (strValue.Length())
           err = filterList->WriteWstrAttr(nsIMsgFilterList::attribActionValue,
@@ -851,7 +863,7 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
     }
   }
   // and here the fun begins - file out term list...
-  nsCAutoString  condition;
+  nsAutoCString  condition;
   err = MsgTermListToString(m_termList, condition);
   if (NS_SUCCEEDED(err))
     err = filterList->WriteStrAttr(nsIMsgFilterList::attribCondition, condition.get(), aStream);
@@ -924,11 +936,11 @@ nsMsgRuleActionType nsMsgFilter::GetActionForFilingStr(nsCString &actionStr)
   return nsMsgFilterAction::None;
 }
 
-PRInt16
+int16_t
 nsMsgFilter::GetVersion()
 {
     if (!m_filterList) return 0;
-    PRInt16 version;
+    int16_t version;
     m_filterList->GetVersion(&version);
     return version;
 }
@@ -936,7 +948,7 @@ nsMsgFilter::GetVersion()
 #ifdef DEBUG
 void nsMsgFilter::Dump()
 {
-  nsCAutoString s;
+  nsAutoCString s;
   LossyCopyUTF16toASCII(m_filterName, s);
   printf("filter %s type = %c desc = %s\n", s.get(), m_type + '0', m_description.get());
 }
