@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Howard Chu <hyc@symas.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Implementation of db search for POP and offline IMAP mail folders
 
@@ -56,6 +23,7 @@
 #include "nsIMsgFilterPlugin.h"
 #include "nsMsgMessageFlags.h"
 #include "nsMsgUtils.h"
+#include "nsIMsgFolder.h"
 
 extern "C"
 {
@@ -93,10 +61,10 @@ nsMsgSearchBoolExpression * nsMsgSearchBoolExpression::AddExpressionTree(nsMsgSe
 
 nsMsgSearchBoolExpression::nsMsgSearchBoolExpression()
 {
-    m_term = nsnull;
+    m_term = nullptr;
     m_boolOp = nsMsgSearchBooleanOp::BooleanAND;
-    m_leftChild = nsnull;
-    m_rightChild = nsnull;
+    m_leftChild = nullptr;
+    m_rightChild = nullptr;
 }
 
 nsMsgSearchBoolExpression::nsMsgSearchBoolExpression (nsIMsgSearchTerm * newTerm, char * encodingStr)
@@ -109,8 +77,8 @@ nsMsgSearchBoolExpression::nsMsgSearchBoolExpression (nsIMsgSearchTerm * newTerm
     m_boolOp = nsMsgSearchBooleanOp::BooleanAND;
 
     // this expression does not contain sub expressions
-    m_leftChild = nsnull;
-    m_rightChild = nsnull;
+    m_leftChild = nullptr;
+    m_rightChild = nullptr;
 }
 
 
@@ -122,7 +90,7 @@ nsMsgSearchBoolExpression::nsMsgSearchBoolExpression (nsMsgSearchBoolExpression
     m_rightChild = expr2;
     m_boolOp = boolOp;
 
-    m_term = nsnull;
+    m_term = nullptr;
 }
 
 nsMsgSearchBoolExpression::~nsMsgSearchBoolExpression()
@@ -161,7 +129,7 @@ nsMsgSearchBoolExpression::leftToRightAddTerm(nsIMsgSearchTerm * newTerm, char *
 // returns true or false depending on what the current expression evaluates to.
 bool nsMsgSearchBoolExpression::OfflineEvaluate(nsIMsgDBHdr *msgToMatch, const char *defaultCharset,
   nsIMsgSearchScopeTerm *scope, nsIMsgDatabase *db, const char *headers,
-  PRUint32 headerSize, bool Filtering)
+  uint32_t headerSize, bool Filtering)
 {
     bool result = true;    // always default to false positives
     bool isAnd;
@@ -201,7 +169,7 @@ bool nsMsgSearchBoolExpression::OfflineEvaluate(nsIMsgDBHdr *msgToMatch, const c
 const int sizeOfORTerm = 6+1;  // 6 bytes if we are combining two sub expressions with an OR term
 const int sizeOfANDTerm = 1+1; // 1 byte if we are combining two sub expressions with an AND term
 
-PRInt32 nsMsgSearchBoolExpression::CalcEncodeStrSize()
+int32_t nsMsgSearchBoolExpression::CalcEncodeStrSize()
 // recursively examine each sub expression and calculate a final size for the entire IMAP/NNTP encoding
 {
     if (!m_term && (!m_leftChild || !m_rightChild))   // is the expression empty?
@@ -238,7 +206,7 @@ void nsMsgSearchBoolExpression::GenerateEncodeStr(nsCString * buffer)
 
         // HACK ALERT!!! if last returned character in the buffer is now a ' ' then we need to remove it because we don't want
         // a ' ' to preceded the closing paren in the OR encoding.
-        PRUint32 lastCharPos = buffer->Length() - 1;
+        uint32_t lastCharPos = buffer->Length() - 1;
         if (buffer->CharAt(lastCharPos) == ' ')
         {
             buffer->SetLength(lastCharPos);
@@ -343,7 +311,7 @@ nsMsgSearchOfflineMail::MatchTermsForFilter(nsIMsgDBHdr *msgToMatch,
                                             nsIMsgSearchScopeTerm * scope,
                                             nsIMsgDatabase * db,
                                             const char * headers,
-                                            PRUint32 headerSize,
+                                            uint32_t headerSize,
                                             nsMsgSearchBoolExpression ** aExpressionTree,
                                             bool *pResult)
 {
@@ -361,12 +329,12 @@ nsMsgSearchOfflineMail::MatchTermsForSearch(nsIMsgDBHdr *msgToMatch,
                                             bool *pResult)
 {
 
-    return MatchTerms(msgToMatch, termList, defaultCharset, scope, db, nsnull, 0, false, aExpressionTree, pResult);
+    return MatchTerms(msgToMatch, termList, defaultCharset, scope, db, nullptr, 0, false, aExpressionTree, pResult);
 }
 
 nsresult nsMsgSearchOfflineMail::ConstructExpressionTree(nsISupportsArray * termList,
-                                            PRUint32 termCount,
-                                            PRUint32 &aStartPosInList,
+                                            uint32_t termCount,
+                                            uint32_t &aStartPosInList,
                                             nsMsgSearchBoolExpression ** aExpressionTree)
 {
   nsMsgSearchBoolExpression * finalExpression = *aExpressionTree;
@@ -409,7 +377,7 @@ nsresult nsMsgSearchOfflineMail::ConstructExpressionTree(nsISupportsArray * term
       }
       else
       {
-        finalExpression = nsMsgSearchBoolExpression::AddSearchTerm(finalExpression, pTerm, nsnull);    // add the term to the expression tree
+        finalExpression = nsMsgSearchBoolExpression::AddSearchTerm(finalExpression, pTerm, nullptr);    // add the term to the expression tree
 
         if (endsGrouping)
           break;
@@ -429,7 +397,7 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
                                             nsIMsgSearchScopeTerm * scope,
                                             nsIMsgDatabase * db,
                                             const char * headers,
-                                            PRUint32 headerSize,
+                                            uint32_t headerSize,
                                             bool Filtering,
                       bool *pResult)
 {
@@ -440,7 +408,7 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
     nsCString  msgCharset;
     const char *charset;
     bool charsetOverride = false; /* XXX BUG 68706 */
-    PRUint32 msgFlags;
+    uint32_t msgFlags;
     bool result;
     bool matchAll;
 
@@ -522,8 +490,8 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
       }
       case nsMsgSearchAttrib::Body:
        {
-         PRUint64 messageOffset;
-         PRUint32 lineCount;
+         uint64_t messageOffset;
+         uint32_t lineCount;
          msgToMatch->GetMessageOffset(&messageOffset);
          msgToMatch->GetLineCount(&lineCount);
          err = aTerm->MatchBody (scope, messageOffset, lineCount, charset, msgToMatch, db, &result);
@@ -550,7 +518,7 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
       }
       case nsMsgSearchAttrib::Size:
       {
-        PRUint32 messageSize;
+        uint32_t messageSize;
         msgToMatch->GetMessageSize(&messageSize);
         err = aTerm->MatchSize (messageSize, &result);
         break;
@@ -604,7 +572,7 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
       {
         // When the junk status is set by the plugin, use junkpercent (if available)
         // Otherwise, use the limits (0 or 100) depending on the junkscore.
-        PRUint32 junkPercent;
+        uint32_t junkPercent;
         nsresult rv;
         nsCString junkScoreOriginStr;
         nsCString junkPercentStr;
@@ -672,9 +640,9 @@ nsresult nsMsgSearchOfflineMail::ProcessSearchTerm(nsIMsgDBHdr *msgToMatch,
           if (attrib >= nsMsgSearchAttrib::OtherHeader &&
               attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes)
           {
-            PRUint32 lineCount;
+            uint32_t lineCount;
             msgToMatch->GetLineCount(&lineCount);
-            PRUint64 messageOffset;
+            uint64_t messageOffset;
             msgToMatch->GetMessageOffset(&messageOffset);
             err = aTerm->MatchArbitraryHeader(scope, lineCount,charset,
                                               charsetOverride, msgToMatch, db,
@@ -695,7 +663,7 @@ nsresult nsMsgSearchOfflineMail::MatchTerms(nsIMsgDBHdr *msgToMatch,
                                             nsIMsgSearchScopeTerm * scope,
                                             nsIMsgDatabase * db,
                                             const char * headers,
-                                            PRUint32 headerSize,
+                                            uint32_t headerSize,
                                             bool Filtering,
                                             nsMsgSearchBoolExpression ** aExpressionTree,
                                             bool *pResult)
@@ -705,8 +673,8 @@ nsresult nsMsgSearchOfflineMail::MatchTerms(nsIMsgDBHdr *msgToMatch,
 
   if (!*aExpressionTree)
   {
-    PRUint32 initialPos = 0;
-    PRUint32 count;
+    uint32_t initialPos = 0;
+    uint32_t count;
     termList->Count(&count);
     err = ConstructExpressionTree(termList, count, initialPos, aExpressionTree);
     if (NS_FAILED(err))
@@ -730,9 +698,9 @@ nsresult nsMsgSearchOfflineMail::Search (bool *aDone)
   NS_ENSURE_ARG(aDone);
   nsresult dbErr = NS_OK;
   nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
-  nsMsgSearchBoolExpression *expressionTree = nsnull;
+  nsMsgSearchBoolExpression *expressionTree = nullptr;
 
-  const PRUint32 kTimeSliceInMS = 200;
+  const uint32_t kTimeSliceInMS = 200;
 
   *aDone = false;
   // Try to open the DB lazily. This will set up a parser if one is required
@@ -773,8 +741,7 @@ nsresult nsMsgSearchOfflineMail::Search (bool *aDone)
           {
             AddResultElement (msgDBHdr);
           }
-          PRIntervalTime elapsedTime;
-          LL_SUB(elapsedTime, PR_IntervalNow(), startTime);
+          PRIntervalTime elapsedTime = PR_IntervalNow() - startTime;
           // check if more than kTimeSliceInMS milliseconds have elapsed in this time slice started
           if (PR_IntervalToMilliseconds(elapsedTime) > kTimeSliceInMS)
             break;
@@ -800,10 +767,10 @@ void nsMsgSearchOfflineMail::CleanUpScope()
   // Let go of the DB when we're done with it so we don't kill the db cache
   if (m_db)
   {
-    m_listContext = nsnull;
+    m_listContext = nullptr;
     m_db->Close(false);
   }
-  m_db = nsnull;
+  m_db = nullptr;
 
   if (m_scope)
     m_scope->CloseInputStream();
@@ -830,7 +797,7 @@ nsMsgSearchOfflineMail::Abort ()
     // Let go of the DB when we're done with it so we don't kill the db cache
     if (m_db)
         m_db->Close(true /* commit in case we downloaded new headers */);
-    m_db = nsnull;
+    m_db = nullptr;
     return nsMsgSearchAdapter::Abort ();
 }
 
@@ -1011,7 +978,7 @@ nsresult SetLocalNews(nsIMsgSearchValidityTable* aTable)
 
 nsresult nsMsgSearchValidityManager::InitLocalNewsTable()
 {
-  NS_ASSERTION (nsnull == m_localNewsTable, "already have local news validity table");
+  NS_ASSERTION (nullptr == m_localNewsTable, "already have local news validity table");
   nsresult rv = NewTable(getter_AddRefs(m_localNewsTable));
   NS_ENSURE_SUCCESS(rv, rv);
   return SetLocalNews(m_localNewsTable);
@@ -1019,7 +986,7 @@ nsresult nsMsgSearchValidityManager::InitLocalNewsTable()
 
 nsresult nsMsgSearchValidityManager::InitLocalNewsBodyTable()
 {
-  NS_ASSERTION (nsnull == m_localNewsBodyTable, "already have local news+body validity table");
+  NS_ASSERTION (nullptr == m_localNewsBodyTable, "already have local news+body validity table");
   nsresult rv = NewTable(getter_AddRefs(m_localNewsBodyTable));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = SetLocalNews(m_localNewsBodyTable);
@@ -1029,7 +996,7 @@ nsresult nsMsgSearchValidityManager::InitLocalNewsBodyTable()
 
 nsresult nsMsgSearchValidityManager::InitLocalNewsJunkTable()
 {
-  NS_ASSERTION (nsnull == m_localNewsJunkTable, "already have local news+junk validity table");
+  NS_ASSERTION (nullptr == m_localNewsJunkTable, "already have local news+junk validity table");
   nsresult rv = NewTable(getter_AddRefs(m_localNewsJunkTable));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = SetLocalNews(m_localNewsJunkTable);
@@ -1039,7 +1006,7 @@ nsresult nsMsgSearchValidityManager::InitLocalNewsJunkTable()
 
 nsresult nsMsgSearchValidityManager::InitLocalNewsJunkBodyTable()
 {
-  NS_ASSERTION (nsnull == m_localNewsJunkBodyTable, "already have local news+junk+body validity table");
+  NS_ASSERTION (nullptr == m_localNewsJunkBodyTable, "already have local news+junk+body validity table");
   nsresult rv = NewTable(getter_AddRefs(m_localNewsJunkBodyTable));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = SetLocalNews(m_localNewsJunkBodyTable);
