@@ -1,41 +1,7 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Alec Flett <alecf@netscape.com>
- *   Seth Spitzer <sspitzer@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * RDF datasource for the account manager
@@ -58,6 +24,8 @@
 #include "nsComponentManagerUtils.h"
 #include "nsArrayEnumerator.h"
 #include "nsMsgUtils.h"
+#include "mozilla/Services.h"
+#include "nsArrayUtils.h"
 
 // turn this on to see useful output
 #undef DEBUG_amds
@@ -76,56 +44,46 @@
 
 #define NC_RDF_ACCOUNTROOT "msgaccounts:/"
 
-typedef struct _serverCreationParams {
-  nsCOMArray<nsIRDFResource> *serverArray;
-  nsIRDFService *rdfService;
-} serverCreationParams;
-
-typedef struct {
-  nsCString serverKey;
-  bool found;
-} findServerByKeyEntry;
-
 // the root resource (msgaccounts:/)
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_AccountRoot=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_AccountRoot=nullptr;
 
 // attributes of accounts
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Name=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeName=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeSimpleName=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_NameSort=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeNameSort=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTag=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_IsDefaultServer=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_SupportsFilters=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetMessages=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetIncomingMessages=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Name=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeName=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeSimpleName=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_NameSort=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_FolderTreeNameSort=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTag=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_IsDefaultServer=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_SupportsFilters=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetMessages=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetIncomingMessages=nullptr;
 
 // containment
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Child=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Settings=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Child=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Settings=nullptr;
 
 
 // properties corresponding to interfaces
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Account=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Server=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Identity=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Junk=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Account=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Server=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Identity=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Junk=nullptr;
 
 // individual pages
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleMain=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleServer=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleCopies=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleSynchronization=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleDiskSpace=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleAddressing=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleSMTP=nsnull;
-nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleJunk=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleMain=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleServer=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleCopies=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleSynchronization=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleDiskSpace=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleAddressing=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleSMTP=nullptr;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTitleJunk=nullptr;
 
 // common literals
-nsIRDFLiteral* nsMsgAccountManagerDataSource::kTrueLiteral = nsnull;
+nsIRDFLiteral* nsMsgAccountManagerDataSource::kTrueLiteral = nullptr;
 
-nsIAtom* nsMsgAccountManagerDataSource::kDefaultServerAtom = nsnull;
+nsIAtom* nsMsgAccountManagerDataSource::kDefaultServerAtom = nullptr;
 
 nsrefcnt nsMsgAccountManagerDataSource::gAccountManagerResourceRefCnt = 0;
 
@@ -247,8 +205,8 @@ nsMsgAccountManagerDataSource::~nsMsgAccountManagerDataSource()
 
 
     NS_IF_RELEASE(kDefaultServerAtom);
-    mAccountArcsOut = nsnull;
-    mAccountRootArcsOut = nsnull;
+    mAccountArcsOut = nullptr;
+    mAccountRootArcsOut = nullptr;
   }
 
 }
@@ -358,8 +316,8 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
         // allow for the accountmanager to be dynamically extended.
 
         nsCOMPtr<nsIStringBundleService> strBundleService =
-          do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-        NS_ENSURE_SUCCESS(rv,rv);
+          mozilla::services::GetStringBundleService();
+        NS_ENSURE_TRUE(strBundleService, NS_ERROR_UNEXPECTED);
 
         const char *sourceValue;
         rv = source->GetValueConst(&sourceValue);
@@ -369,14 +327,14 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
         NS_ENSURE_TRUE(sourceValue && (strlen(sourceValue) > strlen(NC_RDF_PAGETITLE_PREFIX)), NS_ERROR_UNEXPECTED);
 
         nsCOMPtr<nsIMsgAccountManager> am = do_QueryReferent(mAccountManager, &rv);
-        NS_ENSURE_SUCCESS(rv, false);
+        NS_ENSURE_SUCCESS(rv, rv);
 
         // turn NC#PageTitlefoobar into foobar, so we can get the am-foobar.properties bundle
         nsCString chromePackageName;
         rv = am->GetChromePackageName(nsCString(sourceValue + strlen(NC_RDF_PAGETITLE_PREFIX)), chromePackageName);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsCAutoString bundleURL;
+        nsAutoCString bundleURL;
         bundleURL = "chrome://";
         bundleURL += chromePackageName;
         bundleURL += "/locale/am-";
@@ -457,26 +415,14 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
     rv = getServerForFolderNode(source, getter_AddRefs(server));
 
     if (NS_SUCCEEDED(rv) && server) {
-      PRInt32 accountNum;
+      int32_t accountNum;
       nsCOMPtr<nsIMsgAccountManager> am = do_QueryReferent(mAccountManager);
 
       if (isDefaultServer(server))
-        str.AssignLiteral("0000");
+        str.AssignLiteral("000000000");
       else {
-        rv = am->FindServerIndex(server, &accountNum);
-        if (NS_FAILED(rv)) return rv;
-
-        // this is a hack for now - hardcode server order by type
-        nsCString serverType;
-        server->GetType(serverType);
-
-        if (MsgLowerCaseEqualsLiteral(serverType, "none"))
-          accountNum += 2000;
-        else if (MsgLowerCaseEqualsLiteral(serverType, "nntp"))
-          accountNum += 3000;
-        else
-          accountNum += 1000;     // default is to appear at the top
-
+        rv = am->GetSortOrder(server, &accountNum);
+        NS_ENSURE_SUCCESS(rv, rv);
         str.AppendInt(accountNum);
       }
     }
@@ -494,7 +440,7 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
       // so that the folder data source will take care of it.
       if (sourceValue && (strncmp(sourceValue, NC_RDF_PAGETITLE_PREFIX, strlen(NC_RDF_PAGETITLE_PREFIX)) == 0)) {
         if (source == kNC_PageTitleSMTP)
-          str.AssignLiteral("4000");
+          str.AssignLiteral("900000000");
         else if (source == kNC_PageTitleServer)
           str.AssignLiteral("1");
         else if (source == kNC_PageTitleCopies)
@@ -615,15 +561,33 @@ nsMsgAccountManagerDataSource::createRootResources(nsIRDFResource *property,
     nsCOMPtr<nsIMsgAccountManager> am = do_QueryReferent(mAccountManager);
     if (!am) return NS_ERROR_FAILURE;
 
-    nsCOMPtr<nsISupportsArray> servers;
+    nsCOMPtr<nsIArray> servers;
     rv = am->GetAllServers(getter_AddRefs(servers));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    uint32_t length;
+    rv = servers->GetLength(&length);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    for (uint32_t i = 0; i < length; ++i)
+    {
+      nsCOMPtr<nsIMsgIncomingServer> server(do_QueryElementAt(servers, i, &rv));
+      if (NS_FAILED(rv))
+        continue;
+
+      nsCOMPtr<nsIMsgFolder> serverFolder;
+      rv = server->GetRootFolder(getter_AddRefs(serverFolder));
+      if (NS_FAILED(rv))
+        continue;
+
+      // add the resource to the array
+      nsCOMPtr<nsIRDFResource> serverResource = do_QueryInterface(serverFolder);
+      if (serverResource)
+        (void) aNodeArray->AppendObject(serverResource);
+    }
 
-    // fill up the nodes array with the RDF Resources for the servers
-    serverCreationParams params = { aNodeArray, getRDFService() };
-    servers->EnumerateForwards(createServerResources, (void*)&params);
 #ifdef DEBUG_amds
-    PRUint32 nodecount;
+    uint32_t nodecount;
     aNodeArray->GetLength(&nodecount);
     printf("GetTargets(): added %d servers on %s\n", nodecount,
            (const char*)property_arc);
@@ -660,7 +624,7 @@ nsMsgAccountManagerDataSource::appendGenericSettingsResources(nsIMsgIncomingServ
       if (NS_FAILED(rv) || !catEntry)
         break;
 
-      nsCAutoString entryString;
+      nsAutoCString entryString;
       rv = catEntry->GetData(entryString);
       if (NS_FAILED(rv))
         break;
@@ -704,7 +668,7 @@ nsMsgAccountManagerDataSource::appendGenericSetting(const char *name,
 
   nsCOMPtr <nsIRDFResource> resource;
 
-  nsCAutoString resourceStr;
+  nsAutoCString resourceStr;
   resourceStr = NC_RDF_PAGETITLE_PREFIX;
   resourceStr += name;
 
@@ -746,7 +710,7 @@ nsMsgAccountManagerDataSource::createSettingsResources(nsIRDFResource *aSource,
 
     // Check the offline capability before adding
     // offline item
-    PRInt32 offlineSupportLevel = 0;
+    int32_t offlineSupportLevel = 0;
     rv = server->GetOfflineSupportLevel(&offlineSupportLevel);
     NS_ENSURE_SUCCESS(rv,rv);
 
@@ -781,14 +745,14 @@ nsMsgAccountManagerDataSource::serverHasIdentities(nsIMsgIncomingServer* aServer
 
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsISupportsArray> identities;
+  nsCOMPtr<nsIArray> identities;
   rv = am->GetIdentitiesForServer(aServer, getter_AddRefs(identities));
 
   // no identities just means no arcs
   if (NS_FAILED(rv)) return NS_OK;
 
-  PRUint32 count;
-  rv = identities->Count(&count);
+  uint32_t count;
+  rv = identities->GetLength(&count);
   if (NS_FAILED(rv)) return NS_OK;
 
   if (count >0)
@@ -796,34 +760,6 @@ nsMsgAccountManagerDataSource::serverHasIdentities(nsIMsgIncomingServer* aServer
   return NS_OK;
 }
 
-// enumeration function to convert each server (element)
-// to an nsIRDFResource and append it to the array (in data)
-// always return true to try on every element instead of aborting early
-bool
-nsMsgAccountManagerDataSource::createServerResources(nsISupports *element,
-                                                     void *data)
-{
-  nsresult rv;
-  // get parameters out of the data argument
-  serverCreationParams *params = (serverCreationParams*)data;
-  nsCOMArray<nsIRDFResource> *servers = params->serverArray;
-  nsCOMPtr<nsIRDFService> rdf = params->rdfService;
-
-  // the server itself is in the element argument
-  nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(element, &rv);
-  if (NS_FAILED(rv)) return true;
-
-  nsCOMPtr <nsIMsgFolder> serverFolder;
-  rv = server->GetRootFolder(getter_AddRefs(serverFolder));
-  if(NS_FAILED(rv)) return true;
-
-  // add the resource to the array
-  nsCOMPtr<nsIRDFResource> serverResource = do_QueryInterface(serverFolder);
-  if(serverResource)
-    (void) servers->AppendObject(serverResource);
-  return true;
-}
-
 nsresult
 nsMsgAccountManagerDataSource::getAccountArcs(nsIMutableArray **aResult)
 {
@@ -874,15 +810,14 @@ nsMsgAccountManagerDataSource::getAccountRootArcs(nsIMutableArray **aResult)
 NS_IMETHODIMP
 nsMsgAccountManagerDataSource::HasArcOut(nsIRDFResource *source, nsIRDFResource *aArc, bool *result)
 {
-  nsresult rv = NS_OK;
   if (aArc == kNC_Settings) {
     // based on createSettingsResources()
     // we only have settings for local folders and servers with identities
     nsCOMPtr<nsIMsgIncomingServer> server;
-    rv = getServerForFolderNode(source, getter_AddRefs(server));
-    if (server) {
+    nsresult rv = getServerForFolderNode(source, getter_AddRefs(server));
+    if (NS_SUCCEEDED(rv) && server) {
       // Check the offline capability before adding arc
-      PRInt32 offlineSupportLevel = 0;
+      int32_t offlineSupportLevel = 0;
       (void) server->GetOfflineSupportLevel(&offlineSupportLevel);
       if (offlineSupportLevel >= OFFLINE_SUPPORT_LEVEL_REGULAR) {
         *result = true;
@@ -1033,7 +968,7 @@ nsMsgAccountManagerDataSource::canGetMessages(nsIMsgIncomingServer *aServer)
   nsresult rv = aServer->GetType(type);
   NS_ENSURE_SUCCESS(rv, false);
 
-  nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
+  nsAutoCString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
   contractid.Append(type);
 
   nsCOMPtr<nsIMsgProtocolInfo> protocolInfo = do_GetService(contractid.get(), &rv);
@@ -1052,7 +987,7 @@ nsMsgAccountManagerDataSource::canGetIncomingMessages(nsIMsgIncomingServer *aSer
   nsresult rv = aServer->GetType(type);
   NS_ENSURE_SUCCESS(rv, false);
 
-  nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
+  nsAutoCString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
   contractid.Append(type);
 
   nsCOMPtr<nsIMsgProtocolInfo> protocolInfo = do_GetService(contractid.get(), &rv);
@@ -1070,9 +1005,6 @@ nsMsgAccountManagerDataSource::HasAssertionAccountRoot(nsIRDFResource *aProperty
                                                        bool aTruthValue,
                                                        bool *_retval)
 {
-
-  nsresult rv;
-
   // set up default
   *_retval = false;
 
@@ -1080,7 +1012,7 @@ nsMsgAccountManagerDataSource::HasAssertionAccountRoot(nsIRDFResource *aProperty
   if (isContainment(aProperty)) {
 
     nsCOMPtr<nsIMsgIncomingServer> server;
-    rv = getServerForFolderNode(aTarget, getter_AddRefs(server));
+    nsresult rv = getServerForFolderNode(aTarget, getter_AddRefs(server));
     if (NS_FAILED(rv) || !server) return rv;
 
     nsCString serverKey;
@@ -1089,16 +1021,28 @@ nsMsgAccountManagerDataSource::HasAssertionAccountRoot(nsIRDFResource *aProperty
     nsCOMPtr<nsIMsgAccountManager> am = do_QueryReferent(mAccountManager, &rv);
     if (NS_FAILED(rv)) return rv;
 
-    nsCOMPtr<nsISupportsArray> serverArray;
+    nsCOMPtr<nsIArray> serverArray;
     rv = am->GetAllServers(getter_AddRefs(serverArray));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    findServerByKeyEntry entry;
-    entry.serverKey = serverKey;
-    entry.found = false;
+    uint32_t length;
+    rv = serverArray->GetLength(&length);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    serverArray->EnumerateForwards(findServerByKey, &entry);
-    (*_retval) = entry.found;
+    for (uint32_t i = 0; i < length; ++i)
+    {
+      nsCOMPtr<nsIMsgIncomingServer> server(do_QueryElementAt(serverArray, i, &rv));
+      if (NS_FAILED(rv))
+        continue;
+
+      nsCString key;
+      server->GetKey(key);
+      if (key.Equals(serverKey))
+      {
+        *_retval = true;
+        break;
+      }
+    }
   }
 
   return NS_OK;
@@ -1131,37 +1075,14 @@ nsMsgAccountManagerDataSource::getServerForFolderNode(nsIRDFNode *aResource,
   return NS_ERROR_FAILURE;
 }
 
-bool
-nsMsgAccountManagerDataSource::findServerByKey(nsISupports *aElement,
-                                               void *aData)
-{
-  nsresult rv;
-  findServerByKeyEntry *entry = (findServerByKeyEntry*)aData;
-
-  nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(aElement, &rv);
-  if (NS_FAILED(rv)) return true;
-
-  nsCString key;
-  server->GetKey(key);
-  if (key.Equals(entry->serverKey))
-  {
-    entry->found = true;
-    return false;        // stop when found
-  }
-
-  return true;
-}
-
 nsresult
 nsMsgAccountManagerDataSource::getStringBundle()
 {
   if (mStringBundle) return NS_OK;
 
-  nsresult rv;
-
   nsCOMPtr<nsIStringBundleService> strBundleService =
-      do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
+    mozilla::services::GetStringBundleService();
+  NS_ENSURE_TRUE(strBundleService, NS_ERROR_UNEXPECTED);
 
   return strBundleService->CreateBundle("chrome://messenger/locale/prefs.properties",
                                       getter_AddRefs(mStringBundle));
@@ -1177,8 +1098,8 @@ nsMsgAccountManagerDataSource::OnServerLoaded(nsIMsgIncomingServer* aServer)
   nsCOMPtr<nsIRDFResource> serverResource = do_QueryInterface(serverFolder,&rv);
   if (NS_FAILED(rv)) return rv;
 
-  NotifyObservers(kNC_AccountRoot, kNC_Child, serverResource, nsnull, true, false);
-  NotifyObservers(kNC_AccountRoot, kNC_Settings, serverResource, nsnull, true, false);
+  NotifyObservers(kNC_AccountRoot, kNC_Child, serverResource, nullptr, true, false);
+  NotifyObservers(kNC_AccountRoot, kNC_Settings, serverResource, nullptr, true, false);
 
   return NS_OK;
 }
@@ -1194,8 +1115,8 @@ nsMsgAccountManagerDataSource::OnServerUnloaded(nsIMsgIncomingServer* aServer)
   if (NS_FAILED(rv)) return rv;
 
 
-  NotifyObservers(kNC_AccountRoot, kNC_Child, serverResource, nsnull, false, false);
-  NotifyObservers(kNC_AccountRoot, kNC_Settings, serverResource, nsnull, false, false);
+  NotifyObservers(kNC_AccountRoot, kNC_Child, serverResource, nullptr, false, false);
+  NotifyObservers(kNC_AccountRoot, kNC_Settings, serverResource, nullptr, false, false);
 
   return NS_OK;
 }
@@ -1225,7 +1146,7 @@ nsMsgAccountManagerDataSource::OnItemRemoved(nsIMsgFolder *, nsISupports *)
 }
 
 nsresult
-nsMsgAccountManagerDataSource::OnItemPropertyFlagChanged(nsIMsgDBHdr *, nsIAtom *, PRUint32, PRUint32)
+nsMsgAccountManagerDataSource::OnItemPropertyFlagChanged(nsIMsgDBHdr *, nsIAtom *, uint32_t, uint32_t)
 {
   return NS_OK;
 }
@@ -1245,7 +1166,7 @@ nsMsgAccountManagerDataSource::OnItemBoolPropertyChanged(nsIMsgFolder *aItem,
 {
   if (aProperty == kDefaultServerAtom) {
     nsCOMPtr<nsIRDFResource> resource(do_QueryInterface(aItem));
-    NotifyObservers(resource, kNC_IsDefaultServer, kTrueLiteral, nsnull, aNewValue, false);
+    NotifyObservers(resource, kNC_IsDefaultServer, kTrueLiteral, nullptr, aNewValue, false);
   }
   return NS_OK;
 }
@@ -1257,7 +1178,7 @@ nsMsgAccountManagerDataSource::OnItemEvent(nsIMsgFolder *, nsIAtom *)
 }
 
 nsresult
-nsMsgAccountManagerDataSource::OnItemIntPropertyChanged(nsIMsgFolder *, nsIAtom *, PRInt32, PRInt32)
+nsMsgAccountManagerDataSource::OnItemIntPropertyChanged(nsIMsgFolder *, nsIAtom *, int32_t, int32_t)
 {
   return NS_OK;
 }
