@@ -148,7 +148,7 @@ bool nsMsgContentPolicy::IsTrustedDomain(nsIURI * aContentLocation)
 {
   bool trustedDomain = false;
   // get the host name of the server hosting the remote image
-  nsCAutoString host;
+  nsAutoCString host;
   nsresult rv = aContentLocation->GetHost(host);
 
   if (NS_SUCCEEDED(rv) && !mTrustedMailDomains.IsEmpty()) 
@@ -158,13 +158,14 @@ bool nsMsgContentPolicy::IsTrustedDomain(nsIURI * aContentLocation)
 }
 
 NS_IMETHODIMP
-nsMsgContentPolicy::ShouldLoad(PRUint32          aContentType,
+nsMsgContentPolicy::ShouldLoad(uint32_t          aContentType,
                                nsIURI           *aContentLocation,
                                nsIURI           *aRequestingLocation,
                                nsISupports      *aRequestingContext,
                                const nsACString &aMimeGuess,
                                nsISupports      *aExtra,
-                               PRInt16          *aDecision)
+                               nsIPrincipal     *aRequestPrincipal,
+                               int16_t          *aDecision)
 {
   nsresult rv = NS_OK;
   // The default decision at the start of the function is to accept the load.
@@ -197,7 +198,7 @@ nsMsgContentPolicy::ShouldLoad(PRUint32          aContentType,
                                  getter_AddRefs(rootDocShell));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 appType;
+  uint32_t appType;
   rv = rootDocShell->GetAppType(&appType);
   // We only want to deal with mailnews
   if (NS_FAILED(rv) || appType != nsIDocShell::APP_TYPE_MAIL)
@@ -228,6 +229,12 @@ nsMsgContentPolicy::ShouldLoad(PRUint32          aContentType,
     }
     break;
 
+  case nsIContentPolicy::TYPE_CSP_REPORT:
+    // We cannot block CSP reports.
+    *aDecision = nsIContentPolicy::ACCEPT;
+    return NS_OK;
+    break;
+
   default:
     break;
   }
@@ -298,8 +305,8 @@ nsMsgContentPolicy::ShouldLoad(PRUint32          aContentType,
   bool isHttp;
   bool isHttps;
   rv = originatorLocation->SchemeIs("http", &isHttp);
-  rv |= originatorLocation->SchemeIs("https", &isHttps);
-  if (NS_SUCCEEDED(rv) && (isHttp || isHttps))
+  nsresult rv2 = originatorLocation->SchemeIs("https", &isHttps);
+  if (NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) && (isHttp || isHttps))
   {
     *aDecision = nsIContentPolicy::ACCEPT;
     return NS_OK;
@@ -329,9 +336,10 @@ nsMsgContentPolicy::IsSafeRequestingLocation(nsIURI *aRequestingLocation)
   bool isFile;
 
   nsresult rv = aRequestingLocation->SchemeIs("chrome", &isChrome);
-  rv |= aRequestingLocation->SchemeIs("resource", &isRes);
-  rv |= aRequestingLocation->SchemeIs("file", &isFile);
-
+  NS_ENSURE_SUCCESS(rv, false);
+  rv = aRequestingLocation->SchemeIs("resource", &isRes);
+  NS_ENSURE_SUCCESS(rv, false);
+  rv = aRequestingLocation->SchemeIs("file", &isFile);
   NS_ENSURE_SUCCESS(rv, false);
 
   if (isChrome || isRes || isFile)
@@ -360,7 +368,7 @@ nsMsgContentPolicy::IsSafeRequestingLocation(nsIURI *aRequestingLocation)
 bool
 nsMsgContentPolicy::IsExposedProtocol(nsIURI *aContentLocation)
 {
-  nsCAutoString contentScheme;
+  nsAutoCString contentScheme;
   nsresult rv = aContentLocation->GetScheme(contentScheme);
   NS_ENSURE_SUCCESS(rv, false);
 
@@ -381,9 +389,10 @@ nsMsgContentPolicy::IsExposedProtocol(nsIURI *aContentLocation)
   bool isChrome;
   bool isRes;
   rv = aContentLocation->SchemeIs("chrome", &isChrome);
-  rv |= aContentLocation->SchemeIs("resource", &isRes);
-  rv |= aContentLocation->SchemeIs("data", &isData);
-
+  NS_ENSURE_SUCCESS(rv, false);
+  rv = aContentLocation->SchemeIs("resource", &isRes);
+  NS_ENSURE_SUCCESS(rv, false);
+  rv = aContentLocation->SchemeIs("data", &isData);
   NS_ENSURE_SUCCESS(rv, false);
 
   return isChrome || isRes || isData;
@@ -398,11 +407,12 @@ nsMsgContentPolicy::ShouldBlockUnexposedProtocol(nsIURI *aContentLocation)
   bool isHttp;
   bool isHttps;
   bool isFile;
-  nsresult rv = aContentLocation->SchemeIs("http", &isHttp);
-  rv |= aContentLocation->SchemeIs("https", &isHttps);
-  rv |= aContentLocation->SchemeIs("file", &isFile);
-
   // Error condition - we must return true so that we block.
+  nsresult rv = aContentLocation->SchemeIs("http", &isHttp);
+  NS_ENSURE_SUCCESS(rv, true);
+  rv = aContentLocation->SchemeIs("https", &isHttps);
+  NS_ENSURE_SUCCESS(rv, true);
+  rv = aContentLocation->SchemeIs("file", &isFile);
   NS_ENSURE_SUCCESS(rv, true);
 
   return !isHttp && !isHttps && !isFile;
@@ -418,17 +428,17 @@ nsMsgContentPolicy::ShouldBlockUnexposedProtocol(nsIURI *aContentLocation)
  * #3 Allow if the domain for the remote image in our white list.
  * #4 Allow if the author has been specifically white listed.
  */
-PRInt16
+int16_t
 nsMsgContentPolicy::ShouldAcceptRemoteContentForMsgHdr(nsIMsgDBHdr *aMsgHdr,
                                                        nsIURI *aRequestingLocation,
                                                        nsIURI *aContentLocation)
 {
   if (!aMsgHdr)
-    return static_cast<PRInt16>(nsIContentPolicy::REJECT_REQUEST);
+    return static_cast<int16_t>(nsIContentPolicy::REJECT_REQUEST);
 
   // Case #1, check the db hdr for the remote content policy on this particular
   // message.
-  PRUint32 remoteContentPolicy = kNoRemoteContentPolicy;
+  uint32_t remoteContentPolicy = kNoRemoteContentPolicy;
   aMsgHdr->GetUint32Property("remoteContentPolicy", &remoteContentPolicy);
 
   // Case #2, check if the message is in an RSS folder
@@ -446,9 +456,9 @@ nsMsgContentPolicy::ShouldAcceptRemoteContentForMsgHdr(nsIMsgDBHdr *aMsgHdr,
   // Case #4, author is in our white list..
   bool allowForSender = ShouldAcceptRemoteContentForSender(aMsgHdr);
 
-  PRInt16 result = allowForSender ?
-    static_cast<PRInt16>(nsIContentPolicy::ACCEPT) :
-    static_cast<PRInt16>(nsIContentPolicy::REJECT_REQUEST);
+  int16_t result = allowForSender ?
+    static_cast<int16_t>(nsIContentPolicy::ACCEPT) :
+    static_cast<int16_t>(nsIContentPolicy::REJECT_REQUEST);
 
   // kNoRemoteContentPolicy means we have never set a value on the message
   if (result == nsIContentPolicy::REJECT_REQUEST && !remoteContentPolicy)
@@ -492,7 +502,7 @@ private:
 void
 nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(nsIURI *aOriginatorLocation,
                                                        nsIURI *aContentLocation,
-                                                       PRInt16 *aDecision)
+                                                       int16_t *aDecision)
 {
   NS_PRECONDITION(*aDecision == nsIContentPolicy::REJECT_REQUEST,
                   "AllowContentForPotentialMessage expects default decision to be reject!");
@@ -511,14 +521,14 @@ nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(nsIURI *aOriginatorLocati
 
   nsCString resourceURI;
   rv = msgUrl->GetUri(getter_Copies(resourceURI));
-  NS_ENSURE_SUCCESS(rv, );
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
   rv = GetMsgDBHdrFromURI(resourceURI.get(), getter_AddRefs(msgHdr));
-  NS_ENSURE_SUCCESS(rv, );
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl(do_QueryInterface(aOriginatorLocation, &rv));
-  NS_ENSURE_SUCCESS(rv, );
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   // Get a decision on whether or not to allow remote content for this message
   // header.
@@ -551,19 +561,18 @@ nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(nsIURI *aOriginatorLocati
 void nsMsgContentPolicy::ComposeShouldLoad(nsIMsgCompose *aMsgCompose,
                                            nsISupports *aRequestingContext,
                                            nsIURI *aContentLocation,
-                                           PRInt16 *aDecision)
+                                           int16_t *aDecision)
 {
   NS_PRECONDITION(*aDecision == nsIContentPolicy::REJECT_REQUEST,
                   "ComposeShouldLoad expects default decision to be reject!");
 
-  nsresult rv;
   nsCString originalMsgURI;
-  rv = aMsgCompose->GetOriginalMsgURI(getter_Copies(originalMsgURI));
-  NS_ENSURE_SUCCESS(rv, );
+  nsresult rv = aMsgCompose->GetOriginalMsgURI(getter_Copies(originalMsgURI));
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   MSG_ComposeType composeType;
   rv = aMsgCompose->GetType(&composeType);
-  NS_ENSURE_SUCCESS(rv, );
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   // Only allow remote content for new mail compositions or mailto
   // Block remote content for all other types (drafts, templates, forwards, replies, etc)
@@ -576,8 +585,8 @@ void nsMsgContentPolicy::ComposeShouldLoad(nsIMsgCompose *aMsgCompose,
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr;
     rv = GetMsgDBHdrFromURI(originalMsgURI.get(), getter_AddRefs(msgHdr));
-    NS_ENSURE_SUCCESS(rv, );
-    *aDecision = ShouldAcceptRemoteContentForMsgHdr(msgHdr, nsnull,
+    NS_ENSURE_SUCCESS_VOID(rv);
+    *aDecision = ShouldAcceptRemoteContentForMsgHdr(msgHdr, nullptr,
                                                     aContentLocation);
 
     // Special case image elements. When replying to a message, we want to allow
@@ -608,21 +617,21 @@ already_AddRefed<nsIMsgCompose> nsMsgContentPolicy::GetMsgComposeForContext(nsIS
 
   nsIDocShell *shell = NS_CP_GetDocShellFromContext(aRequestingContext);
   nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem(do_QueryInterface(shell, &rv));
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
   nsCOMPtr<nsIDocShellTreeItem> rootItem;
   rv = docShellTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(rootItem));
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(rootItem, &rv));
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
   nsCOMPtr<nsIMsgComposeService> composeService(do_GetService(NS_MSGCOMPOSESERVICE_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
-  nsIMsgCompose* msgCompose = nsnull;
+  nsIMsgCompose* msgCompose = nullptr;
   // Don't bother checking rv, as GetMsgComposeForDocShell returns NS_ERROR_FAILURE
-  // for not found. We default to nsnull, so we're still returning a valid value.
+  // for not found. We default to nullptr, so we're still returning a valid value.
   composeService->GetMsgComposeForDocShell(docShell, &msgCompose);
   return msgCompose;
 }
@@ -669,7 +678,7 @@ nsresult nsMsgContentPolicy::SetDisableItemsOnMailNewsUrlDocshells(
   NS_ENSURE_SUCCESS(rv, rv);
 
   // what sort of docshell is this?
-  PRInt32 itemType;
+  int32_t itemType;
   rv = docshellTreeItem->GetItemType(&itemType);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -740,13 +749,14 @@ nsMsgContentPolicy::GetOriginatingURIForContext(nsISupports *aRequestingContext,
 }
 
 NS_IMETHODIMP
-nsMsgContentPolicy::ShouldProcess(PRUint32          aContentType,
+nsMsgContentPolicy::ShouldProcess(uint32_t          aContentType,
                                   nsIURI           *aContentLocation,
                                   nsIURI           *aRequestingLocation,
                                   nsISupports      *aRequestingContext,
                                   const nsACString &aMimeGuess,
                                   nsISupports      *aExtra,
-                                  PRInt16          *aDecision)
+                                  nsIPrincipal     *aRequestPrincipal,
+                                  int16_t          *aDecision)
 {
   // XXX Returning ACCEPT is presumably only a reasonable thing to do if we
   // think that ShouldLoad is going to catch all possible cases (i.e. that
@@ -783,7 +793,7 @@ NS_IMETHODIMP nsMsgContentPolicy::Observe(nsISupports *aSubject, const char *aTo
  */
 NS_IMETHODIMP 
 nsMsgContentPolicy::OnStateChange(nsIWebProgress *aWebProgress,
-                                  nsIRequest *aRequest, PRUint32 aStateFlags,
+                                  nsIRequest *aRequest, uint32_t aStateFlags,
                                   nsresult aStatus)
 {
   return NS_OK;
@@ -792,10 +802,10 @@ nsMsgContentPolicy::OnStateChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP
 nsMsgContentPolicy::OnProgressChange(nsIWebProgress *aWebProgress,
                                      nsIRequest *aRequest,
-                                     PRInt32 aCurSelfProgress,
-                                     PRInt32 aMaxSelfProgress,
-                                     PRInt32 aCurTotalProgress,
-                                     PRInt32 aMaxTotalProgress)
+                                     int32_t aCurSelfProgress,
+                                     int32_t aMaxSelfProgress,
+                                     int32_t aCurTotalProgress,
+                                     int32_t aMaxTotalProgress)
 {
   return NS_OK;
 }
@@ -803,7 +813,7 @@ nsMsgContentPolicy::OnProgressChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP 
 nsMsgContentPolicy::OnLocationChange(nsIWebProgress *aWebProgress,
                                      nsIRequest *aRequest, nsIURI *aLocation,
-                                     PRUint32 aFlags)
+                                     uint32_t aFlags)
 {
   nsresult rv;
 
@@ -863,7 +873,7 @@ nsMsgContentPolicy::OnStatusChange(nsIWebProgress *aWebProgress,
 
 NS_IMETHODIMP
 nsMsgContentPolicy::OnSecurityChange(nsIWebProgress *aWebProgress,
-                                     nsIRequest *aRequest, PRUint32 aState)
+                                     nsIRequest *aRequest, uint32_t aState)
 {
   return NS_OK;
 }
