@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   David Bienvenu <bienvenu@nventure.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMsgIncomingServer.h"
 #include "nscore.h"
@@ -76,7 +42,13 @@
 #include "nsIMsgMdnGenerator.h"
 #include "nsMsgFolderFlags.h"
 #include "nsMsgUtils.h"
+#include "nsMsgMessageFlags.h"
+#include "nsIMsgSearchTerm.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "mozilla/Services.h"
+#include "nsIMsgFilter.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 
 #define PORT_NOT_SET -1
 
@@ -136,7 +108,7 @@ nsMsgIncomingServer::SetKey(const nsACString& serverKey)
   nsCOMPtr<nsIPrefService> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString branchName;
+  nsAutoCString branchName;
   branchName.AssignLiteral("mail.server.");
   branchName.Append(m_serverKey);
   branchName.Append('.');
@@ -219,7 +191,7 @@ NS_IMETHODIMP nsMsgIncomingServer::SetPerformingBiff(bool aPerformingBiff)
   return NS_OK;
 }
 
-NS_IMPL_GETSET(nsMsgIncomingServer, BiffState, PRUint32, m_biffState)
+NS_IMPL_GETSET(nsMsgIncomingServer, BiffState, uint32_t, m_biffState)
 
 NS_IMETHODIMP nsMsgIncomingServer::WriteToFolderCache(nsIMsgFolderCache *folderCache)
 {
@@ -237,23 +209,23 @@ NS_IMETHODIMP
 nsMsgIncomingServer::Shutdown()
 {
   nsresult rv = CloseCachedConnections();
-  mFilterPlugin = nsnull;
+  mFilterPlugin = nullptr;
   NS_ENSURE_SUCCESS(rv,rv);
 
   if (mFilterList)
   {
     // close the filter log stream
-    rv = mFilterList->SetLogStream(nsnull);
+    rv = mFilterList->SetLogStream(nullptr);
     NS_ENSURE_SUCCESS(rv,rv);
-    mFilterList = nsnull;
+    mFilterList = nullptr;
   }
 
   if (mSpamSettings)
   {
     // close the spam log stream
-    rv = mSpamSettings->SetLogStream(nsnull);
+    rv = mSpamSettings->SetLogStream(nullptr);
     NS_ENSURE_SUCCESS(rv,rv);
-    mSpamSettings = nsnull;
+    mSpamSettings = nullptr;
   }
   return rv;
 }
@@ -332,15 +304,6 @@ nsMsgIncomingServer::GetCanEmptyTrashOnExit(bool *canEmptyTrashOnExit)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgIncomingServer::GetIsSecureServer(bool *isSecureServer)
-{
-  // derived class should override if they need to do this.
-  NS_ENSURE_ARG_POINTER(isSecureServer);
-  *isSecureServer = true;
-  return NS_OK;
-}
-
 // construct <localStoreType>://[<username>@]<hostname
 NS_IMETHODIMP
 nsMsgIncomingServer::GetServerURI(nsACString& aResult)
@@ -448,7 +411,7 @@ nsMsgIncomingServer::SetBoolValue(const char *prefname,
 
 NS_IMETHODIMP
 nsMsgIncomingServer::GetIntValue(const char *prefname,
-                                PRInt32 *val)
+                                int32_t *val)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
@@ -465,7 +428,7 @@ nsMsgIncomingServer::GetIntValue(const char *prefname,
 NS_IMETHODIMP
 nsMsgIncomingServer::GetFileValue(const char* aRelPrefName,
                                   const char* aAbsPrefName,
-                                  nsILocalFile** aLocalFile)
+                                  nsIFile** aLocalFile)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
@@ -482,7 +445,7 @@ nsMsgIncomingServer::GetFileValue(const char* aRelPrefName,
       (*aLocalFile)->Normalize();
   } else {
     rv = mPrefBranch->GetComplexValue(aAbsPrefName,
-                                      NS_GET_IID(nsILocalFile),
+                                      NS_GET_IID(nsIFile),
                                       reinterpret_cast<void**>(aLocalFile));
     if (NS_FAILED(rv))
       return rv;
@@ -502,7 +465,7 @@ nsMsgIncomingServer::GetFileValue(const char* aRelPrefName,
 NS_IMETHODIMP
 nsMsgIncomingServer::SetFileValue(const char* aRelPrefName,
                                   const char* aAbsPrefName,
-                                  nsILocalFile* aLocalFile)
+                                  nsIFile* aLocalFile)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
@@ -519,17 +482,17 @@ nsMsgIncomingServer::SetFileValue(const char* aRelPrefName,
     if (NS_FAILED(rv))
       return rv;
   }
-  return mPrefBranch->SetComplexValue(aAbsPrefName, NS_GET_IID(nsILocalFile), aLocalFile);
+  return mPrefBranch->SetComplexValue(aAbsPrefName, NS_GET_IID(nsIFile), aLocalFile);
 }
 
 NS_IMETHODIMP
 nsMsgIncomingServer::SetIntValue(const char *prefname,
-                                 PRInt32 val)
+                                 int32_t val)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
 
-  PRInt32 defaultVal;
+  int32_t defaultVal;
   nsresult rv = mDefPrefBranch->GetIntPref(prefname, &defaultVal);
 
   if (NS_SUCCEEDED(rv) && defaultVal == val)
@@ -727,8 +690,8 @@ nsresult nsMsgIncomingServer::GetPasswordWithoutUI()
 
   NS_ConvertUTF8toUTF16 currServer(currServerUri);
 
-  PRUint32 numLogins = 0;
-  nsILoginInfo** logins = nsnull;
+  uint32_t numLogins = 0;
+  nsILoginInfo** logins = nullptr;
   rv = loginMgr->FindLogins(&numLogins, currServer, EmptyString(),
                             currServer, &logins);
 
@@ -749,7 +712,7 @@ nsresult nsMsgIncomingServer::GetPasswordWithoutUI()
     NS_ConvertUTF8toUTF16 serverUsername(serverCUsername);
 
     nsString username;
-    for (PRUint32 i = 0; i < numLogins; ++i)
+    for (uint32_t i = 0; i < numLogins; ++i)
     {
       rv = logins[i]->GetUsername(username);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -795,15 +758,12 @@ nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
     // aMsgWindow is required if we need to prompt
     if (aMsgWindow)
     {
-      // prompt the user for the password
-      nsCOMPtr<nsIDocShell> docShell;
-      rv = aMsgWindow->GetRootDocShell(getter_AddRefs(docShell));
-      NS_ENSURE_SUCCESS(rv, rv);
-      dialog = do_GetInterface(docShell, &rv);
+      rv = aMsgWindow->GetAuthPrompt(getter_AddRefs(dialog));
       NS_ENSURE_SUCCESS(rv, rv);
     }
     if (dialog)
     {
+      // prompt the user for the password
       nsCString serverUri;
       rv = GetLocalStoreType(serverUri);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -829,7 +789,7 @@ nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
       // we pass in the previously used password, if any, into PromptPassword
       // so that it will appear as ******. This means we can't use an nsString
       // and getter_Copies.
-      PRUnichar *uniPassword = nsnull;
+      PRUnichar *uniPassword = nullptr;
       if (!aPassword.IsEmpty())
         uniPassword = ToNewUnicode(NS_ConvertASCIItoUTF16(aPassword));
 
@@ -880,7 +840,7 @@ nsMsgIncomingServer::ForgetPassword()
 
   currServerUri.Append(temp);
 
-  PRUint32 count;
+  uint32_t count;
   nsILoginInfo** logins;
 
   NS_ConvertUTF8toUTF16 currServer(currServerUri);
@@ -898,7 +858,7 @@ nsMsgIncomingServer::ForgetPassword()
   // There should only be one-login stored for this url, however just in case
   // there isn't.
   nsString username;
-  for (PRUint32 i = 0; i < count; ++i)
+  for (uint32_t i = 0; i < count; ++i)
   {
     if (NS_SUCCEEDED(logins[i]->GetUsername(username)) &&
         username.Equals(serverUsername))
@@ -921,7 +881,7 @@ nsMsgIncomingServer::ForgetSessionPassword()
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::SetDefaultLocalPath(nsILocalFile *aDefaultLocalPath)
+nsMsgIncomingServer::SetDefaultLocalPath(nsIFile *aDefaultLocalPath)
 {
   nsresult rv;
   nsCOMPtr<nsIMsgProtocolInfo> protocolInfo;
@@ -931,7 +891,7 @@ nsMsgIncomingServer::SetDefaultLocalPath(nsILocalFile *aDefaultLocalPath)
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::GetLocalPath(nsILocalFile **aLocalPath)
+nsMsgIncomingServer::GetLocalPath(nsIFile **aLocalPath)
 {
   nsresult rv;
 
@@ -948,7 +908,7 @@ nsMsgIncomingServer::GetLocalPath(nsILocalFile **aLocalPath)
   rv = getProtocolInfo(getter_AddRefs(protocolInfo));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsILocalFile> localPath;
+  nsCOMPtr<nsIFile> localPath;
   rv = protocolInfo->GetDefaultLocalPath(getter_AddRefs(localPath));
   NS_ENSURE_SUCCESS(rv, rv);
   localPath->Create(nsIFile::DIRECTORY_TYPE, 0755);
@@ -1001,7 +961,7 @@ nsMsgIncomingServer::GetMsgStore(nsIMsgPluggableStore **aMsgStore)
 
 
 NS_IMETHODIMP
-nsMsgIncomingServer::SetLocalPath(nsILocalFile *aLocalPath)
+nsMsgIncomingServer::SetLocalPath(nsIFile *aLocalPath)
 {
   NS_ENSURE_ARG_POINTER(aLocalPath);
   aLocalPath->Create(nsIFile::DIRECTORY_TYPE, 0755);
@@ -1072,7 +1032,7 @@ nsMsgIncomingServer::RemoveFiles()
     NS_ASSERTION(false, "shouldn't remove files for a deferred account");
     return NS_ERROR_FAILURE;
   }
-  nsCOMPtr <nsILocalFile> localPath;
+  nsCOMPtr <nsIFile> localPath;
   nsresult rv = GetLocalPath(getter_AddRefs(localPath));
   NS_ENSURE_SUCCESS(rv, rv);
   return localPath->Remove(true);
@@ -1104,7 +1064,7 @@ nsMsgIncomingServer::GetFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilterList **
 
       if (!filterType.IsEmpty() && !filterType.EqualsLiteral("default"))
       {
-        nsCAutoString contractID("@mozilla.org/filterlist;1?type=");
+        nsAutoCString contractID("@mozilla.org/filterlist;1?type=");
         contractID += filterType;
         ToLowerCase(contractID);
         mFilterList = do_CreateInstance(contractID.get(), &rv);
@@ -1120,7 +1080,7 @@ nsMsgIncomingServer::GetFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilterList **
       // The default case, a local folder, is a bit special. It requires
       // more initialization.
 
-      nsCOMPtr<nsILocalFile> thisFolder;
+      nsCOMPtr<nsIFile> thisFolder;
       rv = msgFolder->GetFilePath(getter_AddRefs(thisFolder));
       NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1135,7 +1095,7 @@ nsMsgIncomingServer::GetFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilterList **
       mFilterFile->Exists(&fileExists);
       if (!fileExists)
       {
-        nsCOMPtr<nsILocalFile> oldFilterFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
+        nsCOMPtr<nsIFile> oldFilterFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
         NS_ENSURE_SUCCESS(rv, rv);
         rv = oldFilterFile->InitWithFile(thisFolder);
         NS_ENSURE_SUCCESS(rv, rv);
@@ -1182,7 +1142,7 @@ nsMsgIncomingServer::GetEditableFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilte
     rv = GetCharValue("filter.editable.type", filterType);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString contractID("@mozilla.org/filterlist;1?type=");
+    nsAutoCString contractID("@mozilla.org/filterlist;1?type=");
     contractID += filterType;
     ToLowerCase(contractID);
     mEditableFilterList = do_CreateInstance(contractID.get(), &rv);
@@ -1213,13 +1173,13 @@ nsMsgIncomingServer::InternalSetHostName(const nsACString& aHostname, const char
 {
   nsCString hostname;
   hostname = aHostname;
-  if (hostname.CountChar(':') == 1)
+  if (MsgCountChar(hostname, ':') == 1)
   {
-    PRInt32 colonPos = hostname.FindChar(':');
-    nsCAutoString portString(Substring(hostname, colonPos));
+    int32_t colonPos = hostname.FindChar(':');
+    nsAutoCString portString(Substring(hostname, colonPos));
     hostname.SetLength(colonPos);
     nsresult err;
-    PRInt32 port = portString.ToInteger(&err);
+    int32_t port = portString.ToInteger(&err);
     if (NS_SUCCEEDED(err))
       SetPort(port);
   }
@@ -1227,7 +1187,9 @@ nsMsgIncomingServer::InternalSetHostName(const nsACString& aHostname, const char
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName, const nsACString& newName)
+nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
+                                             const nsACString& newName,
+                                             bool hostnameChanged)
 {
   nsresult rv;
 
@@ -1247,25 +1209,54 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName, const ns
   // 4. Lastly, replace all occurrences of old name in the acct name with the new one.
   nsString acctName;
   rv = GetPrettyName(acctName);
-  if (NS_SUCCEEDED(rv) && !acctName.IsEmpty())
-  {
-    PRInt32 match = 0;
-    PRUint32 offset = 0;
-    nsString oldSubstr = NS_ConvertASCIItoUTF16(oldName);
-    nsString newSubstr = NS_ConvertASCIItoUTF16(newName);
-    while (offset < acctName.Length()) {
-        match = acctName.Find(oldSubstr, offset);
-        if (match == -1)
-            break;
- 
-        acctName.Replace(offset + match, oldSubstr.Length(), newSubstr);
-        offset += (match + newSubstr.Length());
-    }
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    SetPrettyName(acctName);
+  NS_ENSURE_FALSE(acctName.IsEmpty(), NS_OK);
+
+  // exit if new name contains @ then better do not update the account name
+  if (!hostnameChanged && (newName.FindChar('@') != kNotFound))
+    return NS_OK;
+
+  int32_t atPos = acctName.FindChar('@');
+
+  // get previous username and hostname
+  nsCString userName, hostName;
+  if (hostnameChanged)
+  {
+    rv = GetRealUsername(userName);
+    NS_ENSURE_SUCCESS(rv, rv);
+    hostName.Assign(oldName);
+  }
+  else
+  {
+    userName.Assign(oldName);
+    rv = GetRealHostName(hostName);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  return rv;
+  // switch corresponding part of the account name to the new name...
+  if (!hostnameChanged && (atPos != kNotFound))
+  {
+    // ...if username changed and the previous username was equal to the part
+    // of the account name before @
+    if (StringHead(acctName, atPos).Equals(NS_ConvertASCIItoUTF16(userName)))
+      acctName.Replace(0, userName.Length(), NS_ConvertASCIItoUTF16(newName));
+  }
+  if (hostnameChanged)
+  {
+    // ...if hostname changed and the previous hostname was equal to the part
+    // of the account name after @, or to the whole account name
+    if (atPos == kNotFound)
+      atPos = 0;
+    else
+      atPos += 1;
+    if (Substring(acctName, atPos).Equals(NS_ConvertASCIItoUTF16(hostName))) {
+      acctName.Replace(atPos, acctName.Length() - atPos,
+                       NS_ConvertASCIItoUTF16(newName));
+    }
+  }
+
+  return SetPrettyName(acctName);
 }
 
 NS_IMETHODIMP
@@ -1286,7 +1277,7 @@ nsMsgIncomingServer::SetRealHostName(const nsACString& aHostname)
 
   // A few things to take care of if we're changing the hostname.
   if (!aHostname.Equals(oldName, nsCaseInsensitiveCStringComparator()))
-    rv = OnUserOrHostNameChanged(oldName, aHostname);
+    rv = OnUserOrHostNameChanged(oldName, aHostname, true);
   return rv;
 }
 
@@ -1295,7 +1286,7 @@ nsMsgIncomingServer::GetHostName(nsACString& aResult)
 {
   nsresult rv;
   rv = GetCharValue("hostname", aResult);
-  if (aResult.CountChar(':') == 1)
+  if (MsgCountChar(aResult, ':') == 1)
   {
     // gack, we need to reformat the hostname - SetHostName will do that
     SetHostName(aResult);
@@ -1315,7 +1306,7 @@ nsMsgIncomingServer::GetRealHostName(nsACString& aResult)
   if (aResult.IsEmpty())
     return GetHostName(aResult);
 
-  if (aResult.CountChar(':') == 1)
+  if (MsgCountChar(aResult, ':') == 1)
   {
     SetRealHostName(aResult);
     rv = GetCharValue("realhostname", aResult);
@@ -1343,7 +1334,7 @@ nsMsgIncomingServer::SetRealUsername(const nsACString& aUsername)
   NS_ENSURE_SUCCESS(rv, rv);
   rv = SetCharValue("realuserName", aUsername);
   if (!oldName.Equals(aUsername))
-    rv = OnUserOrHostNameChanged(oldName, aUsername);
+    rv = OnUserOrHostNameChanged(oldName, aUsername, false);
   return rv;
 }
 
@@ -1389,7 +1380,7 @@ nsMsgIncomingServer::SetDoBiff(bool aDoBiff)
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::GetPort(PRInt32 *aPort)
+nsMsgIncomingServer::GetPort(int32_t *aPort)
 {
   NS_ENSURE_ARG_POINTER(aPort);
 
@@ -1405,7 +1396,7 @@ nsMsgIncomingServer::GetPort(PRInt32 *aPort)
   rv = getProtocolInfo(getter_AddRefs(protocolInfo));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 socketType;
+  int32_t socketType;
   rv = GetSocketType(&socketType);
   NS_ENSURE_SUCCESS(rv, rv);
   bool useSSLPort = (socketType == nsMsgSocketType::SSL);
@@ -1413,7 +1404,7 @@ nsMsgIncomingServer::GetPort(PRInt32 *aPort)
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::SetPort(PRInt32 aPort)
+nsMsgIncomingServer::SetPort(int32_t aPort)
 {
   nsresult rv;
 
@@ -1421,12 +1412,12 @@ nsMsgIncomingServer::SetPort(PRInt32 aPort)
   rv = getProtocolInfo(getter_AddRefs(protocolInfo));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 socketType;
+  int32_t socketType;
   rv = GetSocketType(&socketType);
   NS_ENSURE_SUCCESS(rv, rv);
   bool useSSLPort = (socketType == nsMsgSocketType::SSL);
 
-  PRInt32 defaultPort;
+  int32_t defaultPort;
   protocolInfo->GetDefaultServerPort(useSSLPort, &defaultPort);
   return SetIntValue("port", aPort == defaultPort ? PORT_NOT_SET : aPort);
 }
@@ -1441,7 +1432,7 @@ nsMsgIncomingServer::getProtocolInfo(nsIMsgProtocolInfo **aResult)
   rv = GetType(type);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
+  nsAutoCString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
   contractid.Append(type);
 
   nsCOMPtr<nsIMsgProtocolInfo> protocolInfo = do_GetService(contractid.get(), &rv);
@@ -1454,10 +1445,10 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
 {
   NS_ENSURE_ARG_POINTER(settings);
   nsMsgRetainByPreference retainByPreference;
-  PRInt32 daysToKeepHdrs = 0;
-  PRInt32 numHeadersToKeep = 0;
+  int32_t daysToKeepHdrs = 0;
+  int32_t numHeadersToKeep = 0;
   bool keepUnreadMessagesOnly = false;
-  PRInt32 daysToKeepBodies = 0;
+  int32_t daysToKeepBodies = 0;
   bool cleanupBodiesByDays = false;
   bool applyToFlaggedMessages = false;
   nsresult rv = NS_OK;
@@ -1469,7 +1460,7 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
   {
     rv = GetBoolValue("keepUnreadOnly", &keepUnreadMessagesOnly);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = GetIntValue("retainBy", (PRInt32*) &retainByPreference);
+    rv = GetIntValue("retainBy", (int32_t*) &retainByPreference);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = GetIntValue("numHdrsToKeep", &numHeadersToKeep);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1482,7 +1473,7 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
     rv = GetBoolValue("applyToFlaggedMessages", &applyToFlaggedMessages);
     NS_ENSURE_SUCCESS(rv, rv);
     retentionSettings->SetRetainByPreference(retainByPreference);
-    retentionSettings->SetNumHeadersToKeep((PRUint32) numHeadersToKeep);
+    retentionSettings->SetNumHeadersToKeep((uint32_t) numHeadersToKeep);
     retentionSettings->SetKeepUnreadMessagesOnly(keepUnreadMessagesOnly);
     retentionSettings->SetDaysToKeepBodies(daysToKeepBodies);
     retentionSettings->SetDaysToKeepHdrs(daysToKeepHdrs);
@@ -1498,10 +1489,10 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
 NS_IMETHODIMP nsMsgIncomingServer::SetRetentionSettings(nsIMsgRetentionSettings *settings)
 {
   nsMsgRetainByPreference retainByPreference;
-  PRUint32 daysToKeepHdrs = 0;
-  PRUint32 numHeadersToKeep = 0;
+  uint32_t daysToKeepHdrs = 0;
+  uint32_t numHeadersToKeep = 0;
   bool keepUnreadMessagesOnly = false;
-  PRUint32 daysToKeepBodies = 0;
+  uint32_t daysToKeepBodies = 0;
   bool cleanupBodiesByDays = false;
   bool applyToFlaggedMessages = false;
   settings->GetRetainByPreference(&retainByPreference);
@@ -1549,7 +1540,7 @@ NS_IMETHODIMP nsMsgIncomingServer::GetDownloadSettings(nsIMsgDownloadSettings **
   NS_ENSURE_ARG_POINTER(settings);
   bool downloadUnreadOnly = false;
   bool downloadByDate = false;
-  PRUint32 ageLimitOfMsgsToDownload = 0;
+  uint32_t ageLimitOfMsgsToDownload = 0;
   nsresult rv = NS_OK;
   if (!m_downloadSettings)
   {
@@ -1558,7 +1549,7 @@ NS_IMETHODIMP nsMsgIncomingServer::GetDownloadSettings(nsIMsgDownloadSettings **
     {
       rv = GetBoolValue("downloadUnreadOnly", &downloadUnreadOnly);
       rv = GetBoolValue("downloadByDate", &downloadByDate);
-      rv = GetIntValue("ageLimit", (PRInt32 *) &ageLimitOfMsgsToDownload);
+      rv = GetIntValue("ageLimit", (int32_t *) &ageLimitOfMsgsToDownload);
       m_downloadSettings->SetDownloadUnreadOnly(downloadUnreadOnly);
       m_downloadSettings->SetDownloadByDate(downloadByDate);
       m_downloadSettings->SetAgeLimitOfMsgsToDownload(ageLimitOfMsgsToDownload);
@@ -1577,7 +1568,7 @@ NS_IMETHODIMP nsMsgIncomingServer::SetDownloadSettings(nsIMsgDownloadSettings *s
   m_downloadSettings = settings;
   bool downloadUnreadOnly = false;
   bool downloadByDate = false;
-  PRUint32 ageLimitOfMsgsToDownload = 0;
+  uint32_t ageLimitOfMsgsToDownload = 0;
   m_downloadSettings->GetDownloadUnreadOnly(&downloadUnreadOnly);
   m_downloadSettings->GetDownloadByDate(&downloadByDate);
   m_downloadSettings->GetAgeLimitOfMsgsToDownload(&ageLimitOfMsgsToDownload);
@@ -1596,19 +1587,20 @@ nsMsgIncomingServer::GetSupportsDiskSpace(bool *aSupportsDiskSpace)
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::GetOfflineSupportLevel(PRInt32 *aSupportLevel)
+nsMsgIncomingServer::GetOfflineSupportLevel(int32_t *aSupportLevel)
 {
   NS_ENSURE_ARG_POINTER(aSupportLevel);
-  nsresult rv;
 
-  rv = GetIntValue("offline_support_level", aSupportLevel);
+  nsresult rv = GetIntValue("offline_support_level", aSupportLevel);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   if (*aSupportLevel == OFFLINE_SUPPORT_LEVEL_UNDEFINED)
     *aSupportLevel = OFFLINE_SUPPORT_LEVEL_NONE;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::SetOfflineSupportLevel(PRInt32 aSupportLevel)
+nsMsgIncomingServer::SetOfflineSupportLevel(int32_t aSupportLevel)
 {
   SetIntValue("offline_support_level", aSupportLevel);
   return NS_OK;
@@ -1619,12 +1611,12 @@ NS_IMETHODIMP nsMsgIncomingServer::DisplayOfflineMsg(nsIMsgWindow *aMsgWindow)
 {
   NS_ENSURE_ARG_POINTER(aMsgWindow);
 
-  nsresult rv;
-  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIStringBundleService> bundleService =
+    mozilla::services::GetStringBundleService();
+  NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr<nsIStringBundle> bundle;
-  rv = bundleService->CreateBundle(BASE_MSGS_URL, getter_AddRefs(bundle));
+  nsresult rv = bundleService->CreateBundle(BASE_MSGS_URL, getter_AddRefs(bundle));
   NS_ENSURE_SUCCESS(rv, rv);
   if (bundle)
   {
@@ -1672,7 +1664,7 @@ NS_IMETHODIMP
 nsMsgIncomingServer::GetIsSecure(bool *aIsSecure)
 {
   NS_ENSURE_ARG_POINTER(aIsSecure);
-  PRInt32 socketType;
+  int32_t socketType;
   nsresult rv = GetSocketType(&socketType);
   NS_ENSURE_SUCCESS(rv,rv);
   *aIsSecure = (socketType == nsMsgSocketType::alwaysSTARTTLS ||
@@ -1713,7 +1705,7 @@ NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, IncomingDuplicateAction, "dup_action
 
 NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, Hidden, "hidden")
 
-NS_IMETHODIMP nsMsgIncomingServer::GetSocketType(PRInt32 *aSocketType)
+NS_IMETHODIMP nsMsgIncomingServer::GetSocketType(int32_t *aSocketType)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
@@ -1743,12 +1735,12 @@ NS_IMETHODIMP nsMsgIncomingServer::GetSocketType(PRInt32 *aSocketType)
   return rv;
 }
 
-NS_IMETHODIMP nsMsgIncomingServer::SetSocketType(PRInt32 aSocketType)
+NS_IMETHODIMP nsMsgIncomingServer::SetSocketType(int32_t aSocketType)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
 
-  PRInt32 socketType = nsMsgSocketType::plain;
+  int32_t socketType = nsMsgSocketType::plain;
   mPrefBranch->GetIntPref("socketType", &socketType);
 
   nsresult rv = mPrefBranch->SetIntPref("socketType", aSocketType);
@@ -1813,11 +1805,11 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
 
   // For performance reasons, we'll handle clearing of filters if the user turns
   // off the server-side filters from the junk mail controls, in the junk mail controls.
-  nsCAutoString serverFilterName;
+  nsAutoCString serverFilterName;
   spamSettings->GetServerFilterName(serverFilterName);
   if (serverFilterName.IsEmpty())
     return NS_OK;
-  PRInt32 serverFilterTrustFlags = 0;
+  int32_t serverFilterTrustFlags = 0;
   (void) spamSettings->GetServerFilterTrustFlags(&serverFilterTrustFlags);
   if (!serverFilterTrustFlags)
     return NS_OK;
@@ -1850,8 +1842,7 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
   nsCOMPtr<nsIMsgFilterService> filterService = do_GetService(NS_MSGFILTERSERVICE_CONTRACTID, &rv);
   nsCOMPtr<nsIMsgFilterList> serverFilterList;
 
-  nsCOMPtr <nsILocalFile> localFile = do_QueryInterface(file);
-  rv = filterService->OpenFilterList(localFile, NULL, NULL, getter_AddRefs(serverFilterList));
+  rv = filterService->OpenFilterList(file, NULL, NULL, getter_AddRefs(serverFilterList));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = serverFilterList->GetFilterNamed(yesFilterName,
@@ -1874,7 +1865,7 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
     nsCOMPtr<nsISupportsArray> searchTerms;
     rv = newFilter->GetSearchTerms(getter_AddRefs(searchTerms));
     NS_ENSURE_SUCCESS(rv, rv);
-    PRUint32 count = 0;
+    uint32_t count = 0;
     searchTerms->Count(&count);
     if (count > 1) // don't need to group a single term
     {
@@ -1966,7 +1957,7 @@ nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(nsIMsgFilterList *fi
   // this can return success and a null identity...
 
   bool useCustomPrefs = false;
-  PRInt32 incorp = nsIMsgMdnGenerator::eIncorporateInbox;
+  int32_t incorp = nsIMsgMdnGenerator::eIncorporateInbox;
   NS_ENSURE_TRUE(identity, NS_ERROR_NULL_POINTER);
 
   identity->GetBoolAttribute("use_custom_prefs", &useCustomPrefs);
@@ -2094,7 +2085,7 @@ nsMsgIncomingServer::GetSpamSettings(nsISpamSettings **aSpamSettings)
 {
   NS_ENSURE_ARG_POINTER(aSpamSettings);
 
-  nsCAutoString spamActionTargetAccount;
+  nsAutoCString spamActionTargetAccount;
   GetCharValue("spamActionTargetAccount", spamActionTargetAccount);
   if (spamActionTargetAccount.IsEmpty())
   {
@@ -2145,15 +2136,15 @@ nsresult nsMsgIncomingServer::GetDeferredServers(nsIMsgIncomingServer *destServe
   accountManager->FindAccountForServer(destServer, getter_AddRefs(thisAccount));
   if (thisAccount)
   {
-    nsCOMPtr <nsISupportsArray> allServers;
+    nsCOMPtr<nsIArray> allServers;
     nsCString accountKey;
     thisAccount->GetKey(accountKey);
     accountManager->GetAllServers(getter_AddRefs(allServers));
     if (allServers)
     {
-      PRUint32 serverCount;
-      allServers->Count(&serverCount);
-      for (PRUint32 i = 0; i < serverCount; i++)
+      uint32_t serverCount;
+      allServers->GetLength(&serverCount);
+      for (uint32_t i = 0; i < serverCount; i++)
       {
         nsCOMPtr <nsIMsgIncomingServer> server (do_QueryElementAt(allServers, i));
         if (server)
@@ -2181,15 +2172,15 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIsDeferredTo(bool *aIsDeferredTo)
     accountManager->FindAccountForServer(this, getter_AddRefs(thisAccount));
     if (thisAccount)
     {
-      nsCOMPtr <nsISupportsArray> allServers;
+      nsCOMPtr<nsIArray> allServers;
       nsCString accountKey;
       thisAccount->GetKey(accountKey);
       accountManager->GetAllServers(getter_AddRefs(allServers));
       if (allServers)
       {
-        PRUint32 serverCount;
-        allServers->Count(&serverCount);
-        for (PRUint32 i = 0; i < serverCount; i++)
+        uint32_t serverCount;
+        allServers->GetLength(&serverCount);
+        for (uint32_t i = 0; i < serverCount; i++)
         {
           nsCOMPtr <nsIMsgIncomingServer> server (do_QueryElementAt(allServers, i));
           if (server)
@@ -2214,7 +2205,7 @@ const long kMaxDownloadTableSize = 500;
 
 // aClosure is the server, from that we get the cutoff point, below which we evict
 // aData is the arrival index of the msg.
-/* static */PLDHashOperator nsMsgIncomingServer::evictOldEntries(nsCStringHashKey::KeyType aKey, PRInt32 &aData, void *aClosure)
+/* static */PLDHashOperator nsMsgIncomingServer::evictOldEntries(nsCStringHashKey::KeyType aKey, int32_t &aData, void *aClosure)
 {
   nsMsgIncomingServer *server = (nsMsgIncomingServer *) aClosure;
   if (aData < server->m_numMsgsDownloaded - kMaxDownloadTableSize/2)
@@ -2230,7 +2221,15 @@ NS_IMETHODIMP nsMsgIncomingServer::IsNewHdrDuplicate(nsIMsgDBHdr *aNewHdr, bool
   NS_ENSURE_ARG_POINTER(aResult);
   NS_ENSURE_ARG_POINTER(aNewHdr);
   *aResult = false;
-  nsCAutoString strHashKey;
+
+  // If the message has been partially downloaded, the message should not
+  // be considered a duplicated message. See bug 714090.
+  uint32_t flags;
+  aNewHdr->GetFlags(&flags);
+  if (flags & nsMsgMessageFlags::Partial)
+    return NS_OK;
+
+  nsAutoCString strHashKey;
   nsCString messageId, subject;
   aNewHdr->GetMessageId(getter_Copies(messageId));
   strHashKey.Append(messageId);
@@ -2239,7 +2238,7 @@ NS_IMETHODIMP nsMsgIncomingServer::IsNewHdrDuplicate(nsIMsgDBHdr *aNewHdr, bool
   if (subject.IsEmpty() || messageId.IsEmpty())
     return NS_OK;
   strHashKey.Append(subject);
-  PRInt32 hashValue = 0;
+  int32_t hashValue = 0;
   m_downloadedHdrs.Get(strHashKey, &hashValue);
   if (hashValue)
     *aResult = true;
@@ -2261,7 +2260,7 @@ NS_IMETHODIMP
 nsMsgIncomingServer::GetForcePropertyEmpty(const char *aPropertyName, bool *_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
-  nsCAutoString nameEmpty(aPropertyName);
+  nsAutoCString nameEmpty(aPropertyName);
   nameEmpty.Append(NS_LITERAL_CSTRING(".empty"));
   nsCString value;
   GetCharValue(nameEmpty.get(), value);
@@ -2272,8 +2271,16 @@ nsMsgIncomingServer::GetForcePropertyEmpty(const char *aPropertyName, bool *_ret
 NS_IMETHODIMP
 nsMsgIncomingServer::SetForcePropertyEmpty(const char *aPropertyName, bool aValue)
 {
- nsCAutoString nameEmpty(aPropertyName);
+ nsAutoCString nameEmpty(aPropertyName);
  nameEmpty.Append(NS_LITERAL_CSTRING(".empty"));
  return SetCharValue(nameEmpty.get(),
    aValue ? NS_LITERAL_CSTRING("true") : NS_LITERAL_CSTRING(""));
 }
+
+NS_IMETHODIMP
+nsMsgIncomingServer::GetSortOrder(int32_t* aSortOrder)
+{
+  NS_ENSURE_ARG_POINTER(aSortOrder);
+  *aSortOrder = 100000000;
+  return NS_OK;
+}
