@@ -11,7 +11,7 @@
 #include "nsIMsgFilterList.h"
 #include "msgCore.h"
 #include "nsIMsgFolder.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsCOMPtr.h"
 #include "nsWeakReference.h"
 #include "nsIMsgDatabase.h"
@@ -70,18 +70,18 @@ protected:
   nsresult InternalSetHostName(const nsACString& aHostname, const char * prefName);
 
   nsresult getProtocolInfo(nsIMsgProtocolInfo **aResult);
-  nsCOMPtr <nsILocalFile> mFilterFile;
+  nsCOMPtr <nsIFile> mFilterFile;
   nsCOMPtr <nsIMsgFilterList> mFilterList;
   nsCOMPtr <nsIMsgFilterList> mEditableFilterList;
   nsCOMPtr<nsIPrefBranch> mPrefBranch;
   nsCOMPtr<nsIPrefBranch> mDefPrefBranch;
 
   // these allow us to handle duplicate incoming messages, e.g. delete them.
-  nsDataHashtable<nsCStringHashKey,PRInt32> m_downloadedHdrs;
-  PRInt32  m_numMsgsDownloaded;
-static PLDHashOperator evictOldEntries(nsCStringHashKey::KeyType aKey, PRInt32 &aData, void *aClosure);
+  nsDataHashtable<nsCStringHashKey,int32_t> m_downloadedHdrs;
+  int32_t  m_numMsgsDownloaded;
+static PLDHashOperator evictOldEntries(nsCStringHashKey::KeyType aKey, int32_t &aData, void *aClosure);
 private:
-  PRUint32 m_biffState;
+  uint32_t m_biffState;
   bool m_serverBusy;
   nsCOMPtr <nsISpamSettings> mSpamSettings;
   nsCOMPtr<nsIMsgFilterPlugin> mFilterPlugin;  // XXX should be a list
