@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
 #include "nsMsgMailNewsUrl.h"
@@ -56,11 +24,12 @@
 #include "prmem.h"
 #include <time.h>
 #include "nsMsgUtils.h"
+#include "mozilla/Services.h"
 
 nsMsgMailNewsUrl::nsMsgMailNewsUrl()
 {
   // nsIURI specific state
-  m_errorMessage = nsnull;
+  m_errorMessage = nullptr;
   m_runningUrl = false;
   m_updatingFolder = false;
   m_addContentToCache = false;
@@ -150,7 +119,7 @@ nsresult nsMsgMailNewsUrl::UnRegisterListener(nsIUrlListener *aUrlListener)
 {
   NS_ENSURE_ARG_POINTER(aUrlListener);
 
-  PRInt32 index = mUrlListeners.IndexOf(aUrlListener);
+  int32_t index = mUrlListeners.IndexOf(aUrlListener);
   // Due to the way mailnews is structured, some listeners attempt to remove
   // themselves twice. This may in fact be an error in the coding, however
   // if they didn't do it as they do currently, then they could fail to remove
@@ -169,8 +138,8 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(nsIMsgIncomingServer ** aIncomingServe
   // we can look at caching it later.
 
   nsresult rv;
-  nsCAutoString urlstr;
-  nsCAutoString scheme;
+  nsAutoCString urlstr;
+  nsAutoCString scheme;
 
   nsCOMPtr<nsIURL> url = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
@@ -212,7 +181,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(nsIMsgIncomingServer ** aIncomingServe
 NS_IMETHODIMP nsMsgMailNewsUrl::GetMsgWindow(nsIMsgWindow **aMsgWindow)
 {
   NS_ENSURE_ARG_POINTER(aMsgWindow);
-  *aMsgWindow = nsnull;
+  *aMsgWindow = nullptr;
   
   nsCOMPtr<nsIMsgWindow> msgWindow(do_QueryReferent(m_msgWindowWeak));
   msgWindow.swap(*aMsgWindow);
@@ -232,7 +201,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetStatusFeedback(nsIMsgStatusFeedback **aMsgFee
 {
   // note: it is okay to return a null status feedback and not return an error
   // it's possible the url really doesn't have status feedback
-  *aMsgFeedback = nsnull;
+  *aMsgFeedback = nullptr;
   if (!m_statusFeedbackWeak)
   {
     nsCOMPtr<nsIMsgWindow> msgWindow(do_QueryReferent(m_msgWindowWeak));
@@ -254,13 +223,13 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetStatusFeedback(nsIMsgStatusFeedback *aMsgFeed
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailNewsUrl::GetMaxProgress(PRInt64 *aMaxProgress)
+NS_IMETHODIMP nsMsgMailNewsUrl::GetMaxProgress(int64_t *aMaxProgress)
 {
   *aMaxProgress = mMaxProgress;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailNewsUrl::SetMaxProgress(PRInt64 aMaxProgress)
+NS_IMETHODIMP nsMsgMailNewsUrl::SetMaxProgress(int64_t aMaxProgress)
 {
   mMaxProgress = aMaxProgress;
   return NS_OK;
@@ -268,7 +237,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetMaxProgress(PRInt64 aMaxProgress)
 
 NS_IMETHODIMP nsMsgMailNewsUrl::GetLoadGroup(nsILoadGroup **aLoadGroup)
 {
-  *aLoadGroup = nsnull;
+  *aLoadGroup = nullptr;
   // note: it is okay to return a null load group and not return an error
   // it's possible the url really doesn't have load group
   nsCOMPtr<nsILoadGroup> loadGroup (do_QueryReferent(m_loadGroupWeak));
@@ -341,7 +310,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetSuppressErrorMsgs(bool aSuppressErrorMsgs)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailNewsUrl::IsUrlType(PRUint32 type, bool *isType)
+NS_IMETHODIMP nsMsgMailNewsUrl::IsUrlType(uint32_t type, bool *isType)
 {
   //base class doesn't know about any specific types
   NS_ENSURE_ARG(isType);
@@ -383,7 +352,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetSpec(nsACString &aSpec)
 
 NS_IMETHODIMP nsMsgMailNewsUrl::SetSpec(const nsACString &aSpec)
 {
-  nsCAutoString spec(aSpec);
+  nsAutoCString spec(aSpec);
   // Parse out "filename" attribute if present.
   char *start, *end;
   start = PL_strcasestr(spec.BeginWriting(),"?filename=");
@@ -472,12 +441,12 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetHost(const nsACString &aHost)
   return m_baseURL->SetHost(aHost);
 }
 
-NS_IMETHODIMP nsMsgMailNewsUrl::GetPort(PRInt32 *aPort)
+NS_IMETHODIMP nsMsgMailNewsUrl::GetPort(int32_t *aPort)
 {
   return m_baseURL->GetPort(aPort);
 }
 
-NS_IMETHODIMP nsMsgMailNewsUrl::SetPort(PRInt32 aPort)
+NS_IMETHODIMP nsMsgMailNewsUrl::SetPort(int32_t aPort)
 {
   return m_baseURL->SetPort(aPort);
 }
@@ -569,12 +538,13 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SchemeIs(const char *aScheme, bool *_retval)
 NS_IMETHODIMP nsMsgMailNewsUrl::Clone(nsIURI **_retval)
 {
   nsresult rv;
-  nsCAutoString urlSpec;
-  nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoCString urlSpec;
+  nsCOMPtr<nsIIOService> ioService =
+    mozilla::services::GetIOService();
+  NS_ENSURE_TRUE(ioService, NS_ERROR_UNEXPECTED);
   rv = GetSpec(urlSpec);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = ioService->NewURI(urlSpec, nsnull, nsnull, _retval);
+  rv = ioService->NewURI(urlSpec, nullptr, nullptr, _retval);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // add the msg window to the cloned url
@@ -600,9 +570,10 @@ NS_IMETHODIMP nsMsgMailNewsUrl::Resolve(const nsACString &relativePath, nsACStri
   else
   {
     // if relativePath is a complete url with it's own scheme then allow it...
-    nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString scheme;
+    nsCOMPtr<nsIIOService> ioService =
+      mozilla::services::GetIOService();
+    NS_ENSURE_TRUE(ioService, NS_ERROR_UNEXPECTED);
+    nsAutoCString scheme;
 
     rv = ioService->ExtractScheme(relativePath, scheme);
     // if we have a fully qualified scheme then pass the relative path back as the result
@@ -656,7 +627,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetFileExtension(nsACString &aFileExtension)
 {
   if (!mAttachmentFileName.IsEmpty())
   {
-    PRInt32 pos = mAttachmentFileName.RFindChar(PRUnichar('.'));
+    int32_t pos = mAttachmentFileName.RFindChar(PRUnichar('.'));
     if (pos > 0)
       aFileExtension = Substring(mAttachmentFileName, pos + 1 /* skip the '.' */);
     return NS_OK;
@@ -733,7 +704,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl:: GetMemCacheEntry(nsICacheEntryDescriptor **memC
   }
   else
   {
-    *memCacheEntry = nsnull;
+    *memCacheEntry = nullptr;
     return NS_ERROR_NULL_POINTER;
   }
 
@@ -809,7 +780,7 @@ protected:
   nsCOMPtr<nsIFile> m_outputFile;
   bool m_addDummyEnvelope;
   bool m_writtenData;
-  PRUint32 m_leftOver;
+  uint32_t m_leftOver;
   char m_dataBuffer[SAVE_BUF_SIZE+1]; // temporary buffer for this save operation
 
 };
@@ -849,11 +820,11 @@ nsMsgSaveAsListener::OnStopRequest(nsIRequest *request, nsISupports * aCtxt, nsr
 NS_IMETHODIMP nsMsgSaveAsListener::OnDataAvailable(nsIRequest* request, 
                                   nsISupports* aSupport,
                                   nsIInputStream* inStream, 
-                                  PRUint32 srcOffset,
-                                  PRUint32 count)
+                                  uint64_t srcOffset,
+                                  uint32_t count)
 {
   nsresult rv;
-  PRUint32 available;
+  uint64_t available;
   rv = inStream->Available(&available);
   if (!m_writtenData)
   {
@@ -868,12 +839,12 @@ NS_IMETHODIMP nsMsgSaveAsListener::OnDataAvailable(nsIRequest* request,
     msgUrl->GetCanonicalLineEnding(&useCanonicalEnding);
   
   const char *lineEnding = (useCanonicalEnding) ? CRLF : MSG_LINEBREAK;
-  PRUint32 lineEndingLength = (useCanonicalEnding) ? 2 : MSG_LINEBREAK_LEN;
+  uint32_t lineEndingLength = (useCanonicalEnding) ? 2 : MSG_LINEBREAK_LEN;
   
-  PRUint32 readCount, maxReadCount = SAVE_BUF_SIZE - m_leftOver;
-  PRUint32 writeCount;
+  uint32_t readCount, maxReadCount = SAVE_BUF_SIZE - m_leftOver;
+  uint32_t writeCount;
   char *start, *end, lastCharInPrevBuf = '\0';
-  PRUint32 linebreak_len = 0;
+  uint32_t linebreak_len = 0;
 
   while (count > 0)
   {
@@ -917,7 +888,10 @@ NS_IMETHODIMP nsMsgSaveAsListener::OnDataAvailable(nsIRequest* request,
               PL_strncmp(start, "From - ", 7))
           {
               rv = m_outputStream->Write(start, end-start, &writeCount);
-              rv |= m_outputStream->Write(lineEnding, lineEndingLength, &writeCount);
+              nsresult tmp = m_outputStream->Write(lineEnding, lineEndingLength, &writeCount);
+              if (NS_FAILED(tmp)) {
+                rv = tmp;
+              }
           }
           start = end+linebreak_len;
           if (start >= m_dataBuffer + m_leftOver)
@@ -959,15 +933,14 @@ nsresult nsMsgSaveAsListener::SetupMsgWriteStream(nsIFile *aFile, bool addDummyE
   // (Windows doesn't care, btw, just unixoids do...)
   aFile->Remove(false);
 
-  nsCOMPtr <nsILocalFile> localFile = do_QueryInterface(aFile);
   nsresult rv = MsgNewBufferedFileOutputStream(getter_AddRefs(m_outputStream),
-                                               localFile, -1, 00600);
+                                               aFile, -1, 00600);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (m_outputStream && addDummyEnvelope)
   {
-    nsCAutoString result;
-    PRUint32 writeCount;
+    nsAutoCString result;
+    uint32_t writeCount;
 
     time_t now = time((time_t*) 0);
     char *ct = ctime(&now);
@@ -1024,7 +997,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetMsgHeaderSink(nsIMsgHeaderSink * aMsgHdrSink)
 NS_IMETHODIMP nsMsgMailNewsUrl::GetIsMessageUri(bool *aIsMessageUri)
 {
   NS_ENSURE_ARG(aIsMessageUri);
-  nsCAutoString scheme;
+  nsAutoCString scheme;
   m_baseURL->GetScheme(scheme);
   *aIsMessageUri = StringEndsWith(scheme, NS_LITERAL_CSTRING("-message"));
   return NS_OK;
