@@ -42,7 +42,7 @@
 
 #define AD_WORKING_BUFF_SIZE                  8192
 
-extern void         MacGetFileType(nsILocalFile *fs, bool *useDefault, char **type, char **encoding);
+extern void         MacGetFileType(nsIFile *fs, bool *useDefault, char **type, char **encoding);
 
 #include "nsILocalFileMac.h"
 
@@ -110,8 +110,8 @@ nsresult nsSimpleZipper::AddToZip(nsIZipWriter *aZipWriter,
 // Class implementation...
 //
 nsMsgAttachmentHandler::nsMsgAttachmentHandler() :
-  mRequest(nsnull),
-  mCompFields(nsnull),   // Message composition fields for the sender
+  mRequest(nullptr),
+  mCompFields(nullptr),   // Message composition fields for the sender
   m_bogus_attachment(false),
   m_done(false),
   m_already_encoded_p(false),
@@ -138,7 +138,7 @@ nsMsgAttachmentHandler::nsMsgAttachmentHandler() :
   m_file_analyzed(false),
 
   // Mime
-  m_encoder_data(nsnull)
+  m_encoder_data(nullptr)
 {
 }
 
@@ -159,13 +159,13 @@ nsMsgAttachmentHandler::CleanupTempFile()
 #ifdef XP_MACOSX
   if (mEncodedWorkingFile) {
     mEncodedWorkingFile->Remove(false);
-    mEncodedWorkingFile = nsnull;
+    mEncodedWorkingFile = nullptr;
   }
 #endif // XP_MACOSX
 }
 
 void
-nsMsgAttachmentHandler::AnalyzeDataChunk(const char *chunk, PRInt32 length)
+nsMsgAttachmentHandler::AnalyzeDataChunk(const char *chunk, int32_t length)
 {
   unsigned char *s = (unsigned char *) chunk;
   unsigned char *end = s + length;
@@ -225,16 +225,16 @@ void
 nsMsgAttachmentHandler::AnalyzeSnarfedFile(void)
 {
   char chunk[1024];
-  PRUint32 numRead = 0;
+  uint32_t numRead = 0;
 
   if (m_file_analyzed)
     return;
 
   if (mTmpFile)
   {
-    PRInt64 fileSize;
+    int64_t fileSize;
     mTmpFile->GetFileSize(&fileSize);
-    m_size = (PRUint32) fileSize;
+    m_size = (uint32_t) fileSize;
     nsCOMPtr <nsIInputStream> inputFile;
     nsresult rv = NS_NewLocalFileInputStream(getter_AddRefs(inputFile), mTmpFile);
     if (NS_FAILED(rv))
@@ -260,7 +260,7 @@ nsMsgAttachmentHandler::AnalyzeSnarfedFile(void)
 // Given a content-type and some info about the contents of the document,
 // decide what encoding it should have.
 //
-int
+nsresult
 nsMsgAttachmentHandler::PickEncoding(const char *charset, nsIMsgSend *mime_delivery_state)
 {
   nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
@@ -271,7 +271,7 @@ nsMsgAttachmentHandler::PickEncoding(const char *charset, nsIMsgSend *mime_deliv
   if (mSendViaCloud)
   {
     m_encoding = ENCODING_7BIT;
-    return 0;
+    return NS_OK;
   }
   if (m_already_encoded_p)
     goto DONE;
@@ -422,7 +422,7 @@ DONE:
     else
       m_type = TEXT_PLAIN;
   }
-  return 0;
+  return NS_OK;
 }
 
 nsresult
@@ -431,7 +431,7 @@ nsMsgAttachmentHandler::PickCharset()
   if (!m_charset.IsEmpty() || !m_type.LowerCaseEqualsLiteral(TEXT_PLAIN))
     return NS_OK;
 
-  nsCOMPtr<nsILocalFile> tmpFile =
+  nsCOMPtr<nsIFile> tmpFile =
     do_QueryInterface(mTmpFile);
   if (!tmpFile)
     return NS_OK;
@@ -443,13 +443,13 @@ static nsresult
 FetcherURLDoneCallback(nsresult aStatus,
                        const nsACString &aContentType,
                        const nsACString &aCharset,
-                       PRInt32 totalSize,
+                       int32_t totalSize,
                        const PRUnichar* aMsg, void *tagData)
 {
   nsMsgAttachmentHandler *ma = (nsMsgAttachmentHandler *) tagData;
-  NS_ASSERTION(ma != nsnull, "not-null mime attachment");
+  NS_ASSERTION(ma != nullptr, "not-null mime attachment");
 
-  if (ma != nsnull)
+  if (ma != nullptr)
   {
     ma->m_size = totalSize;
     if (!aContentType.IsEmpty())
@@ -525,7 +525,7 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
     rv = GetMessageServiceFromURI(m_uri, getter_AddRefs(messageService));
     if (NS_SUCCEEDED(rv) && messageService)
     {
-      nsCAutoString uri(m_uri);
+      nsAutoCString uri(m_uri);
       uri += (uri.FindChar('?') == kNotFound) ? '?' : '&';
       uri.Append("fetchCompleteMessage=true");
       nsCOMPtr<nsIStreamListener> strListener;
@@ -545,8 +545,8 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
       {
         mimeConverter->SetMimeOutputType(nsMimeOutput::nsMimeMessageDecrypt);
         mimeConverter->SetForwardInline(false);
-        mimeConverter->SetIdentity(nsnull);
-        mimeConverter->SetOriginalMsgURI(nsnull);
+        mimeConverter->SetIdentity(nullptr);
+        mimeConverter->SetOriginalMsgURI(nullptr);
       }
 
       nsCOMPtr<nsIStreamListener> convertedListener = do_QueryInterface(m_mime_parser, &rv);
@@ -554,9 +554,9 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
         goto done;
 
       nsCOMPtr<nsIURI> aURL;
-      rv = messageService->GetUrlForUri(uri.get(), getter_AddRefs(aURL), nsnull);
+      rv = messageService->GetUrlForUri(uri.get(), getter_AddRefs(aURL), nullptr);
 
-      rv = NS_NewInputStreamChannel(getter_AddRefs(m_converter_channel), aURL, nsnull);
+      rv = NS_NewInputStreamChannel(getter_AddRefs(m_converter_channel), aURL, nullptr);
       if (NS_FAILED(rv))
         goto done;
 
@@ -565,7 +565,7 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
       if (NS_FAILED(rv))
         goto done;
 
-      rv = messageService->DisplayMessage(uri.get(), convertedListener, nsnull, nsnull, nsnull, nsnull);
+      rv = messageService->DisplayMessage(uri.get(), convertedListener, nullptr, nullptr, nullptr, nullptr);
     }
   }
 done:
@@ -574,13 +574,13 @@ done:
       if (mOutFile)
       {
         mOutFile->Close();
-        mOutFile = nsnull;
+        mOutFile = nullptr;
       }
 
       if (mTmpFile)
       {
         mTmpFile->Remove(false);
-        mTmpFile = nsnull;
+        mTmpFile = nullptr;
       }
   }
 
@@ -591,7 +591,7 @@ done:
 bool nsMsgAttachmentHandler::HasResourceFork(FSRef *fsRef)
 {
   FSCatalogInfo catalogInfo;
-  OSErr err = FSGetCatalogInfo(fsRef, kFSCatInfoDataSizes + kFSCatInfoRsrcSizes, &catalogInfo, nsnull, nsnull, nsnull);
+  OSErr err = FSGetCatalogInfo(fsRef, kFSCatInfoDataSizes + kFSCatInfoRsrcSizes, &catalogInfo, nullptr, nullptr, nullptr);
   return (err == noErr && catalogInfo.rsrcLogicalSize != 0);
 }
 #endif
@@ -629,7 +629,7 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
       }
     }
     mTmpFile->Remove(false);
-    mTmpFile = nsnull;
+    mTmpFile = nullptr;
     return NS_MSG_UNABLE_TO_OPEN_TMP_FILE;
   }
 
@@ -639,12 +639,12 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
   if (!m_bogus_attachment && StringBeginsWith(sourceURISpec, NS_LITERAL_CSTRING("file://")))
   {
     // Unescape the path (i.e. un-URLify it) before making a FSSpec
-    nsCAutoString filePath;
+    nsAutoCString filePath;
     filePath.Adopt(nsMsgGetLocalFileFromURL(sourceURISpec.get()));
-    nsCAutoString unescapedFilePath;
+    nsAutoCString unescapedFilePath;
     MsgUnescapeString(filePath, 0, unescapedFilePath);
 
-    nsCOMPtr<nsILocalFile> sourceFile;
+    nsCOMPtr<nsIFile> sourceFile;
     NS_NewNativeLocalFile(unescapedFilePath, true, getter_AddRefs(sourceFile));
     if (!sourceFile)
       return NS_ERROR_FAILURE;
@@ -686,7 +686,7 @@ nsresult
 nsMsgAttachmentHandler::ConvertToZipFile(nsILocalFileMac *aSourceFile)
 {
   // append ".zip" to the real file name
-  nsCAutoString zippedName;
+  nsAutoCString zippedName;
   nsresult rv = aSourceFile->GetNativeLeafName(zippedName);
   NS_ENSURE_SUCCESS(rv, rv);
   zippedName.AppendLiteral(".zip");
@@ -726,13 +726,13 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
 
   nsresult rv = aSourceFile->GetFileType(&type);
   if (NS_FAILED(rv))
-    return false;
+    return rv;
   PR_snprintf(fileInfo, sizeof(fileInfo), "%X", type);
   m_xMacType = fileInfo;
 
   rv = aSourceFile->GetFileCreator(&creator);
   if (NS_FAILED(rv))
-    return false;
+    return rv;
   PR_snprintf(fileInfo, sizeof(fileInfo), "%X", creator);
   m_xMacCreator = fileInfo;
 
@@ -749,7 +749,7 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
       rv = fileUrl->SetSpec(aFileURI);
       if (NS_SUCCEEDED(rv))
       {
-        nsCAutoString ext;
+        nsAutoCString ext;
         rv = fileUrl->GetFileExtension(ext);
         if (NS_SUCCEEDED(rv) && !ext.IsEmpty())
         {
@@ -799,7 +799,7 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
     AppleDoubleEncodeObject     *obj = new (AppleDoubleEncodeObject);
     if (obj == NULL)
     {
-      mEncodedWorkingFile = nsnull;
+      mEncodedWorkingFile = nullptr;
       PR_FREEIF(separator);
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -812,9 +812,9 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    PRInt32     bSize = AD_WORKING_BUFF_SIZE;
+    int32_t     bSize = AD_WORKING_BUFF_SIZE;
 
-    char  *working_buff = nsnull;
+    char  *working_buff = nullptr;
     while (!working_buff && (bSize >= 512))
     {
       working_buff = (char *)PR_CALLOC(bSize);
@@ -837,9 +837,9 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
     //
     ap_encode_init(&(obj->ap_encode_obj), aFilePath.get(), separator);
 
-    PRInt32 count;
+    int32_t count;
 
-    nsresult status = noErr;
+    OSErr status = noErr;
     m_size = 0;
     while (status == noErr)
     {
@@ -849,10 +849,10 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
         //
         // we got the encode data, so call the next stream to write it to the disk.
         //
-        PRUint32 bytesWritten;
+        uint32_t bytesWritten;
         obj->fileStream->Write(obj->buff, count, &bytesWritten);
-        if (bytesWritten != (PRUint32) count)
-          status = NS_MSG_ERROR_WRITING_FILE;
+        if (bytesWritten != (uint32_t) count)
+          status = errFileWrite;
       }
     }
 
@@ -921,9 +921,9 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
 #endif // XP_MACOSX
 
 nsresult
-nsMsgAttachmentHandler::LoadDataFromFile(nsILocalFile *file, nsString &sigData, bool charsetConversion)
+nsMsgAttachmentHandler::LoadDataFromFile(nsIFile *file, nsString &sigData, bool charsetConversion)
 {
-  PRInt32       readSize;
+  int32_t       readSize;
   char          *readBuf;
 
   nsCOMPtr <nsIInputStream> inputFile;
@@ -931,16 +931,16 @@ nsMsgAttachmentHandler::LoadDataFromFile(nsILocalFile *file, nsString &sigData,
   if (NS_FAILED(rv))
     return NS_MSG_ERROR_WRITING_FILE;
 
-  PRInt64 fileSize;
+  int64_t fileSize;
   file->GetFileSize(&fileSize);
-  readSize = (PRUint32) fileSize;
+  readSize = (uint32_t) fileSize;
 
   readBuf = (char *)PR_Malloc(readSize + 1);
   if (!readBuf)
     return NS_ERROR_OUT_OF_MEMORY;
   memset(readBuf, 0, readSize + 1);
 
-  PRUint32 bytesRead;
+  uint32_t bytesRead;
   inputFile->Read(readBuf, readSize, &bytesRead);
   inputFile->Close();
 
@@ -967,10 +967,10 @@ nsMsgAttachmentHandler::Abort()
   {
     if (mDeleteFile)
       mTmpFile->Remove(false);
-    mTmpFile = nsnull;
+    mTmpFile = nullptr;
   }
 
-  NS_ASSERTION(m_mime_delivery_state != nsnull, "not-null m_mime_delivery_state");
+  NS_ASSERTION(m_mime_delivery_state != nullptr, "not-null m_mime_delivery_state");
 
   if (m_done)
     return NS_OK;
@@ -981,7 +981,7 @@ nsMsgAttachmentHandler::Abort()
     if (m_mime_delivery_state)
     {
       m_mime_delivery_state->SetStatus(NS_ERROR_ABORT);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, NS_ERROR_ABORT, 0, nsnull);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, NS_ERROR_ABORT, 0, nullptr);
     }
   return NS_OK;
 
@@ -990,15 +990,15 @@ nsMsgAttachmentHandler::Abort()
 nsresult
 nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 {
-  NS_ASSERTION(m_mime_delivery_state != nsnull, "not-null m_mime_delivery_state");
+  NS_ASSERTION(m_mime_delivery_state != nullptr, "not-null m_mime_delivery_state");
 
   // Close the file, but don't delete the disk file (or the file spec.)
   if (mOutFile)
   {
     mOutFile->Close();
-    mOutFile = nsnull;
+    mOutFile = nullptr;
   }
-  // this silliness is because Windows nsILocalFile caches its file size
+  // this silliness is because Windows nsIFile caches its file size
   // so if an output stream writes to it, it will still return the original
   // cached size.
   if (mTmpFile)
@@ -1007,7 +1007,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     mTmpFile->Clone(getter_AddRefs(tmpFile));
     mTmpFile = do_QueryInterface(tmpFile);
   }
-  mRequest = nsnull;
+  mRequest = nullptr;
 
   // First things first, we are now going to see if this is an HTML
   // Doc and if it is, we need to see if we can determine the charset
@@ -1033,7 +1033,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     bool              keepOnGoing = true;
     nsCString    turl;
     nsString     msg;
-    PRUnichar         *printfString = nsnull;
+    PRUnichar         *printfString = nullptr;
     nsresult rv;
     nsCOMPtr<nsIStringBundleService> bundleService =
       mozilla::services::GetStringBundleService();
@@ -1051,7 +1051,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
       printfString = nsTextFormatter::smprintf(msg.get(), m_realName.get());
     else if (NS_SUCCEEDED(mURL->GetSpec(turl)) && !turl.IsEmpty())
     {
-      nsCAutoString unescapedUrl;
+      nsAutoCString unescapedUrl;
       MsgUnescapeString(turl, 0, unescapedUrl);
       if (unescapedUrl.IsEmpty())
         printfString = nsTextFormatter::smprintf(msg.get(), turl.get());
@@ -1069,7 +1069,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
     if (keepOnGoing)
     {
-      status = 0;
+      status = NS_OK;
       m_bogus_attachment = true; //That will cause this attachment to be ignored.
     }
     else
@@ -1077,9 +1077,9 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
       status = NS_ERROR_ABORT;
       m_mime_delivery_state->SetStatus(status);
       nsresult ignoreMe;
-      m_mime_delivery_state->Fail(status, nsnull, &ignoreMe);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, 0, nsnull);
-      SetMimeDeliveryState(nsnull);
+      m_mime_delivery_state->Fail(status, nullptr, &ignoreMe);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, 0, nullptr);
+      SetMimeDeliveryState(nullptr);
       return status;
     }
   }
@@ -1096,20 +1096,6 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
   {
     //
     // Conversion to plain text desired.
-    //
-    PRInt32       width = 72;
-    nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
-    if (pPrefBranch)
-      pPrefBranch->GetIntPref("mailnews.wraplength", &width);
-    // Let sanity reign!
-    if (width == 0)
-      width = 72;
-    else if (width < 10)
-      width = 10;
-    else if (width > 30000)
-      width = 30000;
-
-    //
     // Now use the converter service here to do the right
     // thing and convert this data to plain text for us!
     //
@@ -1127,16 +1113,16 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
         if (NS_SUCCEEDED(rv))
         {
-          nsCAutoString tData;
+          nsAutoCString tData;
           if (NS_FAILED(ConvertFromUnicode(m_charset.get(), conData, tData)))
             LossyCopyUTF16toASCII(conData, tData);
           if (!tData.IsEmpty())
           {
-            PRUint32 bytesWritten;
+            uint32_t bytesWritten;
             (void) outputStream->Write(tData.get(), tData.Length(), &bytesWritten);
           }
           outputStream->Close();
-          // this silliness is because Windows nsILocalFile caches its file size
+          // this silliness is because Windows nsIFile caches its file size
           // so if an output stream writes to it, it will still return the original
           // cached size.
           if (mTmpFile)
@@ -1155,7 +1141,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     m_encoding.Truncate();
   }
 
-  PRUint32 pendingAttachmentCount = 0;
+  uint32_t pendingAttachmentCount = 0;
   m_mime_delivery_state->GetPendingAttachmentCount(&pendingAttachmentCount);
   NS_ASSERTION (pendingAttachmentCount > 0, "no more pending attachment");
 
@@ -1168,10 +1154,10 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     /* Find the next attachment which has not yet been loaded,
      if any, and start it going.
      */
-    PRUint32 i;
+    uint32_t i;
     nsMsgAttachmentHandler *next = 0;
-    nsMsgAttachmentHandler *attachments = nsnull;
-    PRUint32 attachmentCount = 0;
+    nsMsgAttachmentHandler *attachments = nullptr;
+    uint32_t attachmentCount = 0;
 
     m_mime_delivery_state->GetAttachmentCount(&attachmentCount);
     if (attachmentCount)
@@ -1191,10 +1177,11 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
         if ( (!next->mURL) && (next->m_uri.IsEmpty()) )
         {
           attachments[i].m_done = true;
+          attachments[i].SetMimeDeliveryState(nullptr);
           m_mime_delivery_state->GetPendingAttachmentCount(&pendingAttachmentCount);
           m_mime_delivery_state->SetPendingAttachmentCount(pendingAttachmentCount - 1);
           next->mPartUserOmissionOverride = true;
-          next = nsnull;
+          next = nullptr;
           continue;
         }
 
@@ -1204,13 +1191,13 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
     if (next)
     {
-      int status = next->SnarfAttachment(mCompFields);
+      nsresult status = next->SnarfAttachment(mCompFields);
       if (NS_FAILED(status))
       {
         nsresult ignoreMe;
-        m_mime_delivery_state->Fail(status, nsnull, &ignoreMe);
-        m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, 0, nsnull);
-        SetMimeDeliveryState(nsnull);
+        m_mime_delivery_state->Fail(status, nullptr, &ignoreMe);
+        m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, 0, nullptr);
+        SetMimeDeliveryState(nullptr);
         return NS_ERROR_UNEXPECTED;
       }
     }
@@ -1226,8 +1213,8 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     {
       nsresult ignoreMe;
       m_mime_delivery_state->Fail(status, aMsg, &ignoreMe);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, aMsg, nsnull);
-      SetMimeDeliveryState(nsnull);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, aMsg, nullptr);
+      SetMimeDeliveryState(nullptr);
       return NS_ERROR_UNEXPECTED;
     }
     else
@@ -1237,8 +1224,8 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
       {
         nsresult ignoreMe;
         m_mime_delivery_state->Fail(status, aMsg, &ignoreMe);
-        m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, aMsg, nsnull);
-        SetMimeDeliveryState(nsnull);
+        m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, aMsg, nullptr);
+        SetMimeDeliveryState(nullptr);
         return NS_ERROR_UNEXPECTED;
       }
     }
@@ -1254,7 +1241,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     }
   }
 
-  SetMimeDeliveryState(nsnull);
+  SetMimeDeliveryState(nullptr);
   return NS_OK;
 }
 
