@@ -107,16 +107,16 @@ nsMsgCreateTempFileName(const char *tFileName)
                                                 tFileName,
                                                 getter_AddRefs(tmpFile));
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   rv = tmpFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 00600);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   nsCString tempString;
   rv = tmpFile->GetNativePath(tempString);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   char *tString = ToNewCString(tempString);
   if (!tString)
@@ -251,19 +251,19 @@ nsMsgStripLine (char * string)
 char *
 mime_generate_headers (nsMsgCompFields *fields,
                        const char *charset,
-                       nsMsgDeliverMode deliver_mode, nsIPrompt * aPrompt, PRInt32 *status)
+                       nsMsgDeliverMode deliver_mode, nsIPrompt * aPrompt, nsresult *status)
 {
   nsresult rv;
-  *status = 0;
+  *status = NS_OK;
 
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv)) {
     *status = rv;
-    return nsnull;
+    return nullptr;
   }
 
   bool usemime = nsMsgMIMEGetConformToStandard();
-  PRInt32 size = 0;
+  int32_t size = 0;
   char *buffer = 0, *buffer_tail = 0;
   bool isDraft =
     deliver_mode == nsIMsgSend::nsMsgSaveAsDraft ||
@@ -287,12 +287,12 @@ mime_generate_headers (nsMsgCompFields *fields,
 
   bool hasDisclosedRecipient = false;
 
-  nsCAutoString headerBuf;    // accumulate header strings to get length
+  nsAutoCString headerBuf;    // accumulate header strings to get length
   headerBuf.Truncate();
 
   NS_ASSERTION (fields, "null fields");
   if (!fields)
-    return nsnull;
+    return nullptr;
 
   pFrom = fields->GetFrom();
   if (pFrom)
@@ -328,7 +328,7 @@ mime_generate_headers (nsMsgCompFields *fields,
 
   buffer = (char *) PR_Malloc (size);
   if (!buffer)
-    return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+    return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
 
   buffer_tail = buffer;
 
@@ -346,7 +346,7 @@ mime_generate_headers (nsMsgCompFields *fields,
       (deliver_mode != nsIMsgSend::nsMsgSaveAsDraft &&
       deliver_mode != nsIMsgSend::nsMsgSaveAsTemplate))
     {
-        PRInt32 receipt_header_type = nsIMsgMdnGenerator::eDntType;
+        int32_t receipt_header_type = nsIMsgMdnGenerator::eDntType;
         fields->GetReceiptHeaderType(&receipt_header_type);
 
       // nsIMsgMdnGenerator::eDntType = MDN Disposition-Notification-To: ;
@@ -420,7 +420,7 @@ mime_generate_headers (nsMsgCompFields *fields,
       // whether the draft/template has request for either MDN or DNS or both
       // return receipt; since the DNS is out of the picture we now use the
       // header type + 1 to tell whether user has requested the return receipt
-      PRInt32 headerType = 0;
+      int32_t headerType = 0;
       fields->GetReceiptHeaderType(&headerType);
       char *type = PR_smprintf("%d", (int)headerType + 1);
       if (type)
@@ -447,7 +447,7 @@ mime_generate_headers (nsMsgCompFields *fields,
   nsCOMPtr<nsIHttpProtocolHandler> pHTTPHandler = do_GetService(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX "http", &rv);
   if (NS_SUCCEEDED(rv) && pHTTPHandler)
   {
-    nsCAutoString userAgentString;
+    nsAutoCString userAgentString;
     pHTTPHandler->GetUserAgent(userAgentString);
 
     if (!userAgentString.IsEmpty())
@@ -466,7 +466,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     char *duppedNewsGrp = PL_strdup(pNewsGrp);
     if (!duppedNewsGrp) {
       PR_FREEIF(buffer);
-      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+      return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
     }
     char *n2 = nsMsgStripLine(duppedNewsGrp);
 
@@ -502,7 +502,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     nsCOMPtr <nsINntpService> nntpService = do_GetService("@mozilla.org/messenger/nntpservice;1");
     if (NS_FAILED(rv) || !nntpService) {
       *status = NS_ERROR_FAILURE;
-      return nsnull;
+      return nullptr;
     }
 
     nsCString newsgroupsHeaderVal;
@@ -511,7 +511,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     if (NS_FAILED(rv)) 
     {
       *status = rv;
-      return nsnull;
+      return nullptr;
     }
 
     // fixme:the newsgroups header had better be encoded as the server-side
@@ -545,7 +545,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     char *duppedFollowup = PL_strdup(pFollow);
     if (!duppedFollowup) {
       PR_FREEIF(buffer);
-      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+      return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
     }
     char *n2 = nsMsgStripLine (duppedFollowup);
 
@@ -632,8 +632,8 @@ mime_generate_headers (nsMsgCompFields *fields,
 
     // Skip default priority.
     if (priorityValue != nsMsgPriority::Default) {
-      nsCAutoString priorityName;
-      nsCAutoString priorityValueString;
+      nsAutoCString priorityName;
+      nsAutoCString priorityValueString;
 
       NS_MsgGetPriorityValueString(priorityValue, priorityValueString);
       NS_MsgGetUntranslatedPriorityName(priorityValue, priorityName);
@@ -692,7 +692,7 @@ mime_generate_headers (nsMsgCompFields *fields,
   }
 
   if (buffer_tail > buffer + size - 1)
-    return nsnull;
+    return nullptr;
 
   /* realloc it smaller... */
   buffer = (char*)PR_REALLOC (buffer, buffer_tail - buffer + 1);
@@ -701,7 +701,7 @@ mime_generate_headers (nsMsgCompFields *fields,
 }
 
 static void
-GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
+GenerateGlobalRandomBytes(unsigned char *buf, int32_t len)
 {
   static bool      firstTime = true;
 
@@ -709,13 +709,11 @@ GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
   {
     // Seed the random-number generator with current time so that
     // the numbers will be different every time we run.
-    PRInt32 aTime;
-    LL_L2I(aTime, PR_Now());
-    srand( (unsigned)aTime );
+    srand( (unsigned)PR_Now() );
     firstTime = false;
   }
 
-  for( PRInt32 i = 0; i < len; i++ )
+  for( int32_t i = 0; i < len; i++ )
     buf[i] = rand() % 10;
 }
 
@@ -741,7 +739,7 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
 
 static char *
 LegacyParmFolding(const nsCString& aCharset,
-                  const nsCString& aFileName, PRInt32 aParmFolding);
+                  const nsCString& aFileName, int32_t aParmFolding);
 
 char *
 mime_generate_attachment_headers (const char *type,
@@ -762,13 +760,13 @@ mime_generate_attachment_headers (const char *type,
 {
   NS_ASSERTION (encoding, "null encoding");
 
-  PRInt32 parmFolding = 2; // RFC 2231-compliant
+  int32_t parmFolding = 2; // RFC 2231-compliant
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (prefs)
     prefs->GetIntPref("mail.strictly_mime.parm_folding", &parmFolding);
 
   /* Let's encode the real name */
-  char *encodedRealName = nsnull;
+  char *encodedRealName = nullptr;
   nsCString charset;   // actual charset used for MIME encode
   nsAutoString realName;
   if (real_name)
@@ -786,7 +784,7 @@ mime_generate_attachment_headers (const char *type,
         charset.Assign("UTF-8"); // set to UTF-8 if fails again
     }
 
-    encodedRealName = RFC2231ParmFolding("filename", charset, nsnull,
+    encodedRealName = RFC2231ParmFolding("filename", charset, nullptr,
                                          realName);
     // somehow RFC2231ParamFolding failed. fall back to legacy method
     if (!encodedRealName || !*encodedRealName) {
@@ -931,7 +929,7 @@ mime_generate_attachment_headers (const char *type,
 
   if (encodedRealName && *encodedRealName) {
     char *period = PL_strrchr(encodedRealName, '.');
-    PRInt32 pref_content_disposition = 0;
+    int32_t pref_content_disposition = 0;
 
     if (prefs) {
       nsresult rv = prefs->GetIntPref("mail.content_disposition_type",
@@ -981,7 +979,7 @@ mime_generate_attachment_headers (const char *type,
       !PL_strcasecmp (type, TEXT_MDL)) &&
       base_url && *base_url)
   {
-    PRInt32 col = 0;
+    int32_t col = 0;
     const char *s = base_url;
     const char *colon = PL_strchr (s, ':');
     bool useContentLocation = false;   /* rhp - add this  */
@@ -1015,7 +1013,7 @@ CONTENT_LOC_HACK:
 
     while (*s != 0 && *s != '#')
     {
-      PRUint32 ot=buf.Length();
+      uint32_t ot=buf.Length();
       char tmp[]="\x00\x00";
       /* URLs must be wrapped at 40 characters or less. */
       if (col >= 38) {
@@ -1075,25 +1073,19 @@ static bool isValidHost( const char* host )
          && *s != '.'
          )
       {
-       host = nsnull;
+       host = nullptr;
        break;
       }
 
-  return nsnull != host;
+  return nullptr != host;
 }
 
 char *
 msg_generate_message_id (nsIMsgIdentity *identity)
 {
-  PRUint32 now;
-  PRTime prNow = PR_Now();
-  PRInt64 microSecondsPerSecond, intermediateResult;
+  uint32_t now = (uint32_t)(PR_Now() / PR_USEC_PER_SEC);
 
-  LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
-  LL_DIV(intermediateResult, prNow, microSecondsPerSecond);
-    LL_L2UI(now, intermediateResult);
-
-  PRUint32 salt = 0;
+  uint32_t salt = 0;
   const char *host = 0;
 
   nsCString forcedFQDN;
@@ -1141,14 +1133,14 @@ inline static bool is7bitCharset(const nsCString& charset)
 RFC2231ParmFolding(const char *parmName, const nsCString& charset,
                    const char *language, const nsString& parmValue)
 {
-  NS_ENSURE_TRUE(parmName && *parmName && !parmValue.IsEmpty(), nsnull);
+  NS_ENSURE_TRUE(parmName && *parmName && !parmValue.IsEmpty(), nullptr);
 
   bool needEscape;
   nsCString dupParm;
 
   if (!NS_IsAscii(parmValue.get()) || is7bitCharset(charset)) {
     needEscape = true;
-    nsCAutoString nativeParmValue;
+    nsAutoCString nativeParmValue;
     ConvertFromUnicode(charset.get(), parmValue, nativeParmValue);
     MsgEscapeString(nativeParmValue, nsINetUtil::ESCAPE_ALL, dupParm);
   }
@@ -1160,19 +1152,19 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
   }
 
   if (dupParm.IsEmpty())
-    return nsnull;
+    return nullptr;
 
-  PRInt32 parmNameLen = PL_strlen(parmName);
-  PRInt32 parmValueLen = dupParm.Length();
+  int32_t parmNameLen = PL_strlen(parmName);
+  int32_t parmValueLen = dupParm.Length();
 
   if (needEscape)
     parmNameLen += 5;   // *=__'__'___ or *[0]*=__'__'__ or *[1]*=___
   else
     parmNameLen += 5;   // *[0]="___";
 
-  PRInt32 languageLen = language ?  PL_strlen(language) : 0;
-  PRInt32 charsetLen = charset.Length();
-  char *foldedParm = nsnull;
+  int32_t languageLen = language ?  PL_strlen(language) : 0;
+  int32_t charsetLen = charset.Length();
+  char *foldedParm = nullptr;
 
   if ((parmValueLen + parmNameLen + charsetLen + languageLen) <
       PR_MAX_FOLDING_LEN)
@@ -1249,25 +1241,25 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
         // check to see if we are in the middle of escaped char
         if (*end == '%')
         {
-          tmp = '%'; *end = nsnull;
+          tmp = '%'; *end = 0;
         }
         else if (end-1 > start && *(end-1) == '%')
         {
-          end -= 1; tmp = '%'; *end = nsnull;
+          end -= 1; tmp = '%'; *end = 0;
         }
         else if (end-2 > start && *(end-2) == '%')
         {
-          end -= 2; tmp = '%'; *end = nsnull;
+          end -= 2; tmp = '%'; *end = 0;
         }
         else
         {
-          tmp = *end; *end = nsnull;
+          tmp = *end; *end = 0;
         }
       }
       else
       {
         // XXX should check if we are in the middle of escaped char (RFC 822)
-        tmp = *end; *end = nsnull;
+        tmp = *end; *end = 0;
       }
       NS_MsgSACat(&foldedParm, start);
       if (!needEscape)
@@ -1285,7 +1277,7 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
 
 /*static */ char *
 LegacyParmFolding(const nsCString& aCharset,
-                  const nsCString& aFileName, PRInt32 aParmFolding)
+                  const nsCString& aFileName, int32_t aParmFolding)
 {
   bool usemime = nsMsgMIMEGetConformToStandard();
   char *encodedRealName =
@@ -1310,13 +1302,13 @@ LegacyParmFolding(const nsCString& aCharset,
 }
 
 bool
-mime_7bit_data_p (const char *string, PRUint32 size)
+mime_7bit_data_p (const char *string, uint32_t size)
 {
   if ((!string) || (!*string))
     return true;
 
   char *ptr = (char *)string;
-  for (PRUint32 i=0; i<size; i++)
+  for (uint32_t i=0; i<size; i++)
   {
     if ((unsigned char) ptr[i] > 0x7F)
       return false;
@@ -1335,7 +1327,7 @@ mime_fix_header_1 (const char *string, bool addr_p, bool news_p)
   char *new_string;
   const char *in;
   char *out;
-  PRInt32 i, old_size, new_size;
+  int32_t i, old_size, new_size;
 
   if (!string || !*string)
     return 0;
@@ -1600,15 +1592,15 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
     if (StringBeginsWith(url, NS_LITERAL_CSTRING("data:"),
                          nsCaseInsensitiveCStringComparator()))
     {
-      PRInt32 endNonData = url.FindChar(',');
+      int32_t endNonData = url.FindChar(',');
       if (endNonData == -1)
         return;
       nsCString nonDataPart(Substring(url, 5, endNonData - 5));
-      PRInt32 filenamePos = nonDataPart.Find("filename=");
+      int32_t filenamePos = nonDataPart.Find("filename=");
       if (filenamePos != -1)
       {
         filenamePos += 9;
-        PRInt32 endFilename = nonDataPart.FindChar(';', filenamePos);
+        int32_t endFilename = nonDataPart.FindChar(';', filenamePos);
         if (endFilename == -1)
           endFilename = endNonData;
         attachment->m_realName = Substring(nonDataPart, filenamePos,
@@ -1630,7 +1622,7 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
         mimeInfo->GetPrimaryExtension(extension);
         unsigned char filePrefix[10];
         GenerateGlobalRandomBytes(filePrefix, 8);
-        for (PRInt32 i = 0; i < 8; i++)
+        for (int32_t i = 0; i < 8; i++)
           filename.Append(filePrefix[i] + 'a');
         filename.Append('.');
         filename.Append(extension);
@@ -1647,7 +1639,7 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
       if (s2) s = s2+1;
       /* Copy it into the attachment struct. */
       attachment->m_realName = s;
-      PRInt32 charPos = attachment->m_realName.FindChar('?');
+      int32_t charPos = attachment->m_realName.FindChar('?');
       if (charPos != -1)
         attachment->m_realName.SetLength(charPos);
       /* Now trim off any named anchors or search data. */
@@ -1706,20 +1698,20 @@ nsresult
 nsMsgNewURL(nsIURI** aInstancePtrResult, const char * aSpec)
 {
   nsresult rv = NS_OK;
-  if (nsnull == aInstancePtrResult)
+  if (nullptr == aInstancePtrResult)
     return NS_ERROR_NULL_POINTER;
   nsCOMPtr<nsIIOService> pNetService =
     mozilla::services::GetIOService();
   NS_ENSURE_TRUE(pNetService, NS_ERROR_UNEXPECTED);
-  if (PL_strstr(aSpec, "://") == nsnull && strncmp(aSpec, "data:", 5))
+  if (PL_strstr(aSpec, "://") == nullptr && strncmp(aSpec, "data:", 5))
   {
     //XXXjag Temporary fix for bug 139362 until the real problem(bug 70083) get fixed
-    nsCAutoString uri(NS_LITERAL_CSTRING("http://"));
+    nsAutoCString uri(NS_LITERAL_CSTRING("http://"));
     uri.Append(aSpec);
-    rv = pNetService->NewURI(uri, nsnull, nsnull, aInstancePtrResult);
+    rv = pNetService->NewURI(uri, nullptr, nullptr, aInstancePtrResult);
   }
   else
-    rv = pNetService->NewURI(nsDependentCString(aSpec), nsnull, nsnull, aInstancePtrResult);
+    rv = pNetService->NewURI(nsDependentCString(aSpec), nullptr, nullptr, aInstancePtrResult);
   return rv;
 }
 
@@ -1758,18 +1750,18 @@ char
 char *
 nsMsgParseURLHost(const char *url)
 {
-  nsIURI        *workURI = nsnull;
+  nsIURI        *workURI = nullptr;
   nsresult      rv;
 
   rv = nsMsgNewURL(&workURI, url);
   if (NS_FAILED(rv) || !workURI)
-    return nsnull;
+    return nullptr;
 
-  nsCAutoString host;
+  nsAutoCString host;
   rv = workURI->GetHost(host);
   NS_IF_RELEASE(workURI);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   return ToNewCString(host);
 }
@@ -1781,15 +1773,15 @@ GenerateFileNameFromURI(nsIURI *aURL)
   nsCString file;
   nsCString spec;
   char        *returnString;
-  char        *cp = nsnull;
-  char        *cp1 = nsnull;
+  char        *cp = nullptr;
+  char        *cp1 = nullptr;
 
   rv = aURL->GetPath(file);
   if ( NS_SUCCEEDED(rv) && !file.IsEmpty())
   {
     char *newFile = ToNewCString(file);
     if (!newFile)
-      return nsnull;
+      return nullptr;
 
     // strip '/'
     cp = PL_strrchr(newFile, '/');
@@ -1811,11 +1803,11 @@ GenerateFileNameFromURI(nsIURI *aURL)
       }
     }
     else
-      return nsnull;
+      return nullptr;
   }
 
-  cp = nsnull;
-  cp1 = nsnull;
+  cp = nullptr;
+  cp1 = nullptr;
 
 
   rv = aURL->GetSpec(spec);
@@ -1823,7 +1815,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
   {
     char *newSpec = ToNewCString(spec);
     if (!newSpec)
-      return nsnull;
+      return nullptr;
 
     char *cp2 = NULL, *cp3=NULL ;
 
@@ -1851,7 +1843,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
     return returnString;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 //
@@ -1864,12 +1856,12 @@ GenerateFileNameFromURI(nsIURI *aURL)
 //      Content-ID: <part1.36DF1DCE.73B5A330@netscape.com>
 //
 char *
-mime_gen_content_id(PRUint32 aPartNum, const char *aEmailAddress)
+mime_gen_content_id(uint32_t aPartNum, const char *aEmailAddress)
 {
-  PRInt32           randLen = 5;
+  int32_t           randLen = 5;
   unsigned char     rand_buf1[5];
   unsigned char     rand_buf2[5];
-  const char        *domain = nsnull;
+  const char        *domain = nullptr;
   const char        *defaultDomain = "@netscape.com";
 
   memset(rand_buf1, 0, randLen-1);
