@@ -1,43 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jean-Francois Ducarroz <ducarroz@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Seth Spitzer <sspitzer@netscape.com>
- *   Jeff Beckley <beckley@qualcomm.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifdef MOZ_LOGGING
 #define FORCE_PR_LOG /* Allow logging in the release build (sorry this breaks the PCH) */
@@ -71,7 +35,6 @@
 #include "nsIBaseWindow.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranch2.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgAccountManager.h"
 #include "nsIMimeMiscStatus.h"
@@ -88,6 +51,8 @@
 #include "nsUTF8Utils.h"
 #include "nsILineBreaker.h"
 #include "nsLWBrkCIID.h"
+#include "mozilla/Services.h"
+#include "mimemoz2.h"
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 #include "prlog.h"
@@ -96,12 +61,6 @@
 #include "nsMsgUtils.h"
 #endif
 
-// <for functions="HTMLSantinize">
-#include "nsIParser.h"
-#include "nsParserCIID.h"
-#include "nsIContentSink.h"
-#include "mozISanitizingSerializer.h"
-
 #include "nsICommandLine.h"
 #include "nsIAppStartup.h"
 #include "nsMsgUtils.h"
@@ -112,10 +71,6 @@
 #include "nsIWidget.h"
 #endif
 
-static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
-// </for>
-
-
 #define DEFAULT_CHROME  "chrome://messenger/content/messengercompose/messengercompose.xul"
 
 #define PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS  "mail.compose.max_recycled_windows"
@@ -133,11 +88,11 @@ static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
 #define DOMAIN_DELIMITER                           ','
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
-static PRLogModuleInfo *MsgComposeLogModule = nsnull;
+static PRLogModuleInfo *MsgComposeLogModule = nullptr;
 
-static PRUint32 GetMessageSizeFromURI(const char * originalMsgURI)
+static uint32_t GetMessageSizeFromURI(const char * originalMsgURI)
 {
-  PRUint32 msgSize = 0;
+  uint32_t msgSize = 0;
 
   if (originalMsgURI && *originalMsgURI)
   {
@@ -165,7 +120,7 @@ nsMsgComposeService::nsMsgComposeService()
 #endif
 
   mMaxRecycledWindows = 0;
-  mCachedWindows = nsnull;
+  mCachedWindows = nullptr;
 }
 
 NS_IMPL_ISUPPORTS4(nsMsgComposeService,
@@ -191,15 +146,16 @@ nsresult nsMsgComposeService::Init()
   // Register observers
 
   // Register for quit application and profile change, we will need to clear the cache.
-  nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
-  if (NS_SUCCEEDED(rv))
+  nsCOMPtr<nsIObserverService> observerService =
+    mozilla::services::GetObserverService();
+  if (observerService)
   {
     rv = observerService->AddObserver(this, "quit-application", true);
     rv = observerService->AddObserver(this, "profile-do-change", true);
   }
 
   // Register some pref observer
-  nsCOMPtr<nsIPrefBranch2> pbi = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  nsCOMPtr<nsIPrefBranch> pbi = do_GetService(NS_PREFSERVICE_CONTRACTID);
   if (pbi)
     rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, true);
 
@@ -207,6 +163,11 @@ nsresult nsMsgComposeService::Init()
   Reset();
 
   AddGlobalHtmlDomains();
+  // Since the compose service should only be initialized once, we can
+  // be pretty sure there aren't any existing compose windows open.
+  MsgCleanupTempFiles("nsmail", "tmp");
+  MsgCleanupTempFiles("nsemail", "html");
+  MsgCleanupTempFiles("nscopy", "tmp");
   return rv;
 }
 
@@ -218,7 +179,7 @@ void nsMsgComposeService::Reset()
   {
     DeleteCachedWindows();
     delete [] mCachedWindows;
-    mCachedWindows = nsnull;
+    mCachedWindows = nullptr;
     mMaxRecycledWindows = 0;
   }
 
@@ -241,7 +202,7 @@ void nsMsgComposeService::Reset()
 
 void nsMsgComposeService::DeleteCachedWindows()
 {
-  PRInt32 i;
+  int32_t i;
   for (i = 0; i < mMaxRecycledWindows; i ++)
   {
     CloseHiddenCachedWindow(mCachedWindows[i].window);
@@ -286,7 +247,7 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char *chrome,
     rv = DetermineComposeHTML(identity, format, &composeHTML);
     if (NS_SUCCEEDED(rv))
     {
-      PRInt32 i;
+      int32_t i;
       for (i = 0; i < mMaxRecycledWindows; i ++)
       {
         if (mCachedWindows[i].window && (mCachedWindows[i].htmlCompose == composeHTML) && mCachedWindows[i].listener)
@@ -295,7 +256,8 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char *chrome,
              clear the cache entry if everything goes well
           */
           nsCOMPtr<nsIDOMWindow> domWindow(mCachedWindows[i].window);
-          rv = ShowCachedComposeWindow(domWindow, true);
+          nsCOMPtr<nsIXULWindow> xulWindow(mCachedWindows[i].xulWindow);
+          rv = ShowCachedComposeWindow(domWindow, xulWindow, true);
           if (NS_SUCCEEDED(rv))
           {
             mCachedWindows[i].listener->OnReopen(params);
@@ -457,7 +419,7 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
 
   nsCOMPtr<nsIDocShellTreeItem> childAsItem;
   rv = rootDocShellAsNode->FindChildWithName(NS_LITERAL_STRING("messagepane").get(),
-                                             true, false, nsnull, nsnull, getter_AddRefs(childAsItem));
+                                             true, false, nullptr, nullptr, getter_AddRefs(childAsItem));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(childAsItem, &rv));
@@ -470,13 +432,13 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool requireMultipleWords = true;
-  nsCAutoString charsOnlyIf;
+  nsAutoCString charsOnlyIf;
   prefs->GetBoolPref(PREF_MAILNEWS_REPLY_QUOTING_SELECTION_MULTI_WORD, &requireMultipleWords);
   prefs->GetCharPref(PREF_MAILNEWS_REPLY_QUOTING_SELECTION_ONLY_IF, getter_Copies(charsOnlyIf));
   if (sel && (requireMultipleWords || !charsOnlyIf.IsEmpty()))
   {
     nsAutoString selPlain;
-    rv = sel->ToString(getter_Copies(selPlain));
+    rv = sel->ToString(selPlain);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // If "mailnews.reply_quoting_selection.multi_word" is on, then there must be at least
@@ -487,9 +449,9 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
 
       if (NS_SUCCEEDED(rv))
       {
-        const PRUint32 length = selPlain.Length();
+        const uint32_t length = selPlain.Length();
         const PRUnichar* unicodeStr = selPlain.get();
-        PRInt32 endWordPos = lineBreaker->Next(unicodeStr, length, 0);
+        int32_t endWordPos = lineBreaker->Next(unicodeStr, length, 0);
         
         // If there's not even one word, then there's not multiple words
         if (endWordPos == NS_LINEBREAKER_NEED_MORE_TEXT)
@@ -552,7 +514,7 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
   if (type == nsIMsgCompType::ForwardInline || type == nsIMsgCompType::Draft || type == nsIMsgCompType::Template
     || type == nsIMsgCompType::ReplyWithTemplate || type == nsIMsgCompType::Redirect)
   {
-    nsCAutoString uriToOpen(originalMsgURI);
+    nsAutoCString uriToOpen(originalMsgURI);
     uriToOpen += (uriToOpen.FindChar('?') == kNotFound) ? '?' : '&';
     uriToOpen.Append("fetchCompleteMessage=true");
     if (type == nsIMsgCompType::Redirect)
@@ -583,7 +545,7 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
           type == nsIMsgCompType::ReplyToSenderAndGroup ||
           type == nsIMsgCompType::ReplyToList)
       {
-        nsCAutoString selHTML;
+        nsAutoCString selHTML;
         if (NS_SUCCEEDED(GetOrigWindowSelection(type, aMsgWindow, selHTML)))
           pMsgComposeParams->SetHtmlToQuote(selHTML);
       }
@@ -592,11 +554,11 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
       {
         if (type == nsIMsgCompType::NewsPost)
         {
-          nsCAutoString newsURI(originalMsgURI);
-          nsCAutoString group;
-          nsCAutoString host;
+          nsAutoCString newsURI(originalMsgURI);
+          nsAutoCString group;
+          nsAutoCString host;
 
-          PRInt32 slashpos = newsURI.RFindChar('/');
+          int32_t slashpos = newsURI.RFindChar('/');
           if (slashpos > 0 )
           {
             // uri is "[s]news://host[:port]/group"
@@ -607,12 +569,12 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
           else
             group = originalMsgURI;
 
-          nsCAutoString unescapedName;
+          nsAutoCString unescapedName;
           MsgUnescapeString(group,
                             nsINetUtil::ESCAPE_URL_FILE_BASENAME | nsINetUtil::ESCAPE_URL_FORCED,
                             unescapedName);
           pMsgCompFields->SetNewsgroups(NS_ConvertUTF8toUTF16(unescapedName));
-          pMsgCompFields->SetNewshost(host.get());
+          pMsgCompFields->SetNewspostUrl(host.get());
         }
         else
         {
@@ -696,34 +658,14 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
       {
         //For security reason, we must sanitize the message body before accepting any html...
 
-        // Create a parser
-        nsCOMPtr<nsIParser> parser = do_CreateInstance(kParserCID);
+        rv = HTMLSanitize(rawBody, sanitizedBody); // from mimemoz2.h
 
-        // Create the appropriate output sink
-        nsCOMPtr<nsIContentSink> sink = do_CreateInstance(MOZ_SANITIZINGHTMLSERIALIZER_CONTRACTID);
-
-        nsCString allowedTags;
-        nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-        if (prefs)
-          prefs->GetCharPref(MAILNEWS_ROOT_PREF "display.html_sanitizer.allowed_tags", getter_Copies(allowedTags));
-
-        if (parser && sink)
+        if (NS_FAILED(rv))
         {
-          nsCOMPtr<mozISanitizingHTMLSerializer> sanSink(do_QueryInterface(sink));
-          if (sanSink)
-          {
-            sanSink->Initialize(&sanitizedBody, 0, NS_ConvertASCIItoUTF16(allowedTags));
-
-            parser->SetContentSink(sink);
-            rv = parser->Parse(rawBody, 0, NS_LITERAL_CSTRING("text/html"), true);
-            if (NS_FAILED(rv))
-            {
-              // Something went horribly wrong with parsing for html format
-              // in the body.  Set composeHTMLFormat to false so we show the
-              // plain text mail compose.
-              composeHTMLFormat = false;
-            }
-          }
+          // Something went horribly wrong with parsing for html format
+          // in the body.  Set composeHTMLFormat to false so we show the
+          // plain text mail compose.
+          composeHTMLFormat = false;
         }
       }
 
@@ -755,7 +697,7 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
   } // if we had a url...
 
   // if we got here we must have encountered an error
-  *aParams = nsnull;
+  *aParams = nullptr;
   return NS_ERROR_FAILURE;
 }
 
@@ -776,7 +718,7 @@ NS_IMETHODIMP nsMsgComposeService::InitCompose(nsIMsgComposeParams *aParams,
                                                nsIMsgCompose **_retval)
 {
   // We need to remove the window from the cache.
-  PRInt32 i;
+  int32_t i;
   for (i = 0; i < mMaxRecycledWindows; i ++)
     if (mCachedWindows[i].window == aWindow)
     {
@@ -800,18 +742,17 @@ NS_IMETHODIMP
 nsMsgComposeService::GetDefaultIdentity(nsIMsgIdentity **_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
-  *_retval = nsnull;
+  *_retval = nullptr;
 
   nsresult rv;
   nsCOMPtr<nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
-  if (NS_SUCCEEDED(rv) && accountManager)
-  {
-    nsCOMPtr<nsIMsgAccount> defaultAccount;
-    rv = accountManager->GetDefaultAccount(getter_AddRefs(defaultAccount));
-    if (NS_SUCCEEDED(rv) && defaultAccount)
-      defaultAccount->GetDefaultIdentity(_retval);
-  }
-  return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMsgAccount> defaultAccount;
+  rv = accountManager->GetDefaultAccount(getter_AddRefs(defaultAccount));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return defaultAccount->GetDefaultIdentity(_retval);
 }
 
 /* readonly attribute boolean logComposePerformance; */
@@ -858,7 +799,7 @@ nsMsgComposeService::IsCachedWindow(nsIDOMWindow *aCachedWindow, bool *aIsCached
   NS_ENSURE_ARG_POINTER(aCachedWindow);
   NS_ENSURE_ARG_POINTER(aIsCachedWindow);
 
-  PRInt32 i;
+  int32_t i;
   for (i = 0; i < mMaxRecycledWindows; i ++)
     if (mCachedWindows[i].window.get() == aCachedWindow)
     {
@@ -877,18 +818,26 @@ nsMsgComposeService::CacheWindow(nsIDOMWindow *aWindow, bool aComposeHTML, nsIMs
   NS_ENSURE_ARG_POINTER(aListener);
 
   nsresult rv;
+  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(aWindow, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 i;
-  PRInt32 sameTypeId = -1;
-  PRInt32 oppositeTypeId = -1;
+  nsCOMPtr<nsIDocShellTreeItem> treeItem(do_QueryInterface(window->GetDocShell(), &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeItem, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  int32_t i;
+  int32_t sameTypeId = -1;
+  int32_t oppositeTypeId = -1;
 
   for (i = 0; i < mMaxRecycledWindows; i ++)
   {
     if (!mCachedWindows[i].window)
     {
-      rv = ShowCachedComposeWindow(aWindow, false);
+      rv = ShowCachedComposeWindow(aWindow, xulWindow, false);
       if (NS_SUCCEEDED(rv))
-        mCachedWindows[i].Initialize(aWindow, aListener, aComposeHTML);
+        mCachedWindows[i].Initialize(aWindow, xulWindow, aListener, aComposeHTML);
 
       return rv;
     }
@@ -914,9 +863,9 @@ nsMsgComposeService::CacheWindow(nsIDOMWindow *aWindow, bool aComposeHTML, nsIMs
     CloseHiddenCachedWindow(mCachedWindows[oppositeTypeId].window);
     mCachedWindows[oppositeTypeId].Clear();
 
-    rv = ShowCachedComposeWindow(aWindow, false);
+    rv = ShowCachedComposeWindow(aWindow, xulWindow, false);
     if (NS_SUCCEEDED(rv))
-      mCachedWindows[oppositeTypeId].Initialize(aWindow, aListener, aComposeHTML);
+      mCachedWindows[oppositeTypeId].Initialize(aWindow, xulWindow, aListener, aComposeHTML);
 
     return rv;
   }
@@ -989,7 +938,7 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
 
   rv = accountManager->FindAccountForServer(mServer, getter_AddRefs(account));
   NS_ENSURE_SUCCESS(rv, rv);
-  account->GetDefaultIdentity(getter_AddRefs(identity));
+  rv = account->GetDefaultIdentity(getter_AddRefs(identity));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // create the compose params object
@@ -1035,12 +984,12 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
 
   /** initialize nsIMsgCompose, Send the message, wait for send completion response **/
 
-  rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nsnull);
+  rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nullptr);
   NS_ENSURE_SUCCESS(rv,rv);
 
   Release();
 
-  return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nsnull, nsnull, nsnull) ;
+  return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nullptr, nullptr, nullptr) ;
 }
 
 NS_IMETHODIMP
@@ -1065,20 +1014,21 @@ NS_IMETHODIMP
 nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
                                   nsISupports* aSupport,
                                   nsIInputStream* inStream,
-                                  PRUint32 srcOffset,
-                                  PRUint32 count)
+                                  uint64_t srcOffset,
+                                  uint32_t count)
 {
   nsresult rv = NS_OK;
 
   char readBuf[1024];
 
-  PRUint32 available, readCount;
-  PRUint32 maxReadCount = sizeof(readBuf) - 1;
+  uint64_t available;
+  uint32_t readCount;
+  uint32_t maxReadCount = sizeof(readBuf) - 1;
 
   rv = inStream->Available(&available);
   while (NS_SUCCEEDED(rv) && available > 0)
   {
-    PRUint32 bodyOffset = 0, readOffset = 0;
+    uint32_t bodyOffset = 0, readOffset = 0;
     if (!mInMsgBody && mLastBlockChars[0])
     {
       memcpy(readBuf, mLastBlockChars, 3);
@@ -1086,7 +1036,7 @@ nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
       maxReadCount -= 3;
     }
     if (maxReadCount > available)
-      maxReadCount = available;
+      maxReadCount = (uint32_t)available;
     memset(readBuf, 0, sizeof(readBuf));
     rv = inStream->Read(readBuf + readOffset, maxReadCount, &readCount);
     available -= readCount;
@@ -1096,7 +1046,7 @@ nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
     // looks like <CR><CR>, <LF><LF>, or <CRLF><CRLF>
     if (!mInMsgBody)
     {
-      for (PRUint32 charIndex = 0; charIndex < readCount && !bodyOffset; charIndex++)
+      for (uint32_t charIndex = 0; charIndex < readCount && !bodyOffset; charIndex++)
       {
         if (readBuf[charIndex] == '\r' || readBuf[charIndex] == '\n')
         {
@@ -1155,7 +1105,7 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const
   if (!query)
     return NS_ERROR_FAILURE;
 
-  nsCAutoString folderUri(Substring(templateUri, query)); 
+  nsAutoCString folderUri(Substring(templateUri, query)); 
   nsresult rv = GetExistingFolder(folderUri, getter_AddRefs(templateFolder));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = templateFolder->GetMsgDatabase(getter_AddRefs(templateDB));
@@ -1165,8 +1115,8 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const
   if (subject)
   {
     const char *subjectEnd = subject + strlen(subject);
-    nsCAutoString messageId(Substring(query + 11, subject));
-    nsCAutoString subjectString(Substring(subject + 9, subjectEnd));
+    nsAutoCString messageId(Substring(query + 11, subject));
+    nsAutoCString subjectString(Substring(subject + 9, subjectEnd));
     templateDB->GetMsgHdrForMessageID(messageId.get(), getter_AddRefs(helper->mTemplateHdr));
     if (helper->mTemplateHdr)
       templateFolder->GetUriForMsg(helper->mTemplateHdr, templateMsgHdrUri);
@@ -1193,7 +1143,7 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const
   rv = msgService->StreamMessage(templateMsgHdrUri.get(), listenerSupports,
                                  aMsgWindow, helper,
                                  false, // convert data
-                                 EmptyCString(), false, nsnull);
+                                 EmptyCString(), false, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgFolder> folder;
@@ -1211,14 +1161,14 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
                                     nsIMsgDBHdr *aMsgHdr,
                                     nsIMsgWindow *aMsgWindow,
                                     nsIMsgIncomingServer *aServer,
-                                    PRUint32 aForwardType)
+                                    uint32_t aForwardType)
 {
   NS_ENSURE_ARG_POINTER(aMsgHdr);
 
   nsresult rv;
   if (aForwardType == nsIMsgComposeService::kForwardAsDefault)
   {
-    PRInt32 forwardPref = 0;
+    int32_t forwardPref = 0;
     nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
     prefBranch->GetIntPref("mail.forward_message_mode", &forwardPref);
@@ -1231,12 +1181,12 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
 
   nsCOMPtr<nsIMsgFolder> folder;
   aMsgHdr->GetFolder(getter_AddRefs(folder));
-  if (!folder)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(folder, NS_ERROR_NULL_POINTER);
+
   folder->GetUriForMsg(aMsgHdr, msgUri);
 
   // get the MsgIdentity for the above key using AccountManager
-  nsCOMPtr<nsIMsgAccountManager> accountManager = 
+  nsCOMPtr<nsIMsgAccountManager> accountManager =
     do_GetService (NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1246,7 +1196,12 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   rv = accountManager->FindAccountForServer(aServer, getter_AddRefs(account));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = account->GetDefaultIdentity(getter_AddRefs(identity));
-  NS_ENSURE_SUCCESS(rv, rv);
+  // Use default identity if no identity has been found on this account
+  if (NS_FAILED(rv) || !identity)
+  {
+    rv = GetDefaultIdentity(getter_AddRefs(identity));
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   if (aForwardType == nsIMsgComposeService::kForwardInline)
     return RunMessageThroughMimeDraft(msgUri,
@@ -1279,13 +1234,13 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   pMsgComposeParams->SetOriginalMsgURI(msgUri.get());
   // create the nsIMsgCompose object to send the object
   nsCOMPtr<nsIMsgCompose> pMsgCompose (do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   /** initialize nsIMsgCompose, Send the message, wait for send completion response **/
-  rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nsnull);
-  NS_ENSURE_SUCCESS(rv,rv);
+  rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nullptr);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nsnull, nsnull, nsnull);
+  rv = pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nullptr, nullptr, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // nsMsgCompose::ProcessReplyFlags usually takes care of marking messages
@@ -1296,13 +1251,13 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   return folder->AddMessageDispositionState(aMsgHdr, nsIMsgFolder::nsMsgDispositionState_Forwarded);
 }
 
-nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindow *aComposeWindow, bool aShow)
+nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindow *aComposeWindow, nsIXULWindow *aXULWindow, bool aShow)
 {
   nsresult rv = NS_OK;
 
-  nsCOMPtr<nsIObserverService> obs
-    (do_GetService("@mozilla.org/observer-service;1", &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIObserverService> obs =
+    mozilla::services::GetObserverService();
+  NS_ENSURE_TRUE(obs, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr <nsPIDOMWindow> window = do_QueryInterface(aComposeWindow, &rv);
 
@@ -1334,19 +1289,16 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindow *aComposeWind
     baseWindow->SetEnabled(aShow);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    nsCOMPtr <nsIXULWindow> xulWindow = do_GetInterface(treeOwner, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
     nsCOMPtr<nsIWindowMediator> windowMediator =
 	         do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
     // if showing, reinstate the window with the mediator
     if (aShow) {
-      rv = windowMediator->RegisterWindow(xulWindow);
+      rv = windowMediator->RegisterWindow(aXULWindow);
       NS_ENSURE_SUCCESS(rv,rv);
 
-      obs->NotifyObservers(xulWindow, "xul-window-registered", nsnull);
+      obs->NotifyObservers(aXULWindow, "xul-window-registered", nullptr);
     }
 
     // hide (show) the cached window
@@ -1356,10 +1308,10 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindow *aComposeWind
     // if hiding, remove the window from the mediator,
     // so that it will be removed from the task list
     if (!aShow) {
-      rv = windowMediator->UnregisterWindow(xulWindow);
+      rv = windowMediator->UnregisterWindow(aXULWindow);
       NS_ENSURE_SUCCESS(rv,rv);
 
-      obs->NotifyObservers(xulWindow, "xul-window-destroyed", nsnull);
+      obs->NotifyObservers(aXULWindow, "xul-window-destroyed", nullptr);
     }
   }
   else {
@@ -1397,7 +1349,7 @@ nsresult nsMsgComposeService::AddGlobalHtmlDomains()
    * This pref contains the list of html domains that ISP can add to make that user's contain all
    * of these under the HTML domains in the Mail&NewsGrpus|Send Format under global preferences.
    */
-  PRInt32 htmlDomainListCurrentVersion, htmlDomainListDefaultVersion;
+  int32_t htmlDomainListCurrentVersion, htmlDomainListDefaultVersion;
   rv = prefBranch->GetIntPref(HTMLDOMAINUPDATE_VERSION_PREF_NAME, &htmlDomainListCurrentVersion);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1418,7 +1370,7 @@ nsresult nsMsgComposeService::AddGlobalHtmlDomains()
       rv = prefBranch->GetCharPref(USER_CURRENT_HTMLDOMAINLIST_PREF_NAME, getter_Copies(currentHtmlDomainList));
       NS_ENSURE_SUCCESS(rv,rv);
 
-      nsCAutoString newHtmlDomainList(currentHtmlDomainList);
+      nsAutoCString newHtmlDomainList(currentHtmlDomainList);
       // Get the current html domain list into new list var
       ParseString(currentHtmlDomainList, DOMAIN_DELIMITER, domainArray);
 
@@ -1430,7 +1382,7 @@ nsresult nsMsgComposeService::AddGlobalHtmlDomains()
       // Get the current plaintext domain list into new list var
       ParseString(currentPlaintextDomainList, DOMAIN_DELIMITER, domainArray);
 
-      PRUint32 i = domainArray.Length();
+      uint32_t i = domainArray.Length();
       if (i > 0) {
         // Append each domain in the preconfigured html domain list
         globalHtmlDomainList.StripWhitespace();
@@ -1639,17 +1591,17 @@ nsMsgComposeService::RunMessageThroughMimeDraft(
   }
 
   nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewInputStreamChannel(getter_AddRefs(channel), url, nsnull);
+  rv = NS_NewInputStreamChannel(getter_AddRefs(channel), url, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIStreamConverter> converter = do_QueryInterface(mimeConverter);
-  rv = converter->AsyncConvertData(nsnull, nsnull, nsnull, channel);
+  rv = converter->AsyncConvertData(nullptr, nullptr, nullptr, channel);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Now, just plug the two together and get the hell out of the way!
   nsCOMPtr<nsIStreamListener> streamListener = do_QueryInterface(mimeConverter);
   return messageService->DisplayMessage(PromiseFlatCString(aMsgURI).get(), streamListener,
-                                        aMsgWindow, nsnull, mailCharset.get(), nsnull);;
+                                        aMsgWindow, nullptr, mailCharset.get(), nullptr);;
 }
 
 NS_IMETHODIMP
@@ -1658,7 +1610,7 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine)
   NS_ENSURE_ARG_POINTER(aCmdLine);
 
   nsresult rv;
-  PRInt32 found, end, count;
+  int32_t found, end, count;
   nsAutoString uristr;
   bool composeShouldHandle = true;
 
@@ -1728,7 +1680,7 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine)
       arg->SetData(uristr);
 
     nsCOMPtr<nsIDOMWindow> opened;
-    wwatch->OpenWindow(nsnull, DEFAULT_CHROME, "_blank",
+    wwatch->OpenWindow(nullptr, DEFAULT_CHROME, "_blank",
                        "chrome,dialog=no,all", arg, getter_AddRefs(opened));
 
     aCmdLine->SetPreventDefault(true);
@@ -1742,3 +1694,4 @@ nsMsgComposeService::GetHelpInfo(nsACString& aResult)
   aResult.Assign(NS_LITERAL_CSTRING("  -compose           Compose a mail or news message.\n"));
   return NS_OK;
 }
+
