@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2002
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jeff Tsai <jt95070@netscape.net>
- *   Michiel van Leeuwen <mvl@exedo.nl>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMsgMdnGenerator.h"
 #include "nsImapCore.h"
@@ -64,6 +30,9 @@
 #include "nsMsgUtils.h"
 #include "nsNetUtil.h"
 #include "nsIMsgDatabase.h"
+#include "mozilla/Services.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 
 #define MDN_NOT_IN_TO_CC          ((int) 0x0001)
 #define MDN_OUTSIDE_DOMAIN        ((int) 0x0002)
@@ -113,7 +82,7 @@ NS_IMPL_ISUPPORTS2(nsMsgMdnGenerator, nsIMsgMdnGenerator, nsIUrlListener)
 nsMsgMdnGenerator::nsMsgMdnGenerator()
 {
     m_disposeType = eDisplayed;
-    m_outputStream = nsnull;
+    m_outputStream = nullptr;
     m_reallySendMdn = false;
     m_autoSend = false;
     m_autoAction = false;
@@ -132,15 +101,14 @@ nsresult nsMsgMdnGenerator::FormatStringFromName(const PRUnichar *aName,
                                                  PRUnichar **aResultString)
 {
     DEBUG_MDN("nsMsgMdnGenerator::FormatStringFromName");
-    nsresult rv;
 
-    nsCOMPtr<nsIStringBundleService>
-        bundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
-    NS_ENSURE_SUCCESS(rv,rv);
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
+    NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
     nsCOMPtr <nsIStringBundle> bundle;
-    rv = bundleService->CreateBundle(MDN_STRINGBUNDLE_URL,
-                                     getter_AddRefs(bundle));
+    nsresult rv = bundleService->CreateBundle(MDN_STRINGBUNDLE_URL,
+                                              getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv,rv);
 
     const PRUnichar *formatStrings[1] = { aString };
@@ -154,15 +122,14 @@ nsresult nsMsgMdnGenerator::GetStringFromName(const PRUnichar *aName,
                                                PRUnichar **aResultString)
 {
     DEBUG_MDN("nsMsgMdnGenerator::GetStringFromName");
-    nsresult rv;
 
-    nsCOMPtr<nsIStringBundleService>
-        bundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
-    NS_ENSURE_SUCCESS(rv,rv);
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
+    NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
     nsCOMPtr <nsIStringBundle> bundle;
-    rv = bundleService->CreateBundle(MDN_STRINGBUNDLE_URL,
-                                     getter_AddRefs(bundle));
+    nsresult rv = bundleService->CreateBundle(MDN_STRINGBUNDLE_URL,
+                                              getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv,rv);
 
     rv = bundle->GetStringFromName(aName, aResultString);
@@ -178,12 +145,12 @@ nsresult nsMsgMdnGenerator::StoreMDNSentFlag(nsIMsgFolder *folder,
     nsCOMPtr<nsIMsgDatabase> msgDB;
     nsresult rv = folder->GetMsgDatabase(getter_AddRefs(msgDB));
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = msgDB->MarkMDNSent(key, true, nsnull);
+    rv = msgDB->MarkMDNSent(key, true, nullptr);
 
     nsCOMPtr<nsIMsgImapMailFolder> imapFolder = do_QueryInterface(folder);
     // Store the $MDNSent flag if the folder is an Imap Mail Folder
     if (imapFolder)
-      return imapFolder->StoreImapFlags(kImapMsgMDNSentFlag, true, &key, 1, nsnull);
+      return imapFolder->StoreImapFlags(kImapMsgMDNSentFlag, true, &key, 1, nullptr);
     return rv;
 }
 
@@ -195,13 +162,13 @@ nsresult nsMsgMdnGenerator::ClearMDNNeededFlag(nsIMsgFolder *folder,
   nsCOMPtr<nsIMsgDatabase> msgDB;
   nsresult rv = folder->GetMsgDatabase(getter_AddRefs(msgDB));
   NS_ENSURE_SUCCESS(rv, rv);
-  return msgDB->MarkMDNNeeded(key, false, nsnull);
+  return msgDB->MarkMDNNeeded(key, false, nullptr);
 }
 
 bool nsMsgMdnGenerator::ProcessSendMode()
 {
     DEBUG_MDN("nsMsgMdnGenerator::ProcessSendMode");
-    PRInt32 miscState = 0;
+    int32_t miscState = 0;
 
     if (m_identity)
     {
@@ -319,9 +286,9 @@ bool nsMsgMdnGenerator::MailAddrMatch(const char *addr1, const char *addr2)
     // insensitive
     DEBUG_MDN("nsMsgMdnGenerator::MailAddrMatch");
     bool isMatched = true;
-    const char *atSign1 = nsnull, *atSign2 = nsnull;
-    const char *lt = nsnull, *local1 = nsnull, *local2 = nsnull;
-    const char *end1 = nsnull, *end2 = nsnull;
+    const char *atSign1 = nullptr, *atSign2 = nullptr;
+    const char *lt = nullptr, *local1 = nullptr, *local2 = nullptr;
+    const char *end1 = nullptr, *end2 = nullptr;
 
     if (!addr1 || !addr2)
         return false;
@@ -407,10 +374,9 @@ nsresult nsMsgMdnGenerator::CreateMdnMsg()
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = m_file->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 00600);
-    nsCOMPtr <nsILocalFile> localFile = do_QueryInterface(m_file);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = NS_NewLocalFileOutputStream(getter_AddRefs(m_outputStream),
-                                     localFile,
+                                     m_file,
                                      PR_CREATE_FILE | PR_WRONLY | PR_TRUNCATE,
                                      0664);
     NS_ASSERTION(NS_SUCCEEDED(rv),"creating mdn: failed to output stream");
@@ -441,8 +407,8 @@ nsresult nsMsgMdnGenerator::CreateMdnMsg()
 nsresult nsMsgMdnGenerator::CreateFirstPart()
 {
     DEBUG_MDN("nsMsgMdnGenerator::CreateFirstPart");
-    char *convbuf = nsnull, *tmpBuffer = nsnull;
-    char *parm = nsnull;
+    char *convbuf = nullptr, *tmpBuffer = nullptr;
+    char *parm = nullptr;
     nsString firstPart1;
     nsString firstPart2;
     nsresult rv = NS_OK;
@@ -684,8 +650,8 @@ report-type=disposition-notification;\r\n\tboundary=\"%s\"" CRLF CRLF,
 nsresult nsMsgMdnGenerator::CreateSecondPart()
 {
     DEBUG_MDN("nsMsgMdnGenerator::CreateSecondPart");
-    char *tmpBuffer = nsnull;
-    char *convbuf = nsnull;
+    char *tmpBuffer = nullptr;
+    char *convbuf = nullptr;
     nsresult rv = NS_OK;
     nsCOMPtr <nsIMsgCompUtils> compUtils;
     bool conformToStandard = false;
@@ -707,7 +673,7 @@ nsresult nsMsgMdnGenerator::CreateSecondPart()
         do_GetService(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX "http", &rv);
     if (NS_SUCCEEDED(rv) && pHTTPHandler)
     {
-      nsCAutoString userAgentString;
+      nsAutoCString userAgentString;
       pHTTPHandler->GetUserAgent(userAgentString);
 
       if (!userAgentString.IsEmpty())
@@ -771,7 +737,7 @@ nsresult nsMsgMdnGenerator::CreateSecondPart()
 nsresult nsMsgMdnGenerator::CreateThirdPart()
 {
     DEBUG_MDN("nsMsgMdnGenerator::CreateThirdPart");
-    char *tmpBuffer = nsnull;
+    char *tmpBuffer = nullptr;
     nsresult rv = NS_OK;
 
     tmpBuffer = PR_smprintf("--%s" CRLF, m_mimeSeparator.get());
@@ -806,7 +772,7 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders()
 {
     DEBUG_MDN("nsMsgMdnGenerator::OutputAllHeaders");
     nsCString all_headers;
-    PRInt32 all_headers_size = 0;
+    int32_t all_headers_size = 0;
     nsresult rv = NS_OK;
 
     rv = m_headers->GetAllHeaders(getter_Copies(all_headers));
@@ -816,7 +782,6 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders()
     char *buf = (char *) all_headers.get(),
         *buf_end = (char *) all_headers.get()+all_headers_size;
     char *start = buf, *end = buf;
-    PRInt32 count = 0;
 
     while (buf < buf_end)
     {
@@ -882,7 +847,7 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders()
             buf = start;
         }
     }
-    return count;
+    return NS_OK;
 }
 
 nsresult nsMsgMdnGenerator::SendMdnMsg()
@@ -894,7 +859,7 @@ nsresult nsMsgMdnGenerator::SendMdnMsg()
 
     nsCOMPtr<nsIRequest> aRequest;
     smtpService->SendMailMessage(m_file, m_dntRrt.get(), m_identity,
-                                     nsnull, this, nsnull, nsnull, false, nsnull,
+                                     nullptr, this, nullptr, nullptr, false, nullptr,
                                      getter_AddRefs(aRequest));
 
     return NS_OK;
@@ -903,8 +868,8 @@ nsresult nsMsgMdnGenerator::SendMdnMsg()
 nsresult nsMsgMdnGenerator::WriteString( const char *str )
 {
   NS_ENSURE_ARG (str);
-  PRUint32 len = strlen(str);
-  PRUint32 wLen = 0;
+  uint32_t len = strlen(str);
+  uint32_t wLen = 0;
 
   return m_outputStream->Write(str, len, &wLen);
 }
@@ -937,18 +902,18 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool *needToAskUser)
             nsCString mailCC;
             m_headers->ExtractHeader(HEADER_TO, true, getter_Copies(mailTo));
             m_headers->ExtractHeader(HEADER_CC, true, getter_Copies(mailCC));
-            nsCOMPtr<nsISupportsArray> servIdentities;
+            nsCOMPtr<nsIArray> servIdentities;
             accountManager->GetIdentitiesForServer(m_server, getter_AddRefs(servIdentities));
             if (servIdentities)
             {
               nsCOMPtr<nsIMsgIdentity> ident;
               nsCString identEmail;
-              PRUint32 count = 0;
-              servIdentities->Count(&count);
+              uint32_t count = 0;
+              servIdentities->GetLength(&count);
               // First check in the "To:" header
-              for (PRUint32 i = 0; i < count; i++)
+              for (uint32_t i = 0; i < count; i++)
               {
-                rv = servIdentities->QueryElementAt(i, NS_GET_IID(nsIMsgIdentity),getter_AddRefs(ident));
+                ident = do_QueryElementAt(servIdentities, i, &rv);
                 if (NS_FAILED(rv))
                   continue;
                 ident->GetEmail(identEmail);
@@ -962,7 +927,7 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool *needToAskUser)
               // If no match, check the "Cc:" header
               if (!m_identity)
               {
-                for (PRUint32 i = 0; i < count; i++)
+                for (uint32_t i = 0; i < count; i++)
                 {
                   rv = servIdentities->QueryElementAt(i, NS_GET_IID(nsIMsgIdentity),getter_AddRefs(ident));
                   if (NS_FAILED(rv))
@@ -1158,8 +1123,9 @@ NS_IMETHODIMP nsMsgMdnGenerator::OnStopRunningUrl(nsIURI *url,
     const PRUnichar *params[] = { hostStr.get() };
 
     nsCOMPtr<nsIStringBundle> bundle;
-    nsCOMPtr<nsIStringBundleService> bundleService(do_GetService("@mozilla.org/intl/stringbundle;1", &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
+    NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
     rv = bundleService->CreateBundle("chrome://messenger/locale/messengercompose/composeMsgs.properties", getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv, rv);
