@@ -1,42 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   David Drinan <ddrinan@netscape.com>
- *   Stephane Saux <ssaux@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMsgComposeSecure.h"
 
@@ -53,16 +19,23 @@
 #include "nsServiceManagerUtils.h"
 #include "nsMemory.h"
 #include "nsAlgorithm.h"
+#include "mozilla/Services.h"
 
 #define MK_MIME_ERROR_WRITING_FILE -1
 
 #define SMIME_STRBUNDLE_URL "chrome://messenger/locale/am-smime.properties"
 
+// It doesn't make sense to encode the message because the message will be
+// displayed only if the MUA doesn't support MIME.
+// We need to consider what to do in case the server doesn't support 8BITMIME.
+// In short, we can't use non-ASCII characters here.
+static const char crypto_multipart_blurb[] = "This is a cryptographically signed message in MIME format.";
+
 static void mime_crypto_write_base64 (void *closure, const char *buf,
               unsigned long size);
-static nsresult mime_encoder_output_fn(const char *buf, PRInt32 size, void *closure);
-static nsresult mime_nested_encoder_output_fn (const char *buf, PRInt32 size, void *closure);
-static int make_multipart_signed_header_string(bool outer_p,
+static int mime_encoder_output_fn(const char *buf, int32_t size, void *closure);
+static int mime_nested_encoder_output_fn (const char *buf, int32_t size, void *closure);
+static nsresult make_multipart_signed_header_string(bool outer_p,
                   char **header_return,
                   char **boundary_return);
 static char *mime_make_separator(const char *prefix);
@@ -70,64 +43,50 @@ static char *mime_make_separator(const char *prefix);
 // mscott --> FIX ME...for now cloning code from compose\nsMsgEncode.h/.cpp
 
 MimeEncoderData *
-MIME_B64EncoderInit(nsresult (* output_fn) (const char *buf, PRInt32 size, void *closure), void *closure) 
+MIME_B64EncoderInit(MimeConverterOutputCallback output_fn, void *closure)
 {
-  MimeEncoderData *returnEncoderData = nsnull;
-  nsIMimeConverter *converter;
-  nsresult res = CallCreateInstance(NS_MIME_CONVERTER_CONTRACTID, &converter);
-  if (NS_SUCCEEDED(res) && nsnull != converter) 
-  {
-    res = converter->B64EncoderInit(output_fn, closure, &returnEncoderData);
-    NS_RELEASE(converter);
-  }
-  return NS_SUCCEEDED(res) ? returnEncoderData : nsnull;
+  MimeEncoderData *returnEncoderData = nullptr;
+  nsCOMPtr<nsIMimeConverter> converter = do_GetService(NS_MIME_CONVERTER_CONTRACTID);
+  NS_ENSURE_TRUE(converter, nullptr);
+
+  nsresult res = converter->B64EncoderInit(output_fn, closure, &returnEncoderData);
+  return NS_SUCCEEDED(res) ? returnEncoderData : nullptr;
 }
 
-MimeEncoderData * 
-MIME_QPEncoderInit(nsresult (* output_fn) (const char *buf, PRInt32 size, void *closure), void *closure) 
+MimeEncoderData *
+MIME_QPEncoderInit(MimeConverterOutputCallback output_fn, void *closure)
 {
-  MimeEncoderData *returnEncoderData = nsnull;
-  nsIMimeConverter *converter;
-  nsresult res = CallCreateInstance(NS_MIME_CONVERTER_CONTRACTID, &converter);
-  if (NS_SUCCEEDED(res) && nsnull != converter) 
-  {
-    res = converter->QPEncoderInit(output_fn, closure, &returnEncoderData);
-    NS_RELEASE(converter);
-  }
-  return NS_SUCCEEDED(res) ? returnEncoderData : nsnull;
+  MimeEncoderData *returnEncoderData = nullptr;
+  nsCOMPtr<nsIMimeConverter> converter = do_GetService(NS_MIME_CONVERTER_CONTRACTID);
+  NS_ENSURE_TRUE(converter, nullptr);
+
+  nsresult res = converter->QPEncoderInit(output_fn, closure, &returnEncoderData);
+  return NS_SUCCEEDED(res) ? returnEncoderData : nullptr;
 }
 
 nsresult
 MIME_EncoderDestroy(MimeEncoderData *data, bool abort_p) 
 {
-  //MimeEncoderData *returnEncoderData = nsnull;
-  nsIMimeConverter *converter;
-  nsresult res = CallCreateInstance(NS_MIME_CONVERTER_CONTRACTID, &converter);
-  if (NS_SUCCEEDED(res) && nsnull != converter) 
-  {
-    res = converter->EncoderDestroy(data, abort_p);
-    NS_RELEASE(converter);
-  }
+  nsresult rv = NS_OK;
+  nsCOMPtr<nsIMimeConverter> converter = do_GetService(NS_MIME_CONVERTER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  return NS_SUCCEEDED(res) ? 0 : -1;
+  return converter->EncoderDestroy(data, abort_p);
 }
 
 nsresult
-MIME_EncoderWrite(MimeEncoderData *data, const char *buffer, PRInt32 size) 
+MIME_EncoderWrite(MimeEncoderData *data, const char *buffer, int32_t size) 
 {
-  //  MimeEncoderData *returnEncoderData = nsnull;
-  nsIMimeConverter *converter;
-  PRInt32 written = 0;
-  nsresult res = CallCreateInstance(NS_MIME_CONVERTER_CONTRACTID, &converter);
-  if (NS_SUCCEEDED(res) && nsnull != converter) {
-    res = converter->EncoderWrite(data, buffer, size, &written);
-    NS_RELEASE(converter);
-  }
-  return NS_SUCCEEDED(res) ? 0 : -1;
+  nsresult rv;
+  nsCOMPtr<nsIMimeConverter> converter = do_GetService(NS_MIME_CONVERTER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  int32_t written = 0;
+  return converter->EncoderWrite(data, buffer, size, &written);
 }
 
 static void
-GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
+GenerateGlobalRandomBytes(unsigned char *buf, int32_t len)
 {
   static bool      firstTime = true;
   
@@ -135,13 +94,11 @@ GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
   {
     // Seed the random-number generator with current time so that
     // the numbers will be different every time we run.
-    PRInt32 aTime;
-    LL_L2I(aTime, PR_Now());
-    srand( (unsigned)aTime );
+    srand( (unsigned)PR_Now() );
     firstTime = false;
   }
   
-  for( PRInt32 i = 0; i < len; i++ )
+  for( int32_t i = 0; i < len; i++ )
     buf[i] = rand() % 10;
 }
    
@@ -245,12 +202,12 @@ NS_IMETHODIMP nsMsgComposeSecure::RequiresCryptoEncapsulation(nsIMsgIdentity * a
 {
   NS_ENSURE_ARG_POINTER(aRequiresEncryptionWork);
 
-  nsresult rv = NS_OK;
   *aRequiresEncryptionWork = false;
 
   bool alwaysEncryptMessages = false;
   bool signMessage = false;
-  rv = ExtractEncryptionState(aIdentity, aCompFields, &signMessage, &alwaysEncryptMessages);
+  nsresult rv = ExtractEncryptionState(aIdentity, aCompFields, &signMessage, &alwaysEncryptMessages);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (alwaysEncryptMessages || signMessage)
     *aRequiresEncryptionWork = true;
@@ -262,21 +219,20 @@ NS_IMETHODIMP nsMsgComposeSecure::RequiresCryptoEncapsulation(nsIMsgIdentity * a
 nsresult nsMsgComposeSecure::GetSMIMEBundleString(const PRUnichar *name,
                                                   PRUnichar **outString)
 {
-  *outString = nsnull;
+  *outString = nullptr;
 
   NS_ENSURE_ARG_POINTER(name);
 
-  if (!InitializeSMIMEBundle())
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(InitializeSMIMEBundle(), NS_ERROR_FAILURE);
 
-  return NS_SUCCEEDED(mSMIMEBundle->GetStringFromName(name, outString));
+  return mSMIMEBundle->GetStringFromName(name, outString);
 }
 
 nsresult
 nsMsgComposeSecure::
 SMIMEBundleFormatStringFromName(const PRUnichar *name,
                                 const PRUnichar **params,
-                                PRUint32 numParams,
+                                uint32_t numParams,
                                 PRUnichar **outString)
 {
   NS_ENSURE_ARG_POINTER(name);
@@ -294,7 +250,7 @@ bool nsMsgComposeSecure::InitializeSMIMEBundle()
     return true;
 
   nsCOMPtr<nsIStringBundleService> bundleService =
-    do_GetService(NS_STRINGBUNDLE_CONTRACTID);
+    mozilla::services::GetStringBundleService();
   nsresult rv = bundleService->CreateBundle(SMIME_STRBUNDLE_URL,
                                             getter_AddRefs(mSMIMEBundle));
   NS_ENSURE_SUCCESS(rv, false);
@@ -380,7 +336,7 @@ nsresult nsMsgComposeSecure::ExtractEncryptionState(nsIMsgIdentity * aIdentity,
   }
 
   // get the default info from the identity....
-  PRInt32 ep = 0;
+  int32_t ep = 0;
   nsresult testrv = aIdentity->GetIntAttribute("encryptionpolicy", &ep);
   if (NS_FAILED(testrv)) {
     *aEncrypt = false;
@@ -452,7 +408,8 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsIOutputStream * aSt
   case mime_crypto_none:
     /* This can happen if mime_crypto_hack_certs() decided to turn off
      encryption (by asking the user.) */
-    rv = 1;
+    // XXX 1 is not a valid nsresult
+    rv = static_cast<nsresult>(1);
     break;
   default:
     PR_ASSERT(0);
@@ -498,20 +455,21 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(bool aOuter, nsIMsgSendRepo
    */
   nsresult rv = NS_OK;
   char *header = 0;
-  PRUint32 L;
+  uint32_t L;
 
   rv = make_multipart_signed_header_string(aOuter, &header,
                     &mMultipartSignedBoundary);
-  if (NS_FAILED(rv)) goto FAIL;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   L = strlen(header);
 
   if (aOuter){
     /* If this is the outer block, write it to the file. */
-    PRUint32 n;
+    uint32_t n;
     rv = mStream->Write(header, L, &n);
     if (NS_FAILED(rv) || n < L) {
-      rv = MK_MIME_ERROR_WRITING_FILE;
+      // XXX This is -1, not an nsresult
+      rv = static_cast<nsresult>(MK_MIME_ERROR_WRITING_FILE);
     }
   } else {
     /* If this is an inner block, feed it through the crypto stream. */
@@ -519,7 +477,7 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(bool aOuter, nsIMsgSendRepo
   }
 
   PR_Free(header);
-  if (NS_FAILED(rv)) goto FAIL;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   /* Now initialize the crypto library, so that we can compute a hash
    on the object which we are signing.
@@ -529,15 +487,12 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(bool aOuter, nsIMsgSendRepo
 
   PR_SetError(0,0);
   mDataHash = do_CreateInstance("@mozilla.org/security/hash;1", &rv);
-  if (NS_FAILED(rv)) return 0;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = mDataHash->Init(mHashType);
-  if (NS_FAILED(rv)) {
-    goto FAIL;
-  }
+  NS_ENSURE_SUCCESS(rv, rv);
 
   PR_SetError(0,0);
- FAIL:
   return rv;
 }
 
@@ -545,8 +500,8 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
 {
   nsresult rv;
   nsCOMPtr<nsIStringBundleService> bundleSvc =
-                  do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+    mozilla::services::GetStringBundleService();
+  NS_ENSURE_TRUE(bundleSvc, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr<nsIStringBundle> sMIMEBundle;
   nsString mime_smime_enc_content_desc;
@@ -565,7 +520,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
 
   char *s =
   PR_smprintf("Content-Type: " APPLICATION_PKCS7_MIME
-          "; name=\"smime.p7m\"" CRLF
+          "; name=\"smime.p7m\"; smime-type=enveloped-data" CRLF
         "Content-Transfer-Encoding: " ENCODING_BASE64 CRLF
         "Content-Disposition: attachment"
           "; filename=\"smime.p7m\"" CRLF
@@ -573,10 +528,10 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
         CRLF,
         enc_content_desc_utf8.get());
 
-  PRUint32 L;
+  uint32_t L;
   if (!s) return NS_ERROR_OUT_OF_MEMORY;
   L = strlen(s);
-  PRUint32 n;
+  uint32_t n;
   rv = mStream->Write(s, L, &n);
   if (NS_FAILED(rv) || n < L) {
     return NS_ERROR_FAILURE;
@@ -589,7 +544,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
    */
 
   if (!mIsDraft) {
-    PRUint32 numCerts;
+    uint32_t numCerts;
     mCerts->GetLength(&numCerts);
     PR_ASSERT(numCerts > 0);
     if (numCerts == 0) return NS_ERROR_FAILURE;
@@ -653,10 +608,10 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   nsCOMPtr<nsICMSEncoder> encoder = do_CreateInstance(NS_CMSENCODER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  char * header = nsnull;
+  char * header = nullptr;
   nsCOMPtr<nsIStringBundleService> bundleSvc =
-                  do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+    mozilla::services::GetStringBundleService();
+  NS_ENSURE_TRUE(bundleSvc, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr<nsIStringBundle> sMIMEBundle;
   nsString mime_smime_sig_content_desc;
@@ -674,7 +629,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   /* Compute the hash...
    */
 
-  nsCAutoString hashString;
+  nsAutoCString hashString;
   mDataHash->Finish(false, hashString);
 
   mDataHash = 0;
@@ -684,7 +639,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
 
   /* Write out the headers for the signature.
    */
-  PRUint32 L;
+  uint32_t L;
   header =
     PR_smprintf(CRLF
           "--%s" CRLF
@@ -706,10 +661,11 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   L = strlen(header);
   if (aOuter) {
     /* If this is the outer block, write it to the file. */
-    PRUint32 n;
+    uint32_t n;
     rv = mStream->Write(header, L, &n);
     if (NS_FAILED(rv) || n < L) {
-      rv = MK_MIME_ERROR_WRITING_FILE;
+      // XXX This is -1, not an nsresult
+      rv = static_cast<nsresult>(MK_MIME_ERROR_WRITING_FILE);
     } 
   } else {
     /* If this is an inner block, feed it through the crypto stream. */
@@ -772,7 +728,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   /* Now write out the terminating boundary.
    */
   {
-  PRUint32 L;
+  uint32_t L;
   char *header = PR_smprintf(CRLF "--%s--" CRLF,
                  mMultipartSignedBoundary);
   PR_Free(mMultipartSignedBoundary);
@@ -785,10 +741,11 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   L = strlen(header);
   if (aOuter) {
     /* If this is the outer block, write it to the file. */
-    PRUint32 n;
+    uint32_t n;
     rv = mStream->Write(header, L, &n);
     if (NS_FAILED(rv) || n < L)
-      rv = MK_MIME_ERROR_WRITING_FILE;
+      // XXX This is -1, not an nsresult
+      rv = static_cast<nsresult>(MK_MIME_ERROR_WRITING_FILE);
   } else {
     /* If this is an inner block, feed it through the crypto stream. */
     rv = MimeCryptoWriteBlock (header, L);
@@ -849,7 +806,7 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (bool aSign, nsIMsgSendReport
   rv = MIME_EncoderDestroy(mCryptoEncoderData, false);
   mCryptoEncoderData = 0;
 
-  PRUint32 n;
+  uint32_t n;
   rv = mStream->Write(CRLF, 2, &n);
   if (NS_FAILED(rv) || n < 2)
     rv = NS_ERROR_FAILURE;
@@ -868,7 +825,7 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   char *mailbox_list = 0;
   nsCString all_mailboxes, mailboxes;
   const char *mailbox = 0;
-  PRUint32 count = 0;
+  uint32_t count = 0;
   nsCOMPtr<nsIX509CertDB> certdb = do_GetService(NS_X509CERTDB_CONTRACTID);
   nsresult res;
   nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &res);
@@ -884,13 +841,13 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   certdb->FindEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
 
   // must have both the signing and encryption certs to sign
-  if ((mSelfSigningCert == nsnull) && aSign) {
+  if ((mSelfSigningCert == nullptr) && aSign) {
     SetError(sendReport, NS_LITERAL_STRING("NoSenderSigningCert").get());
     res = NS_ERROR_FAILURE;
     goto FAIL;
   }
 
-  if ((mSelfEncryptionCert == nsnull) && aEncrypt) {
+  if ((mSelfEncryptionCert == nullptr) && aEncrypt) {
     SetError(sendReport, NS_LITERAL_STRING("NoSenderEncryptionCert").get());
     res = NS_ERROR_FAILURE;
     goto FAIL;
@@ -902,7 +859,8 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
   pHeader->ParseHeaderAddresses(mailboxes.get(), 0, &mailbox_list, &count);
 
-  if (count < 0) return count;
+  // XXX This is not a valid use of nsresult
+  if (count < 0) return static_cast<nsresult>(count);
 
   if (aEncrypt && mSelfEncryptionCert) {
     // Make sure self's configured cert is prepared for being used
@@ -924,11 +882,11 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
       nsCString mailbox_lowercase;
       ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
       nsCOMPtr<nsIX509Cert> cert;
-      certdb->FindCertByEmailAddress(nsnull, mailbox_lowercase.get(), getter_AddRefs(cert));
+      certdb->FindCertByEmailAddress(nullptr, mailbox_lowercase.get(), getter_AddRefs(cert));
       bool foundValidCert = false;
 
       if (cert) {
-        PRUint32 verification_result;
+        uint32_t verification_result;
 
         if (NS_SUCCEEDED(
             cert->VerifyForUsage(nsIX509Cert::CERT_USAGE_EmailRecipient, &verification_result))
@@ -977,7 +935,7 @@ FAIL:
   return res;
 }
 
-NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32 size)
+NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, int32_t size)
 {
   int status = 0;
   nsresult rv;
@@ -992,16 +950,19 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32
   */
   if (size >= 5 && buf[0] == 'F' && !strncmp(buf, "From ", 5)) {
     char mangle[] = ">";
-    status = MimeCryptoWriteBlock (mangle, 1);
-    if (status < 0)
-    return status;
+    nsresult res = MimeCryptoWriteBlock (mangle, 1);
+    if (NS_FAILED(res))
+      return res;
+    // This value will actually be cast back to an nsresult before use, so this
+    // cast is reasonable under the circumstances.
+    status = static_cast<int>(res);
   }
 
   /* If we're signing, or signing-and-encrypting, feed this data into
    the computation of the hash. */
   if (mDataHash) {
     PR_SetError(0,0);
-    mDataHash->Update((const PRUint8*) buf, size);
+    mDataHash->Update((const uint8_t*) buf, size);
 	  status = PR_GetError();
 	  if (status < 0) goto FAIL;
 	}
@@ -1013,11 +974,11 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32
 
     /* We want to create equally sized encryption strings */
     const char *inputBytesIterator = buf;
-    PRUint32 inputBytesLeft = size;
+    uint32_t inputBytesLeft = size;
 
     while (inputBytesLeft) {
-      const PRUint32 spaceLeftInBuffer = eBufferSize - mBufferedBytes;
-      const PRUint32 bytesToAppend = NS_MIN(inputBytesLeft, spaceLeftInBuffer);
+      const uint32_t spaceLeftInBuffer = eBufferSize - mBufferedBytes;
+      const uint32_t bytesToAppend = NS_MIN(inputBytesLeft, spaceLeftInBuffer);
 
       memcpy(mBuffer+mBufferedBytes, inputBytesIterator, bytesToAppend);
       mBufferedBytes += bytesToAppend;
@@ -1040,14 +1001,17 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32
 	  /* If we're not encrypting (presumably just signing) then write this
 		 data directly to the file. */
 
-    PRUint32 n;
+    uint32_t n;
     rv = mStream->Write(buf, size, &n);
     if (NS_FAILED(rv) || n < size) {
-      return MK_MIME_ERROR_WRITING_FILE;
+      // XXX MK_MIME_ERROR_WRITING_FILE is -1, which is not a valid nsresult
+      return static_cast<nsresult>(MK_MIME_ERROR_WRITING_FILE);
     }
   }
  FAIL:
-  return status;
+  // XXX status sometimes has invalid nsresults like -1 or PR_GetError()
+  // assigned to it
+  return static_cast<nsresult>(status);
 }
 
 /* Returns a string consisting of a Content-Type header, and a boundary
@@ -1055,35 +1019,17 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32
    of a multipart object.  The boundary itself is also returned (so that
    the caller knows what to write to close it off.)
  */
-static int
+static nsresult
 make_multipart_signed_header_string(bool outer_p,
 									char **header_return,
 									char **boundary_return)
 {
   *header_return = 0;
   *boundary_return = mime_make_separator("ms");
-  nsresult rv;
-
-  nsCOMPtr<nsIStringBundleService> bundleSvc =
-                  do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIStringBundle> sMIMEBundle;
-  nsString crypto_multipart_blurb;
-
-  bundleSvc->CreateBundle(SMIME_STRBUNDLE_URL, getter_AddRefs(sMIMEBundle));
-  if (!sMIMEBundle)
-    return NS_ERROR_FAILURE;
 
   if (!*boundary_return)
 	return NS_ERROR_OUT_OF_MEMORY;
 
-  if (outer_p) {
-	  sMIMEBundle->GetStringFromName(NS_LITERAL_STRING("mime_multipartSignedBlurb").get(),
-                                    getter_Copies(crypto_multipart_blurb));
-  }
-  NS_ConvertUTF16toUTF8 multipart_blurb_utf8(crypto_multipart_blurb);
-
   *header_return = PR_smprintf(
         "Content-Type: " MULTIPART_SIGNED "; "
         "protocol=\"" APPLICATION_PKCS7_SIGNATURE "\"; "
@@ -1094,8 +1040,8 @@ make_multipart_signed_header_string(bool outer_p,
 				"--%s" CRLF,
 
 				*boundary_return,
-				(crypto_multipart_blurb.IsEmpty() ? "" : multipart_blurb_utf8.get()),
-				(crypto_multipart_blurb.IsEmpty() ? "" : CRLF CRLF),
+				(outer_p ? crypto_multipart_blurb : ""),
+				(outer_p ? CRLF CRLF : ""),
 				*boundary_return);
 
   if (!*header_return) {
@@ -1104,7 +1050,7 @@ make_multipart_signed_header_string(bool outer_p,
 	  return NS_ERROR_OUT_OF_MEMORY;
 	}
 
-  return 0;
+  return NS_OK;
 }
 
 /* Used as the output function of a SEC_PKCS7EncoderContext -- we feed
@@ -1117,12 +1063,11 @@ make_multipart_signed_header_string(bool outer_p,
    difference is which MimeEncoderData object is used.
  */
 static void
-mime_crypto_write_base64 (void *closure, const char *buf,
-						  unsigned long size)
+mime_crypto_write_base64 (void *closure, const char *buf, unsigned long size)
 {
   MimeEncoderData *data = (MimeEncoderData *) closure;
-  int status = MIME_EncoderWrite (data, buf, size);
-  PR_SetError(status < 0 ? status : 0, 0);
+  nsresult rv = MIME_EncoderWrite (data, buf, size);
+  PR_SetError(NS_FAILED(rv) ? static_cast<uint32_t>(rv) : 0, 0);
 }
 
 
@@ -1130,12 +1075,12 @@ mime_crypto_write_base64 (void *closure, const char *buf,
    the signature for a multipart/signed object, this is used to write the
    base64-encoded representation of the signature to the file.
  */
-nsresult mime_encoder_output_fn(const char *buf, PRInt32 size, void *closure)
+int mime_encoder_output_fn(const char *buf, int32_t size, void *closure)
 {
   nsMsgComposeSecure *state = (nsMsgComposeSecure *) closure;
   nsCOMPtr<nsIOutputStream> stream;
   state->GetOutputStream(getter_AddRefs(stream));
-  PRUint32 n;
+  uint32_t n;
   nsresult rv = stream->Write((char *) buf, size, &n);
   if (NS_FAILED(rv) || n < size)
     return MK_MIME_ERROR_WRITING_FILE;
@@ -1148,9 +1093,10 @@ nsresult mime_encoder_output_fn(const char *buf, PRInt32 size, void *closure)
    signature should be fed into the crypto engine, rather than being written
    directly to the file.
  */
-static nsresult
-mime_nested_encoder_output_fn (const char *buf, PRInt32 size, void *closure)
+static int
+mime_nested_encoder_output_fn (const char *buf, int32_t size, void *closure)
 {
   nsMsgComposeSecure *state = (nsMsgComposeSecure *) closure;
-  return state->MimeCryptoWriteBlock ((char *) buf, size);
+  // XXX MimeCryptoWriteBlock doesn't really return an nsresult, cast to int
+  return static_cast<int>(state->MimeCryptoWriteBlock ((char *) buf, size));
 }
