@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Lorenzo Colitti <lorenzo@colitti.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #ifndef nsImapMailFolder_h__
 #define nsImapMailFolder_h__
 
@@ -59,7 +26,6 @@
 #include "nsIImapMailFolderSink.h"
 #include "nsIImapServerSink.h"
 #include "nsIMsgFilterPlugin.h"
-#include "nsIEventTarget.h"
 #include "nsIThread.h"
 #include "nsDataHashtable.h"
 #include "nsIMutableArray.h"
@@ -98,22 +64,25 @@ public:
 
     nsCOMPtr<nsIMsgMessageService> m_msgService; // source folder message service; can
                                         // be Nntp, Mailbox, or Imap
-    PRBool m_isMove;             // is a move
-    PRBool m_selectedState;      // needs to be in selected state; append msg
-    PRBool m_isCrossServerOp; // are we copying between imap servers?
-    PRUint32 m_curIndex; // message index to the message array which we are
+    bool m_isMove;             // is a move
+    bool m_selectedState;      // needs to be in selected state; append msg
+    bool m_isCrossServerOp; // are we copying between imap servers?
+    uint32_t m_curIndex; // message index to the message array which we are
                          // copying
-    PRUint32 m_totalCount;// total count of messages we have to do
-    PRUint32 m_unreadCount; // num unread messages we're moving
-    PRBool m_streamCopy;
+    uint32_t m_totalCount;// total count of messages we have to do
+    uint32_t m_unreadCount; // num unread messages we're moving
+    bool m_streamCopy;
     char *m_dataBuffer; // temporary buffer for this copy operation
     nsCOMPtr<nsIOutputStream> m_msgFileStream;         // temporary file (processed mail)
-    PRUint32 m_dataBufferSize;
-    PRUint32 m_leftOver;
-    PRBool m_allowUndo;
-    PRBool m_eatLF;
-    PRBool m_newMsgFlags; // only used if there's no m_message
+    uint32_t m_dataBufferSize;
+    uint32_t m_leftOver;
+    bool m_allowUndo;
+    bool m_eatLF;
+    uint32_t m_newMsgFlags; // only used if there's no m_message
     nsCString m_newMsgKeywords; // ditto 
+    // If the server supports UIDPLUS, this is the UID for the append,
+    // if we're doing an append.
+    nsMsgKey m_appendUID;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsImapMailCopyState, NS_IMAPMAILCOPYSTATE_IID)
@@ -144,7 +113,7 @@ public:
   nsMsgIMAPFolderACL(nsImapMailFolder *folder);
   ~nsMsgIMAPFolderACL();
 
-  PRBool SetFolderRightsForUser(const nsACString& userName, const nsACString& rights);
+  bool SetFolderRightsForUser(const nsACString& userName, const nsACString& rights);
 
 public:
 
@@ -152,32 +121,32 @@ public:
   // DO NOT use these for looking up information about the currently authenticated user.
   // (There are some different checks and defaults we do).
   // Instead, use the functions below, GetICan....()
-  PRBool GetCanUserLookupFolder(const nsACString& userName);      // Is folder visible to LIST/LSUB?
-  PRBool GetCanUserReadFolder(const nsACString& userName);        // SELECT, CHECK, FETCH, PARTIAL, SEARCH, COPY from folder?
-  PRBool GetCanUserStoreSeenInFolder(const nsACString& userName); // STORE SEEN flag?
-  PRBool GetCanUserWriteFolder(const nsACString& userName);       // STORE flags other than SEEN and DELETED?
-  PRBool GetCanUserInsertInFolder(const nsACString& userName);    // APPEND, COPY into folder?
-  PRBool GetCanUserPostToFolder(const nsACString& userName);      // Can I send mail to the submission address for folder?
-  PRBool GetCanUserCreateSubfolder(const nsACString& userName);   // Can I CREATE a subfolder of this folder?
-  PRBool GetCanUserDeleteInFolder(const nsACString& userName);    // STORE DELETED flag, perform EXPUNGE?
-  PRBool GetCanUserAdministerFolder(const nsACString& userName);  // perform SETACL?
+  bool GetCanUserLookupFolder(const nsACString& userName);      // Is folder visible to LIST/LSUB?
+  bool GetCanUserReadFolder(const nsACString& userName);        // SELECT, CHECK, FETCH, PARTIAL, SEARCH, COPY from folder?
+  bool GetCanUserStoreSeenInFolder(const nsACString& userName); // STORE SEEN flag?
+  bool GetCanUserWriteFolder(const nsACString& userName);       // STORE flags other than SEEN and DELETED?
+  bool GetCanUserInsertInFolder(const nsACString& userName);    // APPEND, COPY into folder?
+  bool GetCanUserPostToFolder(const nsACString& userName);      // Can I send mail to the submission address for folder?
+  bool GetCanUserCreateSubfolder(const nsACString& userName);   // Can I CREATE a subfolder of this folder?
+  bool GetCanUserDeleteInFolder(const nsACString& userName);    // STORE DELETED flag, perform EXPUNGE?
+  bool GetCanUserAdministerFolder(const nsACString& userName);  // perform SETACL?
 
   // Functions to find out rights for the currently authenticated user.
 
-  PRBool GetCanILookupFolder();      // Is folder visible to LIST/LSUB?
-  PRBool GetCanIReadFolder();        // SELECT, CHECK, FETCH, PARTIAL, SEARCH, COPY from folder?
-  PRBool GetCanIStoreSeenInFolder(); // STORE SEEN flag?
-  PRBool GetCanIWriteFolder();       // STORE flags other than SEEN and DELETED?
-  PRBool GetCanIInsertInFolder();    // APPEND, COPY into folder?
-  PRBool GetCanIPostToFolder();      // Can I send mail to the submission address for folder?
-  PRBool GetCanICreateSubfolder();   // Can I CREATE a subfolder of this folder?
-  PRBool GetCanIDeleteInFolder();    // STORE DELETED flag?
-  PRBool GetCanIAdministerFolder();  // perform SETACL?
-  PRBool GetCanIExpungeFolder();     // perform EXPUNGE?
+  bool GetCanILookupFolder();      // Is folder visible to LIST/LSUB?
+  bool GetCanIReadFolder();        // SELECT, CHECK, FETCH, PARTIAL, SEARCH, COPY from folder?
+  bool GetCanIStoreSeenInFolder(); // STORE SEEN flag?
+  bool GetCanIWriteFolder();       // STORE flags other than SEEN and DELETED?
+  bool GetCanIInsertInFolder();    // APPEND, COPY into folder?
+  bool GetCanIPostToFolder();      // Can I send mail to the submission address for folder?
+  bool GetCanICreateSubfolder();   // Can I CREATE a subfolder of this folder?
+  bool GetCanIDeleteInFolder();    // STORE DELETED flag?
+  bool GetCanIAdministerFolder();  // perform SETACL?
+  bool GetCanIExpungeFolder();     // perform EXPUNGE?
 
-  PRBool GetDoIHaveFullRightsForFolder(); // Returns TRUE if I have full rights on this folder (all of the above return TRUE)
+  bool GetDoIHaveFullRightsForFolder(); // Returns TRUE if I have full rights on this folder (all of the above return TRUE)
 
-  PRBool GetIsFolderShared(); // We use this to see if the ACLs think a folder is shared or not.
+  bool GetIsFolderShared(); // We use this to see if the ACLs think a folder is shared or not.
   // We will define "Shared" in 5.0 to mean:
   // At least one user other than the currently authenticated user has at least one
   // explicitly-listed ACL right on that folder.
@@ -190,14 +159,14 @@ public:
   nsresult GetOtherUsers(nsIUTF8StringEnumerator** aResult);
 
 protected:
-  PRBool GetFlagSetInRightsForUser(const nsACString& userName, char flag, PRBool defaultIfNotFound);
+  bool GetFlagSetInRightsForUser(const nsACString& userName, char flag, bool defaultIfNotFound);
   void BuildInitialACLFromCache();
   void UpdateACLCache();
 
 protected:
   nsDataHashtable <nsCStringHashKey, nsCString> m_rightsHash; // Hash table, mapping username strings to rights strings.
   nsImapMailFolder *m_folder;
-  PRInt32        m_aclCount;
+  int32_t        m_aclCount;
 
 };
 
@@ -222,7 +191,7 @@ class nsImapMailFolder :  public nsMsgDBFolder,
                           public nsICopyMessageListener,
                           public nsIMsgFilterHitNotify
 {
- static const PRUint32 PLAYBACK_TIMER_INTERVAL_IN_MS = 500; 
+ static const uint32_t PLAYBACK_TIMER_INTERVAL_IN_MS = 500; 
 public:
   nsImapMailFolder();
   virtual ~nsImapMailFolder();
@@ -241,36 +210,36 @@ public:
 
   NS_IMETHOD Compact(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow);
   NS_IMETHOD CompactAll(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow,
-                        PRBool aCompactOfflineAlso);
+                        bool aCompactOfflineAlso);
   NS_IMETHOD EmptyTrash(nsIMsgWindow *msgWindow, nsIUrlListener *aListener);
   NS_IMETHOD CopyDataToOutputStreamForAppend(nsIInputStream *aIStream,
-                     PRInt32 aLength, nsIOutputStream *outputStream);
+                     int32_t aLength, nsIOutputStream *outputStream);
   NS_IMETHOD CopyDataDone();
   NS_IMETHOD Delete ();
   NS_IMETHOD Rename (const nsAString& newName, nsIMsgWindow *msgWindow);
   NS_IMETHOD RenameSubFolders(nsIMsgWindow *msgWindow, nsIMsgFolder *oldFolder);
-  NS_IMETHOD GetNoSelect(PRBool *aResult);
+  NS_IMETHOD GetNoSelect(bool *aResult);
 
   NS_IMETHOD GetPrettyName(nsAString& prettyName); // Override of the base, for top-level mail folder
 
   NS_IMETHOD GetFolderURL(nsACString& url);
 
-  NS_IMETHOD UpdateSummaryTotals(PRBool force) ;
+  NS_IMETHOD UpdateSummaryTotals(bool force) ;
 
-  NS_IMETHOD GetDeletable (PRBool *deletable);
-  NS_IMETHOD GetRequiresCleanup(PRBool *requiresCleanup);
+  NS_IMETHOD GetDeletable (bool *deletable);
+  NS_IMETHOD GetRequiresCleanup(bool *requiresCleanup);
 
-  NS_IMETHOD GetSizeOnDisk(PRUint32 * size);
+  NS_IMETHOD GetSizeOnDisk(uint32_t * size);
 
-  NS_IMETHOD GetCanCreateSubfolders(PRBool *aResult);
-  NS_IMETHOD GetCanSubscribe(PRBool *aResult);
+  NS_IMETHOD GetCanCreateSubfolders(bool *aResult);
+  NS_IMETHOD GetCanSubscribe(bool *aResult);
 
   NS_IMETHOD ApplyRetentionSettings();
 
   NS_IMETHOD AddMessageDispositionState(nsIMsgDBHdr *aMessage, nsMsgDispositionState aDispositionFlag);
-  NS_IMETHOD MarkMessagesRead(nsIArray *messages, PRBool markRead);
+  NS_IMETHOD MarkMessagesRead(nsIArray *messages, bool markRead);
   NS_IMETHOD MarkAllMessagesRead(nsIMsgWindow *aMsgWindow);
-  NS_IMETHOD MarkMessagesFlagged(nsIArray *messages, PRBool markFlagged);
+  NS_IMETHOD MarkMessagesFlagged(nsIArray *messages, bool markFlagged);
   NS_IMETHOD MarkThreadRead(nsIMsgThread *thread);
   NS_IMETHOD SetLabelForMessages(nsIArray *aMessages, nsMsgLabelValue aLabel);
   NS_IMETHOD SetJunkScoreForMessages(nsIArray *aMessages, const nsACString& aJunkScore);
@@ -281,44 +250,49 @@ public:
   NS_IMETHOD GetDBFolderInfoAndDB(nsIDBFolderInfo **folderInfo,
                                   nsIMsgDatabase **db);
   NS_IMETHOD DeleteMessages(nsIArray *messages,
-                            nsIMsgWindow *msgWindow, PRBool
-                            deleteStorage, PRBool isMove,
-                            nsIMsgCopyServiceListener* listener, PRBool allowUndo);
+                            nsIMsgWindow *msgWindow, bool
+                            deleteStorage, bool isMove,
+                            nsIMsgCopyServiceListener* listener, bool allowUndo);
   NS_IMETHOD CopyMessages(nsIMsgFolder *srcFolder,
                           nsIArray* messages,
-                          PRBool isMove, nsIMsgWindow *msgWindow,
-                          nsIMsgCopyServiceListener* listener, PRBool isFolder,
-                          PRBool allowUndo);
-  NS_IMETHOD CopyFolder(nsIMsgFolder *srcFolder, PRBool isMove, nsIMsgWindow *msgWindow,
+                          bool isMove, nsIMsgWindow *msgWindow,
+                          nsIMsgCopyServiceListener* listener, bool isFolder,
+                          bool allowUndo);
+  NS_IMETHOD CopyFolder(nsIMsgFolder *srcFolder, bool isMove, nsIMsgWindow *msgWindow,
                         nsIMsgCopyServiceListener* listener);
   NS_IMETHOD CopyFileMessage(nsIFile* file,
                               nsIMsgDBHdr* msgToReplace,
-                              PRBool isDraftOrTemplate,
-                              PRUint32 aNewMsgFlags,
+                              bool isDraftOrTemplate,
+                              uint32_t aNewMsgFlags,
                               const nsACString &aNewMsgKeywords,
                               nsIMsgWindow *msgWindow,
                               nsIMsgCopyServiceListener* listener);
   NS_IMETHOD GetNewMessages(nsIMsgWindow *aWindow, nsIUrlListener *aListener);
 
-  NS_IMETHOD GetFilePath(nsILocalFile** aPathName);
-  NS_IMETHOD SetFilePath(nsILocalFile * aPath);
+  NS_IMETHOD GetFilePath(nsIFile** aPathName);
+  NS_IMETHOD SetFilePath(nsIFile * aPath);
 
-  NS_IMETHOD Shutdown(PRBool shutdownChildren);
+  NS_IMETHOD Shutdown(bool shutdownChildren);
 
   NS_IMETHOD DownloadMessagesForOffline(nsIArray *messages, nsIMsgWindow *msgWindow);
 
   NS_IMETHOD DownloadAllForOffline(nsIUrlListener *listener, nsIMsgWindow *msgWindow);
-  NS_IMETHOD GetCanFileMessages(PRBool *aCanFileMessages);
-  NS_IMETHOD GetCanDeleteMessages(PRBool *aCanDeleteMessages);
-  NS_IMETHOD FetchMsgPreviewText(nsMsgKey *aKeysToFetch, PRUint32 aNumKeys,
-                                                 PRBool aLocalOnly, nsIUrlListener *aUrlListener,
-                                                 PRBool *aAsyncResults);
+  NS_IMETHOD GetCanFileMessages(bool *aCanFileMessages);
+  NS_IMETHOD GetCanDeleteMessages(bool *aCanDeleteMessages);
+  NS_IMETHOD FetchMsgPreviewText(nsMsgKey *aKeysToFetch, uint32_t aNumKeys,
+                                                 bool aLocalOnly, nsIUrlListener *aUrlListener,
+                                                 bool *aAsyncResults);
 
   NS_IMETHOD AddKeywordsToMessages(nsIArray *aMessages, const nsACString& aKeywords);
   NS_IMETHOD RemoveKeywordsFromMessages(nsIArray *aMessages, const nsACString& aKeywords);
 
   NS_IMETHOD NotifyCompactCompleted();
 
+  // overrides nsMsgDBFolder::HasMsgOffline()
+  NS_IMETHOD HasMsgOffline(nsMsgKey msgKey, bool *_retval);
+  // overrides nsMsgDBFolder::GetOfflineFileStream()
+  NS_IMETHOD GetOfflineFileStream(nsMsgKey msgKey, int64_t *offset, uint32_t *size, nsIInputStream **aFileStream);
+
   NS_DECL_NSIMSGIMAPMAILFOLDER
   NS_DECL_NSIIMAPMAILFOLDERSINK
   NS_DECL_NSIIMAPMESSAGESINK
@@ -332,11 +306,21 @@ public:
   NS_DECL_NSIMSGFILTERHITNOTIFY
   NS_DECL_NSIJUNKMAILCLASSIFICATIONLISTENER
 
-  NS_IMETHOD IsCommandEnabled(const nsACString& command, PRBool *result);
+  NS_IMETHOD IsCommandEnabled(const nsACString& command, bool *result);
   NS_IMETHOD SetFilterList(nsIMsgFilterList *aMsgFilterList);
   NS_IMETHOD GetCustomIdentity(nsIMsgIdentity **aIdentity);
 
-  nsresult AddSubfolderWithPath(nsAString& name, nsILocalFile *dbPath, nsIMsgFolder **child, PRBool brandNew = PR_FALSE);
+ /**
+  * This method is used to locate a folder where a msg could be present, not just
+  * the folder where the message first arrives, this method searches for the existence
+  * of msg in all the folders/labels that we retrieve from X-GM-LABELS also.
+  * overrides nsMsgDBFolder::GetOfflineMsgFolder()
+  *  @param msgKey key  of the msg for which we are trying to get the folder;
+  *  @param aMsgFolder  required folder;
+  */
+  NS_IMETHOD GetOfflineMsgFolder(nsMsgKey msgKey, nsIMsgFolder **aMsgFolder);
+
+  nsresult AddSubfolderWithPath(nsAString& name, nsIFile *dbPath, nsIMsgFolder **child, bool brandNew = false);
   nsresult MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
                                   nsIMsgDatabase *sourceDB,
                                   const nsACString& destFolder,
@@ -346,60 +330,59 @@ public:
   // send notification to copy service listener.
   nsresult OnCopyCompleted(nsISupports *srcSupport, nsresult exitCode);
 
-  static nsresult  AllocateUidStringFromKeys(nsMsgKey *keys, PRUint32 numKeys, nsCString &msgIds);
+  static nsresult  AllocateUidStringFromKeys(nsMsgKey *keys, uint32_t numKeys, nsCString &msgIds);
   static nsresult  BuildIdsAndKeyArray(nsIArray* messages, nsCString& msgIds, nsTArray<nsMsgKey>& keyArray);
 
   // these might end up as an nsIImapMailFolder attribute.
-  nsresult SetSupportedUserFlags(PRUint32 userFlags);
-  nsresult GetSupportedUserFlags(PRUint32 *userFlags);
+  nsresult SetSupportedUserFlags(uint32_t userFlags);
+  nsresult GetSupportedUserFlags(uint32_t *userFlags);
 
   // Find the start of a range of msgKeys that can hold srcCount headers.
-  nsresult FindOpenRange(nsMsgKey &fakeBase, PRUint32 srcCount);
+  nsresult FindOpenRange(nsMsgKey &fakeBase, uint32_t srcCount);
 
 protected:
   // Helper methods
 
   virtual nsresult CreateChildFromURI(const nsCString &uri, nsIMsgFolder **folder);
   void FindKeysToAdd(const nsTArray<nsMsgKey> &existingKeys, nsTArray<nsMsgKey>
-    &keysToFetch, PRUint32 &numNewUnread, nsIImapFlagAndUidState *flagState);
+    &keysToFetch, uint32_t &numNewUnread, nsIImapFlagAndUidState *flagState);
   void FindKeysToDelete(const nsTArray<nsMsgKey> &existingKeys, nsTArray<nsMsgKey>
-    &keysToFetch, nsIImapFlagAndUidState *flagState, PRUint32 boxFlags);
-  void PrepareToAddHeadersToMailDB(nsIImapProtocol* aProtocol, const
-    nsTArray<nsMsgKey> &keysToFetch,
-    nsIMailboxSpec *boxSpec);
+    &keysToFetch, nsIImapFlagAndUidState *flagState, uint32_t boxFlags);
+  void PrepareToAddHeadersToMailDB(nsIImapProtocol* aProtocol);
   void TweakHeaderFlags(nsIImapProtocol* aProtocol, nsIMsgDBHdr *tweakMe);
 
   nsresult SyncFlags(nsIImapFlagAndUidState *flagState);
   nsresult HandleCustomFlags(nsMsgKey uidOfMessage, nsIMsgDBHdr *dbHdr,
-                             PRUint16 userFlags, nsCString& keywords);
-  nsresult NotifyMessageFlagsFromHdr(nsIMsgDBHdr *dbHdr, nsMsgKey msgKey, PRUint32 flags);
+                             uint16_t userFlags, nsCString& keywords);
+  nsresult NotifyMessageFlagsFromHdr(nsIMsgDBHdr *dbHdr, nsMsgKey msgKey,
+                                     uint32_t flags);
 
-  nsresult SetupHeaderParseStream(PRUint32 size, const nsACString& content_type, nsIMailboxSpec *boxSpec);
-  nsresult  ParseAdoptedHeaderLine(const char *messageLine, PRUint32 msgKey);
+  nsresult SetupHeaderParseStream(uint32_t size, const nsACString& content_type, nsIMailboxSpec *boxSpec);
+  nsresult  ParseAdoptedHeaderLine(const char *messageLine, uint32_t msgKey);
   nsresult  NormalEndHeaderParseStream(nsIImapProtocol *aProtocol, nsIImapUrl *imapUrl);
 
   void EndOfflineDownload();
-  nsresult CopyFileToOfflineStore(nsILocalFile *srcFile);
+  nsresult CopyFileToOfflineStore(nsIFile *srcFile, nsMsgKey msgKey);
 
-  nsresult MarkMessagesImapDeleted(nsTArray<nsMsgKey> *keyArray, PRBool deleted, nsIMsgDatabase *db);
+  nsresult MarkMessagesImapDeleted(nsTArray<nsMsgKey> *keyArray, bool deleted, nsIMsgDatabase *db);
 
   // Notifies imap autosync that it should update this folder when it
   // gets a chance.
   void NotifyHasPendingMsgs();
   void UpdatePendingCounts();
-  void SetIMAPDeletedFlag(nsIMsgDatabase *mailDB, const nsTArray<nsMsgKey> &msgids, PRBool markDeleted);
-  virtual PRBool ShowDeletedMessages();
-  virtual PRBool DeleteIsMoveToTrash();
+  void SetIMAPDeletedFlag(nsIMsgDatabase *mailDB, const nsTArray<nsMsgKey> &msgids, bool markDeleted);
+  virtual bool ShowDeletedMessages();
+  virtual bool DeleteIsMoveToTrash();
   nsresult GetFolder(const nsACString& name, nsIMsgFolder **pFolder);
   nsresult GetTrashFolder(nsIMsgFolder **pTrashFolder);
-  PRBool TrashOrDescendentOfTrash(nsIMsgFolder* folder);
-  static PRBool ShouldCheckAllFolders(nsIImapIncomingServer *imapServer);
+  bool TrashOrDescendentOfTrash(nsIMsgFolder* folder);
+  static bool ShouldCheckAllFolders(nsIImapIncomingServer *imapServer);
   nsresult GetServerKey(nsACString& serverKey);
   nsresult DisplayStatusMsg(nsIImapUrl *aImapUrl, const nsAString& msg);
 
   //nsresult RenameLocal(const char *newName);
-  nsresult AddDirectorySeparator(nsILocalFile *path);
-  nsresult CreateSubFolders(nsILocalFile *path);
+  nsresult AddDirectorySeparator(nsIFile *path);
+  nsresult CreateSubFolders(nsIFile *path);
   nsresult GetDatabase();
   virtual void GetIncomingServerType(nsCString& serverType) { serverType.AssignLiteral("imap");}
 
@@ -413,22 +396,22 @@ protected:
   // Uber message copy service
   nsresult CopyMessagesWithStream(nsIMsgFolder* srcFolder,
                                     nsIArray* messages,
-                                    PRBool isMove,
-                                    PRBool isCrossServerOp,
+                                    bool isMove,
+                                    bool isCrossServerOp,
                                     nsIMsgWindow *msgWindow,
-                                    nsIMsgCopyServiceListener* listener, PRBool allowUndo);
+                                    nsIMsgCopyServiceListener* listener, bool allowUndo);
   nsresult CopyStreamMessage(nsIMsgDBHdr* message, nsIMsgFolder* dstFolder,
-                            nsIMsgWindow *msgWindow, PRBool isMove);
+                            nsIMsgWindow *msgWindow, bool isMove);
   nsresult InitCopyState(nsISupports* srcSupport,
                           nsIArray* messages,
-                          PRBool isMove,
-                          PRBool selectedState,
-                          PRBool acrossServers,
-                          PRUint32 newMsgFlags,
+                          bool isMove,
+                          bool selectedState,
+                          bool acrossServers,
+                          uint32_t newMsgFlags,
                           const nsACString &newMsgKeywords,
                           nsIMsgCopyServiceListener* listener,
                           nsIMsgWindow *msgWindow,
-                          PRBool allowUndo);
+                          bool allowUndo);
   nsresult GetMoveCoalescer();
   nsresult PlaybackCoalescedOperations();
   virtual nsresult CreateBaseMessageURI(const nsACString& aURI);
@@ -437,34 +420,33 @@ protected:
   nsresult GetOriginalOp(nsIMsgOfflineImapOperation *op, nsIMsgOfflineImapOperation **originalOp, nsIMsgDatabase **originalDB);
   nsresult CopyMessagesOffline(nsIMsgFolder* srcFolder,
                                 nsIArray* messages,
-                                PRBool isMove,
+                                bool isMove,
                                 nsIMsgWindow *msgWindow,
                                 nsIMsgCopyServiceListener* listener);
-  void SetPendingAttributes(nsIArray* messages, PRBool aIsMove);
+  void SetPendingAttributes(nsIArray* messages, bool aIsMove);
 
   nsresult CopyOfflineMsgBody(nsIMsgFolder *srcFolder, nsIMsgDBHdr *destHdr,
                               nsIMsgDBHdr *origHdr, nsIInputStream *inputStream,
                               nsIOutputStream *outputStream);
 
   void GetTrashFolderName(nsAString &aFolderName);
-  PRBool ShowPreviewText();
+  bool ShowPreviewText();
 
   // Pseudo-Offline operation playback timer
-  static 
-  void PlaybackTimerCallback(nsITimer *aTimer, void *aClosure);
-  
+  static void PlaybackTimerCallback(nsITimer *aTimer, void *aClosure);
+
   nsresult CreatePlaybackTimer();
-  
-  // Allocate and initialize associated auto-sync state object 
+
+  // Allocate and initialize associated auto-sync state object.
   void InitAutoSyncState();
 
-  PRBool m_initialized;
-  PRBool m_haveDiscoveredAllFolders;
+  bool m_initialized;
+  bool m_haveDiscoveredAllFolders;
   nsCOMPtr<nsIMsgParseMailMsgState> m_msgParser;
   nsCOMPtr<nsIMsgFilterList> m_filterList;
   nsCOMPtr<nsIMsgFilterPlugin> m_filterPlugin;  // XXX should be a list
   // used with filter plugins to know when we've finished classifying and can playback moves
-  PRBool m_msgMovedByFilter;
+  bool m_msgMovedByFilter;
   nsImapMoveCoalescer *m_moveCoalescer; // strictly owned by the nsImapMailFolder
   nsCOMPtr<nsIMutableArray> m_junkMessagesToMarkAsRead;
   /// list of keys to be moved to the junk folder
@@ -472,63 +454,64 @@ protected:
   /// the junk destination folder
   nsCOMPtr<nsIMsgFolder> mSpamFolder;
   nsMsgKey m_curMsgUid;
-  PRUint32 m_uidValidity;
+  uint32_t m_uidValidity;
 
   // These three vars are used to store counts from STATUS or SELECT command
   // They include deleted messages, so they can differ from the generic
   // folder total and unread counts.
-  PRInt32 m_numServerRecentMessages; 
-  PRInt32 m_numServerUnseenMessages;
-  PRInt32 m_numServerTotalMessages;
+  int32_t m_numServerRecentMessages;
+  int32_t m_numServerUnseenMessages;
+  int32_t m_numServerTotalMessages;
   // if server supports UIDNEXT, we store it here.
-  PRInt32 m_nextUID;
+  int32_t m_nextUID;
 
-  PRInt32  m_nextMessageByteLength;
-  nsCOMPtr<nsIThread> m_thread;
+  int32_t  m_nextMessageByteLength;
   nsCOMPtr<nsIUrlListener> m_urlListener;
-  PRBool m_urlRunning;
+  bool m_urlRunning;
 
-  // *** jt - undo move/copy trasaction support
+  // undo move/copy transaction support
   nsRefPtr<nsMsgTxn> m_pendingUndoTxn;
-  nsCOMPtr<nsImapMailCopyState> m_copyState;
+  nsRefPtr<nsImapMailCopyState> m_copyState;
   char m_hierarchyDelimiter;
-  PRInt32 m_boxFlags;
+  int32_t m_boxFlags;
   nsCString m_onlineFolderName;
   nsCString m_ownerUserName;  // username of the "other user," as in
   // "Other Users' Mailboxes"
 
   nsCString m_adminUrl;   // url to run to set admin privileges for this folder
   nsIMAPNamespace  *m_namespace;
-  PRPackedBool m_verifiedAsOnlineFolder;
-  PRPackedBool m_explicitlyVerify; // whether or not we need to explicitly verify this through LIST
-  PRPackedBool m_folderIsNamespace;
-  PRPackedBool m_folderNeedsSubscribing;
-  PRPackedBool m_folderNeedsAdded;
-  PRPackedBool m_folderNeedsACLListed;
-  PRPackedBool m_performingBiff;
-  PRPackedBool m_folderQuotaCommandIssued;
-  PRPackedBool m_folderQuotaDataIsValid;
-  PRPackedBool m_updatingFolder;
+  bool m_verifiedAsOnlineFolder;
+  bool m_explicitlyVerify; // whether or not we need to explicitly verify this through LIST
+  bool m_folderIsNamespace;
+  bool m_folderNeedsSubscribing;
+  bool m_folderNeedsAdded;
+  bool m_folderNeedsACLListed;
+  bool m_performingBiff;
+  bool m_folderQuotaCommandIssued;
+  bool m_folderQuotaDataIsValid;
+  bool m_updatingFolder;
   // These two vars are used to keep track of compaction state so we can know
   // when to send a done notification.
-  PRPackedBool m_compactingOfflineStore;
-  PRPackedBool m_expunging;
-  PRPackedBool m_applyIncomingFilters; // apply filters to this folder, even if not the inbox
+  bool m_compactingOfflineStore;
+  bool m_expunging;
+  bool m_applyIncomingFilters; // apply filters to this folder, even if not the inbox
   nsMsgIMAPFolderACL *m_folderACL;
-  PRUint32     m_aclFlags;
-  PRUint32     m_supportedUserFlags;
+  uint32_t     m_aclFlags;
+  uint32_t     m_supportedUserFlags;
 
+  // determines if we are on GMail server
+  bool m_isGmailServer;
   // offline imap support
-  PRBool m_downloadingFolderForOfflineUse;
-  PRBool m_filterListRequiresBody;
+  bool m_downloadingFolderForOfflineUse;
+  bool m_filterListRequiresBody;
 
-  // auto-sync (preemptive download) support
+  // auto-sync (automatic message download) support
   nsRefPtr<nsAutoSyncState> m_autoSyncStateObj;
 
   // Quota support
   nsCString m_folderQuotaRoot;
-  PRUint32 m_folderQuotaUsedKB;
-  PRUint32 m_folderQuotaMaxKB;
+  uint32_t m_folderQuotaUsedKB;
+  uint32_t m_folderQuotaMaxKB;
 
   // Pseudo-Offline Playback support
   nsPlaybackRequest *m_pendingPlaybackReq;
@@ -537,5 +520,9 @@ protected:
   // hash table of mapping between messageids and message keys
   // for pseudo hdrs.
   nsDataHashtable<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
+
+  nsTArray<nsMsgKey> m_keysToFetch;
+  uint32_t m_totalKeysToFetch;
+
 };
 #endif
