@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
 #include "netCore.h"
@@ -56,6 +24,7 @@
 #include "nsIMutableArray.h"
 #include "nsIAutoSyncManager.h"
 #include "nsAlgorithm.h"
+#include "nsArrayUtils.h"
 
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
@@ -106,7 +75,7 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   if (m_curTempFile)
   {
     m_curTempFile->Remove(false);
-    m_curTempFile = nsnull;
+    m_curTempFile = nullptr;
   }
   // NS_BINDING_ABORTED is used for the user pressing stop, which
   // should cause us to abort the offline process. Other errors
@@ -150,7 +119,7 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
 
 // leaves m_currentServer at the next imap or local mail "server" that
 // might have offline events to playback. If no more servers,
-// m_currentServer will be left at nsnull.
+// m_currentServer will be left at nullptr.
 // Also, sets up m_serverEnumerator to enumerate over the server
 nsresult nsImapOfflineSync::AdvanceToNextServer()
 {
@@ -159,7 +128,7 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
   if (!m_allServers)
   {
     NS_ASSERTION(!m_currentServer, "this shouldn't be set");
-    m_currentServer = nsnull;
+    m_currentServer = nullptr;
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
              do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     NS_ASSERTION(accountManager && NS_SUCCEEDED(rv), "couldn't get account mgr");
@@ -168,18 +137,26 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
     rv = accountManager->GetAllServers(getter_AddRefs(m_allServers));
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  PRUint32 serverIndex = (m_currentServer) ? m_allServers->IndexOf(m_currentServer) + 1 : 0;
-  m_currentServer = nsnull;
-  PRUint32 numServers; 
-  m_allServers->Count(&numServers);
+  uint32_t serverIndex = 0;
+  if (m_currentServer)
+  {
+    rv = m_allServers->IndexOf(0, m_currentServer, &serverIndex);
+    if (NS_FAILED(rv))
+      serverIndex = -1;
+
+    // Move to the next server
+    ++serverIndex;
+  }
+  m_currentServer = nullptr;
+  uint32_t numServers; 
+  m_allServers->GetLength(&numServers);
   nsCOMPtr <nsIMsgFolder> rootFolder;
 
   while (serverIndex < numServers)
   {
-    nsCOMPtr <nsISupports> serverSupports = getter_AddRefs(m_allServers->ElementAt(serverIndex));
+    nsCOMPtr<nsIMsgIncomingServer> server(do_QueryElementAt(m_allServers, serverIndex));
     serverIndex++;
 
-    nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(serverSupports);
     nsCOMPtr <nsINntpIncomingServer> newsServer = do_QueryInterface(server);
     if (newsServer) // news servers aren't involved in offline imap
       continue;
@@ -214,8 +191,8 @@ nsresult nsImapOfflineSync::AdvanceToNextFolder()
 
   if (m_currentFolder)
   {
-    m_currentFolder->SetMsgDatabase(nsnull);
-    m_currentFolder = nsnull;
+    m_currentFolder->SetMsgDatabase(nullptr);
+    m_currentFolder = nullptr;
   }
 
   if (!m_currentServer)
@@ -238,7 +215,7 @@ nsresult nsImapOfflineSync::AdvanceToNextFolder()
 void nsImapOfflineSync::AdvanceToFirstIMAPFolder()
 {
   nsresult rv;
-  m_currentServer = nsnull;
+  m_currentServer = nullptr;
   nsCOMPtr <nsIMsgImapMailFolder> imapFolder;
   do
   {
@@ -253,7 +230,7 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *op)
 {
   nsCOMPtr <nsIMsgOfflineImapOperation> currentOp = op;
   nsTArray<nsMsgKey> matchingFlagKeys;
-  PRUint32 currentKeyIndex = m_KeyIndex;
+  uint32_t currentKeyIndex = m_KeyIndex;
 
   imapMessageFlagsType matchingFlags;
   currentOp->GetNewFlags(&matchingFlags);
@@ -270,7 +247,7 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *op)
       currentOp->SetPlayingBack(true);
       m_currentOpsToClear.AppendObject(currentOp);
     }
-    currentOp = nsnull;
+    currentOp = nullptr;
     if (++currentKeyIndex < m_CurrentKeys.Length())
       m_currentDB->GetOfflineOpForKey(m_CurrentKeys[currentKeyIndex], false,
         getter_AddRefs(currentOp));
@@ -285,9 +262,9 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *op)
 
   if (!matchingFlagKeys.IsEmpty())
   {
-    nsCAutoString uids;
+    nsAutoCString uids;
     nsImapMailFolder::AllocateUidStringFromKeys(matchingFlagKeys.Elements(), matchingFlagKeys.Length(), uids);
-    PRUint32 curFolderFlags;
+    uint32_t curFolderFlags;
     m_currentFolder->GetFlags(&curFolderFlags);
 
     if (uids.get() && (curFolderFlags & nsMsgFolderFlags::ImapBox)) 
@@ -315,9 +292,9 @@ void nsImapOfflineSync::ProcessKeywordOperation(nsIMsgOfflineImapOperation *op)
 {
   nsCOMPtr <nsIMsgOfflineImapOperation> currentOp = op;
   nsTArray<nsMsgKey> matchingKeywordKeys;
-  PRUint32 currentKeyIndex = m_KeyIndex;
+  uint32_t currentKeyIndex = m_KeyIndex;
 
-  nsCAutoString keywords;
+  nsAutoCString keywords;
   if (mCurrentPlaybackOpType == nsIMsgOfflineImapOperation::kAddKeywords)
     currentOp->GetKeywordsToAdd(getter_Copies(keywords));
   else
@@ -333,13 +310,13 @@ void nsImapOfflineSync::ProcessKeywordOperation(nsIMsgOfflineImapOperation *op)
       currentOp->SetPlayingBack(true);
       m_currentOpsToClear.AppendObject(currentOp);
     }
-    currentOp = nsnull;
+    currentOp = nullptr;
     if (++currentKeyIndex < m_CurrentKeys.Length())
       m_currentDB->GetOfflineOpForKey(m_CurrentKeys[currentKeyIndex], false,
         getter_AddRefs(currentOp));
     if (currentOp)
     {
-      nsCAutoString curOpKeywords;
+      nsAutoCString curOpKeywords;
       nsOfflineImapOperationType operation;
       currentOp->GetOperation(&operation);
       if (mCurrentPlaybackOpType == nsIMsgOfflineImapOperation::kAddKeywords)
@@ -353,7 +330,7 @@ void nsImapOfflineSync::ProcessKeywordOperation(nsIMsgOfflineImapOperation *op)
 
   if (!matchingKeywordKeys.IsEmpty())
   {
-    PRUint32 curFolderFlags;
+    uint32_t curFolderFlags;
     m_currentFolder->GetFlags(&curFolderFlags);
 
     if (curFolderFlags & nsMsgFolderFlags::ImapBox)
@@ -382,7 +359,7 @@ void nsImapOfflineSync::ProcessKeywordOperation(nsIMsgOfflineImapOperation *op)
 }
 
 void
-nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *currentOp, PRInt32 opType)
+nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *currentOp, int32_t opType)
 {
   nsCOMPtr <nsIMsgDBHdr> mailHdr;
   nsMsgKey msgKey;
@@ -390,8 +367,8 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
   nsresult rv = m_currentDB->GetMsgHdrForKey(msgKey, getter_AddRefs(mailHdr)); 
   if (NS_SUCCEEDED(rv) && mailHdr)
   {
-    PRUint64 messageOffset;
-    PRUint32 messageSize;
+    uint64_t messageOffset;
+    uint32_t messageSize;
     mailHdr->GetMessageOffset(&messageOffset);
     mailHdr->GetOfflineMessageSize(&messageSize);
     nsCOMPtr<nsIFile> tmpFile;
@@ -431,8 +408,8 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
               if (NS_SUCCEEDED(rv))
               {
                 // now, copy the dest folder offline store msg to the temp file
-                PRInt32 inputBufferSize = 10240;
-                char *inputBuffer = nsnull;
+                int32_t inputBufferSize = 10240;
+                char *inputBuffer = nullptr;
                 
                 while (!inputBuffer && (inputBufferSize >= 512))
                 {
@@ -440,13 +417,13 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
                   if (!inputBuffer)
                     inputBufferSize /= 2;
                 }
-                PRInt32 bytesLeft;
-                PRUint32 bytesRead, bytesWritten;
+                int32_t bytesLeft;
+                uint32_t bytesRead, bytesWritten;
                 bytesLeft = messageSize;
                 rv = NS_OK;
                 while (bytesLeft > 0 && NS_SUCCEEDED(rv))
                 {
-                  PRInt32 bytesToRead = NS_MIN(inputBufferSize, bytesLeft);
+                  int32_t bytesToRead = NS_MIN(inputBufferSize, bytesLeft);
                   rv = offlineStoreInputStream->Read(inputBuffer, bytesToRead, &bytesRead);
                   if (NS_SUCCEEDED(rv) && bytesRead > 0)
                   {
@@ -468,7 +445,7 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
                   nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID);
                   if (copyService)
                     rv = copyService->CopyFileMessage(cloneTmpFile, destFolder,
-                    /* nsIMsgDBHdr* msgToReplace */ nsnull,
+                    /* nsIMsgDBHdr* msgToReplace */ nullptr,
                     true /* isDraftOrTemplate */,
                     0, // new msg flags - are there interesting flags here?
                     EmptyCString(), /* are there keywords we should get? */
@@ -480,7 +457,7 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
               }
               currentOp->SetPlayingBack(true);
               m_currentOpsToClear.AppendObject(currentOp);
-              m_currentDB->DeleteHeader(mailHdr, nsnull, true, true);
+              m_currentDB->DeleteHeader(mailHdr, nullptr, true, true);
             }
           }
           // want to close in failure case too
@@ -489,12 +466,17 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
       }
     }
   }
+  else
+  {
+    m_currentDB->RemoveOfflineOp(currentOp);
+    ProcessNextOperation();
+  }
 }
 
 void nsImapOfflineSync::ClearCurrentOps()
 {
-  PRInt32 opCount = m_currentOpsToClear.Count();
-  for (PRInt32 i = opCount - 1; i >= 0; i--)
+  int32_t opCount = m_currentOpsToClear.Count();
+  for (int32_t i = opCount - 1; i >= 0; i--)
   {
     m_currentOpsToClear[i]->SetPlayingBack(false);
     m_currentOpsToClear[i]->ClearOperation(mCurrentPlaybackOpType);
@@ -505,7 +487,7 @@ void nsImapOfflineSync::ClearCurrentOps()
 void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *op)
 {
   nsTArray<nsMsgKey> matchingFlagKeys;
-  PRUint32 currentKeyIndex = m_KeyIndex;
+  uint32_t currentKeyIndex = m_KeyIndex;
   nsCString moveDestination;
   op->GetDestinationFolderURI(getter_Copies(moveDestination));
   bool moveMatches = true;
@@ -520,7 +502,7 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *op)
       currentOp->SetPlayingBack(true);
       m_currentOpsToClear.AppendObject(currentOp);
     }
-    currentOp = nsnull;
+    currentOp = nullptr;
     
     if (++currentKeyIndex < m_CurrentKeys.Length())
     {
@@ -564,13 +546,13 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *op)
     nsCOMPtr<nsIMutableArray> messages(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv))
     {
-      for (PRUint32 keyIndex = 0; keyIndex < matchingFlagKeys.Length(); keyIndex++)
+      for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length(); keyIndex++)
       {
-        nsCOMPtr<nsIMsgDBHdr> mailHdr = nsnull;
+        nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
         rv = m_currentFolder->GetMessageHeader(matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
         if (NS_SUCCEEDED(rv) && mailHdr)
         {
-          PRUint32 msgSize;
+          uint32_t msgSize;
           // in case of a move, the header has already been deleted,
           // so we've really got a fake header. We need to get its flags and
           // size from the offline op to have any chance of doing the move.
@@ -578,7 +560,7 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *op)
           if (!msgSize)
           {
             imapMessageFlagsType newImapFlags;
-            PRUint32 msgFlags = 0;
+            uint32_t msgFlags = 0;
             op->GetMsgSize(&msgSize);
             op->GetNewFlags(&newImapFlags);
             // first three bits are the same
@@ -618,7 +600,7 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *aCurren
   nsCOMPtr<nsIMsgOfflineImapOperation> currentOp = aCurrentOp;
 
   nsTArray<nsMsgKey> matchingFlagKeys;
-  PRUint32 currentKeyIndex = m_KeyIndex;
+  uint32_t currentKeyIndex = m_KeyIndex;
   nsCString copyDestination;
   currentOp->GetCopyDestination(0, getter_Copies(copyDestination));
   bool copyMatches = true;
@@ -633,7 +615,7 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *aCurren
       currentOp->SetPlayingBack(true);
       m_currentOpsToClear.AppendObject(currentOp);
     }
-    currentOp = nsnull;
+    currentOp = nullptr;
 
     if (++currentKeyIndex < m_CurrentKeys.Length())
     {
@@ -655,7 +637,7 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *aCurren
   } 
   while (currentOp);
 
-  nsCAutoString uids;
+  nsAutoCString uids;
   nsCOMPtr<nsIMsgFolder> destFolder;
   GetExistingFolder(copyDestination, getter_AddRefs(destFolder));
   // if the dest folder doesn't really exist, these operations are
@@ -678,9 +660,9 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *aCurren
     nsCOMPtr<nsIMutableArray> messages(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
     if (messages && NS_SUCCEEDED(rv))
     {
-      for (PRUint32 keyIndex = 0; keyIndex < matchingFlagKeys.Length(); keyIndex++)
+      for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length(); keyIndex++)
       {
-        nsCOMPtr<nsIMsgDBHdr> mailHdr = nsnull;
+        nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
         rv = m_currentFolder->GetMessageHeader(matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
         if (NS_SUCCEEDED(rv) && mailHdr)
           messages->AppendElement(mailHdr, false);
@@ -703,7 +685,7 @@ bool nsImapOfflineSync::CreateOfflineFolders()
 {
   while (m_currentFolder)
   {
-    PRUint32 flags;
+    uint32_t flags;
     m_currentFolder->GetFlags(&flags);
     bool offlineCreate = (flags & nsMsgFolderFlags::CreatedOffline) != 0;
     if (offlineCreate)
@@ -727,7 +709,7 @@ bool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
   imapFolder->GetOnlineName(onlineName);
 
   NS_ConvertASCIItoUTF16 folderName(onlineName);
-  nsresult rv = imapFolder->PlaybackOfflineFolderCreate(folderName, nsnull,  getter_AddRefs(createFolderURI));
+  nsresult rv = imapFolder->PlaybackOfflineFolderCreate(folderName, nullptr,  getter_AddRefs(createFolderURI));
   if (createFolderURI && NS_SUCCEEDED(rv))
   {
     nsCOMPtr <nsIMsgMailNewsUrl> mailnewsUrl = do_QueryInterface(createFolderURI);
@@ -737,7 +719,7 @@ bool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
   return NS_SUCCEEDED(rv) ? true : false;	// this is asynch, we have to return and be called again by the OfflineOpExitFunction
 }
 
-PRInt32 nsImapOfflineSync::GetCurrentUIDValidity()
+int32_t nsImapOfflineSync::GetCurrentUIDValidity()
 {
   if (m_currentFolder)
   {
@@ -756,7 +738,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
 {
   nsresult rv = NS_OK;
   // find a folder that needs to process operations
-  nsIMsgFolder *deletedAllOfflineEventsInFolder = nsnull;
+  nsIMsgFolder *deletedAllOfflineEventsInFolder = nullptr;
 
   // if we haven't created offline folders, and we're updating all folders,
   // first, find offline folders to create.
@@ -775,7 +757,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
     {
       if (CreateOfflineFolders())
         return NS_OK;
-      m_currentServer = nsnull;
+      m_currentServer = nullptr;
       AdvanceToNextFolder();
     }
     m_createdOfflineFolders = true;
@@ -784,7 +766,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
   if (m_singleFolderToUpdate)
     m_currentFolder = m_singleFolderToUpdate;
   
-  PRUint32 folderFlags;
+  uint32_t folderFlags;
   nsCOMPtr <nsIDBFolderInfo> folderInfo;
   while (m_currentFolder && !m_currentDB)
   {
@@ -803,17 +785,17 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
     {
       m_CurrentKeys.Clear();
       m_KeyIndex = 0;
-      if ((m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys) != 0) || m_CurrentKeys.IsEmpty())
+      if (NS_FAILED(m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys)) || m_CurrentKeys.IsEmpty())
       {
         ClearDB();
-        folderInfo = nsnull; // can't hold onto folderInfo longer than db
+        folderInfo = nullptr; // can't hold onto folderInfo longer than db
         m_currentFolder->ClearFlag(nsMsgFolderFlags::OfflineEvents);
       }
       else
       {
         // trash any ghost msgs
         bool deletedGhostMsgs = false;
-        for (PRUint32 fakeIndex=0; fakeIndex < m_CurrentKeys.Length(); fakeIndex++)
+        for (uint32_t fakeIndex=0; fakeIndex < m_CurrentKeys.Length(); fakeIndex++)
         {
           nsCOMPtr <nsIMsgOfflineImapOperation> currentOp; 
           m_currentDB->GetOfflineOpForKey(m_CurrentKeys[fakeIndex], false, getter_AddRefs(currentOp));
@@ -847,7 +829,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
                   imapFolder->ChangePendingUnread(1);
                 imapFolder->AddMoveResultPseudoKey(curKey);
               }
-              m_currentDB->DeleteMessage(curKey, nsnull, false);
+              m_currentDB->DeleteMessage(curKey, nullptr, false);
             }
           }
         }
@@ -856,7 +838,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
           m_currentFolder->SummaryChanged();
         
         m_CurrentKeys.Clear();
-        if ( (m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys) != 0) || m_CurrentKeys.IsEmpty() )
+        if (NS_FAILED(m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys)) || m_CurrentKeys.IsEmpty())
         {
           ClearDB();
           if (deletedGhostMsgs)
@@ -886,7 +868,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
       if (!m_singleFolderToUpdate)
         AdvanceToNextFolder();
       else
-        m_currentFolder = nsnull;	// force update of this folder now.
+        m_currentFolder = nullptr;	// force update of this folder now.
     }
     
   }
@@ -904,7 +886,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         (m_pseudoOffline || (GetCurrentUIDValidity() != 0) ||
         !(folderFlags & nsMsgFolderFlags::ImapBox)))
     {
-      PRInt32 curFolderUidValidity;
+      int32_t curFolderUidValidity;
       folderInfo->GetImapUidValidity(&curFolderUidValidity);
       bool uidvalidityChanged = (!m_pseudoOffline && folderFlags & nsMsgFolderFlags::ImapBox) && (GetCurrentUIDValidity() != curFolderUidValidity);
       nsCOMPtr <nsIMsgOfflineImapOperation> currentOp;
@@ -920,7 +902,10 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         // loop until we find the next db record that matches the current playback operation
         while (currentOp && !(opType & mCurrentPlaybackOpType))
         {
-          currentOp = nsnull;
+          // remove operations with no type.
+          if (!opType)
+            m_currentDB->RemoveOfflineOp(currentOp);
+          currentOp = nullptr;
           ++m_KeyIndex;
           if (m_KeyIndex < m_CurrentKeys.Length())
             m_currentDB->GetOfflineOpForKey(m_CurrentKeys[m_KeyIndex],
@@ -1007,7 +992,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
           {
             // empty trash is going to delete the db, so we'd better release the
             // reference to the offline operation first.
-            currentOp = nsnull;
+            currentOp = nullptr;
             ProcessEmptyTrash();
           }
           else
@@ -1030,7 +1015,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         return NS_OK;
       }
       else
-        m_currentFolder = nsnull;
+        m_currentFolder = nullptr;
     }
   }
   
@@ -1041,7 +1026,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
     // if we are updating more than one folder then we need the iterator
     if (!m_singleFolderToUpdate)
     {
-      m_currentServer = nsnull;
+      m_currentServer = nullptr;
       AdvanceToNextFolder();
     }
     if (m_singleFolderToUpdate)
@@ -1052,7 +1037,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
       if (imapFolder)
       {
         nsCOMPtr<nsIUrlListener> saveListener = m_listener;
-//        m_listener = nsnull;
+//        m_listener = nullptr;
 //        imapFolder->UpdateFolderWithListener(m_window, saveListener);
       }
     }
@@ -1062,10 +1047,10 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
   printf("done with offline imap sync\n");
 #endif
   nsCOMPtr <nsIUrlListener> saveListener = m_listener;
-  m_listener = nsnull;
+  m_listener = nullptr;
 
   if (saveListener)
-    saveListener->OnStopRunningUrl(nsnull /* don't know url */, rv);
+    saveListener->OnStopRunningUrl(nullptr /* don't know url */, rv);
   return rv;
 }
 
@@ -1080,12 +1065,12 @@ void nsImapOfflineSync::DeleteAllOfflineOpsForCurrentDB()
     // NS_ASSERTION(currentOp->GetOperationFlags() == 0);
     // delete any ops that have already played back
     m_currentDB->RemoveOfflineOp(currentOp);
-    m_currentDB->Commit(nsMsgDBCommitType::kLargeCommit);
-    currentOp = nsnull;
+    currentOp = nullptr;
     
     if (++m_KeyIndex < m_CurrentKeys.Length())
       m_currentDB->GetOfflineOpForKey(m_CurrentKeys[m_KeyIndex], false, getter_AddRefs(currentOp));
   }
+  m_currentDB->Commit(nsMsgDBCommitType::kLargeCommit);
   // turn off nsMsgFolderFlags::OfflineEvents
   if (m_currentFolder)
     m_currentFolder->ClearFlag(nsMsgFolderFlags::OfflineEvents);
@@ -1115,8 +1100,8 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
              do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
-    nsCOMPtr<nsISupportsArray> servers;
-  
+
+    nsCOMPtr<nsIArray> servers;
     rv = accountManager->GetAllServers(getter_AddRefs(servers));
     if (NS_FAILED(rv)) return rv;
   }
@@ -1174,7 +1159,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     }
     else
     {
-      m_allServers = nsnull;
+      m_allServers = nullptr;
       m_mailboxupdatesFinished = true;
     }
   }
@@ -1182,7 +1167,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
 
   while (m_currentFolder)
   {
-    PRUint32 folderFlags;
+    uint32_t folderFlags;
 
     ClearDB();
     nsCOMPtr <nsIMsgImapMailFolder> imapFolder;
@@ -1201,7 +1186,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     AdvanceToNextFolder();
   }
   if (m_listener)
-    m_listener->OnStopRunningUrl(nsnull, NS_OK);
+    m_listener->OnStopRunningUrl(nullptr, NS_OK);
   return rv;
 }
 
@@ -1211,14 +1196,14 @@ NS_IMETHODIMP nsImapOfflineSync::OnStartCopy()
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-/* void OnProgress (in PRUint32 aProgress, in PRUint32 aProgressMax); */
-NS_IMETHODIMP nsImapOfflineSync::OnProgress(PRUint32 aProgress, PRUint32 aProgressMax)
+/* void OnProgress (in uint32_t aProgress, in uint32_t aProgressMax); */
+NS_IMETHODIMP nsImapOfflineSync::OnProgress(uint32_t aProgress, uint32_t aProgressMax)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-/* void SetMessageKey (in PRUint32 aKey); */
-NS_IMETHODIMP nsImapOfflineSync::SetMessageKey(PRUint32 aKey)
+/* void SetMessageKey (in uint32_t aKey); */
+NS_IMETHODIMP nsImapOfflineSync::SetMessageKey(uint32_t aKey)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -1232,7 +1217,7 @@ NS_IMETHODIMP nsImapOfflineSync::GetMessageId(nsACString& messageId)
 /* void OnStopCopy (in nsresult aStatus); */
 NS_IMETHODIMP nsImapOfflineSync::OnStopCopy(nsresult aStatus)
 {
-  return OnStopRunningUrl(nsnull, aStatus);
+  return OnStopRunningUrl(nullptr, aStatus);
 }
 
 void nsImapOfflineSync::ClearDB()
@@ -1240,12 +1225,12 @@ void nsImapOfflineSync::ClearDB()
   m_currentOpsToClear.Clear();
   if (m_currentDB)
     m_currentDB->RemoveListener(this);
-  m_currentDB = nsnull;
+  m_currentDB = nullptr;
 }
 
 NS_IMETHODIMP
 nsImapOfflineSync::OnHdrPropertyChanged(nsIMsgDBHdr *aHdrToChange,
-    bool aPreChange, PRUint32 *aStatus, nsIDBChangeListener * aInstigator)
+    bool aPreChange, uint32_t *aStatus, nsIDBChangeListener * aInstigator)
 {
   return NS_OK;
 }
@@ -1253,21 +1238,21 @@ nsImapOfflineSync::OnHdrPropertyChanged(nsIMsgDBHdr *aHdrToChange,
 
 NS_IMETHODIMP
 nsImapOfflineSync::OnHdrFlagsChanged(nsIMsgDBHdr *aHdrChanged,
-    PRUint32 aOldFlags, PRUint32 aNewFlags, nsIDBChangeListener *aInstigator)
+    uint32_t aOldFlags, uint32_t aNewFlags, nsIDBChangeListener *aInstigator)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsImapOfflineSync::OnHdrDeleted(nsIMsgDBHdr *aHdrChanged,
-    nsMsgKey aParentKey, PRInt32 aFlags, nsIDBChangeListener *aInstigator)
+    nsMsgKey aParentKey, int32_t aFlags, nsIDBChangeListener *aInstigator)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsImapOfflineSync::OnHdrAdded(nsIMsgDBHdr *aHdrAdded,
-    nsMsgKey aParentKey, PRInt32 aFlags, nsIDBChangeListener *aInstigator)
+    nsMsgKey aParentKey, int32_t aFlags, nsIDBChangeListener *aInstigator)
 {
     return NS_OK;
 }
