@@ -132,7 +132,7 @@ bool nsEudoraMac::VerifyEudoraLocation(nsIFile **pFolder, bool findIni)
   bool hasMore;
   nsCOMPtr<nsISimpleEnumerator> directoryEnumerator;
   nsresult rv = (*pFolder)->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, false);
 
   directoryEnumerator->HasMoreElements(&hasMore);
   nsCOMPtr <nsIFile> prefFile;
@@ -143,7 +143,7 @@ bool nsEudoraMac::VerifyEudoraLocation(nsIFile **pFolder, bool findIni)
   {
     nsCOMPtr<nsISupports> aSupport;
     rv = directoryEnumerator->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsILocalFile> entry(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIFile> entry(do_QueryInterface(aSupport, &rv));
     directoryEnumerator->HasMoreElements(&hasMore);
 
 
@@ -185,7 +185,7 @@ bool nsEudoraMac::VerifyEudoraLocation(nsIFile **pFolder, bool findIni)
             {
               // Neither of the pref files was named .bkup
               // Pick the newest one?
-              PRInt64 modDate1, modDate2;
+              int64_t modDate1, modDate2;
 
               entry->GetLastModifiedTime(&modDate2);
               prefFile->GetLastModifiedTime(&modDate1);
@@ -260,7 +260,7 @@ nsresult nsEudoraMac::IterateMailDir(nsIFile *pFolder, nsISupportsArray *pArray,
   {
     nsCOMPtr<nsISupports> aSupport;
     rv = directoryEnumerator->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsILocalFile> entry(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIFile> entry(do_QueryInterface(aSupport, &rv));
     directoryEnumerator->HasMoreElements(&hasMore);
 
     bool isFolder;
@@ -333,16 +333,15 @@ nsresult nsEudoraMac::FoundMailbox(nsIFile *mailFile, const char *pName, nsISupp
   nsresult rv = pImport->CreateNewMailboxDescriptor(getter_AddRefs(desc));
   if (NS_SUCCEEDED(rv))
   {
-    PRInt64    sz = 0;
+    int64_t    sz = 0;
     mailFile->GetFileSize(&sz);
     desc->SetDisplayName(displayName.get());
     desc->SetDepth(m_depth);
-    nsCOMPtr <nsILocalFile> pLocalFile;
+    nsCOMPtr <nsIFile> pLocalFile;
     desc->GetFile(getter_AddRefs(pLocalFile));
     if (pLocalFile)
     {
-      nsCOMPtr <nsILocalFile> localMailFile = do_QueryInterface(mailFile);
-      pLocalFile->InitWithFile(localMailFile);
+      pLocalFile->InitWithFile(mailFile);
     }
     rv = desc->QueryInterface(kISupportsIID, (void **) &pInterface);
     pArray->AppendElement(pInterface);
@@ -353,7 +352,7 @@ nsresult nsEudoraMac::FoundMailbox(nsIFile *mailFile, const char *pName, nsISupp
 }
 
 
-nsresult nsEudoraMac::FoundMailFolder(nsILocalFile *mailFolder, const char *pName, nsISupportsArray *pArray, nsIImportService *pImport)
+nsresult nsEudoraMac::FoundMailFolder(nsIFile *mailFolder, const char *pName, nsISupportsArray *pArray, nsIImportService *pImport)
 {
   nsAutoString          displayName;
   nsCOMPtr<nsIImportMailboxDescriptor>  desc;
@@ -374,11 +373,11 @@ nsresult nsEudoraMac::FoundMailFolder(nsILocalFile *mailFolder, const char *pNam
   nsresult rv = pImport->CreateNewMailboxDescriptor(getter_AddRefs(desc));
   if (NS_SUCCEEDED(rv))
   {
-    PRInt64    sz = 0;
+    int64_t    sz = 0;
     desc->SetDisplayName(displayName.get());
     desc->SetDepth(m_depth);
     desc->SetSize(sz);
-    nsCOMPtr <nsILocalFile> pFile;
+    nsCOMPtr <nsIFile> pFile;
     desc->GetFile(getter_AddRefs(pFile));
     if (pFile)
       pFile->InitWithFile(mailFolder);
@@ -414,7 +413,7 @@ bool nsEudoraMac::CreateTocFromResource(nsIFile *pMail, nsIFile **pToc)
   bool     result = false;
   if (resH)
   {
-    PRInt32 sz = (PRInt32) GetHandleSize(resH);
+    int32_t sz = (int32_t) GetHandleSize(resH);
     if (sz)
     {
       // Create the new TOC file
@@ -434,7 +433,7 @@ bool nsEudoraMac::CreateTocFromResource(nsIFile *pMail, nsIFile **pToc)
       if (NS_SUCCEEDED(rv))
       {
         HLock(resH);
-        PRUint32 written = 0;
+        uint32_t written = 0;
         rv = outputStream->Write(*resH, sz, &written);
         HUnlock(resH);
         outputStream->Close();
@@ -457,7 +456,7 @@ nsresult nsEudoraMac::FindTOCFile(nsIFile *pMailFile, nsIFile **ppTOCFile, bool
   nsresult    rv;
 
   *pDeleteToc = false;
-  *ppTOCFile = nsnull;
+  *ppTOCFile = nullptr;
   nsCString leaf;
   rv = pMailFile->GetNativeLeafName(leaf);
   if (NS_FAILED(rv))
@@ -506,24 +505,24 @@ static StringPtr GetStringFromHandle(Handle aResource,
                                      ResourceIndex aId)
 {
   if (!aResource)
-    return nsnull;
+    return nullptr;
 
-  PRUint8 *data = *(PRUint8**)aResource;
-  PRUint16 count = *(PRUint16*)data;
+  uint8_t *data = *(uint8_t**)aResource;
+  uint16_t count = *(uint16_t*)data;
 #if defined(IS_LITTLE_ENDIAN)
   count = count << 8 | count >> 8;
 #endif
 
   // First 2 bytes are the count of string that this resource has.
   if (count < aId)
-    return nsnull;
+    return nullptr;
 
   // skip count
   data += 2;
 
   // looking for data.  data is in order
   while (--aId > 0)
-    data = data + (*(PRUint8*)data) + 1;
+    data = data + (*(uint8_t*)data) + 1;
 
   return data;
 }
@@ -605,7 +604,7 @@ bool nsEudoraMac::GetSettingsFromResource(nsIFile *pSettings, short resId, nsCSt
       }
       if (idx <= *theStr)
       {
-        PRUint8  save = *theStr;
+        uint8_t  save = *theStr;
         *theStr = idx - 1;
         if (*theStr)
           pStrs[1]->Append((const char *) (theStr + 1), *theStr);
@@ -680,7 +679,7 @@ bool nsEudoraMac::ImportSettings(nsIFile *pIniFile, nsIMsgAccount **localMailAcc
     isIMAP = false;
     if (GetSettingsFromResource(pIniFile, baseResId, pStrs, &isIMAP))
     {
-      pAccount = nsnull;
+      pAccount = nullptr;
       if (!isIMAP)
       {
         // This is a POP account
@@ -751,7 +750,7 @@ bool nsEudoraMac::FindFiltersFile(nsIFile **pFiltersFile)
 bool nsEudoraMac::BuildPOPAccount(nsIMsgAccountManager *accMgr, nsCString **pStrs, nsIMsgAccount **ppAccount, nsString& accName)
 {
   if (ppAccount)
-    *ppAccount = nsnull;
+    *ppAccount = nullptr;
 
 
   if (!pStrs[kPopServerStr]->Length() || !pStrs[kPopAccountNameStr]->Length())
@@ -762,7 +761,7 @@ bool nsEudoraMac::BuildPOPAccount(nsIMsgAccountManager *accMgr, nsCString **pStr
   // I now have a user name/server name pair, find out if it already exists?
   nsCOMPtr<nsIMsgIncomingServer> in;
   nsresult rv = accMgr->FindServer(*(pStrs[kPopAccountNameStr]), *(pStrs[kPopServerStr]), NS_LITERAL_CSTRING("pop3"), getter_AddRefs(in));
-  if (NS_FAILED(rv) || (in == nsnull))
+  if (NS_FAILED(rv) || (in == nullptr))
   {
     // Create the incoming server and an account for it?
     rv = accMgr->CreateIncomingServer(*(pStrs[kPopAccountNameStr]), *(pStrs[kPopServerStr]), NS_LITERAL_CSTRING("pop3"), getter_AddRefs(in));
@@ -786,7 +785,7 @@ bool nsEudoraMac::BuildPOPAccount(nsIMsgAccountManager *accMgr, nsCString **pStr
         IMPORT_LOG0("Created a new account and set the incoming server to the POP3 server.\n");
 
         nsCOMPtr<nsIPop3IncomingServer> pop3Server = do_QueryInterface(in, &rv);
-        NS_ENSURE_SUCCESS(rv,rv);
+        NS_ENSURE_SUCCESS(rv, false);
         pop3Server->SetLeaveMessagesOnServer(pStrs[kLeaveOnServerStr]->First() == 'Y' ? true : false);
 
         // Fiddle with the identities
@@ -813,7 +812,7 @@ bool nsEudoraMac::BuildIMAPAccount(nsIMsgAccountManager *accMgr, nsCString **pSt
 
   nsCOMPtr<nsIMsgIncomingServer>  in;
   nsresult rv = accMgr->FindServer(*(pStrs[kPopAccountNameStr]), *(pStrs[kPopServerStr]), NS_LITERAL_CSTRING("imap"), getter_AddRefs(in));
-  if (NS_FAILED(rv) || (in == nsnull))
+  if (NS_FAILED(rv) || (in == nullptr))
   {
     // Create the incoming server and an account for it?
     rv = accMgr->CreateIncomingServer(*(pStrs[kPopAccountNameStr]), *(pStrs[kPopServerStr]), NS_LITERAL_CSTRING("imap"), getter_AddRefs(in));
@@ -868,7 +867,7 @@ void nsEudoraMac::SetIdentities(nsIMsgAccountManager *accMgr, nsIMsgAccount *acc
       id->SetEmail(*(pStrs[kReturnAddressStr]));
     else
     {
-      nsCAutoString emailAddress;
+      nsAutoCString emailAddress;
       emailAddress = userName;
       emailAddress += "@";
       emailAddress += serverName;
@@ -921,9 +920,9 @@ nsresult nsEudoraMac::GetAttachmentInfo(const char *pFileName, nsIFile *pFile, n
   // Sample attachment line
   // Internet:sandh.jpg (JPEG/JVWR) (0003C2E8)
 
-  OSType    type = '????';
-  OSType    creator = '????';
-  PRUint32  fNum = 0;
+  OSType    type = '?\??\?';
+  OSType    creator = '?\??\?';
+  uint32_t  fNum = 0;
   int      i;
   PRUnichar  c;
 
@@ -958,12 +957,12 @@ nsresult nsEudoraMac::GetAttachmentInfo(const char *pFileName, nsIFile *pFile, n
         str.Right(types, 11);
         if ((types.CharAt(0) == '(') && (types.CharAt(5) == '/') && (types.CharAt(10) == ')'))
         {
-          type = ((PRUint32)types.CharAt(1)) << 24;
-          type |= ((PRUint32)types.CharAt(2)) << 16;
+          type = ((uint32_t)types.CharAt(1)) << 24;
+          type |= ((uint32_t)types.CharAt(2)) << 16;
           type |= types.CharAt(3) << 8;
           type |= types.CharAt(4);
-          creator = ((PRUint32)types.CharAt(6)) << 24;
-          creator |= ((PRUint32)types.CharAt(7)) << 16;
+          creator = ((uint32_t)types.CharAt(6)) << 24;
+          creator |= ((uint32_t)types.CharAt(7)) << 16;
           creator |= types.CharAt(8) << 8;
           creator |= types.CharAt(9);
           str.Left(types, str.Length() - 11);
@@ -989,9 +988,7 @@ nsresult nsEudoraMac::GetAttachmentInfo(const char *pFileName, nsIFile *pFile, n
   memset(&fsRef, 0, sizeof(fsRef));
   {
     nsresult rv;
-    nsCOMPtr <nsILocalFile> pLocalFile = do_QueryInterface(pFile, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    pLocalFile->InitWithNativePath(str);
+    rv = pFile->InitWithNativePath(str);
     if (NS_FAILED(rv))
     {
       IMPORT_LOG0("\tfailed to set native path\n");
@@ -1030,7 +1027,7 @@ bool nsEudoraMac::HasResourceFork(FSRef *fsRef)
   FSCatalogInfo catalogInfo;
   OSErr err = FSGetCatalogInfo(fsRef,
                                kFSCatInfoDataSizes + kFSCatInfoRsrcSizes,
-                               &catalogInfo, nsnull, nsnull, nsnull);
+                               &catalogInfo, nullptr, nullptr, nullptr);
   return (err == noErr && catalogInfo.rsrcLogicalSize != 0);
 }
 
@@ -1072,7 +1069,7 @@ bool nsEudoraMac::IsValidMailboxName(nsCString& fName)
 
 bool nsEudoraMac::IsValidMailboxFile(nsIFile *pFile)
 {
-  PRInt64  size = 0;
+  int64_t  size = 0;
   nsresult rv = pFile->GetFileSize(&size);
   if (size)
   {
@@ -1082,7 +1079,7 @@ bool nsEudoraMac::IsValidMailboxFile(nsIFile *pFile)
     rv = NS_NewLocalFileInputStream(getter_AddRefs(inputStream), pFile);
     if (NS_FAILED(rv))
       return false;
-    PRUint32  read = 0;
+    uint32_t  read = 0;
     char  buffer[6];
     char *  pBuf = buffer;
     rv = inputStream->Read(pBuf, 5, &read);
@@ -1111,10 +1108,9 @@ nsresult nsEudoraMac::FindAddressBooks(nsIFile *pRoot, nsISupportsArray **ppArra
   // additional files in the Nicknames folder
   // Try and find the nickNames file
   nsresult rv;
-  nsCOMPtr<nsILocalFile> file = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
+  nsCOMPtr<nsIFile> file = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  nsCOMPtr<nsILocalFile> localRoot = do_QueryInterface(pRoot);
-  rv = file->InitWithFile(localRoot);
+  rv = file->InitWithFile(pRoot);
   if (NS_FAILED(rv))
     return rv;
   rv = NS_NewISupportsArray(ppArray);
@@ -1130,7 +1126,7 @@ nsresult nsEudoraMac::FindAddressBooks(nsIFile *pRoot, nsISupportsArray **ppArra
 
   nsString displayName;
   nsEudoraStringBundle::GetStringByID(EUDORAIMPORT_NICKNAMES_NAME, displayName);
-  PRInt64  sz = 0;
+  int64_t  sz = 0;
 
   // First find the Nicknames file itself
   rv = file->AppendNative(NS_LITERAL_CSTRING("Eudora Nicknames"));
@@ -1168,7 +1164,7 @@ nsresult nsEudoraMac::FindAddressBooks(nsIFile *pRoot, nsISupportsArray **ppArra
   }
 
   // Now try the directory of address books!
-  rv = file->InitWithFile(localRoot);
+  rv = file->InitWithFile(pRoot);
   if (NS_SUCCEEDED(rv))
     rv = file->AppendNative(NS_LITERAL_CSTRING("Nicknames Folder"));
   exists = false;
@@ -1196,7 +1192,7 @@ nsresult nsEudoraMac::FindAddressBooks(nsIFile *pRoot, nsISupportsArray **ppArra
   {
     nsCOMPtr<nsISupports> aSupport;
     rv = directoryEnumerator->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsILocalFile> entry(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIFile> entry(do_QueryInterface(aSupport, &rv));
     directoryEnumerator->HasMoreElements(&hasMore);
 
     if (NS_SUCCEEDED(rv))
