@@ -102,9 +102,9 @@ bool nsOE5File::IsLocalMailFile(nsIFile *pFile)
   return result;
 }
 
-bool nsOE5File::ReadIndex(nsIInputStream *pInputStream, PRUint32 **ppIndex, PRUint32 *pSize)
+bool nsOE5File::ReadIndex(nsIInputStream *pInputStream, uint32_t **ppIndex, uint32_t *pSize)
 {
-  *ppIndex = nsnull;
+  *ppIndex = nullptr;
   *pSize = 0;
 
   char    signature[4];
@@ -118,8 +118,8 @@ bool nsOE5File::ReadIndex(nsIInputStream *pInputStream, PRUint32 **ppIndex, PRUi
     }
   }
 
-  PRUint32  offset = 0x00e4;
-  PRUint32  indexStart = 0;
+  uint32_t  offset = 0x00e4;
+  uint32_t  indexStart = 0;
   if (!ReadBytes(pInputStream, &indexStart, offset, 4)) {
     IMPORT_LOG0("*** Unable to read offset to index start\n");
     return false;
@@ -128,9 +128,9 @@ bool nsOE5File::ReadIndex(nsIInputStream *pInputStream, PRUint32 **ppIndex, PRUi
   PRUint32Array array;
   array.count = 0;
   array.alloc = kIndexGrowBy;
-  array.pIndex = new PRUint32[kIndexGrowBy];
+  array.pIndex = new uint32_t[kIndexGrowBy];
 
-  PRUint32 next = ReadMsgIndex(pInputStream, indexStart, &array);
+  uint32_t next = ReadMsgIndex(pInputStream, indexStart, &array);
   while (next) {
     next = ReadMsgIndex(pInputStream, next, &array);
   }
@@ -146,7 +146,7 @@ bool nsOE5File::ReadIndex(nsIInputStream *pInputStream, PRUint32 **ppIndex, PRUi
 }
 
 
-PRUint32 nsOE5File::ReadMsgIndex(nsIInputStream *pInputStream, PRUint32 offset, PRUint32Array *pArray)
+uint32_t nsOE5File::ReadMsgIndex(nsIInputStream *pInputStream, uint32_t offset, PRUint32Array *pArray)
 {
   // Record is:
     // 4 byte marker
@@ -162,7 +162,7 @@ PRUint32 nsOE5File::ReadMsgIndex(nsIInputStream *pInputStream, PRUint32 offset,
   //  2 - child index pointer
   //  3 - number of records in child
 
-  PRUint32  marker;
+  uint32_t  marker;
 
   if (!ReadBytes(pInputStream, &marker, offset, 4))
     return 0;
@@ -171,37 +171,37 @@ PRUint32 nsOE5File::ReadMsgIndex(nsIInputStream *pInputStream, PRUint32 offset,
     return 0;
 
 
-  PRUint32  vals[3];
+  uint32_t  vals[3];
 
   if (!ReadBytes(pInputStream, vals, offset + 4, 12))
     return 0;
 
 
-  PRUint8  len[4];
+  uint8_t  len[4];
   if (!ReadBytes(pInputStream, len, offset + 16, 4))
     return 0;
 
 
 
-  PRUint32  cnt = (PRUint32) len[1];
+  uint32_t  cnt = (uint32_t) len[1];
   cnt *= 3;
-  PRUint32  *pData = new PRUint32[cnt];
+  uint32_t  *pData = new uint32_t[cnt];
 
   if (!ReadBytes(pInputStream, pData, offset + 24, cnt * 4)) {
     delete [] pData;
     return 0;
   }
 
-  PRUint32  next;
-  PRUint32  indexOffset;
-  PRUint32 *  pRecord = pData;
-  PRUint32 *  pNewIndex;
+  uint32_t  next;
+  uint32_t  indexOffset;
+  uint32_t *  pRecord = pData;
+  uint32_t *  pNewIndex;
 
-  for (PRUint8 i = 0; i < (PRUint8)len[1]; i++, pRecord += 3) {
+  for (uint8_t i = 0; i < (uint8_t)len[1]; i++, pRecord += 3) {
     indexOffset = pRecord[0];
 
     if (pArray->count >= pArray->alloc) {
-      pNewIndex = new PRUint32[ pArray->alloc + kIndexGrowBy];
+      pNewIndex = new uint32_t[ pArray->alloc + kIndexGrowBy];
       memcpy(pNewIndex, pArray->pIndex, (pArray->alloc * 4));
       (pArray->alloc) += kIndexGrowBy;
       delete [] pArray->pIndex;
@@ -229,7 +229,7 @@ PRUint32 nsOE5File::ReadMsgIndex(nsIInputStream *pInputStream, PRUint32 offset,
   return vals[1];
 }
 
-bool nsOE5File::IsFromLine(char *pLine, PRUint32 len)
+bool nsOE5File::IsFromLine(char *pLine, uint32_t len)
 {
    return (len > 5 && (pLine[0] == 'F') && (pLine[1] == 'r') && (pLine[2] == 'o') && (pLine[3] == 'm') && (pLine[4] == ' '));
 }
@@ -239,11 +239,11 @@ bool nsOE5File::IsFromLine(char *pLine, PRUint32 len)
 #define  kMaxAttrCount       0x0030
 const char *nsOE5File::m_pFromLineSep = "From - Mon Jan 1 00:00:00 1965\x0D\x0A";
 
-nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
+nsresult nsOE5File::ImportMailbox(uint32_t *pBytesDone, bool *pAbort,
                                   nsString& name, nsIFile *inFile,
-                                  nsIMsgFolder *dstFolder, PRUint32 *pCount)
+                                  nsIMsgFolder *dstFolder, uint32_t *pCount)
 {
-  PRInt32    msgCount = 0;
+  int32_t    msgCount = 0;
   if (pCount)
     *pCount = 0;
 
@@ -254,25 +254,25 @@ nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
   rv = dstFolder->GetMsgStore(getter_AddRefs(msgStore));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 *  pIndex;
-  PRUint32  indexSize;
-  PRUint32 *  pFlags;
-  PRUint64  *  pTime;
+  uint32_t *  pIndex;
+  uint32_t  indexSize;
+  uint32_t *  pFlags;
+  uint64_t  *  pTime;
 
   if (!ReadIndex(inputStream, &pIndex, &indexSize)) {
     IMPORT_LOG1("No messages found in mailbox: %s\n", NS_LossyConvertUTF16toASCII(name.get()));
     return NS_OK;
   }
 
-  pTime  = new PRUint64[ indexSize];
-  pFlags = new PRUint32[ indexSize];
+  pTime  = new uint64_t[ indexSize];
+  pFlags = new uint32_t[ indexSize];
   char *  pBuffer = new char[kMailboxBufferSize];
   if (!(*pAbort))
     ConvertIndex(inputStream, pBuffer, pIndex, indexSize, pFlags, pTime);
 
-  PRUint32  block[4];
-  PRInt32   sepLen = (PRInt32) strlen(m_pFromLineSep);
-  PRUint32   written;
+  uint32_t  block[4];
+  int32_t   sepLen = (int32_t) strlen(m_pFromLineSep);
+  uint32_t   written;
 
   /*
       Each block is:
@@ -295,14 +295,14 @@ nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
       5. Reset some of the control variables and repeat step #3.
   */
 
-  PRUint32  didBytes = 0;
-  PRUint32  next, size;
+  uint32_t  didBytes = 0;
+  uint32_t  next, size;
   char *pStart, *pEnd, *partialLineStart;
-  nsCAutoString partialLine, tempLine;
+  nsAutoCString partialLine, tempLine;
   nsCOMPtr<nsIOutputStream> outputStream;
   rv = NS_OK;
 
-  for (PRUint32 i = 0; (i < indexSize) && !(*pAbort); i++)
+  for (uint32_t i = 0; (i < indexSize) && !(*pAbort); i++)
   {
     if (! pIndex[i])
       continue;
@@ -361,7 +361,7 @@ nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
         PL_strcpy(result + 7, buffer);
         PL_strcpy(result + 7 + 24, CRLF);
 
-        rv = outputStream->Write(result, (PRInt32) strlen(result), &written);
+        rv = outputStream->Write(result, (int32_t) strlen(result), &written);
       }
       if (NS_FAILED(rv))
       {
@@ -373,7 +373,7 @@ nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
       }
 
       char statusLine[50];
-      PRUint32 msgFlags = XLATFLAGS(pFlags[i]);
+      uint32_t msgFlags = XLATFLAGS(pFlags[i]);
       PR_snprintf(statusLine, sizeof(statusLine), X_MOZILLA_STATUS_FORMAT MSG_LINEBREAK, msgFlags & 0xFFFF);
       rv = outputStream->Write(statusLine, strlen(statusLine), &written);
       NS_ENSURE_SUCCESS(rv,rv);
@@ -536,62 +536,62 @@ nsresult nsOE5File::ImportMailbox(PRUint32 *pBytesDone, bool *pAbort,
 */
 
 void nsOE5File::ConvertIndex(nsIInputStream *pFile, char *pBuffer,
-                              PRUint32 *pIndex, PRUint32 size,
-                              PRUint32 *pFlags, PRUint64 *pTime)
+                              uint32_t *pIndex, uint32_t size,
+                              uint32_t *pFlags, uint64_t *pTime)
 {
   // for each index record, get the actual message offset!  If there is a
   // problem just record the message offset as 0 and the message reading code
   // can log that error information.
   // XXXTODO- above error reporting is not done
 
-  PRUint8   recordHead[12];
-  PRUint32  marker;
-  PRUint32  recordSize;
-  PRUint32  numAttrs;
-  PRUint32  offset;
-  PRUint32  attrIndex;
-  PRUint32  attrOffset;
-  PRUint8   tag;
-  PRUint32  tagData;
-  PRUint32  flags;
-  PRUint64  time;
-  PRUint32  dataStart;
+  uint8_t   recordHead[12];
+  uint32_t  marker;
+  uint32_t  recordSize;
+  uint32_t  numAttrs;
+  uint32_t  offset;
+  uint32_t  attrIndex;
+  uint32_t  attrOffset;
+  uint8_t   tag;
+  uint32_t  tagData;
+  uint32_t  flags;
+  uint64_t  time;
+  uint32_t  dataStart;
 
-  for (PRUint32 i = 0; i < size; i++) {
+  for (uint32_t i = 0; i < size; i++) {
     offset = 0;
     flags = 0;
     time = 0;
     if (ReadBytes(pFile, recordHead, pIndex[i], 12)) {
       memcpy(&marker, recordHead, 4);
       memcpy(&recordSize, recordHead + 4, 4);
-      numAttrs = (PRUint32) recordHead[10];
+      numAttrs = (uint32_t) recordHead[10];
       if (marker == pIndex[i] && numAttrs <= kMaxAttrCount) {
         dataStart = pIndex[i] + 12 + (numAttrs * 4);
         if (ReadBytes(pFile, pBuffer, kDontSeek, numAttrs * 4)) {
           attrOffset = 0;
           for (attrIndex = 0; attrIndex < numAttrs; attrIndex++, attrOffset += 4) {
-            tag = (PRUint8) pBuffer[attrOffset];
-            if (tag == (PRUint8) 0x84) {
+            tag = (uint8_t) pBuffer[attrOffset];
+            if (tag == (uint8_t) 0x84) {
               tagData = 0;
               memcpy(&tagData, pBuffer + attrOffset + 1, 3);
               offset = tagData;
             }
-            else if (tag == (PRUint8) 0x04) {
+            else if (tag == (uint8_t) 0x04) {
               tagData = 0;
               memcpy(&tagData, pBuffer + attrOffset + 1, 3);
               ReadBytes(pFile, &offset, dataStart + tagData, 4);
             }
-            else if (tag == (PRUint8) 0x81) {
+            else if (tag == (uint8_t) 0x81) {
               tagData = 0;
               memcpy(&tagData, pBuffer + attrOffset +1, 3);
               flags = tagData;
             }
-            else if (tag == (PRUint8) 0x01) {
+            else if (tag == (uint8_t) 0x01) {
               tagData = 0;
               memcpy(&tagData, pBuffer + attrOffset +1, 3);
               ReadBytes(pFile, &flags, dataStart + tagData, 4);
             }
-            else if (tag == (PRUint8) 0x02) {
+            else if (tag == (uint8_t) 0x02) {
               tagData = 0;
               memcpy(&tagData, pBuffer + attrOffset +1, 3);
               ReadBytes(pFile, &time, dataStart + tagData, 4);
@@ -607,7 +607,7 @@ void nsOE5File::ConvertIndex(nsIInputStream *pFile, char *pBuffer,
 }
 
 
-bool nsOE5File::ReadBytes(nsIInputStream *stream, void *pBuffer, PRUint32 offset, PRUint32 bytes)
+bool nsOE5File::ReadBytes(nsIInputStream *stream, void *pBuffer, uint32_t offset, uint32_t bytes)
 {
   nsresult  rv;
 
@@ -621,7 +621,7 @@ bool nsOE5File::ReadBytes(nsIInputStream *stream, void *pBuffer, PRUint32 offset
   if (!bytes)
     return true;
 
-  PRUint32  cntRead;
+  uint32_t  cntRead;
   char *  pReadTo = (char *)pBuffer;
   rv = stream->Read(pReadTo, bytes, &cntRead);
   return NS_SUCCEEDED(rv) && cntRead == bytes;
