@@ -7,7 +7,7 @@
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
 #include "nsIImportService.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIImportMailboxDescriptor.h"
 #include "nsOERegUtil.h"
 #include "nsOE5File.h"
@@ -16,6 +16,8 @@
 #include "nsIInputStream.h"
 #include "nsISeekableStream.h"
 #include "plstr.h"
+#include <windows.h>
+#include "nsIWindowsRegKey.h"
 
 #ifdef MOZILLA_INTERNAL_API
 #include "nsNativeCharsetUtils.h"
@@ -23,6 +25,8 @@
 #include "nsMsgI18N.h"
 #define NS_CopyNativeToUnicode(source, dest) \
         nsMsgI18NConvertToUnicode(nsMsgI18NFileSystemCharset(), source, dest)
+#define NS_CopyUnicodeToNative(source, dest) \
+        nsMsgI18NConvertFromUnicode(nsMsgI18NFileSystemCharset(), source, dest)
 #endif
 
 /*
@@ -36,7 +40,7 @@ static NS_DEFINE_IID(kISupportsIID,      NS_ISUPPORTS_IID);
 
 nsOEScanBoxes::nsOEScanBoxes()
 {
-  m_pFirst = nsnull;
+  m_pFirst = nullptr;
 }
 
 nsOEScanBoxes::~nsOEScanBoxes()
@@ -67,65 +71,71 @@ nsOEScanBoxes::~nsOEScanBoxes()
 
 bool nsOEScanBoxes::Find50Mail(nsIFile *pWhere)
 {
-  nsresult   rv;
-  bool      success = false;
-  HKEY    sKey;
-        nsCOMPtr <nsILocalFile> localWhere = do_QueryInterface(pWhere);
+  nsAutoString userId;
+  nsresult rv = nsOERegUtil::GetDefaultUserId(userId);
+  if (NS_FAILED(rv))
+    return false;
 
-  if (::RegOpenKeyEx(HKEY_CURRENT_USER, "Identities", 0, KEY_QUERY_VALUE, &sKey) == ERROR_SUCCESS) {
-    BYTE *  pBytes = nsOERegUtil::GetValueBytes(sKey, "Default User ID");
-    ::RegCloseKey(sKey);
-    if (pBytes) {
-      nsCString  key("Identities\\");
-      key += (const char *)pBytes;
-      nsOERegUtil::FreeValueBytes(pBytes);
-      key += "\\Software\\Microsoft\\Outlook Express\\5.0";
-      if (::RegOpenKeyEx(HKEY_CURRENT_USER, key.get(), 0, KEY_QUERY_VALUE, &sKey) == ERROR_SUCCESS) {
-        pBytes = nsOERegUtil::GetValueBytes(sKey, "Store Root");
-        if (pBytes) {
-          localWhere->InitWithNativePath(nsDependentCString((const char *)pBytes));
+  nsAutoString path(NS_LITERAL_STRING("Identities\\"));
+  path.Append(userId);
+  path.AppendLiteral("\\Software\\Microsoft\\Outlook Express\\5.0");
 
-          IMPORT_LOG1("Setting native path: %s\n", pBytes);
+  nsCOMPtr<nsIWindowsRegKey> key =
+    do_CreateInstance("@mozilla.org/windows-registry-key;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-          nsOERegUtil::FreeValueBytes(pBytes);
-          bool    isDir = false;
-          rv = pWhere->IsDirectory(&isDir);
-          if (isDir && NS_SUCCEEDED(rv))
-            success = true;
-        }
-        ::RegCloseKey(sKey);
-      }
-    }
-  }
+  rv = key->Open(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
+                 path,
+                 nsIWindowsRegKey::ACCESS_QUERY_VALUE);
+  if (NS_FAILED(rv))
+    return false;
 
-  return success;
+  nsAutoString storeRoot;
+  key->ReadStringValue(NS_LITERAL_STRING("Store Root"), storeRoot);
+  if (NS_FAILED(rv))
+    return false;
+
+  nsCOMPtr<nsIFile> localWhere = do_QueryInterface(pWhere);
+  localWhere->InitWithPath(storeRoot);
+
+  nsAutoCString nativeStoreRoot;
+  NS_CopyUnicodeToNative(storeRoot, nativeStoreRoot);
+  IMPORT_LOG1("Setting native path: %s\n", nativeStoreRoot.get());
+
+  bool isDir = false;
+  rv = localWhere->IsDirectory(&isDir);
+  return isDir;
 }
 
 bool nsOEScanBoxes::FindMail(nsIFile *pWhere)
 {
-  nsresult   rv;
-  bool      success = false;
-  HKEY    sKey;
-        nsCOMPtr <nsILocalFile> localWhere = do_QueryInterface(pWhere);
-
   if (Find50Mail(pWhere))
     return true;
 
-  if (::RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Microsoft\\Outlook Express", 0, KEY_QUERY_VALUE, &sKey) == ERROR_SUCCESS) {
-    LPBYTE  pBytes = nsOERegUtil::GetValueBytes(sKey, "Store Root");
-    if (pBytes) {
-      localWhere->InitWithNativePath(nsDependentCString((const char *) pBytes));
-      pWhere->AppendNative(NS_LITERAL_CSTRING("Mail"));
-      bool    isDir = false;
-      rv = pWhere->IsDirectory(&isDir);
-      if (isDir && NS_SUCCEEDED(rv))
-        success = true;
-      delete [] pBytes;
-    }
-    ::RegCloseKey(sKey);
-  }
+  nsresult rv;
+  nsCOMPtr<nsIWindowsRegKey> key =
+    do_CreateInstance("@mozilla.org/windows-registry-key;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  return success;
+  rv = key->Open(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
+                 NS_LITERAL_STRING("Software\\Microsoft\\Outlook Express"),
+                 nsIWindowsRegKey::ACCESS_QUERY_VALUE);
+  if (NS_FAILED(rv))
+    return false;
+
+  nsAutoString storeRoot;
+  key->ReadStringValue(NS_LITERAL_STRING("Store Root"), storeRoot);
+  if (NS_FAILED(rv))
+    return false;
+
+  nsCOMPtr<nsIFile> localWhere = do_QueryInterface(pWhere);
+  localWhere->InitWithPath(storeRoot);
+  localWhere->AppendNative(NS_LITERAL_CSTRING("Mail"));
+
+  bool isDir = false;
+  rv = localWhere->IsDirectory(&isDir);
+
+  return isDir;
 }
 
 bool nsOEScanBoxes::GetMailboxes(nsIFile *pWhere, nsISupportsArray **pArray)
@@ -186,7 +196,7 @@ void nsOEScanBoxes::Reset(void)
     delete pEntry;
   }
   m_entryArray.Clear();
-  m_pFirst = nsnull;
+  m_pFirst = nullptr;
 }
 
 
@@ -209,17 +219,17 @@ bool nsOEScanBoxes::FindMailBoxes(nsIFile* descFile)
 
   IMPORT_LOG0("Reading the folders.nch file\n");
 
-  PRUint32    curRec;
+  uint32_t    curRec;
   if (!ReadLong(descInputStream, curRec, 20)) {
     return false;
   }
 
   // Now for each record
   bool        done = false;
-  PRUint32    equal;
-  PRUint32    size;
-  PRUint32    previous;
-  PRUint32    next;
+  uint32_t    equal;
+  uint32_t    size;
+  uint32_t    previous;
+  uint32_t    next;
   MailboxEntry *  pEntry;
   bool        failed;
 
@@ -293,43 +303,43 @@ bool nsOEScanBoxes::Find50MailBoxes(nsIFile* descFile)
 
   IMPORT_LOG0("Reading the folders.dbx file\n");
 
-  PRUint32 *    pIndex;
-  PRUint32    indexSize = 0;
+  uint32_t *    pIndex;
+  uint32_t    indexSize = 0;
   if (!nsOE5File::ReadIndex(descInputStream, &pIndex, &indexSize)) {
     IMPORT_LOG0("*** NOT USING FOLDERS.DBX!!!\n");
     return false;
   }
 
-  PRUint32  marker;
-  PRUint32  size;
+  uint32_t  marker;
+  uint32_t  size;
   char  *  pBytes;
-  PRUint32    cntRead;
-  PRInt32    recordId;
-  PRInt32    strOffset;
+  uint32_t    cntRead;
+  int32_t    recordId;
+  int32_t    strOffset;
 
-  PRUint8    tag;
-  PRUint32  data;
-  PRInt32    dataOffset;
+  uint8_t    tag;
+  uint32_t  data;
+  int32_t    dataOffset;
 
-  PRUint32    id;
-  PRUint32    parent;
-  PRUint32    numMessages;
+  uint32_t    id;
+  uint32_t    parent;
+  uint32_t    numMessages;
   char *      pFileName;
   char *      pDataSource;
 
   MailboxEntry *  pEntry;
-  MailboxEntry *  pLastEntry = nsnull;
+  MailboxEntry *  pLastEntry = nullptr;
 
-  PRUint32  localStoreId = 0;
+  uint32_t  localStoreId = 0;
 
-  for (PRUint32 i = 0; i < indexSize; i++) {
+  for (uint32_t i = 0; i < indexSize; i++) {
     if (!ReadLong(descInputStream, marker, pIndex[i])) continue;
     if (marker != pIndex[i]) continue;
     if (!ReadLong(descInputStream, size, pIndex[i] + 4)) continue;
     size += 4;
     pBytes = new char[size];
     rv = descInputStream->Read(pBytes, size, &cntRead);
-    if (NS_FAILED(rv) || ((PRUint32)cntRead != size)) {
+    if (NS_FAILED(rv) || ((uint32_t)cntRead != size)) {
       delete [] pBytes;
       continue;
     }
@@ -341,11 +351,11 @@ bool nsOEScanBoxes::Find50MailBoxes(nsIFile* descFile)
     id = 0;
     parent = 0;
     numMessages = 0;
-    pFileName = nsnull;
-    pDataSource = nsnull;
+    pFileName = nullptr;
+    pDataSource = nullptr;
     dataOffset = 4;
     while (dataOffset < strOffset) {
-      tag = (PRUint8) pBytes[dataOffset];
+      tag = (uint8_t) pBytes[dataOffset];
 
       data = 0; // make sure all bytes are 0 before copying 3 bytes over.
       memcpy(&data, &(pBytes[dataOffset + 1]), 3);
@@ -360,11 +370,11 @@ bool nsOEScanBoxes::Find50MailBoxes(nsIFile* descFile)
           numMessages = data;
         break;
         case 0x03:  // file name for this mailbox
-          if (((PRUint32)strOffset + data) < size)
+          if (((uint32_t)strOffset + data) < size)
             pFileName = (char *)(pBytes + strOffset + data);
         break;
         case 0x05:  // data source for this record (this is not a mailbox!)
-          if (((PRUint32)strOffset + data) < size)
+          if (((uint32_t)strOffset + data) < size)
             pDataSource = (char *) (pBytes + strOffset + data);
         break;
       }
@@ -431,11 +441,11 @@ bool nsOEScanBoxes::Find50MailBoxes(nsIFile* descFile)
   return m_entryArray.Count();
 }
 
-nsOEScanBoxes::MailboxEntry *nsOEScanBoxes::NewMailboxEntry(PRUint32 id, PRUint32 parent, const char *prettyName, char *pFileName)
+nsOEScanBoxes::MailboxEntry *nsOEScanBoxes::NewMailboxEntry(uint32_t id, uint32_t parent, const char *prettyName, char *pFileName)
 {
   MailboxEntry *pEntry = new MailboxEntry();
   if (!pEntry)
-    return nsnull;
+    return nullptr;
 
   pEntry->index = id;
   pEntry->parent = parent;
@@ -449,9 +459,9 @@ nsOEScanBoxes::MailboxEntry *nsOEScanBoxes::NewMailboxEntry(PRUint32 id, PRUint3
   return pEntry;
 }
 
-void nsOEScanBoxes::ProcessPendingChildEntries(PRUint32 parent, PRUint32 rootIndex, nsVoidArray  &childArray)
+void nsOEScanBoxes::ProcessPendingChildEntries(uint32_t parent, uint32_t rootIndex, nsVoidArray  &childArray)
 {
-  PRInt32 i, max;
+  int32_t i, max;
   MailboxEntry *pEntry;
   for (i = 0, max = childArray.Count(); i < max; i++)
   {
@@ -472,7 +482,7 @@ void nsOEScanBoxes::RemoveProcessedChildEntries()
   // Remove already processed entries from the pending list. Note that these entries are also
   // on 'm_entryArray' list so we don't want to deallocate the space for the entries now.
   MailboxEntry * pEntry;
-  PRInt32 i;
+  int32_t i;
   for (i = m_pendingChildArray.Count()-1; i >= 0; i--)
   {
     pEntry = (MailboxEntry *) m_pendingChildArray.ElementAt(i);
@@ -481,7 +491,7 @@ void nsOEScanBoxes::RemoveProcessedChildEntries()
   }
 }
 
-void nsOEScanBoxes::AddChildEntry(MailboxEntry *pEntry, PRUint32 rootIndex)
+void nsOEScanBoxes::AddChildEntry(MailboxEntry *pEntry, uint32_t rootIndex)
 {
   if (!m_pFirst) {
     if (pEntry->parent == rootIndex) {
@@ -494,8 +504,8 @@ void nsOEScanBoxes::AddChildEntry(MailboxEntry *pEntry, PRUint32 rootIndex)
     return;
   }
 
-  MailboxEntry *  pParent = nsnull;
-  MailboxEntry *  pSibling = nsnull;
+  MailboxEntry *  pParent = nullptr;
+  MailboxEntry *  pSibling = nullptr;
   if (pEntry->parent == rootIndex) {
     pSibling = m_pFirst;
   }
@@ -535,7 +545,7 @@ bool nsOEScanBoxes::Scan50MailboxDir(nsIFile * srcDir)
   Reset();
 
   MailboxEntry *  pEntry;
-  PRInt32      index = 1;
+  int32_t      index = 1;
   char *      pLeaf;
 
   bool hasMore;
@@ -552,13 +562,13 @@ bool nsOEScanBoxes::Scan50MailboxDir(nsIFile * srcDir)
   {
     nsCOMPtr<nsISupports> aSupport;
     rv = directoryEnumerator->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsILocalFile> entry(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIFile> entry(do_QueryInterface(aSupport, &rv));
     directoryEnumerator->HasMoreElements(&hasMore);
 
     isFile = false;
     rv = entry->IsFile(&isFile);
     if (NS_SUCCEEDED(rv) && isFile) {
-      pLeaf = nsnull;
+      pLeaf = nullptr;
       rv = entry->GetNativeLeafName(fName);
       if (NS_SUCCEEDED(rv)  &&
         (StringEndsWith(fName, NS_LITERAL_CSTRING(".dbx")))) {
@@ -598,9 +608,9 @@ void nsOEScanBoxes::ScanMailboxDir(nsIFile * srcDir)
   Reset();
 
   MailboxEntry *  pEntry;
-  PRInt32      index = 1;
-  nsCAutoString pLeaf;
-  PRUint32    sLen;
+  int32_t      index = 1;
+  nsAutoCString pLeaf;
+  uint32_t    sLen;
 
   bool hasMore;
   nsCOMPtr<nsISimpleEnumerator> directoryEnumerator;
@@ -619,7 +629,7 @@ void nsOEScanBoxes::ScanMailboxDir(nsIFile * srcDir)
   {
     nsCOMPtr<nsISupports> aSupport;
     rv = directoryEnumerator->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsILocalFile> entry(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIFile> entry(do_QueryInterface(aSupport, &rv));
     directoryEnumerator->HasMoreElements(&hasMore);
 
     isFile = false;
@@ -654,31 +664,31 @@ void nsOEScanBoxes::ScanMailboxDir(nsIFile * srcDir)
 }
 
 
-PRUint32 nsOEScanBoxes::CountMailboxes(MailboxEntry *pBox)
+uint32_t nsOEScanBoxes::CountMailboxes(MailboxEntry *pBox)
 {
-  if (pBox == nsnull) {
-    if (m_pFirst != nsnull)
+  if (pBox == nullptr) {
+    if (m_pFirst != nullptr)
       pBox = m_pFirst;
     else {
       if (m_entryArray.Count() > 0)
         pBox = (MailboxEntry *) m_entryArray.ElementAt(0);
     }
   }
-  PRUint32    count = 0;
+  uint32_t    count = 0;
 
   MailboxEntry *  pChild;
   while (pBox) {
     count++;
     if (pBox->child) {
       pChild = GetIndexEntry(pBox->child);
-      if (pChild != nsnull)
+      if (pChild != nullptr)
         count += CountMailboxes(pChild);
     }
     if (pBox->sibling != -1) {
       pBox = GetIndexEntry(pBox->sibling);
     }
     else
-      pBox = nsnull;
+      pBox = nullptr;
   }
 
   return count;
@@ -692,15 +702,15 @@ bool nsOEScanBoxes::GetMailboxList(nsIFile * root, nsISupportsArray **pArray)
     return false;
   }
 
-  BuildMailboxList(nsnull, root, 1, *pArray);
+  BuildMailboxList(nullptr, root, 1, *pArray);
 
   return true;
 }
 
-void nsOEScanBoxes::BuildMailboxList(MailboxEntry *pBox, nsIFile * root, PRInt32 depth, nsISupportsArray *pArray)
+void nsOEScanBoxes::BuildMailboxList(MailboxEntry *pBox, nsIFile * root, int32_t depth, nsISupportsArray *pArray)
 {
-  if (pBox == nsnull) {
-    if (m_pFirst != nsnull) {
+  if (pBox == nullptr) {
+    if (m_pFirst != nullptr) {
       pBox = m_pFirst;
 
       IMPORT_LOG0("Assigning start of mailbox list to m_pFirst\n");
@@ -713,18 +723,18 @@ void nsOEScanBoxes::BuildMailboxList(MailboxEntry *pBox, nsIFile * root, PRInt32
       }
     }
 
-    if (pBox == nsnull) {
+    if (pBox == nullptr) {
       IMPORT_LOG0("ERROR ASSIGNING STARTING MAILBOX\n");
     }
 
   }
 
   nsresult            rv;
-  nsCOMPtr <nsILocalFile> file;
+  nsCOMPtr <nsIFile> file;
   MailboxEntry *  pChild;
   nsIImportMailboxDescriptor *  pID;
   nsISupports *          pInterface;
-  PRInt64            size;
+  int64_t            size;
 
   nsCOMPtr<nsIImportService> impSvc(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv))
@@ -738,8 +748,7 @@ void nsOEScanBoxes::BuildMailboxList(MailboxEntry *pBox, nsIFile * root, PRInt32
       pID->SetDisplayName((PRUnichar *)pBox->mailName.get());
       if (!pBox->fileName.IsEmpty()) {
         pID->GetFile(getter_AddRefs(file));
-                                nsCOMPtr <nsILocalFile> localRoot = do_QueryInterface(root);
-        file->InitWithFile(localRoot);
+        file->InitWithFile(root);
         file->AppendNative(pBox->fileName);
         size = 0;
         file->GetFileSize(&size);
@@ -753,30 +762,30 @@ void nsOEScanBoxes::BuildMailboxList(MailboxEntry *pBox, nsIFile * root, PRInt32
 
     if (pBox->child) {
       pChild = GetIndexEntry(pBox->child);
-      if (pChild != nsnull)
+      if (pChild != nullptr)
         BuildMailboxList(pChild, root, depth + 1, pArray);
     }
     if (pBox->sibling != -1) {
       pBox = GetIndexEntry(pBox->sibling);
     }
     else
-      pBox = nsnull;
+      pBox = nullptr;
   }
 
 }
 
 
 
-nsOEScanBoxes::MailboxEntry * nsOEScanBoxes::GetIndexEntry(PRUint32 index)
+nsOEScanBoxes::MailboxEntry * nsOEScanBoxes::GetIndexEntry(uint32_t index)
 {
-  PRInt32 max = m_entryArray.Count();
-  for (PRInt32 i = 0; i < max; i++) {
+  int32_t max = m_entryArray.Count();
+  for (int32_t i = 0; i < max; i++) {
     MailboxEntry *pEntry = (MailboxEntry *) m_entryArray.ElementAt(i);
     if (pEntry->index == index)
       return pEntry;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 
@@ -784,7 +793,7 @@ nsOEScanBoxes::MailboxEntry * nsOEScanBoxes::GetIndexEntry(PRUint32 index)
 // File utility routines
 // -------------------------------------------------------
 
-bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, PRInt32& val, PRUint32 offset)
+bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, int32_t& val, uint32_t offset)
 {
   nsresult  rv;
         nsCOMPtr <nsISeekableStream> seekStream = do_QueryInterface(stream, &rv);
@@ -793,14 +802,14 @@ bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, PRInt32& val, PRUint32 off
   if (NS_FAILED(rv))
     return false;
 
-  PRUint32  cntRead;
+  uint32_t  cntRead;
   char * pReadTo = (char *)&val;
   rv = stream->Read(pReadTo, sizeof(val), &cntRead);
 
   return NS_SUCCEEDED(rv) && cntRead == sizeof(val);
 }
 
-bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, PRUint32& val, PRUint32 offset)
+bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, uint32_t& val, uint32_t offset)
 {
   nsresult  rv;
         nsCOMPtr <nsISeekableStream> seekStream = do_QueryInterface(stream, &rv);
@@ -809,7 +818,7 @@ bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, PRUint32& val, PRUint32 of
   if (NS_FAILED(rv))
     return false;
 
-  PRUint32  cntRead;
+  uint32_t  cntRead;
   char * pReadTo = (char *)&val;
   rv = stream->Read(pReadTo, sizeof(val), &cntRead);
 
@@ -823,7 +832,7 @@ bool nsOEScanBoxes::ReadLong(nsIInputStream * stream, PRUint32& val, PRUint32 of
 // but why bother going that far!  If a file name is that long then
 // the heck with it.
 #define  kOutlookExpressStringLength  252
-bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsString& str, PRUint32 offset)
+bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsString& str, uint32_t offset)
 {
   nsresult rv;
   nsCOMPtr <nsISeekableStream> seekStream = do_QueryInterface(stream, &rv);
@@ -833,7 +842,7 @@ bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsString& str, PRUint32
     return false;
 
 
-  PRUint32 cntRead;
+  uint32_t cntRead;
   char buffer[kOutlookExpressStringLength];
   char * pReadTo = buffer;
   rv = stream->Read(pReadTo, kOutlookExpressStringLength, &cntRead);
@@ -845,7 +854,7 @@ bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsString& str, PRUint32
   return true;
 }
 
-bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsCString& str, PRUint32 offset)
+bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsCString& str, uint32_t offset)
 {
   nsresult  rv;
         nsCOMPtr <nsISeekableStream> seekStream = do_QueryInterface(stream, &rv);
@@ -854,7 +863,7 @@ bool nsOEScanBoxes::ReadString(nsIInputStream * stream, nsCString& str, PRUint32
   if (NS_FAILED(rv))
     return false;
 
-  PRUint32  cntRead;
+  uint32_t  cntRead;
   char  buffer[kOutlookExpressStringLength];
   char *  pReadTo = buffer;
   rv = stream->Read(pReadTo, kOutlookExpressStringLength, &cntRead);
