@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
   Outlook mail import
@@ -56,6 +23,9 @@
 #include "nsOutlookMail.h"
 #include "nsUnicharUtils.h"
 #include "nsIOutputStream.h"
+#include "nsIMsgPluggableStore.h"
+#include "nsIMsgHdr.h"
+#include "nsIMsgFolder.h"
 #include "nsMsgI18N.h"
 #include "nsNetUtil.h"
 
@@ -63,8 +33,8 @@ static NS_DEFINE_IID(kISupportsIID,      NS_ISUPPORTS_IID);
 
 /* ------------ Address book stuff ----------------- */
 typedef struct {
-  PRInt32    mozField;
-  PRInt32    multiLine;
+  int32_t    mozField;
+  int32_t    multiLine;
   ULONG    mapiTag;
 } MAPIFields;
 
@@ -138,22 +108,22 @@ nsOutlookMail::~nsOutlookMail()
 //  EmptyAttachments();
 }
 
-nsresult nsOutlookMail::GetMailFolders( nsISupportsArray **pArray)
+nsresult nsOutlookMail::GetMailFolders(nsISupportsArray **pArray)
 {
   if (!m_haveMapi) {
-    IMPORT_LOG0( "GetMailFolders called before Mapi is initialized\n");
-    return( NS_ERROR_FAILURE);
+    IMPORT_LOG0("GetMailFolders called before Mapi is initialized\n");
+    return NS_ERROR_FAILURE;
   }
 
-  nsresult rv = NS_NewISupportsArray( pArray);
-  if (NS_FAILED( rv)) {
-    IMPORT_LOG0( "FAILED to allocate the nsISupportsArray for the mail folder list\n");
-    return( rv);
+  nsresult rv = NS_NewISupportsArray(pArray);
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0("FAILED to allocate the nsISupportsArray for the mail folder list\n");
+    return rv;
   }
 
   nsCOMPtr<nsIImportService> impSvc(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
-  if (NS_FAILED( rv))
-    return( rv);
+  if (NS_FAILED(rv))
+    return rv;
 
   m_gotFolders = true;
 
@@ -163,25 +133,25 @@ nsresult nsOutlookMail::GetMailFolders( nsISupportsArray **pArray)
   m_mapi.LogOn();
 
   if (m_storeList.GetSize() == 0)
-    m_mapi.IterateStores( m_storeList);
+    m_mapi.IterateStores(m_storeList);
 
   int i = 0;
   CMapiFolder *pFolder;
   if (m_storeList.GetSize() > 1) {
-    while ((pFolder = m_storeList.GetItem( i))) {
-      CMapiFolder *pItem = new CMapiFolder( pFolder);
-      pItem->SetDepth( 1);
-      m_folderList.AddItem( pItem);
-      if (!m_mapi.GetStoreFolders( pItem->GetCBEntryID(), pItem->GetEntryID(), m_folderList, 2)) {
-        IMPORT_LOG1( "GetStoreFolders for index %d failed.\n", i);
+    while ((pFolder = m_storeList.GetItem(i))) {
+      CMapiFolder *pItem = new CMapiFolder(pFolder);
+      pItem->SetDepth(1);
+      m_folderList.AddItem(pItem);
+      if (!m_mapi.GetStoreFolders(pItem->GetCBEntryID(), pItem->GetEntryID(), m_folderList, 2)) {
+        IMPORT_LOG1("GetStoreFolders for index %d failed.\n", i);
       }
       i++;
     }
   }
   else {
-    if ((pFolder = m_storeList.GetItem( i))) {
-      if (!m_mapi.GetStoreFolders( pFolder->GetCBEntryID(), pFolder->GetEntryID(), m_folderList, 1)) {
-        IMPORT_LOG1( "GetStoreFolders for index %d failed.\n", i);
+    if ((pFolder = m_storeList.GetItem(i))) {
+      if (!m_mapi.GetStoreFolders(pFolder->GetCBEntryID(), pFolder->GetEntryID(), m_folderList, 1)) {
+        IMPORT_LOG1("GetStoreFolders for index %d failed.\n", i);
       }
     }
   }
@@ -193,71 +163,71 @@ nsresult nsOutlookMail::GetMailFolders( nsISupportsArray **pArray)
   nsString            uniName;
 
   for (i = 0; i < m_folderList.GetSize(); i++) {
-    pFolder = m_folderList.GetItem( i);
-    rv = impSvc->CreateNewMailboxDescriptor( &pID);
-    if (NS_SUCCEEDED( rv)) {
-      pID->SetDepth( pFolder->GetDepth());
-      pID->SetIdentifier( i);
+    pFolder = m_folderList.GetItem(i);
+    rv = impSvc->CreateNewMailboxDescriptor(&pID);
+    if (NS_SUCCEEDED(rv)) {
+      pID->SetDepth(pFolder->GetDepth());
+      pID->SetIdentifier(i);
 
-      pFolder->GetDisplayName( name);
+      pFolder->GetDisplayName(name);
       pID->SetDisplayName(name.get());
 
-      pID->SetSize( 1000);
-      rv = pID->QueryInterface( kISupportsIID, (void **) &pInterface);
-      (*pArray)->AppendElement( pInterface);
+      pID->SetSize(1000);
+      rv = pID->QueryInterface(kISupportsIID, (void **) &pInterface);
+      (*pArray)->AppendElement(pInterface);
       pInterface->Release();
       pID->Release();
     }
   }
 
-  return( NS_OK);
+  return NS_OK;
 }
 
-bool nsOutlookMail::IsAddressBookNameUnique( nsString& name, nsString& list)
+bool nsOutlookMail::IsAddressBookNameUnique(nsString& name, nsString& list)
 {
   nsString    usedName;
   usedName.AppendLiteral("[");
-  usedName.Append( name);
+  usedName.Append(name);
   usedName.AppendLiteral("],");
 
-  return( list.Find( usedName) == -1);
+  return list.Find(usedName) == -1;
 }
 
-void nsOutlookMail::MakeAddressBookNameUnique( nsString& name, nsString& list)
+void nsOutlookMail::MakeAddressBookNameUnique(nsString& name, nsString& list)
 {
   nsString    newName;
   int        idx = 1;
 
   newName = name;
-  while (!IsAddressBookNameUnique( newName, list)) {
+  while (!IsAddressBookNameUnique(newName, list)) {
     newName = name;
     newName.Append(PRUnichar(' '));
-    newName.AppendInt( (PRInt32) idx);
+    newName.AppendInt((int32_t) idx);
     idx++;
   }
 
   name = newName;
   list.AppendLiteral("[");
-  list.Append( name);
+  list.Append(name);
   list.AppendLiteral("],");
 }
 
-nsresult nsOutlookMail::GetAddressBooks( nsISupportsArray **pArray)
+nsresult nsOutlookMail::GetAddressBooks(nsISupportsArray **pArray)
 {
   if (!m_haveMapi) {
-    IMPORT_LOG0( "GetAddressBooks called before Mapi is initialized\n");
-    return( NS_ERROR_FAILURE);
+    IMPORT_LOG0("GetAddressBooks called before Mapi is initialized\n");
+    return NS_ERROR_FAILURE;
   }
 
-  nsresult rv = NS_NewISupportsArray( pArray);
-  if (NS_FAILED( rv)) {
-    IMPORT_LOG0( "FAILED to allocate the nsISupportsArray for the address book list\n");
-    return( rv);
+  nsresult rv = NS_NewISupportsArray(pArray);
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0("FAILED to allocate the nsISupportsArray for the address book list\n");
+    return rv;
   }
 
   nsCOMPtr<nsIImportService> impSvc(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
-  if (NS_FAILED( rv))
-    return( rv);
+  if (NS_FAILED(rv))
+    return rv;
 
   m_gotAddresses = true;
 
@@ -265,25 +235,25 @@ nsresult nsOutlookMail::GetAddressBooks( nsISupportsArray **pArray)
   m_mapi.Initialize();
   m_mapi.LogOn();
   if (m_storeList.GetSize() == 0)
-    m_mapi.IterateStores( m_storeList);
+    m_mapi.IterateStores(m_storeList);
 
   int i = 0;
   CMapiFolder *pFolder;
   if (m_storeList.GetSize() > 1) {
-    while ((pFolder = m_storeList.GetItem( i))) {
-      CMapiFolder *pItem = new CMapiFolder( pFolder);
-      pItem->SetDepth( 1);
-      m_addressList.AddItem( pItem);
-      if (!m_mapi.GetStoreAddressFolders( pItem->GetCBEntryID(), pItem->GetEntryID(), m_addressList)) {
-        IMPORT_LOG1( "GetStoreAddressFolders for index %d failed.\n", i);
+    while ((pFolder = m_storeList.GetItem(i))) {
+      CMapiFolder *pItem = new CMapiFolder(pFolder);
+      pItem->SetDepth(1);
+      m_addressList.AddItem(pItem);
+      if (!m_mapi.GetStoreAddressFolders(pItem->GetCBEntryID(), pItem->GetEntryID(), m_addressList)) {
+        IMPORT_LOG1("GetStoreAddressFolders for index %d failed.\n", i);
       }
       i++;
     }
   }
   else {
-    if ((pFolder = m_storeList.GetItem( i))) {
-      if (!m_mapi.GetStoreAddressFolders( pFolder->GetCBEntryID(), pFolder->GetEntryID(), m_addressList)) {
-        IMPORT_LOG1( "GetStoreFolders for index %d failed.\n", i);
+    if ((pFolder = m_storeList.GetItem(i))) {
+      if (!m_mapi.GetStoreAddressFolders(pFolder->GetCBEntryID(), pFolder->GetEntryID(), m_addressList)) {
+        IMPORT_LOG1("GetStoreFolders for index %d failed.\n", i);
       }
     }
   }
@@ -295,33 +265,33 @@ nsresult nsOutlookMail::GetAddressBooks( nsISupportsArray **pArray)
   nsString            list;
 
   for (i = 0; i < m_addressList.GetSize(); i++) {
-    pFolder = m_addressList.GetItem( i);
+    pFolder = m_addressList.GetItem(i);
     if (!pFolder->IsStore()) {
-      rv = impSvc->CreateNewABDescriptor( &pID);
-      if (NS_SUCCEEDED( rv)) {
-        pID->SetIdentifier( i);
-        pFolder->GetDisplayName( name);
-        MakeAddressBookNameUnique( name, list);
+      rv = impSvc->CreateNewABDescriptor(&pID);
+      if (NS_SUCCEEDED(rv)) {
+        pID->SetIdentifier(i);
+        pFolder->GetDisplayName(name);
+        MakeAddressBookNameUnique(name, list);
         pID->SetPreferredName(name);
-        pID->SetSize( 100);
-        rv = pID->QueryInterface( kISupportsIID, (void **) &pInterface);
-        (*pArray)->AppendElement( pInterface);
+        pID->SetSize(100);
+        rv = pID->QueryInterface(kISupportsIID, (void **) &pInterface);
+        (*pArray)->AppendElement(pInterface);
         pInterface->Release();
         pID->Release();
       }
     }
   }
 
-  return( NS_OK);
+  return NS_OK;
 }
 
-void nsOutlookMail::OpenMessageStore( CMapiFolder *pNextFolder)
+void nsOutlookMail::OpenMessageStore(CMapiFolder *pNextFolder)
 {
   // Open the store specified
   if (pNextFolder->IsStore()) {
-    if (!m_mapi.OpenStore( pNextFolder->GetCBEntryID(), pNextFolder->GetEntryID(), &m_lpMdb)) {
+    if (!m_mapi.OpenStore(pNextFolder->GetCBEntryID(), pNextFolder->GetEntryID(), &m_lpMdb)) {
       m_lpMdb = NULL;
-      IMPORT_LOG0( "CMapiApi::OpenStore failed\n");
+      IMPORT_LOG0("CMapiApi::OpenStore failed\n");
     }
 
     return;
@@ -330,19 +300,19 @@ void nsOutlookMail::OpenMessageStore( CMapiFolder *pNextFolder)
   // Check to see if we should open the one and only store
   if (!m_lpMdb) {
     if (m_storeList.GetSize() == 1) {
-      CMapiFolder * pFolder = m_storeList.GetItem( 0);
+      CMapiFolder * pFolder = m_storeList.GetItem(0);
       if (pFolder) {
-        if (!m_mapi.OpenStore( pFolder->GetCBEntryID(), pFolder->GetEntryID(), &m_lpMdb)) {
+        if (!m_mapi.OpenStore(pFolder->GetCBEntryID(), pFolder->GetEntryID(), &m_lpMdb)) {
           m_lpMdb = NULL;
-          IMPORT_LOG0( "CMapiApi::OpenStore failed\n");
+          IMPORT_LOG0("CMapiApi::OpenStore failed\n");
         }
       }
       else {
-        IMPORT_LOG0( "Error retrieving the one & only message store\n");
+        IMPORT_LOG0("Error retrieving the one & only message store\n");
       }
     }
     else {
-      IMPORT_LOG0( "*** Error importing a folder without a valid message store\n");
+      IMPORT_LOG0("*** Error importing a folder without a valid message store\n");
     }
   }
 }
@@ -356,7 +326,7 @@ void nsOutlookMail::OpenMessageStore( CMapiFolder *pNextFolder)
 //   - For each mail, create one CMapiMessage object
 //
 // nsOutlookCompose
-//   - Establich a TB session
+//   - Establish a TB session
 //   - Connect to all required services
 //   - Perform the composition of the RC822 document from the data gathered by CMapiMessage
 //   - Save the composed message to the TB mailbox
@@ -366,67 +336,74 @@ void nsOutlookMail::OpenMessageStore( CMapiFolder *pNextFolder)
 //   - Encapsulate the MAPI message interface
 //   - Gather the information required to (re)compose the message
 
-nsresult nsOutlookMail::ImportMailbox( PRUint32 *pDoneSoFar, bool *pAbort, PRInt32 index, const PRUnichar *pName, nsIFile *pDest, PRInt32 *pMsgCount)
+nsresult nsOutlookMail::ImportMailbox(uint32_t *pDoneSoFar, bool *pAbort,
+                                      int32_t index, const PRUnichar *pName,
+                                      nsIMsgFolder *dstFolder,
+                                      int32_t *pMsgCount)
 {
   if ((index < 0) || (index >= m_folderList.GetSize())) {
-    IMPORT_LOG0( "*** Bad mailbox identifier, unable to import\n");
+    IMPORT_LOG0("*** Bad mailbox identifier, unable to import\n");
     *pAbort = true;
-    return( NS_ERROR_FAILURE);
+    return NS_ERROR_FAILURE;
   }
 
-  PRInt32    dummyMsgCount = 0;
+  int32_t    dummyMsgCount = 0;
   if (pMsgCount)
     *pMsgCount = 0;
   else
     pMsgCount = &dummyMsgCount;
 
-  CMapiFolder *pFolder = m_folderList.GetItem( index);
-  OpenMessageStore( pFolder);
+  CMapiFolder *pFolder = m_folderList.GetItem(index);
+  OpenMessageStore(pFolder);
   if (!m_lpMdb) {
-    IMPORT_LOG1( "*** Unable to obtain mapi message store for mailbox: %S\n", pName);
-    return( NS_ERROR_FAILURE);
+    IMPORT_LOG1("*** Unable to obtain mapi message store for mailbox: %S\n", pName);
+    return NS_ERROR_FAILURE;
   }
 
   if (pFolder->IsStore())
-    return( NS_OK);
-
-  nsresult  rv;
+    return NS_OK;
 
   // now what?
-  CMapiFolderContents    contents( m_lpMdb, pFolder->GetCBEntryID(), pFolder->GetEntryID());
+  CMapiFolderContents    contents(m_lpMdb, pFolder->GetCBEntryID(), pFolder->GetEntryID());
 
   BOOL    done = FALSE;
   ULONG    cbEid;
   LPENTRYID  lpEid;
   ULONG    oType;
-  LPMESSAGE  lpMsg = nsnull;
+  LPMESSAGE  lpMsg = nullptr;
   ULONG    totalCount;
-  PRFloat64  doneCalc;
+  double  doneCalc;
 
-  nsCOMPtr<nsIOutputStream> destOutputStream;
-  rv = MsgNewBufferedFileOutputStream(getter_AddRefs(destOutputStream), pDest, -1, 0600);
+  nsCOMPtr<nsIOutputStream> outputStream;
+  nsCOMPtr<nsIMsgPluggableStore> msgStore;
+  nsresult rv = dstFolder->GetMsgStore(getter_AddRefs(msgStore));
   NS_ENSURE_SUCCESS(rv, rv);
 
   while (!done) {
-    if (!contents.GetNext( &cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1( "*** Error iterating mailbox: %S\n", pName);
-      return( NS_ERROR_FAILURE);
+    if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
+      IMPORT_LOG1("*** Error iterating mailbox: %S\n", pName);
+      return NS_ERROR_FAILURE;
     }
 
+    nsCOMPtr<nsIMsgDBHdr> msgHdr;
+    bool reusable;
+
+    rv = msgStore->GetNewMsgOutputStream(dstFolder, getter_AddRefs(msgHdr), &reusable,
+                                         getter_AddRefs(outputStream));
     totalCount = contents.GetCount();
     doneCalc = *pMsgCount;
     doneCalc /= totalCount;
     doneCalc *= 1000;
     if (pDoneSoFar) {
-      *pDoneSoFar = (PRUint32) doneCalc;
+      *pDoneSoFar = (uint32_t) doneCalc;
       if (*pDoneSoFar > 1000)
         *pDoneSoFar = 1000;
     }
 
     if (!done && (oType == MAPI_MESSAGE)) {
-      if (!m_mapi.OpenMdbEntry( m_lpMdb, cbEid, lpEid, (LPUNKNOWN *) &lpMsg)) {
-        IMPORT_LOG1( "*** Error opening messages in mailbox: %S\n", pName);
-        return( NS_ERROR_FAILURE);
+      if (!m_mapi.OpenMdbEntry(m_lpMdb, cbEid, lpEid, (LPUNKNOWN *) &lpMsg)) {
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName);
+        return NS_ERROR_FAILURE;
       }
 
       // See if it's a drafts folder. Outlook doesn't allow drafts
@@ -434,24 +411,31 @@ nsresult nsOutlookMail::ImportMailbox( PRUint32 *pDoneSoFar, bool *pAbort, PRInt
       nsAutoString folderName(pName);
       nsMsgDeliverMode mode = nsIMsgSend::nsMsgDeliverNow;
       mode = nsIMsgSend::nsMsgSaveAsDraft;
-      if ( folderName.LowerCaseEqualsLiteral("drafts") )
+      if (folderName.LowerCaseEqualsLiteral("drafts"))
         mode = nsIMsgSend::nsMsgSaveAsDraft;
 
-      rv = ImportMessage(lpMsg, destOutputStream, mode);
-      if (NS_SUCCEEDED( rv)) // No errors & really imported
-        (*pMsgCount)++;
+      rv = ImportMessage(lpMsg, outputStream, mode);
+      if (NS_SUCCEEDED(rv)){ // No errors & really imported
+         (*pMsgCount)++;
+        msgStore->FinishNewMessage(outputStream, msgHdr);
+      }
       else {
         IMPORT_LOG1( "*** Error reading message from mailbox: %S\n", pName);
+        msgStore->DiscardNewMessage(outputStream, msgHdr);
       }
+      if (!reusable)
+        outputStream->Close();
     }
   }
 
-  return( NS_OK);
+  if (outputStream)
+    outputStream->Close();
+  return NS_OK;
 }
 
-nsresult nsOutlookMail::ImportMessage( LPMESSAGE lpMsg, nsIOutputStream *pDest, nsMsgDeliverMode mode)
+nsresult nsOutlookMail::ImportMessage(LPMESSAGE lpMsg, nsIOutputStream *pDest, nsMsgDeliverMode mode)
 {
-  CMapiMessage  msg( lpMsg);
+  CMapiMessage  msg(lpMsg);
   // If we wanted to skip messages that were downloaded in header only mode, we
   // would return NS_ERROR_FAILURE if !msg.FullMessageDownloaded. However, we
   // don't do this because it may cause seemingly wrong import results.
@@ -474,28 +458,26 @@ nsresult nsOutlookMail::ImportMessage( LPMESSAGE lpMsg, nsIOutputStream *pDest,
   nsresult rv = compose.ProcessMessage(mode, msg, pDest);
 
   // Just for YUCKS, let's try an extra endline
-  WriteData( pDest, "\x0D\x0A", 2);
+  WriteData(pDest, "\x0D\x0A", 2);
 
   return rv;
 }
 
-BOOL nsOutlookMail::WriteData( nsIOutputStream *pDest, const char *pData, PRInt32 len)
+BOOL nsOutlookMail::WriteData(nsIOutputStream *pDest, const char *pData, int32_t len)
 {
-  PRUint32    written;
-  nsresult rv = pDest->Write( pData, len, &written);
-  if (NS_FAILED( rv) || (written != len))
-    return( FALSE);
-  return( TRUE);
+  uint32_t    written;
+  nsresult rv = pDest->Write(pData, len, &written);
+  return NS_SUCCEEDED(rv) && written == len;
 }
 
-nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, const PRUnichar *pName, PRUint32 id, nsIAddrDatabase *pDb, nsString& errors)
+nsresult nsOutlookMail::ImportAddresses(uint32_t *pCount, uint32_t *pTotal, const PRUnichar *pName, uint32_t id, nsIAddrDatabase *pDb, nsString& errors)
 {
-  if (id >= (PRUint32)(m_addressList.GetSize())) {
-    IMPORT_LOG0( "*** Bad address identifier, unable to import\n");
-    return( NS_ERROR_FAILURE);
+  if (id >= (uint32_t)(m_addressList.GetSize())) {
+    IMPORT_LOG0("*** Bad address identifier, unable to import\n");
+    return NS_ERROR_FAILURE;
   }
 
-  PRUint32  dummyCount = 0;
+  uint32_t  dummyCount = 0;
   if (pCount)
     *pCount = 0;
   else
@@ -503,38 +485,38 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
 
   CMapiFolder *pFolder;
   if (id > 0) {
-    PRInt32 idx = (PRInt32) id;
+    int32_t idx = (int32_t) id;
     idx--;
     while (idx >= 0) {
-      pFolder = m_addressList.GetItem( idx);
+      pFolder = m_addressList.GetItem(idx);
       if (pFolder->IsStore()) {
-        OpenMessageStore( pFolder);
+        OpenMessageStore(pFolder);
         break;
       }
       idx--;
     }
   }
 
-  pFolder = m_addressList.GetItem( id);
-  OpenMessageStore( pFolder);
+  pFolder = m_addressList.GetItem(id);
+  OpenMessageStore(pFolder);
   if (!m_lpMdb) {
-    IMPORT_LOG1( "*** Unable to obtain mapi message store for address book: %S\n", pName);
-    return( NS_ERROR_FAILURE);
+    IMPORT_LOG1("*** Unable to obtain mapi message store for address book: %S\n", pName);
+    return NS_ERROR_FAILURE;
   }
 
   if (pFolder->IsStore())
-    return( NS_OK);
+    return NS_OK;
 
   nsresult  rv;
 
   nsCOMPtr<nsIImportFieldMap>    pFieldMap;
 
   nsCOMPtr<nsIImportService> impSvc(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
-  if (NS_SUCCEEDED( rv)) {
-    rv = impSvc->CreateNewFieldMap( getter_AddRefs( pFieldMap));
+  if (NS_SUCCEEDED(rv)) {
+    rv = impSvc->CreateNewFieldMap(getter_AddRefs(pFieldMap));
   }
 
-  CMapiFolderContents    contents( m_lpMdb, pFolder->GetCBEntryID(), pFolder->GetEntryID());
+  CMapiFolderContents    contents(m_lpMdb, pFolder->GetCBEntryID(), pFolder->GetEntryID());
 
   BOOL      done = FALSE;
   ULONG      cbEid;
@@ -548,41 +530,41 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
   while (!done) {
     (*pCount)++;
 
-    if (!contents.GetNext( &cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1( "*** Error iterating address book: %S\n", pName);
-      return( NS_ERROR_FAILURE);
+    if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
+      IMPORT_LOG1("*** Error iterating address book: %S\n", pName);
+      return NS_ERROR_FAILURE;
     }
 
     if (pTotal && (*pTotal == 0))
       *pTotal = contents.GetCount();
 
     if (!done && (oType == MAPI_MESSAGE)) {
-      if (!m_mapi.OpenMdbEntry( m_lpMdb, cbEid, lpEid, (LPUNKNOWN *) &lpMsg)) {
-        IMPORT_LOG1( "*** Error opening messages in mailbox: %S\n", pName);
-        return( NS_ERROR_FAILURE);
+      if (!m_mapi.OpenMdbEntry(m_lpMdb, cbEid, lpEid, (LPUNKNOWN *) &lpMsg)) {
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName);
+        return NS_ERROR_FAILURE;
       }
 
       // Get the PR_MESSAGE_CLASS attribute,
       // ensure that it is IPM.Contact
-      pVal = m_mapi.GetMapiProperty( lpMsg, PR_MESSAGE_CLASS);
+      pVal = m_mapi.GetMapiProperty(lpMsg, PR_MESSAGE_CLASS);
       if (pVal) {
         type.Truncate();
-        m_mapi.GetStringFromProp( pVal, type);
+        m_mapi.GetStringFromProp(pVal, type);
         if (type.EqualsLiteral("IPM.Contact")) {
           // This is a contact, add it to the address book!
           subject.Truncate();
-          pVal = m_mapi.GetMapiProperty( lpMsg, PR_SUBJECT);
+          pVal = m_mapi.GetMapiProperty(lpMsg, PR_SUBJECT);
           if (pVal)
-            m_mapi.GetStringFromProp( pVal, subject);
+            m_mapi.GetStringFromProp(pVal, subject);
 
-          nsIMdbRow* newRow = nsnull;
-          pDb->GetNewRow( &newRow);
+          nsIMdbRow* newRow = nullptr;
+          pDb->GetNewRow(&newRow);
           // FIXME: Check with Candice about releasing the newRow if it
           // isn't added to the database.  Candice's code in nsAddressBook
           // never releases it but that doesn't seem right to me!
           if (newRow) {
-            if (BuildCard( subject.get(), pDb, newRow, lpMsg, pFieldMap)) {
-              pDb->AddCardRowToDB( newRow);
+            if (BuildCard(subject.get(), pDb, newRow, lpMsg, pFieldMap)) {
+              pDb->AddCardRowToDB(newRow);
             }
           }
         }
@@ -590,9 +572,9 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
         {
           // This is a list/group, add it to the address book!
           subject.Truncate();
-          pVal = m_mapi.GetMapiProperty( lpMsg, PR_SUBJECT);
+          pVal = m_mapi.GetMapiProperty(lpMsg, PR_SUBJECT);
           if (pVal)
-            m_mapi.GetStringFromProp( pVal, subject);
+            m_mapi.GetStringFromProp(pVal, subject);
           CreateList(subject.get(), pDb, lpMsg, pFieldMap);
         }
       }
@@ -604,7 +586,7 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
   rv = pDb->Commit(nsAddrDBCommitType::kLargeCommit);
   return rv;
 }
-nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
+nsresult nsOutlookMail::CreateList(const PRUnichar * pName,
                                    nsIAddrDatabase *pDb,
                                    LPMAPIPROP pUserList,
                                    nsIImportFieldMap *pFieldMap)
@@ -621,7 +603,7 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
   nsCOMPtr <nsIMdbRow> newListRow;
   rv = pDb->GetNewListRow(getter_AddRefs(newListRow));
   NS_ENSURE_SUCCESS(rv, rv);
-  nsCAutoString column;
+  nsAutoCString column;
   LossyCopyUTF16toASCII(nsDependentString(pName), column);
   rv = pDb->AddListName(newListRow, column.get());
   NS_ENSURE_SUCCESS(rv, rv);
@@ -651,12 +633,12 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
 
   LPENTRYID    lpEid;
   ULONG        cbEid;
-  PRInt32        idx;
+  int32_t        idx;
   LPMESSAGE        lpMsg;
   nsCString        type;
   LPSPropValue    pVal;
   nsString        subject;
-  PRUint32 total;
+  uint32_t total;
 
   total=sa->cValues;
   for (idx = 0; idx < sa->cValues; idx++)
@@ -667,21 +649,21 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
     if (!m_mapi.OpenEntry(cbEid, lpEid, (LPUNKNOWN *) &lpMsg))
     {
 
-      IMPORT_LOG1( "*** Error opening messages in mailbox: %S\n", pName);
+      IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName);
       m_mapi.MAPIFreeBuffer(value);
-      return( NS_ERROR_FAILURE);
+      return NS_ERROR_FAILURE;
     }
     // This is a contact, add it to the address book!
     subject.Truncate();
-    pVal = m_mapi.GetMapiProperty( lpMsg, PR_SUBJECT);
+    pVal = m_mapi.GetMapiProperty(lpMsg, PR_SUBJECT);
     if (pVal)
-      m_mapi.GetStringFromProp( pVal, subject);
+      m_mapi.GetStringFromProp(pVal, subject);
 
     nsCOMPtr <nsIMdbRow> newRow;
     nsCOMPtr <nsIMdbRow> oldRow;
-    pDb->GetNewRow( getter_AddRefs(newRow));
+    pDb->GetNewRow(getter_AddRefs(newRow));
     if (newRow) {
-      if (BuildCard( subject.get(), pDb, newRow, lpMsg, pFieldMap))
+      if (BuildCard(subject.get(), pDb, newRow, lpMsg, pFieldMap))
       {
         nsCOMPtr <nsIAbCard> userCard;
         nsCOMPtr <nsIAbCard> newCard;
@@ -693,18 +675,14 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
         bool bl=false;
         pDb->FindRowByCard(userCard,getter_AddRefs(oldRow));
         if (oldRow)
-        {
           newRow = oldRow;
-        }
         else
-        {
-          pDb->AddCardRowToDB( newRow);
-        }
+          pDb->AddCardRowToDB(newRow);
 
         //add card list
         pDb->AddListCardColumnsToRow(userCard,
                                      newListRow,idx+1, getter_AddRefs(newCard),
-                                     true, nsnull, nsnull);
+                                     true, nullptr, nullptr);
       }
     }
   }
@@ -718,31 +696,31 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
   return rv;
 }
 
-void nsOutlookMail::SanitizeValue( nsString& val)
+void nsOutlookMail::SanitizeValue(nsString& val)
 {
   MsgReplaceSubstring(val, NS_LITERAL_STRING("\r\n"), NS_LITERAL_STRING(", "));
   MsgReplaceChar(val, "\r\n", ',');
 }
 
-void nsOutlookMail::SplitString( nsString& val1, nsString& val2)
+void nsOutlookMail::SplitString(nsString& val1, nsString& val2)
 {
   // Find the last line if there is more than one!
-  PRInt32 idx = val1.RFind( "\x0D\x0A");
-  PRInt32  cnt = 2;
+  int32_t idx = val1.RFind("\x0D\x0A");
+  int32_t  cnt = 2;
   if (idx == -1) {
     cnt = 1;
-    idx = val1.RFindChar( 13);
+    idx = val1.RFindChar(13);
   }
   if (idx == -1)
-    idx= val1.RFindChar( 10);
+    idx= val1.RFindChar(10);
   if (idx != -1) {
     val2 = Substring(val1, idx + cnt);
     val1.SetLength(idx);
-    SanitizeValue( val1);
+    SanitizeValue(val1);
   }
 }
 
-bool nsOutlookMail::BuildCard( const PRUnichar *pName, nsIAddrDatabase *pDb, nsIMdbRow *newRow, LPMAPIPROP pUser, nsIImportFieldMap *pFieldMap)
+bool nsOutlookMail::BuildCard(const PRUnichar *pName, nsIAddrDatabase *pDb, nsIMdbRow *newRow, LPMAPIPROP pUser, nsIImportFieldMap *pFieldMap)
 {
 
   nsString    lastName;
@@ -753,57 +731,57 @@ bool nsOutlookMail::BuildCard( const PRUnichar *pName, nsIAddrDatabase *pDb, nsI
   nsString    secondEMail;
   ULONG       emailTag;
 
-  LPSPropValue  pProp = m_mapi.GetMapiProperty( pUser, PR_EMAIL_ADDRESS);
+  LPSPropValue  pProp = m_mapi.GetMapiProperty(pUser, PR_EMAIL_ADDRESS);
   if (!pProp) {
     emailTag = m_mapi.GetEmailPropertyTag(pUser, OUTLOOK_EMAIL1_MAPI_ID1);
     if (emailTag) {
-      pProp = m_mapi.GetMapiProperty( pUser, emailTag);
+      pProp = m_mapi.GetMapiProperty(pUser, emailTag);
     }
   }
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, eMail);
-    SanitizeValue( eMail);
+    m_mapi.GetStringFromProp(pProp, eMail);
+    SanitizeValue(eMail);
   }
 
   // for secondary email
   emailTag = m_mapi.GetEmailPropertyTag(pUser, OUTLOOK_EMAIL2_MAPI_ID1);
   if (emailTag) {
-    pProp = m_mapi.GetMapiProperty( pUser, emailTag);
+    pProp = m_mapi.GetMapiProperty(pUser, emailTag);
     if (pProp) {
-      m_mapi.GetStringFromProp( pProp, secondEMail);
-      SanitizeValue( secondEMail);
+      m_mapi.GetStringFromProp(pProp, secondEMail);
+      SanitizeValue(secondEMail);
     }
   }
 
-  pProp = m_mapi.GetMapiProperty( pUser, PR_GIVEN_NAME);
+  pProp = m_mapi.GetMapiProperty(pUser, PR_GIVEN_NAME);
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, firstName);
-    SanitizeValue( firstName);
+    m_mapi.GetStringFromProp(pProp, firstName);
+    SanitizeValue(firstName);
   }
-  pProp = m_mapi.GetMapiProperty( pUser, PR_SURNAME);
+  pProp = m_mapi.GetMapiProperty(pUser, PR_SURNAME);
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, lastName);
-    SanitizeValue( lastName);
+    m_mapi.GetStringFromProp(pProp, lastName);
+    SanitizeValue(lastName);
   }
-  pProp = m_mapi.GetMapiProperty( pUser, PR_MIDDLE_NAME);
+  pProp = m_mapi.GetMapiProperty(pUser, PR_MIDDLE_NAME);
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, middleName);
-    SanitizeValue( middleName);
+    m_mapi.GetStringFromProp(pProp, middleName);
+    SanitizeValue(middleName);
   }
-  pProp = m_mapi.GetMapiProperty( pUser, PR_NICKNAME);
+  pProp = m_mapi.GetMapiProperty(pUser, PR_NICKNAME);
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, nickName);
-    SanitizeValue( nickName);
+    m_mapi.GetStringFromProp(pProp, nickName);
+    SanitizeValue(nickName);
   }
   if (firstName.IsEmpty() && lastName.IsEmpty()) {
     firstName = pName;
   }
 
   nsString  displayName;
-  pProp = m_mapi.GetMapiProperty( pUser, PR_DISPLAY_NAME);
+  pProp = m_mapi.GetMapiProperty(pUser, PR_DISPLAY_NAME);
   if (pProp) {
-    m_mapi.GetStringFromProp( pProp, displayName);
-    SanitizeValue( displayName);
+    m_mapi.GetStringFromProp(pProp, displayName);
+    SanitizeValue(displayName);
   }
   if (displayName.IsEmpty()) {
     if (firstName.IsEmpty())
@@ -811,12 +789,12 @@ bool nsOutlookMail::BuildCard( const PRUnichar *pName, nsIAddrDatabase *pDb, nsI
     else {
       displayName = firstName;
       if (!middleName.IsEmpty()) {
-        displayName.Append( PRUnichar(' '));
-        displayName.Append( middleName);
+        displayName.Append(PRUnichar(' '));
+        displayName.Append(middleName);
       }
       if (!lastName.IsEmpty()) {
-        displayName.Append( PRUnichar(' '));
-        displayName.Append( lastName);
+        displayName.Append(PRUnichar(' '));
+        displayName.Append(lastName);
       }
     }
   }
@@ -824,22 +802,22 @@ bool nsOutlookMail::BuildCard( const PRUnichar *pName, nsIAddrDatabase *pDb, nsI
   // We now have the required fields
   // write them out followed by any optional fields!
   if (!displayName.IsEmpty()) {
-    pDb->AddDisplayName( newRow, NS_ConvertUTF16toUTF8(displayName).get());
+    pDb->AddDisplayName(newRow, NS_ConvertUTF16toUTF8(displayName).get());
   }
   if (!firstName.IsEmpty()) {
-    pDb->AddFirstName( newRow, NS_ConvertUTF16toUTF8(firstName).get());
+    pDb->AddFirstName(newRow, NS_ConvertUTF16toUTF8(firstName).get());
   }
   if (!lastName.IsEmpty()) {
-    pDb->AddLastName( newRow, NS_ConvertUTF16toUTF8(lastName).get());
+    pDb->AddLastName(newRow, NS_ConvertUTF16toUTF8(lastName).get());
   }
   if (!nickName.IsEmpty()) {
-    pDb->AddNickName( newRow, NS_ConvertUTF16toUTF8(nickName).get());
+    pDb->AddNickName(newRow, NS_ConvertUTF16toUTF8(nickName).get());
   }
   if (!eMail.IsEmpty()) {
-    pDb->AddPrimaryEmail( newRow, NS_ConvertUTF16toUTF8(eMail).get());
+    pDb->AddPrimaryEmail(newRow, NS_ConvertUTF16toUTF8(eMail).get());
   }
   if (!secondEMail.IsEmpty()) {
-    pDb->Add2ndEmail( newRow, NS_ConvertUTF16toUTF8(secondEMail).get());
+    pDb->Add2ndEmail(newRow, NS_ConvertUTF16toUTF8(secondEMail).get());
   }
 
   // Do all of the extra fields!
@@ -848,31 +826,31 @@ bool nsOutlookMail::BuildCard( const PRUnichar *pName, nsIAddrDatabase *pDb, nsI
   nsString  line2;
 
   if (pFieldMap) {
-    int max = sizeof( gMapiFields) / sizeof( MAPIFields);
+    int max = sizeof(gMapiFields) / sizeof(MAPIFields);
     for (int i = 0; i < max; i++) {
-      pProp = m_mapi.GetMapiProperty( pUser, gMapiFields[i].mapiTag);
+      pProp = m_mapi.GetMapiProperty(pUser, gMapiFields[i].mapiTag);
       if (pProp) {
-        m_mapi.GetStringFromProp( pProp, value);
+        m_mapi.GetStringFromProp(pProp, value);
         if (!value.IsEmpty()) {
           if (gMapiFields[i].multiLine == kNoMultiLine) {
-            SanitizeValue( value);
-            pFieldMap->SetFieldValue( pDb, newRow, gMapiFields[i].mozField, value.get());
+            SanitizeValue(value);
+            pFieldMap->SetFieldValue(pDb, newRow, gMapiFields[i].mozField, value.get());
           }
           else if (gMapiFields[i].multiLine == kIsMultiLine) {
-            pFieldMap->SetFieldValue( pDb, newRow, gMapiFields[i].mozField, value.get());
+            pFieldMap->SetFieldValue(pDb, newRow, gMapiFields[i].mozField, value.get());
           }
           else {
             line2.Truncate();
-            SplitString( value, line2);
+            SplitString(value, line2);
             if (!value.IsEmpty())
-              pFieldMap->SetFieldValue( pDb, newRow, gMapiFields[i].mozField, value.get());
+              pFieldMap->SetFieldValue(pDb, newRow, gMapiFields[i].mozField, value.get());
             if (!line2.IsEmpty())
-              pFieldMap->SetFieldValue( pDb, newRow, gMapiFields[i].multiLine, line2.get());
+              pFieldMap->SetFieldValue(pDb, newRow, gMapiFields[i].multiLine, line2.get());
           }
         }
       }
     }
   }
 
-  return( true);
+  return true;
 }
