@@ -14,7 +14,7 @@
 #include "nsIImportABDescriptor.h"
 #include "nsIImportFieldMap.h"
 #include "nsStringGlue.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIAddrDatabase.h"
 #include "nsIAbManager.h"
 #include "nsIAbLDIFService.h"
@@ -48,7 +48,7 @@ public:
 
   NS_IMETHOD SetData(const char *dataId, nsISupports *pData);
 
-  NS_IMETHOD GetStatus(const char *statusKind, PRInt32 *_retval);
+  NS_IMETHOD GetStatus(const char *statusKind, int32_t *_retval);
 
   NS_IMETHOD WantsProgress(bool *_retval);
 
@@ -56,7 +56,7 @@ public:
 
   NS_IMETHOD ContinueImport(bool *_retval);
 
-  NS_IMETHOD GetProgress(PRInt32 *_retval);
+  NS_IMETHOD GetProgress(int32_t *_retval);
 
   NS_IMETHOD CancelImport(void);
 
@@ -83,7 +83,7 @@ private:
   bool              m_userVerify;
   nsISupportsString *    m_pSuccessLog;
   nsISupportsString *    m_pErrorLog;
-  PRUint32          m_totalSize;
+  uint32_t          m_totalSize;
   bool              m_doImport;
   AddressThreadData *      m_pThreadData;
   char *            m_pDestinationUri;
@@ -96,8 +96,8 @@ public:
   bool              threadAlive;
   bool              abort;
   bool              fatalError;
-  PRUint32          currentTotal;
-  PRUint32          currentSize;
+  uint32_t          currentTotal;
+  uint32_t          currentSize;
   nsISupportsArray *      books;
   nsCOMArray<nsIAddrDatabase>* dBs;
   nsCOMPtr<nsIAbLDIFService> ldifService;
@@ -115,13 +115,13 @@ public:
 
 nsresult NS_NewGenericAddressBooks(nsIImportGeneric** aImportGeneric)
 {
-    NS_PRECONDITION(aImportGeneric != nsnull, "null ptr");
+    NS_PRECONDITION(aImportGeneric != nullptr, "null ptr");
     if (! aImportGeneric)
         return NS_ERROR_NULL_POINTER;
 
   nsImportGenericAddressBooks *pGen = new nsImportGenericAddressBooks();
 
-  if (pGen == nsnull)
+  if (pGen == nullptr)
     return NS_ERROR_OUT_OF_MEMORY;
 
   NS_ADDREF(pGen);
@@ -133,18 +133,18 @@ nsresult NS_NewGenericAddressBooks(nsIImportGeneric** aImportGeneric)
 
 nsImportGenericAddressBooks::nsImportGenericAddressBooks()
 {
-  m_pInterface = nsnull;
-  m_pBooks = nsnull;
-  m_pSuccessLog = nsnull;
-  m_pErrorLog = nsnull;
+  m_pInterface = nullptr;
+  m_pBooks = nullptr;
+  m_pSuccessLog = nullptr;
+  m_pErrorLog = nullptr;
   m_totalSize = 0;
   m_doImport = false;
-  m_pThreadData = nsnull;
-  m_pDestinationUri = nsnull;
-  m_pFieldMap = nsnull;
+  m_pThreadData = nullptr;
+  m_pDestinationUri = nullptr;
+  m_pFieldMap = nullptr;
 
   m_autoFind = false;
-  m_description = nsnull;
+  m_description = nullptr;
   m_gotLocation = false;
   m_found = false;
   m_userVerify = false;
@@ -175,12 +175,12 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsImportGenericAddressBooks, nsIImportGeneric)
 
 NS_IMETHODIMP nsImportGenericAddressBooks::GetData(const char *dataId, nsISupports **_retval)
 {
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
   if (!_retval)
     return NS_ERROR_NULL_POINTER;
 
   nsresult rv;
-  *_retval = nsnull;
+  *_retval = nullptr;
   if (!PL_strcasecmp(dataId, "addressInterface")) {
     *_retval = m_pInterface;
     NS_IF_ADDREF(m_pInterface);
@@ -233,7 +233,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetData(const char *dataId, nsISuppor
   if (!PL_strncasecmp(dataId, "sampleData-", 11)) {
     // extra the record number
     const char *pNum = dataId + 11;
-    PRInt32  rNum = 0;
+    int32_t  rNum = 0;
     while (*pNum) {
       rNum *= 10;
       rNum += (*pNum - '0');
@@ -244,7 +244,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetData(const char *dataId, nsISuppor
       nsCOMPtr<nsISupportsString>  data = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
       if (NS_FAILED(rv))
         return rv;
-      PRUnichar *  pData = nsnull;
+      PRUnichar *  pData = nullptr;
       bool      found = false;
       rv = m_pInterface->GetSampleData(rNum, &found, &pData);
       if (NS_FAILED(rv))
@@ -264,7 +264,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetData(const char *dataId, nsISuppor
 
 NS_IMETHODIMP nsImportGenericAddressBooks::SetData(const char *dataId, nsISupports *item)
 {
-  NS_PRECONDITION(dataId != nsnull, "null ptr");
+  NS_PRECONDITION(dataId != nullptr, "null ptr");
   if (!dataId)
     return NS_ERROR_NULL_POINTER;
 
@@ -280,7 +280,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::SetData(const char *dataId, nsISuppor
   }
 
   if (!PL_strcasecmp(dataId, "addressLocation")) {
-    m_pLocation = nsnull;
+    m_pLocation = nullptr;
 
     if (item) {
       nsresult rv;
@@ -299,8 +299,8 @@ NS_IMETHODIMP nsImportGenericAddressBooks::SetData(const char *dataId, nsISuppor
       if (abString) {
         if (m_pDestinationUri)
           NS_Free(m_pDestinationUri);
-        m_pDestinationUri = nsnull;
-                nsCAutoString tempUri;
+        m_pDestinationUri = nullptr;
+                nsAutoCString tempUri;
                 abString->GetData(tempUri);
                 m_pDestinationUri = ToNewCString(tempUri);
       }
@@ -316,10 +316,10 @@ NS_IMETHODIMP nsImportGenericAddressBooks::SetData(const char *dataId, nsISuppor
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImportGenericAddressBooks::GetStatus(const char *statusKind, PRInt32 *_retval)
+NS_IMETHODIMP nsImportGenericAddressBooks::GetStatus(const char *statusKind, int32_t *_retval)
 {
-  NS_PRECONDITION(statusKind != nsnull, "null ptr");
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
+  NS_PRECONDITION(statusKind != nullptr, "null ptr");
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
   if (!statusKind || !_retval)
     return NS_ERROR_NULL_POINTER;
 
@@ -327,31 +327,31 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetStatus(const char *statusKind, PRI
 
   if (!PL_strcasecmp(statusKind, "isInstalled")) {
     GetDefaultLocation();
-    *_retval = (PRInt32) m_found;
+    *_retval = (int32_t) m_found;
   }
 
   if (!PL_strcasecmp(statusKind, "canUserSetLocation")) {
     GetDefaultLocation();
-    *_retval = (PRInt32) m_userVerify;
+    *_retval = (int32_t) m_userVerify;
   }
 
   if (!PL_strcasecmp(statusKind, "autoFind")) {
     GetDefaultLocation();
-    *_retval = (PRInt32) m_autoFind;
+    *_retval = (int32_t) m_autoFind;
   }
 
   if (!PL_strcasecmp(statusKind, "supportsMultiple")) {
     bool      multi = false;
     if (m_pInterface)
       m_pInterface->GetSupportsMultiple(&multi);
-    *_retval = (PRInt32) multi;
+    *_retval = (int32_t) multi;
   }
 
   if (!PL_strcasecmp(statusKind, "needsFieldMap")) {
     bool      needs = false;
     if (m_pInterface && m_pLocation)
       m_pInterface->GetNeedsFieldMap(m_pLocation, &needs);
-    *_retval = (PRInt32) needs;
+    *_retval = (int32_t) needs;
   }
 
   return NS_OK;
@@ -367,7 +367,7 @@ void nsImportGenericAddressBooks::GetDefaultLocation(void)
 
   if (m_description)
     NS_Free(m_description);
-  m_description = nsnull;
+  m_description = nullptr;
   m_pInterface->GetAutoFind(&m_description, &m_autoFind);
   m_gotLocation = true;
   if (m_autoFind) {
@@ -414,7 +414,7 @@ void nsImportGenericAddressBooks::GetDefaultFieldMap(void)
   if (NS_FAILED(rv))
     return;
 
-  PRInt32  sz = 0;
+  int32_t  sz = 0;
   rv = m_pFieldMap->GetNumMozFields(&sz);
   if (NS_SUCCEEDED(rv))
     rv = m_pFieldMap->DefaultFieldMap(sz);
@@ -429,32 +429,31 @@ void nsImportGenericAddressBooks::GetDefaultFieldMap(void)
 
 NS_IMETHODIMP nsImportGenericAddressBooks::WantsProgress(bool *_retval)
 {
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
-    if (!_retval)
-        return NS_ERROR_NULL_POINTER;
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
+  NS_ENSURE_ARG_POINTER(_retval);
 
   GetDefaultLocation();
   GetDefaultBooks();
 
-  PRUint32    totalSize = 0;
-  bool        result = false;
+  bool result = false;
 
   if (m_pBooks) {
-    PRUint32    count = 0;
-    nsresult     rv = m_pBooks->Count(&count);
-    PRUint32    i;
+    uint32_t    count = 0;
+    uint32_t    i;
     bool        import;
-    PRUint32    size;
+    uint32_t    size;
+    uint32_t    totalSize = 0;
+
+    (void) m_pBooks->Count(&count);
 
     for (i = 0; i < count; i++) {
-      nsCOMPtr<nsIImportABDescriptor> book =
-        do_QueryElementAt(m_pBooks, i);
+      nsCOMPtr<nsIImportABDescriptor> book = do_QueryElementAt(m_pBooks, i);
       if (book) {
         import = false;
         size = 0;
-        rv = book->GetImport(&import);
-        if (import) {
-          rv = book->GetSize(&size);
+        nsresult rv = book->GetImport(&import);
+        if (NS_SUCCEEDED(rv) && import) {
+          (void) book->GetSize(&size);
           result = true;
         }
         totalSize += size;
@@ -489,23 +488,23 @@ void nsImportGenericAddressBooks::SetLogs(nsString& success, nsString& error, ns
 already_AddRefed<nsIAddrDatabase> GetAddressBookFromUri(const char *pUri)
 {
   if (!pUri)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID);
   if (!abManager)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr<nsIAbDirectory> directory;
   abManager->GetDirectory(nsDependentCString(pUri),
                           getter_AddRefs(directory));
   if (!directory)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr<nsIAbMDBDirectory> mdbDirectory = do_QueryInterface(directory);
   if (!mdbDirectory)
-    return nsnull;
+    return nullptr;
 
-  nsIAddrDatabase *pDatabase = nsnull;
+  nsIAddrDatabase *pDatabase = nullptr;
   mdbDirectory->GetDatabase(&pDatabase);
   return pDatabase;
 }
@@ -522,8 +521,8 @@ already_AddRefed<nsIAddrDatabase> GetAddressBook(const PRUnichar *name,
   IMPORT_LOG0("In GetAddressBook\n");
 
   nsresult rv;
-  nsIAddrDatabase *pDatabase = nsnull;
-  nsCOMPtr<nsILocalFile> dbPath;
+  nsIAddrDatabase *pDatabase = nullptr;
+  nsCOMPtr<nsIFile> dbPath;
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv))
   {
@@ -545,7 +544,7 @@ already_AddRefed<nsIAddrDatabase> GetAddressBook(const PRUnichar *name,
           nsCOMPtr<nsIAddrDatabase> addrDBFactory =
             do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
           if (NS_FAILED(rv))
-            return nsnull;
+            return nullptr;
 
           IMPORT_LOG0("Opening the new address book\n");
           rv = addrDBFactory->Open(dbPath, true, true,
@@ -570,8 +569,8 @@ already_AddRefed<nsIAddrDatabase> GetAddressBook(const PRUnichar *name,
                             getter_AddRefs(parentDir));
     if (parentDir)
     {
-      nsCAutoString URI("moz-abmdbdirectory://");
-      nsCAutoString leafName;
+      nsAutoCString URI("moz-abmdbdirectory://");
+      nsAutoCString leafName;
       rv = dbPath->GetNativeLeafName(leafName);
       if (NS_FAILED(rv))
         IMPORT_LOG0("*** Error: Unable to get name of database file\n");
@@ -595,7 +594,7 @@ already_AddRefed<nsIAddrDatabase> GetAddressBook(const PRUnichar *name,
 
 NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, bool *_retval)
 {
-  NS_PRECONDITION(_retval != nsnull, "null ptr");
+  NS_PRECONDITION(_retval != nullptr, "null ptr");
     if (!_retval)
         return NS_ERROR_NULL_POINTER;
 
@@ -654,13 +653,13 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
   if (m_pDestinationUri)
     m_pThreadData->pDestinationUri = strdup(m_pDestinationUri);
 
-  PRUint32 count = 0;
+  uint32_t count = 0;
   m_pBooks->Count(&count);
   // Create/obtain any address books that we need here, so that we don't need
   // to do so inside the import thread which would just proxy the create
   // operations back to the main thread anyway.
   nsCOMPtr<nsIAddrDatabase> db = GetAddressBookFromUri(m_pDestinationUri);
-  for (PRUint32 i = 0; i < count; ++i)
+  for (uint32_t i = 0; i < count; ++i)
   {
     nsCOMPtr<nsIImportABDescriptor> book = do_QueryElementAt(m_pBooks, i);
     if (book)
@@ -683,7 +682,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
 
   ImportAddressThread(m_pThreadData);
   delete m_pThreadData;
-  m_pThreadData = nsnull;
+  m_pThreadData = nullptr;
   *_retval = true;
 
   return NS_OK;
@@ -691,7 +690,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
 
 NS_IMETHODIMP nsImportGenericAddressBooks::ContinueImport(bool *_retval)
 {
-    NS_PRECONDITION(_retval != nsnull, "null ptr");
+    NS_PRECONDITION(_retval != nullptr, "null ptr");
     if (!_retval)
         return NS_ERROR_NULL_POINTER;
 
@@ -705,11 +704,11 @@ NS_IMETHODIMP nsImportGenericAddressBooks::ContinueImport(bool *_retval)
 }
 
 
-NS_IMETHODIMP nsImportGenericAddressBooks::GetProgress(PRInt32 *_retval)
+NS_IMETHODIMP nsImportGenericAddressBooks::GetProgress(int32_t *_retval)
 {
   // This returns the progress from the the currently
   // running import mail or import address book thread.
-    NS_PRECONDITION(_retval != nsnull, "null ptr");
+    NS_PRECONDITION(_retval != nullptr, "null ptr");
     if (!_retval)
         return NS_ERROR_NULL_POINTER;
 
@@ -718,7 +717,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetProgress(PRInt32 *_retval)
     return NS_OK;
   }
 
-  PRUint32 sz = 0;
+  uint32_t sz = 0;
   if (m_pThreadData->currentSize && m_pInterface) {
     if (NS_FAILED(m_pInterface->GetImportProgress(&sz)))
       sz = 0;
@@ -746,7 +745,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::CancelImport(void)
 {
   if (m_pThreadData) {
     m_pThreadData->abort = true;
-    m_pThreadData = nsnull;
+    m_pThreadData = nullptr;
   }
 
   return NS_OK;
@@ -761,14 +760,14 @@ AddressThreadData::AddressThreadData()
   abort = false;
   currentTotal = 0;
   currentSize = 0;
-  books = nsnull;
-  addressImport = nsnull;
-  successLog = nsnull;
-  errorLog = nsnull;
-  pDestinationUri = nsnull;
-  fieldMap = nsnull;
-  stringBundle = nsnull;
-  ldifService = nsnull;
+  books = nullptr;
+  addressImport = nullptr;
+  successLog = nullptr;
+  errorLog = nullptr;
+  pDestinationUri = nullptr;
+  fieldMap = nullptr;
+  stringBundle = nullptr;
+  ldifService = nullptr;
 }
 
 AddressThreadData::~AddressThreadData()
@@ -804,26 +803,28 @@ static void ImportAddressThread(void *stuff)
   IMPORT_LOG0("In Begin ImportAddressThread\n");
 
   AddressThreadData *pData = (AddressThreadData *)stuff;
-  PRUint32  count = 0;
-  nsresult   rv = pData->books->Count(&count);
-  PRUint32          i;
+  uint32_t          count = 0;
+  uint32_t          i;
   bool              import;
-  PRUint32          size;
+  uint32_t          size;
 
   nsString          success;
   nsString          error;
 
+  (void) pData->books->Count(&count);
+
   for (i = 0; (i < count) && !(pData->abort); i++) {
     nsCOMPtr<nsIImportABDescriptor> book =
       do_QueryElementAt(pData->books, i);
+
     if (book) {
       import = false;
       size = 0;
-      rv = book->GetImport(&import);
-      if (import)
+      nsresult rv = book->GetImport(&import);
+      if (NS_SUCCEEDED(rv) && import)
         rv = book->GetSize(&size);
 
-      if (size && import) {
+      if (NS_SUCCEEDED(rv) && size && import) {
         nsString name;
         book->GetPreferredName(name);
 
@@ -832,17 +833,17 @@ static void ImportAddressThread(void *stuff)
         bool fatalError = false;
         pData->currentSize = size;
         if (db) {
-          PRUnichar *pSuccess = nsnull;
-          PRUnichar *pError = nsnull;
+          PRUnichar *pSuccess = nullptr;
+          PRUnichar *pError = nullptr;
 
           /*
           if (pData->fieldMap) {
-            PRInt32    sz = 0;
-            PRInt32    mapIndex;
+            int32_t    sz = 0;
+            int32_t    mapIndex;
             bool      active;
             pData->fieldMap->GetMapSize(&sz);
             IMPORT_LOG1("**** Field Map Size: %d\n", (int) sz);
-            for (PRInt32 i = 0; i < sz; i++) {
+            for (int32_t i = 0; i < sz; i++) {
               pData->fieldMap->GetFieldMap(i, &mapIndex);
               pData->fieldMap->GetFieldActive(i, &active);
               IMPORT_LOG3("Field map #%d: index=%d, active=%d\n", (int) i, (int) mapIndex, (int) active);
@@ -857,7 +858,7 @@ static void ImportAddressThread(void *stuff)
                                                        &pError,
                                                        &pSuccess,
                                                        &fatalError);
-          if (pSuccess) {
+          if (NS_SUCCEEDED(rv) && pSuccess) {
             success.Append(pSuccess);
             NS_Free(pSuccess);
           }
