@@ -13,6 +13,9 @@
 #include "nsNetUtil.h"
 #include "nsMsgUtils.h"
 #include "mdb.h"
+#include "nsIConverterInputStream.h"
+#include "nsIUnicharLineInputStream.h"
+#include "nsMsgUtils.h"
 
 #include "TextDebugLog.h"
 #include "plstr.h"
@@ -26,13 +29,10 @@
 
 #define kWhitespace    " \t\b\r\n"
 
-// If we get a line longer than 32K it's just toooooo bad!
-#define kTextAddressBufferSz    (64 * 1024)
-
 nsTextAddress::nsTextAddress()
 {
-    m_database = nsnull;
-    m_fieldMap = nsnull;
+    m_database = nullptr;
+    m_fieldMap = nullptr;
     m_LFCount = 0;
     m_CRCount = 0;
 }
@@ -43,7 +43,30 @@ nsTextAddress::~nsTextAddress()
     NS_IF_RELEASE(m_fieldMap);
 }
 
-nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, nsIFile *pSrc, nsIAddrDatabase *pDb, nsIImportFieldMap *fieldMap, nsString& errors, PRUint32 *pProgress)
+nsresult nsTextAddress::GetUnicharLineStreamForFile(nsIFile *aFile,
+                                                    nsIInputStream *aInputStream,
+                                                    nsIUnicharLineInputStream **aStream)
+{
+  nsAutoCString charset;
+  nsresult rv = MsgDetectCharsetFromFile(aFile, charset);
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0( "*** Error checking address file for charset detection\n");
+    return rv;
+  }
+
+  nsCOMPtr<nsIConverterInputStream> converterStream =
+    do_CreateInstance("@mozilla.org/intl/converter-input-stream;1", &rv);
+  if (NS_SUCCEEDED(rv)) {
+    rv = converterStream->Init(aInputStream,
+                               charset.get(),
+                               8192,
+                               nsIConverterInputStream::DEFAULT_REPLACEMENT_CHARACTER);
+  }
+
+  return CallQueryInterface(converterStream, aStream);
+}
+
+nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, nsIFile *pSrc, nsIAddrDatabase *pDb, nsIImportFieldMap *fieldMap, nsString& errors, uint32_t *pProgress)
 {
   // Open the source file for reading, read each line and process it!
   NS_IF_RELEASE(m_database);
@@ -63,7 +86,7 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
   // Here we use this to work out the size of the file, so we can update
   // an integer as we go through the file which will update a progress
   // bar if required by the caller.
-  PRUint32 bytesLeft = 0;
+  uint64_t bytesLeft = 0;
   rv = inputStream->Available(&bytesLeft);
   if (NS_FAILED(rv)) {
     IMPORT_LOG0("*** Error checking address file for size\n");
@@ -71,7 +94,7 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
     return rv;
   }
 
-  PRUint32 totalBytes = bytesLeft;
+  uint64_t totalBytes = bytesLeft;
   bool skipRecord = false;
 
   rv = m_fieldMap->GetSkipFirstRecord(&skipRecord);
@@ -80,11 +103,15 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
     return rv;
   }
 
-  nsCOMPtr<nsILineInputStream> lineStream(do_QueryInterface(inputStream, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIUnicharLineInputStream> lineStream;
+  rv = GetUnicharLineStreamForFile(pSrc, inputStream, getter_AddRefs(lineStream));
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0("*** Error opening converter stream for importer\n");
+    return rv;
+  }
 
   bool more = true;
-  nsCString line;
+  nsAutoString line;
 
   // Skip the first record if the user has requested it.
   if (skipRecord)
@@ -95,7 +122,7 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
     rv = ReadRecord(lineStream, line, &more);
     if (NS_SUCCEEDED(rv)) {
       // Now proces it to add it to the database
-      rv = ProcessLine(line.get(), line.Length(), errors);
+      rv = ProcessLine(line, errors);
 
       if (NS_FAILED(rv)) {
         IMPORT_LOG0("*** Error processing text record.\n");
@@ -106,7 +133,7 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
       // considering that lineStream won't give us how many bytes
       // are actually left.
       bytesLeft -= line.Length();
-      *pProgress = totalBytes - bytesLeft;
+      *pProgress = NS_MIN(totalBytes - bytesLeft, PR_UINT32_MAX);
     }
   }
 
@@ -120,12 +147,14 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const PRUnichar *pName, ns
   return pDb->Commit(nsAddrDBCommitType::kLargeCommit);
 }
 
-nsresult nsTextAddress::ReadRecord(nsILineInputStream *aLineStream, nsCString &aLine, bool *aMore)
+nsresult nsTextAddress::ReadRecord(nsIUnicharLineInputStream *aLineStream,
+                                   nsAString &aLine,
+                                   bool *aMore)
 {
   bool more = true;
-  PRUint32 numQuotes = 0;
+  uint32_t numQuotes = 0;
   nsresult rv;
-  nsCString line;
+  nsAutoString line;
 
   // ensure aLine is empty
   aLine.Truncate();
@@ -143,7 +172,7 @@ nsresult nsTextAddress::ReadRecord(nsILineInputStream *aLineStream, nsCString &a
           aLine.AppendLiteral(MSG_LINEBREAK);
         aLine.Append(line);
 
-        numQuotes += MsgCountChar(line, '"');
+        numQuotes += MsgCountChar(line, PRUnichar('"'));
       }
     }
     // Continue whilst everything is ok, and we have an odd number of quotes.
@@ -153,7 +182,7 @@ nsresult nsTextAddress::ReadRecord(nsILineInputStream *aLineStream, nsCString &a
   return rv;
 }
 
-nsresult nsTextAddress::ReadRecordNumber(nsIFile *aSrc, nsCString &aLine, PRInt32 rNum)
+nsresult nsTextAddress::ReadRecordNumber(nsIFile *aSrc, nsAString &aLine, int32_t rNum)
 {
   nsCOMPtr<nsIInputStream> inputStream;
   nsresult rv = NS_NewLocalFileInputStream(getter_AddRefs(inputStream), aSrc);
@@ -162,8 +191,8 @@ nsresult nsTextAddress::ReadRecordNumber(nsIFile *aSrc, nsCString &aLine, PRInt3
     return rv;
   }
 
-  PRInt32 rIndex = 0;
-  PRUint32 bytesLeft = 0;
+  int32_t rIndex = 0;
+  uint64_t bytesLeft = 0;
 
   rv = inputStream->Available(&bytesLeft);
   if (NS_FAILED(rv)) {
@@ -172,8 +201,12 @@ nsresult nsTextAddress::ReadRecordNumber(nsIFile *aSrc, nsCString &aLine, PRInt3
     return rv;
   }
 
-  nsCOMPtr<nsILineInputStream> lineStream(do_QueryInterface(inputStream, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIUnicharLineInputStream> lineStream;
+  rv = GetUnicharLineStreamForFile(aSrc, inputStream, getter_AddRefs(lineStream));
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0("*** Error opening converter stream for importer\n");
+    return rv;
+  }
 
   bool more = true;
 
@@ -194,132 +227,122 @@ nsresult nsTextAddress::ReadRecordNumber(nsIFile *aSrc, nsCString &aLine, PRInt3
   return NS_ERROR_FAILURE;
 }
 
-PRInt32 nsTextAddress::CountFields(const char *pLine, PRInt32 maxLen, char delim)
+int32_t nsTextAddress::CountFields(const nsAString &aLine, PRUnichar delim)
 {
-    const char *pChar = pLine;
-    PRInt32 len = 0;
-    PRInt32 count = 0;
-    char tab = '\t';
+    int32_t pos = 0;
+    int32_t maxLen = aLine.Length();
+    int32_t count = 0;
+    PRUnichar tab = PRUnichar('\t');
+    PRUnichar doubleQuote = PRUnichar('"');
 
     if (delim == tab)
-        tab = 0;
+        tab = PRUnichar('\0');
 
-    while (len < maxLen) {
-        while (((*pChar == ' ') || (*pChar == tab)) && (len < maxLen)) {
-            pChar++;
-            len++;
+    while (pos < maxLen) {
+        while (((aLine[pos] == PRUnichar(' ')) || (aLine[pos] == tab)) &&
+               (pos < maxLen)) {
+            pos++;
         }
-        if ((len < maxLen) && (*pChar == '"')) {
-            pChar++;
-            len++;
-            while ((len < maxLen) && (*pChar != '"')) {
-                len++;
-                pChar++;
-                if (((len + 1) < maxLen) && (*pChar == '"') && (*(pChar + 1) == '"')) {
-                    len += 2;
-                    pChar += 2;
+        if ((pos < maxLen) && (aLine[pos] == doubleQuote)) {
+            pos++;
+            while ((pos < maxLen) && (aLine[pos] != doubleQuote)) {
+                pos++;
+                if (((pos + 1) < maxLen) &&
+                    (aLine[pos] == doubleQuote) &&
+                    (aLine[pos + 1] == doubleQuote)) {
+                    pos += 2;
                 }
             }
-            if (len < maxLen) {
-                pChar++;
-                len++;
-            }
-        }
-        while ((len < maxLen) && (*pChar != delim)) {
-            len++;
-            pChar++;
+            if (pos < maxLen)
+                pos++;
         }
+        while ((pos < maxLen) && (aLine[pos] != delim))
+            pos++;
 
         count++;
-        pChar++;
-        len++;
+        pos++;
     }
 
     return count;
 }
 
-bool nsTextAddress::GetField(const char *pLine, PRInt32 maxLen, PRInt32 index, nsCString& field, char delim)
+bool nsTextAddress::GetField(const nsAString &aLine,
+                             int32_t index,
+                             nsString &field,
+                             PRUnichar delim)
 {
     bool result = false;
-    const char *pChar = pLine;
-    PRInt32 len = 0;
-    char tab = '\t';
+    int32_t pos = 0;
+    int32_t maxLen = aLine.Length();
+    PRUnichar tab = PRUnichar('\t');
+    PRUnichar doubleQuote = PRUnichar('"');
 
     field.Truncate();
 
     if (delim == tab)
         tab = 0;
 
-    while (index && (len < maxLen)) {
-        while (((*pChar == ' ') || (*pChar == tab)) && (len < maxLen)) {
-            pChar++;
-            len++;
+    while (index && (pos < maxLen)) {
+        while (((aLine[pos] == PRUnichar(' ')) || (aLine[pos] == tab)) &&
+               (pos < maxLen)) {
+            pos++;
         }
-        if (len >= maxLen)
+        if (pos >= maxLen)
             break;
-        if (*pChar == '"') {
+        if (aLine[pos] == doubleQuote) {
             do {
-                len++;
-                pChar++;
-                if (((len + 1) < maxLen) && (*pChar == '"') && (*(pChar + 1) == '"')) {
-                    len += 2;
-                    pChar += 2;
+                pos++;
+                if (((pos + 1) < maxLen) &&
+                    (aLine[pos] == doubleQuote) &&
+                    (aLine[pos + 1] == doubleQuote)) {
+                    pos += 2;
                 }
-            } while ((len < maxLen) && (*pChar != '"'));
-            if (len < maxLen) {
-                pChar++;
-                len++;
-            }
+            } while ((pos < maxLen) && (aLine[pos] != doubleQuote));
+            if (pos < maxLen)
+                pos++;
         }
-        if (len >= maxLen)
+        if (pos >= maxLen)
             break;
 
-        while ((len < maxLen) && (*pChar != delim)) {
-            len++;
-            pChar++;
-        }
+        while ((pos < maxLen) && (aLine[pos] != delim))
+            pos++;
 
-        if (len >= maxLen)
+        if (pos >= maxLen)
             break;
 
         index--;
-        pChar++;
-        len++;
+        pos++;
     }
 
-    if (len >= maxLen) {
+    if (pos >= maxLen)
         return result;
-    }
 
     result = true;
 
-    while ((len < maxLen) && ((*pChar == ' ') || (*pChar == tab))) {
-        len++;
-        pChar++;
-    }
+    while ((pos < maxLen) && ((aLine[pos] == ' ') || (aLine[pos] == tab)))
+        pos++;
 
-    const char *pStart = pChar;
-    PRInt32        fLen = 0;
-    bool          quoted = false;
-    if (*pChar == '"') {
-        pStart++;
+    int32_t fLen = 0;
+    int32_t startPos = pos;
+    bool    quoted = false;
+    if (aLine[pos] == '"') {
+        startPos++;
         fLen = -1;
         do {
-            pChar++;
-            len++;
+            pos++;
             fLen++;
-            if (((len + 1) < maxLen) && (*pChar == '"') && (*(pChar + 1) == '"')) {
+            if (((pos + 1) < maxLen) &&
+                (aLine[pos] == doubleQuote) &&
+                (aLine[pos + 1] == doubleQuote)) {
                 quoted = true;
-                len += 2;
-                pChar += 2;
+                pos += 2;
                 fLen += 2;
             }
-        } while ((len < maxLen) && (*pChar != '"'));
+        } while ((pos < maxLen) && (aLine[pos] != doubleQuote));
     }
     else {
-        while ((len < maxLen) && (*pChar != delim)) {
-            pChar++;
-            len++;
+        while ((pos < maxLen) && (aLine[pos] != delim)) {
+            pos++;
             fLen++;
         }
     }
@@ -328,14 +351,14 @@ bool nsTextAddress::GetField(const char *pLine, PRInt32 maxLen, PRInt32 index, n
         return result;
     }
 
-    field.Append(pStart, fLen);
+    field.Append(nsDependentSubstring(aLine, startPos, fLen));
     field.Trim(kWhitespace);
 
     if (quoted) {
-      PRInt32 offset = field.Find(NS_LITERAL_CSTRING("\"\""));
+      int32_t offset = field.Find("\"\"");
       while (offset != -1) {
         field.Cut(offset, 1);
-        offset = field.Find(NS_LITERAL_CSTRING("\"\""), offset + 1);
+        offset = MsgFind(field, "\"\"", false, offset + 1);
       }
     }
 
@@ -351,54 +374,42 @@ nsresult nsTextAddress::DetermineDelim(nsIFile *aSrc)
     return rv;
   }
 
-  char *pLine = new char[kTextAddressBufferSz];
-  if (!pLine)
-    return NS_ERROR_OUT_OF_MEMORY;
+  int32_t lineCount = 0;
+  int32_t tabCount = 0;
+  int32_t commaCount = 0;
+  int32_t tabLines = 0;
+  int32_t commaLines = 0;
+  nsAutoString line;
+  bool more = true;
 
-  PRUint32 bytesLeft = 0;
-  rv = inputStream->Available(&bytesLeft);
+  nsCOMPtr<nsIUnicharLineInputStream> lineStream;
+  rv = GetUnicharLineStreamForFile(aSrc, inputStream, getter_AddRefs(lineStream));
   if (NS_FAILED(rv)) {
-    IMPORT_LOG0("*** Error checking address file for eof\n");
-    inputStream->Close();
-    delete [] pLine;
+    IMPORT_LOG0("*** Error opening converter stream for importer\n");
     return rv;
   }
 
-  PRUint32 left;
-  PRInt32 lineLen = 0;
-  PRInt32 lineCount = 0;
-  PRInt32 tabCount = 0;
-  PRInt32 commaCount = 0;
-  PRInt32 tabLines = 0;
-  PRInt32 commaLines = 0;
-
-  while (bytesLeft && NS_SUCCEEDED(rv) && (lineCount < 100)) {
-    left = 0;
-    rv = inputStream->Read(pLine, kTextAddressBufferSz, &left);
-    if (left)
-      pLine[kTextAddressBufferSz - 1] = 0;
-
+  while (more && NS_SUCCEEDED(rv) && (lineCount < 100)) {
+    rv = lineStream->ReadLine(line, &more);
     if (NS_SUCCEEDED(rv)) {
-      lineLen = strlen(pLine);
-      tabCount = CountFields(pLine, lineLen, 9);
-      commaCount = CountFields(pLine, lineLen, ',');
+      tabCount = CountFields(line, PRUnichar('\t'));
+      commaCount = CountFields(line, PRUnichar(','));
       if (tabCount > commaCount)
         tabLines++;
       else if (commaCount)
         commaLines++;
-      rv = inputStream->Available(&bytesLeft);
     }
     lineCount++;
   }
 
   rv = inputStream->Close();
 
-  delete [] pLine;
-
   if (tabLines > commaLines)
-    m_delim = '\t';
+    m_delim = PRUnichar('\t');
   else
-    m_delim = ',';
+    m_delim = PRUnichar(',');
+
+  IMPORT_LOG2( "Tab count = %d, Comma count = %d\n", tabLines, commaLines);
 
   return rv;
 }
@@ -407,7 +418,7 @@ nsresult nsTextAddress::DetermineDelim(nsIFile *aSrc)
     This is where the real work happens!
     Go through the field map and set the data in a new database row
 */
-nsresult nsTextAddress::ProcessLine(const char *pLine, PRInt32 len, nsString& errors)
+nsresult nsTextAddress::ProcessLine(const nsAString &aLine, nsString& errors)
 {
     if (!m_fieldMap) {
         IMPORT_LOG0("*** Error, text import needs a field map\n");
@@ -419,19 +430,18 @@ nsresult nsTextAddress::ProcessLine(const char *pLine, PRInt32 len, nsString& er
     // Wait until we get our first non-empty field, then create a new row,
     // fill in the data, then add the row to the database.
     nsCOMPtr<nsIMdbRow> newRow;
-    nsString    uVal;
-    nsCString    fieldVal;
-    PRInt32        fieldNum;
-    PRInt32        numFields = 0;
-    bool          active;
+    nsAutoString   fieldVal;
+    int32_t        fieldNum;
+    int32_t        numFields = 0;
+    bool           active;
     rv = m_fieldMap->GetMapSize(&numFields);
-    for (PRInt32 i = 0; (i < numFields) && NS_SUCCEEDED(rv); i++) {
+    for (int32_t i = 0; (i < numFields) && NS_SUCCEEDED(rv); i++) {
         active = false;
         rv = m_fieldMap->GetFieldMap(i, &fieldNum);
         if (NS_SUCCEEDED(rv))
             rv = m_fieldMap->GetFieldActive(i, &active);
         if (NS_SUCCEEDED(rv) && active) {
-            if (GetField(pLine, len, i, fieldVal, m_delim)) {
+            if (GetField(aLine, i, fieldVal, m_delim)) {
                 if (!fieldVal.IsEmpty()) {
                     if (!newRow) {
                         rv = m_database->GetNewRow(getter_AddRefs(newRow));
@@ -440,8 +450,7 @@ nsresult nsTextAddress::ProcessLine(const char *pLine, PRInt32 len, nsString& er
                         }
                     }
                     if (newRow) {
-                        NS_CopyNativeToUnicode(fieldVal, uVal);
-                        rv = m_fieldMap->SetFieldValue(m_database, newRow, fieldNum, uVal.get());
+                        rv = m_fieldMap->SetFieldValue(m_database, newRow, fieldNum, fieldVal.get());
                     }
                 }
             }
