@@ -54,7 +54,7 @@ PRLogModuleInfo *MAILBOX;
 nsMailboxProtocol::nsMailboxProtocol(nsIURI * aURI)
     : nsMsgProtocol(aURI)
 {
-  m_lineStreamBuffer =nsnull;
+  m_lineStreamBuffer =nullptr;
 
   // initialize the pr log if it hasn't been initialiezed already
   if (!MAILBOX)
@@ -67,7 +67,7 @@ nsMailboxProtocol::~nsMailboxProtocol()
   delete m_lineStreamBuffer;
 }
 
-nsresult nsMailboxProtocol::OpenMultipleMsgTransport(PRUint64 offset, PRInt32 size)
+nsresult nsMailboxProtocol::OpenMultipleMsgTransport(uint64_t offset, int32_t size)
 {
   nsresult rv;
 
@@ -76,14 +76,14 @@ nsresult nsMailboxProtocol::OpenMultipleMsgTransport(PRUint64 offset, PRInt32 si
   NS_ENSURE_SUCCESS(rv, rv);
 
   // XXX 64-bit
-  rv = serv->CreateInputTransport(m_multipleMsgMoveCopyStream, PRInt64(offset),
-                                  PRInt64(size), false,
+  rv = serv->CreateInputTransport(m_multipleMsgMoveCopyStream, int64_t(offset),
+                                  int64_t(size), false,
                                   getter_AddRefs(m_transport));
 
   return rv;
 }
 
-nsresult nsMailboxProtocol::OpenFileSocketForReuse(nsIURI * aURL, PRUint64 aStartPosition, PRInt32 aReadCount)
+nsresult nsMailboxProtocol::OpenFileSocketForReuse(nsIURI * aURL, uint64_t aStartPosition, int32_t aReadCount)
 {
   NS_ENSURE_ARG_POINTER(aURL);
 
@@ -136,7 +136,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
         GetFileFromURL(aURL, getter_AddRefs(file));
         if (file)
         {
-          PRInt64 fileSize = 0;
+          int64_t fileSize = 0;
           file->GetFileSize(&fileSize);
           mailnewsUrl->SetMaxProgress(fileSize);
         }
@@ -148,7 +148,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
         // we need to specify a byte range to read in so we read in JUST the message we want.
         rv = SetupMessageExtraction();
         if (NS_FAILED(rv)) return rv;
-        PRUint32 aMsgSize = 0;
+        uint32_t aMsgSize = 0;
         rv = m_runningUrl->GetMessageSize(&aMsgSize);
         NS_ASSERTION(NS_SUCCEEDED(rv), "oops....i messed something up");
         SetContentLength(aMsgSize);
@@ -159,7 +159,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
           rv = OpenFileSocketForReuse(aURL, m_msgOffset, aMsgSize);
           // if we're running multiple msg url, we clear the event sink because the multiple
           // msg urls will handle setting the progress.
-          mProgressEventSink = nsnull;
+          mProgressEventSink = nullptr;
         }
         else
         {
@@ -184,7 +184,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
             if (NS_SUCCEEDED(rv) && msgHdr)
             {
               nsCOMPtr<nsIInputStream> stream;
-              PRInt64 offset = 0;
+              int64_t offset = 0;
               bool reusable = false;
 
               rv = folder->GetMsgInputStream(msgHdr, &reusable, getter_AddRefs(stream));
@@ -198,7 +198,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
               if (NS_FAILED(rv)) return rv;
               m_readCount = aMsgSize;
               rv = sts->CreateInputTransport(stream, offset,
-                                             PRInt64(aMsgSize), true,
+                                             int64_t(aMsgSize), true,
                                              getter_AddRefs(m_transport));
 
               m_socketIsOpen = false;
@@ -245,7 +245,7 @@ bool nsMailboxProtocol::RunningMultipleMsgUrl()
 {
   if (m_mailboxAction == nsIMailboxUrl::ActionCopyMessage || m_mailboxAction == nsIMailboxUrl::ActionMoveMessage)
   {
-    PRUint32 numMoveCopyMsgs;
+    uint32_t numMoveCopyMsgs;
     nsresult rv = m_runningUrl->GetNumMoveCopyMsgs(&numMoveCopyMsgs);
     if (NS_SUCCEEDED(rv) && numMoveCopyMsgs > 1)
       return true;
@@ -281,8 +281,8 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
     
     if (!stopped && NS_SUCCEEDED(aStatus) && (m_mailboxAction == nsIMailboxUrl::ActionCopyMessage || m_mailboxAction == nsIMailboxUrl::ActionMoveMessage))
     {
-      PRUint32 numMoveCopyMsgs;
-      PRUint32 curMoveCopyMsgIndex;
+      uint32_t numMoveCopyMsgs;
+      uint32_t curMoveCopyMsgIndex;
       rv = m_runningUrl->GetNumMoveCopyMsgs(&numMoveCopyMsgs);
       if (NS_SUCCEEDED(rv) && numMoveCopyMsgs > 0)
       {
@@ -303,7 +303,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
           rv = m_runningUrl->GetMoveCopyMsgHdrForIndex(curMoveCopyMsgIndex, getter_AddRefs(nextMsg));
           if (NS_SUCCEEDED(rv) && nextMsg)
           {
-            PRUint32 msgSize = 0;
+            uint32_t msgSize = 0;
             nsCOMPtr <nsIMsgFolder> msgFolder;
             nextMsg->GetFolder(getter_AddRefs(msgFolder));
             NS_ASSERTION(msgFolder, "couldn't get folder for next msg in multiple msg local copy");
@@ -317,7 +317,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
                 msgUrl->SetOriginalSpec(uri.get());
                 msgUrl->SetUri(uri.get());
                 
-                PRUint64 msgOffset;
+                uint64_t msgOffset;
                 nextMsg->GetMessageOffset(&msgOffset);
                 nextMsg->GetMessageSize(&msgSize);
                 // now we have to seek to the right position in the file and
@@ -352,7 +352,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
                 
                 NS_ASSERTION(NS_SUCCEEDED(rv), "AsyncRead failed");
                 if (m_loadGroup)
-                  m_loadGroup->RemoveRequest(static_cast<nsIRequest *>(this), nsnull, aStatus);
+                  m_loadGroup->RemoveRequest(static_cast<nsIRequest *>(this), nullptr, aStatus);
                 m_socketIsOpen = true; // mark the channel as open
                 return aStatus;
               }
@@ -387,7 +387,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
   if (m_multipleMsgMoveCopyStream)
   {
     m_multipleMsgMoveCopyStream->Close();
-    m_multipleMsgMoveCopyStream = nsnull;
+    m_multipleMsgMoveCopyStream = nullptr;
   }
   nsMsgProtocol::OnStopRequest(request, ctxt, aStatus);
   return CloseSocket(); 
@@ -397,7 +397,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest *request, nsISupports
 // End of nsIStreamListenerSupport
 //////////////////////////////////////////////////////////////////////////////////////////////
 
-PRInt32 nsMailboxProtocol::DoneReadingMessage()
+nsresult nsMailboxProtocol::DoneReadingMessage()
 {
   nsresult rv = NS_OK;
   // and close the article file if it was open....
@@ -408,7 +408,7 @@ PRInt32 nsMailboxProtocol::DoneReadingMessage()
   return rv;
 }
 
-PRInt32 nsMailboxProtocol::SetupMessageExtraction()
+nsresult nsMailboxProtocol::SetupMessageExtraction()
 {
   // Determine the number of bytes we are going to need to read out of the 
   // mailbox url....
@@ -418,7 +418,7 @@ PRInt32 nsMailboxProtocol::SetupMessageExtraction()
   NS_ASSERTION(m_runningUrl, "Not running a url");
   if (m_runningUrl)
   {
-    PRUint32 messageSize = 0;
+    uint32_t messageSize = 0;
     m_runningUrl->GetMessageSize(&messageSize);
     if (!messageSize)
     {
@@ -467,7 +467,7 @@ nsresult nsMailboxProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
         nsCOMPtr<nsIMsgMailNewsUrl> msgUrl = do_QueryInterface(m_runningUrl, &rv);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsCAutoString queryStr;
+        nsAutoCString queryStr;
         rv = msgUrl->GetQuery(queryStr);
         NS_ENSURE_SUCCESS(rv,rv);
 
@@ -551,7 +551,7 @@ nsresult nsMailboxProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
   return rv;
 }
 
-PRInt32 nsMailboxProtocol::ReadFolderResponse(nsIInputStream * inputStream, PRUint32 sourceOffset, PRUint32 length)
+int32_t nsMailboxProtocol::ReadFolderResponse(nsIInputStream * inputStream, uint64_t sourceOffset, uint32_t length)
 {
   // okay we are doing a folder read in 8K chunks of a mail folder....
   // this is almost too easy....we can just forward the data in this stream on to our
@@ -563,7 +563,7 @@ PRInt32 nsMailboxProtocol::ReadFolderResponse(nsIInputStream * inputStream, PRUi
   if (m_mailboxParser)
   {
     nsCOMPtr <nsIURI> url = do_QueryInterface(m_runningUrl);
-    rv = m_mailboxParser->OnDataAvailable(nsnull, url, inputStream, sourceOffset, length); // let the parser deal with it...
+    rv = m_mailboxParser->OnDataAvailable(nullptr, url, inputStream, sourceOffset, length); // let the parser deal with it...
   }
   if (NS_FAILED(rv))
   {
@@ -582,10 +582,10 @@ PRInt32 nsMailboxProtocol::ReadFolderResponse(nsIInputStream * inputStream, PRUi
   return 0; 
 }
 
-PRInt32 nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, PRUint32 sourceOffset, PRUint32 length)
+int32_t nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, uint64_t sourceOffset, uint32_t length)
 {
-  char *line = nsnull;
-  PRUint32 status = 0;
+  char *line = nullptr;
+  uint32_t status = 0;
   nsresult rv = NS_OK;
   mCurrentProgress += length;
   
@@ -622,7 +622,7 @@ PRInt32 nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, PRU
       // The dummy header is the From line with the date tag on it.
       if (m_msgFileOutputStream && TestFlag(MAILBOX_MSG_PARSE_FIRST_LINE))
       {
-        PRUint32 count = 0;
+        uint32_t count = 0;
         rv = m_msgFileOutputStream->Write(line, PL_strlen(line), &count);
         if (NS_FAILED(rv))
           break;
@@ -646,7 +646,7 @@ PRInt32 nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, PRU
   SetFlag(MAILBOX_PAUSE_FOR_READ); // wait for more data to become available...
   if (mProgressEventSink && m_runningUrl)
   {
-    PRInt64 maxProgress;
+    int64_t maxProgress;
     nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl(do_QueryInterface(m_runningUrl));
     mailnewsUrl->GetMaxProgress(&maxProgress);
     mProgressEventSink->OnProgress(this, m_channelContext,
@@ -665,10 +665,10 @@ PRInt32 nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, PRU
  *
  * returns zero or more if the transfer needs to be continued.
  */
-nsresult nsMailboxProtocol::ProcessProtocolState(nsIURI * url, nsIInputStream * inputStream, PRUint32 offset, PRUint32 length)
+nsresult nsMailboxProtocol::ProcessProtocolState(nsIURI * url, nsIInputStream * inputStream, uint64_t offset, uint32_t length)
 {
   nsresult rv = NS_OK;
-  PRInt32 status = 0;
+  int32_t status = 0;
   ClearFlag(MAILBOX_PAUSE_FOR_READ); /* already paused; reset */
   
   while(!TestFlag(MAILBOX_PAUSE_FOR_READ))
@@ -677,13 +677,13 @@ nsresult nsMailboxProtocol::ProcessProtocolState(nsIURI * url, nsIInputStream *
     switch(m_nextState) 
     {
     case MAILBOX_READ_MESSAGE:
-      if (inputStream == nsnull)
+      if (inputStream == nullptr)
         SetFlag(MAILBOX_PAUSE_FOR_READ);
       else
         status = ReadMessageResponse(inputStream, offset, length);
       break;
     case MAILBOX_READ_FOLDER:
-      if (inputStream == nsnull)
+      if (inputStream == nullptr)
         SetFlag(MAILBOX_PAUSE_FOR_READ);   // wait for file socket to read in the next chunk...
       else
         status = ReadFolderResponse(inputStream, offset, length);
@@ -726,9 +726,9 @@ nsresult nsMailboxProtocol::CloseSocket()
 {
   // how do you force a release when closing the connection??
   nsMsgProtocol::CloseSocket(); 
-  m_runningUrl = nsnull;
-  m_mailboxParser = nsnull;
-  return 0;
+  m_runningUrl = nullptr;
+  m_mailboxParser = nullptr;
+  return NS_OK;
 }
 
 // vim: ts=2 sw=2
