@@ -24,7 +24,7 @@
 #include "nsIMsgFolder.h"
 #include "nsIPrompt.h"
 
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsMailDirServiceDefs.h"
 #include "nsMsgUtils.h"
 
@@ -47,7 +47,7 @@
 //
 // export NSPR_LOG_MODULES=Movemail:5
 //
-static PRLogModuleInfo *gMovemailLog = nsnull;
+static PRLogModuleInfo *gMovemailLog = nullptr;
 #define LOG(args) PR_LOG(gMovemailLog, PR_LOG_DEBUG, args)
 #else
 #define LOG(args)
@@ -94,9 +94,9 @@ nsMovemailService::CheckForNewMail(nsIUrlListener * aUrlListener,
 }
 
 void
-nsMovemailService::Error(PRInt32 errorCode,
+nsMovemailService::Error(int32_t errorCode,
                          const PRUnichar **params,
-                         PRUint32 length)
+                         uint32_t length)
 {
   if (!mMsgWindow) return;
 
@@ -122,7 +122,7 @@ nsMovemailService::Error(PRInt32 errorCode,
     bundle->GetStringFromID(errorCode, getter_Copies(errStr));
 
   if (!errStr.IsEmpty()) {
-    dialog->Alert(nsnull, errStr.get());
+    dialog->Alert(nullptr, errStr.get());
   }
 }
 
@@ -142,7 +142,7 @@ bool ObtainSpoolLock(const char *aSpoolName,
    * This involves creating a .mozlock file and attempting to hard-link it to
    * the customary .lock file.
    */
-  nsCOMPtr<nsILocalFile> spoolFile;
+  nsCOMPtr<nsIFile> spoolFile;
   nsresult rv = NS_NewNativeLocalFile(nsDependentCString(aSpoolName),
                                       true,
                                       getter_AddRefs(spoolFile));
@@ -191,13 +191,13 @@ bool ObtainSpoolLock(const char *aSpoolName,
   // (step 2a not yet implemented)
 
 
-  nsCAutoString mozlockstr(aSpoolName);
+  nsAutoCString mozlockstr(aSpoolName);
   mozlockstr.Append(".mozlock");
-  nsCAutoString lockstr(aSpoolName);
+  nsAutoCString lockstr(aSpoolName);
   lockstr.Append(".lock");
 
-  // Create nsILocalFile for the spool.mozlock file
-  nsCOMPtr<nsILocalFile> tmplocfile;
+  // Create nsIFile for the spool.mozlock file
+  nsCOMPtr<nsIFile> tmplocfile;
   rv = NS_NewNativeLocalFile(mozlockstr, true, getter_AddRefs(tmplocfile));
   if (NS_FAILED(rv))
     return false;
@@ -253,7 +253,7 @@ bool YieldSpoolLock(const char *aSpoolName, bool aUsingLockFile)
   LOG(("YieldSpoolLock(%s)", aSpoolName));
 
   if (!aUsingLockFile) {
-    nsCOMPtr<nsILocalFile> spoolFile;
+    nsCOMPtr<nsIFile> spoolFile;
     nsresult rv = NS_NewNativeLocalFile(nsDependentCString(aSpoolName),
                                         true,
                                         getter_AddRefs(spoolFile));
@@ -270,13 +270,13 @@ bool YieldSpoolLock(const char *aSpoolName, bool aUsingLockFile)
     return unlockSucceeded;
   }
 
-  nsCAutoString lockstr(aSpoolName);
+  nsAutoCString lockstr(aSpoolName);
   lockstr.Append(".lock");
 
   nsresult rv;
 
-  // Create nsILocalFile for the spool.lock file
-  nsCOMPtr<nsILocalFile> locklocfile;
+  // Create nsIFile for the spool.lock file
+  nsCOMPtr<nsIFile> locklocfile;
   rv = NS_NewNativeLocalFile(lockstr, true, getter_AddRefs(locklocfile));
   if (NS_FAILED(rv))
     return false;
@@ -306,7 +306,7 @@ LocateSpoolFile(nsACString & spoolPath)
   bool isFile;
   nsresult rv;
 
-  nsCOMPtr<nsILocalFile> spoolFile;
+  nsCOMPtr<nsIFile> spoolFile;
   rv = NS_NewNativeLocalFile(EmptyCString(), true, getter_AddRefs(spoolFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -325,8 +325,8 @@ LocateSpoolFile(nsACString & spoolPath)
   else if (userEnv) {
     // Try to build the mailbox path from the username and a number
     // of guessed spool directory paths.
-    nsCAutoString tmpPath;
-    PRUint32 i;
+    nsAutoCString tmpPath;
+    uint32_t i;
     for (i = 0; i < NUM_DEFAULT_SPOOL_PATHS; i++) {
       tmpPath = gDefaultSpoolPaths[i];
       tmpPath += userEnv;
@@ -360,17 +360,17 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
   mMsgWindow = aMsgWindow;
 
   // Attempt to locate the mail spool file
-  nsCAutoString spoolPath;
+  nsAutoCString spoolPath;
   rv = in_server->GetCharValue("spoolDir", spoolPath);
   if (spoolPath.IsEmpty())
     rv = LocateSpoolFile(spoolPath);
   if (NS_FAILED(rv) || spoolPath.IsEmpty()) {
-    Error(MOVEMAIL_SPOOL_FILE_NOT_FOUND, nsnull, 0);
+    Error(MOVEMAIL_SPOOL_FILE_NOT_FOUND, nullptr, 0);
     return NS_ERROR_FAILURE;
   }
 
   // Create an input stream for the spool file
-  nsCOMPtr<nsILocalFile> spoolFile;
+  nsCOMPtr<nsIFile> spoolFile;
   rv = NS_NewNativeLocalFile(spoolPath, true, getter_AddRefs(spoolFile));
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIInputStream> spoolInputStream;
@@ -429,8 +429,8 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
 
   // MIDDLE of the FUN : consume the mailbox data.
   bool isMore = true;
-  nsCAutoString buffer;
-  PRUint32 bytesWritten;
+  nsAutoCString buffer;
+  uint32_t bytesWritten;
 
   while (isMore &&
          NS_SUCCEEDED(lineInputStream->ReadLine(buffer, &isMore)))
@@ -448,7 +448,7 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
       // finish prev header, if any.
       if (newHdr) {
         outputStream->Flush();
-        newMailParser->PublishMsgHeader(nsnull);
+        newMailParser->PublishMsgHeader(nullptr);
         msgStore->FinishNewMessage(outputStream, newHdr);
         newMailParser->Clear();
       }
@@ -457,7 +457,7 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
       NS_ENSURE_SUCCESS(rv, rv);
       nsCOMPtr <nsIInputStream> inputStream = do_QueryInterface(outputStream);
       rv = newMailParser->Init(serverFolder, inbox,
-                               nsnull, newHdr, outputStream);
+                               nullptr, newHdr, outputStream);
       NS_ENSURE_SUCCESS(rv, rv);
       
     }
@@ -476,8 +476,8 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
   }
   if (outputStream) {
     outputStream->Flush();
-    newMailParser->PublishMsgHeader(nsnull);
-    newMailParser->OnStopRequest(nsnull, nsnull, NS_OK);
+    newMailParser->PublishMsgHeader(nullptr);
+    newMailParser->OnStopRequest(nullptr, nullptr, NS_OK);
     msgStore->FinishNewMessage(outputStream, newHdr);
     outputStream->Close();
   }
@@ -508,21 +508,21 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
 
 
 NS_IMETHODIMP
-nsMovemailService::SetDefaultLocalPath(nsILocalFile *aPath)
+nsMovemailService::SetDefaultLocalPath(nsIFile *aPath)
 {
   NS_ENSURE_ARG(aPath);
   return NS_SetPersistentFile(PREF_MAIL_ROOT_MOVEMAIL_REL, PREF_MAIL_ROOT_MOVEMAIL, aPath);
 }
 
 NS_IMETHODIMP
-nsMovemailService::GetDefaultLocalPath(nsILocalFile ** aResult)
+nsMovemailService::GetDefaultLocalPath(nsIFile ** aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
-  *aResult = nsnull;
+  *aResult = nullptr;
 
   nsresult rv;
   bool havePref;
-  nsCOMPtr<nsILocalFile> localFile;
+  nsCOMPtr<nsIFile> localFile;
   rv = NS_GetPersistentFile(PREF_MAIL_ROOT_MOVEMAIL_REL,
                             PREF_MAIL_ROOT_MOVEMAIL,
                             NS_APP_MAIL_50_DIR,
@@ -619,7 +619,7 @@ nsMovemailService::GetDefaultDoBiff(bool *aDoBiff)
 }
 
 NS_IMETHODIMP
-nsMovemailService::GetDefaultServerPort(bool isSecure, PRInt32 *aDefaultPort)
+nsMovemailService::GetDefaultServerPort(bool isSecure, int32_t *aDefaultPort)
 {
   NS_ENSURE_ARG_POINTER(aDefaultPort);
   *aDefaultPort = -1;
