@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  David Bienvenu <dbienvenu@mozilla.com>
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
    Class for handling Berkeley Mailbox stores.
@@ -67,6 +35,8 @@
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 #include "prprf.h"
+#include <cstdlib> // for std::abs(int/long)
+#include <cmath> // for std::abs(float/double)
 
 nsMsgBrkMBoxStore::nsMsgBrkMBoxStore()
 {
@@ -83,77 +53,25 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::DiscoverSubFolders(nsIMsgFolder *aParentFolder,
                                                     bool aDeep)
 {
   NS_ENSURE_ARG_POINTER(aParentFolder);
-  bool isServer;
-  nsresult rv = aParentFolder->GetIsServer(&isServer);
-  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsILocalFile> path;
-  rv = aParentFolder->GetFilePath(getter_AddRefs(path));
+  nsCOMPtr<nsIFile> path;
+  nsresult rv = aParentFolder->GetFilePath(getter_AddRefs(path));
   if (NS_FAILED(rv))
     return rv;
 
-  bool exists, directory;
+  bool exists;
   path->Exists(&exists);
   if (!exists)
     path->Create(nsIFile::DIRECTORY_TYPE, 0755);
 
-  path->IsDirectory(&directory);
-  if (!directory)
-  {
-    nsCOMPtr<nsIFile> dirFile;
-    rv = path->Clone(getter_AddRefs(dirFile));
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsAutoString leafName;
-    dirFile->GetLeafName(leafName);
-    leafName.AppendLiteral(".sbd");
-    dirFile->SetLeafName(leafName);
-    path = do_QueryInterface(dirFile);
-    path->IsDirectory(&directory);
-  }
-
-  if (directory)
-  {
-    aParentFolder->SetFlag(nsMsgFolderFlags::Mail | nsMsgFolderFlags::Elided |
-                           nsMsgFolderFlags::Directory);
-
-    // now, discover those folders
-    rv = AddSubFolders(aParentFolder, path, aDeep);
-    if (NS_FAILED(rv))
-      return rv;
-
-    bool createdDefaultMailboxes = false;
-    nsCOMPtr<nsILocalMailIncomingServer> localMailServer;
-
-    if (isServer)
-    {
-      nsCOMPtr<nsIMsgIncomingServer> server;
-      rv = aParentFolder->GetServer(getter_AddRefs(server));
-      NS_ENSURE_SUCCESS(rv, NS_MSG_INVALID_OR_MISSING_SERVER);
-      localMailServer = do_QueryInterface(server, &rv);
-      NS_ENSURE_SUCCESS(rv, NS_MSG_INVALID_OR_MISSING_SERVER);
-
-      // first create the folders on disk (as empty files)
-      rv = localMailServer->CreateDefaultMailboxes(path);
-      NS_ENSURE_SUCCESS(rv, rv);
-      createdDefaultMailboxes = true;
-      // now, discover those folders
-      rv = AddSubFolders(aParentFolder, path, aDeep);
-      if (NS_FAILED(rv))
-        return rv;
-      
-      rv = localMailServer->SetFlagsOnDefaultMailboxes();
-      if (NS_FAILED(rv))
-        return rv;
-    }
-  }
-  return rv;
+  return AddSubFolders(aParentFolder, path, aDeep);
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::CreateFolder(nsIMsgFolder *aParent,
                                               const nsAString &aFolderName,
                                               nsIMsgFolder **aResult)
 {
-  nsCOMPtr<nsILocalFile> path;
+  nsCOMPtr<nsIFile> path;
   nsCOMPtr<nsIMsgFolder> child;
   nsresult rv = aParent->GetFilePath(getter_AddRefs(path));
   if (NS_FAILED(rv))
@@ -218,13 +136,13 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CreateFolder(nsIMsgFolder *aParent,
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::GetSummaryFile(nsIMsgFolder *aFolder,
-                                                nsILocalFile **aSummaryFile)
+                                                nsIFile **aSummaryFile)
 {
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aSummaryFile);
 
   nsresult rv;
-  nsCOMPtr<nsILocalFile> newSummaryLocation;
+  nsCOMPtr<nsIFile> newSummaryLocation;
   rv = aFolder->GetFilePath(getter_AddRefs(newSummaryLocation));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -244,47 +162,46 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::GetSummaryFile(nsIMsgFolder *aFolder,
 
 // Get the current attributes of the mbox file, corrected for caching
 void nsMsgBrkMBoxStore::GetMailboxModProperties(nsIMsgFolder *aFolder,
-                                                PRInt64 *aSize, PRUint32 *aDate)
+                                                int64_t *aSize, uint32_t *aDate)
 {
   // We'll simply return 0 on errors.
   *aDate = 0;
   *aSize = 0;
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
-  NS_ENSURE_SUCCESS(rv, );
+  NS_ENSURE_SUCCESS_VOID(rv);
 
   rv = pathFile->GetFileSize(aSize);
-  if (NS_FAILED(rv))
-    return;
+  NS_ENSURE_SUCCESS_VOID(rv);
 
-  PRInt64 lastModTime;
+  PRTime lastModTime;
   rv = pathFile->GetLastModifiedTime(&lastModTime);
-  if (NS_FAILED(rv))
-    return;
+  NS_ENSURE_SUCCESS_VOID(rv);
 
-  *aDate = (PRUint32) (lastModTime / PR_MSEC_PER_SEC);
+  *aDate = (uint32_t) (lastModTime / PR_MSEC_PER_SEC);
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::HasSpaceAvailable(nsIMsgFolder *aFolder,
-                                                   PRInt64 aSpaceRequested,
+                                                   int64_t aSpaceRequested,
                                                    bool *aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
   NS_ENSURE_ARG_POINTER(aFolder);
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  PRInt64 fileSize;
+  int64_t fileSize;
   rv = pathFile->GetFileSize(&fileSize);
   NS_ENSURE_SUCCESS(rv, rv);
   // ### I think we're allowing mailboxes > 4GB, so we should be checking
   // for disk space here, not total file size.
+  // 0xFFC00000 = 4 GiB - 4 MiB.
   *aResult = ((fileSize + aSpaceRequested) < 0xFFC00000);
   return NS_OK;
 }
 
 static bool gGotGlobalPrefs = false;
-static PRInt32 gTimeStampLeeway = 60;
+static int32_t gTimeStampLeeway = 60;
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
                                                     nsIMsgDatabase *aDB,
@@ -301,15 +218,15 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
     return NS_OK;
   }
 
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIDBFolderInfo> folderInfo;
   rv = aDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
   NS_ENSURE_SUCCESS(rv, rv);
-  PRUint64 folderSize;
-  PRUint32 folderDate;
-  PRInt32 numUnreadMessages;
+  uint64_t folderSize;
+  uint32_t folderDate;
+  int32_t numUnreadMessages;
 
   *aResult = false;
 
@@ -317,8 +234,8 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
   folderInfo->GetFolderSize(&folderSize);
   folderInfo->GetFolderDate(&folderDate);
 
-  PRInt64 fileSize = 0;
-  PRUint32 actualFolderTimeStamp = 0;
+  int64_t fileSize = 0;
+  uint32_t actualFolderTimeStamp = 0;
   GetMailboxModProperties(aFolder, &fileSize, &actualFolderTimeStamp);
 
   if (folderSize == fileSize && numUnreadMessages >= 0)
@@ -341,7 +258,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
     if (gTimeStampLeeway == 0)
       *aResult = folderDate == actualFolderTimeStamp;
     else
-      *aResult = NS_ABS((PRInt32) (actualFolderTimeStamp - folderDate)) <= gTimeStampLeeway;
+      *aResult = std::abs((int32_t) (actualFolderTimeStamp - folderDate)) <= gTimeStampLeeway;
   }
   return NS_OK;
 }
@@ -357,7 +274,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::SetSummaryFileValid(nsIMsgFolder *aFolder,
   if (!localFolder)
     return NS_OK;
 
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIDBFolderInfo> folderInfo;
@@ -370,8 +287,8 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::SetSummaryFileValid(nsIMsgFolder *aFolder,
 
   if (aValid)
   {
-    PRUint32 actualFolderTimeStamp;
-    PRInt64 fileSize;
+    uint32_t actualFolderTimeStamp;
+    int64_t fileSize;
     GetMailboxModProperties(aFolder, &fileSize, &actualFolderTimeStamp);
     folderInfo->SetFolderSize(fileSize);
     folderInfo->SetFolderDate(actualFolderTimeStamp);
@@ -388,7 +305,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::DeleteFolder(nsIMsgFolder *aFolder)
 {
   NS_ENSURE_ARG_POINTER(aFolder);
   //Delete mailbox
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -416,12 +333,12 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::RenameFolder(nsIMsgFolder *aFolder,
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aNewFolder);
 
-  PRUint32 numChildren;
+  uint32_t numChildren;
   aFolder->GetNumSubFolders(&numChildren);
   nsString existingName;
   aFolder->GetName(existingName);
 
-  nsCOMPtr<nsILocalFile> oldPathFile;
+  nsCOMPtr<nsIFile> oldPathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(oldPathFile));
   if (NS_FAILED(rv))
     return rv;
@@ -433,27 +350,26 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::RenameFolder(nsIMsgFolder *aFolder,
 
   nsCOMPtr<nsISupports> parentSupport = do_QueryInterface(parentFolder);
 
-  nsCOMPtr<nsILocalFile> oldSummaryFile;
+  nsCOMPtr<nsIFile> oldSummaryFile;
   rv = GetSummaryFile(aFolder, getter_AddRefs(oldSummaryFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIFile> dirFile;
   oldPathFile->Clone(getter_AddRefs(dirFile));
-  nsCOMPtr<nsILocalFile> localDirFile(do_QueryInterface(dirFile));
 
   if (numChildren > 0)
   {
-    rv = CreateDirectoryForFolder(localDirFile);
+    rv = CreateDirectoryForFolder(dirFile);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
   nsAutoString safeName(aNewName);
   NS_MsgHashIfNecessary(safeName);
 
-  nsCAutoString oldLeafName;
+  nsAutoCString oldLeafName;
   oldPathFile->GetNativeLeafName(oldLeafName);
 
-  nsCOMPtr<nsILocalFile> parentPathFile;
+  nsCOMPtr<nsIFile> parentPathFile;
   parentFolder->GetFilePath(getter_AddRefs(parentPathFile));
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -470,20 +386,20 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::RenameFolder(nsIMsgFolder *aFolder,
 
   aFolder->ForceDBClosed();
   //save off dir name before appending .msf
-  rv = oldPathFile->MoveTo(nsnull, safeName);
+  rv = oldPathFile->MoveTo(nullptr, safeName);
   if (NS_FAILED(rv))
     return rv;
 
   nsString dbName(safeName);
   dbName += NS_LITERAL_STRING(SUMMARY_SUFFIX);
-  oldSummaryFile->MoveTo(nsnull, dbName);
+  oldSummaryFile->MoveTo(nullptr, dbName);
 
   if (numChildren > 0)
   {
     // rename "*.sbd" directory
     nsAutoString newNameDirStr(safeName);
     newNameDirStr += NS_LITERAL_STRING(".sbd");
-    localDirFile->MoveTo(nsnull, newNameDirStr);
+    dirFile->MoveTo(nullptr, newNameDirStr);
   }
 
   return parentFolder->AddSubfolder(safeName, aNewFolder);
@@ -506,17 +422,17 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   if (localSrcFolder)
     localSrcFolder->GetDatabaseWOReparse(getter_AddRefs(srcDB));
   bool summaryValid = !!srcDB;
-  srcDB = nsnull;
+  srcDB = nullptr;
   aSrcFolder->ForceDBClosed();
 
-  nsCOMPtr<nsILocalFile> oldPath;
+  nsCOMPtr<nsIFile> oldPath;
   nsresult rv = aSrcFolder->GetFilePath(getter_AddRefs(oldPath));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr<nsILocalFile> summaryFile;
+  nsCOMPtr<nsIFile> summaryFile;
   GetSummaryFileLocation(oldPath, getter_AddRefs(summaryFile));
 
-  nsCOMPtr<nsILocalFile> newPath;
+  nsCOMPtr<nsIFile> newPath;
   rv = aDstFolder->GetFilePath(getter_AddRefs(newPath));
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -545,7 +461,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   {
     // Test if the filespec has data
     bool exists;
-    PRInt64 fileSize;
+    int64_t fileSize;
     summaryFile->Exists(&exists);
     summaryFile->GetFileSize(&fileSize);
     if (exists && fileSize > 0)
@@ -576,7 +492,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       destDB->SetSummaryValid(true);
   }
   newMsgFolder->SetPrettyName(folderName);
-  PRUint32 flags;
+  uint32_t flags;
   aSrcFolder->GetFlags(&flags);
   newMsgFolder->SetFlags(flags);
   bool changed = false;
@@ -634,7 +550,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
 
     nsCOMPtr<nsIMsgFolder> msgParent;
     aSrcFolder->GetParent(getter_AddRefs(msgParent));
-    aSrcFolder->SetParent(nsnull);
+    aSrcFolder->SetParent(nullptr);
     if (msgParent)
     {
       // The files have already been moved, so delete storage false
@@ -644,7 +560,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       nsCOMPtr<nsIMsgDatabase> srcDB;
       aSrcFolder->Delete();
 
-      nsCOMPtr<nsILocalFile> parentPath;
+      nsCOMPtr<nsIFile> parentPath;
       rv = msgParent->GetFilePath(getter_AddRefs(parentPath));
       NS_ENSURE_SUCCESS(rv,rv);
 
@@ -668,7 +584,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       nsCOMPtr<nsIMsgFolder> msgParent;
       newMsgFolder->ForceDBClosed();
       newMsgFolder->GetParent(getter_AddRefs(msgParent));
-      newMsgFolder->SetParent(nsnull);
+      newMsgFolder->SetParent(nullptr);
       if (msgParent)
       {
         msgParent->PropagateDelete(newMsgFolder, false, aMsgWindow);
@@ -699,7 +615,7 @@ nsMsgBrkMBoxStore::GetNewMsgOutputStream(nsIMsgFolder *aFolder,
   m_streamOutstandingFolder = aFolder;
 #endif
   *aReusable = true;
-  nsCOMPtr<nsILocalFile> mboxFile;
+  nsCOMPtr<nsIFile> mboxFile;
   aFolder->GetFilePath(getter_AddRefs(mboxFile));
   nsCOMPtr<nsIMsgDatabase> db;
   aFolder->GetMsgDatabase(getter_AddRefs(db));
@@ -738,7 +654,7 @@ nsMsgBrkMBoxStore::GetNewMsgOutputStream(nsIMsgFolder *aFolder,
     NS_ENSURE_SUCCESS(rv, rv);
     m_outputStreams.Put(URI, *aResult);
   }
-  PRInt64 filePos;
+  int64_t filePos;
   seekable->Tell(&filePos);
   if (db && !*aNewMsgHdr)
   {
@@ -763,12 +679,12 @@ nsMsgBrkMBoxStore::DiscardNewMessage(nsIOutputStream *aOutputStream,
   NS_ENSURE_ARG_POINTER(aOutputStream);
   NS_ENSURE_ARG_POINTER(aNewHdr);
 #ifdef _DEBUG
-  m_streamOutstandingFolder = nsnull;
+  m_streamOutstandingFolder = nullptr;
 #endif
-  PRUint64 hdrOffset;
+  uint64_t hdrOffset;
   aNewHdr->GetMessageOffset(&hdrOffset);
   aOutputStream->Close();
-  nsCOMPtr<nsILocalFile> mboxFile;
+  nsCOMPtr<nsIFile> mboxFile;
   nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = aNewHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -781,7 +697,7 @@ nsMsgBrkMBoxStore::FinishNewMessage(nsIOutputStream *aOutputStream,
                                      nsIMsgDBHdr *aNewHdr)
 {
 #ifdef _DEBUG
-  m_streamOutstandingFolder = nsnull;
+  m_streamOutstandingFolder = nullptr;
 #endif
   NS_ENSURE_ARG_POINTER(aOutputStream);
 //  NS_ENSURE_ARG_POINTER(aNewHdr);
@@ -803,10 +719,10 @@ nsMsgBrkMBoxStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
 NS_IMETHODIMP
 nsMsgBrkMBoxStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
                                      const nsACString &aMsgToken,
-                                     PRInt64 *aOffset,
+                                     int64_t *aOffset,
                                      nsIMsgDBHdr *aMsgHdr,
-                                     bool *aReusable NS_OUTPARAM,
-                                     nsIInputStream **aResult NS_OUTPARAM)
+                                     bool *aReusable,
+                                     nsIInputStream **aResult)
 {
   NS_ENSURE_ARG_POINTER(aMsgFolder);
   NS_ENSURE_ARG_POINTER(aResult);
@@ -815,10 +731,10 @@ nsMsgBrkMBoxStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
   // If there is no store token, then we set it to the existing message offset.
   if (aMsgToken.IsEmpty())
   {
-    PRUint64 offset;
+    uint64_t offset;
     NS_ENSURE_ARG_POINTER(aMsgHdr);
     aMsgHdr->GetMessageOffset(&offset);
-    *aOffset = PRInt64(offset);
+    *aOffset = int64_t(offset);
     char storeToken[100];
     PR_snprintf(storeToken, sizeof(storeToken), "%lld", *aOffset);
     aMsgHdr->SetStringProperty("storeToken", storeToken);
@@ -827,7 +743,7 @@ nsMsgBrkMBoxStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
     *aOffset = ParseUint64Str(PromiseFlatCString(aMsgToken).get());
   *aReusable = true;
   nsCString URI;
-  nsCOMPtr<nsILocalFile> mboxFile;
+  nsCOMPtr<nsIFile> mboxFile;
 
   aMsgFolder->GetURI(URI);
   aMsgFolder->GetFilePath(getter_AddRefs(mboxFile));
@@ -843,11 +759,13 @@ NS_IMETHODIMP
 nsMsgBrkMBoxStore::CopyMessages(bool isMove, nsIArray *aHdrArray,
                                nsIMsgFolder *aDstFolder,
                                nsIMsgCopyServiceListener *aListener,
+                               nsITransaction **aUndoAction,
                                bool *aCopyDone)
 {
   NS_ENSURE_ARG_POINTER(aHdrArray);
   NS_ENSURE_ARG_POINTER(aDstFolder);
   NS_ENSURE_ARG_POINTER(aCopyDone);
+  *aUndoAction = nullptr;
   *aCopyDone = false;
   return NS_OK;
 }
@@ -870,7 +788,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CompactFolder(nsIMsgFolder *aFolder,
     do_CreateInstance(NS_MSGLOCALFOLDERCOMPACTOR_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 expungedBytes = 0;
+  uint32_t expungedBytes = 0;
   aFolder->GetExpungedBytes(&expungedBytes);
   // check if we need to compact the folder
   return (expungedBytes > 0) ?
@@ -885,7 +803,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::RebuildIndex(nsIMsgFolder *aFolder,
 {
   NS_ENSURE_ARG_POINTER(aFolder);
   // We don't use aMsgDB, but I think nsMsgMailDirStore needs it.
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   if (NS_FAILED(rv))
     return rv;
@@ -908,14 +826,14 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::RebuildIndex(nsIMsgFolder *aFolder,
   NS_ENSURE_SUCCESS(rv, rv);
 
   return mailboxService->ParseMailbox(aMsgWindow, pathFile, parser, aListener,
-                                      nsnull);
+                                      nullptr);
 }
 
 nsresult
 nsMsgBrkMBoxStore::GetOutputStream(nsIArray *aHdrArray,
                                    nsCOMPtr<nsIOutputStream> &outputStream,
                                    nsCOMPtr<nsISeekableStream> &seekableStream,
-                                   PRInt64 &restorePos)
+                                   int64_t &restorePos)
 {
   nsresult rv;
   nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, 0, &rv);
@@ -932,11 +850,11 @@ nsMsgBrkMBoxStore::GetOutputStream(nsIArray *aHdrArray,
     rv = seekableStream->Tell(&restorePos);
     if (NS_FAILED(rv))
     {
-      outputStream = nsnull;
+      outputStream = nullptr;
       m_outputStreams.Remove(URI);
     }
   }
-  nsCOMPtr<nsILocalFile> mboxFile;
+  nsCOMPtr<nsIFile> mboxFile;
   folder->GetFilePath(getter_AddRefs(mboxFile));
   if (!outputStream)
   {
@@ -962,15 +880,15 @@ void nsMsgBrkMBoxStore::SetDBValid(nsIMsgDBHdr *aHdr)
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeFlags(nsIArray *aHdrArray,
-                                             PRUint32 aFlags,
+                                             uint32_t aFlags,
                                              bool aSet)
 {
   NS_ENSURE_ARG_POINTER(aHdrArray);
   nsCOMPtr<nsIOutputStream> outputStream;
   nsCOMPtr<nsISeekableStream> seekableStream;
-  PRInt64 restoreStreamPos;
+  int64_t restoreStreamPos;
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!messageCount)
@@ -981,7 +899,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeFlags(nsIArray *aHdrArray,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     // Seek to x-mozilla-status offset and rewrite value.
@@ -1011,9 +929,9 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeKeywords(nsIArray *aHdrArray,
   NS_ENSURE_ARG_POINTER(aHdrArray);
   nsCOMPtr<nsIOutputStream> outputStream;
   nsCOMPtr<nsISeekableStream> seekableStream;
-  PRInt64 restoreStreamPos;
+  int64_t restoreStreamPos;
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!messageCount)
@@ -1026,9 +944,8 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeKeywords(nsIArray *aHdrArray,
   nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(outputStream, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsLineBuffer<char> *lineBuffer;
-  rv = NS_InitLineBuffer(&lineBuffer);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoPtr<nsLineBuffer<char> > lineBuffer(new nsLineBuffer<char>);
+  NS_ENSURE_TRUE(lineBuffer, NS_ERROR_OUT_OF_MEMORY);
 
   // For each message, we seek to the beginning of the x-mozilla-status header,
   // and start reading lines, looking for x-mozilla-keys: headers; If we're
@@ -1047,20 +964,20 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeKeywords(nsIArray *aHdrArray,
   ParseString(aKeywords, ' ', keywordArray);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  for (PRUint32 i = 0; i < messageCount; ++i) // for each message
+  for (uint32_t i = 0; i < messageCount; ++i) // for each message
   {
     msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
-    PRUint64 messageOffset;
+    uint64_t messageOffset;
     msgHdr->GetMessageOffset(&messageOffset);
-    PRUint32 statusOffset = 0;
+    uint32_t statusOffset = 0;
     (void)msgHdr->GetStatusOffset(&statusOffset);
-    PRUint64 desiredOffset = messageOffset + statusOffset;
+    uint64_t desiredOffset = messageOffset + statusOffset;
 
     ChangeKeywordsHelper(msgHdr, desiredOffset, lineBuffer, keywordArray,
                          aAdd, outputStream, seekableStream, inputStream);
   }
-  PR_Free(lineBuffer);
+  lineBuffer = nullptr;
   if (restoreStreamPos != -1)
     seekableStream->Seek(nsISeekableStream::NS_SEEK_SET, restoreStreamPos);
   else if (outputStream)
@@ -1076,20 +993,32 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeKeywords(nsIArray *aHdrArray,
 // Iterates over the files in the "path" directory, and adds subfolders to
 // parent for each mailbox file found.
 nsresult
-nsMsgBrkMBoxStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
+nsMsgBrkMBoxStore::AddSubFolders(nsIMsgFolder *parent, nsCOMPtr<nsIFile> &path,
                                  bool deep)
 {
+  nsresult rv;
+  nsCOMPtr<nsIFile> tmp; // at top level so we can safely assign to path
+  bool isDirectory;
+  path->IsDirectory(&isDirectory);
+  if (!isDirectory)
+  {
+    rv = path->Clone(getter_AddRefs(tmp));
+    path = tmp;
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsAutoString leafName;
+    path->GetLeafName(leafName);
+    leafName.AppendLiteral(".sbd");
+    path->SetLeafName(leafName);
+    path->IsDirectory(&isDirectory);
+  }
+  if (!isDirectory)
+    return NS_OK;
   // first find out all the current subfolders and files, before using them
   // while creating new subfolders; we don't want to modify and iterate the same
   // directory at once.
   nsCOMArray<nsIFile> currentDirEntries;
-  bool isDirectory;
-  path->IsDirectory(&isDirectory);
-  if (!isDirectory)
-    return NS_OK;
-
   nsCOMPtr<nsISimpleEnumerator> directoryEnumerator;
-  nsresult rv = path->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
+  rv = path->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool hasMore;
@@ -1104,8 +1033,8 @@ nsMsgBrkMBoxStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
   }
 
   // add the folders
-  PRInt32 count = currentDirEntries.Count();
-  for (PRInt32 i = 0; i < count; ++i)
+  int32_t count = currentDirEntries.Count();
+  for (int32_t i = 0; i < count; ++i)
   {
     nsCOMPtr<nsIFile> currentFile(currentDirEntries[i]);
 
@@ -1127,20 +1056,20 @@ nsMsgBrkMBoxStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
         child->SetPrettyName(leafName);
       if (deep)
       {
-        nsCOMPtr<nsILocalFile> path;
+        nsCOMPtr<nsIFile> path;
         rv = child->GetFilePath(getter_AddRefs(path));
         AddSubFolders(child, path, true);
       }
     }
   }
-  return rv;
+  return rv == NS_MSG_FOLDER_EXISTS ? NS_OK : rv;
 }
 
 /* Finds the directory associated with this folder.  That is if the path is
    c:\Inbox, it will return c:\Inbox.sbd if it succeeds.  If that path doesn't
    currently exist then it will create it. Path is strictly an out parameter.
   */
-nsresult nsMsgBrkMBoxStore::CreateDirectoryForFolder(nsILocalFile *path)
+nsresult nsMsgBrkMBoxStore::CreateDirectoryForFolder(nsIFile *path)
 {
   nsresult rv = NS_OK;
 
