@@ -80,8 +80,8 @@ nsresult nsMsgMaildirStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
   }
 
   // add the folders
-  PRInt32 count = currentDirEntries.Count();
-  for (PRInt32 i = 0; i < count; ++i)
+  int32_t count = currentDirEntries.Count();
+  for (int32_t i = 0; i < count; ++i)
   {
     nsCOMPtr<nsIFile> currentFile(currentDirEntries[i]);
 
@@ -98,7 +98,7 @@ nsresult nsMsgMaildirStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
         child->SetPrettyName(leafName);
       if (deep)
       {
-        nsCOMPtr<nsILocalFile> path;
+        nsCOMPtr<nsIFile> path;
         rv = child->GetFilePath(getter_AddRefs(path));
         NS_ENSURE_SUCCESS(rv, rv);
 
@@ -121,7 +121,7 @@ NS_IMETHODIMP nsMsgMaildirStore::DiscoverSubFolders(nsIMsgFolder *aParentFolder,
 {
   NS_ENSURE_ARG_POINTER(aParentFolder);
 
-  nsCOMPtr<nsILocalFile> path;
+  nsCOMPtr<nsIFile> path;
   nsresult rv = aParentFolder->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -144,13 +144,13 @@ NS_IMETHODIMP nsMsgMaildirStore::DiscoverSubFolders(nsIMsgFolder *aParentFolder,
 *  mail client).
  * aFolderName is already "safe" - it has been through NS_MsgHashIfNecessary
  */
-nsresult nsMsgMaildirStore::CreateMaildir(nsILocalFile *path)
+nsresult nsMsgMaildirStore::CreateMaildir(nsIFile *path)
 {
   nsresult rv = path->Create(nsIFile::DIRECTORY_TYPE, 0700);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Create tmp, new, cur leaves
-  nsCOMPtr<nsILocalFile> leaf(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
+  nsCOMPtr<nsIFile> leaf(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   leaf->InitWithFile(path);
@@ -170,7 +170,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
                                               const nsAString &aFolderName,
                                               nsIMsgFolder **aResult)
 {
-  nsCOMPtr <nsILocalFile> path;
+  nsCOMPtr <nsIFile> path;
   nsresult rv = aParent->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -235,7 +235,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::HasSpaceAvailable(nsIMsgFolder *aFolder,
-                                                   PRInt64 aSpaceRequested,
+                                                   int64_t aSpaceRequested,
                                                    bool *aResult)
 {
   // This should probably check disk space available, though the caller
@@ -259,7 +259,7 @@ NS_IMETHODIMP nsMsgMaildirStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
                                                  aResult);
   if (!*aResult)
   {
-    nsCOMPtr<nsILocalFile> newFile;
+    nsCOMPtr<nsIFile> newFile;
     rv = aFolder->GetFilePath(getter_AddRefs(newFile));
     NS_ENSURE_SUCCESS(rv, rv);
     newFile->Append(NS_LITERAL_STRING("cur"));
@@ -270,7 +270,7 @@ NS_IMETHODIMP nsMsgMaildirStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
     newFile->Exists(&exists);
     if (!exists)
     {
-      PRInt32 numMessages;
+      int32_t numMessages;
       dbFolderInfo->GetNumMessages(&numMessages);
       if (!numMessages)
         *aResult = true;
@@ -291,17 +291,17 @@ NS_IMETHODIMP nsMsgMaildirStore::SetSummaryFileValid(nsIMsgFolder *aFolder,
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::GetSummaryFile(nsIMsgFolder *aFolder,
-                                                nsILocalFile **aSummaryFile)
+                                                nsIFile **aSummaryFile)
 {
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aSummaryFile);
 
   nsresult rv;
-  nsCOMPtr <nsILocalFile> newSummaryLocation =
+  nsCOMPtr <nsIFile> newSummaryLocation =
     do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -324,7 +324,7 @@ NS_IMETHODIMP nsMsgMaildirStore::DeleteFolder(nsIMsgFolder *aFolder)
   NS_ENSURE_ARG_POINTER(aFolder);
 
   // Delete Maildir structure
-  nsCOMPtr<nsILocalFile> pathFile;
+  nsCOMPtr<nsIFile> pathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -345,13 +345,13 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
   NS_ENSURE_ARG_POINTER(aNewFolder);
 
   // old path
-  nsCOMPtr<nsILocalFile> oldPathFile;
+  nsCOMPtr<nsIFile> oldPathFile;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(oldPathFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // old sbd directory
-  nsCOMPtr<nsILocalFile> sbdPathFile;
-  PRUint32 numChildren;
+  nsCOMPtr<nsIFile> sbdPathFile;
+  uint32_t numChildren;
   aFolder->GetNumSubFolders(&numChildren);
   if (numChildren > 0)
   {
@@ -363,7 +363,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
   }
 
   // old summary
-  nsCOMPtr<nsILocalFile> oldSummaryFile;
+  nsCOMPtr<nsIFile> oldSummaryFile;
   rv = GetSummaryFile(aFolder, getter_AddRefs(oldSummaryFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -374,7 +374,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
   aFolder->ForceDBClosed();
 
   // rename folder
-  rv = oldPathFile->MoveTo(nsnull, safeName);
+  rv = oldPathFile->MoveTo(nullptr, safeName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (numChildren > 0)
@@ -382,12 +382,12 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
     // rename "*.sbd" directory
     nsAutoString sbdName = safeName;
     sbdName += NS_LITERAL_STRING(FOLDER_SUFFIX);
-    sbdPathFile->MoveTo(nsnull, sbdName);
+    sbdPathFile->MoveTo(nullptr, sbdName);
   }
 
   // rename summary
   safeName += NS_LITERAL_STRING(SUMMARY_SUFFIX);
-  oldSummaryFile->MoveTo(nsnull, safeName);
+  oldSummaryFile->MoveTo(nullptr, safeName);
 
   nsCOMPtr<nsIMsgFolder> parentFolder;
   rv = aFolder->GetParent(getter_AddRefs(parentFolder));
@@ -412,14 +412,14 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   nsCOMPtr<nsIMsgLocalMailFolder> localSrcFolder(do_QueryInterface(aSrcFolder));
   aSrcFolder->ForceDBClosed();
 
-  nsCOMPtr<nsILocalFile> oldPath;
+  nsCOMPtr<nsIFile> oldPath;
   nsresult rv = aSrcFolder->GetFilePath(getter_AddRefs(oldPath));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr<nsILocalFile> summaryFile;
+  nsCOMPtr<nsIFile> summaryFile;
   GetSummaryFileLocation(oldPath, getter_AddRefs(summaryFile));
 
-  nsCOMPtr<nsILocalFile> newPath;
+  nsCOMPtr<nsIFile> newPath;
   rv = aDstFolder->GetFilePath(getter_AddRefs(newPath));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -444,7 +444,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   {
     // Test if the file is not empty
     bool exists;
-    PRInt64 fileSize;
+    int64_t fileSize;
     summaryFile->Exists(&exists);
     summaryFile->GetFileSize(&fileSize);
     if (exists && fileSize > 0)
@@ -459,7 +459,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   NS_ENSURE_SUCCESS(rv, rv);
 
   newMsgFolder->SetPrettyName(folderName);
-  PRUint32 flags;
+  uint32_t flags;
   aSrcFolder->GetFlags(&flags);
   newMsgFolder->SetFlags(flags);
   bool changed = false;
@@ -517,7 +517,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
 
     nsCOMPtr<nsIMsgFolder> msgParent;
     aSrcFolder->GetParent(getter_AddRefs(msgParent));
-    aSrcFolder->SetParent(nsnull);
+    aSrcFolder->SetParent(nullptr);
     if (msgParent)
     {
       // The files have already been moved, so delete storage false
@@ -526,7 +526,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       nsCOMPtr<nsIMsgDatabase> srcDB; // we need to force closed the source db
       aSrcFolder->Delete();
 
-      nsCOMPtr<nsILocalFile> parentPath;
+      nsCOMPtr<nsIFile> parentPath;
       rv = msgParent->GetFilePath(getter_AddRefs(parentPath));
       NS_ENSURE_SUCCESS(rv,rv);
 
@@ -550,7 +550,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       nsCOMPtr<nsIMsgFolder> msgParent;
       newMsgFolder->ForceDBClosed();
       newMsgFolder->GetParent(getter_AddRefs(msgParent));
-      newMsgFolder->SetParent(nsnull);
+      newMsgFolder->SetParent(nullptr);
       if (msgParent)
       {
         msgParent->PropagateDelete(newMsgFolder, false, aMsgWindow);
@@ -593,7 +593,7 @@ nsMsgMaildirStore::GetNewMsgOutputStream(nsIMsgFolder *aFolder,
   }
   (*aNewMsgHdr)->SetMessageOffset(0);
   // path to the message download folder
-  nsCOMPtr<nsILocalFile> newFile;
+  nsCOMPtr<nsIFile> newFile;
   rv = aFolder->GetFilePath(getter_AddRefs(newFile));
   NS_ENSURE_SUCCESS(rv, rv);
   newFile->Append(NS_LITERAL_STRING("tmp"));
@@ -609,8 +609,8 @@ nsMsgMaildirStore::GetNewMsgOutputStream(nsIMsgFolder *aFolder,
   }
 
   // generate new file name
-  nsCAutoString newName;
-  newName.AppendInt(PR_Now());
+  nsAutoCString newName;
+  newName.AppendInt(static_cast<int64_t>(PR_Now()));
   newFile->AppendNative(newName);
   // CreateUnique, in case we get more than one message per millisecond :-)
   newFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
@@ -630,12 +630,12 @@ nsMsgMaildirStore::DiscardNewMessage(nsIOutputStream *aOutputStream,
 
   aOutputStream->Close();
   // file path is stored in message header property "storeToken"
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsILocalFile> path;
+  nsCOMPtr<nsIFile> path;
   nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = aNewHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -658,7 +658,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream *aOutputStream,
 
   aOutputStream->Close();
 
-  nsCOMPtr<nsILocalFile> folderPath;
+  nsCOMPtr<nsIFile> folderPath;
   nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = aNewHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -666,7 +666,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream *aOutputStream,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // file path is stored in message header property
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
   {
@@ -714,7 +714,7 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
   NS_ENSURE_ARG_POINTER(aDestFolder);
   NS_ENSURE_ARG_POINTER(aResult);
 
-  nsCOMPtr<nsILocalFile> folderPath;
+  nsCOMPtr<nsIFile> folderPath;
   nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = aNewHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -722,7 +722,7 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // file path is stored in message header property
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
   {
@@ -767,10 +767,10 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
 NS_IMETHODIMP
 nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
                                      const nsACString &aMsgToken,
-                                     PRInt64 *aOffset,
+                                     int64_t *aOffset,
                                      nsIMsgDBHdr *aMsgHdr,
-                                     bool *aReusable NS_OUTPARAM,
-                                     nsIInputStream **aResult NS_OUTPARAM)
+                                     bool *aReusable,
+                                     nsIInputStream **aResult)
 {
   NS_ENSURE_ARG_POINTER(aMsgFolder);
   NS_ENSURE_ARG_POINTER(aOffset);
@@ -780,7 +780,7 @@ nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
   *aOffset = 0;
 
   // construct path to file
-  nsCOMPtr<nsILocalFile> path;
+  nsCOMPtr<nsIFile> path;
   nsresult rv = aMsgFolder->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -809,21 +809,21 @@ nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
 
 NS_IMETHODIMP nsMsgMaildirStore::DeleteMessages(nsIArray *aHdrArray)
 {
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIMsgFolder> folder;
 
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     if (NS_FAILED(rv))
       continue;
     msgHdr->GetFolder(getter_AddRefs(folder));
-    nsCOMPtr<nsILocalFile> path;
+    nsCOMPtr<nsIFile> path;
     rv = folder->GetFilePath(getter_AddRefs(path));
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString fileName;
+    nsAutoCString fileName;
     msgHdr->GetStringProperty("storeToken", getter_Copies(fileName));
     if (fileName.IsEmpty())
       return NS_ERROR_FAILURE;
@@ -857,11 +857,11 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
   NS_ENSURE_ARG_POINTER(aDstFolder);
   NS_ENSURE_ARG_POINTER(aCopyDone);
   NS_ENSURE_ARG_POINTER(aUndoAction);
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIMsgFolder> srcFolder;
-  nsCOMPtr<nsILocalFile> destFolderPath;
+  nsCOMPtr<nsIFile> destFolderPath;
   nsCOMPtr<nsIMsgDatabase> destDB;
   nsCOMPtr<nsIMsgDatabase> srcDB;
   aDstFolder->GetMsgDatabase(getter_AddRefs(destDB));
@@ -885,7 +885,7 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
 
   nsCOMPtr<nsIMutableArray> dstHdrs(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     if (NS_FAILED(rv))
@@ -894,10 +894,10 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
     msgHdr->GetMessageKey(&srcKey);
     msgTxn->AddSrcKey(srcKey);
     msgHdr->GetFolder(getter_AddRefs(srcFolder));
-    nsCOMPtr<nsILocalFile> path;
+    nsCOMPtr<nsIFile> path;
     rv = srcFolder->GetFilePath(getter_AddRefs(path));
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString fileName;
+    nsAutoCString fileName;
     msgHdr->GetStringProperty("storeToken", getter_Copies(fileName));
     if (fileName.IsEmpty())
       return NS_ERROR_FAILURE;
@@ -946,10 +946,10 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
                                           dstHdrs);
   if (aIsMove)
   {
-    for (PRUint32 i = 0; i < messageCount; ++i)
+    for (uint32_t i = 0; i < messageCount; ++i)
     {
       nsCOMPtr<nsIMsgDBHdr> msgDBHdr(do_QueryElementAt(aHdrArray, i, &rv));
-      rv = srcDB->DeleteHeader(msgDBHdr, nsnull, false, true);
+      rv = srcDB->DeleteHeader(msgDBHdr, nullptr, false, true);
     }
   }
   *aCopyDone = true;
@@ -1026,22 +1026,20 @@ nsresult MaildirStoreParser::ParseNextMessage(nsIFile *aFile)
 
   newMsgHdr->SetMessageOffset(0);
 
-  nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(aFile);
-  if (NS_SUCCEEDED(rv))
-    rv = NS_NewLocalFileInputStream(getter_AddRefs(inputStream), localFile);
+  rv = NS_NewLocalFileInputStream(getter_AddRefs(inputStream), aFile);
   if (NS_SUCCEEDED(rv) && inputStream)
   {
-    PRInt32 inputBufferSize = 10240;
+    int32_t inputBufferSize = 10240;
     nsMsgLineStreamBuffer *inputStreamBuffer =
       new nsMsgLineStreamBuffer(inputBufferSize, true, false);
-    PRInt64 fileSize;
+    int64_t fileSize;
     aFile->GetFileSize(&fileSize);
     msgParser->SetNewMsgHdr(newMsgHdr);
     msgParser->SetState(nsIMsgParseMailMsgState::ParseHeadersState);
     msgParser->SetEnvelopePos(0);
     bool needMoreData = false;
-    char * newLine = nsnull;
-    PRUint32 numBytesInLine = 0;
+    char * newLine = nullptr;
+    uint32_t numBytesInLine = 0;
     // we only have to read the headers, because we know the message size
     // from the file size. So we can do this in one time slice.
     do
@@ -1057,10 +1055,10 @@ nsresult MaildirStoreParser::ParseNextMessage(nsIFile *aFile)
 
     msgParser->FinishHeader();
     // A single message needs to be less than 4GB
-    newMsgHdr->SetMessageSize((PRUint32) fileSize);
+    newMsgHdr->SetMessageSize((uint32_t) fileSize);
     m_db->AddNewHdrToDB(newMsgHdr, true);
-    nsCAutoString storeToken;
-    localFile->GetNativeLeafName(storeToken);
+    nsAutoCString storeToken;
+    aFile->GetNativeLeafName(storeToken);
     newMsgHdr->SetStringProperty("storeToken", storeToken.get());
   }
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1088,7 +1086,7 @@ void MaildirStoreParser::TimerCallback(nsITimer *aTimer, void *aClosure)
       {
         nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(mailboxurl);
         url->SetUpdatingFolder(true);
-        nsCAutoString uriSpec("mailbox://");
+        nsAutoCString uriSpec("mailbox://");
         // ### TODO - what if SetSpec fails?
         (void) url->SetSpec(uriSpec);
         parser->m_listener->OnStopRunningUrl(url, NS_OK);
@@ -1100,7 +1098,7 @@ void MaildirStoreParser::TimerCallback(nsITimer *aTimer, void *aClosure)
   parser->m_directoryEnumerator->GetNext(getter_AddRefs(aSupport));
   nsresult rv;
   nsCOMPtr<nsIFile> currentFile(do_QueryInterface(aSupport, &rv));
-  NS_ENSURE_SUCCESS(rv,);
+  NS_ENSURE_SUCCESS_VOID(rv);
   parser->ParseNextMessage(currentFile);
   // ### TODO - what if this fails?
 }
@@ -1123,7 +1121,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RebuildIndex(nsIMsgFolder *aFolder,
   NS_ENSURE_ARG_POINTER(aFolder);
   // This code needs to iterate over the maildir files, and parse each
   // file and add a msg hdr to the db for the file.
-  nsCOMPtr<nsILocalFile> path;
+  nsCOMPtr<nsIFile> path;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
   path->Append(NS_LITERAL_STRING("cur"));
@@ -1141,16 +1139,16 @@ NS_IMETHODIMP nsMsgMaildirStore::RebuildIndex(nsIMsgFolder *aFolder,
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::ChangeFlags(nsIArray *aHdrArray,
-                                             PRUint32 aFlags,
+                                             uint32_t aFlags,
                                              bool aSet)
 {
   NS_ENSURE_ARG_POINTER(aHdrArray);
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     // get output stream for header
@@ -1171,7 +1169,7 @@ nsMsgMaildirStore::GetOutputStream(nsIMsgDBHdr *aHdr,
                                    nsCOMPtr<nsIOutputStream> &aOutputStream)
 {
   // file name is stored in message header property "storeToken"
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
     return NS_ERROR_FAILURE;
@@ -1180,7 +1178,7 @@ nsMsgMaildirStore::GetOutputStream(nsIMsgDBHdr *aHdr,
   nsresult rv = aHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsILocalFile> folderPath;
+  nsCOMPtr<nsIFile> folderPath;
   rv = folder->GetFilePath(getter_AddRefs(folderPath));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1189,8 +1187,7 @@ nsMsgMaildirStore::GetOutputStream(nsIMsgDBHdr *aHdr,
   maildirFile->Append(NS_LITERAL_STRING("cur"));
   maildirFile->AppendNative(fileName);
 
-  nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(maildirFile);
-  return MsgGetFileStream(localFile, getter_AddRefs(aOutputStream));
+  return MsgGetFileStream(maildirFile, getter_AddRefs(aOutputStream));
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
@@ -1202,20 +1199,19 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
   nsCOMPtr<nsIOutputStream> outputStream;
   nsCOMPtr<nsISeekableStream> seekableStream;
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!messageCount)
     return NS_ERROR_INVALID_ARG;
 
-  nsLineBuffer<char> *lineBuffer;
-  rv = NS_InitLineBuffer(&lineBuffer);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoPtr<nsLineBuffer<char> > lineBuffer(new nsLineBuffer<char>);
+  NS_ENSURE_TRUE(lineBuffer, NS_ERROR_OUT_OF_MEMORY);
 
   nsTArray<nsCString> keywordArray;
   ParseString(aKeywords, ' ', keywordArray);
 
-  for (PRUint32 i = 0; i < messageCount; ++i) // for each message
+  for (uint32_t i = 0; i < messageCount; ++i) // for each message
   {
     nsCOMPtr<nsIMsgDBHdr> message = do_QueryElementAt(aHdrArray, i, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1227,9 +1223,9 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
     NS_ENSURE_SUCCESS(rv, rv);
     nsCOMPtr <nsISeekableStream> seekableStream(do_QueryInterface(inputStream, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
-    PRUint32 statusOffset = 0;
+    uint32_t statusOffset = 0;
     (void)message->GetStatusOffset(&statusOffset);
-    PRUint64 desiredOffset = statusOffset;
+    uint64_t desiredOffset = statusOffset;
 
     ChangeKeywordsHelper(message, desiredOffset, lineBuffer, keywordArray,
                          aAdd, outputStream, seekableStream, inputStream);
@@ -1239,7 +1235,7 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
     // we need to rewrite the message file with extra room for the keywords,
     // or schedule some sort of background task to do this.
   }
-  PR_Free(lineBuffer);
+  lineBuffer = nullptr;
   return NS_OK;
 }
 
@@ -1248,7 +1244,7 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
  * c:\Inbox, it will return c:\Inbox.sbd if it succeeds. Path is strictly
  * an out parameter.
  */
-nsresult nsMsgMaildirStore::GetDirectoryForFolder(nsILocalFile *path)
+nsresult nsMsgMaildirStore::GetDirectoryForFolder(nsIFile *path)
 {
   // add directory separator to the path
   nsAutoString leafName;
@@ -1257,7 +1253,7 @@ nsresult nsMsgMaildirStore::GetDirectoryForFolder(nsILocalFile *path)
   return path->SetLeafName(leafName);
 }
 
-nsresult nsMsgMaildirStore::CreateDirectoryForFolder(nsILocalFile *path,
+nsresult nsMsgMaildirStore::CreateDirectoryForFolder(nsIFile *path,
                                                      bool aIsServer)
 {
   nsresult rv = NS_OK;
