@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "prmem.h"
 #include "plstr.h"
@@ -57,6 +25,7 @@
 #include "nsIMutableArray.h"
 #include "nsMsgUtils.h"
 #include "nsComponentManagerUtils.h"
+#include "mozilla/Likely.h"
 
 static NS_DEFINE_CID(kCPop3ServiceCID, NS_POP3SERVICE_CID);
 
@@ -199,7 +168,7 @@ NS_IMETHODIMP nsPop3IncomingServer::GetDeferredToAccount(nsACString& aRetVal)
                   rv = subFolderDB->ListAllKeys(keys);
                   nsCOMPtr<nsIMutableArray> hdrsToCopy(do_CreateInstance(NS_ARRAY_CONTRACTID));
                   MsgGetHeadersFromKeys(subFolderDB, keys->m_keys, hdrsToCopy);
-                  PRUint32 numHdrs = 0;
+                  uint32_t numHdrs = 0;
                   if (hdrsToCopy)
                     hdrsToCopy->GetLength(&numHdrs);
                   if (numHdrs)
@@ -212,7 +181,7 @@ NS_IMETHODIMP nsPop3IncomingServer::GetDeferredToAccount(nsACString& aRetVal)
                                                     getter_AddRefs(dest));
                     if (dest)
                       dest->CopyMessages(subFolder, hdrsToCopy, false,
-                                         nsnull, nsnull, false,false);
+                                         nullptr, nullptr, false,false);
                     // Should we copy the folder if the dest doesn't exist?
                   }
                 }
@@ -238,7 +207,7 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(const nsACString& aAcco
 {
   nsCString deferredToAccount;
   GetDeferredToAccount(deferredToAccount);
-  m_rootMsgFolder = nsnull; // clear this so we'll recalculate it on demand.
+  m_rootMsgFolder = nullptr; // clear this so we'll recalculate it on demand.
   //Notify listeners who listen to every folder
 
   nsresult rv =  SetCharValue("deferred_to_account", aAccountKey);
@@ -292,7 +261,7 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(const nsACString& aAcco
                   rv = server->GetRootFolder(getter_AddRefs(rootFolder));
                   NS_ENSURE_SUCCESS(rv, rv);
                   // this will fail if it already exists, which is fine.
-                  rootFolder->CreateSubfolder(NS_LITERAL_STRING("Inbox"), nsnull);
+                  rootFolder->CreateSubfolder(NS_LITERAL_STRING("Inbox"), nullptr);
                 }
               }
             }
@@ -320,14 +289,14 @@ NS_IMETHODIMP nsPop3IncomingServer::SetAuthenticated(bool aAuthenticated)
 }
 
 nsresult
-nsPop3IncomingServer::GetPop3CapabilityFlags(PRUint32 *flags)
+nsPop3IncomingServer::GetPop3CapabilityFlags(uint32_t *flags)
 {
   *flags = m_capabilityFlags;
   return NS_OK;
 }
 
 nsresult
-nsPop3IncomingServer::SetPop3CapabilityFlags(PRUint32 flags)
+nsPop3IncomingServer::SetPop3CapabilityFlags(uint32_t flags)
 {
   m_capabilityFlags = flags;
   return NS_OK;
@@ -389,10 +358,10 @@ nsresult nsPop3IncomingServer::GetInbox(nsIMsgWindow *msgWindow, nsIMsgFolder **
     rv = (*inbox)->GetMsgDatabase(getter_AddRefs(db));
     if (NS_FAILED(rv))
     {
-      (*inbox)->SetMsgDatabase(nsnull);
+      (*inbox)->SetMsgDatabase(nullptr);
       (void) localInbox->SetCheckForNewMessagesAfterParsing(true);
       // this will cause a reparse of the mail folder.
-      localInbox->GetDatabaseWithReparse(nsnull, msgWindow, getter_AddRefs(db));
+      localInbox->GetDatabaseWithReparse(nullptr, msgWindow, getter_AddRefs(db));
       rv = NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
     }
   }
@@ -436,7 +405,7 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow *aMsgWindow)
       if (NS_SUCCEEDED(rv) && db)
         rv = db->GetSummaryValid(&valid);
       if (NS_SUCCEEDED(rv) && valid)
-        rv = pop3Service->GetNewMail(aMsgWindow, urlListener, inbox, this, nsnull);
+        rv = pop3Service->GetNewMail(aMsgWindow, urlListener, inbox, this, nullptr);
       else
       {
         bool isLocked;
@@ -449,7 +418,7 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow *aMsgWindow)
     }
   }
   else
-    rv = pop3Service->CheckForNewMail(aMsgWindow, urlListener, inbox, this, nsnull);
+    rv = pop3Service->CheckForNewMail(aMsgWindow, urlListener, inbox, this, nullptr);
   return NS_OK;
 }
 
@@ -464,13 +433,8 @@ nsPop3IncomingServer::SetFlagsOnDefaultMailboxes()
   NS_ENSURE_SUCCESS(rv, rv);
 
   // pop3 gets an inbox, but no queue (unsent messages)
-  localFolder->SetFlagsOnDefaultMailboxes(nsMsgFolderFlags::Inbox |
-                                          nsMsgFolderFlags::Archive |
-                                          nsMsgFolderFlags::SentMail |
-                                          nsMsgFolderFlags::Drafts |
-                                          nsMsgFolderFlags::Templates |
-                                          nsMsgFolderFlags::Trash |
-                                          nsMsgFolderFlags::Junk);
+  localFolder->SetFlagsOnDefaultMailboxes(nsMsgFolderFlags::SpecialUse &
+                                         ~nsMsgFolderFlags::Queue);
   return NS_OK;
 }
 
@@ -566,7 +530,7 @@ nsPop3IncomingServer::GetNewMessages(nsIMsgFolder *aFolder, nsIMsgWindow *aMsgWi
     aFolder->GetServer(getter_AddRefs(server));
     GetDeferredServers(server, getter_AddRefs(deferredServers));
   }
-  PRUint32 numDeferredServers;
+  uint32_t numDeferredServers;
   if (deferredToAccount.IsEmpty() && deferredServers && NS_SUCCEEDED(deferredServers->Count(&numDeferredServers))
     && numDeferredServers > 0)
   {
@@ -619,7 +583,7 @@ nsPop3IncomingServer::CloseCachedConnections()
 }
 
 NS_IMETHODIMP
-nsPop3IncomingServer::GetOfflineSupportLevel(PRInt32 *aSupportLevel)
+nsPop3IncomingServer::GetOfflineSupportLevel(int32_t *aSupportLevel)
 {
   NS_ENSURE_ARG_POINTER(aSupportLevel);
 
@@ -647,25 +611,23 @@ NS_IMETHODIMP nsPop3IncomingServer::GetRunningProtocol(nsIPop3Protocol **aProtoc
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char *aUidl, PRInt32 aMark)
+NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char *aUidl, int32_t aMark)
 {
-  Pop3UidlEntry *uidlEntry;
-  nsresult rv = NS_ERROR_OUT_OF_MEMORY;
+  NS_ENSURE_ARG_POINTER(aUidl);
 
-  uidlEntry = PR_NEWZAP(Pop3UidlEntry);
-  if (uidlEntry)
-  {
-    uidlEntry->uidl = strdup(aUidl);
-    if (uidlEntry->uidl)
-    {
-      uidlEntry->status = (aMark == POP3_DELETE) ? DELETE_CHAR :
-      (aMark == POP3_FETCH_BODY) ? FETCH_BODY : KEEP;
-      m_uidlsToMark.AppendElement(uidlEntry);
-      rv = NS_OK;
-    } else
-      PR_Free(uidlEntry);
+  Pop3UidlEntry *uidlEntry = PR_NEWZAP(Pop3UidlEntry);
+  NS_ENSURE_TRUE(uidlEntry, NS_ERROR_OUT_OF_MEMORY);
+
+  uidlEntry->uidl = strdup(aUidl);
+  if (MOZ_UNLIKELY(!uidlEntry->uidl)) {
+    PR_Free(uidlEntry);
+    return NS_ERROR_OUT_OF_MEMORY;
   }
-  return rv;
+
+  uidlEntry->status = (aMark == POP3_DELETE) ? DELETE_CHAR :
+                      (aMark == POP3_FETCH_BODY) ? FETCH_BODY : KEEP;
+  m_uidlsToMark.AppendElement(uidlEntry);
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsPop3IncomingServer::MarkMessages()
@@ -677,7 +639,7 @@ NS_IMETHODIMP nsPop3IncomingServer::MarkMessages()
   {
     nsCString hostName;
     nsCString userName;
-    nsCOMPtr<nsILocalFile> localPath;
+    nsCOMPtr<nsIFile> localPath;
 
     GetLocalPath(getter_AddRefs(localPath));
 
@@ -686,8 +648,8 @@ NS_IMETHODIMP nsPop3IncomingServer::MarkMessages()
     // do it all in one fell swoop
     rv = nsPop3Protocol::MarkMsgForHost(hostName.get(), userName.get(), localPath, m_uidlsToMark);
   }
-  PRUint32 count = m_uidlsToMark.Count();
-  for (PRUint32 i = 0; i < count; i++)
+  uint32_t count = m_uidlsToMark.Count();
+  for (uint32_t i = 0; i < count; i++)
   {
     Pop3UidlEntry *ue = static_cast<Pop3UidlEntry*>(m_uidlsToMark[i]);
     PR_Free(ue->uidl);
@@ -745,7 +707,7 @@ nsPop3GetMailChainer::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
 nsresult nsPop3GetMailChainer::RunNextGetNewMail()
 {
   nsresult rv;
-  PRUint32 numServersLeft;
+  uint32_t numServersLeft;
   m_serversToGetNewMailFor->Count(&numServersLeft);
 
   for (; numServersLeft > 0;)
@@ -776,7 +738,7 @@ nsresult nsPop3GetMailChainer::RunNextGetNewMail()
       }
     }
   }
-  rv = m_listener ? m_listener->OnStopRunningUrl(nsnull, NS_OK) : NS_OK;
+  rv = m_listener ? m_listener->OnStopRunningUrl(nullptr, NS_OK) : NS_OK;
   Release(); // release ref to ourself.
   return rv;
 }
