@@ -32,7 +32,7 @@
 #include "nsIMsgHeaderParser.h"
 #include "nsILineInputStream.h"
 #include "nsISeekableStream.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIFileStreams.h"
 #include "nsNetCID.h"
 #include "nsMsgMessageFlags.h"
@@ -65,7 +65,7 @@ STDMETHODIMP CMapiImp::QueryInterface(const IID& aIid, void** aPpv)
     }
     else
     {
-        *aPpv = nsnull;
+        *aPpv = nullptr;
         return E_NOINTERFACE;
     }
 
@@ -80,7 +80,7 @@ STDMETHODIMP_(ULONG) CMapiImp::AddRef()
 
 STDMETHODIMP_(ULONG) CMapiImp::Release() 
 {
-    PRInt32 temp;
+    int32_t temp;
     temp = PR_ATOMIC_DECREMENT(&m_cRef);
     if (m_cRef == 0)
     {
@@ -117,7 +117,7 @@ STDMETHODIMP CMapiImp::Initialize()
     // Initialize MAPI Configuration
 
     nsMAPIConfiguration *pConfig = nsMAPIConfiguration::GetMAPIConfiguration();
-    if (pConfig != nsnull)
+    if (pConfig != nullptr)
       hr = S_OK;
 
     PR_Unlock(m_Lock);
@@ -137,7 +137,7 @@ STDMETHODIMP CMapiImp::Login(unsigned long aUIArg, LOGIN_PW_TYPE aLogin, LOGIN_P
         bNewSession = true;
 
     // Check For Profile Name
-    if (aLogin != nsnull && aLogin[0] != '\0')
+    if (aLogin != nullptr && aLogin[0] != '\0')
     {
         if (!nsMapiHook::VerifyUserName(nsString(aLogin), id_key))
         {
@@ -165,11 +165,11 @@ STDMETHODIMP CMapiImp::Login(unsigned long aUIArg, LOGIN_PW_TYPE aLogin, LOGIN_P
     }
 
     // finally register(create) the session.
-    PRUint32 nSession_Id;
-    PRInt16 nResult = 0;
+    uint32_t nSession_Id;
+    int16_t nResult = 0;
 
     nsMAPIConfiguration *pConfig = nsMAPIConfiguration::GetMAPIConfiguration();
-    if (pConfig != nsnull)
+    if (pConfig != nullptr)
         nResult = pConfig->RegisterSession(aUIArg, aLogin, aPassWord,
                                            (aFlags & MAPI_FORCE_DOWNLOAD), bNewSession,
                                            &nSession_Id, id_key.get());
@@ -415,11 +415,11 @@ STDMETHODIMP CMapiImp::FindNext(unsigned long aSession, unsigned long ulUIParam,
 STDMETHODIMP CMapiImp::ReadMail(unsigned long aSession, unsigned long ulUIParam, LPTSTR lpszMessageID,
                               unsigned long flFlags, unsigned long ulReserved, lpnsMapiMessage *lppMessage)
 {
-  PRInt32 irv;
-  nsCAutoString keyString((char *) lpszMessageID);
+  nsresult irv;
+  nsAutoCString keyString((char *) lpszMessageID);
   PR_LOG(MAPI, PR_LOG_DEBUG, ("CMapiImp::ReadMail asking for key %s\n", (char *) lpszMessageID));
   nsMsgKey msgKey = keyString.ToInteger(&irv);
-  if (irv)
+  if (NS_FAILED(irv))
   {
     NS_ASSERTION(false, "invalid lpszMessageID");
     return MAPI_E_INVALID_MESSAGE;
@@ -441,10 +441,11 @@ STDMETHODIMP CMapiImp::ReadMail(unsigned long aSession, unsigned long ulUIParam,
 STDMETHODIMP CMapiImp::DeleteMail(unsigned long aSession, unsigned long ulUIParam, LPTSTR lpszMessageID,
                               unsigned long flFlags, unsigned long ulReserved)
 {
-  PRInt32 irv;
-  nsCAutoString keyString((char *) lpszMessageID);
+  nsresult irv;
+  nsAutoCString keyString((char *) lpszMessageID);
   nsMsgKey msgKey = keyString.ToInteger(&irv);
-  if (irv)
+  // XXX Why do we return success on failure?
+  if (NS_FAILED(irv))
     return SUCCESS_SUCCESS;
   MsgMapiListContext *listContext;
   LONG ret = InitContext(aSession, &listContext);
@@ -468,7 +469,7 @@ STDMETHODIMP CMapiImp::Logoff (unsigned long aSession)
 {
     nsMAPIConfiguration *pConfig = nsMAPIConfiguration::GetMAPIConfiguration();
 
-    if (pConfig->UnRegisterSession((PRUint32)aSession))
+    if (pConfig->UnRegisterSession((uint32_t)aSession))
         return S_OK;
 
     return E_FAIL;
@@ -511,7 +512,7 @@ MsgMapiListContext::IsIMAPHost(void)
     return FALSE;
   nsCOMPtr <nsIMsgImapMailFolder> imapFolder = do_QueryInterface(m_folder);
 
-  return imapFolder != nsnull;
+  return imapFolder != nullptr;
 }
 
 nsMsgKey MsgMapiListContext::GetNext ()
@@ -539,7 +540,7 @@ nsMsgKey MsgMapiListContext::GetNext ()
         
         // If this is an IMAP message, we have to make sure we have a valid
         // body to work with.
-        PRUint32  flags = 0;
+        uint32_t  flags = 0;
         
         (void) msgHdr->GetFlags(&flags);
         if (flags & nsMsgMessageFlags::Offline) 
@@ -561,7 +562,7 @@ nsresult MsgMapiListContext::MarkRead (nsMsgKey key, bool read)
   nsresult err = NS_ERROR_FAILURE;
   NS_ASSERTION(m_db, "no db");
   if (m_db)
-    err = m_db->MarkRead (key, read, nsnull);
+    err = m_db->MarkRead (key, read, nullptr);
   return err;
 }
 
@@ -582,13 +583,13 @@ lpnsMapiMessage MsgMapiListContext::GetMessage (nsMsgKey key, unsigned long flFl
       msgHdr->GetSubject (getter_Copies(subject));
       message->lpszSubject = (char *) CoTaskMemAlloc(subject.Length() + 1);
       strcpy((char *) message->lpszSubject, subject.get());
-      PRUint32 date;
+      uint32_t date;
       (void) msgHdr->GetDateInSeconds(&date);
       message->lpszDateReceived = ConvertDateToMapiFormat (date);
       
       // Pull out the flags info
       // anything to do with MAPI_SENT? Since we're only reading the Inbox, I guess not
-      PRUint32 ourFlags;
+      uint32_t ourFlags;
       (void) msgHdr->GetFlags(&ourFlags);
       if (!(ourFlags & nsMsgMessageFlags::Read))
         message->flFlags |= MAPI_UNREAD;
@@ -597,7 +598,7 @@ lpnsMapiMessage MsgMapiListContext::GetMessage (nsMsgKey key, unsigned long flFl
       
       nsCOMPtr<nsIMsgHeaderParser> parser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
       if (!parser)
-        return nsnull;
+        return nullptr;
       // Pull out the author/originator info
       message->lpOriginator = (lpnsMapiRecipDesc) CoTaskMemAlloc (sizeof(nsMapiRecipDesc));
       memset(message->lpOriginator, 0, sizeof(nsMapiRecipDesc));
@@ -611,11 +612,11 @@ lpnsMapiMessage MsgMapiListContext::GetMessage (nsMsgKey key, unsigned long flFl
       msgHdr->GetRecipients(getter_Copies(recipients));
       msgHdr->GetCcList(getter_Copies(ccList));
 
-      PRUint32 numToRecips;
-      PRUint32 numCCRecips;
-      parser->ParseHeaderAddresses(recipients.get(), nsnull, nsnull,
+      uint32_t numToRecips;
+      uint32_t numCCRecips;
+      parser->ParseHeaderAddresses(recipients.get(), nullptr, nullptr,
 				   &numToRecips);
-      parser->ParseHeaderAddresses(ccList.get(), nsnull, nsnull, &numCCRecips);
+      parser->ParseHeaderAddresses(ccList.get(), nullptr, nullptr, &numCCRecips);
 
       message->lpRecips = (lpnsMapiRecipDesc) CoTaskMemAlloc ((numToRecips + numCCRecips) * sizeof(MapiRecipDesc));
       memset(message->lpRecips, 0, (numToRecips + numCCRecips) * sizeof(MapiRecipDesc));
@@ -634,7 +635,7 @@ lpnsMapiMessage MsgMapiListContext::GetMessage (nsMsgKey key, unsigned long flFl
       
     }
     if (! (flFlags & (MAPI_PEEK | MAPI_ENVELOPE_ONLY)))
-      m_db->MarkRead(key, true, nsnull);
+      m_db->MarkRead(key, true, nullptr);
   }
   return message;
 }
@@ -659,12 +660,12 @@ char *MsgMapiListContext::ConvertDateToMapiFormat (time_t ourTime)
 void MsgMapiListContext::ConvertRecipientsToMapiFormat (nsIMsgHeaderParser *parser, const char *recipients, lpnsMapiRecipDesc mapiRecips,
                                                         int mapiRecipClass)
 {
-  char *names = nsnull;
-  char *addresses = nsnull;
+  char *names = nullptr;
+  char *addresses = nullptr;
   
   if (!parser)
     return ;
-  PRUint32 numAddresses = 0;
+  uint32_t numAddresses = 0;
   parser->ParseHeaderAddresses(recipients, &names, &addresses, &numAddresses);
   
   if (numAddresses > 0)
@@ -713,15 +714,15 @@ char *MsgMapiListContext::ConvertBodyToMapiFormat (nsIMsgDBHdr *hdr)
   nsCOMPtr <nsIMsgFolder> folder;
   hdr->GetFolder(getter_AddRefs(folder));
   if (!folder)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr <nsIInputStream> inputStream;
-  nsCOMPtr <nsILocalFile> localFile;
+  nsCOMPtr <nsIFile> localFile;
   folder->GetFilePath(getter_AddRefs(localFile));
 
   nsresult rv;
   nsCOMPtr<nsIFileInputStream> fileStream = do_CreateInstance(NS_LOCALFILEINPUTSTREAM_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
   rv = fileStream->Init(localFile,  PR_RDONLY, 0664, false);  //just have to read the messages
   inputStream = do_QueryInterface(fileStream);
@@ -730,16 +731,16 @@ char *MsgMapiListContext::ConvertBodyToMapiFormat (nsIMsgDBHdr *hdr)
   {
     nsCOMPtr <nsILineInputStream> fileLineStream = do_QueryInterface(inputStream);
     if (!fileLineStream)
-      return nsnull;
+      return nullptr;
     // ### really want to skip past headers...
-    PRUint64 messageOffset;
-    PRUint32 lineCount;
+    uint64_t messageOffset;
+    uint32_t lineCount;
     hdr->GetMessageOffset(&messageOffset);
     hdr->GetLineCount(&lineCount);
     nsCOMPtr <nsISeekableStream> seekableStream = do_QueryInterface(inputStream);
     seekableStream->Seek(PR_SEEK_SET, messageOffset);
     bool hasMore = true;
-    nsCAutoString curLine;
+    nsAutoCString curLine;
     bool inMessageBody = false;
     nsresult rv = NS_OK;
     while (hasMore) // advance past message headers
@@ -748,7 +749,7 @@ char *MsgMapiListContext::ConvertBodyToMapiFormat (nsIMsgDBHdr *hdr)
       if (NS_FAILED(rv) || EMPTY_MESSAGE_LINE(curLine))
         break;
     }
-    PRUint32 msgSize;
+    uint32_t msgSize;
     hdr->GetMessageSize(&msgSize);
     if (msgSize > kBufLen)
       msgSize = kBufLen - 1;
@@ -756,8 +757,8 @@ char *MsgMapiListContext::ConvertBodyToMapiFormat (nsIMsgDBHdr *hdr)
     char *body = (char*) CoTaskMemAlloc (msgSize + 1);
 
     if (!body)
-      return nsnull;
-    PRInt32 bytesCopied = 0;
+      return nullptr;
+    int32_t bytesCopied = 0;
     for (hasMore = TRUE; lineCount > 0 && hasMore && NS_SUCCEEDED(rv); lineCount--)
     {
       rv = fileLineStream->ReadLine(curLine, &hasMore);
@@ -776,7 +777,7 @@ char *MsgMapiListContext::ConvertBodyToMapiFormat (nsIMsgDBHdr *hdr)
     body[bytesCopied] = '\0';   // rhp - fix last line garbage...
     return body;
   }
-  return nsnull;
+  return nullptr;
 }
 
 
@@ -822,13 +823,13 @@ extern "C" void MSG_FreeMapiMessage (lpMapiMessage msg)
       msg_FreeMAPIRecipient(msg->lpOriginator);
     
     for (i=0; i<msg->nRecipCount; i++)
-      if (&(msg->lpRecips[i]) != nsnull)
+      if (&(msg->lpRecips[i]) != nullptr)
         msg_FreeMAPIRecipient(&(msg->lpRecips[i]));
       
       CoTaskMemFree(msg->lpRecips);
       
       for (i=0; i<msg->nFileCount; i++)
-        if (&(msg->lpFiles[i]) != nsnull)
+        if (&(msg->lpFiles[i]) != nullptr)
           msg_FreeMAPIFile(&(msg->lpFiles[i]));
         
       CoTaskMemFree(msg->lpFiles);
@@ -862,7 +863,7 @@ MsgMapiListContext::DeleteMessage(nsMsgKey key)
   
   if ( !IsIMAPHost() )
   {
-    return NS_SUCCEEDED((m_db->DeleteMessages(1, &key, nsnull)));
+    return NS_SUCCEEDED((m_db->DeleteMessages(1, &key, nullptr)));
   }
 #if 0 
   else if ( m_folder->GetIMAPFolderInfoMail() )
@@ -871,7 +872,7 @@ MsgMapiListContext::DeleteMessage(nsMsgKey key)
     messageKeys.AppendElement(key);
 
     (m_folder->GetIMAPFolderInfoMail())->DeleteSpecifiedMessages(pane, messageKeys, nsMsgKey_None);
-    m_db->DeleteMessage(key, nsnull, FALSE);
+    m_db->DeleteMessage(key, nullptr, FALSE);
     return TRUE;
   }
 #endif
