@@ -1,42 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Henrik Gemal <mozilla@gemal.dk>
- *   Karsten Düsterloh <mnyromyr@tprac.de>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * formerly listngst.cpp
@@ -59,7 +24,7 @@
 #include "nsIMsgIncomingServer.h"
 #include "nsINntpIncomingServer.h"
 #include "nsMsgBaseCID.h"
-
+#include "nsIMsgFilter.h"
 #include "nsNNTPNewsgroupList.h"
 
 #include "nsINNTPArticleList.h"
@@ -95,6 +60,7 @@
 #include "nsIMsgFilterCustomAction.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
+#include "mozilla/Services.h"
 
 // update status on header download once per second
 #define MIN_STATUS_UPDATE_INTERVAL PRTime(PR_USEC_PER_SEC)
@@ -107,15 +73,15 @@ nsNNTPNewsgroupList::nsNNTPNewsgroupList()
   m_downloadAll(false),
   m_maxArticles(0),
   m_lastPercent(-1),
+  m_lastStatusUpdate(0),
   m_lastProcessedNumber(0),
   m_firstMsgNumber(0),
   m_lastMsgNumber(0),
   m_firstMsgToDownload(0),
   m_lastMsgToDownload(0),
-  m_set(nsnull)
+  m_set(nullptr)
 {
   memset(&m_knownArts, 0, sizeof(m_knownArts));
-  m_lastStatusUpdate = LL_Zero();
 }
 
 nsNNTPNewsgroupList::~nsNNTPNewsgroupList()
@@ -150,14 +116,14 @@ nsNNTPNewsgroupList::Initialize(nsINntpUrl *runningURL, nsIMsgNewsFolder *newsFo
 
   rv = server->GetFilterList(m_msgWindow, getter_AddRefs(m_serverFilterList));
   NS_ENSURE_SUCCESS(rv,rv);
-  nsCAutoString servHeaders;
+  nsAutoCString servHeaders;
   m_serverFilterList->GetArbitraryHeaders(servHeaders);
 
   nsTArray<nsCString> servArray;
   ParseString(servHeaders, ' ', servArray);
 
   // servArray may have duplicates already in m_filterHeaders.
-  for (PRUint32 i = 0; i < servArray.Length(); i++)
+  for (uint32_t i = 0; i < servArray.Length(); i++)
   {
     if (m_filterHeaders.IndexOf(servArray[i]) == m_filterHeaders.NoIndex)
       m_filterHeaders.AppendElement(servArray[i]);
@@ -177,11 +143,11 @@ nsNNTPNewsgroupList::CleanUp()
     {
       nsCOMPtr <nsIDBFolderInfo> folderInfo;
       m_newsDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
-      PRInt32 firstKnown = m_knownArts.set->GetFirstMember();
-      PRInt32 lastKnown =  m_knownArts.set->GetLastMember();
+      int32_t firstKnown = m_knownArts.set->GetFirstMember();
+      int32_t lastKnown =  m_knownArts.set->GetLastMember();
       if (folderInfo)
       {
-        PRUint32 lastMissingCheck;
+        uint32_t lastMissingCheck;
         folderInfo->GetUint32Property("lastMissingCheck", 0, &lastMissingCheck);
         if (lastMissingCheck)
           firstKnown = lastMissingCheck + 1;
@@ -189,7 +155,7 @@ nsNNTPNewsgroupList::CleanUp()
       bool foundMissingArticle = false;
       while (firstKnown <= lastKnown)
       {
-        PRInt32 firstUnreadStart, firstUnreadEnd;
+        int32_t firstUnreadStart, firstUnreadEnd;
         if (firstKnown == 0)
           firstKnown = 1;
         m_set->FirstMissingRange(firstKnown, lastKnown, &firstUnreadStart, &firstUnreadEnd);
@@ -225,19 +191,19 @@ nsNNTPNewsgroupList::CleanUp()
     }
     m_newsDB->Commit(nsMsgDBCommitType::kSessionCommit);
     m_newsDB->Close(true);
-    m_newsDB = nsnull;
+    m_newsDB = nullptr;
   }
 
   if (m_knownArts.set)
   {
     delete m_knownArts.set;
-    m_knownArts.set = nsnull;
+    m_knownArts.set = nullptr;
   }
   if (m_newsFolder)
     m_newsFolder->NotifyFinishedDownloadinghdrs();
 
-  m_newsFolder = nsnull;
-  m_runningURL = nsnull;
+  m_newsFolder = nullptr;
+  m_runningURL = nullptr;
 
   return NS_OK;
 }
@@ -279,12 +245,12 @@ openWindow(nsIMsgWindow *aMsgWindow, const char *chromeURL,
 
 nsresult
 nsNNTPNewsgroupList::GetRangeOfArtsToDownload(nsIMsgWindow *aMsgWindow,
-                                              PRInt32 first_possible,
-                                              PRInt32 last_possible,
-                                              PRInt32 maxextra,
-                                              PRInt32 *first,
-                                              PRInt32 *last,
-                                              PRInt32 *status)
+                                              int32_t first_possible,
+                                              int32_t last_possible,
+                                              int32_t maxextra,
+                                              int32_t *first,
+                                              int32_t *last,
+                                              int32_t *status)
 {
   nsresult rv = NS_OK;
 
@@ -317,7 +283,7 @@ nsNNTPNewsgroupList::GetRangeOfArtsToDownload(nsIMsgWindow *aMsgWindow,
     rv = newsGroupInfo->GetHighWater(&mark);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    if (last_possible < ((PRInt32)mark))
+    if (last_possible < ((int32_t)mark))
       newsGroupInfo->SetHighWater(last_possible);
     if (m_knownArts.set)
       delete m_knownArts.set;
@@ -338,8 +304,9 @@ nsNNTPNewsgroupList::GetRangeOfArtsToDownload(nsIMsgWindow *aMsgWindow,
 
   if (m_knownArts.set->IsMember(last_possible)) {
     nsString statusString;
-    nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
+    NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
     nsCOMPtr<nsIStringBundle> bundle;
     rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
@@ -476,9 +443,9 @@ nsNNTPNewsgroupList::GetRangeOfArtsToDownload(nsIMsgWindow *aMsgWindow,
 }
 
 nsresult
-nsNNTPNewsgroupList::AddToKnownArticles(PRInt32 first, PRInt32 last)
+nsNNTPNewsgroupList::AddToKnownArticles(int32_t first, int32_t last)
 {
-  int status;
+  nsresult status;
 
   if (!m_knownArts.set)
   {
@@ -487,7 +454,8 @@ nsNNTPNewsgroupList::AddToKnownArticles(PRInt32 first, PRInt32 last)
       return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  status = m_knownArts.set->AddRange(first, last);
+  // XXX Casting int to nsresult
+  status = static_cast<nsresult>(m_knownArts.set->AddRange(first, last));
 
   if (m_newsDB) {
     nsresult rv = NS_OK;
@@ -504,7 +472,7 @@ nsNNTPNewsgroupList::AddToKnownArticles(PRInt32 first, PRInt32 last)
 }
 
 nsresult
-nsNNTPNewsgroupList::InitXOVER(PRInt32 first_msg, PRInt32 last_msg)
+nsNNTPNewsgroupList::InitXOVER(int32_t first_msg, int32_t last_msg)
 {
   /* Consistency checks, not that I know what to do if it fails (it will
    probably handle it OK...) */
@@ -529,12 +497,12 @@ nsNNTPNewsgroupList::InitXOVER(PRInt32 first_msg, PRInt32 last_msg)
 #define DATE_HEADER "Date: "
 
 nsresult
-nsNNTPNewsgroupList::ParseLine(char *line, PRUint32 * message_number)
+nsNNTPNewsgroupList::ParseLine(char *line, uint32_t * message_number)
 {
   nsresult rv = NS_OK;
   nsCOMPtr <nsIMsgDBHdr> newMsgHdr;
-  char *dateStr = nsnull;  // keep track of date str, for filters
-  char *authorStr = nsnull; // keep track of author str, for filters
+  char *dateStr = nullptr;  // keep track of date str, for filters
+  char *authorStr = nullptr; // keep track of author str, for filters
 
   if (!line || !message_number) {
     return NS_ERROR_NULL_POINTER;
@@ -562,9 +530,9 @@ nsNNTPNewsgroupList::ParseLine(char *line, PRUint32 * message_number)
   GET_TOKEN (); /* subject */
   if (line) {
     const char *subject = line;  /* #### const evilness */
-    PRUint32 subjectLen = strlen(line);
+    uint32_t subjectLen = strlen(line);
 
-    PRUint32 flags = 0;
+    uint32_t flags = 0;
     // ### should call IsHeaderRead here...
     /* strip "Re: " */
     nsCString modifiedSubject;
@@ -625,7 +593,7 @@ nsNNTPNewsgroupList::ParseLine(char *line, PRUint32 * message_number)
 
   GET_TOKEN (); /* bytes */
   if (line) {
-    PRUint32 msgSize = 0;
+    uint32_t msgSize = 0;
     msgSize = (line) ? atol (line) : 0;
 
     rv = newMsgHdr->SetMessageSize(msgSize);
@@ -634,7 +602,7 @@ nsNNTPNewsgroupList::ParseLine(char *line, PRUint32 * message_number)
 
   GET_TOKEN (); /* lines */
   if (line) {
-    PRUint32 numLines = 0;
+    uint32_t numLines = 0;
     numLines = line ? atol (line) : 0;
     rv = newMsgHdr->SetLineCount(numLines);
     if (NS_FAILED(rv)) return rv;
@@ -656,14 +624,13 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
   // you can't move news messages, so applyMore is always true
   *aApplyMore = true;
 
-  nsCOMPtr<nsISupportsArray> filterActionList;
-  nsresult rv = NS_NewISupportsArray(getter_AddRefs(filterActionList));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = aFilter->GetSortedActionList(filterActionList);
+  nsCOMPtr<nsIArray> filterActionList;
+
+  nsresult rv = aFilter->GetSortedActionList(getter_AddRefs(filterActionList));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 numActions;
-  rv = filterActionList->Count(&numActions);
+  uint32_t numActions;
+  rv = filterActionList->GetLength(&numActions);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool loggingEnabled = false;
@@ -672,11 +639,12 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
   if (NS_SUCCEEDED(rv) && currentFilterList && numActions)
     currentFilterList->GetLoggingEnabled(&loggingEnabled);
 
-  for (PRUint32 actionIndex = 0; actionIndex < numActions; actionIndex++)
+  for (uint32_t actionIndex = 0; actionIndex < numActions; actionIndex++)
   {
     nsCOMPtr<nsIMsgRuleAction> filterAction;
-    filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction), getter_AddRefs(filterAction));
-    if (!filterAction)
+    rv = filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction),
+                                                       getter_AddRefs(filterAction));
+    if (NS_FAILED(rv) || !filterAction)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -688,23 +656,23 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
         m_addHdrToDB = false;
         break;
       case nsMsgFilterAction::MarkRead:
-        m_newsDB->MarkHdrRead(m_newMsgHdr, true, nsnull);
+        m_newsDB->MarkHdrRead(m_newMsgHdr, true, nullptr);
         break;
       case nsMsgFilterAction::MarkUnread:
-        m_newsDB->MarkHdrRead(m_newMsgHdr, false, nsnull);
+        m_newsDB->MarkHdrRead(m_newMsgHdr, false, nullptr);
         break;
       case nsMsgFilterAction::KillThread:
         m_newMsgHdr->SetUint32Property("ProtoThreadFlags", nsMsgMessageFlags::Ignored);
         break;
       case nsMsgFilterAction::KillSubthread:
         {
-          PRUint32 newFlags;
+          uint32_t newFlags;
           m_newMsgHdr->OrFlags(nsMsgMessageFlags::Ignored, &newFlags);
         }
         break;
       case nsMsgFilterAction::WatchThread:
         {
-          PRUint32 newFlags;
+          uint32_t newFlags;
           m_newMsgHdr->OrFlags(nsMsgMessageFlags::Watched, &newFlags);
         }
         break;
@@ -752,7 +720,7 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
         rv = filterAction->GetCustomAction(getter_AddRefs(customAction));
         NS_ENSURE_SUCCESS(rv, rv);
 
-        nsCAutoString value;
+        nsAutoCString value;
         filterAction->GetStrValue(value);
 
         nsCOMPtr<nsIMutableArray> messageArray(
@@ -760,7 +728,7 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
         NS_ENSURE_TRUE(messageArray, rv);
         messageArray->AppendElement(m_newMsgHdr, false);
 
-        customAction->Apply(messageArray, value, nsnull,
+        customAction->Apply(messageArray, value, nullptr,
                             nsMsgFilterType::NewsRule, aMsgWindow);
       }
       break;
@@ -778,10 +746,10 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
 }
 
 nsresult
-nsNNTPNewsgroupList::ProcessXOVERLINE(const char *line, PRUint32 *status)
+nsNNTPNewsgroupList::ProcessXOVERLINE(const char *line, uint32_t *status)
 {
-  PRUint32 message_number=0;
-  //  PRInt32 lines;
+  uint32_t message_number=0;
+  //  int32_t lines;
   bool read_p = false;
   nsresult rv = NS_OK;
 
@@ -796,7 +764,7 @@ nsNNTPNewsgroupList::ProcessXOVERLINE(const char *line, PRUint32 *status)
       return NS_ERROR_OUT_OF_MEMORY;
     rv = ParseLine(xoverline, &message_number);
     PL_strfree(xoverline);
-    xoverline = nsnull;
+    xoverline = nullptr;
     if (NS_FAILED(rv))
       return rv;
   }
@@ -841,10 +809,10 @@ nsNNTPNewsgroupList::ProcessXOVERLINE(const char *line, PRUint32 *status)
   /* Update the progress meter with a percentage of articles retrieved */
   if (m_lastMsgNumber > m_firstMsgNumber)
   {
-    PRInt32 totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
-    PRInt32 lastIndex = m_lastProcessedNumber - m_firstMsgNumber + 1;
-    PRInt32 numDownloaded = lastIndex;
-    PRInt32 totIndex = m_lastMsgNumber - m_firstMsgNumber + 1;
+    int32_t totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
+    int32_t lastIndex = m_lastProcessedNumber - m_firstMsgNumber + 1;
+    int32_t numDownloaded = lastIndex;
+    int32_t totIndex = m_lastMsgNumber - m_firstMsgNumber + 1;
 
     PRTime elapsedTime = PR_Now() - m_lastStatusUpdate;
 
@@ -859,7 +827,7 @@ nsNNTPNewsgroupList::ResetXOVER()
 {
   m_lastMsgNumber = m_firstMsgNumber;
   m_lastProcessedNumber = m_lastMsgNumber;
-  return 0;
+  return NS_OK;
 }
 
 nsresult
@@ -882,7 +850,7 @@ nsNNTPNewsgroupList::FinishXOVERLINE(int status, int *newstatus)
 
   if (k && k->set)
   {
-    PRInt32 n = k->set->FirstNonMember();
+    int32_t n = k->set->FirstNonMember();
     if (n < k->first_possible || n > k->last_possible)
     {
       /* We know we've gotten all there is to know.
@@ -900,7 +868,7 @@ nsNNTPNewsgroupList::FinishXOVERLINE(int status, int *newstatus)
     m_finishingXover = true;
 
     // XXX is this correct?
-    m_runningURL = nsnull;
+    m_runningURL = nullptr;
 
     if (m_lastMsgNumber > 0) {
       nsAutoString firstStr;
@@ -910,8 +878,9 @@ nsNNTPNewsgroupList::FinishXOVERLINE(int status, int *newstatus)
       lastStr.AppendInt(m_lastMsgNumber - m_firstMsgNumber + 1);
 
       nsString statusString;
-      nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-      NS_ENSURE_SUCCESS(rv, rv);
+      nsCOMPtr<nsIStringBundleService> bundleService =
+        mozilla::services::GetStringBundleService();
+      NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
       nsCOMPtr<nsIStringBundle> bundle;
       rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
@@ -949,34 +918,33 @@ nsNNTPNewsgroupList::InitXHDR(nsACString &header)
 NS_IMETHODIMP
 nsNNTPNewsgroupList::ProcessXHDRLine(const nsACString &line)
 {
-  PRInt32 middle = line.FindChar(' ');
+  int32_t middle = line.FindChar(' ');
   nsCString value, key = PromiseFlatCString(line);
   if (middle == -1)
     return NS_OK;
   value = Substring(line, middle+1);
-  key.SetLength((PRUint32)middle);
+  key.SetLength((uint32_t)middle);
 
   // According to RFC 2980, some will send (none) instead.
   // So we don't treat this is an error.
   if (key.CharAt(0) < '0' || key.CharAt(0) > '9')
     return NS_OK;
 
-  nsresult code;
-  PRInt32 number = key.ToInteger(&code);
-  if (code != NS_OK)
-    return NS_ERROR_FAILURE;
+  nsresult rv;
+  int32_t number = key.ToInteger(&rv);
+  NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   // RFC 2980 specifies one or more spaces.
   value.Trim(" ");
 
-  nsCOMPtr <nsIMsgDBHdr> header;
-  nsresult rv = m_newsDB->GetMsgHdrForKey(number, getter_AddRefs(header));
+  nsCOMPtr<nsIMsgDBHdr> header;
+  rv = m_newsDB->GetMsgHdrForKey(number, getter_AddRefs(header));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = header->SetStringProperty(m_filterHeaders[m_currentXHDRIndex].get(), value.get());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
-  PRInt32 numDownloaded = number - m_firstMsgNumber + 1;
+  int32_t totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
+  int32_t numDownloaded = number - m_firstMsgNumber + 1;
 
   PRTime elapsedTime = PR_Now() - m_lastStatusUpdate;
 
@@ -986,7 +954,7 @@ nsNNTPNewsgroupList::ProcessXHDRLine(const nsACString &line)
 }
 
 NS_IMETHODIMP
-nsNNTPNewsgroupList::InitHEAD(PRInt32 number)
+nsNNTPNewsgroupList::InitHEAD(int32_t number)
 {
   if (m_newMsgHdr)
   {
@@ -994,10 +962,10 @@ nsNNTPNewsgroupList::InitHEAD(PRInt32 number)
     // If HEAD didn't properly return, then the header won't be set
     m_newHeaders.AppendObject(m_newMsgHdr);
 
-    PRInt32 totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
-    PRInt32 lastIndex = m_lastProcessedNumber - m_firstMsgNumber + 1;
-    PRInt32 numDownloaded = lastIndex;
-    PRInt32 totIndex = m_lastMsgNumber - m_firstMsgNumber + 1;
+    int32_t totalToDownload = m_lastMsgToDownload - m_firstMsgToDownload + 1;
+    int32_t lastIndex = m_lastProcessedNumber - m_firstMsgNumber + 1;
+    int32_t numDownloaded = lastIndex;
+    int32_t totIndex = m_lastMsgNumber - m_firstMsgNumber + 1;
 
     PRTime elapsedTime = PR_Now() - m_lastStatusUpdate;
 
@@ -1027,7 +995,7 @@ nsNNTPNewsgroupList::InitHEAD(PRInt32 number)
 }
 
 NS_IMETHODIMP
-nsNNTPNewsgroupList::HEADFailed(PRInt32 number)
+nsNNTPNewsgroupList::HEADFailed(int32_t number)
 {
   m_set->Add(number);
   return NS_OK;
@@ -1036,12 +1004,12 @@ nsNNTPNewsgroupList::HEADFailed(PRInt32 number)
 NS_IMETHODIMP
 nsNNTPNewsgroupList::ProcessHEADLine(const nsACString &line)
 {
-  PRInt32 colon = line.FindChar(':');
+  int32_t colon = line.FindChar(':');
   nsCString header = PromiseFlatCString(line), value;
   if (colon != -1)
   {
     value = Substring(line, colon+1);
-    header.SetLength((PRUint32)colon);
+    header.SetLength((uint32_t)colon);
   }
   else if (line.CharAt(0) == ' ' || line.CharAt(0) == '\t') // We are continuing the header
   {
@@ -1086,9 +1054,9 @@ nsNNTPNewsgroupList::AddHeader(const char *header, const char *value)
   else if (PL_strcmp(header, "subject") == 0)
   {
     const char *subject = value;
-    PRUint32 subjectLen = strlen(value);
+    uint32_t subjectLen = strlen(value);
 
-    PRUint32 flags = 0;
+    uint32_t flags = 0;
     // ### should call IsHeaderRead here...
     /* strip "Re: " */
     nsCString modifiedSubject;
@@ -1134,26 +1102,26 @@ nsNNTPNewsgroupList::CallFilters()
   nsCOMPtr <nsIMsgFolder> folder = do_QueryInterface(m_newsFolder, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  PRUint32 filterCount = 0;
+  uint32_t filterCount = 0;
   if (m_filterList)
   {
     rv = m_filterList->GetFilterCount(&filterCount);
     NS_ENSURE_SUCCESS(rv,rv);
   }
 
-  PRUint32 serverFilterCount = 0;
+  uint32_t serverFilterCount = 0;
   if (m_serverFilterList)
   {
     rv = m_serverFilterList->GetFilterCount(&serverFilterCount);
     NS_ENSURE_SUCCESS(rv,rv);
   }
 
-  PRUint32 count = m_newHeaders.Count();
+  uint32_t count = m_newHeaders.Count();
 
   // Notify MsgFolderListeners of message adds
   nsCOMPtr<nsIMsgFolderNotificationService> notifier(do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
 
-  for (PRUint32 i = 0; i < count; i++)
+  for (uint32_t i = 0; i < count; i++)
   {
     m_newMsgHdr = m_newHeaders[i];
     if (!filterCount && !serverFilterCount)
@@ -1194,7 +1162,7 @@ nsNNTPNewsgroupList::CallFilters()
       fullHeaders += '\0';
     }
 
-    for (PRUint32 header = 0; header < m_filterHeaders.Length(); header++)
+    for (uint32_t header = 0; header < m_filterHeaders.Length(); header++)
     {
       nsCString retValue;
       m_newMsgHdr->GetStringProperty(m_filterHeaders[header].get(),
@@ -1243,7 +1211,7 @@ nsNNTPNewsgroupList::CallFilters()
 }
 
 void
-nsNNTPNewsgroupList::SetProgressBarPercent(PRInt32 percent)
+nsNNTPNewsgroupList::SetProgressBarPercent(int32_t percent)
 {
   if (!m_runningURL)
     return;
@@ -1277,11 +1245,11 @@ nsNNTPNewsgroupList::SetProgressStatus(const PRUnichar *message)
 }
 
 void
-nsNNTPNewsgroupList::UpdateStatus(bool filtering, PRInt32 numDLed, PRInt32 totToDL)
+nsNNTPNewsgroupList::UpdateStatus(bool filtering, int32_t numDLed, int32_t totToDL)
 {
-  PRInt32 numerator = (filtering ? m_currentXHDRIndex + 1 : 1) * numDLed;
-  PRInt32 denominator = (m_filterHeaders.Length() + 1) * totToDL;
-  PRInt32 percent = numerator * 100 / denominator;
+  int32_t numerator = (filtering ? m_currentXHDRIndex + 1 : 1) * numDLed;
+  int32_t denominator = (m_filterHeaders.Length() + 1) * totToDL;
+  int32_t percent = numerator * 100 / denominator;
   
   nsAutoString numDownloadedStr;
   numDownloadedStr.AppendInt(numDLed);
@@ -1295,8 +1263,9 @@ nsNNTPNewsgroupList::UpdateStatus(bool filtering, PRInt32 numDLed, PRInt32 totTo
       return;
 
   nsString statusString;
-  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  if (!NS_SUCCEEDED(rv))
+  nsCOMPtr<nsIStringBundleService> bundleService =
+    mozilla::services::GetStringBundleService();
+  if (!bundleService)
     return;
 
   nsCOMPtr<nsIStringBundle> bundle;
