@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
 #include "nntpCore.h"
@@ -56,6 +24,9 @@
 #include "nsServiceManagerUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsMsgUtils.h"
+#include "mozilla/Services.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 
 // This file contains the news article download state machine.
 
@@ -64,7 +35,7 @@
 // or subclasses thereof. News can download marked objects, for example.
 nsresult nsNewsDownloader::DownloadArticles(nsIMsgWindow *window, nsIMsgFolder *folder, nsTArray<nsMsgKey> *pIds)
 {
-  if (pIds != nsnull)
+  if (pIds != nullptr)
     m_keysToDownload.InsertElementsAt(0, pIds->Elements(), pIds->Length());
 
   if (!m_keysToDownload.IsEmpty())
@@ -93,7 +64,7 @@ nsNewsDownloader::nsNewsDownloader(nsIMsgWindow *window, nsIMsgDatabase *msgDB,
   m_listener = listener;
   m_window = window;
   m_lastPercent = -1;
-  LL_I2L(m_lastProgressTime, 0);
+  m_lastProgressTime = 0;
   // not the perfect place for this, but I think it will work.
   if (m_window)
     m_window->SetStopped(false);
@@ -102,11 +73,11 @@ nsNewsDownloader::nsNewsDownloader(nsIMsgWindow *window, nsIMsgDatabase *msgDB,
 nsNewsDownloader::~nsNewsDownloader()
 {
   if (m_listener)
-    m_listener->OnStopRunningUrl(/* don't have a url */nsnull, m_status);
+    m_listener->OnStopRunningUrl(/* don't have a url */nullptr, m_status);
   if (m_newsDB)
   {
     m_newsDB->Commit(nsMsgDBCommitType::kLargeCommit);
-    m_newsDB = nsnull;
+    m_newsDB = nullptr;
   }
 }
 
@@ -139,7 +110,7 @@ nsresult nsNewsDownloader::DownloadNext(bool firstTimeP)
     if (!moreHeaders)
     {
       if (m_listener)
-        m_listener->OnStopRunningUrl(nsnull, NS_OK);
+        m_listener->OnStopRunningUrl(nullptr, NS_OK);
       return NS_OK;
     }
   }
@@ -148,7 +119,7 @@ nsresult nsNewsDownloader::DownloadNext(bool firstTimeP)
   nsCOMPtr <nsINntpService> nntpService = do_GetService(NS_NNTPSERVICE_CONTRACTID,&rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return nntpService->FetchMessage(m_folder, m_keyToDownload, m_window, nsnull, this, nsnull);
+  return nntpService->FetchMessage(m_folder, m_keyToDownload, m_window, nullptr, this, nullptr);
 }
 
 bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve()
@@ -158,7 +129,7 @@ bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve()
   if (m_downloadFromKeys)
     return nsNewsDownloader::GetNextHdrToRetrieve();
 
-  if (m_headerEnumerator == nsnull)
+  if (m_headerEnumerator == nullptr)
     rv = m_newsDB->EnumerateMessages(getter_AddRefs(m_headerEnumerator));
 
   bool hasMore = false;
@@ -169,7 +140,7 @@ bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve()
     rv = m_headerEnumerator->GetNext(getter_AddRefs(supports));
     m_newsHeader = do_QueryInterface(supports);
     NS_ENSURE_SUCCESS(rv, false);
-    PRUint32 hdrFlags;
+    uint32_t hdrFlags;
     m_newsHeader->GetFlags(&hdrFlags);
     if (hdrFlags & nsMsgMessageFlags::Marked)
     {
@@ -178,7 +149,7 @@ bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve()
     }
     else
     {
-      m_newsHeader = nsnull;
+      m_newsHeader = nullptr;
     }
   }
   return hasMore;
@@ -192,31 +163,26 @@ bool nsNewsDownloader::GetNextHdrToRetrieve()
   nsresult rv;
   if (m_downloadFromKeys)
   {
-    if (m_numwrote >= (PRInt32) m_keysToDownload.Length())
+    if (m_numwrote >= (int32_t) m_keysToDownload.Length())
       return false;
 
     m_keyToDownload = m_keysToDownload[m_numwrote++];
-    PRInt32 percent;
-    percent = (100 * m_numwrote) / (PRInt32) m_keysToDownload.Length();
+    int32_t percent;
+    percent = (100 * m_numwrote) / (int32_t) m_keysToDownload.Length();
 
-    PRInt64 nowMS = LL_ZERO;
+    int64_t nowMS = 0;
     if (percent < 100)  // always need to do 100%
     {
-      int64 minIntervalBetweenProgress;
-
-      LL_I2L(minIntervalBetweenProgress, 750);
-      int64 diffSinceLastProgress;
-      LL_I2L(nowMS, PR_IntervalToMilliseconds(PR_IntervalNow()));
-      LL_SUB(diffSinceLastProgress, nowMS, m_lastProgressTime); // r = a - b
-      LL_SUB(diffSinceLastProgress, diffSinceLastProgress, minIntervalBetweenProgress); // r = a - b
-      if (!LL_GE_ZERO(diffSinceLastProgress))
+      nowMS = PR_IntervalToMilliseconds(PR_IntervalNow());
+      if (nowMS - m_lastProgressTime < 750)
         return true;
     }
 
     m_lastProgressTime = nowMS;
     nsCOMPtr <nsIMsgNewsFolder> newsFolder = do_QueryInterface(m_folder);
-    nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, false);
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      mozilla::services::GetStringBundleService();
+    NS_ENSURE_TRUE(bundleService, false);
     nsCOMPtr<nsIStringBundle> bundle;
     rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv, false);
@@ -241,7 +207,7 @@ bool nsNewsDownloader::GetNextHdrToRetrieve()
   return false;  // shouldn't get here if we're not downloading from keys.
 }
 
-nsresult nsNewsDownloader::ShowProgress(const PRUnichar *progressString, PRInt32 percent)
+nsresult nsNewsDownloader::ShowProgress(const PRUnichar *progressString, int32_t percent)
 {
   if (!m_statusFeedback)
   {
@@ -269,7 +235,7 @@ NS_IMETHODIMP DownloadNewsArticlesToOfflineStore::OnStartRunningUrl(nsIURI* url)
 NS_IMETHODIMP DownloadNewsArticlesToOfflineStore::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
 {
   m_status = exitCode;
-  if (m_newsHeader != nsnull)
+  if (m_newsHeader != nullptr)
   {
 #ifdef DEBUG_bienvenu
     //    XP_Trace("finished retrieving %ld\n", m_newsHeader->GetMessageKey());
@@ -278,10 +244,10 @@ NS_IMETHODIMP DownloadNewsArticlesToOfflineStore::OnStopRunningUrl(nsIURI* url,
     {
       nsMsgKey msgKey;
       m_newsHeader->GetMessageKey(&msgKey);
-      m_newsDB->MarkMarked(msgKey, false, nsnull);
+      m_newsDB->MarkMarked(msgKey, false, nullptr);
     }
   }
-  m_newsHeader = nsnull;
+  m_newsHeader = nullptr;
   return nsNewsDownloader::OnStopRunningUrl(url, exitCode);
 }
 
@@ -296,7 +262,7 @@ NS_IMETHODIMP nsNewsDownloader::OnSearchHit(nsIMsgDBHdr *header, nsIMsgFolder *f
   NS_ENSURE_ARG(header);
 
 
-  PRUint32 msgFlags;
+  uint32_t msgFlags;
   header->GetFlags(&msgFlags);
   // only need to download articles we don't already have...
   if (! (msgFlags & nsMsgMessageFlags::Offline))
@@ -313,13 +279,13 @@ NS_IMETHODIMP nsNewsDownloader::OnSearchDone(nsresult status)
   if (m_keysToDownload.IsEmpty())
   {
     if (m_listener)
-      return m_listener->OnStopRunningUrl(nsnull, NS_OK);
+      return m_listener->OnStopRunningUrl(nullptr, NS_OK);
   }
   nsresult rv = DownloadArticles(m_window, m_folder,
-                  /* we've already set m_keysToDownload, so don't pass it in */ nsnull);
+                  /* we've already set m_keysToDownload, so don't pass it in */ nullptr);
   if (NS_FAILED(rv))
     if (m_listener)
-      m_listener->OnStopRunningUrl(nsnull, rv);
+      m_listener->OnStopRunningUrl(nullptr, rv);
 
   return rv;
 }
@@ -367,7 +333,7 @@ nsMsgDownloadAllNewsgroups::nsMsgDownloadAllNewsgroups(nsIMsgWindow *window, nsI
 {
   m_window = window;
   m_listener = listener;
-  m_downloaderForGroup = new DownloadMatchingNewsArticlesToNewsDB(window, nsnull, nsnull, this);
+  m_downloaderForGroup = new DownloadMatchingNewsArticlesToNewsDB(window, nullptr, nullptr, this);
   NS_IF_ADDREF(m_downloaderForGroup);
   m_downloadedHdrsForCurGroup = false;
 }
@@ -414,7 +380,7 @@ nsMsgDownloadAllNewsgroups::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
 
 // leaves m_currentServer at the next nntp "server" that
 // might have folders to download for offline use. If no more servers,
-// m_currentServer will be left at nsnull.
+// m_currentServer will be left at nullptr.
 // Also, sets up m_serverEnumerator to enumerate over the server
 // If no servers found, m_serverEnumerator will be left at null,
 nsresult nsMsgDownloadAllNewsgroups::AdvanceToNextServer(bool *done)
@@ -434,10 +400,18 @@ nsresult nsMsgDownloadAllNewsgroups::AdvanceToNextServer(bool *done)
     rv = accountManager->GetAllServers(getter_AddRefs(m_allServers));
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  PRUint32 serverIndex = (m_currentServer) ? m_allServers->IndexOf(m_currentServer) + 1 : 0;
-  m_currentServer = nsnull;
-  PRUint32 numServers;
-  m_allServers->Count(&numServers);
+  uint32_t serverIndex = 0;
+  if (m_currentServer)
+  {
+    rv = m_allServers->IndexOf(0, m_currentServer, &serverIndex);
+    if (NS_FAILED(rv))
+      serverIndex = -1;
+
+    ++serverIndex;
+  }
+  m_currentServer = nullptr;
+  uint32_t numServers;
+  m_allServers->GetLength(&numServers);
   nsCOMPtr <nsIMsgFolder> rootFolder;
 
   while (serverIndex < numServers)
@@ -490,13 +464,13 @@ nsresult nsMsgDownloadAllNewsgroups::AdvanceToNextGroup(bool *done)
     if (NS_SUCCEEDED(rv) && session)
     {
       bool folderOpen;
-      PRUint32 folderFlags;
+      uint32_t folderFlags;
       m_currentFolder->GetFlags(&folderFlags);
       session->IsFolderOpenInWindow(m_currentFolder, &folderOpen);
       if (!folderOpen && ! (folderFlags & (nsMsgFolderFlags::Trash | nsMsgFolderFlags::Inbox)))
-        m_currentFolder->SetMsgDatabase(nsnull);
+        m_currentFolder->SetMsgDatabase(nullptr);
     }
-    m_currentFolder = nsnull;
+    m_currentFolder = nullptr;
   }
 
   *done = false;
@@ -525,13 +499,15 @@ nsresult DownloadMatchingNewsArticlesToNewsDB::RunSearch(nsIMsgFolder *folder, n
   m_searchSession = searchSession;
 
   m_keysToDownload.Clear();
-  nsresult rv;
+
   NS_ENSURE_ARG(searchSession);
   NS_ENSURE_ARG(folder);
 
   searchSession->RegisterListener(this,
                                   nsIMsgSearchSession::allNotifications);
-  rv = searchSession->AddScopeTerm(nsMsgSearchScope::localNews, folder);
+  nsresult rv = searchSession->AddScopeTerm(nsMsgSearchScope::localNews, folder);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   return searchSession->Search(m_window);
 }
 
@@ -545,7 +521,7 @@ nsresult nsMsgDownloadAllNewsgroups::ProcessNextGroup()
     rv = AdvanceToNextGroup(&done);
     if (m_currentFolder)
     {
-      PRUint32 folderFlags;
+      uint32_t folderFlags;
       m_currentFolder->GetFlags(&folderFlags);
       if (folderFlags & nsMsgFolderFlags::Offline)
         break;
@@ -554,7 +530,7 @@ nsresult nsMsgDownloadAllNewsgroups::ProcessNextGroup()
   if (NS_FAILED(rv) || done)
   {
     if (m_listener)
-      return m_listener->OnStopRunningUrl(nsnull, NS_OK);
+      return m_listener->OnStopRunningUrl(nullptr, NS_OK);
   }
   m_downloadedHdrsForCurGroup = true;
   return m_currentFolder ? m_currentFolder->GetNewMessages(m_window, this) : NS_ERROR_NOT_INITIALIZED;
@@ -577,7 +553,7 @@ nsresult nsMsgDownloadAllNewsgroups::DownloadMsgsForCurrentGroup()
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool downloadByDate, downloadUnreadOnly;
-  PRUint32 ageLimitOfMsgsToDownload;
+  uint32_t ageLimitOfMsgsToDownload;
 
   downloadSettings->GetDownloadByDate(&downloadByDate);
   downloadSettings->GetDownloadUnreadOnly(&downloadUnreadOnly);
@@ -594,17 +570,17 @@ nsresult nsMsgDownloadAllNewsgroups::DownloadMsgsForCurrentGroup()
   {
     value->SetAttrib(nsMsgSearchAttrib::MsgStatus);
     value->SetStatus(nsMsgMessageFlags::Read);
-    searchSession->AddSearchTerm(nsMsgSearchAttrib::MsgStatus, nsMsgSearchOp::Isnt, value, true, nsnull);
+    searchSession->AddSearchTerm(nsMsgSearchAttrib::MsgStatus, nsMsgSearchOp::Isnt, value, true, nullptr);
   }
   if (downloadByDate)
   {
     value->SetAttrib(nsMsgSearchAttrib::AgeInDays);
     value->SetAge(ageLimitOfMsgsToDownload);
-    searchSession->AddSearchTerm(nsMsgSearchAttrib::AgeInDays, nsMsgSearchOp::IsLessThan, value, nsMsgSearchBooleanOp::BooleanAND, nsnull);
+    searchSession->AddSearchTerm(nsMsgSearchAttrib::AgeInDays, nsMsgSearchOp::IsLessThan, value, nsMsgSearchBooleanOp::BooleanAND, nullptr);
   }
   value->SetAttrib(nsMsgSearchAttrib::MsgStatus);
   value->SetStatus(nsMsgMessageFlags::Offline);
-  searchSession->AddSearchTerm(nsMsgSearchAttrib::MsgStatus, nsMsgSearchOp::Isnt, value, nsMsgSearchBooleanOp::BooleanAND, nsnull);
+  searchSession->AddSearchTerm(nsMsgSearchAttrib::MsgStatus, nsMsgSearchOp::Isnt, value, nsMsgSearchBooleanOp::BooleanAND, nullptr);
 
   m_downloaderForGroup->RunSearch(m_currentFolder, db, searchSession);
   return rv;
