@@ -1,49 +1,14 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Seth Spitzer <sspitzer@netscape.com>
- *   David Bienvenu <bienvenu@nventure.com>
- *   Henrik Gemal <mozilla@gemal.dk>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsNntpIncomingServer.h"
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 #include "nsNewsFolder.h"
 #include "nsIMsgFolder.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsCOMPtr.h"
 #include "nsINntpService.h"
 #include "nsINNTPProtocol.h"
@@ -66,6 +31,7 @@
 #include "nsNetUtil.h"
 #include "nsISimpleEnumerator.h"
 #include "nsMsgUtils.h"
+#include "mozilla/Services.h"
 
 #define INVALID_VERSION         0
 #define VALID_VERSION           2
@@ -79,13 +45,13 @@
 
 // this platform specific junk is so the newsrc filenames we create
 // will resemble the migrated newsrc filenames.
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#if defined(XP_UNIX)
 #define NEWSRC_FILE_PREFIX "newsrc-"
 #define NEWSRC_FILE_SUFFIX ""
 #else
 #define NEWSRC_FILE_PREFIX ""
 #define NEWSRC_FILE_SUFFIX ".rc"
-#endif /* XP_UNIX || XP_BEOS */
+#endif /* XP_UNIX */
 
 // ###tw  This really ought to be the most
 // efficient file reading size for the current
@@ -135,7 +101,6 @@ nsNntpIncomingServer::nsNntpIncomingServer()
   mUniqueId = 0;
   mHasSeenBeginGroups = false;
   mPostingAllowed = false;
-  m_userAuthenticated = false;
   mLastUpdatedTime = 0;
 
   // these atoms are used for subscribe search
@@ -154,7 +119,7 @@ nsNntpIncomingServer::~nsNntpIncomingServer()
 
     if (mNewsrcSaveTimer) {
         mNewsrcSaveTimer->Cancel();
-        mNewsrcSaveTimer = nsnull;
+        mNewsrcSaveTimer = nullptr;
     }
     rv = ClearInner();
     NS_ASSERTION(NS_SUCCEEDED(rv), "ClearInner failed");
@@ -183,7 +148,7 @@ nsNntpIncomingServer::CreateRootFolderFromUri(const nsCString &serverUri,
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetNewsrcFilePath(nsILocalFile **aNewsrcFilePath)
+nsNntpIncomingServer::GetNewsrcFilePath(nsIFile **aNewsrcFilePath)
 {
   nsresult rv;
   if (mNewsrcFilePath)
@@ -207,7 +172,7 @@ nsNntpIncomingServer::GetNewsrcFilePath(nsILocalFile **aNewsrcFilePath)
   rv = GetHostName(hostname);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString newsrcFileName(NEWSRC_FILE_PREFIX);
+  nsAutoCString newsrcFileName(NEWSRC_FILE_PREFIX);
   newsrcFileName.Append(hostname);
   newsrcFileName.Append(NEWSRC_FILE_SUFFIX);
   rv = mNewsrcFilePath->AppendNative(newsrcFileName);
@@ -222,7 +187,7 @@ nsNntpIncomingServer::GetNewsrcFilePath(nsILocalFile **aNewsrcFilePath)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetNewsrcFilePath(nsILocalFile *aFile)
+nsNntpIncomingServer::SetNewsrcFilePath(nsIFile *aFile)
 {
     NS_ENSURE_ARG_POINTER(aFile);
 
@@ -244,17 +209,17 @@ nsNntpIncomingServer::GetLocalStoreType(nsACString& type)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetNewsrcRootPath(nsILocalFile *aNewsrcRootPath)
+nsNntpIncomingServer::SetNewsrcRootPath(nsIFile *aNewsrcRootPath)
 {
     NS_ENSURE_ARG(aNewsrcRootPath);
     return NS_SetPersistentFile(PREF_MAIL_NEWSRC_ROOT_REL, PREF_MAIL_NEWSRC_ROOT, aNewsrcRootPath);
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetNewsrcRootPath(nsILocalFile **aNewsrcRootPath)
+nsNntpIncomingServer::GetNewsrcRootPath(nsIFile **aNewsrcRootPath)
 {
     NS_ENSURE_ARG_POINTER(aNewsrcRootPath);
-    *aNewsrcRootPath = nsnull;
+    *aNewsrcRootPath = nullptr;
 
     bool havePref;
     nsresult rv = NS_GetPersistentFile(PREF_MAIL_NEWSRC_ROOT_REL,
@@ -287,9 +252,9 @@ nsNntpIncomingServer::GetNewsrcRootPath(nsILocalFile **aNewsrcRootPath)
 
 nsresult nsNntpIncomingServer::SetupNewsrcSaveTimer()
 {
-  PRInt64 ms(300000);   // hard code, 5 minutes.
+  int64_t ms(300000);   // hard code, 5 minutes.
   //Convert biffDelay into milliseconds
-  PRUint32 timeInMSUint32 = (PRUint32)ms;
+  uint32_t timeInMSUint32 = (uint32_t)ms;
   //Can't currently reset a timer when it's in the process of
   //calling Notify. So, just release the timer here and create a new one.
   if(mNewsrcSaveTimer)
@@ -309,17 +274,18 @@ nsNntpIncomingServer::SetCharset(const nsACString & aCharset)
 NS_IMETHODIMP
 nsNntpIncomingServer::GetCharset(nsACString & aCharset)
 {
-  nsresult rv;
   //first we get the per-server settings mail.server.<serverkey>.charset
-  rv = GetCharValue("charset", aCharset);
+  nsresult rv = GetCharValue("charset", aCharset);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   //if the per-server setting is empty,we get the default charset from
   //mailnews.view_default_charset setting and set it as per-server preference.
-  if(aCharset.IsEmpty()){
+  if (aCharset.IsEmpty()) {
     nsString defaultCharset;
-    rv = NS_GetLocalizedUnicharPreferenceWithDefault(nsnull,
+    rv = NS_GetLocalizedUnicharPreferenceWithDefault(nullptr,
          PREF_MAILNEWS_VIEW_DEFAULT_CHARSET,
          NS_LITERAL_STRING("ISO-8859-1"), defaultCharset);
+    NS_ENSURE_SUCCESS(rv, rv);
     LossyCopyUTF16toASCII(defaultCharset, aCharset);
     SetCharset(aCharset);
   }
@@ -345,7 +311,7 @@ nsNntpIncomingServer::WriteNewsrcFile()
 #ifdef DEBUG_NEWS
         printf("write newsrc file for %s\n", hostname.get());
 #endif
-        nsCOMPtr <nsILocalFile> newsrcFile;
+        nsCOMPtr <nsIFile> newsrcFile;
         rv = GetNewsrcFilePath(getter_AddRefs(newsrcFile));
         if (NS_FAILED(rv)) return rv;
 
@@ -362,7 +328,7 @@ nsNntpIncomingServer::WriteNewsrcFile()
         nsCOMPtr <nsIMsgNewsFolder> newsFolder = do_QueryInterface(rootFolder, &rv);
         if (NS_FAILED(rv)) return rv;
 
-        PRUint32 bytesWritten;
+        uint32_t bytesWritten;
         nsCString optionLines;
         rv = newsFolder->GetOptionLines(optionLines);
         if (NS_SUCCEEDED(rv) && !optionLines.IsEmpty()) {
@@ -443,8 +409,6 @@ nsNntpIncomingServer::GetNewsrcHasChanged(bool *aNewsrcHasChanged)
     return NS_OK;
 }
 
-NS_IMPL_GETSET(nsNntpIncomingServer, UserAuthenticated, bool, m_userAuthenticated)
-
 NS_IMETHODIMP
 nsNntpIncomingServer::CloseCachedConnections()
 {
@@ -452,9 +416,9 @@ nsNntpIncomingServer::CloseCachedConnections()
   nsCOMPtr<nsINNTPProtocol> connection;
 
   // iterate through the connection cache and close the connections.
-  PRInt32 cnt = mConnectionCache.Count();
+  int32_t cnt = mConnectionCache.Count();
 
-  for (PRInt32 i = 0; i < cnt; ++i)
+  for (int32_t i = 0; i < cnt; ++i)
   {
     connection = mConnectionCache[0];
     if (connection)
@@ -486,20 +450,14 @@ bool
 nsNntpIncomingServer::ConnectionTimeOut(nsINNTPProtocol* aConnection)
 {
     bool retVal = false;
-    if (!aConnection) return retVal;
-    nsresult rv;
+    if (!aConnection)
+      return retVal;
 
-    PRTime cacheTimeoutLimits;
-
-    LL_I2L(cacheTimeoutLimits, 170 * 1000000); // 170 seconds in microseconds
     PRTime lastActiveTimeStamp;
-    rv = aConnection->GetLastActiveTimeStamp(&lastActiveTimeStamp);
+    if (NS_FAILED(aConnection->GetLastActiveTimeStamp(&lastActiveTimeStamp)))
+      return retVal;
 
-    PRTime elapsedTime;
-    LL_SUB(elapsedTime, PR_Now(), lastActiveTimeStamp);
-    PRTime t;
-    LL_SUB(t, elapsedTime, cacheTimeoutLimits);
-    if (LL_GE_ZERO(t))
+    if (PR_Now() - lastActiveTimeStamp >= PRTime(170) * PR_USEC_PER_SEC)
     {
 #ifdef DEBUG_seth
       printf("XXX connection timed out, close it, and remove it from the connection cache\n");
@@ -540,7 +498,7 @@ nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
 {
   // Get our maximum connection count. We need at least 1. If the value is 0,
   // we use the default. If it's negative, we treat that as 1.
-  PRInt32 maxConnections = kMaxConnectionsPerHost;
+  int32_t maxConnections = kMaxConnectionsPerHost;
   nsresult rv = GetMaximumConnectionsNumber(&maxConnections);
   if (NS_FAILED(rv) || maxConnections == 0)
   {
@@ -555,8 +513,8 @@ nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
 
   // Find a non-busy connection
   nsCOMPtr<nsINNTPProtocol> connection;
-  PRInt32 cnt = mConnectionCache.Count();
-  for (PRInt32 i = 0; i < cnt; i++)
+  int32_t cnt = mConnectionCache.Count();
+  for (int32_t i = 0; i < cnt; i++)
   {
     connection = mConnectionCache[i];
     if (connection)
@@ -565,13 +523,13 @@ nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
       connection->GetIsBusy(&isBusy);
       if (!isBusy)
         break;
-      connection = nsnull;
+      connection = nullptr;
     }
   }
 
   if (ConnectionTimeOut(connection))
   {
-    connection = nsnull;
+    connection = nullptr;
     // We have one less connection, since we closed this one.
     --cnt;
   }
@@ -592,7 +550,7 @@ nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
   {
     // We maxed out our connection count. The caller must therefore enqueue the
     // call.
-    *aNntpConnection = nsnull;
+    *aNntpConnection = nullptr;
     return NS_OK;
   }
 
@@ -710,14 +668,14 @@ nsNntpIncomingServer::DownloadMail(nsIMsgWindow *aMsgWindow)
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIMsgFolder> group(do_QueryInterface(nextGroup));
-    rv = group->GetNewMessages(aMsgWindow, nsnull);
+    rv = group->GetNewMessages(aMsgWindow, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
   }
   return rv;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::DisplaySubscribedGroup(nsIMsgNewsFolder *aMsgFolder, PRInt32 aFirstMessage, PRInt32 aLastMessage, PRInt32 aTotalMessages)
+nsNntpIncomingServer::DisplaySubscribedGroup(nsIMsgNewsFolder *aMsgFolder, int32_t aFirstMessage, int32_t aLastMessage, int32_t aTotalMessages)
 {
   nsresult rv;
 
@@ -765,10 +723,12 @@ nsNntpIncomingServer::OnStopRunningUrl(nsIURI *url, nsresult exitCode)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::ContainsNewsgroup(const nsACString &name,
+nsNntpIncomingServer::ContainsNewsgroup(const nsACString &aName,
                                         bool *containsGroup)
 {
-    if (name.IsEmpty()) return NS_ERROR_FAILURE;
+    NS_ENSURE_ARG_POINTER(containsGroup);
+    NS_ENSURE_FALSE(aName.IsEmpty(), NS_ERROR_FAILURE);
+
     if (mSubscribedNewsgroups.Length() == 0)
     {
       // If this is empty, we may need to discover folders
@@ -780,9 +740,9 @@ nsNntpIncomingServer::ContainsNewsgroup(const nsACString &name,
         rootFolder->GetSubFolders(getter_AddRefs(subfolders));
       }
     }
-    nsCAutoString unescapedName;
-    MsgUnescapeString(name, 0, unescapedName);
-    *containsGroup = mSubscribedNewsgroups.Contains(name);
+    nsAutoCString unescapedName;
+    MsgUnescapeString(aName, 0, unescapedName);
+    *containsGroup = mSubscribedNewsgroups.Contains(aName);
     return NS_OK;
 }
 
@@ -790,17 +750,21 @@ NS_IMETHODIMP
 nsNntpIncomingServer::SubscribeToNewsgroup(const nsACString &aName)
 {
     NS_ASSERTION(!aName.IsEmpty(), "no name");
-    if (aName.IsEmpty()) return NS_ERROR_FAILURE;
+    NS_ENSURE_FALSE(aName.IsEmpty(), NS_ERROR_FAILURE);
+
+    // If we already have this newsgroup, do nothing and report success.
+    bool containsGroup = false;
+    nsresult rv = ContainsNewsgroup(aName, &containsGroup);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (containsGroup)
+      return NS_OK;
 
     nsCOMPtr<nsIMsgFolder> msgfolder;
-    nsresult rv = GetRootMsgFolder(getter_AddRefs(msgfolder));
-    if (NS_FAILED(rv)) return rv;
-    if (!msgfolder) return NS_ERROR_FAILURE;
+    rv = GetRootMsgFolder(getter_AddRefs(msgfolder));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_TRUE(msgfolder, NS_ERROR_FAILURE);
 
-    rv = msgfolder->CreateSubfolder(NS_ConvertUTF8toUTF16(aName), nsnull);
-    if (NS_FAILED(rv)) return rv;
-
-    return NS_OK;
+    return msgfolder->CreateSubfolder(NS_ConvertUTF8toUTF16(aName), nullptr);
 }
 
 bool
@@ -818,7 +782,7 @@ writeGroupToHostInfoFile(nsCString &aElement, void *aData)
 
 void nsNntpIncomingServer::WriteLine(nsIOutputStream *stream, nsCString &str)
 {
-  PRUint32 bytesWritten;
+  uint32_t bytesWritten;
   str.Append(MSG_LINEBREAK);
   stream->Write(str.get(), str.Length(), &bytesWritten);
 }
@@ -827,11 +791,9 @@ nsNntpIncomingServer::WriteHostInfoFile()
 {
   if (!mHostInfoHasChanged)
     return NS_OK;
-  PRInt32 firstnewdate;
+  int32_t firstnewdate = (int32_t)mFirstNewDate;
 
-  LL_L2I(firstnewdate, mFirstNewDate);
-
-  mLastUpdatedTime = PRUint32(PR_Now() / PR_USEC_PER_SEC);
+  mLastUpdatedTime = uint32_t(PR_Now() / PR_USEC_PER_SEC);
 
   nsCString hostname;
   nsresult rv = GetHostName(hostname);
@@ -844,19 +806,19 @@ nsNntpIncomingServer::WriteHostInfoFile()
   NS_ENSURE_SUCCESS(rv, rv);
 
   // todo, missing some formatting, see the 4.x code
-  nsCAutoString header("# News host information file.");
+  nsAutoCString header("# News host information file.");
   WriteLine(hostInfoStream, header);
   header.Assign("# This is a generated file!  Do not edit.");
   WriteLine(hostInfoStream, header);
   header.Truncate();
   WriteLine(hostInfoStream, header);
-  nsCAutoString version("version=");
+  nsAutoCString version("version=");
   version.AppendInt(VALID_VERSION);
   WriteLine(hostInfoStream, version);
-  nsCAutoString newsrcname("newsrcname=");
+  nsAutoCString newsrcname("newsrcname=");
   newsrcname.Append(hostname);
   WriteLine(hostInfoStream, hostname);
-  nsCAutoString dateStr("lastgroupdate=");
+  nsAutoCString dateStr("lastgroupdate=");
   dateStr.AppendInt(mLastUpdatedTime);
   WriteLine(hostInfoStream, dateStr);
   dateStr ="firstnewdate=";
@@ -869,10 +831,10 @@ nsNntpIncomingServer::WriteHostInfoFile()
   WriteLine(hostInfoStream, header);
 
   // XXX todo, sort groups first?
-  PRUint32 length = mGroupsOnServer.Length();
-  for (PRUint32 i = 0; i < length; ++i)
+  uint32_t length = mGroupsOnServer.Length();
+  for (uint32_t i = 0; i < length; ++i)
   {
-    PRUint32 bytesWritten;
+    uint32_t bytesWritten;
     hostInfoStream->Write(mGroupsOnServer[i].get(), mGroupsOnServer[i].Length(),
                           &bytesWritten);
     hostInfoStream->Write(MSG_LINEBREAK, MSG_LINEBREAK_LEN, &bytesWritten);
@@ -906,17 +868,17 @@ nsNntpIncomingServer::LoadHostInfoFile()
 
   nsCOMPtr<nsIInputStream> fileStream;
   rv = NS_NewLocalFileInputStream(getter_AddRefs(fileStream), mHostInfoFile);
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsILineInputStream> lineInputStream(do_QueryInterface(fileStream, &rv));
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   bool more = true;
   nsCString line;
 
   while (more && NS_SUCCEEDED(rv))
   {
-    lineInputStream->ReadLine(line, &more);
+    rv = lineInputStream->ReadLine(line, &more);
     if (line.IsEmpty())
       continue;
     HandleLine(line.get(), line.Length());
@@ -1019,7 +981,7 @@ nsNntpIncomingServer::AddNewsgroupToList(const char *aName)
     nsresult rv;
 
     nsAutoString newsgroupName;
-    nsCAutoString dataCharset;
+    nsAutoCString dataCharset;
     rv = GetCharset(dataCharset);
     NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1061,13 +1023,13 @@ nsNntpIncomingServer::ClearInner()
     nsresult rv = NS_OK;
 
     if (mInner) {
-        rv = mInner->SetSubscribeListener(nsnull);
+        rv = mInner->SetSubscribeListener(nullptr);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        rv = mInner->SetIncomingServer(nsnull);
+        rv = mInner->SetIncomingServer(nullptr);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        mInner = nsnull;
+        mInner = nullptr;
     }
     return NS_OK;
 }
@@ -1123,8 +1085,8 @@ nsNntpIncomingServer::UpdateSubscribed()
   nsresult rv = EnsureInner();
   NS_ENSURE_SUCCESS(rv,rv);
   mTempSubscribed.Clear();
-  PRUint32 length = mSubscribedNewsgroups.Length();
-  for (PRUint32 i = 0; i < length; ++i)
+  uint32_t length = mSubscribedNewsgroups.Length();
+  for (uint32_t i = 0; i < length; ++i)
     SetAsSubscribed(mSubscribedNewsgroups[i]);
   return NS_OK;
 }
@@ -1217,7 +1179,7 @@ nsNntpIncomingServer::Unsubscribe(const PRUnichar *aUnicharName)
 
   // to handle non-ASCII newsgroup names, we store them internally as escaped.
   // so we need to escape and encode the name, in order to find it.
-  nsCAutoString escapedName;
+  nsAutoCString escapedName;
   rv = NS_MsgEscapeEncodeURLPath(nsDependentString(aUnicharName), escapedName);
 
   nsCOMPtr <nsIMsgFolder> newsgroupFolder;
@@ -1230,7 +1192,7 @@ nsNntpIncomingServer::Unsubscribe(const PRUnichar *aUnicharName)
   if (!newsgroupFolder)
     return NS_ERROR_FAILURE;
 
-  rv = serverFolder->PropagateDelete(newsgroupFolder, true /* delete storage */, nsnull);
+  rv = serverFolder->PropagateDelete(newsgroupFolder, true /* delete storage */, nullptr);
   if (NS_FAILED(rv))
     return rv;
 
@@ -1242,14 +1204,16 @@ nsNntpIncomingServer::Unsubscribe(const PRUnichar *aUnicharName)
   return NS_OK;
 }
 
-PRInt32
-nsNntpIncomingServer::HandleLine(const char* line, PRUint32 line_size)
+nsresult
+nsNntpIncomingServer::HandleLine(const char* line, uint32_t line_size)
 {
   NS_ASSERTION(line, "line is null");
-  if (!line) return 0;
+  if (!line)
+    return NS_OK;
 
   // skip blank lines and comments
-  if (line[0] == '#' || line[0] == '\0') return 0;
+  if (line[0] == '#' || line[0] == '\0')
+    return NS_OK;
   // ###TODO - make this truly const, maybe pass in an nsCString &
 
   if (mHasSeenBeginGroups) {
@@ -1278,19 +1242,18 @@ nsNntpIncomingServer::HandleLine(const char* line, PRUint32 line_size)
     if (equalPos) {
       *equalPos++ = '\0';
       if (PL_strcmp(line, "lastgroupdate") == 0) {
-        mLastUpdatedTime = strtoul(equalPos, nsnull, 10);
+        mLastUpdatedTime = strtoul(equalPos, nullptr, 10);
       } else if (PL_strcmp(line, "firstnewdate") == 0) {
-        PRInt32 firstnewdate = strtol(equalPos, nsnull, 16);
-        LL_I2L(mFirstNewDate, firstnewdate);
+        mFirstNewDate = strtol(equalPos, nullptr, 16);
       } else if (PL_strcmp(line, "uniqueid") == 0) {
-        mUniqueId = strtol(equalPos, nsnull, 16);
+        mUniqueId = strtol(equalPos, nullptr, 16);
       } else if (PL_strcmp(line, "version") == 0) {
-        mVersion = strtol(equalPos, nsnull, 16);
+        mVersion = strtol(equalPos, nullptr, 16);
       }
     }
   }
 
-  return 0;
+  return NS_OK;
 }
 
 nsresult
@@ -1411,9 +1374,7 @@ nsNntpIncomingServer::ForgetPassword()
     NS_ENSURE_SUCCESS(rv,rv);
     if (!newsFolder) return NS_ERROR_FAILURE;
 
-    rv = newsFolder->ForgetGroupUsername();
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = newsFolder->ForgetGroupPassword();
+    rv = newsFolder->ForgetAuthenticationCredentials();
     NS_ENSURE_SUCCESS(rv,rv);
 
     // clear password of all child folders
@@ -1433,9 +1394,7 @@ nsNntpIncomingServer::ForgetPassword()
         if (NS_SUCCEEDED(rv) && child) {
             newsFolder = do_QueryInterface(child, &rv);
             if (NS_SUCCEEDED(rv) && newsFolder) {
-                rv = newsFolder->ForgetGroupUsername();
-                if (NS_FAILED(rv)) return_rv = rv;
-                rv = newsFolder->ForgetGroupPassword();
+                rv = newsFolder->ForgetAuthenticationCredentials();
                 if (NS_FAILED(rv)) return_rv = rv;
             }
             else {
@@ -1490,14 +1449,14 @@ nsNntpIncomingServer::SetPostingAllowed(bool aPostingAllowed)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetLastUpdatedTime(PRUint32 *aLastUpdatedTime)
+nsNntpIncomingServer::GetLastUpdatedTime(uint32_t *aLastUpdatedTime)
 {
   *aLastUpdatedTime = mLastUpdatedTime;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetLastUpdatedTime(PRUint32 aLastUpdatedTime)
+nsNntpIncomingServer::SetLastUpdatedTime(uint32_t aLastUpdatedTime)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -1551,7 +1510,7 @@ nsNntpIncomingServer::FindGroup(const nsACString &name, nsIMsgNewsFolder **resul
   if (!serverFolder) return NS_ERROR_FAILURE;
 
   // Escape the name for using FindSubFolder
-  nsCAutoString escapedName;
+  nsAutoCString escapedName;
   rv = MsgEscapeString(name, nsINetUtil::ESCAPE_URL_PATH, escapedName);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1594,12 +1553,13 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow *aMsgWindow,
 
   if (!prompt) {
     nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
-    rv = wwatch->GetNewPrompter(nsnull, getter_AddRefs(prompt));
+    rv = wwatch->GetNewPrompter(nullptr, getter_AddRefs(prompt));
     NS_ENSURE_SUCCESS(rv,rv);
   }
 
-  nsCOMPtr <nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID,&rv);
-  NS_ENSURE_SUCCESS(rv,rv);
+  nsCOMPtr <nsIStringBundleService> bundleService =
+    mozilla::services::GetStringBundleService();
+  NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr <nsIStringBundle> bundle;
   rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
@@ -1621,7 +1581,7 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow *aMsgWindow,
   NS_ENSURE_SUCCESS(rv,rv);
 
   bool confirmResult = false;
-  rv = prompt->Confirm(nsnull, confirmText.get(), &confirmResult);
+  rv = prompt->Confirm(nullptr, confirmText.get(), &confirmResult);
   NS_ENSURE_SUCCESS(rv,rv);
 
   if (confirmResult) {
@@ -1648,7 +1608,7 @@ nsNntpIncomingServer::GetCanSearchMessages(bool *canSearchMessages)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetOfflineSupportLevel(PRInt32 *aSupportLevel)
+nsNntpIncomingServer::GetOfflineSupportLevel(int32_t *aSupportLevel)
 {
     NS_ENSURE_ARG_POINTER(aSupportLevel);
     nsresult rv;
@@ -1698,8 +1658,8 @@ nsNntpIncomingServer::SetSearchValue(const nsAString &searchValue)
   }
 
   mSubscribeSearchResult.Clear();
-  PRUint32 length = mGroupsOnServer.Length();
-  for (PRUint32 i = 0; i < length; i++)
+  uint32_t length = mGroupsOnServer.Length();
+  for (uint32_t i = 0; i < length; i++)
   {
     if (CaseInsensitiveFindInReadable(mSearchValue, NS_ConvertUTF8toUTF16(mGroupsOnServer[i])))
       mSubscribeSearchResult.AppendElement(mGroupsOnServer[i]);
@@ -1725,7 +1685,7 @@ nsNntpIncomingServer::GetSupportsSubscribeSearch(bool *retVal)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetRowCount(PRInt32 *aRowCount)
+nsNntpIncomingServer::GetRowCount(int32_t *aRowCount)
 {
     *aRowCount = mSubscribeSearchResult.Length();
     return NS_OK;
@@ -1747,13 +1707,13 @@ nsNntpIncomingServer::SetSelection(nsITreeSelection * aSelection)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetRowProperties(PRInt32 index, nsISupportsArray *properties)
+nsNntpIncomingServer::GetRowProperties(int32_t index, nsISupportsArray *properties)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetCellProperties(PRInt32 row, nsITreeColumn* col, nsISupportsArray *properties)
+nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *properties)
 {
     if (!IsValidRow(row))
       return NS_ERROR_UNEXPECTED;
@@ -1788,26 +1748,26 @@ nsNntpIncomingServer::GetColumnProperties(nsITreeColumn* col, nsISupportsArray *
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsContainer(PRInt32 index, bool *_retval)
+nsNntpIncomingServer::IsContainer(int32_t index, bool *_retval)
 {
     *_retval = false;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsContainerOpen(PRInt32 index, bool *_retval)
+nsNntpIncomingServer::IsContainerOpen(int32_t index, bool *_retval)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsContainerEmpty(PRInt32 index, bool *_retval)
+nsNntpIncomingServer::IsContainerEmpty(int32_t index, bool *_retval)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsSeparator(PRInt32 index, bool *_retval)
+nsNntpIncomingServer::IsSeparator(int32_t index, bool *_retval)
 {
     *_retval = false;
     return NS_OK;
@@ -1820,8 +1780,8 @@ nsNntpIncomingServer::IsSorted(bool *_retval)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::CanDrop(PRInt32 index,
-                              PRInt32 orientation,
+nsNntpIncomingServer::CanDrop(int32_t index,
+                              int32_t orientation,
                               nsIDOMDataTransfer *dataTransfer,
                               bool *_retval)
 {
@@ -1829,58 +1789,58 @@ nsNntpIncomingServer::CanDrop(PRInt32 index,
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::Drop(PRInt32 row,
-                           PRInt32 orientation,
+nsNntpIncomingServer::Drop(int32_t row,
+                           int32_t orientation,
                            nsIDOMDataTransfer *dataTransfer)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetParentIndex(PRInt32 rowIndex, PRInt32 *_retval)
+nsNntpIncomingServer::GetParentIndex(int32_t rowIndex, int32_t *_retval)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::HasNextSibling(PRInt32 rowIndex, PRInt32 afterIndex, bool *_retval)
+nsNntpIncomingServer::HasNextSibling(int32_t rowIndex, int32_t afterIndex, bool *_retval)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetLevel(PRInt32 index, PRInt32 *_retval)
+nsNntpIncomingServer::GetLevel(int32_t index, int32_t *_retval)
 {
     *_retval = 0;
     return NS_OK;
 }
 
-nsresult
-nsNntpIncomingServer::IsValidRow(PRInt32 row)
+bool
+nsNntpIncomingServer::IsValidRow(int32_t row)
 {
-  return ((row >= 0) && (row < (PRInt32)mSubscribeSearchResult.Length()));
+  return ((row >= 0) && (row < (int32_t)mSubscribeSearchResult.Length()));
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetImageSrc(PRInt32 row, nsITreeColumn* col, nsAString& _retval)
+nsNntpIncomingServer::GetImageSrc(int32_t row, nsITreeColumn* col, nsAString& _retval)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetProgressMode(PRInt32 row, nsITreeColumn* col, PRInt32* _retval)
+nsNntpIncomingServer::GetProgressMode(int32_t row, nsITreeColumn* col, int32_t* _retval)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetCellValue(PRInt32 row, nsITreeColumn* col, nsAString& _retval)
+nsNntpIncomingServer::GetCellValue(int32_t row, nsITreeColumn* col, nsAString& _retval)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetCellText(PRInt32 row, nsITreeColumn* col, nsAString& _retval)
+nsNntpIncomingServer::GetCellText(int32_t row, nsITreeColumn* col, nsAString& _retval)
 {
     if (!IsValidRow(row))
       return NS_ERROR_UNEXPECTED;
@@ -1892,7 +1852,7 @@ nsNntpIncomingServer::GetCellText(PRInt32 row, nsITreeColumn* col, nsAString& _r
 
     nsresult rv = NS_OK;
     if (colID[0] == 'n') {
-      nsCAutoString str;
+      nsAutoCString str;
       if (mSearchResultSortDescending)
         row = mSubscribeSearchResult.Length() - 1 - row;
       // some servers have newsgroup names that are non ASCII.  we store
@@ -1931,7 +1891,7 @@ nsNntpIncomingServer::SetTree(nsITreeBoxObject *tree)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::ToggleOpenState(PRInt32 index)
+nsNntpIncomingServer::ToggleOpenState(int32_t index)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -1962,33 +1922,33 @@ nsNntpIncomingServer::SelectionChanged()
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::CycleCell(PRInt32 row, nsITreeColumn* col)
+nsNntpIncomingServer::CycleCell(int32_t row, nsITreeColumn* col)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsEditable(PRInt32 row, nsITreeColumn* col, bool *_retval)
+nsNntpIncomingServer::IsEditable(int32_t row, nsITreeColumn* col, bool *_retval)
 {
     *_retval = false;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::IsSelectable(PRInt32 row, nsITreeColumn* col, bool *_retval)
+nsNntpIncomingServer::IsSelectable(int32_t row, nsITreeColumn* col, bool *_retval)
 {
     *_retval = false;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetCellValue(PRInt32 row, nsITreeColumn* col, const nsAString& value)
+nsNntpIncomingServer::SetCellValue(int32_t row, nsITreeColumn* col, const nsAString& value)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetCellText(PRInt32 row, nsITreeColumn* col, const nsAString& value)
+nsNntpIncomingServer::SetCellText(int32_t row, nsITreeColumn* col, const nsAString& value)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -2000,13 +1960,13 @@ nsNntpIncomingServer::PerformAction(const PRUnichar *action)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::PerformActionOnRow(const PRUnichar *action, PRInt32 row)
+nsNntpIncomingServer::PerformActionOnRow(const PRUnichar *action, int32_t row)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::PerformActionOnCell(const PRUnichar *action, PRInt32 row, nsITreeColumn* col)
+nsNntpIncomingServer::PerformActionOnCell(const PRUnichar *action, int32_t row, nsITreeColumn* col)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -2047,7 +2007,7 @@ nsNntpIncomingServer::GetSearchScope(nsMsgSearchScopeValue *searchScope)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetSocketType(PRInt32 *aSocketType)
+nsNntpIncomingServer::GetSocketType(int32_t *aSocketType)
 {
   NS_ENSURE_ARG_POINTER(aSocketType);
   if (!mPrefBranch)
@@ -2084,7 +2044,7 @@ nsNntpIncomingServer::GetSocketType(PRInt32 *aSocketType)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::SetSocketType(PRInt32 aSocketType)
+nsNntpIncomingServer::SetSocketType(int32_t aSocketType)
 {
   if (!mPrefBranch)
     return NS_ERROR_NOT_INITIALIZED;
@@ -2104,16 +2064,18 @@ nsNntpIncomingServer::SetSocketType(PRInt32 aSocketType)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName, const nsACString& newName)
+nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
+                                              const nsACString& newName,
+                                              bool hostnameChanged)
 {
   nsresult rv;
   // 1. Do common things in the base class.
-  rv = nsMsgIncomingServer::OnUserOrHostNameChanged(oldName, newName);
+  rv = nsMsgIncomingServer::OnUserOrHostNameChanged(oldName, newName, hostnameChanged);
   NS_ENSURE_SUCCESS(rv,rv);
 
   // 2. Remove file hostinfo.dat so that the new subscribe
   //    list will be reloaded from the new server.
-  nsCOMPtr <nsILocalFile> hostInfoFile;
+  nsCOMPtr <nsIFile> hostInfoFile;
   rv = GetLocalPath(getter_AddRefs(hostInfoFile));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = hostInfoFile->AppendNative(NS_LITERAL_CSTRING(HOSTINFO_FILE_NAME));
@@ -2154,9 +2116,9 @@ nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName, const n
     return NS_OK;
 
   // Now unsubscribe & subscribe.
-  PRUint32 i;
-  PRUint32 cnt = groupList.Length();
-  nsCAutoString cname;
+  uint32_t i;
+  uint32_t cnt = groupList.Length();
+  nsAutoCString cname;
   for (i = 0; i < cnt; i++)
   {
     // unsubscribe.
@@ -2174,3 +2136,11 @@ nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName, const n
   // Force updating the rc file.
   return CommitSubscribeChanges();
 }
+
+NS_IMETHODIMP
+nsNntpIncomingServer::GetSortOrder(int32_t* aSortOrder)
+{
+  NS_ENSURE_ARG_POINTER(aSortOrder);
+  *aSortOrder = 500000000;
+  return NS_OK;
+}
