@@ -1,45 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Seth Spitzer <sspitzer@netscape.com>
- *   Scott MacGregor <mscott@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Håkan Waara <hwaara@chello.se>
- *   David Bienvenu <bienvenu@nventure.com>
- *   Markus Hossner <markushossner@gmx.de>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"    // precompiled header...
 #include "nntpCore.h"
@@ -87,6 +49,7 @@
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIMsgMailSession.h"
 #include "nsISupportsPrimitives.h"
+#include "nsArrayUtils.h"
 
 #undef GetPort  // XXX Windows!
 #undef SetPort  // XXX Windows!
@@ -181,7 +144,7 @@ nsNntpService::SaveMessageToDisk(const char *aMessageURI,
       nsCOMPtr <nsIStreamListener> saveAsListener;
       mailNewsUrl->GetSaveAsListener(aAddDummyEnvelope, aFile, getter_AddRefs(saveAsListener));
 
-      rv = DisplayMessage(aMessageURI, saveAsListener, /* nsIMsgWindow *aMsgWindow */nsnull, aUrlListener, nsnull /*aCharsetOverride */, aURL);
+      rv = DisplayMessage(aMessageURI, saveAsListener, /* nsIMsgWindow *aMsgWindow */nullptr, aUrlListener, nullptr /*aCharsetOverride */, aURL);
     }
     return rv;
 }
@@ -220,7 +183,7 @@ nsNntpService::CreateMessageIDURL(nsIMsgFolder *folder, nsMsgKey key, char **url
     rv = folder->GetName(groupName);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    nsCAutoString uri;
+    nsAutoCString uri;
     uri = rootFolderURI.get();
     uri += '/';
     uri += escapedMessageID;
@@ -248,7 +211,7 @@ nsNntpService::DisplayMessage(const char* aMessageURI, nsISupports * aDisplayCon
   rv = DecomposeNewsMessageURI(aMessageURI, getter_AddRefs(folder), &key);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCAutoString urlStr;
+  nsAutoCString urlStr;
   // if we are displaying (or printing), we want the news://host/message-id url
   // we keep the original uri around, for cancelling and so we can get to the
   // articles by doing GROUP and then ARTICLE <n>.
@@ -293,14 +256,14 @@ nsNntpService::DisplayMessage(const char* aMessageURI, nsISupports * aDisplayCon
     rv = folder->GetServer(getter_AddRefs(server));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    PRInt32 port = 0;
+    int32_t port = 0;
     rv = url->GetPort(&port);
     if (NS_FAILED(rv) || (port <= 0))
     {
       rv = server->GetPort(&port);
       if (NS_FAILED(rv) || (port <= 0))
       {
-        PRInt32 socketType;
+        int32_t socketType;
         rv = server->GetSocketType(&socketType);
         NS_ENSURE_SUCCESS(rv, rv);
 
@@ -321,7 +284,7 @@ nsNntpService::DisplayMessage(const char* aMessageURI, nsISupports * aDisplayCon
     // Now look in the memory cache
     if (!hasMsgOffline)
     {
-      rv = IsMsgInMemCache(url, folder, nsnull, &hasMsgOffline);
+      rv = IsMsgInMemCache(url, folder, nullptr, &hasMsgOffline);
       NS_ENSURE_SUCCESS(rv, rv);
     }
 
@@ -445,7 +408,7 @@ NS_IMETHODIMP nsNntpService::FetchMimePart(nsIURI *aURI, const char *aMessageURI
 // while offline working
 //    if (msgMessageUrl)
 //    {
-//      nsCAutoString spec;
+//      nsAutoCString spec;
 //      aURI->GetSpec(spec);
 //      msgMessageUrl->SetOriginalSpec(spec.get());
 //    }
@@ -465,14 +428,14 @@ NS_IMETHODIMP nsNntpService::OpenAttachment(const char *aContentType,
 
   nsCOMPtr<nsIURI> url;
   nsresult rv = NS_OK;
-  nsCAutoString newsUrl;
+  nsAutoCString newsUrl;
   newsUrl = aUrl;
   newsUrl += "&type=";
   newsUrl += aContentType;
   newsUrl += "&filename=";
   newsUrl += aFileName;
 
-  NewURI(newsUrl, nsnull, nsnull, getter_AddRefs(url));
+  NewURI(newsUrl, nullptr, nullptr, getter_AddRefs(url));
 
   if (NS_SUCCEEDED(rv) && url)
   {
@@ -527,7 +490,7 @@ NS_IMETHODIMP nsNntpService::GetUrlForUri(const char *aMessageURI, nsIURI **aURL
   NS_ENSURE_SUCCESS(rv,rv);
 
   // this is only called by view message source
-  rv = ConstructNntpUrl(messageIdURL.get(), nsnull, aMsgWindow, aMessageURI, nsINntpUrl::ActionFetchArticle, aURL);
+  rv = ConstructNntpUrl(messageIdURL.get(), nullptr, aMsgWindow, aMessageURI, nsINntpUrl::ActionFetchArticle, aURL);
   NS_ENSURE_SUCCESS(rv,rv);
   if (folder && *aURL)
   {
@@ -571,7 +534,7 @@ nsNntpService::DecomposeNewsMessageURI(const char * aMessageURI, nsIMsgFolder **
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Get the group name and key from the url
-  nsCAutoString groupName;
+  nsAutoCString groupName;
   rv = nntpUrl->GetGroup(groupName);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -598,7 +561,7 @@ nsNntpService::GetFolderFromUri(const char *aUri, nsIMsgFolder **aFolder)
   nsresult rv = NS_NewURI(getter_AddRefs(uri), nsDependentCString(aUri));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCAutoString path;
+  nsAutoCString path;
   rv = uri->GetPath(path);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -648,12 +611,12 @@ nsNntpService::CopyMessage(const char * aSrcMessageURI, nsIStreamListener * aMai
   nsCOMPtr<nsISupports> streamSupport = do_QueryInterface(aMailboxCopyHandler, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  rv = DisplayMessage(aSrcMessageURI, streamSupport, aMsgWindow, aUrlListener, nsnull, aURL);
+  rv = DisplayMessage(aSrcMessageURI, streamSupport, aMsgWindow, aUrlListener, nullptr, aURL);
   return rv;
 }
 
 NS_IMETHODIMP
-nsNntpService::CopyMessages(PRUint32 aNumKeys, nsMsgKey *akeys,
+nsNntpService::CopyMessages(uint32_t aNumKeys, nsMsgKey *akeys,
                             nsIMsgFolder *srcFolder,
                             nsIStreamListener * aMailboxCopyHandler,
                             bool moveMessage,
@@ -664,40 +627,6 @@ nsNntpService::CopyMessages(PRUint32 aNumKeys, nsMsgKey *akeys,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-struct findNewsServerEntry {
-  const char *newsgroup;
-  nsINntpIncomingServer *server;
-};
-
-
-bool
-nsNntpService::findNewsServerWithGroup(nsISupports *aElement, void *data)
-{
-  nsresult rv;
-
-  nsCOMPtr<nsINntpIncomingServer> newsserver = do_QueryInterface(aElement, &rv);
-  if (NS_FAILED(rv) || ! newsserver) return true;
-
-  findNewsServerEntry *entry = (findNewsServerEntry*) data;
-
-  bool containsGroup = false;
-  NS_ASSERTION(MsgIsUTF8(nsDependentCString(entry->newsgroup)),
-               "newsgroup is not in UTF-8");
-  rv = newsserver->ContainsNewsgroup(nsDependentCString(entry->newsgroup),
-                                     &containsGroup);
-  if (NS_FAILED(rv)) return true;
-
-  if (containsGroup)
-  {
-    entry->server = newsserver;
-    return false;            // stop on first find
-  }
-  else
-  {
-    return true;
-  }
-}
-
 nsresult
 nsNntpService::FindServerWithNewsgroup(nsCString &host, nsCString &groupName)
 {
@@ -705,25 +634,39 @@ nsNntpService::FindServerWithNewsgroup(nsCString &host, nsCString &groupName)
 
   nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
-  nsCOMPtr<nsISupportsArray> servers;
 
+  nsCOMPtr<nsIArray> servers;
   rv = accountManager->GetAllServers(getter_AddRefs(servers));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  findNewsServerEntry serverInfo;
-  serverInfo.server = nsnull;
-  serverInfo.newsgroup = groupName.get();
+  NS_ASSERTION(MsgIsUTF8(groupName),
+               "newsgroup is not in UTF-8");
 
   // XXX TODO
   // this only looks at the list of subscribed newsgroups.
   // fix to use the hostinfo.dat information
-  servers->EnumerateForwards(findNewsServerWithGroup, (void *)&serverInfo);
-  if (serverInfo.server)
-  {
-    nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(serverInfo.server);
-    rv = server->GetHostName(host);
-  }
 
+  uint32_t length;
+  rv = servers->GetLength(&length);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  for (uint32_t i = 0; i < length; ++i)
+  {
+    nsCOMPtr<nsINntpIncomingServer> newsserver(do_QueryElementAt(servers, i, &rv));
+    if (NS_FAILED(rv))
+      continue;
+
+    bool containsGroup = false;
+    rv = newsserver->ContainsNewsgroup(groupName,
+                                       &containsGroup);
+    if (containsGroup)
+    {
+      nsCOMPtr<nsIMsgIncomingServer> server(do_QueryInterface(newsserver, &rv));
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      return server->GetHostName(host);
+    }
+  }
   return rv;
 }
 
@@ -747,7 +690,7 @@ nsNntpService::SetUpNntpUrlForPosting(const char *aAccountKey, char **newsUrlSpe
   nsresult rv = NS_OK;
 
   nsCString host;
-  PRInt32 port;
+  int32_t port;
 
   nsCOMPtr<nsIMsgIncomingServer> nntpServer;
   rv = GetNntpServerByAccount(aAccountKey, getter_AddRefs(nntpServer));
@@ -782,20 +725,19 @@ nsNntpService::GenerateNewsHeaderValsForPosting(const nsACString& newsgroupsList
   // we are not going to allow the user to cross post to multiple hosts.
   // if we detect that, we stop and return error.
 
-  nsCAutoString host;
-  nsCAutoString str;
-  nsCAutoString newsgroups;
+  nsAutoCString host;
+  nsAutoCString newsgroups;
 
   nsTArray<nsCString> list;
   ParseString(newsgroupsList, ',', list);
-  for (PRUint32 index = 0; index < list.Length(); index++)
+  for (uint32_t index = 0; index < list.Length(); index++)
   {
     list[index].StripWhitespace();
     if (!list[index].IsEmpty())
     {
-      nsCAutoString currentHost;
-      nsCAutoString theRest;
-      // does str start with "news:/"?
+      nsAutoCString currentHost;
+      nsAutoCString theRest;
+      // does list[index] start with "news:/"?
       if (StringBeginsWith(list[index], NS_LITERAL_CSTRING(kNewsRootURI)))
       {
         // we have news://group or news://host/group
@@ -811,10 +753,10 @@ nsNntpService::GenerateNewsHeaderValsForPosting(const nsACString& newsgroupsList
         theRest = list[index];
 
       // theRest is "group" or "host/group"
-      PRInt32 slashpos = theRest.FindChar('/');
+      int32_t slashpos = theRest.FindChar('/');
       if (slashpos > 0 )
       {
-        nsCAutoString currentGroup;
+        nsAutoCString currentGroup;
 
         // theRest is "host/group"
         currentHost = StringHead(theRest, slashpos);
@@ -833,14 +775,14 @@ nsNntpService::GenerateNewsHeaderValsForPosting(const nsACString& newsgroupsList
       }
       else
       {
-        // str is "group"
-        rv = FindHostFromGroup(currentHost, str);
+        // theRest is "group"
+        rv = FindHostFromGroup(currentHost, theRest);
         if (NS_FAILED(rv))
           return rv;
         // build up the newsgroups
         if (!newsgroups.IsEmpty())
           newsgroups += ",";
-        newsgroups += list[index];
+        newsgroups += theRest;
       }
 
       if (!currentHost.IsEmpty())
@@ -895,44 +837,43 @@ nsNntpService::PostMessage(nsIFile *aFileToPost, const char *newsgroupsNames, co
   // aMsgWindow might be null
   NS_ENSURE_ARG_POINTER(newsgroupsNames);
 
-  if (*newsgroupsNames == '\0') return NS_ERROR_INVALID_ARG;
+  NS_ENSURE_ARG(*newsgroupsNames);
 
   NS_LOCK_INSTANCE();
 
   nsresult rv;
 
   nsCOMPtr <nsINntpUrl> nntpUrl = do_CreateInstance(NS_NNTPURL_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = nntpUrl->SetNewsAction(nsINntpUrl::ActionPostArticle);
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCString newsUrlSpec;
   rv = SetUpNntpUrlForPosting(aAccountKey, getter_Copies(newsUrlSpec));
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgMailNewsUrl> mailnewsurl = do_QueryInterface(nntpUrl, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
-  if (!mailnewsurl) return NS_ERROR_FAILURE;
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  mailnewsurl->SetSpec(newsUrlSpec);
+  rv = mailnewsurl->SetSpec(newsUrlSpec);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (aUrlListener) // register listener if there is one...
     mailnewsurl->RegisterListener(aUrlListener);
 
-  nsCOMPtr <nsINNTPNewsgroupPost> post = do_CreateInstance(NS_NNTPNEWSGROUPPOST_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
-  if (!post) return NS_ERROR_FAILURE;
+  nsCOMPtr<nsINNTPNewsgroupPost> post = do_CreateInstance(NS_NNTPNEWSGROUPPOST_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = post->SetPostMessageFile(aFileToPost);
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = nntpUrl->SetMessageToPost(post);
-  NS_ENSURE_SUCCESS(rv,rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr <nsIURI> url = do_QueryInterface(nntpUrl);
-  rv = RunNewsUrl(url, aMsgWindow, nsnull /* consumer */);
-  NS_ENSURE_SUCCESS(rv,rv);
+  nsCOMPtr<nsIURI> url = do_QueryInterface(nntpUrl);
+  rv = RunNewsUrl(url, aMsgWindow, nullptr /* consumer */);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (_retval)
     rv = CallQueryInterface(nntpUrl, _retval);
@@ -943,7 +884,7 @@ nsNntpService::PostMessage(nsIFile *aFileToPost, const char *newsgroupsNames, co
 }
 
 nsresult
-nsNntpService::ConstructNntpUrl(const char *urlString, nsIUrlListener *aUrlListener, nsIMsgWindow *aMsgWindow, const char *originalMessageUri, PRInt32 action, nsIURI ** aUrl)
+nsNntpService::ConstructNntpUrl(const char *urlString, nsIUrlListener *aUrlListener, nsIMsgWindow *aMsgWindow, const char *originalMessageUri, int32_t action, nsIURI ** aUrl)
 {
   nsresult rv = NS_OK;
 
@@ -954,7 +895,8 @@ nsNntpService::ConstructNntpUrl(const char *urlString, nsIUrlListener *aUrlListe
   mailnewsurl->SetMsgWindow(aMsgWindow);
   nsCOMPtr <nsIMsgMessageUrl> msgUrl = do_QueryInterface(nntpUrl);
   msgUrl->SetUri(originalMessageUri);
-  mailnewsurl->SetSpec(nsDependentCString(urlString));
+  rv = mailnewsurl->SetSpec(nsDependentCString(urlString));
+  NS_ENSURE_SUCCESS(rv, rv);
   nntpUrl->SetNewsAction(action);
 
   if (originalMessageUri)
@@ -974,7 +916,7 @@ nsNntpService::ConstructNntpUrl(const char *urlString, nsIUrlListener *aUrlListe
 
 nsresult
 nsNntpService::CreateNewsAccount(const char *aHostname, bool aUseSSL,
-                                 PRInt32 aPort, nsIMsgIncomingServer **aServer)
+                                 int32_t aPort, nsIMsgIncomingServer **aServer)
 {
   NS_ENSURE_ARG_POINTER(aHostname);
   NS_ENSURE_ARG_POINTER(aServer);
@@ -1029,10 +971,10 @@ nsNntpService::CreateNewsAccount(const char *aHostname, bool aUseSSL,
 nsresult
 nsNntpService::GetServerForUri(nsIURI *aUri, nsINntpIncomingServer **aServer)
 {
-  nsCAutoString hostName;
-  nsCAutoString scheme;
-  nsCAutoString path;
-  PRInt32 port = 0;
+  nsAutoCString hostName;
+  nsAutoCString scheme;
+  nsAutoCString path;
+  int32_t port = 0;
   nsresult rv;
 
   rv = aUri->GetAsciiHost(hostName);
@@ -1051,7 +993,7 @@ nsNntpService::GetServerForUri(nsIURI *aUri, nsINntpIncomingServer **aServer)
   // Grab all servers for if this is a no-authority URL. This also loads
   // accounts if they haven't been loaded, i.e., we're running this straight
   // from the command line
-  nsCOMPtr <nsISupportsArray> servers;
+  nsCOMPtr <nsIArray> servers;
   rv = accountManager->GetAllServers(getter_AddRefs(servers));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1090,7 +1032,7 @@ nsNntpService::GetServerForUri(nsIURI *aUri, nsINntpIncomingServer **aServer)
 
   NS_IF_ADDREF(*aServer = nntpServer);
 
-  nsCAutoString spec;
+  nsAutoCString spec;
   rv = aUri->GetSpec(spec);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1162,7 +1104,7 @@ NS_IMETHODIMP nsNntpService::GetNewNews(nsINntpIncomingServer *nntpServer, const
   if (strncmp(uri, kNewsRootURI, kNewsRootURILen) == 0)
   {
     nsCOMPtr<nsIURI> aUrl;
-    rv = ConstructNntpUrl(uri, aUrlListener, aMsgWindow, nsnull, nsINntpUrl::ActionGetNewNews, getter_AddRefs(aUrl));
+    rv = ConstructNntpUrl(uri, aUrlListener, aMsgWindow, nullptr, nsINntpUrl::ActionGetNewNews, getter_AddRefs(aUrl));
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsINntpUrl> nntpUrl = do_QueryInterface(aUrl);
@@ -1176,7 +1118,7 @@ NS_IMETHODIMP nsNntpService::GetNewNews(nsINntpIncomingServer *nntpServer, const
     if (mailNewsUrl)
       mailNewsUrl->SetUpdatingFolder(true);
 
-    rv = RunNewsUrl(aUrl, aMsgWindow, nsnull);
+    rv = RunNewsUrl(aUrl, aMsgWindow, nullptr);
 
     if (_retval)
       NS_IF_ADDREF(*_retval = aUrl);
@@ -1230,21 +1172,21 @@ NS_IMETHODIMP nsNntpService::GetDefaultDoBiff(bool *aDoBiff)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpService::GetDefaultPort(PRInt32 *aDefaultPort)
+NS_IMETHODIMP nsNntpService::GetDefaultPort(int32_t *aDefaultPort)
 {
     NS_ENSURE_ARG_POINTER(aDefaultPort);
     *aDefaultPort = nsINntpUrl::DEFAULT_NNTP_PORT;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpService::AllowPort(PRInt32 port, const char *scheme, bool *_retval)
+NS_IMETHODIMP nsNntpService::AllowPort(int32_t port, const char *scheme, bool *_retval)
 {
     *_retval = true; // allow news on any port
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpService::GetDefaultServerPort(bool aUseSSL, PRInt32 *aDefaultPort)
+nsNntpService::GetDefaultServerPort(bool aUseSSL, int32_t *aDefaultPort)
 {
     nsresult rv = NS_OK;
 
@@ -1257,7 +1199,7 @@ nsNntpService::GetDefaultServerPort(bool aUseSSL, PRInt32 *aDefaultPort)
     return rv;
 }
 
-NS_IMETHODIMP nsNntpService::GetProtocolFlags(PRUint32 *aUritype)
+NS_IMETHODIMP nsNntpService::GetProtocolFlags(uint32_t *aUritype)
 {
     NS_ENSURE_ARG_POINTER(aUritype);
     *aUritype = URI_NORELATIVE | URI_FORBIDS_AUTOMATIC_DOCUMENT_REPLACEMENT |
@@ -1282,7 +1224,7 @@ NS_IMETHODIMP nsNntpService::NewURI(const nsACString &aSpec,
 
     if (aBaseURI)
     {
-      nsCAutoString newSpec;
+      nsAutoCString newSpec;
       aBaseURI->Resolve(aSpec, newSpec);
       rv = nntpUri->SetSpec(newSpec);
     }
@@ -1303,24 +1245,24 @@ NS_IMETHODIMP nsNntpService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
   nsCOMPtr<nsINntpIncomingServer> server;
   rv = GetServerForUri(aURI, getter_AddRefs(server));
   NS_ENSURE_SUCCESS(rv, rv);
-  return server->GetNntpChannel(aURI, nsnull, _retval);
+  return server->GetNntpChannel(aURI, nullptr, _retval);
 }
 
 NS_IMETHODIMP
-nsNntpService::SetDefaultLocalPath(nsILocalFile *aPath)
+nsNntpService::SetDefaultLocalPath(nsIFile *aPath)
 {
     NS_ENSURE_ARG(aPath);
     return NS_SetPersistentFile(PREF_MAIL_ROOT_NNTP_REL, PREF_MAIL_ROOT_NNTP, aPath);
 }
 
 NS_IMETHODIMP
-nsNntpService::GetDefaultLocalPath(nsILocalFile ** aResult)
+nsNntpService::GetDefaultLocalPath(nsIFile ** aResult)
 {
     NS_ENSURE_ARG_POINTER(aResult);
-    *aResult = nsnull;
+    *aResult = nullptr;
 
     bool havePref;
-    nsCOMPtr<nsILocalFile> localFile;
+    nsCOMPtr<nsIFile> localFile;
     nsresult rv = NS_GetPersistentFile(PREF_MAIL_ROOT_NNTP_REL,
                               PREF_MAIL_ROOT_NNTP,
                               NS_APP_NEWS_50_DIR,
@@ -1425,7 +1367,7 @@ NS_IMETHODIMP nsNntpService::DisplayMessageForPrinting(const char* aMessageURI,
                                                   nsIMsgWindow *aMsgWindow, nsIUrlListener * aUrlListener, nsIURI ** aURL)
 {
   mPrintingOperation = true;
-  nsresult rv = DisplayMessage(aMessageURI, aDisplayConsumer, aMsgWindow, aUrlListener, nsnull, aURL);
+  nsresult rv = DisplayMessage(aMessageURI, aDisplayConsumer, aMsgWindow, aUrlListener, nullptr, aURL);
   mPrintingOperation = false;
   return rv;
 }
@@ -1442,7 +1384,7 @@ nsNntpService::StreamMessage(const char *aMessageURI, nsISupports *aConsumer,
     // The nntp protocol object will look for "header=filter" to decide if it wants to convert
     // the data instead of using aConvertData. It turns out to be way too hard to pass aConvertData
     // all the way over to the nntp protocol object.
-    nsCAutoString aURIString(aMessageURI);
+    nsAutoCString aURIString(aMessageURI);
 
     if (!aAdditionalHeader.IsEmpty())
     {
@@ -1456,7 +1398,7 @@ nsNntpService::StreamMessage(const char *aMessageURI, nsISupports *aConsumer,
     nsresult rv = DecomposeNewsMessageURI(aMessageURI, getter_AddRefs(folder), &key);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString urlStr;
+    nsAutoCString urlStr;
     rv = CreateMessageIDURL(folder, key, getter_Copies(urlStr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1481,14 +1423,14 @@ nsNntpService::StreamMessage(const char *aMessageURI, nsISupports *aConsumer,
         rv = folder->GetServer(getter_AddRefs(server));
         NS_ENSURE_SUCCESS(rv, rv);
 
-        PRInt32 socketType;
+        int32_t socketType;
         rv = server->GetSocketType(&socketType);
         NS_ENSURE_SUCCESS(rv, rv);
 
         url->SetPort((socketType == nsMsgSocketType::SSL) ?
                      nsINntpUrl::DEFAULT_NNTPS_PORT : nsINntpUrl::DEFAULT_NNTP_PORT);
 
-        rv = IsMsgInMemCache(url, folder, nsnull, &hasMsgOffline);
+        rv = IsMsgInMemCache(url, folder, nullptr, &hasMsgOffline);
         NS_ENSURE_SUCCESS(rv, rv);
       }
 
@@ -1527,20 +1469,20 @@ NS_IMETHODIMP nsNntpService::StreamHeaders(const char *aMessageURI,
   folder->HasMsgOffline(key, &hasMsgOffline);
   if (hasMsgOffline)
   {
-    PRInt64 messageOffset;
-    PRUint32 messageSize;
+    int64_t messageOffset;
+    uint32_t messageSize;
     folder->GetOfflineFileStream(key, &messageOffset, &messageSize, getter_AddRefs(inputStream));
     if (inputStream)
       return MsgStreamMsgHeaders(inputStream, aConsumer);
   }
-  nsCAutoString urlStr;
+  nsAutoCString urlStr;
   rv = CreateMessageIDURL(folder, key, getter_Copies(urlStr));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsNewsAction action = nsINntpUrl::ActionFetchArticle;
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(urlStr.get(), aUrlListener, nsnull, aMessageURI,
+  rv = ConstructNntpUrl(urlStr.get(), aUrlListener, nullptr, aMessageURI,
                         action, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsICacheEntryDescriptor> cacheEntry;
@@ -1571,10 +1513,10 @@ NS_IMETHODIMP nsNntpService::IsMsgInMemCache(nsIURI *aUrl,
   if (mCacheSession)
   {
     // check if message is in memory cache
-    nsCAutoString cacheKey;
+    nsAutoCString cacheKey;
     aUrl->GetAsciiSpec(cacheKey);
     // nntp urls are truncated at the query part when used as cache keys
-    PRInt32 pos = cacheKey.FindChar('?');
+    int32_t pos = cacheKey.FindChar('?');
     if (pos != -1)
       cacheKey.SetLength(pos);
 
@@ -1607,7 +1549,7 @@ NS_IMETHODIMP nsNntpService::Search(nsIMsgSearchSession *aSearchSession, nsIMsgW
 
   nsCOMPtr <nsIUrlListener> urlListener = do_QueryInterface(aSearchSession);
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(searchUrl.get(), urlListener, aMsgWindow, nsnull, nsINntpUrl::ActionSearch, getter_AddRefs(url));
+  rv = ConstructNntpUrl(searchUrl.get(), urlListener, aMsgWindow, nullptr, nsINntpUrl::ActionSearch, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsCOMPtr<nsIMsgMailNewsUrl> msgurl (do_QueryInterface(url));
@@ -1615,7 +1557,7 @@ NS_IMETHODIMP nsNntpService::Search(nsIMsgSearchSession *aSearchSession, nsIMsgW
     msgurl->SetSearchSession(aSearchSession);
 
   // run the url to update the counts
-  return RunNewsUrl(url, nsnull, nsnull);
+  return RunNewsUrl(url, nullptr, nullptr);
 }
 
 NS_IMETHODIMP
@@ -1631,20 +1573,27 @@ nsNntpService::GetListOfGroupsOnServer(nsINntpIncomingServer *aNntpServer, nsIMs
 
   nsCString serverUri;
   rv = server->GetServerURI(serverUri);
+  nsNewsAction newsAction;
   if (aGetOnlyNew)
+  {
     serverUri.AppendLiteral("/?newgroups");
+    newsAction = nsINntpUrl::ActionListNewGroups;
+  }
   else
+  {
     serverUri.AppendLiteral("/*");
+    newsAction = nsINntpUrl::ActionListGroups;
+  }
 
   nsCOMPtr <nsIUrlListener> listener = do_QueryInterface(aNntpServer, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(serverUri.get(), listener, aMsgWindow, nsnull, nsINntpUrl::ActionListGroups, getter_AddRefs(url));
+  rv = ConstructNntpUrl(serverUri.get(), listener, aMsgWindow, nullptr, newsAction, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // now run the url to add the rest of the groups
-  return RunNewsUrl(url, aMsgWindow, nsnull);
+  return RunNewsUrl(url, aMsgWindow, nullptr);
 }
 
 
@@ -1662,9 +1611,9 @@ nsNntpService::Handle(nsICommandLine* aCmdLine)
     NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
 
     nsCOMPtr<nsIDOMWindow> opened;
-    wwatch->OpenWindow(nsnull, "chrome://messenger/content/", "_blank",
+    wwatch->OpenWindow(nullptr, "chrome://messenger/content/", "_blank",
                        "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",
-                       nsnull, getter_AddRefs(opened));
+                       nullptr, getter_AddRefs(opened));
     aCmdLine->SetPreventDefault(true);
   }
 
@@ -1708,7 +1657,11 @@ nsNntpService::HandleContent(const char * aContentType, nsIInterfaceRequestor* a
       nsCString folderURL;
       rv = msgFolder->GetURI(folderURL);
       NS_ENSURE_SUCCESS(rv, rv);
-      
+
+      // this is all we need for listing newsgroup ids.
+      if (!PL_strcasecmp(aContentType, "x-application-newsgroup-listids"))
+        return NS_OK;
+
       nsCOMPtr<nsIMsgWindow> msgWindow;
       mailUrl->GetMsgWindow(getter_AddRefs(msgWindow));
       if (!msgWindow)
@@ -1732,7 +1685,7 @@ nsNntpService::HandleContent(const char * aContentType, nsIInterfaceRequestor* a
           arg->SetData(folderURL);
 
           nsCOMPtr<nsIDOMWindow> newWindow;
-          rv = wwatcher->OpenWindow(nsnull, "chrome://messenger/content/",
+          rv = wwatcher->OpenWindow(nullptr, "chrome://messenger/content/",
             "_blank", "chome,all,dialog=no", arg, getter_AddRefs(newWindow));
           NS_ENSURE_SUCCESS(rv, rv);
         }
