@@ -5,17 +5,17 @@
 
 #include "WebGLContext.h"
 
-#include "mozilla/Preferences.h"
 #include "mozilla/CheckedInt.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/Services.h"
 
 #include "jsfriendapi.h"
 
-#if defined(USE_ANGLE)
 #include "angle/ShaderLang.h"
-#endif
 
 #include <algorithm>
 
+#include "mozilla/Services.h"
 #include "nsIObserverService.h"
 
 using namespace mozilla;
@@ -26,9 +26,9 @@ using namespace mozilla;
 bool
 WebGLProgram::UpdateInfo()
 {
-    mIdentifierMap = nsnull;
-    mIdentifierReverseMap = nsnull;
-    mUniformInfoMap = nsnull;
+    mIdentifierMap = nullptr;
+    mIdentifierReverseMap = nullptr;
+    mUniformInfoMap = nullptr;
 
     mAttribMaxNameLength = 0;
 
@@ -38,8 +38,13 @@ WebGLProgram::UpdateInfo()
     GLint attribCount;
     mContext->gl->fGetProgramiv(mGLName, LOCAL_GL_ACTIVE_ATTRIBUTES, &attribCount);
 
-    mAttribsInUse.resize(mContext->mGLMaxVertexAttribs);
-    std::fill(mAttribsInUse.begin(), mAttribsInUse.end(), false);
+    if (!mAttribsInUse.SetLength(mContext->mGLMaxVertexAttribs)) {
+        mContext->ErrorOutOfMemory("updateInfo: out of memory to allocate %d attribs", mContext->mGLMaxVertexAttribs);
+        return false;
+    }
+
+    for (size_t i = 0; i < mAttribsInUse.Length(); i++)
+        mAttribsInUse[i] = false;
 
     nsAutoArrayPtr<char> nameBuf(new char[mAttribMaxNameLength]);
 
@@ -69,7 +74,7 @@ WebGLProgram::UpdateInfo()
  */
 
 bool
-WebGLContext::ValidateBuffers(int32_t *maxAllowedCount, const char *info)
+WebGLContext::ValidateBuffers(uint32_t *maxAllowedCount, const char *info)
 {
 #ifdef DEBUG
     GLint currentProgram = 0;
@@ -81,8 +86,12 @@ WebGLContext::ValidateBuffers(int32_t *maxAllowedCount, const char *info)
         return false;
 #endif
 
-    *maxAllowedCount = -1;
+    if (mMinInUseAttribArrayLengthCached) {
+        *maxAllowedCount = mMinInUseAttribArrayLength;
+        return true;
+    }
 
+    uint32_t maxAllowed = UINT32_MAX;
     uint32_t attribs = mAttribBuffers.Length();
     for (uint32_t i = 0; i < attribs; ++i) {
         const WebGLVertexAttribData& vd = mAttribBuffers[i];
@@ -92,7 +101,7 @@ WebGLContext::ValidateBuffers(int32_t *maxAllowedCount, const char *info)
         if (!vd.enabled)
             continue;
 
-        if (vd.buf == nsnull) {
+        if (vd.buf == nullptr) {
             ErrorInvalidOperation("%s: no VBO bound to enabled vertex attrib index %d!", info, i);
             return false;
         }
@@ -103,34 +112,40 @@ WebGLContext::ValidateBuffers(int32_t *maxAllowedCount, const char *info)
             continue;
 
         // the base offset
-        CheckedInt32 checked_byteLength
-          = CheckedInt32(vd.buf->ByteLength()) - vd.byteOffset;
-        CheckedInt32 checked_sizeOfLastElement
-          = CheckedInt32(vd.componentSize()) * vd.size;
+        CheckedUint32 checked_byteLength
+            = CheckedUint32(vd.buf->ByteLength()) - vd.byteOffset;
+        CheckedUint32 checked_sizeOfLastElement
+            = CheckedUint32(vd.componentSize()) * vd.size;
 
         if (!checked_byteLength.isValid() ||
             !checked_sizeOfLastElement.isValid())
         {
-          ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
-          return false;
+            ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
+            return false;
         }
 
         if (checked_byteLength.value() < checked_sizeOfLastElement.value()) {
-          *maxAllowedCount = 0;
+            maxAllowed = 0;
+            break;
         } else {
-          CheckedInt32 checked_maxAllowedCount
-            = ((checked_byteLength - checked_sizeOfLastElement) / vd.actualStride()) + 1;
+            CheckedUint32 checked_maxAllowedCount
+                = ((checked_byteLength - checked_sizeOfLastElement) / vd.actualStride()) + 1;
 
-          if (!checked_maxAllowedCount.isValid()) {
-            ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
-            return false;
-          }
+            if (!checked_maxAllowedCount.isValid()) {
+                ErrorInvalidOperation("%s: integer overflow occured while checking vertex attrib %d", info, i);
+                return false;
+            }
 
-          if (*maxAllowedCount == -1 || *maxAllowedCount > checked_maxAllowedCount.value())
-              *maxAllowedCount = checked_maxAllowedCount.value();
+            if (maxAllowed > checked_maxAllowedCount.value())
+                maxAllowed = checked_maxAllowedCount.value();
         }
     }
 
+    *maxAllowedCount = maxAllowed;
+
+    mMinInUseAttribArrayLengthCached = true;
+    mMinInUseAttribArrayLength = *maxAllowedCount;
+
     return true;
 }
 
@@ -364,37 +379,52 @@ bool WebGLContext::ValidateTexImage2DTarget(WebGLenum target, WebGLsizei width,
     return true;
 }
 
-bool WebGLContext::ValidateCompressedTextureSize(WebGLint level, WebGLenum format, WebGLsizei width,
-                                                 WebGLsizei height, uint32_t byteLength, const char* info)
+bool WebGLContext::ValidateCompressedTextureSize(WebGLenum target, WebGLint level,
+                                                 WebGLenum format,
+                                                 WebGLsizei width, WebGLsizei height, uint32_t byteLength, const char* info)
 {
-    CheckedUint32 calculated_byteLength = 0;
-    CheckedUint32 checked_byteLength = byteLength;
-    if (!checked_byteLength.isValid()) {
-        ErrorInvalidValue("%s: data length out of bounds", info);
+    if (!ValidateLevelWidthHeightForTarget(target, level, width, height, info)) {
         return false;
     }
 
+    // negative width and height must already have been handled above
+    MOZ_ASSERT(width >= 0 && height >= 0);
+
+    CheckedUint32 required_byteLength = 0;
+
     switch (format) {
         case LOCAL_GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
         case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
+        case LOCAL_GL_ATC_RGB:
         {
-            calculated_byteLength = ((CheckedUint32(width) + 3) / 4) * ((CheckedUint32(height) + 3) / 4) * 8;
-            if (!calculated_byteLength.isValid() || !(checked_byteLength == calculated_byteLength)) {
-                ErrorInvalidValue("%s: data size does not match dimensions", info);
-                return false;
-            }
+            required_byteLength = ((CheckedUint32(width) + 3) / 4) * ((CheckedUint32(height) + 3) / 4) * 8;
             break;
         }
         case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
         case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+        case LOCAL_GL_ATC_RGBA_EXPLICIT_ALPHA:
+        case LOCAL_GL_ATC_RGBA_INTERPOLATED_ALPHA:
         {
-            calculated_byteLength = ((CheckedUint32(width) + 3) / 4) * ((CheckedUint32(height) + 3) / 4) * 16;
-            if (!calculated_byteLength.isValid() || !(checked_byteLength == calculated_byteLength)) {
-                ErrorInvalidValue("%s: data size does not match dimensions", info);
-                return false;
-            }
+            required_byteLength = ((CheckedUint32(width) + 3) / 4) * ((CheckedUint32(height) + 3) / 4) * 16;
             break;
         }
+        case LOCAL_GL_COMPRESSED_RGB_PVRTC_4BPPV1:
+        case LOCAL_GL_COMPRESSED_RGBA_PVRTC_4BPPV1:
+        {
+            required_byteLength = CheckedUint32(NS_MAX(width, 8)) * CheckedUint32(NS_MAX(height, 8)) / 2;
+            break;
+        }
+        case LOCAL_GL_COMPRESSED_RGB_PVRTC_2BPPV1:
+        case LOCAL_GL_COMPRESSED_RGBA_PVRTC_2BPPV1:
+        {
+            required_byteLength = CheckedUint32(NS_MAX(width, 16)) * CheckedUint32(NS_MAX(height, 8)) / 4;
+            break;
+        }
+    }
+
+    if (!required_byteLength.isValid() || required_byteLength.value() != byteLength) {
+        ErrorInvalidValue("%s: data size does not match dimensions", info);
+        return false;
     }
 
     switch (format) {
@@ -404,19 +434,32 @@ bool WebGLContext::ValidateCompressedTextureSize(WebGLint level, WebGLenum forma
         case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
         {
             if (level == 0 && width % 4 == 0 && height % 4 == 0) {
-                return true;
+                break;
             }
             if (level > 0
                 && (width == 0 || width == 1 || width == 2 || width % 4 == 0)
                 && (height == 0 || height == 1 || height == 2 || height % 4 == 0))
             {
-                return true;
+                break;
+            }
+            ErrorInvalidOperation("%s: level parameter does not match width and height", info);
+            return false;
+        }
+        case LOCAL_GL_COMPRESSED_RGB_PVRTC_4BPPV1:
+        case LOCAL_GL_COMPRESSED_RGB_PVRTC_2BPPV1:
+        case LOCAL_GL_COMPRESSED_RGBA_PVRTC_4BPPV1:
+        case LOCAL_GL_COMPRESSED_RGBA_PVRTC_2BPPV1:
+        {
+            if (!is_pot_assuming_nonnegative(width) ||
+                !is_pot_assuming_nonnegative(height))
+            {
+                ErrorInvalidValue("%s: width and height must be powers of two", info);
+                return false;
             }
         }
     }
 
-    ErrorInvalidOperation("%s: level parameter does not match width and height", info);
-    return false;
+    return true;
 }
 
 bool WebGLContext::ValidateLevelWidthHeightForTarget(WebGLenum target, WebGLint level, WebGLsizei width,
@@ -429,7 +472,9 @@ bool WebGLContext::ValidateLevelWidthHeightForTarget(WebGLenum target, WebGLint
         return false;
     }
 
-    if (!(maxTextureSize >> level)) {
+    WebGLsizei maxAllowedSize = maxTextureSize >> level;
+
+    if (!maxAllowedSize) {
         ErrorInvalidValue("%s: 2^level exceeds maximum texture size", info);
         return false;
     }
@@ -439,8 +484,8 @@ bool WebGLContext::ValidateLevelWidthHeightForTarget(WebGLenum target, WebGLint
         return false;
     }
 
-    if (width > maxTextureSize || height > maxTextureSize) {
-        ErrorInvalidValue("%s: width or height exceeds maximum texture size", info);
+    if (width > maxAllowedSize || height > maxAllowedSize) {
+        ErrorInvalidValue("%s: the maximum texture size for level %d is %d", info, level, maxAllowedSize);
         return false;
     }
 
@@ -449,6 +494,21 @@ bool WebGLContext::ValidateLevelWidthHeightForTarget(WebGLenum target, WebGLint
 
 uint32_t WebGLContext::GetBitsPerTexel(WebGLenum format, WebGLenum type)
 {
+    // If there is no defined format or type, we're not taking up any memory
+    if (!format || !type) {
+        return 0;
+    }
+
+    if (format == LOCAL_GL_DEPTH_COMPONENT) {
+        if (type == LOCAL_GL_UNSIGNED_SHORT)
+            return 2;
+        else if (type == LOCAL_GL_UNSIGNED_INT)
+            return 4;
+    } else if (format == LOCAL_GL_DEPTH_STENCIL) {
+        if (type == LOCAL_GL_UNSIGNED_INT_24_8_EXT)
+            return 4;
+    }
+
     if (type == LOCAL_GL_UNSIGNED_BYTE || type == LOCAL_GL_FLOAT) {
         int multiplier = type == LOCAL_GL_FLOAT ? 32 : 8;
         switch (format) {
@@ -461,11 +521,19 @@ uint32_t WebGLContext::GetBitsPerTexel(WebGLenum format, WebGLenum type)
                 return 3 * multiplier;
             case LOCAL_GL_RGBA:
                 return 4 * multiplier;
+            case LOCAL_GL_COMPRESSED_RGB_PVRTC_2BPPV1:
+            case LOCAL_GL_COMPRESSED_RGBA_PVRTC_2BPPV1:
+                return 2;
             case LOCAL_GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
             case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
+            case LOCAL_GL_ATC_RGB:
+            case LOCAL_GL_COMPRESSED_RGB_PVRTC_4BPPV1:
+            case LOCAL_GL_COMPRESSED_RGBA_PVRTC_4BPPV1:
                 return 4;
             case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
             case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+            case LOCAL_GL_ATC_RGBA_EXPLICIT_ALPHA:
+            case LOCAL_GL_ATC_RGBA_INTERPOLATED_ALPHA:
                 return 8;
             default:
                 break;
@@ -484,8 +552,50 @@ uint32_t WebGLContext::GetBitsPerTexel(WebGLenum format, WebGLenum type)
 bool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type, int jsArrayType,
                                               uint32_t *texelSize, const char *info)
 {
+    if (IsExtensionEnabled(WEBGL_depth_texture)) {
+        if (format == LOCAL_GL_DEPTH_COMPONENT) {
+            if (jsArrayType != -1) {
+                if ((type == LOCAL_GL_UNSIGNED_SHORT && jsArrayType != js::ArrayBufferView::TYPE_UINT16) ||
+                    (type == LOCAL_GL_UNSIGNED_INT && jsArrayType != js::ArrayBufferView::TYPE_UINT32)) {
+                    ErrorInvalidOperation("%s: invalid typed array type for given texture data type", info);
+                    return false;
+                }
+            }
+
+            switch(type) {
+                case LOCAL_GL_UNSIGNED_SHORT:
+                    *texelSize = 2;
+                    break;
+                case LOCAL_GL_UNSIGNED_INT:
+                    *texelSize = 4;
+                    break;
+                default:
+                    ErrorInvalidOperation("%s: invalid type 0x%x", info, type);
+                    return false;
+            }
+
+            return true;
+
+        } else if (format == LOCAL_GL_DEPTH_STENCIL) {
+            if (type != LOCAL_GL_UNSIGNED_INT_24_8_EXT) {
+                ErrorInvalidOperation("%s: invalid format 0x%x", info, format);
+                return false;
+            }
+            if (jsArrayType != -1) {
+                if (jsArrayType != js::ArrayBufferView::TYPE_UINT32) {
+                    ErrorInvalidOperation("%s: invalid typed array type for given texture data type", info);
+                    return false;
+                }
+            }
+
+            *texelSize = 4;
+            return true;
+        }
+    }
+
+
     if (type == LOCAL_GL_UNSIGNED_BYTE ||
-        (IsExtensionEnabled(WebGL_OES_texture_float) && type == LOCAL_GL_FLOAT))
+        (IsExtensionEnabled(OES_texture_float) && type == LOCAL_GL_FLOAT))
     {
         if (jsArrayType != -1) {
             if ((type == LOCAL_GL_UNSIGNED_BYTE && jsArrayType != js::ArrayBufferView::TYPE_UINT8) ||
@@ -555,6 +665,142 @@ bool WebGLContext::ValidateTexFormatAndType(WebGLenum format, WebGLenum type, in
     return false;
 }
 
+bool
+WebGLContext::ValidateUniformLocation(const char* info, WebGLUniformLocation *location_object)
+{
+    if (!ValidateObjectAllowNull(info, location_object))
+        return false;
+    if (!location_object)
+        return false;
+    /* the need to check specifically for !mCurrentProgram here is explained in bug 657556 */
+    if (!mCurrentProgram) {
+        ErrorInvalidOperation("%s: no program is currently bound", info);
+        return false;
+    }
+    if (mCurrentProgram != location_object->Program()) {
+        ErrorInvalidOperation("%s: this uniform location doesn't correspond to the current program", info);
+        return false;
+    }
+    if (mCurrentProgram->Generation() != location_object->ProgramGeneration()) {
+        ErrorInvalidOperation("%s: This uniform location is obsolete since the program has been relinked", info);
+        return false;
+    }
+    return true;
+}
+
+bool
+WebGLContext::ValidateAttribArraySetter(const char* name, uint32_t cnt, uint32_t arrayLength)
+{
+    if (!IsContextStable()) {
+        return false;
+    }
+    if (arrayLength < cnt) {
+        ErrorInvalidOperation("%s: array must be >= %d elements", name, cnt);
+        return false;
+    }
+    return true;
+}
+
+bool
+WebGLContext::ValidateUniformArraySetter(const char* name, uint32_t expectedElemSize, WebGLUniformLocation *location_object,
+                                         GLint& location, uint32_t& numElementsToUpload, uint32_t arrayLength)
+{
+    if (!IsContextStable())
+        return false;
+    if (!ValidateUniformLocation(name, location_object))
+        return false;
+    location = location_object->Location();
+    uint32_t uniformElemSize = location_object->ElementSize();
+    if (expectedElemSize != uniformElemSize) {
+        ErrorInvalidOperation("%s: this function expected a uniform of element size %d,"
+                              " got a uniform of element size %d", name,
+                              expectedElemSize,
+                              uniformElemSize);
+        return false;
+    }
+    if (arrayLength == 0 ||
+        arrayLength % expectedElemSize)
+    {
+        ErrorInvalidValue("%s: expected an array of length a multiple"
+                          " of %d, got an array of length %d", name,
+                          expectedElemSize,
+                          arrayLength);
+        return false;
+    }
+    const WebGLUniformInfo& info = location_object->Info();
+    if (!info.isArray &&
+        arrayLength != expectedElemSize) {
+        ErrorInvalidOperation("%s: expected an array of length exactly"
+                              " %d (since this uniform is not an array"
+                              " uniform), got an array of length %d", name,
+                              expectedElemSize,
+                              arrayLength);
+        return false;
+    }
+    numElementsToUpload =
+        NS_MIN(info.arraySize, arrayLength / expectedElemSize);
+    return true;
+}
+
+bool
+WebGLContext::ValidateUniformMatrixArraySetter(const char* name, int dim, WebGLUniformLocation *location_object,
+                                              GLint& location, uint32_t& numElementsToUpload, uint32_t arrayLength,
+                                              WebGLboolean aTranspose)
+{
+    uint32_t expectedElemSize = (dim)*(dim);
+    if (!IsContextStable())
+        return false;
+    if (!ValidateUniformLocation(name, location_object))
+        return false;
+    location = location_object->Location();
+    uint32_t uniformElemSize = location_object->ElementSize();
+    if (expectedElemSize != uniformElemSize) {
+        ErrorInvalidOperation("%s: this function expected a uniform of element size %d,"
+                              " got a uniform of element size %d", name,
+                              expectedElemSize,
+                              uniformElemSize);
+        return false;
+    }
+    if (arrayLength == 0 ||
+        arrayLength % expectedElemSize)
+    {
+        ErrorInvalidValue("%s: expected an array of length a multiple"
+                          " of %d, got an array of length %d", name,
+                          expectedElemSize,
+                          arrayLength);
+        return false;
+    }
+    const WebGLUniformInfo& info = location_object->Info();
+    if (!info.isArray &&
+        arrayLength != expectedElemSize) {
+        ErrorInvalidOperation("%s: expected an array of length exactly"
+                              " %d (since this uniform is not an array"
+                              " uniform), got an array of length %d", name,
+                              expectedElemSize,
+                              arrayLength);
+        return false;
+    }
+    if (aTranspose) {
+        ErrorInvalidValue("%s: transpose must be FALSE as per the "
+                          "OpenGL ES 2.0 spec", name);
+        return false;
+    }
+    numElementsToUpload =
+        NS_MIN(info.arraySize, arrayLength / (expectedElemSize));
+    return true;
+}
+
+bool
+WebGLContext::ValidateUniformSetter(const char* name, WebGLUniformLocation *location_object, GLint& location)
+{
+    if (!IsContextStable())
+        return false;
+    if (!ValidateUniformLocation(name, location_object))
+        return false;
+    location = location_object->Location();
+    return true;
+}
+
 bool WebGLContext::ValidateAttribIndex(WebGLuint index, const char *info)
 {
     if (index >= mAttribBuffers.Length()) {
@@ -600,22 +846,10 @@ WebGLContext::InitAndValidateGL()
         return false;
     }
 
-#ifdef ANDROID
-    // bug 736123, blacklist WebGL on Adreno
-    bool forceEnabled = Preferences::GetBool("webgl.force-enabled", false);
-    if (!forceEnabled) {
-        int renderer = gl->Renderer();
-        if (renderer == gl::GLContext::RendererAdreno200 ||
-            renderer == gl::GLContext::RendererAdreno205)
-        {
-            GenerateWarning("WebGL blocked on this Adreno driver!");
-            return false;
-        }
-    }
-#endif
-
     mMinCapability = Preferences::GetBool("webgl.min_capability_mode", false);
     mDisableExtensions = Preferences::GetBool("webgl.disable-extensions", false);
+    mLoseContextOnHeapMinimize = Preferences::GetBool("webgl.lose-context-on-heap-minimize", false);
+    mCanLoseContextInForeground = Preferences::GetBool("webgl.can-lose-context-in-foreground", true);
 
     mActiveTexture = 0;
     mWebGLError = LOCAL_GL_NO_ERROR;
@@ -625,12 +859,12 @@ WebGLContext::InitAndValidateGL()
     mBound2DTextures.Clear();
     mBoundCubeMapTextures.Clear();
 
-    mBoundArrayBuffer = nsnull;
-    mBoundElementArrayBuffer = nsnull;
-    mCurrentProgram = nsnull;
+    mBoundArrayBuffer = nullptr;
+    mBoundElementArrayBuffer = nullptr;
+    mCurrentProgram = nullptr;
 
-    mBoundFramebuffer = nsnull;
-    mBoundRenderbuffer = nsnull;
+    mBoundFramebuffer = nullptr;
+    mBoundRenderbuffer = nullptr;
 
     MakeContextCurrent();
 
@@ -670,11 +904,13 @@ WebGLContext::InitAndValidateGL()
     if (MinCapabilityMode()) {
         mGLMaxTextureSize = MINVALUE_GL_MAX_TEXTURE_SIZE;
         mGLMaxCubeMapTextureSize = MINVALUE_GL_MAX_CUBE_MAP_TEXTURE_SIZE;
+        mGLMaxRenderbufferSize = MINVALUE_GL_MAX_RENDERBUFFER_SIZE;
         mGLMaxTextureImageUnits = MINVALUE_GL_MAX_TEXTURE_IMAGE_UNITS;
         mGLMaxVertexTextureImageUnits = MINVALUE_GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS;
     } else {
         gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_SIZE, &mGLMaxTextureSize);
         gl->fGetIntegerv(LOCAL_GL_MAX_CUBE_MAP_TEXTURE_SIZE, &mGLMaxCubeMapTextureSize);
+        gl->fGetIntegerv(LOCAL_GL_MAX_RENDERBUFFER_SIZE, &mGLMaxRenderbufferSize);
         gl->fGetIntegerv(LOCAL_GL_MAX_TEXTURE_IMAGE_UNITS, &mGLMaxTextureImageUnits);
         gl->fGetIntegerv(LOCAL_GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS, &mGLMaxVertexTextureImageUnits);
     }
@@ -753,13 +989,21 @@ WebGLContext::InitAndValidateGL()
         }
     }
 
+#ifdef XP_MACOSX
+    if (gl->WorkAroundDriverBugs() &&
+        gl->Vendor() == gl::GLContext::VendorATI) {
+        // The Mac ATI driver, in all known OSX version up to and including 10.8,
+        // renders points sprites upside-down. Apple bug 11778921
+        gl->fPointParameterf(LOCAL_GL_POINT_SPRITE_COORD_ORIGIN, LOCAL_GL_LOWER_LEFT);
+    }
+#endif
+
     // Check the shader validator pref
     NS_ENSURE_TRUE(Preferences::GetRootBranch(), false);
 
     mShaderValidation =
         Preferences::GetBool("webgl.shader_validator", mShaderValidation);
 
-#if defined(USE_ANGLE)
     // initialize shader translator
     if (mShaderValidation) {
         if (!ShInitialize()) {
@@ -767,7 +1011,6 @@ WebGLContext::InitAndValidateGL()
             return false;
         }
     }
-#endif
 
     // Mesa can only be detected with the GL_VERSION string, of the form "2.1 Mesa 7.11.0"
     mIsMesa = strstr((const char *)(gl->fGetString(LOCAL_GL_VERSION)), "Mesa");
