@@ -17,7 +17,7 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsDOMLists.h"
 #include "nsGUIEvent.h"
-#include "nsDOMError.h"
+#include "nsError.h"
 #include "nsIDragService.h"
 #include "nsIScriptableRegion.h"
 #include "nsContentUtils.h"
@@ -26,6 +26,7 @@
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIWebNavigation.h"
 #include "nsIDocShellTreeItem.h"
+#include "nsIScriptContext.h"
 
 using namespace mozilla;
 
@@ -33,15 +34,15 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(nsDOMDataTransfer)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsDOMDataTransfer)
   if (tmp->mFiles) {
     tmp->mFiles->Disconnect();
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mFiles)
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mFiles)
   }
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDragTarget)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDragImage)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDragTarget)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDragImage)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsDOMDataTransfer)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mFiles)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDragTarget)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDragImage)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mFiles)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDragTarget)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDragImage)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsDOMDataTransfer)
@@ -74,7 +75,7 @@ nsDOMDataTransfer::nsDOMDataTransfer()
 {
 }
 
-nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType)
+nsDOMDataTransfer::nsDOMDataTransfer(uint32_t aEventType)
   : mEventType(aEventType),
     mDropEffect(nsIDragService::DRAGDROP_ACTION_NONE),
     mEffectAllowed(nsIDragService::DRAGDROP_ACTION_UNINITIALIZED),
@@ -89,16 +90,16 @@ nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType)
   CacheExternalFormats();
 }
 
-nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType,
-                                     const PRUint32 aEffectAllowed,
+nsDOMDataTransfer::nsDOMDataTransfer(uint32_t aEventType,
+                                     const uint32_t aEffectAllowed,
                                      bool aCursorState,
                                      bool aIsExternal,
                                      bool aUserCancelled,
                                      bool aIsCrossDomainSubFrameDrop,
                                      nsTArray<nsTArray<TransferItem> >& aItems,
                                      nsIDOMElement* aDragImage,
-                                     PRUint32 aDragImageX,
-                                     PRUint32 aDragImageY)
+                                     uint32_t aDragImageX,
+                                     uint32_t aDragImageY)
   : mEventType(aEventType),
     mDropEffect(nsIDragService::DRAGDROP_ACTION_NONE),
     mEffectAllowed(aEffectAllowed),
@@ -133,7 +134,7 @@ NS_IMETHODIMP
 nsDOMDataTransfer::SetDropEffect(const nsAString& aDropEffect)
 {
   // the drop effect can only be 'none', 'copy', 'move' or 'link'.
-  for (PRUint32 e = 0; e <= nsIDragService::DRAGDROP_ACTION_LINK; e++) {
+  for (uint32_t e = 0; e <= nsIDragService::DRAGDROP_ACTION_LINK; e++) {
     if (aDropEffect.EqualsASCII(sEffects[e])) {
       // don't allow copyMove
       if (e != (nsIDragService::DRAGDROP_ACTION_COPY |
@@ -169,7 +170,7 @@ nsDOMDataTransfer::SetEffectAllowed(const nsAString& aEffectAllowed)
   PR_STATIC_ASSERT(nsIDragService::DRAGDROP_ACTION_MOVE == 2);
   PR_STATIC_ASSERT(nsIDragService::DRAGDROP_ACTION_LINK == 4);
 
-  for (PRUint32 e = 0; e < ArrayLength(sEffects); e++) {
+  for (uint32_t e = 0; e < ArrayLength(sEffects); e++) {
     if (aEffectAllowed.EqualsASCII(sEffects[e])) {
       mEffectAllowed = e;
       break;
@@ -180,28 +181,28 @@ nsDOMDataTransfer::SetEffectAllowed(const nsAString& aEffectAllowed)
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::GetDropEffectInt(PRUint32* aDropEffect)
+nsDOMDataTransfer::GetDropEffectInt(uint32_t* aDropEffect)
 {
   *aDropEffect = mDropEffect;
   return  NS_OK;
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::SetDropEffectInt(PRUint32 aDropEffect)
+nsDOMDataTransfer::SetDropEffectInt(uint32_t aDropEffect)
 {
   mDropEffect = aDropEffect;
   return  NS_OK;
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::GetEffectAllowedInt(PRUint32* aEffectAllowed)
+nsDOMDataTransfer::GetEffectAllowedInt(uint32_t* aEffectAllowed)
 {
   *aEffectAllowed = mEffectAllowed;
   return  NS_OK;
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::SetEffectAllowedInt(PRUint32 aEffectAllowed)
+nsDOMDataTransfer::SetEffectAllowedInt(uint32_t aEffectAllowed)
 {
   mEffectAllowed = aEffectAllowed;
   return  NS_OK;
@@ -217,7 +218,7 @@ nsDOMDataTransfer::GetMozUserCancelled(bool* aUserCancelled)
 NS_IMETHODIMP
 nsDOMDataTransfer::GetFiles(nsIDOMFileList** aFileList)
 {
-  *aFileList = nsnull;
+  *aFileList = nullptr;
 
   if (mEventType != NS_DRAGDROP_DROP && mEventType != NS_DRAGDROP_DRAGDROP)
     return NS_OK;
@@ -226,9 +227,9 @@ nsDOMDataTransfer::GetFiles(nsIDOMFileList** aFileList)
     mFiles = new nsDOMFileList(static_cast<nsIDOMDataTransfer*>(this));
     NS_ENSURE_TRUE(mFiles, NS_ERROR_OUT_OF_MEMORY);
 
-    PRUint32 count = mItems.Length();
+    uint32_t count = mItems.Length();
 
-    for (PRUint32 i = 0; i < count; i++) {
+    for (uint32_t i = 0; i < count; i++) {
       nsCOMPtr<nsIVariant> variant;
       nsresult rv = MozGetDataAt(NS_ConvertUTF8toUTF16(kFileMime), i, getter_AddRefs(variant));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -262,14 +263,14 @@ nsDOMDataTransfer::GetFiles(nsIDOMFileList** aFileList)
 NS_IMETHODIMP
 nsDOMDataTransfer::GetTypes(nsIDOMDOMStringList** aTypes)
 {
-  *aTypes = nsnull;
+  *aTypes = nullptr;
 
   nsRefPtr<nsDOMStringList> types = new nsDOMStringList();
   NS_ENSURE_TRUE(types, NS_ERROR_OUT_OF_MEMORY);
 
   if (mItems.Length()) {
     nsTArray<TransferItem>& item = mItems[0];
-    for (PRUint32 i = 0; i < item.Length(); i++)
+    for (uint32_t i = 0; i < item.Length(); i++)
       types->Add(item[i].mFormat);
 
     bool filePresent, filePromisePresent;
@@ -311,8 +312,8 @@ nsDOMDataTransfer::GetData(const nsAString& aFormat, nsAString& aData)
     }
     
     if (lowercaseFormat.EqualsLiteral("url")) {
-      PRInt32 lastidx = 0, idx;
-      PRInt32 length = stringdata.Length();
+      int32_t lastidx = 0, idx;
+      int32_t length = stringdata.Length();
       while (lastidx < length) {
         idx = stringdata.FindChar('\n', lastidx);
         // lines beginning with # are comments
@@ -358,7 +359,7 @@ nsDOMDataTransfer::ClearData(const nsAString& aFormat)
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::GetMozItemCount(PRUint32* aCount)
+nsDOMDataTransfer::GetMozItemCount(uint32_t* aCount)
 {
   *aCount = mItems.Length();
   return NS_OK;
@@ -387,7 +388,7 @@ nsDOMDataTransfer::SetMozCursor(const nsAString& aCursorState)
 NS_IMETHODIMP
 nsDOMDataTransfer::GetMozSourceNode(nsIDOMNode** aSourceNode)
 {
-  *aSourceNode = nsnull;
+  *aSourceNode = nullptr;
 
   nsCOMPtr<nsIDragSession> dragSession = nsContentUtils::GetDragSession();
   if (!dragSession)
@@ -403,9 +404,9 @@ nsDOMDataTransfer::GetMozSourceNode(nsIDOMNode** aSourceNode)
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::MozTypesAt(PRUint32 aIndex, nsIDOMDOMStringList** aTypes)
+nsDOMDataTransfer::MozTypesAt(uint32_t aIndex, nsIDOMDOMStringList** aTypes)
 {
-  *aTypes = nsnull;
+  *aTypes = nullptr;
 
   nsRefPtr<nsDOMStringList> types = new nsDOMStringList();
   NS_ENSURE_TRUE(types, NS_ERROR_OUT_OF_MEMORY);
@@ -413,7 +414,7 @@ nsDOMDataTransfer::MozTypesAt(PRUint32 aIndex, nsIDOMDOMStringList** aTypes)
   if (aIndex < mItems.Length()) {
     // note that you can retrieve the types regardless of their principal
     nsTArray<TransferItem>& item = mItems[aIndex];
-    for (PRUint32 i = 0; i < item.Length(); i++)
+    for (uint32_t i = 0; i < item.Length(); i++)
       types->Add(item[i].mFormat);
   }
 
@@ -425,10 +426,10 @@ nsDOMDataTransfer::MozTypesAt(PRUint32 aIndex, nsIDOMDOMStringList** aTypes)
 
 NS_IMETHODIMP
 nsDOMDataTransfer::MozGetDataAt(const nsAString& aFormat,
-                                PRUint32 aIndex,
+                                uint32_t aIndex,
                                 nsIVariant** aData)
 {
-  *aData = nsnull;
+  *aData = nullptr;
 
   if (aFormat.IsEmpty())
     return NS_OK;
@@ -442,22 +443,22 @@ nsDOMDataTransfer::MozGetDataAt(const nsAString& aFormat,
   nsTArray<TransferItem>& item = mItems[aIndex];
 
   // Check if the caller is allowed to access the drag data. Callers with
-  // UniversalXPConnect privileges can always read the data. During the
+  // chrome privileges can always read the data. During the
   // drop event, allow retrieving the data except in the case where the
   // source of the drag is in a child frame of the caller. In that case,
   // we only allow access to data of the same principal. During other events,
   // only allow access to the data with the same principal.
-  nsIPrincipal* principal = nsnull;
+  nsIPrincipal* principal = nullptr;
   if (mIsCrossDomainSubFrameDrop ||
       (mEventType != NS_DRAGDROP_DROP && mEventType != NS_DRAGDROP_DRAGDROP &&
-       !nsContentUtils::CallerHasUniversalXPConnect())) {
+       !nsContentUtils::IsCallerChrome())) {
     nsresult rv = NS_OK;
     principal = GetCurrentPrincipal(&rv);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  PRUint32 count = item.Length();
-  for (PRUint32 i = 0; i < count; i++) {
+  uint32_t count = item.Length();
+  for (uint32_t i = 0; i < count; i++) {
     TransferItem& formatitem = item[i];
     if (formatitem.mFormat.Equals(format)) {
       bool subsumes;
@@ -500,7 +501,7 @@ nsDOMDataTransfer::MozGetDataAt(const nsAString& aFormat,
 NS_IMETHODIMP
 nsDOMDataTransfer::MozSetDataAt(const nsAString& aFormat,
                                 nsIVariant* aData,
-                                PRUint32 aIndex)
+                                uint32_t aIndex)
 {
   NS_ENSURE_TRUE(aData, NS_ERROR_NULL_POINTER);
 
@@ -519,7 +520,7 @@ nsDOMDataTransfer::MozSetDataAt(const nsAString& aFormat,
   // XXX perhaps this should also limit any non-string type as well
   if ((aFormat.EqualsLiteral("application/x-moz-file-promise") ||
        aFormat.EqualsLiteral("application/x-moz-file")) &&
-       !nsContentUtils::CallerHasUniversalXPConnect()) {
+       !nsContentUtils::IsCallerChrome()) {
     return NS_ERROR_DOM_SECURITY_ERR;
   }
 
@@ -530,7 +531,7 @@ nsDOMDataTransfer::MozSetDataAt(const nsAString& aFormat,
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::MozClearDataAt(const nsAString& aFormat, PRUint32 aIndex)
+nsDOMDataTransfer::MozClearDataAt(const nsAString& aFormat, uint32_t aIndex)
 {
   if (mReadOnly)
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
@@ -551,7 +552,7 @@ nsDOMDataTransfer::MozClearDataAt(const nsAString& aFormat, PRUint32 aIndex)
   nsTArray<TransferItem>& item = mItems[aIndex];
   // count backwards so that the count and index don't have to be adjusted
   // after removing an element
-  for (PRInt32 i = item.Length() - 1; i >= 0; i--) {
+  for (int32_t i = item.Length() - 1; i >= 0; i--) {
     TransferItem& formatitem = item[i];
     if (clearall || formatitem.mFormat.Equals(format)) {
       // don't allow removing data that has a stronger principal
@@ -577,7 +578,7 @@ nsDOMDataTransfer::MozClearDataAt(const nsAString& aFormat, PRUint32 aIndex)
 }
 
 NS_IMETHODIMP
-nsDOMDataTransfer::SetDragImage(nsIDOMElement* aImage, PRInt32 aX, PRInt32 aY)
+nsDOMDataTransfer::SetDragImage(nsIDOMElement* aImage, int32_t aX, int32_t aY)
 {
   if (mReadOnly)
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
@@ -611,7 +612,7 @@ nsDOMDataTransfer::AddElement(nsIDOMElement* aElement)
 }
 
 nsresult
-nsDOMDataTransfer::Clone(PRUint32 aEventType, bool aUserCancelled,
+nsDOMDataTransfer::Clone(uint32_t aEventType, bool aUserCancelled,
                          bool aIsCrossDomainSubFrameDrop,
                          nsIDOMDataTransfer** aNewDataTransfer)
 {
@@ -627,21 +628,32 @@ nsDOMDataTransfer::Clone(PRUint32 aEventType, bool aUserCancelled,
 }
 
 void
-nsDOMDataTransfer::GetTransferables(nsISupportsArray** aArray)
+nsDOMDataTransfer::GetTransferables(nsISupportsArray** aArray,
+                                    nsIDOMNode* aDragTarget)
 {
-  *aArray = nsnull;
+  MOZ_ASSERT(aDragTarget);
+
+  *aArray = nullptr;
 
   nsCOMPtr<nsISupportsArray> transArray =
     do_CreateInstance("@mozilla.org/supports-array;1");
   if (!transArray)
     return;
 
+  nsCOMPtr<nsINode> dragNode = do_QueryInterface(aDragTarget);
+  if (!dragNode)
+    return;
+  nsIDocument* doc = dragNode->GetCurrentDoc();
+  if (!doc)
+    return;
+  nsILoadContext* loadContext = doc->GetLoadContext();
+
   bool added = false;
-  PRUint32 count = mItems.Length();
-  for (PRUint32 i = 0; i < count; i++) {
+  uint32_t count = mItems.Length();
+  for (uint32_t i = 0; i < count; i++) {
 
     nsTArray<TransferItem>& item = mItems[i];
-    PRUint32 count = item.Length();
+    uint32_t count = item.Length();
     if (!count)
       continue;
 
@@ -649,13 +661,14 @@ nsDOMDataTransfer::GetTransferables(nsISupportsArray** aArray)
       do_CreateInstance("@mozilla.org/widget/transferable;1");
     if (!transferable)
       return;
+    transferable->Init(loadContext);
 
-    for (PRUint32 f = 0; f < count; f++) {
+    for (uint32_t f = 0; f < count; f++) {
       TransferItem& formatitem = item[f];
       if (!formatitem.mData) // skip empty items
         continue;
 
-      PRUint32 length;
+      uint32_t length;
       nsCOMPtr<nsISupports> convertedData;
       if (!ConvertFromVariant(formatitem.mData, getter_AddRefs(convertedData), &length))
         continue;
@@ -695,12 +708,12 @@ nsDOMDataTransfer::GetTransferables(nsISupportsArray** aArray)
 bool
 nsDOMDataTransfer::ConvertFromVariant(nsIVariant* aVariant,
                                       nsISupports** aSupports,
-                                      PRUint32* aLength)
+                                      uint32_t* aLength)
 {
-  *aSupports = nsnull;
+  *aSupports = nullptr;
   *aLength = 0;
 
-  PRUint16 type;
+  uint16_t type;
   aVariant->GetDataType(&type);
   if (type == nsIDataType::VTYPE_INTERFACE ||
       type == nsIDataType::VTYPE_INTERFACE_IS) {
@@ -732,7 +745,7 @@ nsDOMDataTransfer::ConvertFromVariant(nsIVariant* aVariant,
   }
 
   PRUnichar* chrs;
-  PRUint32 len = 0;
+  uint32_t len = 0;
   nsresult rv = aVariant->GetAsWStringWithSize(&len, &chrs);
   if (NS_FAILED(rv))
     return false;
@@ -765,7 +778,7 @@ nsDOMDataTransfer::ClearAll()
 nsresult
 nsDOMDataTransfer::SetDataWithPrincipal(const nsAString& aFormat,
                                         nsIVariant* aData,
-                                        PRUint32 aIndex,
+                                        uint32_t aIndex,
                                         nsIPrincipal* aPrincipal)
 {
   nsAutoString format;
@@ -776,8 +789,8 @@ nsDOMDataTransfer::SetDataWithPrincipal(const nsAString& aFormat,
   TransferItem* formatitem;
   if (aIndex < mItems.Length()) {
     nsTArray<TransferItem>& item = mItems[aIndex];
-    PRUint32 count = item.Length();
-    for (PRUint32 i = 0; i < count; i++) {
+    uint32_t count = item.Length();
+    for (uint32_t i = 0; i < count; i++) {
       TransferItem& itemformat = item[i];
       if (itemformat.mFormat.Equals(format)) {
         // don't allow replacing data that has a stronger principal
@@ -821,7 +834,7 @@ nsDOMDataTransfer::GetCurrentPrincipal(nsresult* rv)
 
   nsCOMPtr<nsIPrincipal> currentPrincipal;
   *rv = ssm->GetSubjectPrincipal(getter_AddRefs(currentPrincipal));
-  NS_ENSURE_SUCCESS(*rv, nsnull);
+  NS_ENSURE_SUCCESS(*rv, nullptr);
 
   if (!currentPrincipal)
     ssm->GetSystemPrincipal(getter_AddRefs(currentPrincipal));
@@ -866,10 +879,10 @@ nsDOMDataTransfer::CacheExternalFormats()
   // XXXndeakin there are some other formats but those are platform specific.
   const char* formats[] = { kFileMime, kHTMLMime, kURLMime, kURLDataMime, kUnicodeMime };
 
-  PRUint32 count;
+  uint32_t count;
   dragSession->GetNumDropItems(&count);
-  for (PRUint32 c = 0; c < count; c++) {
-    for (PRUint32 f = 0; f < ArrayLength(formats); f++) {
+  for (uint32_t c = 0; c < count; c++) {
+    for (uint32_t f = 0; f < ArrayLength(formats); f++) {
       // IsDataFlavorSupported doesn't take an index as an argument and just
       // checks if any of the items support a particular flavor, even though
       // the GetData method does take an index. Here, we just assume that
@@ -880,12 +893,12 @@ nsDOMDataTransfer::CacheExternalFormats()
       // the data. When retrieved, GetRealData will read the data.
       if (supported) {
         if (strcmp(formats[f], kUnicodeMime) == 0) {
-          SetDataWithPrincipal(NS_LITERAL_STRING("text/plain"), nsnull, c, sysPrincipal);
+          SetDataWithPrincipal(NS_LITERAL_STRING("text/plain"), nullptr, c, sysPrincipal);
         }
         else {
           if (strcmp(formats[f], kURLDataMime) == 0)
-            SetDataWithPrincipal(NS_LITERAL_STRING("text/uri-list"), nsnull, c, sysPrincipal);
-          SetDataWithPrincipal(NS_ConvertUTF8toUTF16(formats[f]), nsnull, c, sysPrincipal);
+            SetDataWithPrincipal(NS_LITERAL_STRING("text/uri-list"), nullptr, c, sysPrincipal);
+          SetDataWithPrincipal(NS_ConvertUTF8toUTF16(formats[f]), nullptr, c, sysPrincipal);
         }
       }
     }
@@ -893,16 +906,11 @@ nsDOMDataTransfer::CacheExternalFormats()
 }
 
 void
-nsDOMDataTransfer::FillInExternalDragData(TransferItem& aItem, PRUint32 aIndex)
+nsDOMDataTransfer::FillInExternalDragData(TransferItem& aItem, uint32_t aIndex)
 {
   NS_PRECONDITION(mIsExternal, "Not an external drag");
 
   if (!aItem.mData) {
-    nsCOMPtr<nsITransferable> trans =
-      do_CreateInstance("@mozilla.org/widget/transferable;1");
-    if (!trans)
-      return;
-
     NS_ConvertUTF16toUTF8 utf8format(aItem.mFormat);
     const char* format = utf8format.get();
     if (strcmp(format, "text/plain") == 0)
@@ -914,10 +922,20 @@ nsDOMDataTransfer::FillInExternalDragData(TransferItem& aItem, PRUint32 aIndex)
     if (!dragSession)
       return;
 
+    nsCOMPtr<nsITransferable> trans =
+      do_CreateInstance("@mozilla.org/widget/transferable;1");
+    if (!trans)
+      return;
+
+    nsCOMPtr<nsIDOMDocument> domDoc;
+    dragSession->GetSourceDocument(getter_AddRefs(domDoc));
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
+    trans->Init(doc ? doc->GetLoadContext() : nullptr);
+
     trans->AddDataFlavor(format);
     dragSession->GetData(trans, aIndex);
 
-    PRUint32 length = 0;
+    uint32_t length = 0;
     nsCOMPtr<nsISupports> data;
     trans->GetTransferData(format, getter_AddRefs(data), &length);
     if (!data)
