@@ -14,6 +14,10 @@ typedef nsSVGPathGeometryElement nsSVGPolyElementBase;
 
 class gfxContext;
 
+namespace mozilla {
+class DOMSVGPointList;
+}
+
 class nsSVGPolyElement : public nsSVGPolyElementBase,
                          public nsIDOMSVGAnimatedPoints
 {
@@ -42,6 +46,10 @@ public:
   virtual void GetMarkPoints(nsTArray<nsSVGMark> *aMarks);
   virtual void ConstructPath(gfxContext *aCtx);
 
+  // WebIDL
+  already_AddRefed<mozilla::DOMSVGPointList> Points();
+  already_AddRefed<mozilla::DOMSVGPointList> AnimatedPoints();
+
 protected:
   SVGAnimatedPointList mPoints;
 };
