@@ -4,6 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nscore.h"
+#include "nsError.h"
 #include "nsIContent.h"
 #include "nsINodeInfo.h"
 #include "nsIDOMElement.h"
@@ -83,27 +84,27 @@ protected:
      * <treerow> in the rule's <action>.
      */
     nsresult
-    GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult);
+    GetTemplateActionRowFor(int32_t aRow, nsIContent** aResult);
 
     /**
      * Given a row and a column ID, use the row's match to figure out
      * the appropriate <treecell> in the rule's <action>.
      */
     nsresult
-    GetTemplateActionCellFor(PRInt32 aRow, nsITreeColumn* aCol, nsIContent** aResult);
+    GetTemplateActionCellFor(int32_t aRow, nsITreeColumn* aCol, nsIContent** aResult);
 
     /**
      * Return the resource corresponding to a row in the tree.
      */
     nsresult
-    GetResourceFor(PRInt32 aRow, nsIRDFResource** aResource);
+    GetResourceFor(int32_t aRow, nsIRDFResource** aResource);
 
     /**
      * Open a container row, inserting the container's children into
      * the view.
      */
     nsresult
-    OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult);
+    OpenContainer(int32_t aIndex, nsIXULTemplateResult* aResult);
 
     /**
      * Helper for OpenContainer, recursively open subtrees, remembering
@@ -111,24 +112,24 @@ protected:
      */
     nsresult
     OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
-                  PRInt32 aIndex,
+                  int32_t aIndex,
                   nsIXULTemplateResult *aResult,
-                  PRInt32* aDelta);
+                  int32_t* aDelta);
 
     nsresult
     OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
-                           PRInt32 aIndex,
+                           int32_t aIndex,
                            nsIXULTemplateResult *aResult,
                            nsTemplateQuerySet* aQuerySet,
-                           PRInt32* aDelta,
-                           nsTArray<PRInt32>& open);
+                           int32_t* aDelta,
+                           nsTArray<int32_t>& open);
 
     /**
      * Close a container row, removing the container's childrem from
      * the view.
      */
     nsresult
-    CloseContainer(PRInt32 aIndex);
+    CloseContainer(int32_t aIndex);
 
     /**
      * Remove the matches for the rows in a subtree
@@ -154,7 +155,7 @@ protected:
     /**
      * The real sort routine
      */
-    PRInt32
+    int32_t
     CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResult* aRight);
 
     /**
@@ -234,12 +235,12 @@ protected:
     /*
      * Sort hints (compare case, etc)
      */
-    PRUint32 mSortHints;
+    uint32_t mSortHints;
 
     /** 
      * The builder observers.
      */
-    nsCOMPtr<nsISupportsArray> mObservers;
+    nsCOMArray<nsIXULTreeBuilderObserver> mObservers;
 };
 
 //----------------------------------------------------------------------
@@ -247,9 +248,9 @@ protected:
 nsresult
 NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
-    *aResult = nsnull;
+    *aResult = nullptr;
 
-    NS_PRECONDITION(aOuter == nsnull, "no aggregation");
+    NS_PRECONDITION(aOuter == nullptr, "no aggregation");
     if (aOuter)
         return NS_ERROR_NO_AGGREGATION;
 
@@ -275,28 +276,17 @@ NS_IMPL_ADDREF_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
 NS_IMPL_RELEASE_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mBoxObject)
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mSelection)
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mPersistStateStore)
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mObservers)
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mBoxObject)
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mSelection)
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mPersistStateStore)
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mObservers)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
-static bool TraverseObservers(nsISupports* aElement, void *aData)
-{
-    nsCycleCollectionTraversalCallback *cb =
-        static_cast<nsCycleCollectionTraversalCallback*>(aData);
-    NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*cb, "mObservers[i]");
-    cb->NoteXPCOMChild(aElement);
-    return true;
-}
-
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mBoxObject)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mSelection)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mPersistStateStore)
-    if (tmp->mObservers) {
-        tmp->mObservers->EnumerateForwards(TraverseObservers, &cb);
-    }
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mBoxObject)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mSelection)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPersistStateStore)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObservers)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 DOMCI_DATA(XULTreeBuilder, nsXULTreeBuilder)
@@ -316,7 +306,7 @@ nsXULTreeBuilder::nsXULTreeBuilder()
 void
 nsXULTreeBuilder::Uninit(bool aIsFinal)
 {
-    PRInt32 count = mRows.Count();
+    int32_t count = mRows.Count();
     mRows.Clear();
 
     if (mBoxObject) {
@@ -337,7 +327,7 @@ nsXULTreeBuilder::Uninit(bool aIsFinal)
 //
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetResourceAtIndex(PRInt32 aRowIndex, nsIRDFResource** aResult)
+nsXULTreeBuilder::GetResourceAtIndex(int32_t aRowIndex, nsIRDFResource** aResult)
 {
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
@@ -346,7 +336,7 @@ nsXULTreeBuilder::GetResourceAtIndex(PRInt32 aRowIndex, nsIRDFResource** aResult
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetIndexOfResource(nsIRDFResource* aResource, PRInt32* aResult)
+nsXULTreeBuilder::GetIndexOfResource(nsIRDFResource* aResource, int32_t* aResult)
 {
     NS_ENSURE_ARG_POINTER(aResource);
     nsTreeRows::iterator iter = mRows.FindByResource(aResource);
@@ -360,20 +350,13 @@ nsXULTreeBuilder::GetIndexOfResource(nsIRDFResource* aResource, PRInt32* aResult
 NS_IMETHODIMP
 nsXULTreeBuilder::AddObserver(nsIXULTreeBuilderObserver* aObserver)
 {
-    nsresult rv;  
-    if (!mObservers) {
-        rv = NS_NewISupportsArray(getter_AddRefs(mObservers));
-        if (NS_FAILED(rv))
-            return rv;
-    }
-
-    return mObservers->AppendElement(aObserver);
+    return mObservers.AppendObject(aObserver) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsXULTreeBuilder::RemoveObserver(nsIXULTreeBuilderObserver* aObserver)
 {
-    return mObservers ? mObservers->RemoveElement(aObserver) : NS_ERROR_FAILURE;
+    return mObservers.RemoveObject(aObserver) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -445,7 +428,7 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
 //
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetRowCount(PRInt32* aRowCount)
+nsXULTreeBuilder::GetRowCount(int32_t* aRowCount)
 {
     *aRowCount = mRows.Count();
     return NS_OK;
@@ -469,7 +452,7 @@ nsXULTreeBuilder::SetSelection(nsITreeSelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties)
+nsXULTreeBuilder::GetRowProperties(int32_t aIndex, nsISupportsArray* aProperties)
 {
     NS_ENSURE_ARG_POINTER(aProperties);
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
@@ -494,7 +477,7 @@ nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellProperties(PRInt32 aRow, nsITreeColumn* aCol, nsISupportsArray* aProperties)
+nsXULTreeBuilder::GetCellProperties(int32_t aRow, nsITreeColumn* aCol, nsISupportsArray* aProperties)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_ENSURE_ARG_POINTER(aProperties);
@@ -530,7 +513,7 @@ nsXULTreeBuilder::GetColumnProperties(nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainer(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsContainer(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -550,7 +533,7 @@ nsXULTreeBuilder::IsContainer(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, bool* aOpen)
+nsXULTreeBuilder::IsContainerOpen(int32_t aIndex, bool* aOpen)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -572,7 +555,7 @@ nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, bool* aOpen)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsContainerEmpty(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -604,7 +587,7 @@ nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsSeparator(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsSeparator(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -620,7 +603,7 @@ nsXULTreeBuilder::IsSeparator(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
+nsXULTreeBuilder::GetParentIndex(int32_t aRowIndex, int32_t* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -634,7 +617,7 @@ nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
 
     // Now walk through our previous siblings, subtracting off each
     // one's subtree size
-    PRInt32 index = iter.GetChildIndex();
+    int32_t index = iter.GetChildIndex();
     while (--index >= 0)
         aRowIndex -= mRows.GetSubtreeSizeFor(parent, index) + 1;
 
@@ -644,7 +627,7 @@ nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, bool* aResult)
+nsXULTreeBuilder::HasNextSibling(int32_t aRowIndex, int32_t aAfterIndex, bool* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -663,7 +646,7 @@ nsXULTreeBuilder::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, bool* a
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetLevel(PRInt32 aRowIndex, PRInt32* aResult)
+nsXULTreeBuilder::GetLevel(int32_t aRowIndex, int32_t* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -677,7 +660,7 @@ nsXULTreeBuilder::GetLevel(PRInt32 aRowIndex, PRInt32* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetImageSrc(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetImageSrc(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -701,7 +684,7 @@ nsXULTreeBuilder::GetImageSrc(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aRes
 
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetProgressMode(PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aResult)
+nsXULTreeBuilder::GetProgressMode(int32_t aRow, nsITreeColumn* aCol, int32_t* aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -730,7 +713,7 @@ nsXULTreeBuilder::GetProgressMode(PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aR
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellValue(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetCellValue(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -753,7 +736,7 @@ nsXULTreeBuilder::GetCellValue(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aRe
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetCellText(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -834,7 +817,7 @@ nsXULTreeBuilder::SetTree(nsITreeBoxObject* aTree)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
+nsXULTreeBuilder::ToggleOpenState(int32_t aIndex)
 {
     if (aIndex < 0 || aIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
@@ -854,14 +837,11 @@ nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
             return rv;
     }
 
-    if (mObservers) {
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnToggleOpenState(aIndex);
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnToggleOpenState(aIndex);
     }
 
     if (mPersistStateStore) {
@@ -907,17 +887,14 @@ nsXULTreeBuilder::CycleHeader(nsITreeColumn* aCol)
     nsCOMPtr<nsIDOMElement> element;
     aCol->GetElement(getter_AddRefs(element));
 
-    if (mObservers) {
-        nsAutoString id;
-        aCol->GetId(id);
+    nsAutoString id;
+    aCol->GetId(id);
 
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnCycleHeader(id.get(), element);
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnCycleHeader(id.get(), element);
     }
 
     return Sort(element);
@@ -926,41 +903,36 @@ nsXULTreeBuilder::CycleHeader(nsITreeColumn* aCol)
 NS_IMETHODIMP
 nsXULTreeBuilder::SelectionChanged()
 {
-    if (mObservers) {
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnSelectionChanged();
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnSelectionChanged();
     }
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::CycleCell(PRInt32 aRow, nsITreeColumn* aCol)
+nsXULTreeBuilder::CycleCell(int32_t aRow, nsITreeColumn* aCol)
 {
     NS_ENSURE_ARG_POINTER(aCol);
-    if (mObservers) {
-        nsAutoString id;
-        aCol->GetId(id);
 
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnCycleCell(aRow, id.get());
-        }
+    nsAutoString id;
+    aCol->GetId(id);
+
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnCycleCell(aRow, id.get());
     }
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsEditable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
+nsXULTreeBuilder::IsEditable(int32_t aRow, nsITreeColumn* aCol, bool* _retval)
 {
     *_retval = true;
     NS_ENSURE_ARG_POINTER(aCol);
@@ -986,7 +958,7 @@ nsXULTreeBuilder::IsEditable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
+nsXULTreeBuilder::IsSelectable(int32_t aRow, nsITreeColumn* aCol, bool* _retval)
 {
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
     if (aRow < 0 || aRow >= mRows.Count())
@@ -1012,14 +984,14 @@ nsXULTreeBuilder::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::SetCellValue(PRInt32 aRow, nsITreeColumn* aCol, const nsAString& aValue)
+nsXULTreeBuilder::SetCellValue(int32_t aRow, nsITreeColumn* aCol, const nsAString& aValue)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::SetCellText(PRInt32 aRow, nsITreeColumn* aCol, const nsAString& aValue)
+nsXULTreeBuilder::SetCellText(int32_t aRow, nsITreeColumn* aCol, const nsAString& aValue)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     return NS_OK;
@@ -1028,50 +1000,41 @@ nsXULTreeBuilder::SetCellText(PRInt32 aRow, nsITreeColumn* aCol, const nsAString
 NS_IMETHODIMP
 nsXULTreeBuilder::PerformAction(const PRUnichar* aAction)
 {
-    if (mObservers) {  
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnPerformAction(aAction);
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnPerformAction(aAction);
     }
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::PerformActionOnRow(const PRUnichar* aAction, PRInt32 aRow)
+nsXULTreeBuilder::PerformActionOnRow(const PRUnichar* aAction, int32_t aRow)
 {
-    if (mObservers) {  
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnPerformActionOnRow(aAction, aRow);
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnPerformActionOnRow(aAction, aRow);
     }
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRow, nsITreeColumn* aCol)
+nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow, nsITreeColumn* aCol)
 {
     NS_ENSURE_ARG_POINTER(aCol);
-    if (mObservers) {  
-        nsAutoString id;
-        aCol->GetId(id);
+    nsAutoString id;
+    aCol->GetId(id);
 
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer)
-                observer->OnPerformActionOnCell(aAction, aRow, id.get());
-        }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer)
+            observer->OnPerformActionOnCell(aAction, aRow, id.get());
     }
 
     return NS_OK;
@@ -1082,8 +1045,7 @@ void
 nsXULTreeBuilder::NodeWillBeDestroyed(const nsINode* aNode)
 {
     nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
-    if (mObservers)
-        mObservers->Clear();
+    mObservers.Clear();
 
     nsXULTemplateBuilder::NodeWillBeDestroyed(aNode);
 }
@@ -1115,7 +1077,7 @@ bool
 nsXULTreeBuilder::GetInsertionLocations(nsIXULTemplateResult* aResult,
                                         nsCOMArray<nsIContent>** aLocations)
 {
-    *aLocations = nsnull;
+    *aLocations = nullptr;
 
     // Get the reference point and check if it is an open container. Rows
     // should not be generated otherwise.
@@ -1159,12 +1121,12 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
             return NS_ERROR_FAILURE;
 
         // Remove the rows from the view
-        PRInt32 row = iter.GetRowIndex();
+        int32_t row = iter.GetRowIndex();
 
         // If the row contains children, remove the matches from the
         // children so that they can be regenerated again if the element
         // gets added back.
-        PRInt32 delta = mRows.GetSubtreeSizeFor(iter);
+        int32_t delta = mRows.GetSubtreeSizeFor(iter);
         if (delta)
             RemoveMatchesFor(*(iter->mSubtree));
 
@@ -1190,8 +1152,8 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
 
     if (aNewMatch && aNewMatch->mResult) {
         // Insertion.
-        PRInt32 row = -1;
-        nsTreeRows::Subtree* parent = nsnull;
+        int32_t row = -1;
+        nsTreeRows::Subtree* parent = nullptr;
         nsIXULTemplateResult* result = aNewMatch->mResult;
 
         nsAutoString ref;
@@ -1239,17 +1201,17 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
             // If we get here, then we're inserting into an open
             // container. By default, place the new element at the
             // end of the container
-            PRInt32 index = parent->Count();
+            int32_t index = parent->Count();
 
             if (mSortVariable) {
                 // Figure out where to put the new element by doing an
                 // insertion sort.
-                PRInt32 left = 0;
-                PRInt32 right = index;
+                int32_t left = 0;
+                int32_t right = index;
 
                 while (left < right) {
                     index = (left + right) / 2;
-                    PRInt32 cmp = CompareResults((*parent)[index].mMatch->mResult, result);
+                    int32_t cmp = CompareResults((*parent)[index].mMatch->mResult, result);
                     if (cmp < 0)
                         left = ++index;
                     else if (cmp > 0)
@@ -1300,7 +1262,7 @@ nsXULTreeBuilder::SynchronizeResult(nsIXULTemplateResult* aResult)
         if (iter == mRows.Last())
             return NS_ERROR_FAILURE;
 
-        PRInt32 row = iter.GetRowIndex();
+        int32_t row = iter.GetRowIndex();
         if (row >= 0)
             mBoxObject->InvalidateRow(row);
 
@@ -1341,7 +1303,7 @@ nsXULTreeBuilder::EnsureSortVariables()
                     mSortVariable = do_GetAtom(sort);
 
                     static nsIContent::AttrValuesArray strings[] =
-                      {&nsGkAtoms::ascending, &nsGkAtoms::descending, nsnull};
+                      {&nsGkAtoms::ascending, &nsGkAtoms::descending, nullptr};
                     switch (child->FindAttrValueIn(kNameSpaceID_None,
                                                    nsGkAtoms::sortDirection,
                                                    strings, eCaseMatters)) {
@@ -1380,7 +1342,7 @@ nsXULTreeBuilder::RebuildAll()
         Uninit(false);
     }
     else if (mBoxObject) {
-        PRInt32 count = mRows.Count();
+        int32_t count = mRows.Count();
         mRows.Clear();
         mBoxObject->RowCountChanged(0, -count);
     }
@@ -1412,7 +1374,7 @@ nsXULTreeBuilder::RebuildAll()
 }
 
 nsresult
-nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
+nsXULTreeBuilder::GetTemplateActionRowFor(int32_t aRow, nsIContent** aResult)
 {
     // Get the template in the DOM from which we're supposed to
     // generate text
@@ -1421,7 +1383,7 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
     // The match stores the indices of the rule and query to use. Use these
     // to look up the right nsTemplateRule and use that rule's action to get
     // the treerow in the template.
-    PRInt16 ruleindex = row.mMatch->RuleIndex();
+    int16_t ruleindex = row.mMatch->RuleIndex();
     if (ruleindex >= 0) {
         nsTemplateQuerySet* qs = mQuerySets[row.mMatch->QuerySetPriority()];
         nsTemplateRule* rule = qs->GetRuleAt(ruleindex);
@@ -1444,16 +1406,16 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
         }
     }
 
-    *aResult = nsnull;
+    *aResult = nullptr;
     return NS_OK;
 }
 
 nsresult
-nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
+nsXULTreeBuilder::GetTemplateActionCellFor(int32_t aRow,
                                            nsITreeColumn* aCol,
                                            nsIContent** aResult)
 {
-    *aResult = nsnull;
+    *aResult = nullptr;
 
     if (!aCol) return NS_ERROR_INVALID_ARG;
 
@@ -1461,11 +1423,11 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
     GetTemplateActionRowFor(aRow, getter_AddRefs(row));
     if (row) {
         nsCOMPtr<nsIAtom> colAtom;
-        PRInt32 colIndex;
+        int32_t colIndex;
         aCol->GetAtom(getter_AddRefs(colAtom));
         aCol->GetIndex(&colIndex);
 
-        PRUint32 j = 0;
+        uint32_t j = 0;
         for (nsIContent* child = row->GetFirstChild();
              child;
              child = child->GetNextSibling()) {
@@ -1478,7 +1440,7 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
                     *aResult = child;
                     break;
                 }
-                else if (j == (PRUint32)colIndex)
+                else if (j == (uint32_t)colIndex)
                     *aResult = child;
                 j++;
             }
@@ -1490,14 +1452,14 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
 }
 
 nsresult
-nsXULTreeBuilder::GetResourceFor(PRInt32 aRow, nsIRDFResource** aResource)
+nsXULTreeBuilder::GetResourceFor(int32_t aRow, nsIRDFResource** aResource)
 {
     nsTreeRows::Row& row = *(mRows[aRow]);
     return GetResultResource(row.mMatch->mResult, aResource);
 }
 
 nsresult
-nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
+nsXULTreeBuilder::OpenContainer(int32_t aIndex, nsIXULTemplateResult* aResult)
 {
     // A row index of -1 in this case means ``open tree body''
     NS_ASSERTION(aIndex >= -1 && aIndex < mRows.Count(), "bad row");
@@ -1519,7 +1481,7 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
     if (! container)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    PRInt32 count;
+    int32_t count;
     OpenSubtreeOf(container, aIndex, aResult, &count);
 
     // Notify the box object
@@ -1536,31 +1498,31 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
 
 nsresult
 nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
-                                PRInt32 aIndex,
+                                int32_t aIndex,
                                 nsIXULTemplateResult *aResult,
-                                PRInt32* aDelta)
+                                int32_t* aDelta)
 {
-    nsAutoTArray<PRInt32, 8> open;
-    PRInt32 count = 0;
+    nsAutoTArray<int32_t, 8> open;
+    int32_t count = 0;
 
-    PRInt32 rulecount = mQuerySets.Length();
+    int32_t rulecount = mQuerySets.Length();
 
-    for (PRInt32 r = 0; r < rulecount; r++) {
+    for (int32_t r = 0; r < rulecount; r++) {
         nsTemplateQuerySet* queryset = mQuerySets[r];
         OpenSubtreeForQuerySet(aSubtree, aIndex, aResult, queryset, &count, open);
     }
 
     // Now recursively deal with any open sub-containers that just got
     // inserted. We need to do this back-to-front to avoid skewing offsets.
-    for (PRInt32 i = open.Length() - 1; i >= 0; --i) {
-        PRInt32 index = open[i];
+    for (int32_t i = open.Length() - 1; i >= 0; --i) {
+        int32_t index = open[i];
 
         nsTreeRows::Subtree* child =
             mRows.EnsureSubtreeFor(aSubtree, index);
 
         nsIXULTemplateResult* result = (*aSubtree)[index].mMatch->mResult;
 
-        PRInt32 delta;
+        int32_t delta;
         OpenSubtreeOf(child, aIndex + index, result, &delta);
         count += delta;
     }
@@ -1580,13 +1542,13 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
 
 nsresult
 nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
-                                         PRInt32 aIndex,
+                                         int32_t aIndex,
                                          nsIXULTemplateResult* aResult,
                                          nsTemplateQuerySet* aQuerySet,
-                                         PRInt32* aDelta,
-                                         nsTArray<PRInt32>& open)
+                                         int32_t* aDelta,
+                                         nsTArray<int32_t>& open)
 {
-    PRInt32 count = *aDelta;
+    int32_t count = *aDelta;
     
     nsCOMPtr<nsISimpleEnumerator> results;
     nsresult rv = mQueryProcessor->GenerateResults(mDataSource, aResult,
@@ -1623,8 +1585,8 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 
         bool generateContent = true;
 
-        nsTemplateMatch* prevmatch = nsnull;
-        nsTemplateMatch* existingmatch = nsnull;
+        nsTemplateMatch* prevmatch = nullptr;
+        nsTemplateMatch* existingmatch = nullptr;
         if (mMatchMap.Get(resultid, &existingmatch)){
             // check if there is an existing match that matched a rule
             while (existingmatch) {
@@ -1637,7 +1599,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 
         nsTemplateMatch *newmatch =
             nsTemplateMatch::Create(mPool, aQuerySet->Priority(),
-                                    nextresult, nsnull);
+                                    nextresult, nullptr);
         if (!newmatch)
             return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1667,9 +1629,9 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                 continue;
             }
 
-            PRInt16 ruleindex;
-            nsTemplateRule* matchedrule = nsnull;
-            rv = DetermineMatchedRule(nsnull, nextresult, aQuerySet,
+            int16_t ruleindex;
+            nsTemplateRule* matchedrule = nullptr;
+            rv = DetermineMatchedRule(nullptr, nextresult, aQuerySet,
                                       &matchedrule, &ruleindex);
             if (NS_FAILED(rv)) {
                 nsTemplateMatch::Destroy(mPool, newmatch, false);
@@ -1692,7 +1654,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                 bool isOpen = false;
                 IsContainerOpen(nextresult, &isOpen);
                 if (isOpen) {
-                    if (open.AppendElement(count) == nsnull)
+                    if (open.AppendElement(count) == nullptr)
                         return NS_ERROR_OUT_OF_MEMORY;
                 }
 
@@ -1717,7 +1679,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 }
 
 nsresult
-nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
+nsXULTreeBuilder::CloseContainer(int32_t aIndex)
 {
     NS_ASSERTION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -1729,7 +1691,7 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
         RemoveMatchesFor(*iter->mSubtree);
 
 
-    PRInt32 count = mRows.GetSubtreeSizeFor(iter);
+    int32_t count = mRows.GetSubtreeSizeFor(iter);
     mRows.RemoveSubtreeFor(iter);
 
     iter->mContainerState = nsTreeRows::eContainerState_Closed;
@@ -1747,7 +1709,7 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
 nsresult
 nsXULTreeBuilder::RemoveMatchesFor(nsTreeRows::Subtree& subtree)
 {
-    for (PRInt32 i = subtree.Count() - 1; i >= 0; --i) {
+    for (int32_t i = subtree.Count() - 1; i >= 0; --i) {
         nsTreeRows::Row& row = subtree[i];
 
         nsTemplateMatch* match = row.mMatch;
@@ -1821,7 +1783,7 @@ nsXULTreeBuilder::Compare(const void* aLeft, const void* aRight, void* aClosure)
     return self->CompareResults(left->mMatch->mResult, right->mMatch->mResult);
 }
 
-PRInt32
+int32_t
 nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResult* aRight)
 {
     // this is an extra check done for RDF queries such that results appear in
@@ -1849,7 +1811,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
         nsCOMPtr<nsISupports> ref;
         nsresult rv = aLeft->GetBindingObjectFor(mRefVariable, getter_AddRefs(ref));
         if (NS_FAILED(rv))
-            return rv;
+            return 0;
 
         nsCOMPtr<nsIRDFResource> container = do_QueryInterface(ref);
         if (container) {
@@ -1858,7 +1820,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
             if (isSequence) {
                 // Determine the indices of the left and right elements
                 // in the container.
-                PRInt32 lindex = 0, rindex = 0;
+                int32_t lindex = 0, rindex = 0;
 
                 nsCOMPtr<nsIRDFResource> leftitem;
                 aLeft->GetResource(getter_AddRefs(leftitem));
@@ -1881,7 +1843,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
         }
     }
 
-    PRInt32 sortorder;
+    int32_t sortorder;
     if (!mQueryProcessor)
         return 0;
 
@@ -1901,7 +1863,7 @@ nsXULTreeBuilder::SortSubtree(nsTreeRows::Subtree* aSubtree)
                  Compare,
                  this);
 
-    for (PRInt32 i = aSubtree->Count() - 1; i >= 0; --i) {
+    for (int32_t i = aSubtree->Count() - 1; i >= 0; --i) {
         nsTreeRows::Subtree* child = (*aSubtree)[i].mSubtree;
         if (child)
             SortSubtree(child);
@@ -1913,20 +1875,17 @@ nsXULTreeBuilder::SortSubtree(nsTreeRows::Subtree* aSubtree)
 
 /* boolean canDrop (in long index, in long orientation); */
 NS_IMETHODIMP
-nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation,
+nsXULTreeBuilder::CanDrop(int32_t index, int32_t orientation,
                           nsIDOMDataTransfer* dataTransfer, bool *_retval)
 {
     *_retval = false;
-    if (mObservers) {
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer) {
-                observer->CanDrop(index, orientation, dataTransfer, _retval);
-                if (*_retval)
-                    break;
-            }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer) {
+            observer->CanDrop(index, orientation, dataTransfer, _retval);
+            if (*_retval)
+                break;
         }
     }
 
@@ -1934,19 +1893,16 @@ nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation,
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient, nsIDOMDataTransfer* dataTransfer)
+nsXULTreeBuilder::Drop(int32_t row, int32_t orient, nsIDOMDataTransfer* dataTransfer)
 {
-    if (mObservers) {
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
-            if (observer) {
-                bool canDrop = false;
-                observer->CanDrop(row, orient, dataTransfer, &canDrop);
-                if (canDrop)
-                    observer->OnDrop(row, orient, dataTransfer);
-            }
+    uint32_t count = mObservers.Count();
+    for (uint32_t i = 0; i < count; ++i) {
+        nsCOMPtr<nsIXULTreeBuilderObserver> observer = mObservers.SafeObjectAt(i);
+        if (observer) {
+            bool canDrop = false;
+            observer->CanDrop(row, orient, dataTransfer, &canDrop);
+            if (canDrop)
+                observer->OnDrop(row, orient, dataTransfer);
         }
     }
 
@@ -1956,7 +1912,7 @@ nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient, nsIDOMDataTransfer* dataTran
 NS_IMETHODIMP
 nsXULTreeBuilder::IsSorted(bool *_retval)
 {
-  *_retval = (mSortVariable != nsnull);
+  *_retval = (mSortVariable != nullptr);
   return NS_OK;
 }
 
