@@ -8,7 +8,6 @@
 /* This must occur *after* layers/PLayers.h to avoid typedefs conflicts. */
 #include "mozilla/Util.h"
 
-#include "prtypes.h"
 #include "prmem.h"
 #include "prenv.h"
 #include "prclist.h"
@@ -19,9 +18,11 @@
 #include "nsNPAPIPlugin.h"
 #include "nsNPAPIPluginInstance.h"
 #include "nsNPAPIPluginStreamListener.h"
+#include "nsPluginStreamListenerPeer.h"
 #include "nsIServiceManager.h"
 #include "nsThreadUtils.h"
 #include "mozilla/Preferences.h"
+#include "nsPluginInstanceOwner.h"
 
 #include "nsPluginsDir.h"
 #include "nsPluginSafety.h"
@@ -30,7 +31,6 @@
 #include "nsIJSContextStack.h"
 
 #include "nsIDOMElement.h"
-#include "nsIDOMDocument.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDocument.h"
 #include "nsIContent.h"
@@ -55,11 +55,13 @@
 #endif
 
 // needed for nppdf plugin
-#ifdef MOZ_WIDGET_GTK2
+#if (MOZ_WIDGET_GTK)
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#if (MOZ_WIDGET_GTK == 2)
 #include "gtk2xtbin.h"
 #endif
+#endif
 
 #ifdef XP_OS2
 #define INCL_DOS
@@ -91,6 +93,10 @@ using mozilla::plugins::PluginModuleParent;
 
 #ifdef XP_WIN
 #include <windows.h>
+#include "nsWindowsHelpers.h"
+#ifdef ACCESSIBILITY
+#include "mozilla/a11y/Compatibility.h"
+#endif
 #endif
 
 #ifdef MOZ_WIDGET_ANDROID
@@ -98,6 +104,7 @@ using mozilla::plugins::PluginModuleParent;
 #include "android_npapi.h"
 #include "ANPBase.h"
 #include "AndroidBridge.h"
+#undef LOG
 #define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "GeckoPlugins" , ## args)
 #endif
 
@@ -168,7 +175,7 @@ static NPNetscapeFuncs sBrowserFuncs = {
   _setcurrentasyncsurface
 };
 
-static Mutex *sPluginThreadAsyncCallLock = nsnull;
+static Mutex *sPluginThreadAsyncCallLock = nullptr;
 static PRCList sPendingAsyncCalls = PR_INIT_STATIC_CLIST(&sPendingAsyncCalls);
 
 // POST/GET stream type
@@ -179,12 +186,20 @@ enum eNPPStreamTypeInternal {
 
 static NS_DEFINE_IID(kMemoryCID, NS_MEMORY_CID);
 
+PRIntervalTime NS_NotifyBeginPluginCall(NSPluginCallReentry aReentryState)
+{
+  nsNPAPIPluginInstance::BeginPluginCall(aReentryState);
+  return PR_IntervalNow();
+}
+
 // This function sends a notification using the observer service to any object
 // registered to listen to the "experimental-notify-plugin-call" subject.
 // Each "experimental-notify-plugin-call" notification carries with it the run
 // time value in milliseconds that the call took to execute.
-void NS_NotifyPluginCall(PRIntervalTime startTime) 
+void NS_NotifyPluginCall(PRIntervalTime startTime, NSPluginCallReentry aReentryState)
 {
+  nsNPAPIPluginInstance::EndPluginCall(aReentryState);
+
   PRIntervalTime endTime = PR_IntervalNow() - startTime;
   nsCOMPtr<nsIObserverService> notifyUIService =
     mozilla::services::GetObserverService();
@@ -195,7 +210,7 @@ void NS_NotifyPluginCall(PRIntervalTime startTime)
   nsAutoString runTimeString;
   runTimeString.AppendFloat(runTimeInSeconds);
   const PRUnichar* runTime = runTimeString.get();
-  notifyUIService->NotifyObservers(nsnull, "experimental-notify-plugin-call",
+  notifyUIService->NotifyObservers(nullptr, "experimental-notify-plugin-call",
                                    runTime);
 }
 
@@ -222,13 +237,13 @@ nsNPAPIPlugin::nsNPAPIPlugin()
   mPluginFuncs.size = sizeof(mPluginFuncs);
   mPluginFuncs.version = (NP_VERSION_MAJOR << 8) | NP_VERSION_MINOR;
 
-  mLibrary = nsnull;
+  mLibrary = nullptr;
 }
 
 nsNPAPIPlugin::~nsNPAPIPlugin()
 {
   delete mLibrary;
-  mLibrary = nsnull;
+  mLibrary = nullptr;
 }
 
 void
@@ -239,60 +254,6 @@ nsNPAPIPlugin::PluginCrashed(const nsAString& pluginDumpID,
   host->PluginCrashed(this, pluginDumpID, browserDumpID);
 }
 
-#if defined(XP_MACOSX) && defined(__i386__)
-static PRInt32 OSXVersion()
-{
-  static PRInt32 gOSXVersion = 0x0;
-  if (gOSXVersion == 0x0) {
-    OSErr err = ::Gestalt(gestaltSystemVersion, (SInt32*)&gOSXVersion);
-    if (err != noErr) {
-      // This should probably be changed when our minimum version changes
-      NS_ERROR("Couldn't determine OS X version, assuming 10.5");
-      gOSXVersion = 0x00001050;
-    }
-  }
-  return gOSXVersion;
-}
-
-// Detects machines with Intel GMA9xx GPUs.
-// kCGLRendererIDMatchingMask and kCGLRendererIntel900ID are only defined in the 10.6 SDK.
-#define CGLRendererIDMatchingMask 0x00FE7F00
-#define CGLRendererIntel900ID 0x00024000
-static bool GMA9XXGraphics()
-{
-  bool hasIntelGMA9XX = false;
-  CGLRendererInfoObj renderer = 0;
-  GLint rendererCount = 0;
-  if (::CGLQueryRendererInfo(0xffffffff, &renderer, &rendererCount) == kCGLNoError) {
-    for (GLint c = 0; c < rendererCount; c++) {
-      GLint rendProp = 0;
-      if (::CGLDescribeRenderer(renderer, c, kCGLRPRendererID, &rendProp) == kCGLNoError) {
-        if ((rendProp & CGLRendererIDMatchingMask) == CGLRendererIntel900ID) {
-          hasIntelGMA9XX = true;
-          break;
-        }
-      }
-    }
-    ::CGLDestroyRendererInfo(renderer);
-  }
-  return hasIntelGMA9XX;
-}
-#endif
-
-#ifdef XP_WIN
-static bool
-IsVistaOrLater()
-{
-  OSVERSIONINFO info;
-
-  ZeroMemory(&info, sizeof(OSVERSIONINFO));
-  info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-  GetVersionEx(&info);
-
-  return info.dwMajorVersion >= 6;
-}
-#endif
-
 bool
 nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
 {
@@ -304,39 +265,25 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
     return false;
   }
 
+#ifdef ACCESSIBILITY
+  // Certain assistive technologies don't want oop Flash, thus we have a special
+  // pref for them to disable oop Flash (refer to bug 785047 for details).
+  bool useA11yPref = false;
+#ifdef XP_WIN
+  useA11yPref =  a11y::Compatibility::IsJAWS();
+#endif
+#endif
+
 #ifdef XP_WIN
   // On Windows Vista+, we force Flash to run in OOPP mode because Adobe
   // doesn't test Flash in-process and there are known stability bugs.
   if (aPluginTag->mIsFlashPlugin && IsVistaOrLater()) {
+#ifdef ACCESSIBILITY
+    if (!useA11yPref)
+      return true;
+#else
     return true;
-  }
 #endif
-
-#if defined(XP_MACOSX) && defined(__i386__)
-  // Only allow on Mac OS X 10.6 or higher.
-  if (OSXVersion() < 0x00001060) {
-    return false;
-  }
-  // Blacklist Flash 10.0 or lower since it may try to negotiate Carbon/Quickdraw
-  // which are not supported out of process. Also blacklist Flash 10.1 if this
-  // machine has an Intel GMA9XX GPU because Flash will negotiate Quickdraw graphics.
-  // Never blacklist Flash >= 10.2.
-  if (aPluginTag->mFileName.EqualsIgnoreCase("flash player.plugin")) {
-    // If the first '.' is before position 2 or the version 
-    // starts with 10.0 then we are dealing with Flash 10 or less.
-    if (aPluginTag->mVersion.FindChar('.') < 2) {
-      return false;
-    }
-    if (aPluginTag->mVersion.Length() >= 4) {
-      nsCString versionPrefix;
-      aPluginTag->mVersion.Left(versionPrefix, 4);
-      if (versionPrefix.EqualsASCII("10.0")) {
-        return false;
-      }
-      if (versionPrefix.EqualsASCII("10.1") && GMA9XXGraphics()) {
-        return false;
-      }
-    }
   }
 #endif
 
@@ -350,8 +297,8 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
   // of "dom.ipc.plugins.enabled"
   // The "filename.dll" part can contain shell wildcard pattern
 
-  nsCAutoString prefFile(aPluginTag->mFullPath.get());
-  PRInt32 slashPos = prefFile.RFindCharInSet("/\\");
+  nsAutoCString prefFile(aPluginTag->mFullPath.get());
+  int32_t slashPos = prefFile.RFindCharInSet("/\\");
   if (kNotFound == slashPos)
     return false;
   prefFile.Cut(0, slashPos + 1);
@@ -359,14 +306,19 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
 
 #ifdef XP_MACOSX
 #if defined(__i386__)
-  nsCAutoString prefGroupKey("dom.ipc.plugins.enabled.i386.");
+  nsAutoCString prefGroupKey("dom.ipc.plugins.enabled.i386.");
 #elif defined(__x86_64__)
-  nsCAutoString prefGroupKey("dom.ipc.plugins.enabled.x86_64.");
+  nsAutoCString prefGroupKey("dom.ipc.plugins.enabled.x86_64.");
 #elif defined(__ppc__)
-  nsCAutoString prefGroupKey("dom.ipc.plugins.enabled.ppc.");
+  nsAutoCString prefGroupKey("dom.ipc.plugins.enabled.ppc.");
 #endif
 #else
-  nsCAutoString prefGroupKey("dom.ipc.plugins.enabled.");
+  nsAutoCString prefGroupKey("dom.ipc.plugins.enabled.");
+#endif
+
+#ifdef ACCESSIBILITY
+  if (useA11yPref)
+    prefGroupKey.AssignLiteral("dom.ipc.plugins.enabled.a11y.");
 #endif
 
   // Java plugins include a number of different file names,
@@ -376,7 +328,7 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
     return false;
   }
 
-  PRUint32 prefCount;
+  uint32_t prefCount;
   char** prefNames;
   nsresult rv = prefs->GetChildList(prefGroupKey.get(),
                                     &prefCount, &prefNames);
@@ -385,8 +337,8 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
   bool prefSet = false;
 
   if (NS_SUCCEEDED(rv) && prefCount > 0) {
-    PRUint32 prefixLength = prefGroupKey.Length();
-    for (PRUint32 currentPref = 0; currentPref < prefCount; currentPref++) {
+    uint32_t prefixLength = prefGroupKey.Length();
+    for (uint32_t currentPref = 0; currentPref < prefCount; currentPref++) {
       // Get the mask
       const char* maskStart = prefNames[currentPref] + prefixLength;
       bool match = false;
@@ -423,6 +375,9 @@ nsNPAPIPlugin::RunPluginOOP(const nsPluginTag *aPluginTag)
     Preferences::GetBool("dom.ipc.plugins.enabled.ppc", false);
 #endif
 #else
+#ifdef ACCESSIBILITY
+    useA11yPref ? Preferences::GetBool("dom.ipc.plugins.enabled.a11y", false) :
+#endif
     Preferences::GetBool("dom.ipc.plugins.enabled", false);
 #endif
   }
@@ -434,7 +389,7 @@ inline PluginLibrary*
 GetNewPluginLibrary(nsPluginTag *aPluginTag)
 {
   if (!aPluginTag) {
-    return nsnull;
+    return nullptr;
   }
 
   if (nsNPAPIPlugin::RunPluginOOP(aPluginTag)) {
@@ -447,7 +402,7 @@ GetNewPluginLibrary(nsPluginTag *aPluginTag)
 nsresult
 nsNPAPIPlugin::CreatePlugin(nsPluginTag *aPluginTag, nsNPAPIPlugin** aResult)
 {
-  *aResult = nsnull;
+  *aResult = nullptr;
 
   if (!aPluginTag) {
     return NS_ERROR_FAILURE;
@@ -547,19 +502,20 @@ nsNPAPIPlugin::RetainStream(NPStream *pstream, nsISupports **aRetainedPeer)
   *aRetainedPeer = NULL;
 
   if (!pstream || !pstream->ndata)
-    return NPERR_INVALID_PARAM;
+    return NS_ERROR_NULL_POINTER;
 
   nsNPAPIStreamWrapper* streamWrapper = static_cast<nsNPAPIStreamWrapper*>(pstream->ndata);
   nsNPAPIPluginStreamListener* listener = streamWrapper->GetStreamListener();
   if (!listener) {
-    return NPERR_GENERIC_ERROR;
+    return NS_ERROR_NULL_POINTER;
   }
 
-  nsPluginStreamListenerPeer* peer = listener->GetStreamListenerPeer();
-  if (!peer)
-    return NPERR_GENERIC_ERROR;
+  nsIStreamListener* streamListener = listener->GetStreamListenerPeer();
+  if (!streamListener) {
+    return NS_ERROR_NULL_POINTER;
+  }
 
-  *aRetainedPeer = (nsISupports*) peer;
+  *aRetainedPeer = streamListener;
   NS_ADDREF(*aRetainedPeer);
   return NS_OK;
 }
@@ -570,8 +526,8 @@ NPError
 MakeNewNPAPIStreamInternal(NPP npp, const char *relativeURL, const char *target,
                           eNPPStreamTypeInternal type,
                           bool bDoNotify = false,
-                          void *notifyData = nsnull, uint32_t len = 0,
-                          const char *buf = nsnull, NPBool file = false)
+                          void *notifyData = nullptr, uint32_t len = 0,
+                          const char *buf = nullptr, NPBool file = false)
 {
   if (!npp)
     return NPERR_INVALID_INSTANCE_ERROR;
@@ -653,12 +609,12 @@ public:
 
   void Invalidate()
   {
-    mFunc = nsnull;
+    mFunc = nullptr;
   }
 
   bool IsValid()
   {
-    return (mFunc != nsnull);
+    return (mFunc != nullptr);
   }
 
 private:  
@@ -670,16 +626,15 @@ private:
 static nsIDocument *
 GetDocumentFromNPP(NPP npp)
 {
-  NS_ENSURE_TRUE(npp, nsnull);
+  NS_ENSURE_TRUE(npp, nullptr);
 
   nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance *)npp->ndata;
-  NS_ENSURE_TRUE(inst, nsnull);
+  NS_ENSURE_TRUE(inst, nullptr);
 
   PluginDestructionGuard guard(inst);
 
-  nsCOMPtr<nsIPluginInstanceOwner> owner;
-  inst->GetOwner(getter_AddRefs(owner));
-  NS_ENSURE_TRUE(owner, nsnull);
+  nsRefPtr<nsPluginInstanceOwner> owner = inst->GetOwner();
+  NS_ENSURE_TRUE(owner, nullptr);
 
   nsCOMPtr<nsIDocument> doc;
   owner->GetDocument(getter_AddRefs(doc));
@@ -691,10 +646,10 @@ static JSContext *
 GetJSContextFromDoc(nsIDocument *doc)
 {
   nsIScriptGlobalObject *sgo = doc->GetScriptGlobalObject();
-  NS_ENSURE_TRUE(sgo, nsnull);
+  NS_ENSURE_TRUE(sgo, nullptr);
 
   nsIScriptContext *scx = sgo->GetContext();
-  NS_ENSURE_TRUE(scx, nsnull);
+  NS_ENSURE_TRUE(scx, nullptr);
 
   return scx->GetNativeContext();
 }
@@ -703,11 +658,42 @@ static JSContext *
 GetJSContextFromNPP(NPP npp)
 {
   nsIDocument *doc = GetDocumentFromNPP(npp);
-  NS_ENSURE_TRUE(doc, nsnull);
+  NS_ENSURE_TRUE(doc, nullptr);
 
   return GetJSContextFromDoc(doc);
 }
 
+static nsresult
+GetPrivacyFromNPP(NPP npp, bool* aPrivate)
+{
+  nsCOMPtr<nsIDocument> doc = GetDocumentFromNPP(npp);
+  NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
+  nsCOMPtr<nsPIDOMWindow> domwindow = doc->GetWindow();
+  NS_ENSURE_TRUE(domwindow, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDocShell> docShell = domwindow->GetDocShell();
+  nsCOMPtr<nsILoadContext> loadContext = do_QueryInterface(docShell);
+  *aPrivate = loadContext && loadContext->UsePrivateBrowsing();
+  return NS_OK;
+}
+
+static already_AddRefed<nsIChannel>
+GetChannelFromNPP(NPP npp)
+{
+  nsCOMPtr<nsIDocument> doc = GetDocumentFromNPP(npp);
+  if (!doc)
+    return nullptr;
+  nsCOMPtr<nsPIDOMWindow> domwindow = doc->GetWindow();
+  nsCOMPtr<nsIChannel> channel;
+  if (domwindow) {
+    nsCOMPtr<nsIDocShell> docShell = domwindow->GetDocShell();
+    if (docShell) {
+      docShell->GetCurrentDocumentChannel(getter_AddRefs(channel));
+    }
+  }
+  return channel.forget();
+}
+
 static NPIdentifier
 doGetIdentifier(JSContext *cx, const NPUTF8* name)
 {
@@ -745,7 +731,7 @@ InHeap(HANDLE hHeap, LPVOID lpMem)
 NPPExceptionAutoHolder::NPPExceptionAutoHolder()
   : mOldException(gNPPException)
 {
-  gNPPException = nsnull;
+  gNPPException = nullptr;
 }
 
 NPPExceptionAutoHolder::~NPPExceptionAutoHolder()
@@ -762,7 +748,7 @@ nsPluginThreadRunnable::nsPluginThreadRunnable(NPP instance,
 {
   if (!sPluginThreadAsyncCallLock) {
     // Failed to create lock, not much we can do here then...
-    mFunc = nsnull;
+    mFunc = nullptr;
 
     return;
   }
@@ -775,7 +761,7 @@ nsPluginThreadRunnable::nsPluginThreadRunnable(NPP instance,
     nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance *)instance->ndata;
     if (!inst || !inst->IsRunning()) {
       // The plugin was stopped, ignore this async call.
-      mFunc = nsnull;
+      mFunc = nullptr;
 
       return;
     }
@@ -803,7 +789,8 @@ nsPluginThreadRunnable::Run()
   if (mFunc) {
     PluginDestructionGuard guard(mInstance);
 
-    NS_TRY_SAFE_CALL_VOID(mFunc(mUserData), nsnull);
+    NS_TRY_SAFE_CALL_VOID(mFunc(mUserData), nullptr,
+                          NS_PLUGIN_CALL_UNSAFE_TO_REENTER_GECKO);
   }
 
   return NS_OK;
@@ -845,7 +832,7 @@ OnShutdown()
   if (sPluginThreadAsyncCallLock) {
     delete sPluginThreadAsyncCallLock;
 
-    sPluginThreadAsyncCallLock = nsnull;
+    sPluginThreadAsyncCallLock = nullptr;
   }
 }
 
@@ -864,7 +851,7 @@ AsyncCallbackAutoLock::~AsyncCallbackAutoLock()
 }
 
 
-NPP NPPStack::sCurrentNPP = nsnull;
+NPP NPPStack::sCurrentNPP = nullptr;
 
 const char *
 PeekException()
@@ -880,7 +867,7 @@ PopException()
   if (gNPPException) {
     free(gNPPException);
 
-    gNPPException = nsnull;
+    gNPPException = nullptr;
   }
 }
 
@@ -915,7 +902,7 @@ _geturl(NPP npp, const char* relativeURL, const char* target)
     nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance *) npp->ndata;
 
     
-    const char *name = nsnull;
+    const char *name = nullptr;
     nsRefPtr<nsPluginHost> host = dont_AddRef(nsPluginHost::GetInst());
     host->GetPluginName(inst, &name);
 
@@ -988,7 +975,7 @@ _posturl(NPP npp, const char *relativeURL, const char *target,
   PluginDestructionGuard guard(npp);
 
   return MakeNewNPAPIStreamInternal(npp, relativeURL, target,
-                                    eNPPStreamTypeInternal_Post, false, nsnull,
+                                    eNPPStreamTypeInternal_Post, false, nullptr,
                                     len, buf, file);
 }
 
@@ -1012,7 +999,7 @@ _newstream(NPP npp, NPMIMEType type, const char* target, NPStream* *result)
     nsCOMPtr<nsIOutputStream> stream;
     if (NS_SUCCEEDED(inst->NewStreamFromPlugin((const char*) type, target,
                                                getter_AddRefs(stream)))) {
-      nsNPAPIStreamWrapper* wrapper = new nsNPAPIStreamWrapper(stream, nsnull);
+      nsNPAPIStreamWrapper* wrapper = new nsNPAPIStreamWrapper(stream, nullptr);
       if (wrapper) {
         (*result) = &wrapper->mNPStream;
         err = NPERR_NO_ERROR;
@@ -1053,7 +1040,7 @@ _write(NPP npp, NPStream *pstream, int32_t len, void *buffer)
     return -1;
   }
 
-  PRUint32 count = 0;
+  uint32_t count = 0;
   nsresult rv = stream->Write((char *)buffer, len, &count);
 
   if (NS_FAILED(rv)) {
@@ -1092,7 +1079,7 @@ _destroystream(NPP npp, NPStream *pstream, NPError reason)
     //
     // Calling OnStopBinding on the listener may cause it to be deleted due to the
     // releasing of its last references.
-    listener->OnStopBinding(nsnull, NS_BINDING_ABORTED);
+    listener->OnStopBinding(nullptr, NS_BINDING_ABORTED);
   } else {
     // This type of stream (NPStream) was created via NPN_NewStream. The plugin holds
     // the reference until it is to be deleted here. Deleting the wrapper will
@@ -1233,10 +1220,10 @@ _getwindowobject(NPP npp)
 {
   if (!NS_IsMainThread()) {
     NPN_PLUGIN_LOG(PLUGIN_LOG_ALWAYS,("NPN_getwindowobject called from the wrong thread\n"));
-    return nsnull;
+    return nullptr;
   }
   JSContext *cx = GetJSContextFromNPP(npp);
-  NS_ENSURE_TRUE(cx, nsnull);
+  NS_ENSURE_TRUE(cx, nullptr);
 
   // Using ::JS_GetGlobalObject(cx) is ok here since the window we
   // want to return here is the outer window, *not* the inner (since
@@ -1249,34 +1236,34 @@ _getpluginelement(NPP npp)
 {
   if (!NS_IsMainThread()) {
     NPN_PLUGIN_LOG(PLUGIN_LOG_ALWAYS,("NPN_getpluginelement called from the wrong thread\n"));
-    return nsnull;
+    return nullptr;
   }
 
   nsNPAPIPluginInstance* inst = static_cast<nsNPAPIPluginInstance*>(npp->ndata);
   if (!inst)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr<nsIDOMElement> element;
   inst->GetDOMElement(getter_AddRefs(element));
 
   if (!element)
-    return nsnull;
+    return nullptr;
 
   JSContext *cx = GetJSContextFromNPP(npp);
-  NS_ENSURE_TRUE(cx, nsnull);
+  NS_ENSURE_TRUE(cx, nullptr);
 
   nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID()));
-  NS_ENSURE_TRUE(xpc, nsnull);
+  NS_ENSURE_TRUE(xpc, nullptr);
 
   nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
   xpc->WrapNative(cx, ::JS_GetGlobalObject(cx), element,
                   NS_GET_IID(nsIDOMElement),
                   getter_AddRefs(holder));
-  NS_ENSURE_TRUE(holder, nsnull);
+  NS_ENSURE_TRUE(holder, nullptr);
 
-  JSObject* obj = nsnull;
+  JSObject* obj = nullptr;
   holder->GetJSObject(&obj);
-  NS_ENSURE_TRUE(obj, nsnull);
+  NS_ENSURE_TRUE(obj, nullptr);
 
   return nsJSObjWrapper::GetNewOrUsed(npp, cx, obj);
 }
@@ -1354,7 +1341,7 @@ _utf8fromidentifier(NPIdentifier id)
     return NULL;
 
   if (!NPIdentifierIsString(id)) {
-    return nsnull;
+    return nullptr;
   }
 
   JSString *str = NPIdentifierToString(id);
@@ -1372,7 +1359,7 @@ _intfromidentifier(NPIdentifier id)
   }
 
   if (!NPIdentifierIsInt(id)) {
-    return PR_INT32_MIN;
+    return INT32_MIN;
   }
 
   return NPIdentifierToInt(id);
@@ -1393,12 +1380,12 @@ _createobject(NPP npp, NPClass* aClass)
 {
   if (!NS_IsMainThread()) {
     NPN_PLUGIN_LOG(PLUGIN_LOG_ALWAYS,("NPN_createobject called from the wrong thread\n"));
-    return nsnull;
+    return nullptr;
   }
   if (!npp) {
     NS_ERROR("Null npp passed to _createobject()!");
 
-    return nsnull;
+    return nullptr;
   }
 
   PluginDestructionGuard guard(npp);
@@ -1406,7 +1393,7 @@ _createobject(NPP npp, NPClass* aClass)
   if (!aClass) {
     NS_ERROR("Null class passed to _createobject()!");
 
-    return nsnull;
+    return nullptr;
   }
 
   NPPAutoPusher nppPusher(npp);
@@ -1441,7 +1428,7 @@ _retainobject(NPObject* npobj)
 #ifdef NS_BUILD_REFCNT_LOGGING
     int32_t refCnt =
 #endif
-      PR_ATOMIC_INCREMENT((PRInt32*)&npobj->referenceCount);
+      PR_ATOMIC_INCREMENT((int32_t*)&npobj->referenceCount);
     NS_LOG_ADDREF(npobj, refCnt, "BrowserNPObject", sizeof(NPObject));
   }
 
@@ -1457,7 +1444,7 @@ _releaseobject(NPObject* npobj)
   if (!npobj)
     return;
 
-  int32_t refCnt = PR_ATOMIC_DECREMENT((PRInt32*)&npobj->referenceCount);
+  int32_t refCnt = PR_ATOMIC_DECREMENT((int32_t*)&npobj->referenceCount);
   NS_LOG_RELEASE(npobj, refCnt, "BrowserNPObject");
 
   if (refCnt == 0) {
@@ -1573,7 +1560,7 @@ _evaluate(NPP npp, NPObject* npobj, NPString *script, NPVariant *result)
 
   nsIPrincipal *principal = doc->NodePrincipal();
 
-  nsCAutoString specStr;
+  nsAutoCString specStr;
   const char *spec;
 
   nsCOMPtr<nsIURI> uri;
@@ -1598,7 +1585,7 @@ _evaluate(NPP npp, NPObject* npobj, NPString *script, NPVariant *result)
       uri->GetSpec(specStr);
       spec = specStr.get();
     } else {
-      spec = nsnull;
+      spec = nullptr;
     }
   }
 
@@ -1607,7 +1594,7 @@ _evaluate(NPP npp, NPObject* npobj, NPString *script, NPVariant *result)
                   npp, npobj, script->UTF8Characters));
 
   nsresult rv = scx->EvaluateStringWithValue(utf16script, obj, principal,
-                                             spec, 0, 0, rval, nsnull);
+                                             spec, 0, 0, false, rval, nullptr);
 
   return NS_SUCCEEDED(rv) &&
          (!result || JSValToNPVariant(npp, cx, *rval, result));
@@ -1670,7 +1657,7 @@ _getproperty(NPP npp, NPObject* npobj, NPIdentifier property,
     return true;
 
   NPVariant doc_v;
-  NPObject* document_obj = nsnull;
+  NPObject* document_obj = nullptr;
   NPIdentifier doc_id = _getstringidentifier("document");
   bool ok = npobj->_class->getProperty(window_obj, doc_id, &doc_v);
   _releaseobject(window_obj);
@@ -1994,9 +1981,8 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
 
     nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance *) npp->ndata;
 
-    nsCOMPtr<nsIPluginInstanceOwner> owner;
-    inst->GetOwner(getter_AddRefs(owner));
-    NS_ENSURE_TRUE(owner, nsnull);
+    nsRefPtr<nsPluginInstanceOwner> owner = inst->GetOwner();
+    NS_ENSURE_TRUE(owner, NPERR_NO_ERROR);
 
     if (NS_SUCCEEDED(owner->GetNetscapeWindow(result))) {
       return NPERR_NO_ERROR;
@@ -2033,7 +2019,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
   }
 
   case NPNVToolkit: {
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
     *((NPNToolkitType*)result) = NPNVGtk2;
 #endif
 
@@ -2048,7 +2034,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
   }
 
   case NPNVSupportsXEmbedBool: {
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
     *(NPBool*)result = true;
 #elif defined(MOZ_WIDGET_QT)
     // Desktop Flash fail to initialize if browser does not support NPNVSupportsXEmbedBool
@@ -2075,7 +2061,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
 
   case NPNVSupportsWindowless: {
 #if defined(XP_WIN) || defined(XP_MACOSX) || \
-    (defined(MOZ_X11) && (defined(MOZ_WIDGET_GTK2) || defined(MOZ_WIDGET_QT)))
+    (defined(MOZ_X11) && (defined(MOZ_WIDGET_GTK) || defined(MOZ_WIDGET_QT)))
     *(NPBool*)result = true;
 #else
     *(NPBool*)result = false;
@@ -2084,16 +2070,12 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
   }
 
   case NPNVprivateModeBool: {
-    nsCOMPtr<nsIDocument> doc = GetDocumentFromNPP(npp);
-    NS_ENSURE_TRUE(doc, NPERR_GENERIC_ERROR);
-    nsCOMPtr<nsPIDOMWindow> domwindow = doc->GetWindow();
-    if (domwindow) {
-      nsCOMPtr<nsIDocShell> docShell = domwindow->GetDocShell();
-      nsCOMPtr<nsILoadContext> loadContext = do_QueryInterface(docShell);
-      *(NPBool*)result = (NPBool)(loadContext && loadContext->UsePrivateBrowsing());
-      return NPERR_NO_ERROR;
-    }
-    return NPERR_GENERIC_ERROR;
+    bool privacy;
+    nsresult rv = GetPrivacyFromNPP(npp, &privacy);
+    if (NS_FAILED(rv))
+      return NPERR_GENERIC_ERROR;
+    *(NPBool*)result = (NPBool)privacy;
+    return NPERR_NO_ERROR;
   }
 
   case NPNVdocumentOrigin: {
@@ -2142,7 +2124,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
       nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance*)npp->ndata;
       if (inst) {
         NPDrawingModel drawingModel;
-        inst->GetDrawingModel((PRInt32*)&drawingModel);
+        inst->GetDrawingModel((int32_t*)&drawingModel);
         *(NPDrawingModel*)result = drawingModel;
         return NPERR_NO_ERROR;
       }
@@ -2154,7 +2136,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
 
 #ifndef NP_NO_QUICKDRAW
   case NPNVsupportsQuickDrawBool: {
-    *(NPBool*)result = true;
+    *(NPBool*)result = false;
     
     return NPERR_NO_ERROR;
   }
@@ -2181,7 +2163,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
 
 #ifndef NP_NO_CARBON
   case NPNVsupportsCarbonBool: {
-    *(NPBool*)result = true;
+    *(NPBool*)result = false;
 
     return NPERR_NO_ERROR;
   }
@@ -2196,6 +2178,14 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
     *(NPBool*)result = true;
     return NPERR_NO_ERROR;
   }
+
+  case NPNVcontentsScaleFactor: {
+    nsNPAPIPluginInstance *inst =
+      (nsNPAPIPluginInstance *) (npp ? npp->ndata : nullptr);
+    double scaleFactor = inst ? inst->GetContentsScaleFactor() : 1.0;
+    *(double*)result = scaleFactor;
+    return NPERR_NO_ERROR;
+  }
 #endif
 
 #ifdef MOZ_WIDGET_ANDROID
@@ -2300,6 +2290,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
       jfieldID field = env->GetStaticFieldID(cls, "mAppContext",
                                              "Lorg/mozilla/gecko/GeckoApp;");
       jobject ret = env->GetStaticObjectField(cls, field);
+      env->DeleteLocalRef(cls);
       int32_t* i  = reinterpret_cast<int32_t*>(result);
       *i = reinterpret_cast<int32_t>(ret);
       return NPERR_NO_ERROR;
@@ -2379,7 +2370,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
   case NPNVserviceManager:
     // old XPCOM objects, no longer supported, but null out the out
     // param to avoid crashing plugins that still try to use this.
-    *(nsISupports**)result = nsnull;
+    *(nsISupports**)result = nullptr;
     // fall through
   default:
     NPN_PLUGIN_LOG(PLUGIN_LOG_NORMAL, ("NPN_getvalue unhandled get value: %d\n", variable));
@@ -2422,12 +2413,12 @@ _setvalue(NPP npp, NPPVariable variable, void *result)
       // (e.g. Safari and Opera).
       return NPERR_NO_ERROR;
 #else
-      NPBool bWindowless = (result == nsnull);
+      NPBool bWindowless = (result == nullptr);
       return inst->SetWindowless(bWindowless);
 #endif
     }
     case NPPVpluginTransparentBool: {
-      NPBool bTransparent = (result != nsnull);
+      NPBool bTransparent = (result != nullptr);
       return inst->SetTransparent(bTransparent);
     }
 
@@ -2437,26 +2428,27 @@ _setvalue(NPP npp, NPPVariable variable, void *result)
         nsCOMPtr<nsIJSContextStack> contextStack =
           do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
         if (NS_SUCCEEDED(rv)) {
-          NPBool bPushCaller = (result != nsnull);
+          NPBool bPushCaller = (result != nullptr);
           if (bPushCaller) {
             JSContext *cx;
             rv = inst->GetJSContext(&cx);
             if (NS_SUCCEEDED(rv))
               rv = contextStack->Push(cx);
           } else {
-            rv = contextStack->Pop(nsnull);
+            rv = contextStack->Pop(nullptr);
           }
         }
         return NS_SUCCEEDED(rv) ? NPERR_NO_ERROR : NPERR_GENERIC_ERROR;
       }
 
     case NPPVpluginKeepLibraryInMemory: {
-      NPBool bCached = (result != nsnull);
-      return inst->SetCached(bCached);
+      NPBool bCached = (result != nullptr);
+      inst->SetCached(bCached);
+      return NPERR_NO_ERROR;
     }
 
     case NPPVpluginUsesDOMForCursorBool: {
-      bool useDOMForCursor = (result != nsnull);
+      bool useDOMForCursor = (result != nullptr);
       return inst->SetUsesDOMForCursor(useDOMForCursor);
     }
 
@@ -2508,7 +2500,7 @@ _requestread(NPStream *pstream, NPByteRange *rangeList)
                                      (void*)pstream));
 
 #ifdef PLUGIN_LOGGING
-  for(NPByteRange * range = rangeList; range != nsnull; range = range->next)
+  for(NPByteRange * range = rangeList; range != nullptr; range = range->next)
     PR_LOG(nsPluginLogging::gNPNLog,PLUGIN_LOG_NOISY,
     ("%i-%i", range->offset, range->offset + range->length - 1));
 
@@ -2525,21 +2517,21 @@ _requestread(NPStream *pstream, NPByteRange *rangeList)
     return NPERR_GENERIC_ERROR;
   }
 
-  PRInt32 streamtype = NP_NORMAL;
+  int32_t streamtype = NP_NORMAL;
 
   streamlistener->GetStreamType(&streamtype);
 
   if (streamtype != NP_SEEK)
     return NPERR_STREAM_NOT_SEEKABLE;
 
-  if (!streamlistener->mStreamInfo)
+  if (!streamlistener->mStreamListenerPeer)
     return NPERR_GENERIC_ERROR;
 
-  nsresult rv = streamlistener->mStreamInfo->RequestRead((NPByteRange *)rangeList);
+  nsresult rv = streamlistener->mStreamListenerPeer->RequestRead((NPByteRange *)rangeList);
   if (NS_FAILED(rv))
     return NPERR_GENERIC_ERROR;
 
-  return NS_OK;
+  return NPERR_NO_ERROR;
 }
 
 // Deprecated, only stubbed out
@@ -2555,20 +2547,20 @@ _useragent(NPP npp)
 {
   if (!NS_IsMainThread()) {
     NPN_PLUGIN_LOG(PLUGIN_LOG_ALWAYS,("NPN_useragent called from the wrong thread\n"));
-    return nsnull;
+    return nullptr;
   }
   NPN_PLUGIN_LOG(PLUGIN_LOG_NORMAL, ("NPN_UserAgent: npp=%p\n", (void*)npp));
 
   nsCOMPtr<nsIPluginHost> pluginHostCOM(do_GetService(MOZ_PLUGIN_HOST_CONTRACTID));
   nsPluginHost *pluginHost = static_cast<nsPluginHost*>(pluginHostCOM.get());
   if (!pluginHost) {
-    return nsnull;
+    return nullptr;
   }
 
   const char *retstr;
   nsresult rv = pluginHost->UserAgent(&retstr);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   return retstr;
 }
@@ -2674,7 +2666,9 @@ _getvalueforurl(NPP instance, NPNURLVariable variable, const char *url,
         return NPERR_GENERIC_ERROR;
       }
 
-      if (NS_FAILED(cookieService->GetCookieString(uri, nsnull, value)) ||
+      nsCOMPtr<nsIChannel> channel = GetChannelFromNPP(instance);
+
+      if (NS_FAILED(cookieService->GetCookieString(uri, channel, value)) ||
           !*value) {
         return NPERR_GENERIC_ERROR;
       }
@@ -2720,17 +2714,19 @@ _setvalueforurl(NPP instance, NPNURLVariable variable, const char *url,
         return NPERR_GENERIC_ERROR;
 
       nsCOMPtr<nsIURI> uriIn;
-      rv = ioService->NewURI(nsDependentCString(url), nsnull, nsnull, getter_AddRefs(uriIn));
+      rv = ioService->NewURI(nsDependentCString(url), nullptr, nullptr, getter_AddRefs(uriIn));
       if (NS_FAILED(rv))
         return NPERR_GENERIC_ERROR;
 
       nsCOMPtr<nsIPrompt> prompt;
-      nsPluginHost::GetPrompt(nsnull, getter_AddRefs(prompt));
+      nsPluginHost::GetPrompt(nullptr, getter_AddRefs(prompt));
+
+      nsCOMPtr<nsIChannel> channel = GetChannelFromNPP(instance);
 
       char *cookie = (char*)value;
       char c = cookie[len];
       cookie[len] = '\0';
-      rv = cookieService->SetCookieString(uriIn, prompt, cookie, nsnull);
+      rv = cookieService->SetCookieString(uriIn, prompt, cookie, channel);
       cookie[len] = c;
       if (NS_SUCCEEDED(rv))
         return NPERR_NO_ERROR;
@@ -2757,8 +2753,8 @@ _getauthenticationinfo(NPP instance, const char *protocol, const char *host,
       !ulen || !password || !plen)
     return NPERR_INVALID_PARAM;
 
-  *username = nsnull;
-  *password = nsnull;
+  *username = nullptr;
+  *password = nullptr;
   *ulen = 0;
   *plen = 0;
 
@@ -2773,12 +2769,19 @@ _getauthenticationinfo(NPP instance, const char *protocol, const char *host,
   if (!authManager)
     return NPERR_GENERIC_ERROR;
 
+  bool authPrivate = false;
+  GetPrivacyFromNPP(instance, &authPrivate);
+
+  nsIDocument *doc = GetDocumentFromNPP(instance);
+  NS_ENSURE_TRUE(doc, NPERR_GENERIC_ERROR);
+  nsIPrincipal *principal = doc->NodePrincipal();
+
   nsAutoString unused, uname16, pwd16;
   if (NS_FAILED(authManager->GetAuthIdentity(proto, nsDependentCString(host),
                                              port, nsDependentCString(scheme),
                                              nsDependentCString(realm),
                                              EmptyCString(), unused, uname16,
-                                             pwd16))) {
+                                             pwd16, authPrivate, principal))) {
     return NPERR_GENERIC_ERROR;
   }
 
@@ -2807,7 +2810,13 @@ _scheduletimer(NPP instance, uint32_t interval, NPBool repeat, PluginTimerFunc t
 void NP_CALLBACK
 _unscheduletimer(NPP instance, uint32_t timerID)
 {
+#ifdef MOZ_WIDGET_ANDROID
+  // Sometimes Flash calls this with a dead NPP instance. Ensure the one we have
+  // here is valid and maps to a nsNPAPIPluginInstance.
+  nsNPAPIPluginInstance *inst = nsNPAPIPluginInstance::GetFromNPP(instance);
+#else
   nsNPAPIPluginInstance *inst = (nsNPAPIPluginInstance *)instance->ndata;
+#endif
   if (!inst)
     return;
 
