@@ -3,32 +3,49 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsHTMLObjectResizer.h"
-
-#include "nsIDOMEventTarget.h"
-#include "nsIDOMText.h"
-
-#include "nsIDOMCSSValue.h"
-#include "nsIDOMCSSPrimitiveValue.h"
-
-#include "nsIContent.h"
-#include "nsIDocument.h"
-#include "nsIDocumentObserver.h"
-#include "nsIEditor.h"
-#include "nsIPresShell.h"
-#include "nsPIDOMWindow.h"
-
-#include "nsHTMLEditor.h"
-#include "nsEditor.h"
-#include "nsEditorUtils.h"
-#include "nsHTMLEditUtils.h"
-
-#include "nsPoint.h"
-
-#include "nsIServiceManager.h"
-#include "mozilla/Preferences.h"
-
 #include "mozilla/LookAndFeel.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/mozalloc.h"
+#include "nsAString.h"
+#include "nsAlgorithm.h"
+#include "nsAutoPtr.h"
+#include "nsCOMArray.h"
+#include "nsCOMPtr.h"
+#include "nsDebug.h"
+#include "nsEditProperty.h"
+#include "nsEditorUtils.h"
+#include "nsError.h"
+#include "nsHTMLCSSUtils.h"
+#include "nsHTMLEditUtils.h"
+#include "nsHTMLEditor.h"
+#include "nsHTMLObjectResizer.h"
+#include "nsIAtom.h"
+#include "nsIContent.h"
+#include "nsID.h"
+#include "nsIDOMDocument.h"
+#include "nsIDOMElement.h"
+#include "nsIDOMEvent.h"
+#include "nsIDOMEventTarget.h"
+#include "nsIDOMMouseEvent.h"
+#include "nsIDOMNode.h"
+#include "nsIDOMText.h"
+#include "nsIDocument.h"
+#include "nsIEditor.h"
+#include "nsIHTMLEditor.h"
+#include "nsIHTMLObjectResizeListener.h"
+#include "nsIHTMLObjectResizer.h"
+#include "nsIPresShell.h"
+#include "nsISupportsUtils.h"
+#include "nsPIDOMWindow.h"
+#include "nsReadableUtils.h"
+#include "nsString.h"
+#include "nsStringFwd.h"
+#include "nsSubstringTuple.h"
+#include "nscore.h"
+#include <cstdlib> // for std::abs(int/long)
+#include <cmath> // for std::abs(float/double)
+
+class nsISelection;
 
 using namespace mozilla;
 
@@ -73,7 +90,7 @@ ResizerSelectionListener::~ResizerSelectionListener()
 }
 
 NS_IMETHODIMP
-ResizerSelectionListener::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aSelection, PRInt16 aReason)
+ResizerSelectionListener::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aSelection, int16_t aReason)
 {
   if ((aReason & (nsISelectionListener::MOUSEDOWN_REASON |
                   nsISelectionListener::KEYPRESS_REASON |
@@ -129,7 +146,7 @@ ResizerMouseMotionListener::HandleEvent(nsIDOMEvent* aMouseEvent)
 // ==================================================================
 
 nsresult
-nsHTMLEditor::CreateResizer(nsIDOMElement ** aReturn, PRInt16 aLocation, nsIDOMNode * aParentNode)
+nsHTMLEditor::CreateResizer(nsIDOMElement ** aReturn, int16_t aLocation, nsIDOMNode * aParentNode)
 {
   nsresult res = CreateAnonymousElement(NS_LITERAL_STRING("span"),
                                         aParentNode,
@@ -221,10 +238,10 @@ nsHTMLEditor::SetAllResizersPosition()
 {
   NS_ENSURE_TRUE(mTopLeftHandle, NS_ERROR_FAILURE);
 
-  PRInt32 x = mResizedObjectX;
-  PRInt32 y = mResizedObjectY;
-  PRInt32 w = mResizedObjectWidth;
-  PRInt32 h = mResizedObjectHeight;
+  int32_t x = mResizedObjectX;
+  int32_t y = mResizedObjectY;
+  int32_t w = mResizedObjectWidth;
+  int32_t h = mResizedObjectHeight;
 
   // now let's place all the resizers around the image
 
@@ -237,8 +254,8 @@ nsHTMLEditor::SetAllResizersPosition()
   mHTMLCSSUtils->GetComputedProperty(mTopLeftHandle, nsEditProperty::cssHeight, value);
   mHTMLCSSUtils->ParseLength(value, &resizerHeight, getter_AddRefs(dummyUnit));
 
-  PRInt32 rw  = (PRInt32)((resizerWidth + 1) / 2);
-  PRInt32 rh =  (PRInt32)((resizerHeight+ 1) / 2);
+  int32_t rw  = (int32_t)((resizerWidth + 1) / 2);
+  int32_t rh =  (int32_t)((resizerHeight+ 1) / 2);
 
   SetAnonymousElementPosition(x-rw,     y-rh, mTopLeftHandle);
   SetAnonymousElementPosition(x+w/2-rw, y-rh, mTopHandle);
@@ -399,47 +416,47 @@ nsHTMLEditor::HideResizers(void)
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mTopLeftHandle, parentContent, ps);
-  mTopLeftHandle = nsnull;
+  mTopLeftHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mTopHandle, parentContent, ps);
-  mTopHandle = nsnull;
+  mTopHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mTopRightHandle, parentContent, ps);
-  mTopRightHandle = nsnull;
+  mTopRightHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mLeftHandle, parentContent, ps);
-  mLeftHandle = nsnull;
+  mLeftHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mRightHandle, parentContent, ps);
-  mRightHandle = nsnull;
+  mRightHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mBottomLeftHandle, parentContent, ps);
-  mBottomLeftHandle = nsnull;
+  mBottomLeftHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mBottomHandle, parentContent, ps);
-  mBottomHandle = nsnull;
+  mBottomHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mBottomRightHandle, parentContent, ps);
-  mBottomRightHandle = nsnull;
+  mBottomRightHandle = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mResizingShadow, parentContent, ps);
-  mResizingShadow = nsnull;
+  mResizingShadow = nullptr;
 
   RemoveListenerAndDeleteRef(mousedown, mEventListener, true,
                              mResizingInfo, parentContent, ps);
-  mResizingInfo = nsnull;
+  mResizingInfo = nullptr;
 
   if (mActivatedHandle) {
     mActivatedHandle->RemoveAttribute(NS_LITERAL_STRING("_moz_activated"));
-    mActivatedHandle = nsnull;
+    mActivatedHandle = nullptr;
   }
 
   // don't forget to remove the listeners !
@@ -452,7 +469,7 @@ nsHTMLEditor::HideResizers(void)
                                       mMouseMotionListenerP, true);
     NS_ASSERTION(NS_SUCCEEDED(res), "failed to remove mouse motion listener");
   }
-  mMouseMotionListenerP = nsnull;
+  mMouseMotionListenerP = nullptr;
 
   nsCOMPtr<nsIDocument> doc = GetDocument();
   if (!doc) { return NS_ERROR_NULL_POINTER; }
@@ -463,10 +480,10 @@ nsHTMLEditor::HideResizers(void)
     res = target->RemoveEventListener(NS_LITERAL_STRING("resize"), mResizeEventListenerP, false);
     NS_ASSERTION(NS_SUCCEEDED(res), "failed to remove resize event listener");
   }
-  mResizeEventListenerP = nsnull;
+  mResizeEventListenerP = nullptr;
 
   mResizedObject->RemoveAttribute(NS_LITERAL_STRING("_moz_resizing"));
-  mResizedObject = nsnull;
+  mResizedObject = nullptr;
 
   return NS_OK;
 }
@@ -484,10 +501,10 @@ nsresult
 nsHTMLEditor::StartResizing(nsIDOMElement *aHandle)
 {
   // First notify the listeners if any
-  PRInt32 listenersCount = objectResizeEventListeners.Count();
+  int32_t listenersCount = objectResizeEventListeners.Count();
   if (listenersCount) {
     nsCOMPtr<nsIHTMLObjectResizeListener> listener;
-    PRInt32 index;
+    int32_t index;
     for (index = 0; index < listenersCount; index++) {
       listener = objectResizeEventListeners[index];
       listener->OnStartResizing(mResizedObject);
@@ -563,7 +580,7 @@ nsHTMLEditor::StartResizing(nsIDOMElement *aHandle)
 
 
 NS_IMETHODIMP 
-nsHTMLEditor::MouseDown(PRInt32 aClientX, PRInt32 aClientY,
+nsHTMLEditor::MouseDown(int32_t aClientX, int32_t aClientY,
                         nsIDOMElement *aTarget, nsIDOMEvent* aEvent)
 {
   bool anonElement = false;
@@ -592,7 +609,7 @@ nsHTMLEditor::MouseDown(PRInt32 aClientX, PRInt32 aClientY,
 }
 
 NS_IMETHODIMP 
-nsHTMLEditor::MouseUp(PRInt32 aClientX, PRInt32 aClientY,
+nsHTMLEditor::MouseUp(int32_t aClientX, int32_t aClientY,
                       nsIDOMElement *aTarget)
 {
   if (mIsResizing) {
@@ -616,8 +633,8 @@ nsHTMLEditor::MouseUp(PRInt32 aClientX, PRInt32 aClientY,
 
 
 void
-nsHTMLEditor::SetResizeIncrements(PRInt32 aX, PRInt32 aY,
-                                  PRInt32 aW, PRInt32 aH,
+nsHTMLEditor::SetResizeIncrements(int32_t aX, int32_t aY,
+                                  int32_t aW, int32_t aH,
                                   bool aPreserveRatio)
 {
   mXIncrementFactor = aX;
@@ -628,7 +645,7 @@ nsHTMLEditor::SetResizeIncrements(PRInt32 aX, PRInt32 aY,
 }
 
 nsresult
-nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt32 aH)
+nsHTMLEditor::SetResizingInfoPosition(int32_t aX, int32_t aY, int32_t aW, int32_t aH)
 {
   nsCOMPtr<nsIDOMDocument> domdoc = GetDOMDocument();
 
@@ -640,8 +657,8 @@ nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt3
   // resizer is the "activated handle".  For example, place the resizing
   // info box at the bottom-right corner of the new element, if the element
   // is being resized by the bottom-right resizer.
-  PRInt32 infoXPosition;
-  PRInt32 infoYPosition;
+  int32_t infoXPosition;
+  int32_t infoYPosition;
 
   if (mActivatedHandle == mTopLeftHandle ||
       mActivatedHandle == mLeftHandle ||
@@ -681,15 +698,15 @@ nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt3
   if (textInfo) {
     res = mResizingInfo->RemoveChild(textInfo, getter_AddRefs(junk));
     NS_ENSURE_SUCCESS(res, res);
-    textInfo = nsnull;
-    junk = nsnull;
+    textInfo = nullptr;
+    junk = nullptr;
   }
 
   nsAutoString widthStr, heightStr, diffWidthStr, diffHeightStr;
   widthStr.AppendInt(aW);
   heightStr.AppendInt(aH);
-  PRInt32 diffWidth  = aW - mResizedObjectWidth;
-  PRInt32 diffHeight = aH - mResizedObjectHeight;
+  int32_t diffWidth  = aW - mResizedObjectWidth;
+  int32_t diffHeight = aH - mResizedObjectHeight;
   if (diffWidth > 0)
     diffWidthStr.AssignLiteral("+");
   if (diffHeight > 0)
@@ -719,8 +736,8 @@ nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt3
 nsresult
 nsHTMLEditor::SetShadowPosition(nsIDOMElement * aShadow,
                                 nsIDOMElement * aOriginalObject,
-                                PRInt32 aOriginalObjectX,
-                                PRInt32 aOriginalObjectY)
+                                int32_t aOriginalObjectX,
+                                int32_t aOriginalObjectY)
 {
   SetAnonymousElementPosition(aOriginalObjectX, aOriginalObjectY, aShadow);
 
@@ -735,10 +752,10 @@ nsHTMLEditor::SetShadowPosition(nsIDOMElement * aShadow,
   return NS_OK;
 }
 
-PRInt32
-nsHTMLEditor::GetNewResizingIncrement(PRInt32 aX, PRInt32 aY, PRInt32 aID)
+int32_t
+nsHTMLEditor::GetNewResizingIncrement(int32_t aX, int32_t aY, int32_t aID)
 {
-  PRInt32 result = 0;
+  int32_t result = 0;
   if (!mPreserveRatio) {
     switch (aID) {
       case kX:
@@ -753,8 +770,8 @@ nsHTMLEditor::GetNewResizingIncrement(PRInt32 aX, PRInt32 aY, PRInt32 aID)
     return result;
   }
 
-  PRInt32 xi = (aX - mOriginalX) * mWidthIncrementFactor;
-  PRInt32 yi = (aY - mOriginalY) * mHeightIncrementFactor;
+  int32_t xi = (aX - mOriginalX) * mWidthIncrementFactor;
+  int32_t yi = (aY - mOriginalY) * mHeightIncrementFactor;
   float objectSizeRatio = 
               ((float)mResizedObjectWidth) / ((float)mResizedObjectHeight);
   result = (xi > yi) ? xi : yi;
@@ -762,50 +779,50 @@ nsHTMLEditor::GetNewResizingIncrement(PRInt32 aX, PRInt32 aY, PRInt32 aID)
     case kX:
     case kWidth:
       if (result == yi)
-        result = (PRInt32) (((float) result) * objectSizeRatio);
-      result = (PRInt32) (((float) result) * mWidthIncrementFactor);
+        result = (int32_t) (((float) result) * objectSizeRatio);
+      result = (int32_t) (((float) result) * mWidthIncrementFactor);
       break;
     case kY:
     case kHeight:
       if (result == xi)
-        result =  (PRInt32) (((float) result) / objectSizeRatio);
-      result = (PRInt32) (((float) result) * mHeightIncrementFactor);
+        result =  (int32_t) (((float) result) / objectSizeRatio);
+      result = (int32_t) (((float) result) * mHeightIncrementFactor);
       break;
   }
   return result;
 }
 
-PRInt32
-nsHTMLEditor::GetNewResizingX(PRInt32 aX, PRInt32 aY)
+int32_t
+nsHTMLEditor::GetNewResizingX(int32_t aX, int32_t aY)
 {
-  PRInt32 resized = mResizedObjectX +
+  int32_t resized = mResizedObjectX +
                     GetNewResizingIncrement(aX, aY, kX) * mXIncrementFactor;
-  PRInt32 max =   mResizedObjectX + mResizedObjectWidth;
+  int32_t max =   mResizedObjectX + mResizedObjectWidth;
   return NS_MIN(resized, max);
 }
 
-PRInt32
-nsHTMLEditor::GetNewResizingY(PRInt32 aX, PRInt32 aY)
+int32_t
+nsHTMLEditor::GetNewResizingY(int32_t aX, int32_t aY)
 {
-  PRInt32 resized = mResizedObjectY +
+  int32_t resized = mResizedObjectY +
                     GetNewResizingIncrement(aX, aY, kY) * mYIncrementFactor;
-  PRInt32 max =   mResizedObjectY + mResizedObjectHeight;
+  int32_t max =   mResizedObjectY + mResizedObjectHeight;
   return NS_MIN(resized, max);
 }
 
-PRInt32
-nsHTMLEditor::GetNewResizingWidth(PRInt32 aX, PRInt32 aY)
+int32_t
+nsHTMLEditor::GetNewResizingWidth(int32_t aX, int32_t aY)
 {
-  PRInt32 resized = mResizedObjectWidth +
+  int32_t resized = mResizedObjectWidth +
                      GetNewResizingIncrement(aX, aY, kWidth) *
                          mWidthIncrementFactor;
   return NS_MAX(resized, 1);
 }
 
-PRInt32
-nsHTMLEditor::GetNewResizingHeight(PRInt32 aX, PRInt32 aY)
+int32_t
+nsHTMLEditor::GetNewResizingHeight(int32_t aX, int32_t aY)
 {
-  PRInt32 resized = mResizedObjectHeight +
+  int32_t resized = mResizedObjectHeight +
                      GetNewResizingIncrement(aX, aY, kHeight) *
                          mHeightIncrementFactor;
   return NS_MAX(resized, 1);
@@ -822,14 +839,14 @@ nsHTMLEditor::MouseMove(nsIDOMEvent* aMouseEvent)
     // we are resizing and the mouse pointer's position has changed
     // we have to resdisplay the shadow
     nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
-    PRInt32 clientX, clientY;
+    int32_t clientX, clientY;
     mouseEvent->GetClientX(&clientX);
     mouseEvent->GetClientY(&clientY);
 
-    PRInt32 newX = GetNewResizingX(clientX, clientY);
-    PRInt32 newY = GetNewResizingY(clientX, clientY);
-    PRInt32 newWidth  = GetNewResizingWidth(clientX, clientY);
-    PRInt32 newHeight = GetNewResizingHeight(clientX, clientY);
+    int32_t newX = GetNewResizingX(clientX, clientY);
+    int32_t newY = GetNewResizingY(clientX, clientY);
+    int32_t newWidth  = GetNewResizingWidth(clientX, clientY);
+    int32_t newHeight = GetNewResizingHeight(clientX, clientY);
 
     mHTMLCSSUtils->SetCSSPropertyPixels(mResizingShadow,
                                         leftStr,
@@ -849,29 +866,29 @@ nsHTMLEditor::MouseMove(nsIDOMEvent* aMouseEvent)
 
   if (mGrabberClicked) {
     nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
-    PRInt32 clientX, clientY;
+    int32_t clientX, clientY;
     mouseEvent->GetClientX(&clientX);
     mouseEvent->GetClientY(&clientY);
 
-    PRInt32 xThreshold =
+    int32_t xThreshold =
       LookAndFeel::GetInt(LookAndFeel::eIntID_DragThresholdX, 1);
-    PRInt32 yThreshold =
+    int32_t yThreshold =
       LookAndFeel::GetInt(LookAndFeel::eIntID_DragThresholdY, 1);
 
-    if (NS_ABS(clientX - mOriginalX ) * 2 >= xThreshold ||
-        NS_ABS(clientY - mOriginalY ) * 2 >= yThreshold) {
+    if (std::abs(clientX - mOriginalX ) * 2 >= xThreshold ||
+        std::abs(clientY - mOriginalY ) * 2 >= yThreshold) {
       mGrabberClicked = false;
-      StartMoving(nsnull);
+      StartMoving(nullptr);
     }
   }
   if (mIsMoving) {
     nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
-    PRInt32 clientX, clientY;
+    int32_t clientX, clientY;
     mouseEvent->GetClientX(&clientX);
     mouseEvent->GetClientY(&clientY);
 
-    PRInt32 newX = mPositionedObjectX + clientX - mOriginalX;
-    PRInt32 newY = mPositionedObjectY + clientY - mOriginalY;
+    int32_t newX = mPositionedObjectX + clientX - mOriginalX;
+    int32_t newY = mPositionedObjectY + clientY - mOriginalY;
 
     SnapToGrid(newX, newY);
 
@@ -882,7 +899,7 @@ nsHTMLEditor::MouseMove(nsIDOMEvent* aMouseEvent)
 }
 
 void
-nsHTMLEditor::SetFinalSize(PRInt32 aX, PRInt32 aY)
+nsHTMLEditor::SetFinalSize(int32_t aX, int32_t aY)
 {
   if (!mResizedObject) {
     // paranoia
@@ -891,20 +908,20 @@ nsHTMLEditor::SetFinalSize(PRInt32 aX, PRInt32 aY)
 
   if (mActivatedHandle) {
     mActivatedHandle->RemoveAttribute(NS_LITERAL_STRING("_moz_activated"));
-    mActivatedHandle = nsnull;
+    mActivatedHandle = nullptr;
   }
 
   // we have now to set the new width and height of the resized object
   // we don't set the x and y position because we don't control that in
   // a normal HTML layout
-  PRInt32 left   = GetNewResizingX(aX, aY);
-  PRInt32 top    = GetNewResizingY(aX, aY);
-  PRInt32 width  = GetNewResizingWidth(aX, aY);
-  PRInt32 height = GetNewResizingHeight(aX, aY);
+  int32_t left   = GetNewResizingX(aX, aY);
+  int32_t top    = GetNewResizingY(aX, aY);
+  int32_t width  = GetNewResizingWidth(aX, aY);
+  int32_t height = GetNewResizingHeight(aX, aY);
   bool setWidth  = !mResizedObjectIsAbsolutelyPositioned || (width != mResizedObjectWidth);
   bool setHeight = !mResizedObjectIsAbsolutelyPositioned || (height != mResizedObjectHeight);
   
-  PRInt32 x, y;
+  int32_t x, y;
   x = left - ((mResizedObjectIsAbsolutelyPositioned) ? mResizedObjectBorderLeft+mResizedObjectMarginLeft : 0);
   y = top - ((mResizedObjectIsAbsolutelyPositioned) ? mResizedObjectBorderTop+mResizedObjectMarginTop : 0);
 
@@ -986,10 +1003,10 @@ nsHTMLEditor::SetFinalSize(PRInt32 aX, PRInt32 aY)
                                       false);
   }
   // finally notify the listeners if any
-  PRInt32 listenersCount = objectResizeEventListeners.Count();
+  int32_t listenersCount = objectResizeEventListeners.Count();
   if (listenersCount) {
     nsCOMPtr<nsIHTMLObjectResizeListener> listener;
-    PRInt32 index;
+    int32_t index;
     for (index = 0; index < listenersCount; index++) {
       listener = objectResizeEventListeners[index];
       listener->OnEndResizing(mResizedObject,
