@@ -13,9 +13,7 @@
 //Interfaces Needed
 #include "nsReadableUtils.h"
 #include "nsIComponentManager.h"
-#include "nsIDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMXULDocument.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMElement.h"
 #include "nsIInterfaceRequestor.h"
@@ -64,21 +62,21 @@ static NS_DEFINE_CID(kChildCID, NS_CHILD_CID);
 //***    nsWebBrowser: Object Management
 //*****************************************************************************
 
-nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull), 
-   mInitInfo(nsnull),
+nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nullptr), 
+   mInitInfo(nullptr),
    mContentType(typeContentWrapper),
    mActivating(false),
    mShouldEnableHistory(true),
    mIsActive(true),
-   mParentNativeWindow(nsnull),
-   mProgressListener(nsnull),
+   mParentNativeWindow(nullptr),
+   mProgressListener(nullptr),
    mBackgroundColor(0),
    mPersistCurrentState(nsIWebBrowserPersist::PERSIST_STATE_READY),
    mPersistResult(NS_OK),
    mPersistFlags(nsIWebBrowserPersist::PERSIST_FLAGS_NONE),
-   mStream(nsnull),
-   mParentWidget(nsnull),
-   mListenerArray(nsnull)
+   mStream(nullptr),
+   mParentWidget(nullptr),
+   mListenerArray(nullptr)
 {
     mInitInfo = new nsWebBrowserInitInfo();
     mWWatch = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
@@ -94,31 +92,31 @@ NS_IMETHODIMP nsWebBrowser::InternalDestroy()
 {
 
    if (mInternalWidget) {
-     mInternalWidget->SetClientData(0);
+     mInternalWidget->SetWidgetListener(nullptr);
      mInternalWidget->Destroy();
-     mInternalWidget = nsnull; // Force release here.
+     mInternalWidget = nullptr; // Force release here.
    }
 
-   SetDocShell(nsnull);
+   SetDocShell(nullptr);
 
    if(mDocShellTreeOwner)
       {
-      mDocShellTreeOwner->WebBrowser(nsnull);
+      mDocShellTreeOwner->WebBrowser(nullptr);
       NS_RELEASE(mDocShellTreeOwner);
       }
    if(mInitInfo)
       {
       delete mInitInfo;
-      mInitInfo = nsnull;
+      mInitInfo = nullptr;
       }
 
    if (mListenerArray) {
-      for (PRUint32 i = 0, end = mListenerArray->Length(); i < end; i++) {
+      for (uint32_t i = 0, end = mListenerArray->Length(); i < end; i++) {
          nsWebBrowserListenerState *state = mListenerArray->ElementAt(i);
          delete state;
       }
       delete mListenerArray;
-      mListenerArray = nsnull;
+      mListenerArray = nullptr;
    }
 
    return NS_OK;
@@ -260,7 +258,7 @@ NS_IMETHODIMP nsWebBrowser::RemoveWebBrowserListener(nsIWeakReference *aListener
         if (!mListenerArray) return NS_ERROR_FAILURE;
 
         // iterate the array and remove the queued listener
-        PRInt32 count = mListenerArray->Length();
+        int32_t count = mListenerArray->Length();
         while (count > 0) {
             nsWebBrowserListenerState *state = mListenerArray->ElementAt(count);
             NS_ASSERTION(state, "list construction problem");
@@ -275,12 +273,12 @@ NS_IMETHODIMP nsWebBrowser::RemoveWebBrowserListener(nsIWeakReference *aListener
 
         // if we've emptied the array, get rid of it.
         if (0 >= mListenerArray->Length()) {
-            for (PRUint32 i = 0, end = mListenerArray->Length(); i < end; i++) {
+            for (uint32_t i = 0, end = mListenerArray->Length(); i < end; i++) {
                nsWebBrowserListenerState *state = mListenerArray->ElementAt(i);
                delete state;
             }
             delete mListenerArray;
-            mListenerArray = nsnull;
+            mListenerArray = nullptr;
         }
 
     } else {
@@ -332,7 +330,7 @@ NS_IMETHODIMP nsWebBrowser::GetContainerWindow(nsIWebBrowserChrome** aTopWindow)
    if(mDocShellTreeOwner) {
       *aTopWindow = mDocShellTreeOwner->GetWebBrowserChrome().get();
    } else {
-      *aTopWindow = nsnull;
+      *aTopWindow = nullptr;
    }
 
    return NS_OK;
@@ -348,7 +346,7 @@ NS_IMETHODIMP nsWebBrowser::GetParentURIContentListener(nsIURIContentListener**
    aParentContentListener)
 {
    NS_ENSURE_ARG_POINTER(aParentContentListener);
-   *aParentContentListener = nsnull;
+   *aParentContentListener = nullptr;
 
    // get the interface from the docshell
    nsCOMPtr<nsIURIContentListener> listener(do_GetInterface(mDocShell));
@@ -445,7 +443,7 @@ NS_IMETHODIMP nsWebBrowser::NameEquals(const PRUnichar *aName, bool *_retval)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::GetItemType(PRInt32* aItemType)
+NS_IMETHODIMP nsWebBrowser::GetItemType(int32_t* aItemType)
 {
    NS_ENSURE_ARG_POINTER(aItemType);
 
@@ -453,26 +451,26 @@ NS_IMETHODIMP nsWebBrowser::GetItemType(PRInt32* aItemType)
    return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::SetItemType(PRInt32 aItemType)
+NS_IMETHODIMP nsWebBrowser::SetItemType(int32_t aItemType)
 {
     NS_ENSURE_TRUE((aItemType == typeContentWrapper || aItemType == typeChromeWrapper), NS_ERROR_FAILURE);
     mContentType = aItemType;
     if (mDocShellAsItem)
         mDocShellAsItem->SetItemType(mContentType == typeChromeWrapper
-                                         ? static_cast<PRInt32>(typeChrome)
-                                         : static_cast<PRInt32>(typeContent));
+                                         ? static_cast<int32_t>(typeChrome)
+                                         : static_cast<int32_t>(typeContent));
     return NS_OK;
 }
 
 NS_IMETHODIMP nsWebBrowser::GetParent(nsIDocShellTreeItem** aParent)
 {
-   *aParent = nsnull;
+   *aParent = nullptr;
    return NS_OK;
 }
 
 NS_IMETHODIMP nsWebBrowser::GetSameTypeParent(nsIDocShellTreeItem** aParent)
 {
-   *aParent = nsnull;
+   *aParent = nullptr;
 
    return NS_OK;
 }
@@ -525,7 +523,7 @@ NS_IMETHODIMP nsWebBrowser::FindItemWithName(const PRUnichar *aName,
 NS_IMETHODIMP nsWebBrowser::GetTreeOwner(nsIDocShellTreeOwner** aTreeOwner)
 {  
     NS_ENSURE_ARG_POINTER(aTreeOwner);
-    *aTreeOwner = nsnull;
+    *aTreeOwner = nullptr;
     if(mDocShellTreeOwner)
     {
         if (mDocShellTreeOwner->mTreeOwner)
@@ -551,7 +549,7 @@ NS_IMETHODIMP nsWebBrowser::SetTreeOwner(nsIDocShellTreeOwner* aTreeOwner)
 // nsWebBrowser::nsIDocShellTreeItem
 //*****************************************************************************
 
-NS_IMETHODIMP nsWebBrowser::GetChildCount(PRInt32 * aChildCount)
+NS_IMETHODIMP nsWebBrowser::GetChildCount(int32_t * aChildCount)
 {
     NS_ENSURE_ARG_POINTER(aChildCount);
     *aChildCount = 0;
@@ -568,7 +566,7 @@ NS_IMETHODIMP nsWebBrowser::RemoveChild(nsIDocShellTreeItem * aChild)
     return NS_ERROR_UNEXPECTED;
 }
 
-NS_IMETHODIMP nsWebBrowser::GetChildAt(PRInt32 aIndex,
+NS_IMETHODIMP nsWebBrowser::GetChildAt(int32_t aIndex,
                                        nsIDocShellTreeItem ** aChild)
 {
     return NS_ERROR_UNEXPECTED;
@@ -583,7 +581,7 @@ NS_IMETHODIMP nsWebBrowser::FindChildWithName(
 {
     NS_ENSURE_ARG_POINTER(_retval);
 
-    *_retval = nsnull;
+    *_retval = nullptr;
     return NS_OK;
 }
 
@@ -620,7 +618,7 @@ NS_IMETHODIMP nsWebBrowser::GoForward()
 }
 
 NS_IMETHODIMP nsWebBrowser::LoadURI(const PRUnichar* aURI,
-                                    PRUint32 aLoadFlags,
+                                    uint32_t aLoadFlags,
                                     nsIURI* aReferringURI,
                                     nsIInputStream* aPostDataStream,
                                     nsIInputStream* aExtraHeaderStream)
@@ -634,21 +632,21 @@ NS_IMETHODIMP nsWebBrowser::LoadURI(const PRUnichar* aURI,
                                   aExtraHeaderStream);
 }
 
-NS_IMETHODIMP nsWebBrowser::Reload(PRUint32 aReloadFlags)
+NS_IMETHODIMP nsWebBrowser::Reload(uint32_t aReloadFlags)
 {
    NS_ENSURE_STATE(mDocShell);
 
    return mDocShellAsNav->Reload(aReloadFlags);
 }
 
-NS_IMETHODIMP nsWebBrowser::GotoIndex(PRInt32 aIndex)
+NS_IMETHODIMP nsWebBrowser::GotoIndex(int32_t aIndex)
 {
    NS_ENSURE_STATE(mDocShell);
 
    return mDocShellAsNav->GotoIndex(aIndex);
 }
 
-NS_IMETHODIMP nsWebBrowser::Stop(PRUint32 aStopFlags)
+NS_IMETHODIMP nsWebBrowser::Stop(uint32_t aStopFlags)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -706,7 +704,7 @@ NS_IMETHODIMP nsWebBrowser::GetDocument(nsIDOMDocument** aDocument)
 //*****************************************************************************
 
 /* void setProperty (in unsigned long aId, in unsigned long aValue); */
-NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
+NS_IMETHODIMP nsWebBrowser::SetProperty(uint32_t aId, uint32_t aValue)
 {
     nsresult rv = NS_OK;
     
@@ -770,8 +768,8 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
     case nsIWebBrowserSetup::SETUP_IS_CHROME_WRAPPER:
         {
            NS_ENSURE_TRUE((aValue == true || aValue == false), NS_ERROR_INVALID_ARG);
-           SetItemType(aValue ? static_cast<PRInt32>(typeChromeWrapper)
-                              : static_cast<PRInt32>(typeContentWrapper));
+           SetItemType(aValue ? static_cast<int32_t>(typeChromeWrapper)
+                              : static_cast<int32_t>(typeContentWrapper));
         }
         break;
     default:
@@ -787,7 +785,7 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
 //*****************************************************************************
 
 /* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long aStateFlags, in nsresult aStatus); */
-NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
+NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, uint32_t aStateFlags, nsresult aStatus)
 {
     if (mPersist)
     {
@@ -795,7 +793,7 @@ NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIReque
     }
     if (aStateFlags & STATE_IS_NETWORK && aStateFlags & STATE_STOP)
     {
-        mPersist = nsnull;
+        mPersist = nullptr;
     }
     if (mProgressListener)
     {
@@ -805,7 +803,7 @@ NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIReque
 }
 
 /* void onProgressChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aCurSelfProgress, in long aMaxSelfProgress, in long aCurTotalProgress, in long aMaxTotalProgress); */
-NS_IMETHODIMP nsWebBrowser::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress, PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
+NS_IMETHODIMP nsWebBrowser::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, int32_t aCurSelfProgress, int32_t aMaxSelfProgress, int32_t aCurTotalProgress, int32_t aMaxTotalProgress)
 {
     if (mPersist)
     {
@@ -819,7 +817,7 @@ NS_IMETHODIMP nsWebBrowser::OnProgressChange(nsIWebProgress *aWebProgress, nsIRe
 }
 
 /* void onLocationChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsIURI location, in unsigned long aFlags); */
-NS_IMETHODIMP nsWebBrowser::OnLocationChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, nsIURI *location, PRUint32 aFlags)
+NS_IMETHODIMP nsWebBrowser::OnLocationChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, nsIURI *location, uint32_t aFlags)
 {
     if (mProgressListener)
     {
@@ -839,7 +837,7 @@ NS_IMETHODIMP nsWebBrowser::OnStatusChange(nsIWebProgress *aWebProgress, nsIRequ
 }
 
 /* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
-NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 state)
+NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, uint32_t state)
 {
     if (mProgressListener)
     {
@@ -853,7 +851,7 @@ NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRe
 //*****************************************************************************
 
 /* attribute unsigned long persistFlags; */
-NS_IMETHODIMP nsWebBrowser::GetPersistFlags(PRUint32 *aPersistFlags)
+NS_IMETHODIMP nsWebBrowser::GetPersistFlags(uint32_t *aPersistFlags)
 {
     NS_ENSURE_ARG_POINTER(aPersistFlags);
     nsresult rv = NS_OK;
@@ -864,7 +862,7 @@ NS_IMETHODIMP nsWebBrowser::GetPersistFlags(PRUint32 *aPersistFlags)
     *aPersistFlags = mPersistFlags;
     return rv;
 }
-NS_IMETHODIMP nsWebBrowser::SetPersistFlags(PRUint32 aPersistFlags)
+NS_IMETHODIMP nsWebBrowser::SetPersistFlags(uint32_t aPersistFlags)
 {
     nsresult rv = NS_OK;
     mPersistFlags = aPersistFlags;
@@ -878,7 +876,7 @@ NS_IMETHODIMP nsWebBrowser::SetPersistFlags(PRUint32 aPersistFlags)
 
 
 /* readonly attribute unsigned long currentState; */
-NS_IMETHODIMP nsWebBrowser::GetCurrentState(PRUint32 *aCurrentState)
+NS_IMETHODIMP nsWebBrowser::GetCurrentState(uint32_t *aCurrentState)
 {
     NS_ENSURE_ARG_POINTER(aCurrentState);
     if (mPersist)
@@ -889,8 +887,8 @@ NS_IMETHODIMP nsWebBrowser::GetCurrentState(PRUint32 *aCurrentState)
     return NS_OK;
 }
 
-/* readonly attribute unsigned long result; */
-NS_IMETHODIMP nsWebBrowser::GetResult(PRUint32 *aResult)
+/* readonly attribute nsresult result; */
+NS_IMETHODIMP nsWebBrowser::GetResult(nsresult *aResult)
 {
     NS_ENSURE_ARG_POINTER(aResult);
     if (mPersist)
@@ -918,18 +916,26 @@ NS_IMETHODIMP nsWebBrowser::SetProgressListener(nsIWebProgressListener * aProgre
 
 /* void saveURI (in nsIURI aURI, in nsIURI aReferrer,
    in nsISupports aCacheKey, in nsIInputStream aPostData, in wstring aExtraHeaders,
-   in nsISupports aFile); */
+   in nsISupports aFile, in nsILoadContext aPrivacyContext); */
 NS_IMETHODIMP nsWebBrowser::SaveURI(
     nsIURI *aURI, nsISupports *aCacheKey, nsIURI *aReferrer, nsIInputStream *aPostData,
-    const char *aExtraHeaders, nsISupports *aFile)
+    const char *aExtraHeaders, nsISupports *aFile, nsILoadContext* aPrivacyContext)
+{
+    return SavePrivacyAwareURI(aURI, aCacheKey, aReferrer, aPostData, aExtraHeaders,
+                               aFile, aPrivacyContext && aPrivacyContext->UsePrivateBrowsing());
+}
+
+NS_IMETHODIMP nsWebBrowser::SavePrivacyAwareURI(
+    nsIURI *aURI, nsISupports *aCacheKey, nsIURI *aReferrer, nsIInputStream *aPostData,
+    const char *aExtraHeaders, nsISupports *aFile, bool aIsPrivate)
 {
     if (mPersist)
     {
-        PRUint32 currentState;
+        uint32_t currentState;
         mPersist->GetCurrentState(&currentState);
         if (currentState == PERSIST_STATE_FINISHED)
         {
-            mPersist = nsnull;
+            mPersist = nullptr;
         }
         else
         {
@@ -959,10 +965,11 @@ NS_IMETHODIMP nsWebBrowser::SaveURI(
     mPersist->SetProgressListener(this);
     mPersist->SetPersistFlags(mPersistFlags);
     mPersist->GetCurrentState(&mPersistCurrentState);
-    rv = mPersist->SaveURI(uri, aCacheKey, aReferrer, aPostData, aExtraHeaders, aFile);
+    rv = mPersist->SavePrivacyAwareURI(uri, aCacheKey, aReferrer, aPostData,
+                                       aExtraHeaders, aFile, aIsPrivate);
     if (NS_FAILED(rv))
     {
-        mPersist = nsnull;
+        mPersist = nullptr;
     }
     return rv;
 }
@@ -973,11 +980,11 @@ NS_IMETHODIMP nsWebBrowser::SaveChannel(
 {
     if (mPersist)
     {
-        PRUint32 currentState;
+        uint32_t currentState;
         mPersist->GetCurrentState(&currentState);
         if (currentState == PERSIST_STATE_FINISHED)
         {
-            mPersist = nsnull;
+            mPersist = nullptr;
         }
         else
         {
@@ -996,7 +1003,7 @@ NS_IMETHODIMP nsWebBrowser::SaveChannel(
     rv = mPersist->SaveChannel(aChannel, aFile);
     if (NS_FAILED(rv))
     {
-        mPersist = nsnull;
+        mPersist = nullptr;
     }
     return rv;
 }
@@ -1004,15 +1011,15 @@ NS_IMETHODIMP nsWebBrowser::SaveChannel(
 /* void saveDocument (in nsIDOMDocument document, in nsISupports aFile, in nsISupports aDataPath); */
 NS_IMETHODIMP nsWebBrowser::SaveDocument(
     nsIDOMDocument *aDocument, nsISupports *aFile, nsISupports *aDataPath,
-    const char *aOutputContentType, PRUint32 aEncodingFlags, PRUint32 aWrapColumn)
+    const char *aOutputContentType, uint32_t aEncodingFlags, uint32_t aWrapColumn)
 {
     if (mPersist)
     {
-        PRUint32 currentState;
+        uint32_t currentState;
         mPersist->GetCurrentState(&currentState);
         if (currentState == PERSIST_STATE_FINISHED)
         {
-            mPersist = nsnull;
+            mPersist = nullptr;
         }
         else
         {
@@ -1048,7 +1055,7 @@ NS_IMETHODIMP nsWebBrowser::SaveDocument(
     rv = mPersist->SaveDocument(doc, aFile, aDataPath, aOutputContentType, aEncodingFlags, aWrapColumn);
     if (NS_FAILED(rv))
     {
-        mPersist = nsnull;
+        mPersist = nullptr;
     }
     return rv;
 }
@@ -1081,7 +1088,7 @@ NS_IMETHODIMP nsWebBrowser::Cancel(nsresult aReason)
 //*****************************************************************************
 
 NS_IMETHODIMP nsWebBrowser::InitWindow(nativeWindow aParentNativeWindow,
-   nsIWidget* aParentWidget, PRInt32 aX, PRInt32 aY, PRInt32 aCX, PRInt32 aCY)   
+   nsIWidget* aParentWidget, int32_t aX, int32_t aY, int32_t aCX, int32_t aCY)   
 {
    NS_ENSURE_ARG(aParentNativeWindow || aParentWidget);
    NS_ENSURE_STATE(!mDocShell || mInitInfo);
@@ -1120,9 +1127,8 @@ NS_IMETHODIMP nsWebBrowser::Create()
       widgetInit.mWindowType = eWindowType_child;
       nsIntRect bounds(mInitInfo->x, mInitInfo->y, mInitInfo->cx, mInitInfo->cy);
       
-      mInternalWidget->SetClientData(static_cast<nsWebBrowser *>(this));
-      mInternalWidget->Create(nsnull, mParentNativeWindow, bounds, nsWebBrowser::HandleEvent,
-                              nsnull, &widgetInit);  
+      mInternalWidget->SetWidgetListener(this);
+      mInternalWidget->Create(nullptr, mParentNativeWindow, bounds, nullptr, &widgetInit);
       }
 
     nsCOMPtr<nsIDocShell> docShell(do_CreateInstance("@mozilla.org/docshell;1", &rv));
@@ -1137,8 +1143,8 @@ NS_IMETHODIMP nsWebBrowser::Create()
    // the docshell has been set so we now have our listener registrars.
    if (mListenerArray) {
       // we had queued up some listeners, let's register them now.
-      PRUint32 count = mListenerArray->Length();
-      PRUint32 i = 0;
+      uint32_t count = mListenerArray->Length();
+      uint32_t i = 0;
       NS_ASSERTION(count > 0, "array construction problem");
       while (i < count) {
           nsWebBrowserListenerState *state = mListenerArray->ElementAt(i);
@@ -1148,12 +1154,12 @@ NS_IMETHODIMP nsWebBrowser::Create()
           (void)BindListener(listener, state->mID);
           i++;
       }
-      for (PRUint32 i = 0, end = mListenerArray->Length(); i < end; i++) {
+      for (uint32_t i = 0, end = mListenerArray->Length(); i < end; i++) {
          nsWebBrowserListenerState *state = mListenerArray->ElementAt(i);
          delete state;
       }
       delete mListenerArray;
-      mListenerArray = nsnull;
+      mListenerArray = nullptr;
    }
 
    // HACK ALERT - this registration registers the nsDocShellTreeOwner as a 
@@ -1161,12 +1167,12 @@ NS_IMETHODIMP nsWebBrowser::Create()
    // progress callbacks. If we can register the MouseListener another way, this 
    // registration can go away, and nsDocShellTreeOwner can stop implementing
    // nsIWebProgressListener.
-   nsCOMPtr<nsISupports> supports = nsnull;
+   nsCOMPtr<nsISupports> supports = nullptr;
    (void)mDocShellTreeOwner->QueryInterface(NS_GET_IID(nsIWebProgressListener),
                              static_cast<void**>(getter_AddRefs(supports)));
    (void)BindListener(supports, NS_GET_IID(nsIWebProgressListener));
 
-   NS_ENSURE_SUCCESS(mDocShellAsWin->InitWindow(nsnull,
+   NS_ENSURE_SUCCESS(mDocShellAsWin->InitWindow(nullptr,
       docShellParentWidget, mInitInfo->x, mInitInfo->y, mInitInfo->cx,
       mInitInfo->cy), NS_ERROR_FAILURE);
 
@@ -1218,7 +1224,7 @@ NS_IMETHODIMP nsWebBrowser::Create()
    mDocShellTreeOwner->AddChromeListeners();
 
    delete mInitInfo;
-   mInitInfo = nsnull;
+   mInitInfo = nullptr;
 
    return NS_OK; 
 }
@@ -1233,38 +1239,44 @@ NS_IMETHODIMP nsWebBrowser::Destroy()
    return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::SetPosition(PRInt32 aX, PRInt32 aY)
+NS_IMETHODIMP nsWebBrowser::GetUnscaledDevicePixelsPerCSSPixel(double *aScale)
 {
-   PRInt32 cx = 0;
-   PRInt32 cy = 0;
+  *aScale = mParentWidget ? mParentWidget->GetDefaultScale() : 1.0;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsWebBrowser::SetPosition(int32_t aX, int32_t aY)
+{
+   int32_t cx = 0;
+   int32_t cy = 0;
 
    GetSize(&cx, &cy);
 
    return SetPositionAndSize(aX, aY, cx, cy, false);
 }
 
-NS_IMETHODIMP nsWebBrowser::GetPosition(PRInt32* aX, PRInt32* aY)
+NS_IMETHODIMP nsWebBrowser::GetPosition(int32_t* aX, int32_t* aY)
 {
-   return GetPositionAndSize(aX, aY, nsnull, nsnull);
+   return GetPositionAndSize(aX, aY, nullptr, nullptr);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetSize(PRInt32 aCX, PRInt32 aCY, bool aRepaint)
+NS_IMETHODIMP nsWebBrowser::SetSize(int32_t aCX, int32_t aCY, bool aRepaint)
 {
-   PRInt32 x = 0;
-   PRInt32 y = 0;
+   int32_t x = 0;
+   int32_t y = 0;
 
    GetPosition(&x, &y);
 
    return SetPositionAndSize(x, y, aCX, aCY, aRepaint);
 }
 
-NS_IMETHODIMP nsWebBrowser::GetSize(PRInt32* aCX, PRInt32* aCY)
+NS_IMETHODIMP nsWebBrowser::GetSize(int32_t* aCX, int32_t* aCY)
 {
-   return GetPositionAndSize(nsnull, nsnull, aCX, aCY);
+   return GetPositionAndSize(nullptr, nullptr, aCX, aCY);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetPositionAndSize(PRInt32 aX, PRInt32 aY,
-   PRInt32 aCX, PRInt32 aCY, bool aRepaint)
+NS_IMETHODIMP nsWebBrowser::SetPositionAndSize(int32_t aX, int32_t aY,
+   int32_t aCX, int32_t aCY, bool aRepaint)
 {
    if(!mDocShell)
       {
@@ -1275,8 +1287,8 @@ NS_IMETHODIMP nsWebBrowser::SetPositionAndSize(PRInt32 aX, PRInt32 aY,
       }
    else
       {
-      PRInt32 doc_x = aX;
-      PRInt32 doc_y = aY;
+      int32_t doc_x = aX;
+      int32_t doc_y = aY;
 
       // If there is an internal widget we need to make the docShell coordinates
       // relative to the internal widget rather than the calling app's parent.
@@ -1295,8 +1307,8 @@ NS_IMETHODIMP nsWebBrowser::SetPositionAndSize(PRInt32 aX, PRInt32 aY,
    return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::GetPositionAndSize(PRInt32* aX, PRInt32* aY, 
-   PRInt32* aCX, PRInt32* aCY)
+NS_IMETHODIMP nsWebBrowser::GetPositionAndSize(int32_t* aX, int32_t* aY, 
+   int32_t* aCX, int32_t* aCY)
 {
    if(!mDocShell)
       {
@@ -1357,7 +1369,7 @@ NS_IMETHODIMP nsWebBrowser::SetParentWidget(nsIWidget* aParentWidget)
    if(mParentWidget)
       mParentNativeWindow = mParentWidget->GetNativeData(NS_NATIVE_WIDGET);
    else
-      mParentNativeWindow = nsnull;
+      mParentNativeWindow = nullptr;
 
    return NS_OK;
 }
@@ -1380,6 +1392,12 @@ NS_IMETHODIMP nsWebBrowser::SetParentNativeWindow(nativeWindow aParentNativeWind
    return NS_OK;
 }
 
+NS_IMETHODIMP nsWebBrowser::GetNativeHandle(nsAString& aNativeHandle)
+{
+   // the nativeHandle should be accessed from nsIXULWindow
+   return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsWebBrowser::GetVisibility(bool* visibility)
 {
    NS_ENSURE_ARG_POINTER(visibility);
@@ -1406,10 +1424,13 @@ NS_IMETHODIMP nsWebBrowser::SetVisibility(bool aVisibility)
    return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::GetEnabled(bool *aEnabled)
+NS_IMETHODIMP nsWebBrowser::GetEnabled(bool* aEnabled)
 {
-  if (mInternalWidget)
-    return mInternalWidget->IsEnabled(aEnabled);
+  if (mInternalWidget) {
+    *aEnabled = mInternalWidget->IsEnabled();
+    return NS_OK;
+  }
+
   return NS_ERROR_FAILURE;
 }
 
@@ -1466,24 +1487,24 @@ NS_IMETHODIMP nsWebBrowser::SetTitle(const PRUnichar* aTitle)
 // nsWebBrowser::nsIScrollable
 //*****************************************************************************
 
-NS_IMETHODIMP nsWebBrowser::GetCurScrollPos(PRInt32 aScrollOrientation, 
-   PRInt32* aCurPos)
+NS_IMETHODIMP nsWebBrowser::GetCurScrollPos(int32_t aScrollOrientation, 
+   int32_t* aCurPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
    return mDocShellAsScrollable->GetCurScrollPos(aScrollOrientation, aCurPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetCurScrollPos(PRInt32 aScrollOrientation, 
-   PRInt32 aCurPos)
+NS_IMETHODIMP nsWebBrowser::SetCurScrollPos(int32_t aScrollOrientation, 
+   int32_t aCurPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
    return mDocShellAsScrollable->SetCurScrollPos(aScrollOrientation, aCurPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetCurScrollPosEx(PRInt32 aCurHorizontalPos, 
-   PRInt32 aCurVerticalPos)
+NS_IMETHODIMP nsWebBrowser::SetCurScrollPosEx(int32_t aCurHorizontalPos, 
+   int32_t aCurVerticalPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1491,8 +1512,8 @@ NS_IMETHODIMP nsWebBrowser::SetCurScrollPosEx(PRInt32 aCurHorizontalPos,
       aCurVerticalPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::GetScrollRange(PRInt32 aScrollOrientation,
-   PRInt32* aMinPos, PRInt32* aMaxPos)
+NS_IMETHODIMP nsWebBrowser::GetScrollRange(int32_t aScrollOrientation,
+   int32_t* aMinPos, int32_t* aMaxPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1500,8 +1521,8 @@ NS_IMETHODIMP nsWebBrowser::GetScrollRange(PRInt32 aScrollOrientation,
       aMaxPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetScrollRange(PRInt32 aScrollOrientation,
-   PRInt32 aMinPos, PRInt32 aMaxPos)
+NS_IMETHODIMP nsWebBrowser::SetScrollRange(int32_t aScrollOrientation,
+   int32_t aMinPos, int32_t aMaxPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1509,8 +1530,8 @@ NS_IMETHODIMP nsWebBrowser::SetScrollRange(PRInt32 aScrollOrientation,
       aMaxPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetScrollRangeEx(PRInt32 aMinHorizontalPos,
-   PRInt32 aMaxHorizontalPos, PRInt32 aMinVerticalPos, PRInt32 aMaxVerticalPos)
+NS_IMETHODIMP nsWebBrowser::SetScrollRangeEx(int32_t aMinHorizontalPos,
+   int32_t aMaxHorizontalPos, int32_t aMinVerticalPos, int32_t aMaxVerticalPos)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1518,8 +1539,8 @@ NS_IMETHODIMP nsWebBrowser::SetScrollRangeEx(PRInt32 aMinHorizontalPos,
       aMaxHorizontalPos, aMinVerticalPos, aMaxVerticalPos);
 }
 
-NS_IMETHODIMP nsWebBrowser::GetDefaultScrollbarPreferences(PRInt32 aScrollOrientation,
-   PRInt32* aScrollbarPref)
+NS_IMETHODIMP nsWebBrowser::GetDefaultScrollbarPreferences(int32_t aScrollOrientation,
+   int32_t* aScrollbarPref)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1527,8 +1548,8 @@ NS_IMETHODIMP nsWebBrowser::GetDefaultScrollbarPreferences(PRInt32 aScrollOrient
       aScrollbarPref);
 }
 
-NS_IMETHODIMP nsWebBrowser::SetDefaultScrollbarPreferences(PRInt32 aScrollOrientation,
-   PRInt32 aScrollbarPref)
+NS_IMETHODIMP nsWebBrowser::SetDefaultScrollbarPreferences(int32_t aScrollOrientation,
+   int32_t aScrollbarPref)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1549,14 +1570,14 @@ NS_IMETHODIMP nsWebBrowser::GetScrollbarVisibility(bool* aVerticalVisible,
 // nsWebBrowser::nsITextScroll
 //*****************************************************************************   
 
-NS_IMETHODIMP nsWebBrowser::ScrollByLines(PRInt32 aNumLines)
+NS_IMETHODIMP nsWebBrowser::ScrollByLines(int32_t aNumLines)
 {
    NS_ENSURE_STATE(mDocShell);
 
    return mDocShellAsTextScroll->ScrollByLines(aNumLines);
 }
 
-NS_IMETHODIMP nsWebBrowser::ScrollByPages(PRInt32 aNumPages)
+NS_IMETHODIMP nsWebBrowser::ScrollByPages(int32_t aNumPages)
 {
    NS_ENSURE_STATE(mDocShell);
 
@@ -1611,14 +1632,14 @@ NS_IMETHODIMP nsWebBrowser::SetDocShell(nsIDocShell* aDocShell)
          if (mDocShellAsWin)
            mDocShellAsWin->Destroy();
 
-         mDocShell = nsnull;
-         mDocShellAsReq = nsnull;
-         mDocShellAsWin = nsnull;
-         mDocShellAsItem = nsnull;
-         mDocShellAsNav = nsnull;
-         mDocShellAsScrollable = nsnull;
-         mDocShellAsTextScroll = nsnull;
-         mWebProgress = nsnull;
+         mDocShell = nullptr;
+         mDocShellAsReq = nullptr;
+         mDocShellAsWin = nullptr;
+         mDocShellAsItem = nullptr;
+         mDocShellAsNav = nullptr;
+         mDocShellAsScrollable = nullptr;
+         mDocShellAsTextScroll = nullptr;
+         mWebProgress = nullptr;
      }
 
      return NS_OK; 
@@ -1652,69 +1673,45 @@ static void DrawThebesLayer(ThebesLayer* aLayer,
   aContext->Fill();  
 }
 
-/* static */
-nsEventStatus nsWebBrowser::HandleEvent(nsGUIEvent *aEvent)
+void nsWebBrowser::WindowRaised(nsIWidget* aWidget)
 {
-  nsWebBrowser  *browser = nsnull;
-  void          *data = nsnull;
-  nsIWidget     *widget = aEvent->widget;
-
-  if (!widget)
-    return nsEventStatus_eIgnore;
-
-  widget->GetClientData(data);
-  if (!data)
-    return nsEventStatus_eIgnore;
-
-  browser = static_cast<nsWebBrowser *>(data);
-
-  switch(aEvent->message) {
-
-  case NS_PAINT: {
-      LayerManager* layerManager = widget->GetLayerManager();
-      NS_ASSERTION(layerManager, "Must be in paint event");
-
-      layerManager->BeginTransaction();
-      nsRefPtr<ThebesLayer> root = layerManager->CreateThebesLayer();
-      nsPaintEvent* paintEvent = static_cast<nsPaintEvent*>(aEvent);
-      nsIntRect dirtyRect = paintEvent->region.GetBounds();
-      if (root) {
-          root->SetVisibleRegion(dirtyRect);
-          layerManager->SetRoot(root);
-      }
-      layerManager->EndTransaction(DrawThebesLayer, &browser->mBackgroundColor);
-      return nsEventStatus_eConsumeDoDefault;
-    }
-
-  case NS_ACTIVATE: {
 #if defined(DEBUG_smaug)
-    nsCOMPtr<nsIDOMDocument> domDocument = do_GetInterface(browser->mDocShell);
-    nsAutoString documentURI;
-    domDocument->GetDocumentURI(documentURI);
-    printf("nsWebBrowser::NS_ACTIVATE %p %s\n", (void*)browser,
-           NS_ConvertUTF16toUTF8(documentURI).get());
+  nsCOMPtr<nsIDOMDocument> domDocument = do_GetInterface(mDocShell);
+  nsAutoString documentURI;
+  domDocument->GetDocumentURI(documentURI);
+  printf("nsWebBrowser::NS_ACTIVATE %p %s\n", (void*)this,
+         NS_ConvertUTF16toUTF8(documentURI).get());
 #endif
-    browser->Activate();
-    break;
-  }
+  Activate();
+}
 
-  case NS_DEACTIVATE: {
+void nsWebBrowser::WindowLowered(nsIWidget* aWidget)
+{
 #if defined(DEBUG_smaug)
-    nsCOMPtr<nsIDOMDocument> domDocument = do_GetInterface(browser->mDocShell);
-    nsAutoString documentURI;
-    domDocument->GetDocumentURI(documentURI);
-    printf("nsWebBrowser::NS_DEACTIVATE %p %s\n", (void*)browser,
-           NS_ConvertUTF16toUTF8(documentURI).get());
+  nsCOMPtr<nsIDOMDocument> domDocument = do_GetInterface(mDocShell);
+  nsAutoString documentURI;
+  domDocument->GetDocumentURI(documentURI);
+  printf("nsWebBrowser::NS_DEACTIVATE %p %s\n", (void*)this,
+         NS_ConvertUTF16toUTF8(documentURI).get());
 #endif
-    browser->Deactivate();
-    break;
+  Deactivate();
+}
+
+bool nsWebBrowser::PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion, uint32_t aFlags)
+{
+  LayerManager* layerManager = aWidget->GetLayerManager();
+  NS_ASSERTION(layerManager, "Must be in paint event");
+
+  layerManager->BeginTransaction();
+  nsRefPtr<ThebesLayer> root = layerManager->CreateThebesLayer();
+  if (root) {
+    nsIntRect dirtyRect = aRegion.GetBounds();
+    root->SetVisibleRegion(dirtyRect);
+    layerManager->SetRoot(root);
   }
 
-  default:
-    break;
-  }
-
-  return nsEventStatus_eIgnore;
+  layerManager->EndTransaction(DrawThebesLayer, &mBackgroundColor);
+  return true;
 }
 
 NS_IMETHODIMP nsWebBrowser::GetPrimaryContentWindow(nsIDOMWindow** aDOMWindow)
@@ -1778,7 +1775,7 @@ NS_IMETHODIMP nsWebBrowser::SetFocusAtLastElement(void)
 NS_IMETHODIMP nsWebBrowser::GetFocusedWindow(nsIDOMWindow * *aFocusedWindow)
 {
   NS_ENSURE_ARG_POINTER(aFocusedWindow);
-  *aFocusedWindow = nsnull;
+  *aFocusedWindow = nullptr;
 
   nsCOMPtr<nsIDOMWindow> window = do_GetInterface(mDocShell);
   NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
@@ -1804,7 +1801,7 @@ NS_IMETHODIMP nsWebBrowser::GetFocusedElement(nsIDOMElement * *aFocusedElement)
   NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
-  return fm ? fm->GetFocusedElementForWindow(window, true, nsnull, aFocusedElement) : NS_OK;
+  return fm ? fm->GetFocusedElementForWindow(window, true, nullptr, aFocusedElement) : NS_OK;
 }
 
 NS_IMETHODIMP nsWebBrowser::SetFocusedElement(nsIDOMElement * aFocusedElement)
@@ -1839,7 +1836,7 @@ NS_IMETHODIMP nsWebBrowser::OpenStream(nsIURI *aBaseURI, const nsACString& aCont
 
 /* void appendToStream([const, array, size_is(aLen)] in octet aData,
  * in unsigned long aLen); */
-NS_IMETHODIMP nsWebBrowser::AppendToStream(const PRUint8 *aData, PRUint32 aLen)
+NS_IMETHODIMP nsWebBrowser::AppendToStream(const uint8_t *aData, uint32_t aLen)
 {
   if (!mStream)
     return NS_ERROR_FAILURE;
