@@ -128,7 +128,6 @@ function initMenus()
         [
          ["join"],
          // Planned future menu items, not implemented yet.
-         //["attach"],
          //["-"],
          //["manage-networks"],
          ["-"],
@@ -411,6 +410,8 @@ function initMenus()
          ["disconnect",  {visibleif: NetConnected}],
          ["reconnect",   {visibleif: NetDisconnected}],
          ["-"],
+         ["rename"],
+         ["-"],
          ["toggle-text-dir"]
         ]
     };
