@@ -100,21 +100,26 @@ function initPrefs()
          ["aliases",            [],       "lists.aliases"],
          ["autoAwayCap",        300,      "global"],
          ["autoAwayPeriod",     2,        "appearance.misc"],
+         ["autoperform.channel", [],      "lists.autoperform"],
+         ["autoperform.client",  [],      "lists.autoperform"],
+         ["autoperform.network", [],      "lists.autoperform"],
+         ["autoperform.user",   ["whois"], "lists.autoperform"],
          ["autoRejoin",         false,    ".connect"],
          ["away",               "",       "hidden"],
-         ["awayNick",           "",       ".ident"],
-         ["awayIdleTime",       0,        ".ident"],
          ["awayIdleMsg",        "",       ".ident"],
+         ["awayIdleTime",       0,        ".ident"],
+         ["awayNick",           "",       ".ident"],
          ["bugURL",           "https://bugzilla.mozilla.org/show_bug.cgi?id=%s",
                                           "appearance.misc"],
          ["bugURL.comment",     "#c%s",   "appearance.misc"],
-         ["ceip.userid",        "",       "hidden"],
-         ["ceip.uploadUrl",     "http://silver.warwickcompsoc.co.uk/mozilla/chatzilla/ceip/1/upload", "hidden"],
-         ["ceip.uploadSize",    32 * 1024, "hidden"],
          ["ceip.log.client",    false,    "hidden"],
          ["ceip.log.command",   false,    "hidden"],
-         ["ceip.log.menu",      false,    "hidden"],
          ["ceip.log.dialog",    false,    "hidden"],
+         ["ceip.log.menu",      false,    "hidden"],
+         ["ceip.uploadSize",    32 * 1024, "hidden"],
+         ["ceip.uploadUrl",     "http://silver.warwickcompsoc.co.uk/mozilla/chatzilla/ceip/1/upload",
+                                          "hidden"],
+         ["ceip.userid",        "",       "hidden"],
          ["channelHeader",      true,     "global.header"],
          ["channelLog",         false,    "global.log"],
          ["channelMaxLines",    500,      "global.maxLines"],
@@ -125,51 +130,56 @@ function initPrefs()
          ["conference.limit",   150,      "appearance.misc"],
          ["connectTries",       -1,       ".connect"],
          ["copyMessages",       true,     "global"],
+         ["dcc.autoAccept.delay", 10000,  "hidden"],
+         ["dcc.downloadsFolder", getURLSpecFromFile(downloadsPath.path),
+                                          "dcc"],
+         ["dcc.enabled",        true,     "dcc"],
+         ["dcc.listenPorts",    [],       "dcc.ports"],
+         ["dcc.useServerIP",    true,     "dcc"],
          ["dccUserHeader",      true,     "global.header"],
          ["dccUserLog",         false,    "global.log"],
          ["dccUserMaxLines",    500,      "global.maxLines"],
-         ["dcc.enabled",        true,     "dcc"],
-         ["dcc.autoAccept.delay", 10000,  "hidden"],
-         ["dcc.downloadsFolder", getURLSpecFromFile(downloadsPath.path), "dcc"],
-         ["dcc.listenPorts",    [],       "dcc.ports"],
-         ["dcc.useServerIP",    true,     "dcc"],
          ["debugMode",          "",       "hidden"],
          ["defaultQuitMsg",     "",       ".connect"],
-         ["desc",               "New Now Know How", ".ident"],
          ["deleteOnPart",       true,     "global"],
+         ["desc",               "New Now Know How", ".ident"],
          ["displayHeader",      true,     "appearance.misc"],
-         ["guessCommands",      true,     "hidden"],
-         ["hasPrefs",           false,    "hidden"],
          ["font.family",        "default", "appearance.misc"],
          ["font.size",          0,        "appearance.misc"],
+         ["guessCommands",      true,     "hidden"],
+         ["hasPrefs",           false,    "hidden"],
          ["identd.enabled",     false,    client.prefManager.identGroup],
+         ["initialScripts",     ["scripts/"], "startup.initialScripts"],
          ["initialURLs",        [],       "startup.initialURLs"],
-         ["initialScripts",     ["scripts/"],
-                                          "startup.initialScripts"],
-         ["instrumentation.key", 0,       "hidden"],
+         ["inputSpellcheck",     true,    "global"],
          ["instrumentation.ceip", false,  "hidden"],
+         ["instrumentation.key", 0,       "hidden"],
          ["link.focus",         true,     "global.links"],
-         ["log",                false,                                  ".log"],
-         ["logFileName",        makeLogNameClient,                    "hidden"],
-         ["logFile.client",     "client.$y-$m-$d.log",                "hidden"],
-         ["logFile.network",    "$(network)/$(network).$y-$m-$d.log", "hidden"],
+         ["log",                false,
+                                          ".log"],
          ["logFile.channel",    "$(network)/channels/$(channel).$y-$m-$d.log",
-                                                                      "hidden"],
+                                          "hidden"],
+         ["logFile.client",     "client.$y-$m-$d.log",
+                                          "hidden"],
+         ["logFile.dccuser",    "dcc/$(user)/$(user).$y-$m-$d.log",
+                                          "hidden"],
+         ["logFile.network",    "$(network)/$(network).$y-$m-$d.log",
+                                          "hidden"],
          ["logFile.user",       "$(network)/users/$(user).$y-$m-$d.log",
-                                                                      "hidden"],
-         ["logFile.dccuser",    "dcc/$(user)/$(user).$y-$m-$d.log",   "hidden"],
+                                          "hidden"],
+         ["logFileName",        makeLogNameClient,
+                                          "hidden"],
          ["logFolder",          getURLSpecFromFile(logPath.path), ".log"],
          ["messages.click",     gotos[0],   "global.links"],
          ["messages.ctrlClick", gotos[1],   "global.links"],
          ["messages.metaClick", gotos[2],   "global.links"],
          ["messages.middleClick", gotos[3], "global.links"],
+         ["motif.current",      "chrome://chatzilla/skin/output-light.css",
+                                          "appearance.motif"],
          ["motif.dark",         "chrome://chatzilla/skin/output-dark.css",
                                           "appearance.motif"],
          ["motif.light",        "chrome://chatzilla/skin/output-light.css",
                                           "appearance.motif"],
-         ["motif.current",      "chrome://chatzilla/skin/output-light.css",
-                                          "appearance.motif"],
-         //["msgBeep",            "beep beep", "global.sounds"],
          ["multiline",          false,    "hidden"],
          ["munger.bold",        true,     "munger"],
          ["munger.bugzilla-link", true,   "munger"],
@@ -190,47 +200,43 @@ function initPrefs()
          ["networkLog",         false,    "global.log"],
          ["networkMaxLines",    200,      "global.maxLines"],
          ["newTabLimit",        30,       "global"],
-         ["notify.aggressive",  true,     "global"],
          ["nickCompleteStr",    ":",      "global"],
          ["nickname",           defaultNick, ".ident"],
          ["nicknameList",       [],       "lists.nicknameList"],
+         ["notify.aggressive",  true,     "global"],
          ["outgoing.colorCodes",  true,   "global"],
          ["outputWindowURL",   "chrome://chatzilla/content/output-window.html",
                                           "hidden"],
          ["proxy.typeOverride", "",       ".connect"],
-         ["sortUsersByMode",    true,     "appearance.userlist"],
-         //["queryBeep",          "beep",   "global.sounds"],
          ["reconnect",          true,     ".connect"],
          ["showModeSymbols",    false,    "appearance.userlist"],
-         //["stalkBeep",          "beep",   "global.sounds"],
-         ["stalkWholeWords",    true,     "lists.stalkWords"],
-         ["stalkWords",         [],       "lists.stalkWords"],
-         // Start == When view it opened.
-         // Event == "Superfluous" activity.
+         ["sortUsersByMode",    true,     "appearance.userlist"],
          // Chat  == "Activity" activity.
+         // Event == "Superfluous" activity.
          // Stalk == "Attention" activity.
+         // Start == When view it opened.
+         ["sound.channel.chat",  "",          "global.soundEvts"],
+         ["sound.channel.event", "",          "global.soundEvts"],
+         ["sound.channel.stalk", "beep",      "global.soundEvts"],
+         ["sound.channel.start", "",          "global.soundEvts"],
          ["sound.enabled",       true,        "global.sounds"],
          ["sound.overlapDelay",  2000,        "global.sounds"],
-         //["sound.surpressActive",false,       "global.sounds"],
-         ["sound.channel.start", "",          "global.soundEvts"],
-         ["sound.channel.event", "",          "global.soundEvts"],
-         ["sound.channel.chat",  "",          "global.soundEvts"],
-         ["sound.channel.stalk", "beep",      "global.soundEvts"],
-         ["sound.user.start",    "beep beep", "global.soundEvts"],
          ["sound.user.stalk",    "beep",      "global.soundEvts"],
-         ["inputSpellcheck",     true,                  "global"],
-         ["tabGotoKeyModifiers", 0,           "hidden"],
-         ["timestamps",         false,        "appearance.timestamps"],
-         ["timestamps.display", "[%H:%M]",    "appearance.timestamps"],
+         ["sound.user.start",    "beep beep", "global.soundEvts"],
+         ["stalkWholeWords",    true,     "lists.stalkWords"],
+         ["stalkWords",         [],       "lists.stalkWords"],
+         ["tabGotoKeyModifiers", 0,       "hidden"],
+         ["timestamps",         false,    "appearance.timestamps"],
+         ["timestamps.display", "[%H:%M]", "appearance.timestamps"],
          ["timestamps.log",     "[%Y-%m-%d %H:%M:%S]", "hidden"],
-         ["urls.store.max",     100,      "global"],
          ["urls.display",       10,       "hidden"],
-         ["username",           "chatzilla", ".ident"],
-         ["usermode",           "+i",     ".ident"],
+         ["urls.store.max",     100,      "global"],
          ["userHeader",         true,     "global.header"],
          ["userlistLeft",       true,     "appearance.userlist"],
          ["userLog",            false,    "global.log"],
          ["userMaxLines",       200,      "global.maxLines"],
+         ["usermode",           "+i",     ".ident"],
+         ["username",           "chatzilla", ".ident"],
          ["warnOnClose",        true,     "global"]
         ];
 
@@ -393,6 +399,7 @@ function getNetworkPrefManager(network)
     var prefs =
         [
          ["autoAwayPeriod",   defer, "appearance.misc"],
+         ["autoperform",      [],    "lists.autoperform"],
          ["autoRejoin",       defer, ".connect"],
          ["away",             defer, "hidden"],
          ["awayNick",         defer, ".ident"],
@@ -403,13 +410,12 @@ function getNetworkPrefManager(network)
          ["collapseMsgs",     defer, "appearance.misc"],
          ["conference.limit", defer, "appearance.misc"],
          ["connectTries",     defer, ".connect"],
-         ["dcc.useServerIP",  defer, "dcc"],
-         ["dcc.downloadsFolder", defer, "dcc"],
          ["dcc.autoAccept.list", [], "dcc.autoAccept"],
+         ["dcc.downloadsFolder", defer, "dcc"],
+         ["dcc.useServerIP",  defer, "dcc"],
          ["defaultQuitMsg",   defer, ".connect"],
          ["desc",             defer, ".ident"],
-         ["displayHeader",    client.prefs["networkHeader"],
-                                                             "appearance.misc"],
+         ["displayHeader",    client.prefs["networkHeader"], "appearance.misc"],
          ["font.family",      defer, "appearance.misc"],
          ["font.size",        defer, "appearance.misc"],
          ["hasPrefs",         false, "hidden"],
@@ -427,9 +433,8 @@ function getNetworkPrefManager(network)
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
          ["timestamps.log",     defer, "hidden"],
-         ["username",         defer, ".ident"],
          ["usermode",         defer, ".ident"],
-         ["autoperform",      [],    "lists.autoperform"]
+         ["username",         defer, ".ident"]
         ];
 
     var branch = "extensions.irc.networks." + pref_mungeName(network.encodedName) +
@@ -494,6 +499,7 @@ function getChannelPrefManager(channel)
 
     var prefs =
         [
+         ["autoperform",      [],    "lists.autoperform"],
          ["autoRejoin",       defer, ".connect"],
          ["bugURL",           defer, "appearance.misc"],
          ["bugURL.comment",   defer, "appearance.misc"],
@@ -502,18 +508,17 @@ function getChannelPrefManager(channel)
          ["collapseMsgs",     defer, "appearance.misc"],
          ["conference.enabled", false, "hidden"],
          ["conference.limit", defer, "appearance.misc"],
-         ["displayHeader",    client.prefs["channelHeader"],
-                                                             "appearance.misc"],
+         ["displayHeader",    client.prefs["channelHeader"], "appearance.misc"],
          ["font.family",      defer, "appearance.misc"],
          ["font.size",        defer, "appearance.misc"],
          ["hasPrefs",         false, "hidden"],
          ["log",              client.prefs["channelLog"], ".log"],
          ["logFileName",      makeLogNameChannel, "hidden"],
          ["motif.current",    defer, "appearance.motif"],
+         ["outputWindowURL",  defer, "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
-         ["timestamps.log",     defer, "hidden"],
-         ["outputWindowURL",  defer, "hidden"]
+         ["timestamps.log",     defer, "hidden"]
         ];
 
     var branch = "extensions.irc.networks." + pref_mungeName(network.encodedName) +
@@ -554,6 +559,7 @@ function getUserPrefManager(user)
 
     var prefs =
         [
+         ["autoperform",      [],    "lists.autoperform"],
          ["charset",          defer, ".connect"],
          ["collapseActions",  defer, "appearance.misc"],
          ["collapseMsgs",     defer, "appearance.misc"],
@@ -561,10 +567,10 @@ function getUserPrefManager(user)
          ["font.family",      defer, "appearance.misc"],
          ["font.size",        defer, "appearance.misc"],
          ["hasPrefs",         false, "hidden"],
-         ["motif.current",    defer, "appearance.motif"],
-         ["outputWindowURL",  defer, "hidden"],
          ["log",              client.prefs["userLog"], ".log"],
          ["logFileName",      makeLogNameUser, "hidden"],
+         ["motif.current",    defer, "appearance.motif"],
+         ["outputWindowURL",  defer, "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
          ["timestamps.log",     defer, "hidden"]
@@ -607,10 +613,10 @@ function getDCCUserPrefManager(user)
          ["font.family",      defer, "appearance.misc"],
          ["font.size",        defer, "appearance.misc"],
          ["hasPrefs",         false, "hidden"],
-         ["motif.current",    defer, "appearance.motif"],
-         ["outputWindowURL",  defer, "hidden"],
          ["log",              client.prefs["dccUserLog"], ".log"],
          ["logFileName",      makeLogNameUser, "hidden"],
+         ["motif.current",    defer, "appearance.motif"],
+         ["outputWindowURL",  defer, "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
          ["timestamps.log",     defer, "hidden"]
