@@ -29,12 +29,6 @@
 namespace mozilla {
 namespace gfx {
 
-SkColor ColorToSkColor(const Color &color, Float aAlpha)
-{
-  //XXX: do a better job converting to int
-  return SkColorSetARGB(color.a*aAlpha*255.0, color.r*255.0, color.g*255.0, color.b*255.0);
-}
-
 class GradientStopsSkia : public GradientStops
 {
 public:
@@ -80,67 +74,6 @@ public:
   ExtendMode mExtendMode;
 };
 
-SkXfermode::Mode
-GfxOpToSkiaOp(CompositionOp op)
-{
-  switch (op)
-  {
-    case OP_OVER:
-      return SkXfermode::kSrcOver_Mode;
-    case OP_ADD:
-      return SkXfermode::kPlus_Mode;
-    case OP_ATOP:
-      return SkXfermode::kSrcATop_Mode;
-    case OP_OUT:
-      return SkXfermode::kSrcOut_Mode;
-    case OP_IN:
-      return SkXfermode::kSrcIn_Mode;
-    case OP_SOURCE:
-      return SkXfermode::kSrc_Mode;
-    case OP_DEST_IN:
-      return SkXfermode::kDstIn_Mode;
-    case OP_DEST_OUT:
-      return SkXfermode::kDstOut_Mode;
-    case OP_DEST_OVER:
-      return SkXfermode::kDstOver_Mode;
-    case OP_DEST_ATOP:
-      return SkXfermode::kDstATop_Mode;
-    case OP_XOR:
-      return SkXfermode::kXor_Mode;
-    case OP_COUNT:
-      return SkXfermode::kSrcOver_Mode;
-  }
-  return SkXfermode::kSrcOver_Mode;
-}
-
-
-SkRect
-RectToSkRect(const Rect& aRect)
-{
-  return SkRect::MakeXYWH(SkFloatToScalar(aRect.x), SkFloatToScalar(aRect.y), 
-                          SkFloatToScalar(aRect.width), SkFloatToScalar(aRect.height));
-}
-
-SkRect
-IntRectToSkRect(const IntRect& aRect)
-{
-  return SkRect::MakeXYWH(SkIntToScalar(aRect.x), SkIntToScalar(aRect.y), 
-                          SkIntToScalar(aRect.width), SkIntToScalar(aRect.height));
-}
-
-SkIRect
-RectToSkIRect(const Rect& aRect)
-{
-  return SkIRect::MakeXYWH(aRect.x, aRect.y, aRect.width, aRect.height);
-}
-
-SkIRect
-IntRectToSkIRect(const IntRect& aRect)
-{
-  return SkIRect::MakeXYWH(aRect.x, aRect.y, aRect.width, aRect.height);
-}
-
-
 DrawTargetSkia::DrawTargetSkia()
 {
 }
@@ -162,27 +95,12 @@ DrawTargetSkia::Snapshot()
 {
   RefPtr<SourceSurfaceSkia> source = new SourceSurfaceSkia();
   if (!source->InitWithBitmap(mBitmap, mFormat, this)) {
-    return NULL;
+    return nullptr;
   }
   AppendSnapshot(source);
   return source;
 }
 
-SkShader::TileMode
-ExtendModeToTileMode(ExtendMode aMode)
-{
-  switch (aMode)
-  {
-    case EXTEND_CLAMP:
-      return SkShader::kClamp_TileMode;
-    case EXTEND_REPEAT:
-      return SkShader::kRepeat_TileMode;
-    case EXTEND_REFLECT:
-      return SkShader::kMirror_TileMode;
-  }
-  return SkShader::kClamp_TileMode;
-}
-
 void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern, Float aAlpha = 1.0)
 {
   switch (aPattern.GetType()) {
@@ -206,10 +124,14 @@ void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern, Float aAlpha = 1.
                                                           &stops->mPositions.front(), 
                                                           stops->mCount, 
                                                           mode);
-        SkMatrix mat;
-        GfxMatrixToSkiaMatrix(pat.mMatrix, mat);
-        shader->setLocalMatrix(mat);
-        SkSafeUnref(aPaint.setShader(shader));
+
+        if (shader) {
+            SkMatrix mat;
+            GfxMatrixToSkiaMatrix(pat.mMatrix, mat);
+            shader->setLocalMatrix(mat);
+            SkSafeUnref(aPaint.setShader(shader));
+        }
+
       } else {
         aPaint.setColor(SkColorSetARGB(0, 0, 0, 0));
       }
@@ -225,18 +147,21 @@ void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern, Float aAlpha = 1.
         points[0] = SkPoint::Make(SkFloatToScalar(pat.mCenter1.x), SkFloatToScalar(pat.mCenter1.y));
         points[1] = SkPoint::Make(SkFloatToScalar(pat.mCenter2.x), SkFloatToScalar(pat.mCenter2.y));
 
-        SkShader* shader = SkGradientShader::CreateTwoPointRadial(points[0], 
-                                                                  SkFloatToScalar(pat.mRadius1),
-                                                                  points[1], 
-                                                                  SkFloatToScalar(pat.mRadius2),
-                                                                  &stops->mColors.front(), 
-                                                                  &stops->mPositions.front(), 
-                                                                  stops->mCount, 
-                                                                  mode);
-        SkMatrix mat;
-        GfxMatrixToSkiaMatrix(pat.mMatrix, mat);
-        shader->setLocalMatrix(mat);
-        SkSafeUnref(aPaint.setShader(shader));
+        SkShader* shader = SkGradientShader::CreateTwoPointConical(points[0], 
+                                                                   SkFloatToScalar(pat.mRadius1),
+                                                                   points[1], 
+                                                                   SkFloatToScalar(pat.mRadius2),
+                                                                   &stops->mColors.front(), 
+                                                                   &stops->mPositions.front(), 
+                                                                   stops->mCount, 
+                                                                   mode);
+        if (shader) {
+            SkMatrix mat;
+            GfxMatrixToSkiaMatrix(pat.mMatrix, mat);
+            shader->setLocalMatrix(mat);
+            SkSafeUnref(aPaint.setShader(shader));
+        }
+
       } else {
         aPaint.setColor(SkColorSetARGB(0, 0, 0, 0));
       }
@@ -302,12 +227,12 @@ struct AutoPaintSetup {
       mPaint.setXfermodeMode(SkXfermode::kSrcOver_Mode);
       SkPaint temp;
       temp.setXfermodeMode(GfxOpToSkiaOp(aOptions.mCompositionOp));
-      temp.setAlpha(aOptions.mAlpha*255);
+      temp.setAlpha(U8CPU(aOptions.mAlpha*255));
       //TODO: Get a rect here
-      mCanvas->saveLayer(NULL, &temp);
+      mCanvas->saveLayer(nullptr, &temp);
       mNeedsRestore = true;
     } else {
-      mPaint.setAlpha(aOptions.mAlpha*255.0);
+      mPaint.setAlpha(U8CPU(aOptions.mAlpha*255.0));
       mAlpha = aOptions.mAlpha;
     }
     mPaint.setFilterBitmap(true);
@@ -418,7 +343,8 @@ DrawTargetSkia::DrawSurfaceWithShadow(SourceSurface *aSurface,
   paint.setXfermodeMode(GfxOpToSkiaOp(aOperator));
   SkSafeUnref(paint.setLooper(dl));
 
-  SkRect rect = RectToSkRect(Rect(aDest.x, aDest.y, bitmap.width(), bitmap.height()));
+  SkRect rect = RectToSkRect(Rect(Float(aDest.x), Float(aDest.y),
+                                  Float(bitmap.width()), Float(bitmap.height())));
   mCanvas->drawRect(rect, paint);
   mCanvas->restore();
 }
@@ -442,6 +368,7 @@ DrawTargetSkia::Stroke(const Path *aPath,
                        const DrawOptions &aOptions)
 {
   MarkChanged();
+  MOZ_ASSERT(aPath, "Null path");
   if (aPath->GetBackendType() != BACKEND_SKIA) {
     return;
   }
@@ -514,7 +441,9 @@ DrawTargetSkia::FillGlyphs(ScaledFont *aFont,
                            const DrawOptions &aOptions,
                            const GlyphRenderingOptions*)
 {
-  if (aFont->GetType() != FONT_MAC && aFont->GetType() != FONT_SKIA) {
+  if (aFont->GetType() != FONT_MAC &&
+      aFont->GetType() != FONT_SKIA &&
+      aFont->GetType() != FONT_GDI) {
     return;
   }
 
@@ -560,7 +489,7 @@ DrawTargetSkia::Mask(const Pattern &aSource,
   // Take our destination bounds and convert them into user space to use
   // as the path to draw.
   SkPath path;
-  path.addRect(SkRect::MakeWH(mSize.width, mSize.height));
+  path.addRect(SkRect::MakeWH(SkScalar(mSize.width), SkScalar(mSize.height)));
  
   Matrix temp = mTransform;
   temp.Invert();
@@ -581,7 +510,7 @@ DrawTargetSkia::CreateSourceSurfaceFromData(unsigned char *aData,
 
   if (!newSurf->InitFromData(aData, aSize, aStride, aFormat)) {
     gfxDebug() << *this << ": Failure to create source surface from data. Size: " << aSize;
-    return NULL;
+    return nullptr;
   }
     
   return newSurf;
@@ -592,7 +521,7 @@ DrawTargetSkia::CreateSimilarDrawTarget(const IntSize &aSize, SurfaceFormat aFor
 {
   RefPtr<DrawTargetSkia> target = new DrawTargetSkia();
   if (!target->Init(aSize, aFormat)) {
-    return NULL;
+    return nullptr;
   }
   return target;
 }
@@ -600,13 +529,13 @@ DrawTargetSkia::CreateSimilarDrawTarget(const IntSize &aSize, SurfaceFormat aFor
 TemporaryRef<SourceSurface>
 DrawTargetSkia::OptimizeSourceSurface(SourceSurface *aSurface) const
 {
-  return NULL;
+  return nullptr;
 }
 
 TemporaryRef<SourceSurface>
 DrawTargetSkia::CreateSourceSurfaceFromNativeSurface(const NativeSurface &aSurface) const
 {
-  return NULL;
+  return nullptr;
 }
 
 void
@@ -630,7 +559,17 @@ DrawTargetSkia::CopySurface(SourceSurface *aSurface,
   SkIRect source = IntRectToSkIRect(aSourceRect);
   mCanvas->clipRect(dest, SkRegion::kReplace_Op);
   SkPaint paint;
-  paint.setXfermodeMode(GfxOpToSkiaOp(OP_SOURCE));
+
+  if (mBitmap.config() == SkBitmap::kRGB_565_Config &&
+      mCanvas->getDevice()->config() == SkBitmap::kRGB_565_Config) {
+    // Set the xfermode to SOURCE_OVER to workaround
+    // http://code.google.com/p/skia/issues/detail?id=628
+    // RGB565 is opaque so they're equivalent anyway
+    paint.setXfermodeMode(SkXfermode::kSrcOver_Mode);
+  } else {
+    paint.setXfermodeMode(SkXfermode::kSrc_Mode);
+  }
+
   mCanvas->drawBitmapRect(bitmap, &source, dest, &paint);
   mCanvas->restore();
 }
@@ -656,6 +595,12 @@ DrawTargetSkia::Init(const IntSize &aSize, SurfaceFormat aFormat)
 void
 DrawTargetSkia::Init(unsigned char* aData, const IntSize &aSize, int32_t aStride, SurfaceFormat aFormat)
 {
+  if (aFormat == FORMAT_B8G8R8X8) {
+    // We have to manually set the A channel to be 255 as Skia doesn't understand BGRX
+    ConvertBGRXToBGRA(aData, aSize, aStride);
+    mBitmap.setIsOpaque(true);
+  }
+
   mBitmap.setConfig(GfxFormatToSkiaConfig(aFormat), aSize.width, aSize.height, aStride);
   mBitmap.setPixels(aData);
   
