@@ -23,7 +23,7 @@ using namespace mozilla;
 
 gfxXlibSurface::gfxXlibSurface(Display *dpy, Drawable drawable, Visual *visual)
     : mPixmapTaken(false), mDisplay(dpy), mDrawable(drawable)
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
     , mGLXPixmap(None)
 #endif
 {
@@ -34,7 +34,7 @@ gfxXlibSurface::gfxXlibSurface(Display *dpy, Drawable drawable, Visual *visual)
 
 gfxXlibSurface::gfxXlibSurface(Display *dpy, Drawable drawable, Visual *visual, const gfxIntSize& size)
     : mPixmapTaken(false), mDisplay(dpy), mDrawable(drawable), mSize(size)
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
     , mGLXPixmap(None)
 #endif
 {
@@ -49,7 +49,7 @@ gfxXlibSurface::gfxXlibSurface(Screen *screen, Drawable drawable, XRenderPictFor
                                const gfxIntSize& size)
     : mPixmapTaken(false), mDisplay(DisplayOfScreen(screen)),
       mDrawable(drawable), mSize(size)
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
       , mGLXPixmap(None)
 #endif
 {
@@ -67,7 +67,7 @@ gfxXlibSurface::gfxXlibSurface(cairo_surface_t *csurf)
     : mPixmapTaken(false),
       mSize(cairo_xlib_surface_get_width(csurf),
             cairo_xlib_surface_get_height(csurf))
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
       , mGLXPixmap(None)
 #endif
 {
@@ -82,9 +82,9 @@ gfxXlibSurface::gfxXlibSurface(cairo_surface_t *csurf)
 
 gfxXlibSurface::~gfxXlibSurface()
 {
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
     if (mGLXPixmap) {
-        gl::sGLXLibrary.DestroyPixmap(mGLXPixmap);
+        gl::sDefGLXLib.DestroyPixmap(mGLXPixmap);
     }
 #endif
     // gfxASurface's destructor calls RecordMemoryFreed().
@@ -140,14 +140,14 @@ gfxXlibSurface::Create(Screen *screen, Visual *visual,
         CreatePixmap(screen, size, DepthOfVisual(screen, visual),
                      relatedDrawable);
     if (!drawable)
-        return nsnull;
+        return nullptr;
 
     nsRefPtr<gfxXlibSurface> result =
         new gfxXlibSurface(DisplayOfScreen(screen), drawable, visual, size);
     result->TakePixmap();
 
     if (result->CairoStatus() != 0)
-        return nsnull;
+        return nullptr;
 
     return result.forget();
 }
@@ -160,14 +160,14 @@ gfxXlibSurface::Create(Screen *screen, XRenderPictFormat *format,
     Drawable drawable =
         CreatePixmap(screen, size, format->depth, relatedDrawable);
     if (!drawable)
-        return nsnull;
+        return nullptr;
 
     nsRefPtr<gfxXlibSurface> result =
         new gfxXlibSurface(screen, drawable, format, size);
     result->TakePixmap();
 
     if (result->CairoStatus() != 0)
-        return nsnull;
+        return nullptr;
 
     return result.forget();
 }
@@ -182,7 +182,7 @@ gfxXlibSurface::CreateSimilarSurface(gfxContentType aContent,
                                      const gfxIntSize& aSize)
 {
     if (!mSurface || !mSurfaceValid) {
-      return nsnull;
+      return nullptr;
     }
 
     if (aContent == CONTENT_COLOR) {
@@ -214,6 +214,18 @@ gfxXlibSurface::CreateSimilarSurface(gfxContentType aContent,
     return gfxASurface::CreateSimilarSurface(aContent, aSize);
 }
 
+void
+gfxXlibSurface::Finish()
+{
+#if defined(GL_PROVIDER_GLX)
+    if (mGLXPixmap) {
+        gl::sDefGLXLib.DestroyPixmap(mGLXPixmap);
+        mGLXPixmap = None;
+    }
+#endif
+    gfxASurface::Finish();
+}
+
 void
 gfxXlibSurface::DoSizeQuery()
 {
@@ -325,7 +337,7 @@ DisplayTable::GetColormapAndVisual(Screen* aScreen, XRenderPictFormat* aFormat,
     }
 
     nsTArray<DisplayInfo>* displays = &sDisplayTable->mDisplays;
-    PRUint32 d = displays->IndexOf(display, 0, FindDisplay());
+    uint32_t d = displays->IndexOf(display, 0, FindDisplay());
 
     if (d == displays->NoIndex) {
         d = displays->Length();
@@ -345,7 +357,7 @@ DisplayTable::GetColormapAndVisual(Screen* aScreen, XRenderPictFormat* aFormat,
 
     // Only a small number of formats are expected to be used, so just do a
     // simple linear search.
-    for (PRUint32 i = 0; i < entries->Length(); ++i) {
+    for (uint32_t i = 0; i < entries->Length(); ++i) {
         const ColormapEntry& entry = entries->ElementAt(i);
         // Only the format and screen need to match.  (The visual may differ.)
         // If there is no format (e.g. no RENDER extension) then just compare
@@ -380,7 +392,7 @@ DisplayTable::DisplayClosing(Display *display, XExtCodes* codes)
     sDisplayTable->mDisplays.RemoveElement(display, FindDisplay());
     if (sDisplayTable->mDisplays.Length() == 0) {
         delete sDisplayTable;
-        sDisplayTable = nsnull;
+        sDisplayTable = nullptr;
     }
     return 0;
 }
@@ -506,12 +518,20 @@ gfxXlibSurface::XRenderFormat()
     return cairo_xlib_surface_get_xrender_format(CairoSurface());
 }
 
-#if defined(MOZ_WIDGET_GTK2) && !defined(MOZ_PLATFORM_MAEMO)
+#if defined(GL_PROVIDER_GLX)
 GLXPixmap
 gfxXlibSurface::GetGLXPixmap()
 {
     if (!mGLXPixmap) {
-        mGLXPixmap = gl::sGLXLibrary.CreatePixmap(this);
+#ifdef DEBUG
+        // cairo_surface_has_show_text_glyphs is used solely for the
+        // side-effect of setting the error on surface if
+        // cairo_surface_finish() has been called.
+        cairo_surface_has_show_text_glyphs(CairoSurface());
+        NS_ASSERTION(CairoStatus() != CAIRO_STATUS_SURFACE_FINISHED,
+            "GetGLXPixmap called after surface finished");
+#endif
+        mGLXPixmap = gl::sDefGLXLib.CreatePixmap(this);
     }
     return mGLXPixmap;
 }
