@@ -8,10 +8,10 @@
 
 #include "prtypes.h"
 
-extern const PRUint16 g_ufBig5Mapping[];
-extern const PRUint16 g_utBIG5Mapping[];
-extern const PRUint16 g_ASCIIMapping[];
-extern const PRUint16 g_ufBig5HKSCSMapping[];
-extern const PRUint16 g_utBig5HKSCSMapping[];
+extern const uint16_t g_ufBig5Mapping[];
+extern const uint16_t g_utBIG5Mapping[];
+extern const uint16_t g_ASCIIMappingTable[];
+extern const uint16_t g_ufBig5HKSCSMapping[];
+extern const uint16_t g_utBig5HKSCSMapping[];
 
 #endif /* nsUCvTWDll_h___ */
