@@ -23,6 +23,7 @@
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
 #include "nsCycleCollectionParticipant.h"
+#include "mozilla/Attributes.h"
 
 /* XXX DOM dependency */
 #include "nsIScriptContext.h"
@@ -89,11 +90,11 @@ const char jsdARObserverCtrID[] = "@mozilla.org/js/jsd/app-start-observer;2";
 const char jsdASObserverCtrID[] = "service,@mozilla.org/js/jsd/app-start-observer;2";
 
 #ifdef DEBUG_verbose
-PRUint32 gScriptCount   = 0;
-PRUint32 gValueCount    = 0;
-PRUint32 gPropertyCount = 0;
-PRUint32 gContextCount  = 0;
-PRUint32 gFrameCount  = 0;
+uint32_t gScriptCount   = 0;
+uint32_t gValueCount    = 0;
+uint32_t gPropertyCount = 0;
+uint32_t gContextCount  = 0;
+uint32_t gFrameCount  = 0;
 #endif
 
 static jsdService          *gJsds               = 0;
@@ -104,7 +105,7 @@ static struct DeadScript {
     PRCList     links;
     JSDContext *jsdc;
     jsdIScript *script;
-} *gDeadScripts = nsnull;
+} *gDeadScripts = nullptr;
 
 enum PatternType {
     ptIgnore     = 0U,
@@ -120,14 +121,14 @@ static struct FilterRecord {
     void        *glob;
     nsCString    urlPattern;
     PatternType  patternType;
-    PRUint32     startLine;
-    PRUint32     endLine;
-} *gFilters = nsnull;
+    uint32_t     startLine;
+    uint32_t     endLine;
+} *gFilters = nullptr;
 
-static struct LiveEphemeral *gLiveValues      = nsnull;
-static struct LiveEphemeral *gLiveProperties  = nsnull;
-static struct LiveEphemeral *gLiveContexts    = nsnull;
-static struct LiveEphemeral *gLiveStackFrames = nsnull;
+static struct LiveEphemeral *gLiveValues      = nullptr;
+static struct LiveEphemeral *gLiveProperties  = nullptr;
+static struct LiveEphemeral *gLiveContexts    = nullptr;
+static struct LiveEphemeral *gLiveStackFrames = nullptr;
 
 /*******************************************************************************
  * utility functions for ephemeral lists
@@ -136,7 +137,7 @@ already_AddRefed<jsdIEphemeral>
 jsds_FindEphemeral (LiveEphemeral **listHead, void *key)
 {
     if (!*listHead)
-        return nsnull;
+        return nullptr;
     
     LiveEphemeral *lv_record = 
         reinterpret_cast<LiveEphemeral *>
@@ -153,7 +154,7 @@ jsds_FindEphemeral (LiveEphemeral **listHead, void *key)
     }
     while (lv_record != *listHead);
 
-    return nsnull;
+    return nullptr;
 }
 
 void
@@ -198,7 +199,7 @@ jsds_RemoveEphemeral (LiveEphemeral **listHead, LiveEphemeral *item)
          * null out the list head */
         NS_ASSERTION (*listHead == item,
                       "How could we not be the head of a one item list?");
-        *listHead = nsnull;
+        *listHead = nullptr;
     }
     else if (item == *listHead)
     {
@@ -228,7 +229,7 @@ jsds_SyncFilter (FilterRecord *rec, jsdIFilter *filter)
     NS_ASSERTION (rec, "jsds_SyncFilter without rec");
     NS_ASSERTION (filter, "jsds_SyncFilter without filter");
     
-    JSObject *glob_proper = nsnull;
+    JSObject *glob_proper = nullptr;
     nsCOMPtr<nsISupports> glob;
     nsresult rv = filter->GetGlobalObject(getter_AddRefs(glob));
     if (NS_FAILED(rv))
@@ -239,22 +240,22 @@ jsds_SyncFilter (FilterRecord *rec, jsdIFilter *filter)
             glob_proper = nsiglob->GetGlobalJSObject();
     }
     
-    PRUint32 startLine;
+    uint32_t startLine;
     rv = filter->GetStartLine(&startLine);
     if (NS_FAILED(rv))
         return false;
 
-    PRUint32 endLine;
+    uint32_t endLine;
     rv = filter->GetStartLine(&endLine);
     if (NS_FAILED(rv))
         return false;    
 
-    nsCAutoString urlPattern;
+    nsAutoCString urlPattern;
     rv = filter->GetUrlPattern (urlPattern);
     if (NS_FAILED(rv))
         return false;
     
-    PRUint32 len = urlPattern.Length();
+    uint32_t len = urlPattern.Length();
     if (len) {
         if (urlPattern[0] == '*') {
             /* pattern starts with a *, shift all chars once to the left,
@@ -307,7 +308,7 @@ FilterRecord *
 jsds_FindFilter (jsdIFilter *filter)
 {
     if (!gFilters)
-        return nsnull;
+        return nullptr;
     
     FilterRecord *current = gFilters;
     
@@ -318,7 +319,7 @@ jsds_FindFilter (jsdIFilter *filter)
                                   (PR_NEXT_LINK(&current->links));
     } while (current != gFilters);
     
-    return nsnull;
+    return nullptr;
 }
 
 /* returns true if the hook should be executed. */
@@ -355,11 +356,11 @@ jsds_FilterHook (JSDContext *jsdc, JSDThreadState *state)
     if (!gFilters)
         return true;    
 
-    PRUint32 currentLine = JSD_GetClosestLine (jsdc, script, pc);
-    PRUint32 len = 0;
+    uint32_t currentLine = JSD_GetClosestLine (jsdc, script, pc);
+    uint32_t len = 0;
     FilterRecord *currentFilter = gFilters;
     do {
-        PRUint32 flags = 0;
+        uint32_t flags = 0;
 
 #ifdef DEBUG
         nsresult rv =
@@ -385,7 +386,7 @@ jsds_FilterHook (JSDContext *jsdc, JSDThreadState *state)
                 if (!len)
                     len = url.Length();
                 nsCString urlPattern = currentFilter->urlPattern;
-                PRUint32 patternLength = urlPattern.Length();
+                uint32_t patternLength = urlPattern.Length();
                 if (len >= patternLength) {
                     switch (currentFilter->patternType) {
                         case ptEquals:
@@ -436,18 +437,18 @@ jsds_NotifyPendingDeadScripts (JSRuntime *rt)
     if (jsds) {
         NS_ADDREF(jsds);
         jsds->GetScriptHook (getter_AddRefs(hook));
-        jsds->DoPause(nsnull, true);
+        jsds->DoPause(nullptr, true);
     }
 
     DeadScript *deadScripts = gDeadScripts;
-    gDeadScripts = nsnull;
+    gDeadScripts = nullptr;
     while (deadScripts) {
         DeadScript *ds = deadScripts;
         /* get next deleted script */
         deadScripts = reinterpret_cast<DeadScript *>
                                        (PR_NEXT_LINK(&ds->links));
         if (deadScripts == ds)
-            deadScripts = nsnull;
+            deadScripts = nullptr;
 
         if (hook)
         {
@@ -471,7 +472,7 @@ jsds_NotifyPendingDeadScripts (JSRuntime *rt)
     }
 
     if (jsds) {
-        jsds->DoUnPause(nsnull, true);
+        jsds->DoUnPause(nullptr, true);
         NS_RELEASE(jsds);
     }
 }
@@ -519,11 +520,11 @@ jsds_ErrorHookProc (JSDContext *jsdc, JSContext *cx, const char *message,
         val = getter_AddRefs(jsdValue::FromPtr(jsdc, jsdv));
     }
     
-    nsCAutoString fileName;
-    PRUint32    line;
-    PRUint32    pos;
-    PRUint32    flags;
-    PRUint32    errnum;
+    nsAutoCString fileName;
+    uint32_t    line;
+    uint32_t    pos;
+    uint32_t    flags;
+    uint32_t    errnum;
     bool        rval;
     if (report) {
         fileName.Assign(report->filename);
@@ -540,9 +541,9 @@ jsds_ErrorHookProc (JSDContext *jsdc, JSContext *cx, const char *message,
         errnum   = 0;
     }
     
-    gJsds->DoPause(nsnull, true);
+    gJsds->DoPause(nullptr, true);
     hook->OnError (nsDependentCString(message), fileName, line, pos, flags, errnum, val, &rval);
-    gJsds->DoUnPause(nsnull, true);
+    gJsds->DoUnPause(nullptr, true);
     
     running = false;
     if (!rval)
@@ -583,20 +584,20 @@ jsds_CallHookProc (JSDContext* jsdc, JSDThreadState* jsdthreadstate,
     nsCOMPtr<jsdIStackFrame> frame =
         getter_AddRefs(jsdStackFrame::FromPtr(jsdc, jsdthreadstate,
                                               native_frame));
-    gJsds->DoPause(nsnull, true);
+    gJsds->DoPause(nullptr, true);
     hook->OnCall(frame, type);    
-    gJsds->DoUnPause(nsnull, true);
+    gJsds->DoUnPause(nullptr, true);
     jsdStackFrame::InvalidateAll();
 
     return JS_TRUE;
 }
 
-static PRUint32
+static uint32_t
 jsds_ExecutionHookProc (JSDContext* jsdc, JSDThreadState* jsdthreadstate,
                         unsigned type, void* callerdata, jsval* rval)
 {
     nsCOMPtr<jsdIExecutionHook> hook(0);
-    PRUint32 hook_rv = JSD_HOOK_RETURN_CONTINUE;
+    uint32_t hook_rv = JSD_HOOK_RETURN_CONTINUE;
     nsCOMPtr<jsdIValue> js_rv;
 
     switch (type)
@@ -615,7 +616,7 @@ jsds_ExecutionHookProc (JSDContext* jsdc, JSDThreadState* jsdthreadstate,
                 /* we can't pause breakpoints the way we pause the other
                  * execution hooks (at least, not easily.)  Instead we bail
                  * here if the service is paused. */
-                PRUint32 level;
+                uint32_t level;
                 gJsds->GetPauseDepth(&level);
                 if (!level)
                     gJsds->GetBreakpointHook(getter_AddRefs(hook));
@@ -645,13 +646,13 @@ jsds_ExecutionHookProc (JSDContext* jsdc, JSDThreadState* jsdthreadstate,
     nsCOMPtr<jsdIStackFrame> frame =
         getter_AddRefs(jsdStackFrame::FromPtr(jsdc, jsdthreadstate,
                                               native_frame));
-    gJsds->DoPause(nsnull, true);
+    gJsds->DoPause(nullptr, true);
     jsdIValue *inout_rv = js_rv;
     NS_IF_ADDREF(inout_rv);
     hook->OnExecute (frame, type, &inout_rv, &hook_rv);
     js_rv = inout_rv;
     NS_IF_RELEASE(inout_rv);
-    gJsds->DoUnPause(nsnull, true);
+    gJsds->DoUnPause(nullptr, true);
     jsdStackFrame::InvalidateAll();
         
     if (hook_rv == JSD_HOOK_RETURN_RET_WITH_VAL ||
@@ -691,9 +692,9 @@ jsds_ScriptHookProc (JSDContext* jsdc, JSDScript* jsdscript, JSBool creating,
 #ifdef CAUTIOUS_SCRIPTHOOK
         JS_UNKEEP_ATOMS(rt);
 #endif
-        gJsds->DoPause(nsnull, true);
+        gJsds->DoPause(nullptr, true);
         hook->OnScriptCreated (script);
-        gJsds->DoUnPause(nsnull, true);
+        gJsds->DoUnPause(nullptr, true);
 #ifdef CAUTIOUS_SCRIPTHOOK
         JS_KEEP_ATOMS(rt);
 #endif
@@ -720,9 +721,9 @@ jsds_ScriptHookProc (JSDContext* jsdc, JSDScript* jsdscript, JSBool creating,
             JS_UNKEEP_ATOMS(rt);
 #endif
                 
-            gJsds->DoPause(nsnull, true);
+            gJsds->DoPause(nullptr, true);
             hook->OnScriptDestroyed (jsdis);
-            gJsds->DoUnPause(nsnull, true);
+            gJsds->DoUnPause(nullptr, true);
 #ifdef CAUTIOUS_SCRIPTHOOK
             JS_KEEP_ATOMS(rt);
 #endif
@@ -790,7 +791,7 @@ jsdObject::GetCreatorURL(nsACString &_rval)
 }
 
 NS_IMETHODIMP
-jsdObject::GetCreatorLine(PRUint32 *_rval)
+jsdObject::GetCreatorLine(uint32_t *_rval)
 {
     *_rval = JSD_GetObjectNewLineNumber(mCx, mObject);
     return NS_OK;
@@ -804,7 +805,7 @@ jsdObject::GetConstructorURL(nsACString &_rval)
 }
 
 NS_IMETHODIMP
-jsdObject::GetConstructorLine(PRUint32 *_rval)
+jsdObject::GetConstructorLine(uint32_t *_rval)
 {
     *_rval = JSD_GetObjectConstructorLineNumber(mCx, mObject);
     return NS_OK;
@@ -886,7 +887,7 @@ jsdProperty::GetAlias(jsdIValue **_rval)
 }
 
 NS_IMETHODIMP
-jsdProperty::GetFlags(PRUint32 *_rval)
+jsdProperty::GetFlags(uint32_t *_rval)
 {
     *_rval = JSD_GetPropertyFlags (mCx, mProperty);
     return NS_OK;
@@ -910,13 +911,6 @@ jsdProperty::GetValue(jsdIValue **_rval)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-jsdProperty::GetVarArgSlot(PRUint32 *_rval)
-{
-    *_rval = JSD_GetPropertyVarArgSlot (mCx, mProperty);
-    return NS_OK;
-}
-
 /* Scripts */
 NS_IMPL_THREADSAFE_ISUPPORTS2(jsdScript, jsdIScript, jsdIEphemeral)
 
@@ -930,8 +924,8 @@ AssignToJSString(nsACString *x, JSString *str)
     size_t length = JS_GetStringEncodingLength(NULL, str);
     if (length == size_t(-1))
         return NS_ERROR_FAILURE;
-    x->SetLength(PRUint32(length));
-    if (x->Length() != PRUint32(length))
+    x->SetLength(uint32_t(length));
+    if (x->Length() != uint32_t(length))
         return NS_ERROR_OUT_OF_MEMORY;
     JS_EncodeStringToBuffer(str, x->BeginWriting(), length);
     return NS_OK;
@@ -998,7 +992,7 @@ jsdScript::CreatePPLineMap()
     JSObject   *obj = JS_NewObject(cx, NULL, NULL, NULL);
     JSFunction *fun = JSD_GetJSFunction (mCx, mScript);
     JSScript   *script; /* In JSD compartment */
-    PRUint32    baseLine;
+    uint32_t    baseLine;
     JSString   *jsstr;
     size_t      length;
     const jschar *chars;
@@ -1007,19 +1001,16 @@ jsdScript::CreatePPLineMap()
         unsigned nargs;
 
         {
-            JSAutoEnterCompartment ac;
-            if (!ac.enter(cx, JS_GetFunctionObject(fun)))
-                return nsnull;
-
+            JSAutoCompartment ac(cx, JS_GetFunctionObject(fun));
             nargs = JS_GetFunctionArgumentCount(cx, fun);
             if (nargs > 12)
-                return nsnull;
+                return nullptr;
             jsstr = JS_DecompileFunctionBody (cx, fun, 4);
             if (!jsstr)
-                return nsnull;
+                return nullptr;
 
             if (!(chars = JS_GetStringCharsAndLength(cx, jsstr, &length)))
-                return nsnull;
+                return nullptr;
         }
 
         JS::Anchor<JSString *> kungFuDeathGrip(jsstr);
@@ -1029,43 +1020,41 @@ jsdScript::CreatePPLineMap()
         fun = JS_CompileUCFunction (cx, obj, "ppfun", nargs, argnames, chars,
                                     length, "x-jsd:ppbuffer?type=function", 3);
         if (!fun || !(script = JS_GetFunctionScript(cx, fun)))
-            return nsnull;
+            return nullptr;
         baseLine = 3;
     } else {
         script = JSD_GetJSScript(mCx, mScript);
         JSString *jsstr;
 
         {
-            JS::AutoEnterScriptCompartment ac;
-            if (!ac.enter(cx, script))
-                return nsnull;
+            JSAutoCompartment ac(cx, script);
 
             jsstr = JS_DecompileScript (cx, script, "ppscript", 4);
             if (!jsstr)
-                return nsnull;
+                return nullptr;
 
             if (!(chars = JS_GetStringCharsAndLength(cx, jsstr, &length)))
-                return nsnull;
+                return nullptr;
         }
 
         JS::Anchor<JSString *> kungFuDeathGrip(jsstr);
         script = JS_CompileUCScript (cx, obj, chars, length, "x-jsd:ppbuffer?type=script", 1);
         if (!script)
-            return nsnull;
+            return nullptr;
         baseLine = 1;
     }
 
-    PRUint32 scriptExtent = JS_GetScriptLineExtent (cx, script);
+    uint32_t scriptExtent = JS_GetScriptLineExtent (cx, script);
     jsbytecode* firstPC = JS_LineNumberToPC (cx, script, 0);
     /* allocate worst case size of map (number of lines in script + 1
      * for our 0 record), we'll shrink it with a realloc later. */
     PCMapEntry *lineMap =
         static_cast<PCMapEntry *>
                    (PR_Malloc((scriptExtent + 1) * sizeof (PCMapEntry)));
-    PRUint32 lineMapSize = 0;
+    uint32_t lineMapSize = 0;
 
     if (lineMap) {
-        for (PRUint32 line = baseLine; line < scriptExtent + baseLine; ++line) {
+        for (uint32_t line = baseLine; line < scriptExtent + baseLine; ++line) {
             jsbytecode* pc = JS_LineNumberToPC (cx, script, line);
             if (line == JS_PCToLineNumber (cx, script, pc)) {
                 lineMap[lineMapSize].line = line;
@@ -1089,12 +1078,12 @@ jsdScript::CreatePPLineMap()
     return mPPLineMap = lineMap;
 }
 
-PRUint32
-jsdScript::PPPcToLine (PRUint32 aPC)
+uint32_t
+jsdScript::PPPcToLine (uint32_t aPC)
 {
     if (!mPPLineMap && !CreatePPLineMap())
         return 0;
-    PRUint32 i;
+    uint32_t i;
     for (i = 1; i < mPCMapSize; ++i) {
         if (mPPLineMap[i].pc > aPC)
             return mPPLineMap[i - 1].line;            
@@ -1103,12 +1092,12 @@ jsdScript::PPPcToLine (PRUint32 aPC)
     return mPPLineMap[mPCMapSize - 1].line;
 }
 
-PRUint32
-jsdScript::PPLineToPc (PRUint32 aLine)
+uint32_t
+jsdScript::PPLineToPc (uint32_t aLine)
 {
     if (!mPPLineMap && !CreatePPLineMap())
         return 0;
-    PRUint32 i;
+    uint32_t i;
     for (i = 1; i < mPCMapSize; ++i) {
         if (mPPLineMap[i].line > aLine)
             return mPPLineMap[i - 1].pc;
@@ -1134,20 +1123,18 @@ jsdScript::GetJSDScript(JSDScript **_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::GetVersion (PRInt32 *_rval)
+jsdScript::GetVersion (int32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     JSContext *cx = JSD_GetDefaultJSContext (mCx);
     JSScript *script = JSD_GetJSScript(mCx, mScript);
-    JS::AutoEnterScriptCompartment ac;
-    if (!ac.enter(cx, script))
-        return NS_ERROR_FAILURE;
-    *_rval = static_cast<PRInt32>(JS_GetScriptVersion(cx, script));
+    JSAutoCompartment ac(cx, script);
+    *_rval = static_cast<int32_t>(JS_GetScriptVersion(cx, script));
     return NS_OK;
 }
 
 NS_IMETHODIMP
-jsdScript::GetTag(PRUint32 *_rval)
+jsdScript::GetTag(uint32_t *_rval)
 {
     if (!mTag)
         mTag = ++jsdScript::LastTag;
@@ -1199,7 +1186,7 @@ jsdScript::GetIsValid(bool *_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::SetFlags(PRUint32 flags)
+jsdScript::SetFlags(uint32_t flags)
 {
     ASSERT_VALID_EPHEMERAL;
     JSD_SetScriptFlags(mCx, mScript, flags);
@@ -1207,7 +1194,7 @@ jsdScript::SetFlags(PRUint32 flags)
 }
 
 NS_IMETHODIMP
-jsdScript::GetFlags(PRUint32 *_rval)
+jsdScript::GetFlags(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     *_rval = JSD_GetScriptFlags(mCx, mScript);
@@ -1229,7 +1216,7 @@ jsdScript::GetFunctionName(nsACString &_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::GetParameterNames(PRUint32* count, PRUnichar*** paramNames)
+jsdScript::GetParameterNames(uint32_t* count, PRUnichar*** paramNames)
 {
     ASSERT_VALID_EPHEMERAL;
     JSContext *cx = JSD_GetDefaultJSContext (mCx);
@@ -1240,20 +1227,18 @@ jsdScript::GetParameterNames(PRUint32* count, PRUnichar*** paramNames)
     JSFunction *fun = JSD_GetJSFunction (mCx, mScript);
     if (!fun) {
         *count = 0;
-        *paramNames = nsnull;
+        *paramNames = nullptr;
         return NS_OK;
     }
 
     JSAutoRequest ar(cx);
-    JSAutoEnterCompartment ac;
-    if (!ac.enter(cx, JS_GetFunctionObject(fun)))
-        return NS_ERROR_FAILURE;
+    JSAutoCompartment ac(cx, JS_GetFunctionObject(fun));
 
     unsigned nargs;
     if (!JS_FunctionHasLocalNames(cx, fun) ||
         (nargs = JS_GetFunctionArgumentCount(cx, fun)) == 0) {
         *count = 0;
-        *paramNames = nsnull;
+        *paramNames = nullptr;
         return NS_OK;
     }
 
@@ -1334,16 +1319,13 @@ jsdScript::GetFunctionSource(nsAString & aFunctionSource)
     JSAutoRequest ar(cx);
 
     JSString *jsstr;
-    JSAutoEnterCompartment ac;
-    JS::AutoEnterScriptCompartment asc;
+    mozilla::Maybe<JSAutoCompartment> ac;
     if (fun) {
-        if (!ac.enter(cx, JS_GetFunctionObject(fun)))
-            return NS_ERROR_FAILURE;
+        ac.construct(cx, JS_GetFunctionObject(fun));
         jsstr = JS_DecompileFunction (cx, fun, 4);
     } else {
         JSScript *script = JSD_GetJSScript (mCx, mScript);
-        if (!asc.enter(cx, script))
-            return NS_ERROR_FAILURE;
+        ac.construct(cx, script);
         jsstr = JS_DecompileScript (cx, script, "ppscript", 4);
     }
     if (!jsstr)
@@ -1359,21 +1341,21 @@ jsdScript::GetFunctionSource(nsAString & aFunctionSource)
 }
 
 NS_IMETHODIMP
-jsdScript::GetBaseLineNumber(PRUint32 *_rval)
+jsdScript::GetBaseLineNumber(uint32_t *_rval)
 {
     *_rval = mBaseLineNumber;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-jsdScript::GetLineExtent(PRUint32 *_rval)
+jsdScript::GetLineExtent(uint32_t *_rval)
 {
     *_rval = mLineExtent;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-jsdScript::GetCallCount(PRUint32 *_rval)
+jsdScript::GetCallCount(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     *_rval = JSD_GetScriptCallCount (mCx, mScript);
@@ -1381,7 +1363,7 @@ jsdScript::GetCallCount(PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::GetMaxRecurseDepth(PRUint32 *_rval)
+jsdScript::GetMaxRecurseDepth(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     *_rval = JSD_GetScriptMaxRecurseDepth (mCx, mScript);
@@ -1445,7 +1427,7 @@ jsdScript::ClearProfileData()
 }
 
 NS_IMETHODIMP
-jsdScript::PcToLine(PRUint32 aPC, PRUint32 aPcmap, PRUint32 *_rval)
+jsdScript::PcToLine(uint32_t aPC, uint32_t aPcmap, uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     if (aPcmap == PCMAP_SOURCETEXT) {
@@ -1460,7 +1442,7 @@ jsdScript::PcToLine(PRUint32 aPC, PRUint32 aPcmap, PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::LineToPc(PRUint32 aLine, PRUint32 aPcmap, PRUint32 *_rval)
+jsdScript::LineToPc(uint32_t aLine, uint32_t aPcmap, uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     if (aPcmap == PCMAP_SOURCETEXT) {
@@ -1488,8 +1470,8 @@ jsdScript::EnableSingleStepInterrupts(bool enable)
 }
 
 NS_IMETHODIMP
-jsdScript::GetExecutableLines(PRUint32 aPcmap, PRUint32 aStartLine, PRUint32 aMaxLines,
-                              PRUint32* aCount, PRUint32** aExecutableLines)
+jsdScript::GetExecutableLines(uint32_t aPcmap, uint32_t aStartLine, uint32_t aMaxLines,
+                              uint32_t* aCount, uint32_t** aExecutableLines)
 {
     ASSERT_VALID_EPHEMERAL;
     if (aPcmap == PCMAP_SOURCETEXT) {
@@ -1498,7 +1480,7 @@ jsdScript::GetExecutableLines(PRUint32 aPcmap, PRUint32 aStartLine, PRUint32 aMa
                        + JSD_GetScriptLineExtent(mCx, mScript) - 1;
         uintptr_t end = JSD_GetClosestPC(mCx, mScript, lastLine + 1);
 
-        *aExecutableLines = static_cast<PRUint32*>(NS_Alloc((end - start + 1) * sizeof(PRUint32)));
+        *aExecutableLines = static_cast<uint32_t*>(NS_Alloc((end - start + 1) * sizeof(uint32_t)));
         if (!JSD_GetLinePCs(mCx, mScript, aStartLine, aMaxLines, aCount, aExecutableLines, NULL))
             return NS_ERROR_OUT_OF_MEMORY;
         
@@ -1511,8 +1493,8 @@ jsdScript::GetExecutableLines(PRUint32 aPcmap, PRUint32 aStartLine, PRUint32 aMa
                 return NS_ERROR_OUT_OF_MEMORY;
         }
 
-        nsTArray<PRUint32> lines;
-        PRUint32 i;
+        nsTArray<uint32_t> lines;
+        uint32_t i;
 
         for (i = 0; i < mPCMapSize; ++i) {
             if (mPPLineMap[i].line >= aStartLine)
@@ -1526,7 +1508,7 @@ jsdScript::GetExecutableLines(PRUint32 aPcmap, PRUint32 aStartLine, PRUint32 aMa
         if (aCount)
             *aCount = lines.Length();
 
-        *aExecutableLines = static_cast<PRUint32*>(NS_Alloc(lines.Length() * sizeof(PRUint32)));
+        *aExecutableLines = static_cast<uint32_t*>(NS_Alloc(lines.Length() * sizeof(uint32_t)));
         if (!*aExecutableLines)
             return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1540,7 +1522,7 @@ jsdScript::GetExecutableLines(PRUint32 aPcmap, PRUint32 aStartLine, PRUint32 aMa
 }
 
 NS_IMETHODIMP
-jsdScript::IsLineExecutable(PRUint32 aLine, PRUint32 aPcmap, bool *_rval)
+jsdScript::IsLineExecutable(uint32_t aLine, uint32_t aPcmap, bool *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     if (aPcmap == PCMAP_SOURCETEXT) {    
@@ -1550,7 +1532,7 @@ jsdScript::IsLineExecutable(PRUint32 aLine, PRUint32 aPcmap, bool *_rval)
         if (!mPPLineMap && !CreatePPLineMap())
             return NS_ERROR_OUT_OF_MEMORY;
         *_rval = false;
-        for (PRUint32 i = 0; i < mPCMapSize; ++i) {
+        for (uint32_t i = 0; i < mPCMapSize; ++i) {
             if (mPPLineMap[i].line >= aLine) {
                 *_rval = (mPPLineMap[i].line == aLine);
                 break;
@@ -1564,7 +1546,7 @@ jsdScript::IsLineExecutable(PRUint32 aLine, PRUint32 aPcmap, bool *_rval)
 }
 
 NS_IMETHODIMP
-jsdScript::SetBreakpoint(PRUint32 aPC)
+jsdScript::SetBreakpoint(uint32_t aPC)
 {
     ASSERT_VALID_EPHEMERAL;
     uintptr_t pc = mFirstPC + aPC;
@@ -1573,7 +1555,7 @@ jsdScript::SetBreakpoint(PRUint32 aPC)
 }
 
 NS_IMETHODIMP
-jsdScript::ClearBreakpoint(PRUint32 aPC)
+jsdScript::ClearBreakpoint(uint32_t aPC)
 {
     ASSERT_VALID_EPHEMERAL;    
     uintptr_t pc = mFirstPC + aPC;
@@ -1598,7 +1580,7 @@ jsdIContext *
 jsdContext::FromPtr (JSDContext *aJSDCx, JSContext *aJSCx)
 {
     if (!aJSDCx || !aJSCx)
-        return nsnull;
+        return nullptr;
 
     nsCOMPtr<jsdIContext> jsdicx;
     nsCOMPtr<jsdIEphemeral> eph = 
@@ -1615,7 +1597,7 @@ jsdContext::FromPtr (JSDContext *aJSDCx, JSContext *aJSCx)
         jsdicx = new jsdContext (aJSDCx, aJSCx, iscx);
     }
 
-    jsdIContext *ctx = nsnull;
+    jsdIContext *ctx = nullptr;
     jsdicx.swap(ctx);
     return ctx;
 }
@@ -1673,7 +1655,7 @@ jsdContext::GetJSContext(JSContext **_rval)
 }
 
 NS_IMETHODIMP
-jsdContext::GetOptions(PRUint32 *_rval)
+jsdContext::GetOptions(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     *_rval = JS_GetOptions(mJSCx);
@@ -1681,10 +1663,10 @@ jsdContext::GetOptions(PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdContext::SetOptions(PRUint32 options)
+jsdContext::SetOptions(uint32_t options)
 {
     ASSERT_VALID_EPHEMERAL;
-    PRUint32 lastOptions = JS_GetOptions(mJSCx);
+    uint32_t lastOptions = JS_GetOptions(mJSCx);
 
     /* don't let users change this option, they'd just be shooting themselves
      * in the foot. */
@@ -1699,7 +1681,7 @@ NS_IMETHODIMP
 jsdContext::GetPrivateData(nsISupports **_rval)
 {
     ASSERT_VALID_EPHEMERAL;
-    PRUint32 options = JS_GetOptions(mJSCx);
+    uint32_t options = JS_GetOptions(mJSCx);
     if (options & JSOPTION_PRIVATE_IS_NSISUPPORTS)
     {
         *_rval = static_cast<nsISupports*>(JS_GetContextPrivate(mJSCx));
@@ -1707,7 +1689,7 @@ jsdContext::GetPrivateData(nsISupports **_rval)
     }
     else
     {
-        *_rval = nsnull;
+        *_rval = nullptr;
     }
     
     return NS_OK;
@@ -1722,7 +1704,7 @@ jsdContext::GetWrappedContext(nsISupports **_rval)
 }
 
 NS_IMETHODIMP
-jsdContext::GetTag(PRUint32 *_rval)
+jsdContext::GetTag(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     if (!mTag)
@@ -1733,15 +1715,15 @@ jsdContext::GetTag(PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdContext::GetVersion (PRInt32 *_rval)
+jsdContext::GetVersion (int32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
-    *_rval = static_cast<PRInt32>(JS_GetVersion(mJSCx));
+    *_rval = static_cast<int32_t>(JS_GetVersion(mJSCx));
     return NS_OK;
 }
 
 NS_IMETHODIMP
-jsdContext::SetVersion (PRInt32 id)
+jsdContext::SetVersion (int32_t id)
 {
     ASSERT_VALID_EPHEMERAL;
     JSVersion ver = static_cast<JSVersion>(id);
@@ -1831,7 +1813,7 @@ jsdStackFrame::FromPtr (JSDContext *aCx, JSDThreadState *aThreadState,
                         JSDStackFrameInfo *aStackFrameInfo)
 {
     if (!aStackFrameInfo)
-        return nsnull;
+        return nullptr;
 
     jsdIStackFrame *rv;
     nsCOMPtr<jsdIStackFrame> frame;
@@ -1959,7 +1941,7 @@ jsdStackFrame::GetScript(jsdIScript **_rval)
 }
 
 NS_IMETHODIMP
-jsdStackFrame::GetPc(PRUint32 *_rval)
+jsdStackFrame::GetPc(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     JSDScript *script = JSD_GetScriptForStackFrame (mCx, mThreadState,
@@ -1977,7 +1959,7 @@ jsdStackFrame::GetPc(PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdStackFrame::GetLine(PRUint32 *_rval)
+jsdStackFrame::GetLine(uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     JSDScript *script = JSD_GetScriptForStackFrame (mCx, mThreadState,
@@ -2027,7 +2009,7 @@ jsdStackFrame::GetThisValue(jsdIValue **_rval)
 
 NS_IMETHODIMP
 jsdStackFrame::Eval (const nsAString &bytes, const nsACString &fileName,
-                     PRUint32 line, jsdIValue **result, bool *_rval)
+                     uint32_t line, jsdIValue **result, bool *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
 
@@ -2077,7 +2059,7 @@ jsdStackFrame::Eval (const nsAString &bytes, const nsACString &fileName,
     rv = stack->Pop(&poppedCX);
     NS_ASSERTION(NS_SUCCEEDED(rv) && poppedCX == cx, "bad pop");
 #else
-    (void) stack->Pop(nsnull);
+    (void) stack->Pop(nullptr);
 #endif
 
     JSDValue *jsdv = JSD_NewValue (mCx, jv);
@@ -2098,7 +2080,7 @@ jsdValue::FromPtr (JSDContext *aCx, JSDValue *aValue)
     /* value will be dropped by te jsdValue destructor. */
 
     if (!aValue)
-        return nsnull;
+        return nullptr;
     
     jsdIValue *rv = new jsdValue (aCx, aValue);
     NS_IF_ADDREF(rv);
@@ -2187,7 +2169,7 @@ jsdValue::GetIsPrimitive (bool *_rval)
 }
 
 NS_IMETHODIMP
-jsdValue::GetJsType (PRUint32 *_rval)
+jsdValue::GetJsType (uint32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     jsval val;
@@ -2276,7 +2258,7 @@ jsdValue::GetDoubleValue(double *_rval)
 }
 
 NS_IMETHODIMP
-jsdValue::GetIntValue(PRInt32 *_rval)
+jsdValue::GetIntValue(int32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     *_rval = JSD_GetValueInt (mCx, mValue);
@@ -2319,7 +2301,7 @@ jsdValue::GetStringValue(nsACString &_rval)
 }
 
 NS_IMETHODIMP
-jsdValue::GetPropertyCount (PRInt32 *_rval)
+jsdValue::GetPropertyCount (int32_t *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     if (JSD_IsValueObject(mCx, mValue))
@@ -2330,14 +2312,14 @@ jsdValue::GetPropertyCount (PRInt32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdValue::GetProperties (jsdIProperty ***propArray, PRUint32 *length)
+jsdValue::GetProperties (jsdIProperty ***propArray, uint32_t *length)
 {
     ASSERT_VALID_EPHEMERAL;
-    *propArray = nsnull;
+    *propArray = nullptr;
     if (length)
         *length = 0;
 
-    PRUint32 prop_count = JSD_IsValueObject(mCx, mValue)
+    uint32_t prop_count = JSD_IsValueObject(mCx, mValue)
         ? JSD_GetCountOfProperties (mCx, mValue)
         : 0;
     NS_ENSURE_TRUE(prop_count, NS_OK);
@@ -2348,7 +2330,7 @@ jsdValue::GetProperties (jsdIProperty ***propArray, PRUint32 *length)
                                        prop_count));
     NS_ENSURE_TRUE(pa_temp, NS_ERROR_OUT_OF_MEMORY);
 
-    PRUint32     i    = 0;
+    uint32_t     i    = 0;
     JSDProperty *iter = NULL;
     JSDProperty *prop;
     while ((prop = JSD_IterateProperties (mCx, mValue, &iter))) {
@@ -2428,28 +2410,28 @@ NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(jsdService)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(jsdService)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mErrorHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mBreakpointHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDebugHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDebuggerHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mInterruptHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mScriptHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mThrowHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mTopLevelHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mFunctionHook)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mActivationCallback)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mErrorHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mBreakpointHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDebugHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDebuggerHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mInterruptHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mScriptHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mThrowHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mTopLevelHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mFunctionHook)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mActivationCallback)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(jsdService)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mErrorHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mBreakpointHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDebugHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDebuggerHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mInterruptHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mScriptHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mThrowHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mTopLevelHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mFunctionHook)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mActivationCallback)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mErrorHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mBreakpointHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDebugHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDebuggerHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mInterruptHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mScriptHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mThrowHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mTopLevelHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mFunctionHook)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mActivationCallback)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(jsdService)
@@ -2463,7 +2445,7 @@ jsdService::GetJSDContext(JSDContext **_rval)
 }
 
 NS_IMETHODIMP
-jsdService::GetFlags (PRUint32 *_rval)
+jsdService::GetFlags (uint32_t *_rval)
 {
     ASSERT_VALID_CONTEXT;
     *_rval = JSD_GetContextFlags (mCx);
@@ -2471,7 +2453,7 @@ jsdService::GetFlags (PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdService::SetFlags (PRUint32 flags)
+jsdService::SetFlags (uint32_t flags)
 {
     ASSERT_VALID_CONTEXT;
     JSD_SetContextFlags (mCx, flags);
@@ -2486,14 +2468,14 @@ jsdService::GetImplementationString(nsACString &aImplementationString)
 }
 
 NS_IMETHODIMP
-jsdService::GetImplementationMajor(PRUint32 *_rval)
+jsdService::GetImplementationMajor(uint32_t *_rval)
 {
     *_rval = JSDS_MAJOR_VERSION;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-jsdService::GetImplementationMinor(PRUint32 *_rval)
+jsdService::GetImplementationMinor(uint32_t *_rval)
 {
     *_rval = JSDS_MINOR_VERSION;
     return NS_OK;
@@ -2556,8 +2538,8 @@ jsdService::DeactivateDebugger ()
     
     JSD_DebuggerOff (mCx);
 
-    mCx = nsnull;
-    mRuntime = nsnull;
+    mCx = nullptr;
+    mRuntime = nullptr;
     mOn = false;
 
     return NS_OK;
@@ -2669,7 +2651,7 @@ jsdService::Off (void)
 }
 
 NS_IMETHODIMP
-jsdService::GetPauseDepth(PRUint32 *_rval)
+jsdService::GetPauseDepth(uint32_t *_rval)
 {
     NS_ENSURE_ARG_POINTER(_rval);
     *_rval = mPauseLevel;
@@ -2677,13 +2659,13 @@ jsdService::GetPauseDepth(PRUint32 *_rval)
 }
     
 NS_IMETHODIMP
-jsdService::Pause(PRUint32 *_rval)
+jsdService::Pause(uint32_t *_rval)
 {
     return DoPause(_rval, false);
 }
 
 nsresult
-jsdService::DoPause(PRUint32 *_rval, bool internalCall)
+jsdService::DoPause(uint32_t *_rval, bool internalCall)
 {
     if (!mCx)
         return NS_ERROR_NOT_INITIALIZED;
@@ -2715,13 +2697,13 @@ jsdService::DoPause(PRUint32 *_rval, bool internalCall)
 }
 
 NS_IMETHODIMP
-jsdService::UnPause(PRUint32 *_rval)
+jsdService::UnPause(uint32_t *_rval)
 {
     return DoUnPause(_rval, false);
 }
 
 nsresult
-jsdService::DoUnPause(PRUint32 *_rval, bool internalCall)
+jsdService::DoUnPause(uint32_t *_rval, bool internalCall)
 {
     if (!mCx)
         return NS_ERROR_NOT_INITIALIZED;
@@ -2934,7 +2916,7 @@ jsdService::RemoveFilter (jsdIFilter *filter)
                                    (PR_NEXT_LINK(&rec->links));
         /* If we're the only filter left, null out the list head. */
         if (gFilters == rec)
-            gFilters = nsnull;
+            gFilters = nullptr;
     }
 
     
@@ -3002,7 +2984,7 @@ jsdService::EnumerateFilters (jsdIFilterEnumerator *enumerator)
 NS_IMETHODIMP
 jsdService::RefreshFilters ()
 {
-    return EnumerateFilters(nsnull);
+    return EnumerateFilters(nullptr);
 }
 
 NS_IMETHODIMP
@@ -3022,7 +3004,7 @@ jsdService::ClearFilters ()
     } while (current != gFilters);
     
     jsds_FreeFilter(current);
-    gFilters = nsnull;
+    gFilters = nullptr;
     
     return NS_OK;
 }
@@ -3052,7 +3034,7 @@ jsdService::WrapValue(const JS::Value &value, jsdIValue **_rval)
 
 
 NS_IMETHODIMP
-jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
+jsdService::EnterNestedEventLoop (jsdINestCallback *callback, uint32_t *_rval)
 {
     // Nesting event queues is a thing of the past.  Now, we just spin the
     // current event loop.
@@ -3062,15 +3044,15 @@ jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
         stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv));
     if (NS_FAILED(rv))
         return rv;
-    PRUint32 nestLevel = ++mNestedLoopLevel;
+    uint32_t nestLevel = ++mNestedLoopLevel;
     
     nsCOMPtr<nsIThread> thread = do_GetCurrentThread();
 
-    if (NS_SUCCEEDED(stack->Push(nsnull))) {
+    if (NS_SUCCEEDED(stack->Push(nullptr))) {
         if (callback) {
-            DoPause(nsnull, true);
+            DoPause(nullptr, true);
             rv = callback->OnNest();
-            DoUnPause(nsnull, true);
+            DoUnPause(nullptr, true);
         }
         
         while (NS_SUCCEEDED(rv) && mNestedLoopLevel >= nestLevel) {
@@ -3080,7 +3062,7 @@ jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
 
         JSContext* cx;
         stack->Pop(&cx);
-        NS_ASSERTION(cx == nsnull, "JSContextStack mismatch");
+        NS_ASSERTION(cx == nullptr, "JSContextStack mismatch");
     }
     else
         rv = NS_ERROR_FAILURE;
@@ -3095,7 +3077,7 @@ jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
 }
 
 NS_IMETHODIMP
-jsdService::ExitNestedEventLoop (PRUint32 *_rval)
+jsdService::ExitNestedEventLoop (uint32_t *_rval)
 {
     if (mNestedLoopLevel > 0)
         --mNestedLoopLevel;
@@ -3352,17 +3334,17 @@ jsdService::GetFunctionHook (jsdICallHook **aHook)
 jsdService::~jsdService()
 {
     ClearFilters();
-    mErrorHook = nsnull;
-    mBreakpointHook = nsnull;
-    mDebugHook = nsnull;
-    mDebuggerHook = nsnull;
-    mInterruptHook = nsnull;
-    mScriptHook = nsnull;
-    mThrowHook = nsnull;
-    mTopLevelHook = nsnull;
-    mFunctionHook = nsnull;
+    mErrorHook = nullptr;
+    mBreakpointHook = nullptr;
+    mDebugHook = nullptr;
+    mDebuggerHook = nullptr;
+    mInterruptHook = nullptr;
+    mScriptHook = nullptr;
+    mThrowHook = nullptr;
+    mTopLevelHook = nullptr;
+    mFunctionHook = nullptr;
     Off();
-    gJsds = nsnull;
+    gJsds = nullptr;
 }
 
 jsdService *
@@ -3381,7 +3363,7 @@ NS_GENERIC_FACTORY_SINGLETON_CONSTRUCTOR(jsdService, jsdService::GetService)
  * and/or removed from the app-start category by the jsdService::initAtStartup
  * property.
  */
-class jsdASObserver : public nsIObserver 
+class jsdASObserver MOZ_FINAL : public nsIObserver
 {
   public:
     NS_DECL_ISUPPORTS
@@ -3473,7 +3455,7 @@ jsdThreadState::GetJSDThreadState(JSDThreadState **_rval)
 }
 
 NS_IMETHODIMP
-jsdThreadState::GetFrameCount (PRUint32 *_rval)
+jsdThreadState::GetFrameCount (uint32_t *_rval)
 {
     *_rval = JSD_GetCountOfStackFrames (mCx, mThreadState);
     return NS_OK;
