@@ -92,16 +92,16 @@ bool callTrackingActive(JSContext *);
 bool wantNativeAddressInfo(JSContext *);
 
 /* Entering a JS function */
-bool enterJSFun(JSContext *, JSFunction *, JSScript *, int counter = 1);
+bool enterScript(JSContext *, UnrootedScript, UnrootedFunction , StackFrame *);
 
 /* About to leave a JS function */
-bool exitJSFun(JSContext *, JSFunction *, JSScript *, int counter = 0);
+bool exitScript(JSContext *, UnrootedScript, UnrootedFunction , StackFrame *);
 
 /* Executing a script */
-bool startExecution(JSContext *cx, JSScript *script);
+bool startExecution(UnrootedScript script);
 
 /* Script has completed execution */
-bool stopExecution(JSContext *cx, JSScript *script);
+bool stopExecution(UnrootedScript script);
 
 /* Heap has been resized */
 bool resizeHeap(JSCompartment *compartment, size_t oldSize, size_t newSize);
@@ -147,10 +147,10 @@ bool compileScriptBegin(const char *filename, int lineno);
 bool compileScriptEnd(const char *filename, int lineno);
 
 /* About to make a call from JS into native code */
-bool calloutBegin(JSContext *cx, JSFunction *fun);
+bool calloutBegin(JSContext *cx, UnrootedFunction fun);
 
 /* Native code called by JS has terminated */
-bool calloutEnd(JSContext *cx, JSFunction *fun);
+bool calloutEnd(JSContext *cx, UnrootedFunction fun);
 
 /* Unimplemented */
 bool acquireMemory(JSContext *cx, void *address, size_t nbytes);
@@ -197,88 +197,20 @@ enum JITReportGranularity {
     JITREPORT_GRANULARITY_OP = 3
 };
 
-/*
- * Observer class for JIT code allocation/deallocation. Currently, this only
- * handles the method JIT, and does not get notifications when JIT code is
- * changed (patched) with no new allocation.
- */
-class JITWatcher {
-public:
-    struct NativeRegion {
-        mjit::JSActiveFrame *frame;
-        JSScript *script;
-        size_t inlinedOffset;
-        jsbytecode *pc;
-        jsbytecode *endpc;
-        uintptr_t mainOffset;
-        uintptr_t stubOffset;
-        bool enter;
-    };
-
-    typedef Vector<NativeRegion, 0, RuntimeAllocPolicy> RegionVector;
-
-    virtual JITReportGranularity granularityRequested() = 0;
-
-#ifdef JS_METHODJIT
-    static bool CollectNativeRegions(RegionVector &regions,
-                                     JSRuntime *rt,
-                                     mjit::JITChunk *jit,
-                                     mjit::JSActiveFrame *outerFrame,
-                                     mjit::JSActiveFrame **inlineFrames);
-
-    virtual void registerMJITCode(JSContext *cx, js::mjit::JITChunk *chunk,
-                                  mjit::JSActiveFrame *outerFrame,
-                                  mjit::JSActiveFrame **inlineFrames,
-                                  void *mainCodeAddress, size_t mainCodeSize,
-                                  void *stubCodeAddress, size_t stubCodeSize) = 0;
-
-    virtual void discardMJITCode(FreeOp *fop, mjit::JITScript *jscr, mjit::JITChunk *chunk,
-                                 void* address) = 0;
-
-    virtual void registerICCode(JSContext *cx,
-                                js::mjit::JITChunk *chunk, JSScript *script, jsbytecode* pc,
-                                void *start, size_t size) = 0;
-#endif
-
-    virtual void discardExecutableRegion(void *start, size_t size) = 0;
-};
-
-/*
- * Register a JITWatcher subclass to be informed of JIT code
- * allocation/deallocation.
- */
-bool
-addJITWatcher(JITWatcher *watcher);
-
-/*
- * Remove (and destroy) a registered JITWatcher. rt may be NULL. Returns false
- * if the watcher is not found.
- */
-bool
-removeJITWatcher(JSRuntime *rt, JITWatcher *watcher);
-
-/*
- * Remove (and destroy) all registered JITWatchers. rt may be NULL.
- */
-void
-removeAllJITWatchers(JSRuntime *rt);
-
 /*
  * Finest granularity of JIT information desired by all watchers.
  */
 JITReportGranularity
-JITGranularityRequested();
+JITGranularityRequested(JSContext *cx);
 
 #ifdef JS_METHODJIT
 /*
  * New method JIT code has been created
  */
-void
+bool
 registerMJITCode(JSContext *cx, js::mjit::JITChunk *chunk,
                  mjit::JSActiveFrame *outerFrame,
-                 mjit::JSActiveFrame **inlineFrames,
-                 void *mainCodeAddress, size_t mainCodeSize,
-                 void *stubCodeAddress, size_t stubCodeSize);
+                 mjit::JSActiveFrame **inlineFrames);
 
 /*
  * Method JIT code is about to be discarded
@@ -289,9 +221,9 @@ discardMJITCode(FreeOp *fop, mjit::JITScript *jscr, mjit::JITChunk *chunk, void*
 /*
  * IC code has been allocated within the given JITChunk
  */
-void
+bool
 registerICCode(JSContext *cx,
-               mjit::JITChunk *chunk, JSScript *script, jsbytecode* pc,
+               mjit::JITChunk *chunk, UnrootedScript script, jsbytecode* pc,
                void *start, size_t size);
 #endif /* JS_METHODJIT */
 
@@ -303,13 +235,13 @@ void
 discardExecutableRegion(void *start, size_t size);
 
 /*
- * Internal: DTrace-specific functions to be called during Probes::enterJSFun
- * and Probes::exitJSFun. These will not be inlined, but the argument
+ * Internal: DTrace-specific functions to be called during Probes::enterScript
+ * and Probes::exitScript. These will not be inlined, but the argument
  * marshalling required for these probe points is expensive enough that it
  * shouldn't really matter.
  */
-void DTraceEnterJSFun(JSContext *cx, JSFunction *fun, JSScript *script);
-void DTraceExitJSFun(JSContext *cx, JSFunction *fun, JSScript *script);
+void DTraceEnterJSFun(JSContext *cx, UnrootedFunction fun, UnrootedScript script);
+void DTraceExitJSFun(JSContext *cx, UnrootedFunction fun, UnrootedScript script);
 
 /*
  * Internal: ETW-specific probe functions
@@ -319,9 +251,9 @@ void DTraceExitJSFun(JSContext *cx, JSFunction *fun, JSScript *script);
 bool ETWCreateRuntime(JSRuntime *rt);
 bool ETWDestroyRuntime(JSRuntime *rt);
 bool ETWShutdown();
-bool ETWCallTrackingActive(JSContext *cx);
-bool ETWEnterJSFun(JSContext *cx, JSFunction *fun, JSScript *script, int counter);
-bool ETWExitJSFun(JSContext *cx, JSFunction *fun, JSScript *script, int counter);
+bool ETWCallTrackingActive();
+bool ETWEnterJSFun(JSContext *cx, UnrootedFunction fun, UnrootedScript script, int counter);
+bool ETWExitJSFun(JSContext *cx, UnrootedFunction fun, UnrootedScript script, int counter);
 bool ETWCreateObject(JSContext *cx, JSObject *obj);
 bool ETWFinalizeObject(JSObject *obj);
 bool ETWResizeObject(JSContext *cx, JSObject *obj, size_t oldSize, size_t newSize);
@@ -329,8 +261,8 @@ bool ETWCreateString(JSContext *cx, JSString *string, size_t length);
 bool ETWFinalizeString(JSString *string);
 bool ETWCompileScriptBegin(const char *filename, int lineno);
 bool ETWCompileScriptEnd(const char *filename, int lineno);
-bool ETWCalloutBegin(JSContext *cx, JSFunction *fun);
-bool ETWCalloutEnd(JSContext *cx, JSFunction *fun);
+bool ETWCalloutBegin(JSContext *cx, UnrootedFunction fun);
+bool ETWCalloutEnd(JSContext *cx, UnrootedFunction fun);
 bool ETWAcquireMemory(JSContext *cx, void *address, size_t nbytes);
 bool ETWReleaseMemory(JSContext *cx, void *address, size_t nbytes);
 bool ETWGCStart();
@@ -342,8 +274,8 @@ bool ETWGCEndSweepPhase();
 bool ETWCustomMark(JSString *string);
 bool ETWCustomMark(const char *string);
 bool ETWCustomMark(int marker);
-bool ETWStartExecution(JSContext *cx, JSScript *script);
-bool ETWStopExecution(JSContext *cx, JSScript *script);
+bool ETWStartExecution(UnrootedScript script);
+bool ETWStopExecution(JSContext *cx, UnrootedScript script);
 bool ETWResizeHeap(JSCompartment *compartment, size_t oldSize, size_t newSize);
 #endif
 
@@ -366,7 +298,7 @@ Probes::callTrackingActive(JSContext *cx)
         return true;
 #endif
 #ifdef MOZ_ETW
-    if (ProfilingActive && ETWCallTrackingActive(cx))
+    if (ProfilingActive && ETWCallTrackingActive())
         return true;
 #endif
     return false;
@@ -376,47 +308,65 @@ inline bool
 Probes::wantNativeAddressInfo(JSContext *cx)
 {
     return (cx->reportGranularity >= JITREPORT_GRANULARITY_FUNCTION &&
-            JITGranularityRequested() >= JITREPORT_GRANULARITY_FUNCTION);
+            JITGranularityRequested(cx) >= JITREPORT_GRANULARITY_FUNCTION);
 }
 
 inline bool
-Probes::enterJSFun(JSContext *cx, JSFunction *fun, JSScript *script, int counter)
+Probes::enterScript(JSContext *cx, UnrootedScript script, UnrootedFunction maybeFun,
+                    StackFrame *fp)
 {
     bool ok = true;
 #ifdef INCLUDE_MOZILLA_DTRACE
     if (JAVASCRIPT_FUNCTION_ENTRY_ENABLED())
-        DTraceEnterJSFun(cx, fun, script);
+        DTraceEnterJSFun(cx, maybeFun, script);
 #endif
 #ifdef MOZ_TRACE_JSCALLS
-    cx->doFunctionCallback(fun, script, counter);
+    cx->doFunctionCallback(maybeFun, script, 1);
 #endif
 #ifdef MOZ_ETW
-    if (ProfilingActive && !ETWEnterJSFun(cx, fun, script, counter))
+    if (ProfilingActive && !ETWEnterJSFun(cx, maybeFun, script, 1))
         ok = false;
 #endif
 
+    JSRuntime *rt = cx->runtime;
+    if (rt->spsProfiler.enabled()) {
+        rt->spsProfiler.enter(cx, script, maybeFun);
+        JS_ASSERT_IF(!fp->isGeneratorFrame(), !fp->hasPushedSPSFrame());
+        fp->setPushedSPSFrame();
+    }
+
     return ok;
 }
 
 inline bool
-Probes::exitJSFun(JSContext *cx, JSFunction *fun, JSScript *script, int counter)
+Probes::exitScript(JSContext *cx, UnrootedScript script, UnrootedFunction maybeFun,
+                   StackFrame *fp)
 {
     bool ok = true;
 
 #ifdef INCLUDE_MOZILLA_DTRACE
     if (JAVASCRIPT_FUNCTION_RETURN_ENABLED())
-        DTraceExitJSFun(cx, fun, script);
+        DTraceExitJSFun(cx, maybeFun, script);
 #endif
 #ifdef MOZ_TRACE_JSCALLS
-    if (counter > 0)
-        counter = -counter;
-    cx->doFunctionCallback(fun, script, counter);
+    cx->doFunctionCallback(maybeFun, script, 0);
 #endif
 #ifdef MOZ_ETW
-    if (ProfilingActive && !ETWExitJSFun(cx, fun, script, counter))
+    if (ProfilingActive && !ETWExitJSFun(cx, maybeFun, script, 0))
         ok = false;
 #endif
 
+    JSRuntime *rt = cx->runtime;
+    /*
+     * Coming from IonMonkey, the fp might not be known (fp == NULL), but
+     * IonMonkey will only call exitScript() when absolutely necessary, so it is
+     * guaranteed that fp->hasPushedSPSFrame() would have been true
+     */
+    if ((fp == NULL && rt->spsProfiler.enabled()) ||
+        (fp != NULL && fp->hasPushedSPSFrame()))
+    {
+        rt->spsProfiler.exit(cx, script, maybeFun);
+    }
     return ok;
 }
 
@@ -562,7 +512,7 @@ Probes::compileScriptEnd(const char *filename, int lineno)
 }
 
 inline bool
-Probes::calloutBegin(JSContext *cx, JSFunction *fun)
+Probes::calloutBegin(JSContext *cx, UnrootedFunction fun)
 {
     bool ok = true;
 
@@ -575,7 +525,7 @@ Probes::calloutBegin(JSContext *cx, JSFunction *fun)
 }
 
 inline bool
-Probes::calloutEnd(JSContext *cx, JSFunction *fun)
+Probes::calloutEnd(JSContext *cx, UnrootedFunction fun)
 {
     bool ok = true;
 
@@ -731,7 +681,7 @@ Probes::CustomMark(int marker)
 }
 
 inline bool
-Probes::startExecution(JSContext *cx, JSScript *script)
+Probes::startExecution(UnrootedScript script)
 {
     bool ok = true;
 
@@ -741,7 +691,7 @@ Probes::startExecution(JSContext *cx, JSScript *script)
                                  script->lineno);
 #endif
 #ifdef MOZ_ETW
-    if (ProfilingActive && !ETWStartExecution(cx, script))
+    if (ProfilingActive && !ETWStartExecution(script))
         ok = false;
 #endif
 
@@ -749,7 +699,7 @@ Probes::startExecution(JSContext *cx, JSScript *script)
 }
 
 inline bool
-Probes::stopExecution(JSContext *cx, JSScript *script)
+Probes::stopExecution(UnrootedScript script)
 {
     bool ok = true;
 
@@ -759,32 +709,13 @@ Probes::stopExecution(JSContext *cx, JSScript *script)
                                 script->lineno);
 #endif
 #ifdef MOZ_ETW
-    if (ProfilingActive && !ETWStopExecution(cx, script))
+    if (ProfilingActive && !ETWStopExecution(script))
         ok = false;
 #endif
 
     return ok;
 }
 
-struct AutoFunctionCallProbe {
-    JSContext * const cx;
-    JSFunction *fun;
-    JSScript *script;
-    JS_DECL_USE_GUARD_OBJECT_NOTIFIER
-
-    AutoFunctionCallProbe(JSContext *cx, JSFunction *fun, JSScript *script
-                          JS_GUARD_OBJECT_NOTIFIER_PARAM)
-      : cx(cx), fun(fun), script(script)
-    {
-        JS_GUARD_OBJECT_NOTIFIER_INIT;
-        Probes::enterJSFun(cx, fun, script);
-    }
-
-    ~AutoFunctionCallProbe() {
-        Probes::exitJSFun(cx, fun, script);
-    }
-};
-
 } /* namespace js */
 
 /*
