@@ -7,21 +7,17 @@
 
 #ifndef jsscope_h___
 #define jsscope_h___
-/*
- * JS symbol tables.
- */
-#include <new>
-#ifdef DEBUG
-#include <stdio.h>
-#endif
+
+#include "mozilla/Attributes.h"
+#include "mozilla/GuardObjects.h"
 
 #include "jsobj.h"
 #include "jspropertytree.h"
 #include "jstypes.h"
 
 #include "js/HashTable.h"
+#include "gc/Heap.h"
 #include "gc/Root.h"
-#include "mozilla/Attributes.h"
 
 #ifdef _MSC_VER
 #pragma warning(push)
@@ -95,8 +91,12 @@
  * a single BaseShape.
  */
 
+struct JSObject;
+
 namespace js {
 
+class Bindings;
+
 /* Limit on the number of slotful properties in an object. */
 static const uint32_t SHAPE_INVALID_SLOT = JS_BIT(24) - 1;
 static const uint32_t SHAPE_MAXIMUM_SLOT = JS_BIT(24) - 2;
@@ -130,14 +130,14 @@ struct ShapeTable {
     }
 
     ~ShapeTable() {
-        js::UnwantedForeground::free_(entries);
+        js_free(entries);
     }
 
     /* By definition, hashShift = HASH_BITS - log2(capacity). */
     uint32_t capacity() const { return JS_BIT(HASH_BITS - hashShift); }
 
     /* Computes the size of the entries array for a given capacity. */
-    static size_t sizeOfEntries(size_t cap) { return cap * sizeof(Shape *); }
+    static size_t sizeOfEntries(size_t cap) { return cap * sizeof(RawShape); }
 
     /*
      * This counts the ShapeTable object itself (which must be
@@ -165,19 +165,11 @@ struct ShapeTable {
      * cope or ignore. They do however use JSRuntime's calloc_ method in order
      * to update the malloc counter on success.
      */
-    bool            init(JSRuntime *rt, js::Shape *lastProp);
+    bool            init(JSRuntime *rt, UnrootedShape lastProp);
     bool            change(int log2Delta, JSContext *cx);
-    js::Shape       **search(jsid id, bool adding);
+    Shape           **search(jsid id, bool adding);
 };
 
-} /* namespace js */
-
-struct JSObject;
-
-namespace js {
-
-class PropertyTree;
-
 /*
  * Reuse the API-only JSPROP_INDEX attribute to mean shadowability.
  */
@@ -231,12 +223,15 @@ class PropertyTree;
  * an earlier property, however.
  */
 
-class UnownedBaseShape;
+ForwardDeclare(UnownedBaseShape);
+ForwardDeclare(BaseShape);
+ForwardDeclare(Shape);
+struct StackBaseShape;
 
 class BaseShape : public js::gc::Cell
 {
   public:
-    friend struct Shape;
+    friend class Shape;
     friend struct StackBaseShape;
     friend struct StackShape;
 
@@ -254,19 +249,17 @@ class BaseShape : public js::gc::Cell
          * last property changes.
          */
 
-        EXTENSIBLE_PARENTS =    0x8,
-        DELEGATE           =   0x10,
-        SYSTEM             =   0x20,
-        NOT_EXTENSIBLE     =   0x40,
-        INDEXED            =   0x80,
-        BOUND_FUNCTION     =  0x100,
-        VAROBJ             =  0x200,
-        WATCHED            =  0x400,
-        ITERATED_SINGLETON =  0x800,
-        NEW_TYPE_UNKNOWN   = 0x1000,
-        UNCACHEABLE_PROTO  = 0x2000,
+        DELEGATE           =    0x8,
+        NOT_EXTENSIBLE     =   0x10,
+        INDEXED            =   0x20,
+        BOUND_FUNCTION     =   0x40,
+        VAROBJ             =   0x80,
+        WATCHED            =  0x100,
+        ITERATED_SINGLETON =  0x200,
+        NEW_TYPE_UNKNOWN   =  0x400,
+        UNCACHEABLE_PROTO  =  0x800,
 
-        OBJECT_FLAG_MASK   = 0x3ff8
+        OBJECT_FLAG_MASK   = 0x1ff8
     };
 
   private:
@@ -277,13 +270,13 @@ class BaseShape : public js::gc::Cell
                                          * dictionary last properties. */
 
     union {
-        js::PropertyOp  rawGetter;      /* getter hook for shape */
+        PropertyOp      rawGetter;      /* getter hook for shape */
         JSObject        *getterObj;     /* user-defined callable "get" object or
                                            null if shape->hasGetterValue() */
     };
 
     union {
-        js::StrictPropertyOp rawSetter; /* setter hook for shape */
+        StrictPropertyOp rawSetter;     /* setter hook for shape */
         JSObject        *setterObj;     /* user-defined callable "set" object or
                                            null if shape->hasSetterValue() */
     };
@@ -314,8 +307,8 @@ class BaseShape : public js::gc::Cell
     inline bool matchesGetterSetter(PropertyOp rawGetter,
                                     StrictPropertyOp rawSetter) const;
 
-    inline void adoptUnowned(UnownedBaseShape *other);
-    inline void setOwned(UnownedBaseShape *unowned);
+    inline void adoptUnowned(UnrootedUnownedBaseShape other);
+    inline void setOwned(UnrootedUnownedBaseShape unowned);
 
     JSObject *getObjectParent() const { return parent; }
     uint32_t getObjectFlags() const { return flags & OBJECT_FLAG_MASK; }
@@ -334,16 +327,16 @@ class BaseShape : public js::gc::Cell
     void setSlotSpan(uint32_t slotSpan) { JS_ASSERT(isOwned()); slotSpan_ = slotSpan; }
 
     /* Lookup base shapes from the compartment's baseShapes table. */
-    static UnownedBaseShape *getUnowned(JSContext *cx, const StackBaseShape &base);
+    static UnownedBaseShape* getUnowned(JSContext *cx, const StackBaseShape &base);
 
     /* Get the canonical base shape. */
-    inline UnownedBaseShape *unowned();
+    inline UnownedBaseShape* unowned();
 
     /* Get the canonical base shape for an owned one. */
-    inline UnownedBaseShape *baseUnowned();
+    inline UnownedBaseShape* baseUnowned();
 
     /* Get the canonical base shape for an unowned one (i.e. identity). */
-    inline UnownedBaseShape *toUnowned();
+    inline UnownedBaseShape* toUnowned();
 
     /* Check that an owned base shape is consistent with its unowned base. */
     inline void assertConsistency();
@@ -353,9 +346,9 @@ class BaseShape : public js::gc::Cell
     static inline size_t offsetOfParent() { return offsetof(BaseShape, parent); }
     static inline size_t offsetOfFlags() { return offsetof(BaseShape, flags); }
 
-    static inline void writeBarrierPre(BaseShape *shape);
-    static inline void writeBarrierPost(BaseShape *shape, void *addr);
-    static inline void readBarrier(BaseShape *shape);
+    static inline void writeBarrierPre(RawBaseShape shape);
+    static inline void writeBarrierPost(RawBaseShape shape, void *addr);
+    static inline void readBarrier(RawBaseShape shape);
 
     static inline ThingRootKind rootKind() { return THING_ROOT_BASE_SHAPE; }
 
@@ -369,19 +362,19 @@ class BaseShape : public js::gc::Cell
 
 class UnownedBaseShape : public BaseShape {};
 
-UnownedBaseShape *
+UnownedBaseShape*
 BaseShape::unowned()
 {
     return isOwned() ? baseUnowned() : toUnowned();
 }
 
-UnownedBaseShape *
+UnownedBaseShape*
 BaseShape::toUnowned()
 {
-    JS_ASSERT(!isOwned() && !unowned_); return static_cast<UnownedBaseShape *>(this);
+    JS_ASSERT(!isOwned() && !unowned_); return static_cast<RawUnownedBaseShape>(this);
 }
 
-UnownedBaseShape *
+UnownedBaseShape*
 BaseShape::baseUnowned()
 {
     JS_ASSERT(isOwned() && unowned_); return unowned_;
@@ -398,7 +391,7 @@ struct StackBaseShape
     PropertyOp rawGetter;
     StrictPropertyOp rawSetter;
 
-    StackBaseShape(BaseShape *base)
+    explicit StackBaseShape(UnrootedBaseShape base)
       : flags(base->flags & BaseShape::OBJECT_FLAG_MASK),
         clasp(base->clasp),
         parent(base->parent),
@@ -414,23 +407,23 @@ struct StackBaseShape
         rawSetter(NULL)
     {}
 
-    inline StackBaseShape(Shape *shape);
+    inline StackBaseShape(UnrootedShape shape);
 
     inline void updateGetterSetter(uint8_t attrs,
                                    PropertyOp rawGetter,
                                    StrictPropertyOp rawSetter);
 
     static inline HashNumber hash(const StackBaseShape *lookup);
-    static inline bool match(UnownedBaseShape *key, const StackBaseShape *lookup);
+    static inline bool match(RawUnownedBaseShape key, const StackBaseShape *lookup);
 
     class AutoRooter : private AutoGCRooter
     {
       public:
         explicit AutoRooter(JSContext *cx, const StackBaseShape *base_
-                            JS_GUARD_OBJECT_NOTIFIER_PARAM)
+                            MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
           : AutoGCRooter(cx, STACKBASESHAPE), base(base_), skip(cx, base_)
         {
-            JS_GUARD_OBJECT_NOTIFIER_INIT;
+            MOZ_GUARD_OBJECT_NOTIFIER_INIT;
         }
 
         friend void AutoGCRooter::trace(JSTracer *trc);
@@ -438,7 +431,7 @@ struct StackBaseShape
       private:
         const StackBaseShape *base;
         SkipRoot skip;
-        JS_DECL_USE_GUARD_OBJECT_NOTIFIER
+        MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
     };
 };
 
@@ -446,10 +439,10 @@ typedef HashSet<ReadBarriered<UnownedBaseShape>,
                 StackBaseShape,
                 SystemAllocPolicy> BaseShapeSet;
 
-struct Shape : public js::gc::Cell
+class Shape : public js::gc::Cell
 {
     friend struct ::JSObject;
-    friend struct ::JSFunction;
+    friend class ::JSFunction;
     friend class js::Bindings;
     friend class js::ObjectImpl;
     friend class js::PropertyTree;
@@ -459,7 +452,7 @@ struct Shape : public js::gc::Cell
 
   protected:
     HeapPtrBaseShape    base_;
-    HeapId              propid_;
+    EncapsulatedId      propid_;
 
     JS_ENUM_HEADER(SlotInfo, uint32_t)
     {
@@ -503,12 +496,10 @@ struct Shape : public js::gc::Cell
                                    last, else to obj->shape_ */
     };
 
-    static inline Shape *search(JSContext *cx, Shape *start, jsid id,
-                                Shape ***pspp, bool adding = false);
+    static inline UnrootedShape search(JSContext *cx, Shape *start, jsid id,
+                                       Shape ***pspp, bool adding = false);
 
-#ifdef DEBUG
-    static inline Shape *searchNoAllocation(JSContext *cx, Shape *start, jsid id);
-#endif
+    static inline UnrootedShape searchNoAllocation(UnrootedShape start, jsid id);
 
     inline void removeFromDictionary(JSObject *obj);
     inline void insertIntoDictionary(HeapPtrShape *dictp);
@@ -516,28 +507,29 @@ struct Shape : public js::gc::Cell
     inline void initDictionaryShape(const StackShape &child, uint32_t nfixed,
                                     HeapPtrShape *dictp);
 
-    Shape *getChildBinding(JSContext *cx, const StackShape &child);
+    UnrootedShape getChildBinding(JSContext *cx, const StackShape &child);
 
     /* Replace the base shape of the last shape in a non-dictionary lineage with base. */
-    static Shape *replaceLastProperty(JSContext *cx, const StackBaseShape &base,
-                                      JSObject *proto, Shape *shape);
+    static UnrootedShape replaceLastProperty(JSContext *cx, const StackBaseShape &base,
+                                             TaggedProto proto, HandleShape shape);
 
-    bool hashify(JSContext *cx);
-    void handoffTableTo(Shape *newShape);
+    static bool hashify(JSContext *cx, HandleShape shape);
+    void handoffTableTo(UnrootedShape newShape);
 
-    inline void setParent(js::Shape *p);
+    inline void setParent(UnrootedShape p);
 
     bool ensureOwnBaseShape(JSContext *cx) {
         if (base()->isOwned())
             return true;
-        return makeOwnBaseShape(cx);
+        RootedShape self(cx, this);
+        return makeOwnBaseShape(cx, self);
     }
 
-    bool makeOwnBaseShape(JSContext *cx);
+    static bool makeOwnBaseShape(JSContext *cx, HandleShape shape);
 
   public:
     bool hasTable() const { return base()->hasTable(); }
-    js::ShapeTable &table() const { return base()->table(); }
+    ShapeTable &table() const { return base()->table(); }
 
     void sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf,
                              size_t *propTableSize, size_t *kidsSize) const {
@@ -558,17 +550,19 @@ struct Shape : public js::gc::Cell
 
     class Range {
       protected:
-        friend struct Shape;
-        const Shape *cursor;
+        friend class Shape;
+
+        /* |cursor| is rooted manually when necessary using Range::AutoRooter. */
+        RawShape cursor;
 
       public:
-        Range(const Shape *shape) : cursor(shape) { }
+        Range(UnrootedShape shape) : cursor(shape) { }
 
         bool empty() const {
-            return cursor->isEmptyShape();
+            return !cursor || cursor->isEmptyShape();
         }
 
-        const Shape &front() const {
+        Shape &front() const {
             JS_ASSERT(!empty());
             return *cursor;
         }
@@ -582,10 +576,10 @@ struct Shape : public js::gc::Cell
         {
           public:
             explicit AutoRooter(JSContext *cx, Range *r_
-                                JS_GUARD_OBJECT_NOTIFIER_PARAM)
+                                MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
               : AutoGCRooter(cx, SHAPERANGE), r(r_), skip(cx, r_)
             {
-                JS_GUARD_OBJECT_NOTIFIER_INIT;
+                MOZ_GUARD_OBJECT_NOTIFIER_INIT;
             }
 
             friend void AutoGCRooter::trace(JSTracer *trc);
@@ -594,19 +588,19 @@ struct Shape : public js::gc::Cell
           private:
             Range *r;
             SkipRoot skip;
-            JS_DECL_USE_GUARD_OBJECT_NOTIFIER
+            MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
         };
     };
 
-    Range all() const {
+    Range all() {
         return Range(this);
     }
 
     Class *getObjectClass() const { return base()->clasp; }
     JSObject *getObjectParent() const { return base()->parent; }
 
-    static Shape *setObjectParent(JSContext *cx, JSObject *obj, JSObject *proto, Shape *last);
-    static Shape *setObjectFlag(JSContext *cx, BaseShape::Flag flag, JSObject *proto, Shape *last);
+    static UnrootedShape setObjectParent(JSContext *cx, JSObject *obj, TaggedProto proto, Shape *last);
+    static UnrootedShape setObjectFlag(JSContext *cx, BaseShape::Flag flag, TaggedProto proto, Shape *last);
 
     uint32_t getObjectFlags() const { return base()->getObjectFlags(); }
     bool hasObjectFlag(BaseShape::Flag flag) const {
@@ -634,7 +628,7 @@ struct Shape : public js::gc::Cell
     Shape(const StackShape &other, uint32_t nfixed);
 
     /* Used by EmptyShape (see jsscopeinlines.h). */
-    Shape(UnownedBaseShape *base, uint32_t nfixed);
+    Shape(UnrootedUnownedBaseShape base, uint32_t nfixed);
 
     /* Copy constructor disabled, to avoid misuse of the above form. */
     Shape(const Shape &other) MOZ_DELETE;
@@ -666,7 +660,7 @@ struct Shape : public js::gc::Cell
     // Per ES5, decode null getterObj as the undefined value, which encodes as null.
     Value getterValue() const {
         JS_ASSERT(hasGetterValue());
-        return base()->getterObj ? js::ObjectValue(*base()->getterObj) : js::UndefinedValue();
+        return base()->getterObj ? ObjectValue(*base()->getterObj) : UndefinedValue();
     }
 
     Value getterOrUndefined() const {
@@ -683,7 +677,7 @@ struct Shape : public js::gc::Cell
     // Per ES5, decode null setterObj as the undefined value, which encodes as null.
     Value setterValue() const {
         JS_ASSERT(hasSetterValue());
-        return base()->setterObj ? js::ObjectValue(*base()->setterObj) : js::UndefinedValue();
+        return base()->setterObj ? ObjectValue(*base()->setterObj) : UndefinedValue();
     }
 
     Value setterOrUndefined() const {
@@ -692,18 +686,18 @@ struct Shape : public js::gc::Cell
                : UndefinedValue();
     }
 
-    void update(js::PropertyOp getter, js::StrictPropertyOp setter, uint8_t attrs);
+    void update(PropertyOp getter, StrictPropertyOp setter, uint8_t attrs);
 
-    inline bool matches(const Shape *other) const;
+    inline bool matches(const UnrootedShape other) const;
     inline bool matches(const StackShape &other) const;
-    inline bool matchesParamsAfterId(BaseShape *base,
+    inline bool matchesParamsAfterId(UnrootedBaseShape base,
                                      uint32_t aslot, unsigned aattrs, unsigned aflags,
                                      int ashortid) const;
 
-    bool get(JSContext* cx, HandleObject receiver, JSObject *obj, JSObject *pobj, js::Value* vp) const;
-    bool set(JSContext* cx, HandleObject obj, bool strict, js::Value* vp) const;
+    bool get(JSContext* cx, HandleObject receiver, JSObject *obj, JSObject *pobj, MutableHandleValue vp);
+    bool set(JSContext* cx, HandleObject obj, HandleObject receiver, bool strict, MutableHandleValue vp);
 
-    BaseShape *base() const { return base_; }
+    RawBaseShape base() const { return base_.get(); }
 
     bool hasSlot() const { return (attrs & JSPROP_SHARED) == 0; }
     uint32_t slot() const { JS_ASSERT(hasSlot() && !hasMissingSlot()); return maybeSlot(); }
@@ -747,12 +741,12 @@ struct Shape : public js::gc::Cell
         slotInfo = slotInfo | ((count + 1) << LINEAR_SEARCHES_SHIFT);
     }
 
-    const HeapId &propid() const {
+    const EncapsulatedId &propid() const {
         JS_ASSERT(!isEmptyShape());
         JS_ASSERT(!JSID_IS_VOID(propid_));
         return propid_;
     }
-    HeapId &propidRef() { JS_ASSERT(!JSID_IS_VOID(propid_)); return propid_; }
+    EncapsulatedId &propidRef() { JS_ASSERT(!JSID_IS_VOID(propid_)); return propid_; }
 
     int16_t shortid() const { JS_ASSERT(hasShortID()); return maybeShortid(); }
     int16_t maybeShortid() const { return shortid_; }
@@ -780,6 +774,16 @@ struct Shape : public js::gc::Cell
         return (attrs & (JSPROP_SETTER | JSPROP_GETTER)) != 0;
     }
 
+    PropDesc::Writability writability() const {
+        return (attrs & JSPROP_READONLY) ? PropDesc::NonWritable : PropDesc::Writable;
+    }
+    PropDesc::Enumerability enumerability() const {
+        return (attrs & JSPROP_ENUMERATE) ? PropDesc::Enumerable : PropDesc::NonEnumerable;
+    }
+    PropDesc::Configurability configurability() const {
+        return (attrs & JSPROP_PERMANENT) ? PropDesc::NonConfigurable : PropDesc::Configurable;
+    }
+
     /*
      * For ES5 compatibility, we allow properties with PropertyOp-flavored
      * setters to be shadowed when set. The "own" property thereby created in
@@ -791,68 +795,22 @@ struct Shape : public js::gc::Cell
         return hasSlot() || (attrs & JSPROP_SHADOWABLE);
     }
 
-    /*
-     * Sometimes call objects and run-time block objects need unique shapes, but
-     * sometimes they don't.
-     *
-     * Property cache entries only record the shapes of the first and last
-     * objects along the search path, so if the search traverses more than those
-     * two objects, then those first and last shapes must determine the shapes
-     * of everything else along the path. The js_PurgeScopeChain stuff takes
-     * care of making this work, but that suffices only because we require that
-     * start points with the same shape have the same successor object in the
-     * search path --- a cache hit means the starting shapes were equal, which
-     * means the search path tail (everything but the first object in the path)
-     * was shared, which in turn means the effects of a purge will be seen by
-     * all affected starting search points.
-     *
-     * For call and run-time block objects, the "successor object" is the scope
-     * chain parent. Unlike prototype objects (of which there are usually few),
-     * scope chain parents are created frequently (possibly on every call), so
-     * following the shape-implies-parent rule blindly would lead one to give
-     * every call and block its own shape.
-     *
-     * In many cases, however, it's not actually necessary to give call and
-     * block objects their own shapes, and we can do better. If the code will
-     * always be used with the same global object, and none of the enclosing
-     * call objects could have bindings added to them at runtime (by direct eval
-     * calls or function statements), then we can use a fixed set of shapes for
-     * those objects. You could think of the shapes in the functions' bindings
-     * and compile-time blocks as uniquely identifying the global object(s) at
-     * the end of the scope chain.
-     *
-     * (In fact, some JSScripts we do use against multiple global objects (see
-     * bug 618497), and using the fixed shapes isn't sound there.)
-     *
-     * In deciding whether a call or block has any extensible parents, we
-     * actually only need to consider enclosing calls; blocks are never
-     * extensible, and the other sorts of objects that appear in the scope
-     * chains ('with' blocks, say) are not CacheableNonGlobalScopes.
-     *
-     * If the hasExtensibleParents flag is set for the last property in a
-     * script's bindings or a compiler-generated Block object, then created
-     * Call or Block objects need unique shapes. If the flag is clear, then we
-     * can use lastBinding's shape.
-     */
-    static Shape *setExtensibleParents(JSContext *cx, Shape *shape);
-    bool extensibleParents() const { return !!(base()->flags & BaseShape::EXTENSIBLE_PARENTS); }
-
-    uint32_t entryCount() const {
+    uint32_t entryCount() {
         if (hasTable())
             return table().entryCount;
 
-        const js::Shape *shape = this;
+        UnrootedShape shape = this;
         uint32_t count = 0;
-        for (js::Shape::Range r = shape->all(); !r.empty(); r.popFront())
+        for (Shape::Range r = shape->all(); !r.empty(); r.popFront())
             ++count;
         return count;
     }
 
-    bool isBigEnoughForAShapeTable() const {
+    bool isBigEnoughForAShapeTable() {
         JS_ASSERT(!hasTable());
-        const js::Shape *shape = this;
+        UnrootedShape shape = this;
         uint32_t count = 0;
-        for (js::Shape::Range r = shape->all(); !r.empty(); r.popFront()) {
+        for (Shape::Range r = shape->all(); !r.empty(); r.popFront()) {
             ++count;
             if (count >= ShapeTable::MIN_ENTRIES)
                 return true;
@@ -866,22 +824,27 @@ struct Shape : public js::gc::Cell
 #endif
 
     void finalize(FreeOp *fop);
-    void removeChild(js::Shape *child);
+    void removeChild(UnrootedShape child);
 
-    static inline void writeBarrierPre(const Shape *shape);
-    static inline void writeBarrierPost(const Shape *shape, void *addr);
+    static inline void writeBarrierPre(UnrootedShape shape);
+    static inline void writeBarrierPost(UnrootedShape shape, void *addr);
 
     /*
      * All weak references need a read barrier for incremental GC. This getter
      * method implements the read barrier. It's used to obtain initial shapes
      * from the compartment.
      */
-    static inline void readBarrier(const Shape *shape);
+    static inline void readBarrier(UnrootedShape shape);
 
     static inline ThingRootKind rootKind() { return THING_ROOT_SHAPE; }
 
     inline void markChildren(JSTracer *trc);
 
+    inline UnrootedShape search(JSContext *cx, jsid id) {
+        Shape **_;
+        return search(cx, this, id, &_);
+    }
+
     /* For JIT usage */
     static inline size_t offsetOfBase() { return offsetof(Shape, base_); }
 
@@ -903,7 +866,10 @@ class AutoRooterGetterSetter
             : AutoGCRooter(cx, GETTERSETTER), attrs(attrs),
               pgetter(pgetter_), psetter(psetter_),
               getterRoot(cx, pgetter_), setterRoot(cx, psetter_)
-        {}
+        {
+            JS_ASSERT_IF(attrs & JSPROP_GETTER, !IsPoisonedPtr(*pgetter));
+            JS_ASSERT_IF(attrs & JSPROP_SETTER, !IsPoisonedPtr(*psetter));
+        }
 
         friend void AutoGCRooter::trace(JSTracer *trc);
 
@@ -917,29 +883,29 @@ class AutoRooterGetterSetter
   public:
     explicit AutoRooterGetterSetter(JSContext *cx, uint8_t attrs,
                                     PropertyOp *pgetter, StrictPropertyOp *psetter
-                                    JS_GUARD_OBJECT_NOTIFIER_PARAM)
+                                    MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
     {
         if (attrs & (JSPROP_GETTER | JSPROP_SETTER))
             inner.construct(cx, attrs, pgetter, psetter);
-        JS_GUARD_OBJECT_NOTIFIER_INIT;
+        MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     }
 
     friend void AutoGCRooter::trace(JSTracer *trc);
 
   private:
-    Maybe<Inner> inner;
-    JS_DECL_USE_GUARD_OBJECT_NOTIFIER
+    mozilla::Maybe<Inner> inner;
+    MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 };
 
 struct EmptyShape : public js::Shape
 {
-    EmptyShape(UnownedBaseShape *base, uint32_t nfixed);
+    EmptyShape(UnrootedUnownedBaseShape base, uint32_t nfixed);
 
     /*
      * Lookup an initial shape matching the given parameters, creating an empty
      * shape if none was found.
      */
-    static Shape *getInitialShape(JSContext *cx, Class *clasp, JSObject *proto,
+    static Shape *getInitialShape(JSContext *cx, Class *clasp, TaggedProto proto,
                                   JSObject *parent, gc::AllocKind kind, uint32_t objectFlags = 0);
 
     /*
@@ -947,7 +913,7 @@ struct EmptyShape : public js::Shape
      * getInitialShape calls, until the new shape becomes unreachable in a GC
      * and the table entry is purged.
      */
-    static void insertInitialShape(JSContext *cx, Shape *shape, JSObject *proto);
+    static void insertInitialShape(JSContext *cx, HandleShape shape, HandleObject proto);
 };
 
 /*
@@ -967,16 +933,16 @@ struct InitialShapeEntry
      * Matching prototype for the entry. The shape of an object determines its
      * prototype, but the prototype cannot be determined from the shape itself.
      */
-    JSObject *proto;
+    TaggedProto proto;
 
     /* State used to determine a match on an initial shape. */
     struct Lookup {
         Class *clasp;
-        JSObject *proto;
+        TaggedProto proto;
         JSObject *parent;
         uint32_t nfixed;
         uint32_t baseFlags;
-        Lookup(Class *clasp, JSObject *proto, JSObject *parent, uint32_t nfixed,
+        Lookup(Class *clasp, TaggedProto proto, JSObject *parent, uint32_t nfixed,
                uint32_t baseFlags)
             : clasp(clasp), proto(proto), parent(parent),
               nfixed(nfixed), baseFlags(baseFlags)
@@ -984,9 +950,9 @@ struct InitialShapeEntry
     };
 
     inline InitialShapeEntry();
-    inline InitialShapeEntry(const ReadBarriered<Shape> &shape, JSObject *proto);
+    inline InitialShapeEntry(const ReadBarriered<Shape> &shape, TaggedProto proto);
 
-    inline Lookup getLookup();
+    inline Lookup getLookup() const;
 
     static inline HashNumber hash(const Lookup &lookup);
     static inline bool match(const InitialShapeEntry &key, const Lookup &lookup);
@@ -996,15 +962,16 @@ typedef HashSet<InitialShapeEntry, InitialShapeEntry, SystemAllocPolicy> Initial
 
 struct StackShape
 {
-    UnownedBaseShape *base;
-    jsid             propid;
-    uint32_t         slot_;
-    uint8_t          attrs;
-    uint8_t          flags;
-    int16_t          shortid;
+    /* For performance, StackShape only roots when absolutely necessary. */
+    RawUnownedBaseShape base;
+    RawId               propid;
+    uint32_t            slot_;
+    uint8_t             attrs;
+    uint8_t             flags;
+    int16_t             shortid;
 
-    StackShape(UnownedBaseShape *base, jsid propid, uint32_t slot,
-               uint32_t nfixed, unsigned attrs, unsigned flags, int shortid)
+    explicit StackShape(UnrootedUnownedBaseShape base, jsid propid, uint32_t slot,
+                        uint32_t nfixed, unsigned attrs, unsigned flags, int shortid)
       : base(base),
         propid(propid),
         slot_(slot),
@@ -1017,9 +984,9 @@ struct StackShape
         JS_ASSERT(slot <= SHAPE_INVALID_SLOT);
     }
 
-    StackShape(const Shape *shape)
+    StackShape(const UnrootedShape &shape)
       : base(shape->base()->unowned()),
-        propid(const_cast<Shape *>(shape)->propidRef()),
+        propid(shape->propidRef()),
         slot_(shape->slotInfo & Shape::SLOT_MASK),
         attrs(shape->attrs),
         flags(shape->flags),
@@ -1048,10 +1015,10 @@ struct StackShape
     {
       public:
         explicit AutoRooter(JSContext *cx, const StackShape *shape_
-                            JS_GUARD_OBJECT_NOTIFIER_PARAM)
+                            MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
           : AutoGCRooter(cx, STACKSHAPE), shape(shape_), skip(cx, shape_)
         {
-            JS_GUARD_OBJECT_NOTIFIER_INIT;
+            MOZ_GUARD_OBJECT_NOTIFIER_INIT;
         }
 
         friend void AutoGCRooter::trace(JSTracer *trc);
@@ -1059,7 +1026,7 @@ struct StackShape
       private:
         const StackShape *shape;
         SkipRoot skip;
-        JS_DECL_USE_GUARD_OBJECT_NOTIFIER
+        MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
     };
  };
 
@@ -1067,28 +1034,37 @@ struct StackShape
 
 /* js::Shape pointer tag bit indicating a collision. */
 #define SHAPE_COLLISION                 (uintptr_t(1))
-#define SHAPE_REMOVED                   ((js::Shape *) SHAPE_COLLISION)
+#define SHAPE_REMOVED                   ((RawShape) SHAPE_COLLISION)
 
 /* Macros to get and set shape pointer values and collision flags. */
 #define SHAPE_IS_FREE(shape)            ((shape) == NULL)
 #define SHAPE_IS_REMOVED(shape)         ((shape) == SHAPE_REMOVED)
 #define SHAPE_IS_LIVE(shape)            ((shape) > SHAPE_REMOVED)
-#define SHAPE_FLAG_COLLISION(spp,shape) (*(spp) = (js::Shape *)               \
+#define SHAPE_FLAG_COLLISION(spp,shape) (*(spp) = (RawShape)                  \
                                          (uintptr_t(shape) | SHAPE_COLLISION))
 #define SHAPE_HAD_COLLISION(shape)      (uintptr_t(shape) & SHAPE_COLLISION)
 #define SHAPE_FETCH(spp)                SHAPE_CLEAR_COLLISION(*(spp))
 
 #define SHAPE_CLEAR_COLLISION(shape)                                          \
-    ((js::Shape *) (uintptr_t(shape) & ~SHAPE_COLLISION))
+    ((RawShape) (uintptr_t(shape) & ~SHAPE_COLLISION))
 
 #define SHAPE_STORE_PRESERVING_COLLISION(spp, shape)                          \
-    (*(spp) = (js::Shape *) (uintptr_t(shape) | SHAPE_HAD_COLLISION(*(spp))))
+    (*(spp) = (RawShape) (uintptr_t(shape) | SHAPE_HAD_COLLISION(*(spp))))
 
 namespace js {
 
-inline Shape *
+inline UnrootedShape
 Shape::search(JSContext *cx, Shape *start, jsid id, Shape ***pspp, bool adding)
 {
+    AssertCanGC();
+#ifdef DEBUG
+    {
+        SkipRoot skip0(cx, &start);
+        SkipRoot skip1(cx, &id);
+        MaybeCheckStackRoots(cx);
+    }
+#endif
+
     if (start->inDictionary()) {
         *pspp = start->table().search(id, adding);
         return SHAPE_FETCH(*pspp);
@@ -1105,8 +1081,8 @@ Shape::search(JSContext *cx, Shape *start, jsid id, Shape ***pspp, bool adding)
         if (start->isBigEnoughForAShapeTable()) {
             RootedShape startRoot(cx, start);
             RootedId idRoot(cx, id);
-            if (start->hashify(cx)) {
-                Shape **spp = start->table().search(id, adding);
+            if (Shape::hashify(cx, startRoot)) {
+                Shape **spp = startRoot->table().search(idRoot, adding);
                 return SHAPE_FETCH(spp);
             }
             start = startRoot;
@@ -1121,31 +1097,35 @@ Shape::search(JSContext *cx, Shape *start, jsid id, Shape ***pspp, bool adding)
         start->incrementNumLinearSearches();
     }
 
-    for (Shape *shape = start; shape; shape = shape->parent) {
+    for (UnrootedShape shape = start; shape; shape = shape->parent) {
         if (shape->propidRef() == id)
             return shape;
     }
 
-    return NULL;
+    return UnrootedShape(NULL);
 }
 
-#ifdef DEBUG
-/* static */ inline Shape *
-Shape::searchNoAllocation(JSContext *cx, Shape *start, jsid id)
+/* static */ inline UnrootedShape
+Shape::searchNoAllocation(UnrootedShape start, jsid id)
 {
     if (start->hasTable()) {
         Shape **spp = start->table().search(id, false);
         return SHAPE_FETCH(spp);
     }
 
-    for (Shape *shape = start; shape; shape = shape->parent) {
+    for (UnrootedShape shape = start; shape; shape = shape->parent) {
         if (shape->propidRef() == id)
             return shape;
     }
 
-    return NULL;
+    return UnrootedShape(NULL);
 }
-#endif /* DEBUG */
+
+void
+MarkNonNativePropertyFound(HandleObject obj, MutableHandleShape propp);
+
+template<> struct RootKind<Shape *> : SpecificRootKind<Shape *, THING_ROOT_SHAPE> {};
+template<> struct RootKind<BaseShape *> : SpecificRootKind<BaseShape *, THING_ROOT_BASE_SHAPE> {};
 
 } // namespace js
 
@@ -1155,8 +1135,8 @@ Shape::searchNoAllocation(JSContext *cx, Shape *start, jsid id)
 #endif
 
 namespace JS {
-    template<> class AnchorPermitted<js::Shape *> { };
-    template<> class AnchorPermitted<const js::Shape *> { };
+template<> class AnchorPermitted<js::Shape *> { };
+template<> class AnchorPermitted<const js::Shape *> { };
 }
 
 #endif /* jsscope_h___ */
