@@ -9,6 +9,7 @@
 #define ObjectImpl_h___
 
 #include "mozilla/Assertions.h"
+#include "mozilla/GuardObjects.h"
 #include "mozilla/StandardInteger.h"
 
 #include "jsfriendapi.h"
@@ -16,12 +17,15 @@
 #include "jsval.h"
 
 #include "gc/Barrier.h"
+#include "gc/Heap.h"
 #include "vm/NumericConversions.h"
+#include "vm/String.h"
 
 namespace js {
 
 class Debugger;
 class ObjectImpl;
+ForwardDeclare(Shape);
 
 class AutoPropDescArrayRooter;
 
@@ -37,6 +41,58 @@ CastAsStrictPropertyOp(JSObject *object)
     return JS_DATA_TO_FUNC_PTR(StrictPropertyOp, object);
 }
 
+/*
+ * Properties are stored differently depending on the type of the key.  If the
+ * key is an unsigned 32-bit integer (i.e. an index), we call such properties
+ * "elements" and store them in one of a number of forms (optimized for dense
+ * property storage, typed array data, and so on).  All other properties are
+ * stored using shapes and shape trees.  Keys for these properties are either
+ * PropertyNames (that is, atomized strings whose contents are not unsigned
+ * 32-bit integers) or SpecialIds (object values for E4X and a couple other
+ * things, see jsid for details); the union of these types, used in individual
+ * shapes, is PropertyId.
+ */
+class PropertyId
+{
+    jsid id;
+
+  public:
+    bool isName() const {
+        MOZ_ASSERT(JSID_IS_STRING(id) || JSID_IS_SPECIAL(id));
+        return JSID_IS_STRING(id);
+    }
+    bool isSpecial() const {
+        MOZ_ASSERT(JSID_IS_STRING(id) || JSID_IS_SPECIAL(id));
+        return !isName();
+    }
+
+    PropertyId() {
+        *this = PropertyId(SpecialId());
+    }
+    explicit PropertyId(PropertyName *name)
+      : id(NON_INTEGER_ATOM_TO_JSID(name))
+    { }
+    explicit PropertyId(const SpecialId &sid)
+      : id(SPECIALID_TO_JSID(sid))
+    { }
+
+    PropertyName * asName() const {
+        return JSID_TO_STRING(id)->asAtom().asPropertyName();
+    }
+    SpecialId asSpecial() const {
+        return JSID_TO_SPECIALID(id);
+    }
+    const jsid &asId() const {
+        return id;
+    }
+    jsid &asId() {
+        return id;
+    }
+
+    bool operator==(const PropertyId &rhs) const { return id == rhs.id; }
+    bool operator!=(const PropertyId &rhs) const { return id != rhs.id; }
+};
+
 /*
  * A representation of ECMA-262 ed. 5's internal Property Descriptor data
  * structure.
@@ -102,6 +158,9 @@ struct PropDesc {
         isUndefined_(false)
     {}
 
+    inline PropDesc(const Value &getter, const Value &setter,
+                    Enumerability enumerable, Configurability configurable);
+
     /*
      * 8.10.5 ToPropertyDescriptor(Obj)
      *
@@ -112,7 +171,16 @@ struct PropDesc {
      * are expected to be Debugger.Object wrappers of functions, which are not
      * themselves callable.)
      */
-    bool initialize(JSContext* cx, const Value &v, bool checkAccessors = true);
+    bool initialize(JSContext *cx, const Value &v, bool checkAccessors = true);
+
+    /*
+     * If IsGenericDescriptor(desc) or IsDataDescriptor(desc) is true, then if
+     * the value of an attribute field of desc, considered as a data
+     * descriptor, is absent, set it to its default value. Else if the value of
+     * an attribute field of desc, considered as an attribute descriptor, is
+     * absent, set it to its default value.
+     */
+    void complete();
 
     /*
      * 8.10.4 FromPropertyDescriptor(Desc)
@@ -175,9 +243,9 @@ struct PropDesc {
         return (attrs & JSPROP_READONLY) == 0;
     }
 
-    const Value & value() const {
+    HandleValue value() const {
         MOZ_ASSERT(hasValue());
-        return value_;
+        return HandleValue::fromMarkedLocation(&value_);
     }
 
     JSObject * getterObject() const {
@@ -191,15 +259,15 @@ struct PropDesc {
         return set_.isUndefined() ? NULL : &set_.toObject();
     }
 
-    const Value & getterValue() const {
+    HandleValue getterValue() const {
         MOZ_ASSERT(!isUndefined());
         MOZ_ASSERT(hasGet());
-        return get_;
+        return HandleValue::fromMarkedLocation(&get_);
     }
-    const Value & setterValue() const {
+    HandleValue setterValue() const {
         MOZ_ASSERT(!isUndefined());
         MOZ_ASSERT(hasSet());
-        return set_;
+        return HandleValue::fromMarkedLocation(&set_);
     }
 
     /*
@@ -231,10 +299,10 @@ struct PropDesc {
     {
       public:
         explicit AutoRooter(JSContext *cx, PropDesc *pd_
-                            JS_GUARD_OBJECT_NOTIFIER_PARAM)
+                            MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
           : AutoGCRooter(cx, PROPDESC), pd(pd_), skip(cx, pd_)
         {
-            JS_GUARD_OBJECT_NOTIFIER_INIT;
+            MOZ_GUARD_OBJECT_NOTIFIER_INIT;
         }
 
         friend void AutoGCRooter::trace(JSTracer *trc);
@@ -242,7 +310,7 @@ struct PropDesc {
       private:
         PropDesc *pd;
         SkipRoot skip;
-        JS_DECL_USE_GUARD_OBJECT_NOTIFIER
+        MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
      };
 };
 
@@ -292,8 +360,12 @@ class ElementsHeader
         } dense;
         class {
             friend class SparseElementsHeader;
-            Shape * shape;
+            RawShape shape;
         } sparse;
+        class {
+            friend class ArrayBufferElementsHeader;
+            JSObject * views;
+        } buffer;
     };
 
     void staticAsserts() {
@@ -358,13 +430,15 @@ class DenseElementsHeader : public ElementsHeader
         return ElementsHeader::length;
     }
 
-    bool getOwnElement(JSContext *cx, ObjectImpl *obj, uint32_t index, PropDesc *desc);
+    bool getOwnElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       unsigned resolveFlags, PropDesc *desc);
 
-    bool defineElement(JSContext *cx, ObjectImpl *obj, uint32_t index, const PropDesc &desc,
-                       bool shouldThrow, bool *succeeded);
+    bool defineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       const PropDesc &desc, bool shouldThrow, unsigned resolveFlags,
+                       bool *succeeded);
 
-    bool setElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index,
-                    const Value &v, bool *succeeded);
+    bool setElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+                    uint32_t index, const Value &v, unsigned resolveFlags, bool *succeeded);
 
   private:
     inline bool isDenseElements() const MOZ_DELETE;
@@ -377,7 +451,7 @@ class DenseElementsHeader : public ElementsHeader
 class SparseElementsHeader : public ElementsHeader
 {
   public:
-    Shape * shape() {
+    UnrootedShape shape() {
         MOZ_ASSERT(ElementsHeader::isSparseElements());
         return sparse.shape;
     }
@@ -387,13 +461,15 @@ class SparseElementsHeader : public ElementsHeader
         return ElementsHeader::length;
     }
 
-    bool getOwnElement(JSContext *cx, ObjectImpl *obj, uint32_t index, PropDesc *desc);
+    bool getOwnElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       unsigned resolveFlags, PropDesc *desc);
 
-    bool defineElement(JSContext *cx, ObjectImpl *obj, uint32_t index, const PropDesc &desc,
-                       bool shouldThrow, bool *succeeded);
+    bool defineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       const PropDesc &desc, bool shouldThrow, unsigned resolveFlags,
+                       bool *succeeded);
 
-    bool setElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index,
-                    const Value &v, bool *succeeded);
+    bool setElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+                    uint32_t index, const Value &v, unsigned resolveFlags, bool *succeeded);
 
   private:
     inline bool isSparseElements() const MOZ_DELETE;
@@ -511,13 +587,15 @@ class TypedElementsHeader : public ElementsHeader
         return ElementsHeader::length;
     }
 
-    bool getOwnElement(JSContext *cx, ObjectImpl *obj, uint32_t index, PropDesc *desc);
+    bool getOwnElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       unsigned resolveFlags, PropDesc *desc);
 
-    bool defineElement(JSContext *cx, ObjectImpl *obj, uint32_t index, const PropDesc &desc,
-                       bool shouldThrow, bool *succeeded);
+    bool defineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       const PropDesc &desc, bool shouldThrow, unsigned resolveFlags,
+                       bool *succeeded);
 
-    bool setElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index,
-                    const Value &v, bool *succeeded);
+    bool setElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+                    uint32_t index, const Value &v, unsigned resolveFlags, bool *succeeded);
 
   private:
     TypedElementsHeader(const TypedElementsHeader &other) MOZ_DELETE;
@@ -670,13 +748,17 @@ class Uint8ClampedElementsHeader : public TypedElementsHeader<uint8_clamped>
 class ArrayBufferElementsHeader : public ElementsHeader
 {
   public:
-    bool getOwnElement(JSContext *cx, ObjectImpl *obj, uint32_t index, PropDesc *desc);
+    bool getOwnElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       unsigned resolveFlags, PropDesc *desc);
 
-    bool defineElement(JSContext *cx, ObjectImpl *obj, uint32_t index, const PropDesc &desc,
-                       bool shouldThrow, bool *succeeded);
+    bool defineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index,
+                       const PropDesc &desc, bool shouldThrow, unsigned resolveFlags,
+                       bool *succeeded);
 
-    bool setElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index,
-                    const Value &v, bool *succeeded);
+    bool setElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+                    uint32_t index, const Value &v, unsigned resolveFlags, bool *succeeded);
+
+    JSObject **viewList() { return &buffer.views; }
 
   private:
     inline bool isArrayBufferElements() const MOZ_DELETE;
@@ -835,7 +917,7 @@ extern HeapSlot *emptyObjectElements;
 struct Class;
 struct GCMarker;
 struct ObjectOps;
-struct Shape;
+class Shape;
 
 class NewObjectCache;
 
@@ -979,11 +1061,9 @@ class ObjectImpl : public gc::Cell
 
   protected:
     friend struct GCMarker;
-    friend struct Shape;
+    friend class Shape;
     friend class NewObjectCache;
 
-    inline bool hasContiguousSlots(uint32_t start, uint32_t count) const;
-
     inline void invalidateSlotRange(uint32_t start, uint32_t count);
     inline void initializeSlotRange(uint32_t start, uint32_t count);
 
@@ -1077,11 +1157,21 @@ class ObjectImpl : public gc::Cell
     /* Compute dynamicSlotsCount() for this object. */
     inline uint32_t numDynamicSlots() const;
 
-    const Shape * nativeLookup(JSContext *cx, jsid id);
+    UnrootedShape nativeLookup(JSContext *cx, jsid id);
+    inline UnrootedShape nativeLookup(JSContext *cx, PropertyId pid);
+    inline UnrootedShape nativeLookup(JSContext *cx, PropertyName *name);
 
-#ifdef DEBUG
-    const Shape * nativeLookupNoAllocation(JSContext *cx, jsid id);
-#endif
+    UnrootedShape nativeLookupNoAllocation(jsid id);
+    inline UnrootedShape nativeLookupNoAllocation(PropertyId pid);
+    inline UnrootedShape nativeLookupNoAllocation(PropertyName *name);
+
+    inline bool nativeContains(JSContext *cx, Handle<jsid> id);
+    inline bool nativeContains(JSContext *cx, Handle<PropertyName*> name);
+    inline bool nativeContains(JSContext *cx, Handle<Shape*> shape);
+
+    inline bool nativeContainsNoAllocation(jsid id);
+    inline bool nativeContainsNoAllocation(PropertyName *name);
+    inline bool nativeContainsNoAllocation(Shape &shape);
 
     inline Class *getClass() const;
     inline JSClass *getJSClass() const;
@@ -1140,7 +1230,9 @@ class ObjectImpl : public gc::Cell
     inline const Value &nativeGetSlot(uint32_t slot) const;
 
     inline void setSlot(uint32_t slot, const Value &value);
+    inline void setCrossCompartmentSlot(uint32_t slot, const Value &value);
     inline void initSlot(uint32_t slot, const Value &value);
+    inline void initCrossCompartmentSlot(uint32_t slot, const Value &value);
     inline void initSlotUnchecked(uint32_t slot, const Value &value);
 
     /* For slots which are known to always be fixed, due to the way they are allocated. */
@@ -1201,11 +1293,12 @@ class ObjectImpl : public gc::Cell
     }
 
     /* GC support. */
+    static inline ThingRootKind rootKind() { return THING_ROOT_OBJECT; }
     static inline void readBarrier(ObjectImpl *obj);
     static inline void writeBarrierPre(ObjectImpl *obj);
     static inline void writeBarrierPost(ObjectImpl *obj, void *addr);
     inline void privateWriteBarrierPre(void **oldval);
-    inline void privateWriteBarrierPost(void **oldval);
+    inline void privateWriteBarrierPost(void **pprivate);
     void markChildren(JSTracer *trc);
 
     /* Private data accessors. */
@@ -1215,6 +1308,7 @@ class ObjectImpl : public gc::Cell
     inline bool hasPrivate() const;
     inline void *getPrivate() const;
     inline void setPrivate(void *data);
+    inline void setPrivateGCThing(gc::Cell *cell);
     inline void setPrivateUnbarriered(void *data);
     inline void initPrivate(void *data);
 
@@ -1248,24 +1342,76 @@ ObjectValue(ObjectImpl &obj)
     return v;
 }
 
+inline Handle<JSObject*>
+Downcast(Handle<ObjectImpl*> obj)
+{
+    return Handle<JSObject*>::fromMarkedLocation(reinterpret_cast<JSObject* const*>(obj.address()));
+}
+
+extern JSObject *
+ArrayBufferDelegate(JSContext *cx, Handle<ObjectImpl*> obj);
+
+/* Generic [[GetOwnProperty]] method. */
 bool
-GetOwnElement(JSContext *cx, ObjectImpl *obj, uint32_t index, PropDesc *desc);
+GetOwnElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index, unsigned resolveFlags,
+              PropDesc *desc);
+extern bool
+GetOwnProperty(JSContext *cx, Handle<ObjectImpl*> obj, PropertyId pid, unsigned resolveFlags,
+               PropDesc *desc);
+inline bool
+GetOwnProperty(JSContext *cx, Handle<ObjectImpl*> obj, Handle<PropertyName*> name,
+               unsigned resolveFlags, PropDesc *desc)
+{
+    return GetOwnProperty(cx, obj, PropertyId(name), resolveFlags, desc);
+}
+inline bool
+GetOwnProperty(JSContext *cx, Handle<ObjectImpl*> obj, Handle<SpecialId> sid, unsigned resolveFlags,
+               PropDesc *desc)
+{
+    return GetOwnProperty(cx, obj, PropertyId(sid), resolveFlags, desc);
+}
 
 /* Proposed default [[GetP]](Receiver, P) method. */
 extern bool
-GetElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index, Value *vp);
+GetElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver, uint32_t index,
+           unsigned resolveFlags, Value *vp);
+extern bool
+GetProperty(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+            Handle<PropertyId> pid, unsigned resolveFlags, MutableHandle<Value> vp);
+inline bool
+GetProperty(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+            Handle<PropertyName*> name, unsigned resolveFlags, MutableHandle<Value> vp)
+{
+    Rooted<PropertyId> pid(cx, PropertyId(name));
+    return GetProperty(cx, obj, receiver, pid, resolveFlags, vp);
+}
+inline bool
+GetProperty(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver,
+            Handle<SpecialId> sid, unsigned resolveFlags, MutableHandle<Value> vp)
+{
+    Rooted<PropertyId> pid(cx, PropertyId(sid));
+    return GetProperty(cx, obj, receiver, pid, resolveFlags, vp);
+}
 
 extern bool
-DefineElement(JSContext *cx, ObjectImpl *obj, uint32_t index, const PropDesc &desc,
-              bool shouldThrow, bool *succeeded);
+DefineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index, const PropDesc &desc,
+              bool shouldThrow, unsigned resolveFlags, bool *succeeded);
 
 /* Proposed default [[SetP]](Receiver, P, V) method. */
 extern bool
-SetElement(JSContext *cx, ObjectImpl *obj, ObjectImpl *receiver, uint32_t index, const Value &v,
-           bool *succeeded);
+SetElement(JSContext *cx, Handle<ObjectImpl*> obj, Handle<ObjectImpl*> receiver, uint32_t index,
+           const Value &v, unsigned resolveFlags, bool *succeeded);
 
 extern bool
-HasElement(JSContext *cx, ObjectImpl *obj, uint32_t index, bool *found);
+HasElement(JSContext *cx, Handle<ObjectImpl*> obj, uint32_t index, unsigned resolveFlags,
+           bool *found);
+
+template <> struct RootMethods<PropertyId>
+{
+    static PropertyId initial() { return PropertyId(); }
+    static ThingRootKind kind() { return THING_ROOT_PROPERTY_ID; }
+    static bool poisoned(PropertyId propid) { return IsPoisonedId(propid.asId()); }
+};
 
 } /* namespace js */
 
