@@ -6,6 +6,8 @@
 
 /* state used in reflow of block frames */
 
+#include "mozilla/DebugOnly.h"
+
 #include "nsBlockReflowContext.h"
 #include "nsBlockReflowState.h"
 #include "nsBlockFrame.h"
@@ -19,8 +21,6 @@
 
 #include "nsINameSpaceManager.h"
 
-#include "mozilla/Util.h" // for DebugOnly
-
 #ifdef DEBUG
 #include "nsBlockDebugFlags.h"
 #endif
@@ -38,14 +38,14 @@ nsBlockReflowState::nsBlockReflowState(const nsHTMLReflowState& aReflowState,
   : mBlock(aFrame),
     mPresContext(aPresContext),
     mReflowState(aReflowState),
-    mPushedFloats(nsnull),
-    mOverflowTracker(nsnull),
+    mPushedFloats(nullptr),
+    mOverflowTracker(nullptr),
     mPrevBottomMargin(),
     mLineNumber(0),
     mFlags(0),
     mFloatBreakType(NS_STYLE_CLEAR_NONE)
 {
-  SetFlag(BRS_ISFIRSTINFLOW, aFrame->GetPrevInFlow() == nsnull);
+  SetFlag(BRS_ISFIRSTINFLOW, aFrame->GetPrevInFlow() == nullptr);
   SetFlag(BRS_ISOVERFLOWCONTAINER,
           IS_TRUE_OVERFLOW_CONTAINER(aFrame));
 
@@ -107,7 +107,7 @@ nsBlockReflowState::nsBlockReflowState(const nsHTMLReflowState& aReflowState,
   mContentArea.x = borderPadding.left;
   mY = mContentArea.y = borderPadding.top;
 
-  mPrevChild = nsnull;
+  mPrevChild = nullptr;
   mCurrentLine = aFrame->end_lines();
 
   mMinLineHeight = aReflowState.CalcLineHeight();
@@ -670,7 +670,7 @@ nsBlockReflowState::FlowAndPlaceFloat(nsIFrame* aFloat)
 
       // see if the previous float is also a table and has "align"
       nsFloatCache* fc = mCurrentLineFloats.Head();
-      nsIFrame* prevFrame = nsnull;
+      nsIFrame* prevFrame = nullptr;
       while (fc) {
         if (fc->mFloat == aFloat) {
           break;
@@ -760,15 +760,29 @@ nsBlockReflowState::FlowAndPlaceFloat(nsIFrame* aFloat)
   // (This code is only for DISABLE_FLOAT_BREAKING_IN_COLUMNS .)
   //
   // Likewise, if none of the float fit, and it needs to be pushed in
-  // its entirety to the next page (NS_FRAME_IS_TRUNCATED), we need to
-  // do the same.
+  // its entirety to the next page (NS_FRAME_IS_TRUNCATED or
+  // NS_INLINE_IS_BREAK_BEFORE), we need to do the same.
   if ((mContentArea.height != NS_UNCONSTRAINEDSIZE &&
        adjustedAvailableSpace.height == NS_UNCONSTRAINEDSIZE &&
        !mustPlaceFloat &&
        aFloat->GetSize().height + floatMargin.TopBottom() >
          mContentArea.YMost() - floatY) ||
-      NS_FRAME_IS_TRUNCATED(reflowStatus)) {
+      NS_FRAME_IS_TRUNCATED(reflowStatus) ||
+      NS_INLINE_IS_BREAK_BEFORE(reflowStatus)) {
+    PushFloatPastBreak(aFloat);
+    return false;
+  }
 
+  // We can't use aFloat->ShouldAvoidBreakInside(mReflowState) here since
+  // its mIsTopOfPage may be true even though the float isn't at the
+  // top when floatY > 0.
+  if (mContentArea.height != NS_UNCONSTRAINEDSIZE &&
+      !mustPlaceFloat && (!mReflowState.mFlags.mIsTopOfPage || floatY > 0) &&
+      NS_STYLE_PAGE_BREAK_AVOID == aFloat->GetStyleDisplay()->mBreakInside &&
+      (!NS_FRAME_IS_FULLY_COMPLETE(reflowStatus) ||
+       aFloat->GetSize().height + floatMargin.TopBottom() >
+         mContentArea.YMost() - floatY) &&
+      !aFloat->GetPrevInFlow()) {
     PushFloatPastBreak(aFloat);
     return false;
   }
@@ -790,7 +804,6 @@ nsBlockReflowState::FlowAndPlaceFloat(nsIFrame* aFloat)
     aFloat->SetPosition(origin);
     nsContainerFrame::PositionFrameView(aFloat);
     nsContainerFrame::PositionChildViews(aFloat);
-    FrameLayerBuilder::InvalidateThebesLayersInSubtree(aFloat);
   }
 
   // Update the float combined area state
@@ -805,8 +818,8 @@ nsBlockReflowState::FlowAndPlaceFloat(nsIFrame* aFloat)
       (NS_UNCONSTRAINEDSIZE != mContentArea.height)) {
     region.height = NS_MAX(region.height, mContentArea.height - floatY);
   }
-  nsresult rv =
-  mFloatManager->AddFloat(aFloat, region);
+  DebugOnly<nsresult> rv =
+    mFloatManager->AddFloat(aFloat, region);
   NS_ABORT_IF_FALSE(NS_SUCCEEDED(rv), "bad float placement");
   // store region
   nsFloatManager::StoreRegionFor(aFloat, region);
@@ -905,9 +918,9 @@ nsBlockReflowState::PlaceBelowCurrentLineFloats(nsFloatCacheFreeList& aList,
 }
 
 nscoord
-nsBlockReflowState::ClearFloats(nscoord aY, PRUint8 aBreakType,
+nsBlockReflowState::ClearFloats(nscoord aY, uint8_t aBreakType,
                                 nsIFrame *aReplacedBlock,
-                                PRUint32 aFlags)
+                                uint32_t aFlags)
 {
 #ifdef DEBUG
   if (nsBlockFrame::gNoisyReflow) {
