@@ -39,13 +39,13 @@ public:
   ~nsLineLayout();
 
   void Init(nsBlockReflowState* aState, nscoord aMinLineHeight,
-            PRInt32 aLineNumber) {
+            int32_t aLineNumber) {
     mBlockRS = aState;
     mMinLineHeight = aMinLineHeight;
     mLineNumber = aLineNumber;
   }
 
-  PRInt32 GetLineNumber() const {
+  int32_t GetLineNumber() const {
     return mLineNumber;
   }
 
@@ -53,7 +53,7 @@ public:
                        nscoord aWidth, nscoord aHeight,
                        bool aImpactedByFloats,
                        bool aIsTopOfPage,
-                       PRUint8 aDirection);
+                       uint8_t aDirection);
 
   void EndLineReflow();
 
@@ -77,9 +77,9 @@ public:
   // Returns the width of the span
   nscoord EndSpan(nsIFrame* aFrame);
 
-  PRInt32 GetCurrentSpanCount() const;
+  int32_t GetCurrentSpanCount() const;
 
-  void SplitLineTo(PRInt32 aNewCount);
+  void SplitLineTo(int32_t aNewCount);
 
   bool IsZeroHeight();
 
@@ -110,48 +110,9 @@ public:
    */
   void RelativePositionFrames(nsOverflowAreas& aOverflowAreas);
 
-  //----------------------------------------
-
-  // Supporting methods and data for flags
-protected:
-#define LL_FIRSTLETTERSTYLEOK          0x00000008
-#define LL_ISTOPOFPAGE                 0x00000010
-#define LL_IMPACTEDBYFLOATS            0x00000040
-#define LL_LASTFLOATWASLETTERFRAME     0x00000080
-#define LL_LINEISEMPTY                 0x00000100
-#define LL_LINEENDSINBR                0x00000200
-#define LL_NEEDBACKUP                  0x00000400
-#define LL_INFIRSTLINE                 0x00000800
-#define LL_GOTLINEBOX                  0x00001000
-#define LL_INFIRSTLETTER               0x00002000
-#define LL_HASBULLET                   0x00004000
-#define LL_DIRTYNEXTLINE               0x00008000
-#define LL_LINEATSTART                 0x00010000
-#define LL_LASTFLAG                    LL_LINEATSTART
-
-  void SetFlag(PRUint32 aFlag, bool aValue)
-  {
-    NS_ASSERTION(aFlag<=LL_LASTFLAG, "bad flag");
-    NS_ASSERTION(aValue==false || aValue==true, "bad value");
-    if (aValue) { // set flag
-      mFlags |= aFlag;
-    }
-    else {        // unset flag
-      mFlags &= ~aFlag;
-    }
-  }
-
-  bool GetFlag(PRUint32 aFlag) const
-  {
-    NS_ASSERTION(aFlag<=LL_LASTFLAG, "bad flag");
-    return !!(mFlags & aFlag);
-  }
-
-public:
-
   // Support methods for word-wrapping during line reflow
 
-  void SetTextJustificationWeights(PRInt32 aNumSpaces, PRInt32 aNumLetters) {
+  void SetTextJustificationWeights(int32_t aNumSpaces, int32_t aNumLetters) {
     mTextJustificationNumSpaces = aNumSpaces;
     mTextJustificationNumLetters = aNumLetters;
   }
@@ -162,7 +123,7 @@ public:
    */
   bool LineIsEmpty() const
   {
-    return GetFlag(LL_LINEISEMPTY);
+    return mLineIsEmpty;
   }
 
   /**
@@ -172,19 +133,19 @@ public:
    */
   bool LineAtStart() const
   {
-    return GetFlag(LL_LINEATSTART);
+    return mLineAtStart;
   }
 
   bool LineIsBreakable() const;
 
   bool GetLineEndsInBR() const 
   { 
-    return GetFlag(LL_LINEENDSINBR); 
+    return mLineEndsInBR;
   }
 
   void SetLineEndsInBR(bool aOn) 
   { 
-    SetFlag(LL_LINEENDSINBR, aOn); 
+    mLineEndsInBR = aOn;
   }
 
   //----------------------------------------
@@ -202,36 +163,36 @@ public:
   //----------------------------------------
 
   bool GetFirstLetterStyleOK() const {
-    return GetFlag(LL_FIRSTLETTERSTYLEOK);
+    return mFirstLetterStyleOK;
   }
 
   void SetFirstLetterStyleOK(bool aSetting) {
-    SetFlag(LL_FIRSTLETTERSTYLEOK, aSetting);
+    mFirstLetterStyleOK = aSetting;
   }
 
   bool GetInFirstLetter() const {
-    return GetFlag(LL_INFIRSTLETTER);
+    return mInFirstLetter;
   }
 
   void SetInFirstLetter(bool aSetting) {
-    SetFlag(LL_INFIRSTLETTER, aSetting);
+    mInFirstLetter = aSetting;
   }
 
   bool GetInFirstLine() const {
-    return GetFlag(LL_INFIRSTLINE);
+    return mInFirstLine;
   }
 
   void SetInFirstLine(bool aSetting) {
-    SetFlag(LL_INFIRSTLINE, aSetting);
+    mInFirstLine = aSetting;
   }
 
   // Calling this during block reflow ensures that the next line of inlines
   // will be marked dirty, if there is one.
   void SetDirtyNextLine() {
-    SetFlag(LL_DIRTYNEXTLINE, true);
+    mDirtyNextLine = true;
   }
   bool GetDirtyNextLine() {
-    return GetFlag(LL_DIRTYNEXTLINE);
+    return mDirtyNextLine;
   }
 
   //----------------------------------------
@@ -244,7 +205,7 @@ public:
    * can call SetLastOptionalBreakPosition to record where a break could
    * have been made, but wasn't because we decided to place more content on
    * the line. For non-text frames, offset 0 means
-   * before the content, offset PR_INT32_MAX means after the content.
+   * before the content, offset INT32_MAX means after the content.
    * 
    * Currently this is used to handle cases where a single word comprises
    * multiple frames, and the first frame fits on the line but the whole word
@@ -262,9 +223,9 @@ public:
    * @return true if we are actually reflowing with forced break position and we
    * should break here
    */
-  bool NotifyOptionalBreakPosition(nsIContent* aContent, PRInt32 aOffset,
+  bool NotifyOptionalBreakPosition(nsIContent* aContent, int32_t aOffset,
                                      bool aFits, gfxBreakPriority aPriority) {
-    NS_ASSERTION(!aFits || !GetFlag(LL_NEEDBACKUP),
+    NS_ASSERTION(!aFits || !mNeedBackup,
                   "Shouldn't be updating the break position with a break that fits after we've already flagged an overrun");
     // Remember the last break position that fits; if there was no break that fit,
     // just remember the first break
@@ -278,11 +239,11 @@ public:
       mForceBreakContentOffset == aOffset;
   }
   /**
-   * Like NotifyOptionalBreakPosition, but here it's OK for LL_NEEDBACKUP
+   * Like NotifyOptionalBreakPosition, but here it's OK for mNeedBackup
    * to be set, because the caller is merely pruning some saved break position(s)
    * that are actually not feasible.
    */
-  void RestoreSavedBreakPosition(nsIContent* aContent, PRInt32 aOffset,
+  void RestoreSavedBreakPosition(nsIContent* aContent, int32_t aOffset,
                                  gfxBreakPriority aPriority) {
     mLastOptionalBreakContent = aContent;
     mLastOptionalBreakContentOffset = aOffset;
@@ -292,14 +253,14 @@ public:
    * Signal that no backing up will be required after all.
    */
   void ClearOptionalBreakPosition() {
-    SetFlag(LL_NEEDBACKUP, false);
-    mLastOptionalBreakContent = nsnull;
+    mNeedBackup = false;
+    mLastOptionalBreakContent = nullptr;
     mLastOptionalBreakContentOffset = -1;
     mLastOptionalBreakPriority = eNoBreak;
   }
   // Retrieve last set optional break position. When this returns null, no
   // optional break has been recorded (which means that the line can't break yet).
-  nsIContent* GetLastOptionalBreakPosition(PRInt32* aOffset,
+  nsIContent* GetLastOptionalBreakPosition(int32_t* aOffset,
                                            gfxBreakPriority* aPriority) {
     *aOffset = mLastOptionalBreakContentOffset;
     *aPriority = mLastOptionalBreakPriority;
@@ -310,7 +271,7 @@ public:
    * Check whether frames overflowed the available width and CanPlaceFrame
    * requested backing up to a saved break position.
    */  
-  bool NeedsBackup() { return GetFlag(LL_NEEDBACKUP); }
+  bool NeedsBackup() { return mNeedBackup; }
   
   // Line layout may place too much content on a line, overflowing its available
   // width. When that happens, if SetLastOptionalBreakPosition has been
@@ -321,12 +282,12 @@ public:
   // Record that we want to break at the given content+offset (which
   // should have been previously returned by GetLastOptionalBreakPosition
   // from another nsLineLayout).
-  void ForceBreakAtPosition(nsIContent* aContent, PRInt32 aOffset) {
+  void ForceBreakAtPosition(nsIContent* aContent, int32_t aOffset) {
     mForceBreakContent = aContent;
     mForceBreakContentOffset = aOffset;
   }
-  bool HaveForcedBreakPosition() { return mForceBreakContent != nsnull; }
-  PRInt32 GetForcedBreakPosition(nsIContent* aContent) {
+  bool HaveForcedBreakPosition() { return mForceBreakContent != nullptr; }
+  int32_t GetForcedBreakPosition(nsIContent* aContent) {
     return mForceBreakContent == aContent ? mForceBreakContentOffset : -1;
   }
 
@@ -340,10 +301,10 @@ public:
     return mBlockReflowState;
   }
   const nsLineList::iterator* GetLine() const {
-    return GetFlag(LL_GOTLINEBOX) ? &mLineBox : nsnull;
+    return mGotLineBox ? &mLineBox : nullptr;
   }
   nsLineList::iterator* GetLine() {
-    return GetFlag(LL_GOTLINEBOX) ? &mLineBox : nsnull;
+    return mGotLineBox ? &mLineBox : nullptr;
   }
   
   /**
@@ -405,11 +366,11 @@ protected:
     nsMargin mOffsets;
 
     // state for text justification
-    PRInt32 mJustificationNumSpaces;
-    PRInt32 mJustificationNumLetters;
+    int32_t mJustificationNumSpaces;
+    int32_t mJustificationNumLetters;
     
     // Other state we use
-    PRUint8 mVerticalAlign;
+    uint8_t mVerticalAlign;
 
 // PerFrameData flags
 #define PFD_RELATIVEPOS                 0x00000001
@@ -422,12 +383,12 @@ protected:
 #define PFD_SKIPWHENTRIMMINGWHITESPACE  0x00000080
 #define PFD_LASTFLAG                    PFD_SKIPWHENTRIMMINGWHITESPACE
 
-    PRUint8 mFlags;
+    uint8_t mFlags;
 
-    void SetFlag(PRUint32 aFlag, bool aValue)
+    void SetFlag(uint32_t aFlag, bool aValue)
     {
       NS_ASSERTION(aFlag<=PFD_LASTFLAG, "bad flag");
-      NS_ASSERTION(aFlag<=PR_UINT8_MAX, "bad flag");
+      NS_ASSERTION(aFlag<=UINT8_MAX, "bad flag");
       NS_ASSERTION(aValue==false || aValue==true, "bad value");
       if (aValue) { // set flag
         mFlags |= aFlag;
@@ -437,7 +398,7 @@ protected:
       }
     }
 
-    bool GetFlag(PRUint32 aFlag) const
+    bool GetFlag(uint32_t aFlag) const
     {
       NS_ASSERTION(aFlag<=PFD_LASTFLAG, "bad flag");
       return !!(mFlags & aFlag);
@@ -465,7 +426,7 @@ protected:
 
     const nsHTMLReflowState* mReflowState;
     bool mNoWrap;
-    PRUint8 mDirection;
+    uint8_t mDirection;
     bool mChangedFrameDirection;
     bool mZeroEffectiveSpanBox;
     bool mContainsFloat;
@@ -481,7 +442,7 @@ protected:
     nscoord* mBaseline;
 
     void AppendFrame(PerFrameData* pfd) {
-      if (nsnull == mLastFrame) {
+      if (nullptr == mLastFrame) {
         mFirstFrame = pfd;
       }
       else {
@@ -496,8 +457,8 @@ protected:
   PerSpanData* mCurrentSpan;
 
   gfxBreakPriority mLastOptionalBreakPriority;
-  PRInt32     mLastOptionalBreakContentOffset;
-  PRInt32     mForceBreakContentOffset;
+  int32_t     mLastOptionalBreakContentOffset;
+  int32_t     mForceBreakContentOffset;
 
   nscoord mMinLineHeight;
   
@@ -507,11 +468,11 @@ protected:
 
   // This state varies during the reflow of a line but is line
   // "global" state not span "local" state.
-  PRInt32 mLineNumber;
-  PRInt32 mTextJustificationNumSpaces;
-  PRInt32 mTextJustificationNumLetters;
+  int32_t mLineNumber;
+  int32_t mTextJustificationNumSpaces;
+  int32_t mTextJustificationNumLetters;
 
-  PRInt32 mTotalPlacedFrames;
+  int32_t mTotalPlacedFrames;
 
   nscoord mTopEdge;
   nscoord mMaxTopBoxHeight;
@@ -526,15 +487,27 @@ protected:
   // Amount of trimmable whitespace width for the trailing text frame, if any
   nscoord mTrimmableWidth;
 
-  PRInt32 mSpanDepth;
+  bool mFirstLetterStyleOK      : 1;
+  bool mIsTopOfPage             : 1;
+  bool mImpactedByFloats        : 1;
+  bool mLastFloatWasLetterFrame : 1;
+  bool mLineIsEmpty             : 1;
+  bool mLineEndsInBR            : 1;
+  bool mNeedBackup              : 1;
+  bool mInFirstLine             : 1;
+  bool mGotLineBox              : 1;
+  bool mInFirstLetter           : 1;
+  bool mHasBullet               : 1;
+  bool mDirtyNextLine           : 1;
+  bool mLineAtStart             : 1;
+
+  int32_t mSpanDepth;
 #ifdef DEBUG
-  PRInt32 mSpansAllocated, mSpansFreed;
-  PRInt32 mFramesAllocated, mFramesFreed;
+  int32_t mSpansAllocated, mSpansFreed;
+  int32_t mFramesAllocated, mFramesFreed;
 #endif
   PLArenaPool mArena; // Per span and per frame data, 4 byte aligned
 
-  PRUint32 mFlags;
-
   nsresult NewPerFrameData(PerFrameData** aResult);
 
   nsresult NewPerSpanData(PerSpanData** aResult);
@@ -551,7 +524,7 @@ protected:
                         nsHTMLReflowState& aReflowState);
 
   bool CanPlaceFrame(PerFrameData* pfd,
-                       PRUint8 aFrameDirection,
+                       uint8_t aFrameDirection,
                        bool aNotSafeToBreak,
                        bool aFrameCanContinueTextRun,
                        bool aCanRollBackBeforeFrame,
@@ -572,15 +545,15 @@ protected:
 
   bool TrimTrailingWhiteSpaceIn(PerSpanData* psd, nscoord* aDeltaWidth);
 
-  void ComputeJustificationWeights(PerSpanData* psd, PRInt32* numSpaces, PRInt32* numLetters);
+  void ComputeJustificationWeights(PerSpanData* psd, int32_t* numSpaces, int32_t* numLetters);
 
   struct FrameJustificationState {
-    PRInt32 mTotalNumSpaces;
-    PRInt32 mTotalNumLetters;
+    int32_t mTotalNumSpaces;
+    int32_t mTotalNumLetters;
     nscoord mTotalWidthForSpaces;
     nscoord mTotalWidthForLetters;
-    PRInt32 mNumSpacesProcessed;
-    PRInt32 mNumLettersProcessed;
+    int32_t mNumSpacesProcessed;
+    int32_t mNumLettersProcessed;
     nscoord mWidthForSpacesProcessed;
     nscoord mWidthForLettersProcessed;
   };
@@ -592,7 +565,7 @@ protected:
 
 
 #ifdef DEBUG
-  void DumpPerSpanData(PerSpanData* psd, PRInt32 aIndent);
+  void DumpPerSpanData(PerSpanData* psd, int32_t aIndent);
 #endif
 };
 
