@@ -5,67 +5,66 @@
 
 package org.mozilla.gecko;
 
+import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.gfx.Layer;
+import org.mozilla.gecko.util.GeckoAsyncTask;
+
+import org.json.JSONException;
+import org.json.JSONObject;
+
 import android.content.ContentResolver;
 import android.database.ContentObserver;
 import android.graphics.Bitmap;
 import android.graphics.Color;
 import android.graphics.drawable.BitmapDrawable;
 import android.graphics.drawable.Drawable;
-import android.os.AsyncTask;
+import android.net.Uri;
+import android.os.Build;
+import android.text.TextUtils;
 import android.util.Log;
-import android.view.Surface;
 import android.view.View;
 
-import org.json.JSONException;
-import org.json.JSONObject;
-import org.mozilla.gecko.db.BrowserDB;
-import org.mozilla.gecko.gfx.Layer;
-
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.List;
-import java.util.regex.Pattern;
 import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
-public final class Tab {
+public class Tab {
     private static final String LOGTAG = "GeckoTab";
-    private static final int kThumbnailWidth = 136;
-    private static final int kThumbnailHeight = 78;
 
-    private static float sDensity = 0.0f;
     private static Pattern sColorPattern;
-    private int mId;
+    private final int mId;
+    private long mLastUsed;
     private String mUrl;
     private String mTitle;
-    private Drawable mFavicon;
+    private Bitmap mFavicon;
     private String mFaviconUrl;
     private int mFaviconSize;
     private JSONObject mIdentityData;
     private boolean mReaderEnabled;
-    private Drawable mThumbnail;
+    private BitmapDrawable mThumbnail;
     private int mHistoryIndex;
     private int mHistorySize;
     private int mParentId;
     private boolean mExternal;
     private boolean mBookmark;
-    private HashMap<String, DoorHanger> mDoorHangers;
+    private boolean mReadingListItem;
     private long mFaviconLoadId;
     private String mDocumentURI;
     private String mContentType;
     private boolean mHasTouchListeners;
-    private boolean mAllowZoom;
-    private float mDefaultZoom;
-    private float mMinZoom;
-    private float mMaxZoom;
+    private ZoomConstraints mZoomConstraints;
     private ArrayList<View> mPluginViews;
     private HashMap<Object, Layer> mPluginLayers;
     private ContentResolver mContentResolver;
     private ContentObserver mContentObserver;
-    private int mCheckerboardColor = Color.WHITE;
+    private int mBackgroundColor = Color.WHITE;
     private int mState;
+    private Bitmap mThumbnailBitmap;
     private boolean mDesktopMode;
+    private boolean mEnteringReaderMode;
 
     public static final int STATE_DELAYED = 0;
     public static final int STATE_LOADING = 1;
@@ -74,28 +73,31 @@ public final class Tab {
 
     public Tab(int id, String url, boolean external, int parentId, String title) {
         mId = id;
+        mLastUsed = 0;
         mUrl = url;
         mExternal = external;
         mParentId = parentId;
-        mTitle = title;
+        mTitle = title == null ? "" : title;
         mFavicon = null;
         mFaviconUrl = null;
         mFaviconSize = 0;
         mIdentityData = null;
         mReaderEnabled = false;
+        mEnteringReaderMode = false;
         mThumbnail = null;
         mHistoryIndex = -1;
         mHistorySize = 0;
         mBookmark = false;
-        mDoorHangers = new HashMap<String, DoorHanger>();
+        mReadingListItem = false;
         mFaviconLoadId = 0;
         mDocumentURI = "";
         mContentType = "";
+        mZoomConstraints = new ZoomConstraints(false);
         mPluginViews = new ArrayList<View>();
         mPluginLayers = new HashMap<Object, Layer>();
-        mState = "about:home".equals(url) ? STATE_SUCCESS : STATE_LOADING;
+        mState = GeckoApp.shouldShowProgress(url) ? STATE_SUCCESS : STATE_LOADING;
         mContentResolver = Tabs.getInstance().getContentResolver();
-        mContentObserver = new ContentObserver(GeckoAppShell.getHandler()) {
+        mContentObserver = new ContentObserver(null) {
             public void onChange(boolean selfChange) {
                 updateBookmark();
             }
@@ -104,24 +106,33 @@ public final class Tab {
     }
 
     public void onDestroy() {
-        mDoorHangers = new HashMap<String, DoorHanger>();
         BrowserDB.unregisterContentObserver(mContentResolver, mContentObserver);
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.CLOSED);
     }
 
     public int getId() {
         return mId;
     }
 
+    public synchronized void onChange() {
+        mLastUsed = System.currentTimeMillis();
+    }
+
+    public synchronized long getLastUsed() {
+        return mLastUsed;
+    }
+
     public int getParentId() {
         return mParentId;
     }
 
     // may be null if user-entered query hasn't yet been resolved to a URI
-    public String getURL() {
+    public synchronized String getURL() {
         return mUrl;
     }
 
-    public String getTitle() {
+    // mTitle should never be null, but it may be an empty string
+    public synchronized String getTitle() {
         return mTitle;
     }
 
@@ -133,7 +144,7 @@ public final class Tab {
         return mUrl;
     }
 
-    public Drawable getFavicon() {
+    public Bitmap getFavicon() {
         return mFavicon;
     }
 
@@ -141,54 +152,48 @@ public final class Tab {
         return mThumbnail;
     }
 
-    float getDensity() {
-        if (sDensity == 0.0f) {
-            sDensity = GeckoApp.mAppContext.getDisplayMetrics().density;
+    public Bitmap getThumbnailBitmap(int width, int height) {
+        if (mThumbnailBitmap != null) {
+            // Bug 787318 - Honeycomb has a bug with bitmap caching, we can't
+            // reuse the bitmap there.
+            boolean honeycomb = (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB
+                              && Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2);
+            boolean sizeChange = mThumbnailBitmap.getWidth() != width
+                              || mThumbnailBitmap.getHeight() != height;
+            if (honeycomb || sizeChange) {
+                mThumbnailBitmap = null;
+            }
         }
-        return sDensity;
-    }
 
-    int getThumbnailWidth() {
-        int desiredWidth = (int)(kThumbnailWidth * getDensity());
-        return desiredWidth & ~0x1;
-    }
+        if (mThumbnailBitmap == null) {
+            mThumbnailBitmap = Bitmap.createBitmap(width, height, Bitmap.Config.RGB_565);
+        }
 
-    int getThumbnailHeight() {
-        int desiredHeight = (int)(kThumbnailHeight * getDensity());
-        return desiredHeight & ~0x1;
+        return mThumbnailBitmap;
     }
 
     public void updateThumbnail(final Bitmap b) {
-        final Tab tab = this;
         GeckoAppShell.getHandler().post(new Runnable() {
             public void run() {
                 if (b != null) {
                     try {
-                        Bitmap bitmap = Bitmap.createScaledBitmap(b, getThumbnailWidth(), getThumbnailHeight(), false);
-
+                        mThumbnail = new BitmapDrawable(b);
                         if (mState == Tab.STATE_SUCCESS)
-                            saveThumbnailToDB(new BitmapDrawable(bitmap));
-
-                        mThumbnail = new BitmapDrawable(bitmap);
-                        if (bitmap != b)
-                            b.recycle();
+                            saveThumbnailToDB();
                     } catch (OutOfMemoryError oom) {
-                        Log.e(LOGTAG, "Unable to create/scale bitmap", oom);
+                        Log.w(LOGTAG, "Unable to create/scale bitmap.", oom);
                         mThumbnail = null;
                     }
                 } else {
                     mThumbnail = null;
                 }
-                GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
-                    public void run() {
-                        Tabs.getInstance().notifyListeners(tab, Tabs.TabEvents.THUMBNAIL);
-                    }
-                });
+
+                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.THUMBNAIL);
             }
         });
     }
 
-    public String getFaviconURL() {
+    public synchronized String getFaviconURL() {
         return mFaviconUrl;
     }
 
@@ -213,16 +218,18 @@ public final class Tab {
         return mBookmark;
     }
 
+    public boolean isReadingListItem() {
+        return mReadingListItem;
+    }
+
     public boolean isExternal() {
         return mExternal;
     }
 
-    public void updateURL(String url) {
+    public synchronized void updateURL(String url) {
         if (url != null && url.length() > 0) {
             mUrl = url;
-            Log.i(LOGTAG, "Updated url: " + url + " for tab with id: " + mId);
             updateBookmark();
-            updateHistory(mUrl, mTitle);
         }
     }
 
@@ -235,28 +242,35 @@ public final class Tab {
     }
 
     public void setContentType(String contentType) {
-        mContentType = contentType;
+        mContentType = (contentType == null) ? "" : contentType;
     }
 
     public String getContentType() {
         return mContentType;
     }
 
-    public void updateTitle(String title) {
+    public synchronized void updateTitle(String title) {
+        // Keep the title unchanged while entering reader mode
+        if (mEnteringReaderMode)
+            return;
+
         mTitle = (title == null ? "" : title);
 
-        Log.i(LOGTAG, "Updated title: " + mTitle + " for tab with id: " + mId);
-        updateHistory(mUrl, mTitle);
-        final Tab tab = this;
+        if (mUrl != null)
+            updateHistory(mUrl, mTitle);
 
-        GeckoAppShell.getMainHandler().post(new Runnable() {
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.TITLE);
+    }
+
+    protected void addHistory(final String uri) {
+        GeckoAppShell.getHandler().post(new Runnable() {
             public void run() {
-                Tabs.getInstance().notifyListeners(tab, Tabs.TabEvents.TITLE);
+                GlobalHistory.getInstance().add(uri);
             }
         });
     }
 
-    private void updateHistory(final String uri, final String title) {
+    protected void updateHistory(final String uri, final String title) {
         GeckoAppShell.getHandler().post(new Runnable() {
             public void run() {
                 GlobalHistory.getInstance().update(uri, title);
@@ -266,42 +280,21 @@ public final class Tab {
 
     public void setState(int state) {
         mState = state;
+
+        if (mState != Tab.STATE_LOADING)
+            mEnteringReaderMode = false;
     }
 
     public int getState() {
         return mState;
     }
 
-    public void setAllowZoom(boolean aValue) {
-        mAllowZoom = aValue;
+    public void setZoomConstraints(ZoomConstraints constraints) {
+        mZoomConstraints = constraints;
     }
 
-    public boolean getAllowZoom() {
-        return mAllowZoom;
-    }
-
-    public void setDefaultZoom(float aValue) {
-        mDefaultZoom = aValue;
-    }
-
-    public float getDefaultZoom() {
-        return mDefaultZoom;
-    }
-
-    public void setMinZoom(float aValue) {
-        mMinZoom = aValue;
-    }
-
-    public float getMinZoom() {
-        return mMinZoom;
-    }
-
-    public void setMaxZoom(float aValue) {
-        mMaxZoom = aValue;
-    }
-
-    public float getMaxZoom() {
-        return mMaxZoom;
+    public ZoomConstraints getZoomConstraints() {
+        return mZoomConstraints;
     }
 
     public void setHasTouchListeners(boolean aValue) {
@@ -320,12 +313,11 @@ public final class Tab {
         return mFaviconLoadId;
     }
 
-    public void updateFavicon(Drawable favicon) {
+    public void updateFavicon(Bitmap favicon) {
         mFavicon = favicon;
-        Log.i(LOGTAG, "Updated favicon for tab with id: " + mId);
     }
 
-    public void updateFaviconURL(String faviconUrl, int size) {
+    public synchronized void updateFaviconURL(String faviconUrl, int size) {
         // If we already have an "any" sized icon, don't update the icon.
         if (mFaviconSize == -1)
             return;
@@ -335,49 +327,43 @@ public final class Tab {
         if (size == -1 || size >= mFaviconSize) {
             mFaviconUrl = faviconUrl;
             mFaviconSize = size;
-            Log.i(LOGTAG, "Updated favicon URL for tab with id: " + mId);
         }
     }
 
-    public void clearFavicon() {
+    public synchronized void clearFavicon() {
+        // Keep the favicon unchanged while entering reader mode
+        if (mEnteringReaderMode)
+            return;
+
         mFavicon = null;
         mFaviconUrl = null;
         mFaviconSize = 0;
     }
 
-
     public void updateIdentityData(JSONObject identityData) {
         mIdentityData = identityData;
     }
 
     public void setReaderEnabled(boolean readerEnabled) {
         mReaderEnabled = readerEnabled;
-        GeckoAppShell.getMainHandler().post(new Runnable() {
-            public void run() {
-                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
-            }
-        });
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.MENU_UPDATED);
     }
 
     private void updateBookmark() {
-        final String url = getURL();
-        if (url == null)
-            return;
+        GeckoAppShell.getHandler().post(new Runnable() {
+            public void run() {
+                final String url = getURL();
+                if (url == null)
+                    return;
 
-        (new GeckoAsyncTask<Void, Void, Void>() {
-            @Override
-            public Void doInBackground(Void... params) {
                 if (url.equals(getURL())) {
                     mBookmark = BrowserDB.isBookmark(mContentResolver, url);
+                    mReadingListItem = BrowserDB.isReadingListItem(mContentResolver, url);
                 }
-                return null;
-            }
 
-            @Override
-            public void onPostExecute(Void result) {
                 Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
             }
-        }).execute();
+        });
     }
 
     public void addBookmark() {
@@ -387,7 +373,7 @@ public final class Tab {
                 if (url == null)
                     return;
 
-                BrowserDB.addBookmark(mContentResolver, getTitle(), url);
+                BrowserDB.addBookmark(mContentResolver, mTitle, url);
             }
         });
     }
@@ -408,147 +394,132 @@ public final class Tab {
         if (!mReaderEnabled)
             return;
 
-        GeckoAppShell.getHandler().post(new Runnable() {
-            public void run() {
-                String url = getURL();
-                if (url == null)
-                    return;
-
-                BrowserDB.addReadingListItem(mContentResolver, getTitle(), url);
-            }
-        });
+        GeckoEvent e = GeckoEvent.createBroadcastEvent("Reader:Add", String.valueOf(getId()));
+        GeckoAppShell.sendEventToGecko(e);
     }
 
     public void readerMode() {
         if (!mReaderEnabled)
             return;
 
-        // Do nothing for now
+        mEnteringReaderMode = true;
+        Tabs.getInstance().loadUrl(ReaderModeUtils.getAboutReaderForUrl(getURL(), mId, mReadingListItem));
     }
 
-    public boolean doReload() {
+    public boolean isEnteringReaderMode() {
+        return mEnteringReaderMode;
+    }
+
+    public void doReload() {
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Reload", "");
         GeckoAppShell.sendEventToGecko(e);
-        return true;
     }
 
+    // Our version of nsSHistory::GetCanGoBack
     public boolean canDoBack() {
-        return (mHistoryIndex < 1 ? false : true);
+        return mHistoryIndex > 0;
     }
 
     public boolean doBack() {
-        if (mHistoryIndex < 1) {
+        if (!canDoBack())
             return false;
-        }
+
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Back", "");
         GeckoAppShell.sendEventToGecko(e);
         return true;
     }
 
-    public boolean doStop() {
+    public void doStop() {
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Stop", "");
         GeckoAppShell.sendEventToGecko(e);
-        return true;
     }
 
+    // Our version of nsSHistory::GetCanGoForward
     public boolean canDoForward() {
-        return (mHistoryIndex + 1 < mHistorySize);
+        return mHistoryIndex < mHistorySize - 1;
     }
 
     public boolean doForward() {
-        if (mHistoryIndex + 1 >= mHistorySize) {
+        if (!canDoForward())
             return false;
-        }
+
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Forward", "");
         GeckoAppShell.sendEventToGecko(e);
         return true;
     }
 
-    public void addDoorHanger(String value, DoorHanger dh) {
-        mDoorHangers.put(value, dh);
-    }
-
-    public void removeDoorHanger(String value) {
-        mDoorHangers.remove(value);
-    }
-
-    public void removeTransientDoorHangers() {
-        // Make a temporary set to avoid a ConcurrentModificationException
-        final HashSet<String> valuesToRemove = new HashSet<String>(); 
-
-        for (String value : mDoorHangers.keySet()) {
-            DoorHanger dh = mDoorHangers.get(value);
-            if (dh.shouldRemove())
-                valuesToRemove.add(value);
-        }
-
-        for (String value : valuesToRemove) {
-            mDoorHangers.remove(value);
-        }
-    }
-
-    public DoorHanger getDoorHanger(String value) {
-        if (mDoorHangers == null)
-            return null;
-
-        if (mDoorHangers.containsKey(value))
-            return mDoorHangers.get(value);
-
-        return null;
-    }
-
-    public HashMap<String, DoorHanger> getDoorHangers() {
-        return mDoorHangers;
-    }
-
     void handleSessionHistoryMessage(String event, JSONObject message) throws JSONException {
         if (event.equals("New")) {
-            final String uri = message.getString("uri");
+            final String url = message.getString("url");
             mHistoryIndex++;
             mHistorySize = mHistoryIndex + 1;
-            GeckoAppShell.getHandler().post(new Runnable() {
-                public void run() {
-                    GlobalHistory.getInstance().add(uri);
-                }
-            });
         } else if (event.equals("Back")) {
-            if (mHistoryIndex - 1 < 0) {
-                Log.e(LOGTAG, "Received unexpected back notification");
+            if (!canDoBack()) {
+                Log.w(LOGTAG, "Received unexpected back notification");
                 return;
             }
             mHistoryIndex--;
         } else if (event.equals("Forward")) {
-            if (mHistoryIndex + 1 >= mHistorySize) {
-                Log.e(LOGTAG, "Received unexpected forward notification");
+            if (!canDoForward()) {
+                Log.w(LOGTAG, "Received unexpected forward notification");
                 return;
             }
             mHistoryIndex++;
         } else if (event.equals("Goto")) {
             int index = message.getInt("index");
             if (index < 0 || index >= mHistorySize) {
-                Log.e(LOGTAG, "Received unexpected history-goto notification");
+                Log.w(LOGTAG, "Received unexpected history-goto notification");
                 return;
             }
             mHistoryIndex = index;
         } else if (event.equals("Purge")) {
-            int numEntries = message.getInt("index");
+            int numEntries = message.getInt("numEntries");
+            if (numEntries > mHistorySize) {
+                Log.w(LOGTAG, "Received unexpectedly large number of history entries to purge");
+                mHistoryIndex = -1;
+                mHistorySize = 0;
+                return;
+            }
+
             mHistorySize -= numEntries;
             mHistoryIndex -= numEntries;
-            if (mHistorySize < 0 || mHistoryIndex < -1) {
-                Log.e(LOGTAG, "Unexpected history state: index = " + mHistoryIndex + ", size = " + mHistorySize);
-                mHistorySize = 0;
-                mHistoryIndex = -1;
-            }
+
+            // If we weren't at the last history entry, mHistoryIndex may have become too small
+            if (mHistoryIndex < -1)
+                 mHistoryIndex = -1;
         }
     }
 
-    private void saveThumbnailToDB(BitmapDrawable thumbnail) {
+    void handleLocationChange(JSONObject message) throws JSONException {
+        final String uri = message.getString("uri");
+        mEnteringReaderMode = ReaderModeUtils.isEnteringReaderMode(mUrl, uri);
+        updateURL(uri);
+
+        setDocumentURI(message.getString("documentURI"));
+        if (message.getBoolean("sameDocument")) {
+            // We can get a location change event for the same document with an anchor tag
+            return;
+        }
+
+        setContentType(message.getString("contentType"));
+        clearFavicon();
+        updateTitle(null);
+        updateIdentityData(null);
+        setReaderEnabled(false);
+        setZoomConstraints(new ZoomConstraints(true));
+        setHasTouchListeners(false);
+        setBackgroundColor(Color.WHITE);
+
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.LOCATION_CHANGE, uri);
+    }
+
+    protected void saveThumbnailToDB() {
         try {
             String url = getURL();
             if (url == null)
                 return;
 
-            BrowserDB.updateThumbnailForUrl(mContentResolver, url, thumbnail);
+            BrowserDB.updateThumbnailForUrl(mContentResolver, url, mThumbnail);
         } catch (Exception e) {
             // ignore
         }
@@ -567,33 +538,41 @@ public final class Tab {
     }
 
     public void addPluginLayer(Object surfaceOrView, Layer layer) {
-        mPluginLayers.put(surfaceOrView, layer);
+        synchronized(mPluginLayers) {
+            mPluginLayers.put(surfaceOrView, layer);
+        }
     }
 
     public Layer getPluginLayer(Object surfaceOrView) {
-        return mPluginLayers.get(surfaceOrView);
+        synchronized(mPluginLayers) {
+            return mPluginLayers.get(surfaceOrView);
+        }
     }
 
     public Collection<Layer> getPluginLayers() {
-        return mPluginLayers.values();
+        synchronized(mPluginLayers) {
+            return new ArrayList<Layer>(mPluginLayers.values());
+        }
     }
 
     public Layer removePluginLayer(Object surfaceOrView) {
-        return mPluginLayers.remove(surfaceOrView);
+        synchronized(mPluginLayers) {
+            return mPluginLayers.remove(surfaceOrView);
+        }
     }
 
-    public int getCheckerboardColor() {
-        return mCheckerboardColor;
+    public int getBackgroundColor() {
+        return mBackgroundColor;
     }
 
-    /** Sets a new color for the checkerboard. */
-    public void setCheckerboardColor(int color) {
-        mCheckerboardColor = color;
+    /** Sets a new color for the background. */
+    public void setBackgroundColor(int color) {
+        mBackgroundColor = color;
     }
 
-    /** Parses and sets a new color for the checkerboard. */
-    public void setCheckerboardColor(String newColor) {
-        setCheckerboardColor(parseColorFromGecko(newColor));
+    /** Parses and sets a new color for the background. */
+    public void setBackgroundColor(String newColor) {
+        setBackgroundColor(parseColorFromGecko(newColor));
     }
 
     // Parses a color from an RGB triple of the form "rgb([0-9]+, [0-9]+, [0-9]+)". If the color
@@ -621,4 +600,8 @@ public final class Tab {
     public boolean getDesktopMode() {
         return mDesktopMode;
     }
+
+    public boolean isPrivate() {
+        return false;
+    }
 }
