@@ -33,10 +33,10 @@ NS_INTERFACE_MAP_END
 
 SpdySession2::SpdySession2(nsAHttpTransaction *aHttpTransaction,
                          nsISocketTransport *aSocketTransport,
-                         PRInt32 firstPriority)
+                         int32_t firstPriority)
   : mSocketTransport(aSocketTransport),
-    mSegmentReader(nsnull),
-    mSegmentWriter(nsnull),
+    mSegmentReader(nullptr),
+    mSegmentWriter(nullptr),
     mSendingChunkSize(ASpdySession::kSendingChunkSize),
     mNextStreamID(1),
     mConcurrentHighWater(0),
@@ -44,8 +44,8 @@ SpdySession2::SpdySession2(nsAHttpTransaction *aHttpTransaction,
     mInputFrameBufferSize(kDefaultBufferSize),
     mInputFrameBufferUsed(0),
     mInputFrameDataLast(false),
-    mInputFrameDataStream(nsnull),
-    mNeedsCleanup(nsnull),
+    mInputFrameDataStream(nullptr),
+    mNeedsCleanup(nullptr),
     mDecompressBufferSize(kDefaultBufferSize),
     mDecompressBufferUsed(0),
     mShouldGoAway(false),
@@ -60,8 +60,7 @@ SpdySession2::SpdySession2(nsAHttpTransaction *aHttpTransaction,
     mOutputQueueSent(0),
     mLastReadEpoch(PR_IntervalNow()),
     mPingSentEpoch(0),
-    mNextPingID(1),
-    mPingThresholdExperiment(false)
+    mNextPingID(1)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
@@ -77,44 +76,11 @@ SpdySession2::SpdySession2(nsAHttpTransaction *aHttpTransaction,
   zlibInit();
   
   mSendingChunkSize = gHttpHandler->SpdySendingChunkSize();
-  if (!aHttpTransaction->QueryNullTransaction())
+  if (!aHttpTransaction->IsNullTransaction())
     AddStream(aHttpTransaction, firstPriority);
   mLastDataReadEpoch = mLastReadEpoch;
   
-  DeterminePingThreshold();
-}
-
-void
-SpdySession2::DeterminePingThreshold()
-{
   mPingThreshold = gHttpHandler->SpdyPingThreshold();
-
-  if (!mPingThreshold || !gHttpHandler->AllowExperiments())
-    return;
-
-  PRUint32 randomVal = gHttpHandler->Get32BitsOfPseudoRandom();
-  
-  // Use the lower 10 bits to select 1 in 1024 sessions for the
-  // ping threshold experiment. Somewhat less than that will actually be
-  // used because random values greater than the total http idle timeout
-  // for the session are discarded.
-  if ((randomVal & 0x3ff) != 1)  // lottery
-    return;
-  
-  randomVal = randomVal >> 10; // those bits are used up
-
-  // This session has been selected - use a random ping threshold of 10 +
-  // a random number from 0 to 255, based on the next 8 bits of the
-  // random buffer
-  PRIntervalTime randomThreshold =
-    PR_SecondsToInterval((randomVal & 0xff) + 10);
-  if (randomThreshold > gHttpHandler->IdleTimeout())
-    return;
-  
-  mPingThreshold = randomThreshold;
-  mPingThresholdExperiment = true;
-  LOG3(("SpdySession2 %p Ping Threshold Experimental Selection : %dsec\n",
-        this, PR_IntervalToSeconds(mPingThreshold)));
 }
 
 PLDHashOperator
@@ -153,7 +119,7 @@ SpdySession2::~SpdySession2()
 
 void
 SpdySession2::LogIO(SpdySession2 *self, SpdyStream2 *stream, const char *label,
-                   const char *data, PRUint32 datalen)
+                   const char *data, uint32_t datalen)
 {
   if (!LOG4_ENABLED())
     return;
@@ -163,7 +129,7 @@ SpdySession2::LogIO(SpdySession2 *self, SpdyStream2 *stream, const char *label,
 
   // Max line is (16 * 3) + 10(prefix) + newline + null
   char linebuf[128];
-  PRUint32 index;
+  uint32_t index;
   char *line = linebuf;
 
   linebuf[127] = 0;
@@ -191,7 +157,7 @@ SpdySession2::LogIO(SpdySession2 *self, SpdyStream2 *stream, const char *label,
 typedef nsresult  (*Control_FX) (SpdySession2 *self);
 static Control_FX sControlFunctions[] = 
 {
-  nsnull,
+  nullptr,
   SpdySession2::HandleSynStream,
   SpdySession2::HandleSynReply,
   SpdySession2::HandleRstStream,
@@ -241,7 +207,7 @@ SpdySession2::ReadTimeoutTick(PRIntervalTime now)
     if ((now - mLastReadEpoch) < mPingThreshold) {
       // recent activity means ping is not an issue
       if (mPingSentEpoch)
-        ClearPing(true);
+        mPingSentEpoch = 0;
       return;
     }
 
@@ -250,7 +216,7 @@ SpdySession2::ReadTimeoutTick(PRIntervalTime now)
       if ((now - mPingSentEpoch) >= gHttpHandler->SpdyPingTimeout()) {
         LOG(("SpdySession2::ReadTimeoutTick %p Ping Timer Exhaustion\n",
              this));
-        ClearPing(false);
+        mPingSentEpoch = 0;
         Close(NS_ERROR_NET_TIMEOUT);
       }
       return;
@@ -270,6 +236,7 @@ SpdySession2::ReadTimeoutTick(PRIntervalTime now)
       mPingSentEpoch = 1; // avoid the 0 sentinel value
     GeneratePing(mNextPingID);
     mNextPingID += 2;
+    ResumeRecv(); // read the ping reply
 
     if (mNextPingID == 0xffffffff) {
       LOG(("SpdySession2::ReadTimeoutTick %p "
@@ -278,28 +245,7 @@ SpdySession2::ReadTimeoutTick(PRIntervalTime now)
     }
 }
 
-void
-SpdySession2::ClearPing(bool pingOK)
-{
-  mPingSentEpoch = 0;
-
-  if (mPingThresholdExperiment) {
-    LOG3(("SpdySession2::ClearPing %p mPingThresholdExperiment %dsec %s\n",
-          this, PR_IntervalToSeconds(mPingThreshold),
-          pingOK ? "pass" :"fail"));
-
-    if (pingOK)
-      Telemetry::Accumulate(Telemetry::SPDY_PING_EXPERIMENT_PASS,
-                            PR_IntervalToSeconds(mPingThreshold));
-    else
-      Telemetry::Accumulate(Telemetry::SPDY_PING_EXPERIMENT_FAIL,
-                            PR_IntervalToSeconds(mPingThreshold));
-    mPingThreshold = gHttpHandler->SpdyPingThreshold();
-    mPingThresholdExperiment = false;
-  }
-}
-
-PRUint32
+uint32_t
 SpdySession2::RegisterStreamID(SpdyStream2 *stream)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
@@ -310,7 +256,7 @@ SpdySession2::RegisterStreamID(SpdyStream2 *stream)
   NS_ABORT_IF_FALSE(mNextStreamID < 0xfffffff0,
                     "should have stopped admitting streams");
   
-  PRUint32 result = mNextStreamID;
+  uint32_t result = mNextStreamID;
   mNextStreamID += 2;
 
   // We've used up plenty of ID's on this session. Start
@@ -333,7 +279,7 @@ SpdySession2::RegisterStreamID(SpdyStream2 *stream)
 
 bool
 SpdySession2::AddStream(nsAHttpTransaction *aHttpTransaction,
-                       PRInt32 aPriority)
+                       int32_t aPriority)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   NS_ABORT_IF_FALSE(!mStreamTransactionHash.Get(aHttpTransaction),
@@ -393,8 +339,8 @@ SpdySession2::ActivateStream(SpdyStream2 *stream)
   // This won't work for stream id=1 because there is no segment reader
   // yet.
   if (mSegmentReader) {
-    PRUint32 countRead;
-    ReadSegments(nsnull, kDefaultBufferSize, &countRead);
+    uint32_t countRead;
+    ReadSegments(nullptr, kDefaultBufferSize, &countRead);
   }
 }
 
@@ -415,10 +361,15 @@ SpdySession2::ProcessPending()
 
 nsresult
 SpdySession2::NetworkRead(nsAHttpSegmentWriter *writer, char *buf,
-                         PRUint32 count, PRUint32 *countWritten)
+                         uint32_t count, uint32_t *countWritten)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
+  if (!count) {
+    *countWritten = 0;
+    return NS_OK;
+  }
+
   nsresult rv = writer->OnWriteSegment(buf, count, countWritten);
   if (NS_SUCCEEDED(rv) && *countWritten > 0)
     mLastReadEpoch = PR_IntervalNow();
@@ -432,6 +383,16 @@ SpdySession2::SetWriteCallbacks()
       mConnection->ResumeSend();
 }
 
+void
+SpdySession2::RealignOutputQueue()
+{
+  mOutputQueueUsed -= mOutputQueueSent;
+  memmove(mOutputQueueBuffer.get(),
+          mOutputQueueBuffer.get() + mOutputQueueSent,
+          mOutputQueueUsed);
+  mOutputQueueSent = 0;
+}
+
 void
 SpdySession2::FlushOutputQueue()
 {
@@ -439,8 +400,8 @@ SpdySession2::FlushOutputQueue()
     return;
   
   nsresult rv;
-  PRUint32 countRead;
-  PRUint32 avail = mOutputQueueUsed - mOutputQueueSent;
+  uint32_t countRead;
+  uint32_t avail = mOutputQueueUsed - mOutputQueueSent;
 
   rv = mSegmentReader->
     OnReadSegment(mOutputQueueBuffer.get() + mOutputQueueSent, avail,
@@ -465,11 +426,7 @@ SpdySession2::FlushOutputQueue()
   
   if ((mOutputQueueSent >= kQueueMinimumCleanup) &&
       ((mOutputQueueSize - mOutputQueueUsed) < kQueueTailRoom)) {
-    mOutputQueueUsed -= mOutputQueueSent;
-    memmove(mOutputQueueBuffer.get(),
-            mOutputQueueBuffer.get() + mOutputQueueSent,
-            mOutputQueueUsed);
-    mOutputQueueSent = 0;
+    RealignOutputQueue();
   }
 }
 
@@ -481,7 +438,7 @@ SpdySession2::DontReuse()
     Close(NS_OK);
 }
 
-PRUint32
+uint32_t
 SpdySession2::GetWriteQueueSize()
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
@@ -516,14 +473,14 @@ SpdySession2::ResetDownstreamState()
     }
   }
   mInputFrameBufferUsed = 0;
-  mInputFrameDataStream = nsnull;
+  mInputFrameDataStream = nullptr;
 }
 
 void
 SpdySession2::EnsureBuffer(nsAutoArrayPtr<char> &buf,
-                          PRUint32 newSize,
-                          PRUint32 preserve,
-                          PRUint32 &objSize)
+                          uint32_t newSize,
+                          uint32_t preserve,
+                          uint32_t &objSize)
 {
   if (objSize >= newSize)
       return;
@@ -563,12 +520,13 @@ SpdySession2::zlibInit()
 }
 
 nsresult
-SpdySession2::DownstreamUncompress(char *blockStart, PRUint32 blockLen)
+SpdySession2::DownstreamUncompress(char *blockStart, uint32_t blockLen)
 {
   mDecompressBufferUsed = 0;
 
   mDownstreamZlib.avail_in = blockLen;
   mDownstreamZlib.next_in = reinterpret_cast<unsigned char *>(blockStart);
+  bool triedDictionary = false;
 
   do {
     mDownstreamZlib.next_out =
@@ -577,11 +535,18 @@ SpdySession2::DownstreamUncompress(char *blockStart, PRUint32 blockLen)
     mDownstreamZlib.avail_out = mDecompressBufferSize - mDecompressBufferUsed;
     int zlib_rv = inflate(&mDownstreamZlib, Z_NO_FLUSH);
 
-    if (zlib_rv == Z_NEED_DICT)
+    if (zlib_rv == Z_NEED_DICT) {
+      if (triedDictionary) {
+        LOG3(("SpdySession2::DownstreamUncompress %p Dictionary Error\n", this));
+        return NS_ERROR_FAILURE;
+      }
+      
+      triedDictionary = true;
       inflateSetDictionary(&mDownstreamZlib,
                            reinterpret_cast<const unsigned char *>
                            (SpdyStream2::kDictionary),
                            strlen(SpdyStream2::kDictionary) + 1);
+    }
     
     if (zlib_rv == Z_DATA_ERROR || zlib_rv == Z_MEM_ERROR)
       return NS_ERROR_FAILURE;
@@ -615,12 +580,12 @@ SpdySession2::FindHeader(nsCString name,
     (mDecompressBuffer.get()) + mDecompressBufferUsed;
   if (lastHeaderByte < nvpair)
     return NS_ERROR_ILLEGAL_VALUE;
-  PRUint16 numPairs =
-    PR_ntohs(reinterpret_cast<PRUint16 *>(mDecompressBuffer.get())[0]);
-  for (PRUint16 index = 0; index < numPairs; ++index) {
+  uint16_t numPairs =
+    PR_ntohs(reinterpret_cast<uint16_t *>(mDecompressBuffer.get())[0]);
+  for (uint16_t index = 0; index < numPairs; ++index) {
     if (lastHeaderByte < nvpair + 2)
       return NS_ERROR_ILLEGAL_VALUE;
-    PRUint32 nameLen = (nvpair[0] << 8) + nvpair[1];
+    uint32_t nameLen = (nvpair[0] << 8) + nvpair[1];
     if (lastHeaderByte < nvpair + 2 + nameLen)
       return NS_ERROR_ILLEGAL_VALUE;
     nsDependentCSubstring nameString =
@@ -628,7 +593,7 @@ SpdySession2::FindHeader(nsCString name,
                 reinterpret_cast<const char *>(nvpair) + 2 + nameLen);
     if (lastHeaderByte < nvpair + 4 + nameLen)
       return NS_ERROR_ILLEGAL_VALUE;
-    PRUint16 valueLen = (nvpair[2 + nameLen] << 8) + nvpair[3 + nameLen];
+    uint16_t valueLen = (nvpair[2 + nameLen] << 8) + nvpair[3 + nameLen];
     if (lastHeaderByte < nvpair + 4 + nameLen + valueLen)
       return NS_ERROR_ILLEGAL_VALUE;
     if (nameString.Equals(name)) {
@@ -667,14 +632,14 @@ SpdySession2::ConvertHeaders(nsDependentCSubstring &status,
   if (lastHeaderByte < nvpair)
     return NS_ERROR_ILLEGAL_VALUE;
 
-  PRUint16 numPairs =
-    PR_ntohs(reinterpret_cast<PRUint16 *>(mDecompressBuffer.get())[0]);
+  uint16_t numPairs =
+    PR_ntohs(reinterpret_cast<uint16_t *>(mDecompressBuffer.get())[0]);
 
-  for (PRUint16 index = 0; index < numPairs; ++index) {
+  for (uint16_t index = 0; index < numPairs; ++index) {
     if (lastHeaderByte < nvpair + 2)
       return NS_ERROR_ILLEGAL_VALUE;
 
-    PRUint32 nameLen = (nvpair[0] << 8) + nvpair[1];
+    uint32_t nameLen = (nvpair[0] << 8) + nvpair[1];
     if (lastHeaderByte < nvpair + 2 + nameLen)
       return NS_ERROR_ILLEGAL_VALUE;
 
@@ -719,7 +684,7 @@ SpdySession2::ConvertHeaders(nsDependentCSubstring &status,
       return NS_ERROR_ILLEGAL_VALUE;
     }
 
-    PRUint16 valueLen = (nvpair[2 + nameLen] << 8) + nvpair[3 + nameLen];
+    uint16_t valueLen = (nvpair[2 + nameLen] << 8) + nvpair[3 + nameLen];
     if (lastHeaderByte < nvpair + 4 + nameLen + valueLen)
       return NS_ERROR_ILLEGAL_VALUE;
 
@@ -764,7 +729,7 @@ SpdySession2::ConvertHeaders(nsDependentCSubstring &status,
 }
 
 void
-SpdySession2::GeneratePing(PRUint32 aID)
+SpdySession2::GeneratePing(uint32_t aID)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   LOG3(("SpdySession2::GeneratePing %p 0x%X\n", this, aID));
@@ -790,7 +755,7 @@ SpdySession2::GeneratePing(PRUint32 aID)
 }
 
 void
-SpdySession2::GenerateRstStream(PRUint32 aStatusCode, PRUint32 aID)
+SpdySession2::GenerateRstStream(uint32_t aStatusCode, uint32_t aID)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   LOG3(("SpdySession2::GenerateRst %p 0x%X %d\n", this, aID, aStatusCode));
@@ -843,15 +808,20 @@ SpdySession2::GenerateGoAway()
 // perform a bunch of integrity checks on the stream.
 // returns true if passed, false (plus LOG and ABORT) if failed.
 bool
-SpdySession2::VerifyStream(SpdyStream2 *aStream, PRUint32 aOptionalID = 0)
+SpdySession2::VerifyStream(SpdyStream2 *aStream, uint32_t aOptionalID = 0)
 {
   // This is annoying, but at least it is O(1)
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
+#ifndef DEBUG
+  // Only do the real verification in debug builds
+  return true;
+#endif
+
   if (!aStream)
     return true;
 
-  PRUint32 test = 0;
+  uint32_t test = 0;
   
   do {
     if (aStream->StreamID() == kDeadStreamID)
@@ -940,13 +910,13 @@ SpdySession2::CloseStream(SpdyStream2 *aStream, nsresult aResult)
   if (aStream == mInputFrameDataStream) {
     LOG3(("Stream had active partial read frame on close"));
     ChangeDownstreamState(DISCARDING_DATA_FRAME);
-    mInputFrameDataStream = nsnull;
+    mInputFrameDataStream = nullptr;
   }
 
   // check the streams blocked on write, this is linear but the list
   // should be pretty short.
-  PRUint32 size = mReadyForWrite.GetSize();
-  for (PRUint32 count = 0; count < size; ++count) {
+  uint32_t size = mReadyForWrite.GetSize();
+  for (uint32_t count = 0; count < size; ++count) {
     SpdyStream2 *stream = static_cast<SpdyStream2 *>(mReadyForWrite.PopFront());
     if (stream != aStream)
       mReadyForWrite.Push(stream);
@@ -955,7 +925,7 @@ SpdySession2::CloseStream(SpdyStream2 *aStream, nsresult aResult)
   // Check the streams blocked on urgent (i.e. window update) writing.
   // This should also be short.
   size = mUrgentForWrite.GetSize();
-  for (PRUint32 count = 0; count < size; ++count) {
+  for (uint32_t count = 0; count < size; ++count) {
     SpdyStream2 *stream = static_cast<SpdyStream2 *>(mUrgentForWrite.PopFront());
     if (stream != aStream)
       mUrgentForWrite.Push(stream);
@@ -964,7 +934,7 @@ SpdySession2::CloseStream(SpdyStream2 *aStream, nsresult aResult)
   // Check the streams queued for activation. Because we normally accept a high
   // level of parallelization this should also be short.
   size = mQueuedStreams.GetSize();
-  for (PRUint32 count = 0; count < size; ++count) {
+  for (uint32_t count = 0; count < size; ++count) {
     SpdyStream2 *stream = static_cast<SpdyStream2 *>(mQueuedStreams.PopFront());
     if (stream != aStream)
       mQueuedStreams.Push(stream);
@@ -986,10 +956,10 @@ SpdySession2::HandleSynStream(SpdySession2 *self)
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
-  PRUint32 streamID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
-  PRUint32 associatedID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[3]);
+  uint32_t streamID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t associatedID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[3]);
 
   LOG3(("SpdySession2::HandleSynStream %p recv SYN_STREAM (push) "
         "for ID 0x%X associated with 0x%X.",
@@ -1024,7 +994,7 @@ SpdySession2::HandleSynStream(SpdySession2 *self)
 }
 
 nsresult
-SpdySession2::SetInputFrameDataStream(PRUint32 streamID)
+SpdySession2::SetInputFrameDataStream(uint32_t streamID)
 {
   mInputFrameDataStream = mStreamIDHash.Get(streamID);
   if (VerifyStream(mInputFrameDataStream, streamID))
@@ -1032,7 +1002,7 @@ SpdySession2::SetInputFrameDataStream(PRUint32 streamID)
 
   LOG(("SpdySession2::SetInputFrameDataStream failed to verify 0x%X\n",
        streamID));
-  mInputFrameDataStream = nsnull;
+  mInputFrameDataStream = nullptr;
   return NS_ERROR_UNEXPECTED;
 }
 
@@ -1062,8 +1032,8 @@ SpdySession2::HandleSynReply(SpdySession2 *self)
 
   LOG3(("SpdySession2::HandleSynReply %p lookup via streamID in syn_reply.\n",
         self));
-  PRUint32 streamID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t streamID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
   nsresult rv = self->SetInputFrameDataStream(streamID);
   if (NS_FAILED(rv))
     return rv;
@@ -1123,7 +1093,7 @@ SpdySession2::HandleSynReplyForValidStream()
   Telemetry::Accumulate(Telemetry::SPDY_SYN_REPLY_SIZE,
                         mInputFrameDataSize - 6);
   if (mDecompressBufferUsed) {
-    PRUint32 ratio =
+    uint32_t ratio =
       (mInputFrameDataSize - 6) * 100 / mDecompressBufferUsed;
     Telemetry::Accumulate(Telemetry::SPDY_SYN_REPLY_RATIO, ratio);
   }
@@ -1165,13 +1135,13 @@ SpdySession2::HandleRstStream(SpdySession2 *self)
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
-  PRUint8 flags = reinterpret_cast<PRUint8 *>(self->mInputFrameBuffer.get())[4];
+  uint8_t flags = reinterpret_cast<uint8_t *>(self->mInputFrameBuffer.get())[4];
 
-  PRUint32 streamID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t streamID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
 
   self->mDownstreamRstReason =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[3]);
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[3]);
 
   LOG3(("SpdySession2::HandleRstStream %p RST_STREAM Reason Code %u ID %x "
         "flags %x", self, self->mDownstreamRstReason, streamID, flags));
@@ -1220,8 +1190,8 @@ SpdySession2::HandleSettings(SpdySession2 *self)
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
-  PRUint32 numEntries =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t numEntries =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
 
   // Ensure frame is large enough for supplied number of entries
   // Each entry is 8 bytes, frame data is reduced by 4 to account for
@@ -1235,7 +1205,7 @@ SpdySession2::HandleSettings(SpdySession2 *self)
   LOG3(("SpdySession2::HandleSettings %p SETTINGS Control Frame with %d entries",
         self, numEntries));
 
-  for (PRUint32 index = 0; index < numEntries; ++index) {
+  for (uint32_t index = 0; index < numEntries; ++index) {
     // To clarify the v2 spec:
     // Each entry is a 24 bits of a little endian id
     // followed by 8 bits of flags
@@ -1244,9 +1214,9 @@ SpdySession2::HandleSettings(SpdySession2 *self)
     unsigned char *setting = reinterpret_cast<unsigned char *>
       (self->mInputFrameBuffer.get()) + 12 + index * 8;
 
-    PRUint32 id = (setting[2] << 16) + (setting[1] << 8) + setting[0];
-    PRUint32 flags = setting[3];
-    PRUint32 value =  PR_ntohl(reinterpret_cast<PRUint32 *>(setting)[1]);
+    uint32_t id = (setting[2] << 16) + (setting[1] << 8) + setting[0];
+    uint32_t flags = setting[3];
+    uint32_t value =  PR_ntohl(reinterpret_cast<uint32_t *>(setting)[1]);
 
     LOG3(("Settings ID %d, Flags %X, Value %d", id, flags, value));
 
@@ -1321,14 +1291,14 @@ SpdySession2::HandlePing(SpdySession2 *self)
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
-  PRUint32 pingID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t pingID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
 
   LOG3(("SpdySession2::HandlePing %p PING ID 0x%X.", self, pingID));
 
   if (pingID & 0x01) {
     // presumably a reply to our timeout ping
-    self->ClearPing(true);
+    self->mPingSentEpoch = 0;
   }
   else {
     // Servers initiate even numbered pings, go ahead and echo it back
@@ -1353,7 +1323,7 @@ SpdySession2::HandleGoAway(SpdySession2 *self)
 
   self->mShouldGoAway = true;
   self->mGoAwayID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
   self->mCleanShutdown = true;
   
   LOG3(("SpdySession2::HandleGoAway %p GOAWAY Last-Good-ID 0x%X.",
@@ -1375,8 +1345,8 @@ SpdySession2::HandleHeaders(SpdySession2 *self)
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
-  PRUint32 streamID =
-    PR_ntohl(reinterpret_cast<PRUint32 *>(self->mInputFrameBuffer.get())[2]);
+  uint32_t streamID =
+    PR_ntohl(reinterpret_cast<uint32_t *>(self->mInputFrameBuffer.get())[2]);
 
   // this is actually not legal in the HTTP mapping of SPDY. All
   // headers are in the syn or syn reply. Log and ignore it.
@@ -1413,7 +1383,7 @@ SpdySession2::HandleWindowUpdate(SpdySession2 *self)
 void
 SpdySession2::OnTransportStatus(nsITransport* aTransport,
                                nsresult aStatus,
-                               PRUint64 aProgress)
+                               uint64_t aProgress)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
 
@@ -1467,8 +1437,8 @@ SpdySession2::OnTransportStatus(nsITransport* aTransport,
 
 nsresult
 SpdySession2::ReadSegments(nsAHttpSegmentReader *reader,
-                          PRUint32 count,
-                          PRUint32 *countRead)
+                          uint32_t count,
+                          uint32_t *countRead)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   
@@ -1574,8 +1544,8 @@ SpdySession2::ReadSegments(nsAHttpSegmentReader *reader,
 
 nsresult
 SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
-                           PRUint32 count,
-                           PRUint32 *countWritten)
+                           uint32_t count,
+                           uint32_t *countWritten)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   
@@ -1611,7 +1581,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
       return rv;
     }
 
-    LogIO(this, nsnull, "Reading Frame Header",
+    LogIO(this, nullptr, "Reading Frame Header",
           mInputFrameBuffer + mInputFrameBufferUsed, *countWritten);
 
     mInputFrameBufferUsed += *countWritten;
@@ -1627,7 +1597,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
     // For both control and data frames the second 32 bit word of the header
     // is 8-flags, 24-length. (network byte order)
     mInputFrameDataSize =
-      PR_ntohl(reinterpret_cast<PRUint32 *>(mInputFrameBuffer.get())[1]);
+      PR_ntohl(reinterpret_cast<uint32_t *>(mInputFrameBuffer.get())[1]);
     mInputFrameDataSize &= 0x00ffffff;
     mInputFrameDataRead = 0;
     
@@ -1638,12 +1608,12 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
       
       // The first 32 bit word of the header is
       // 1 ctrl - 15 version - 16 type
-      PRUint16 version =
-        PR_ntohs(reinterpret_cast<PRUint16 *>(mInputFrameBuffer.get())[0]);
+      uint16_t version =
+        PR_ntohs(reinterpret_cast<uint16_t *>(mInputFrameBuffer.get())[0]);
       version &= 0x7fff;
       
       mFrameControlType =
-        PR_ntohs(reinterpret_cast<PRUint16 *>(mInputFrameBuffer.get())[1]);
+        PR_ntohs(reinterpret_cast<uint16_t *>(mInputFrameBuffer.get())[1]);
       
       LOG3(("SpdySession2::WriteSegments %p - Control Frame Identified "
             "type %d version %d data len %d",
@@ -1663,8 +1633,8 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
     else {
       ChangeDownstreamState(PROCESSING_DATA_FRAME);
 
-      PRUint32 streamID =
-        PR_ntohl(reinterpret_cast<PRUint32 *>(mInputFrameBuffer.get())[0]);
+      uint32_t streamID =
+        PR_ntohl(reinterpret_cast<uint32_t *>(mInputFrameBuffer.get())[0]);
       rv = SetInputFrameDataStream(streamID);
       if (NS_FAILED(rv)) {
         LOG(("SpdySession2::WriteSegments %p lookup streamID 0x%X failed. "
@@ -1725,11 +1695,11 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
     // The cleanup stream should only be set while stream->WriteSegments is
     // on the stack and then cleaned up in this code block afterwards.
     NS_ABORT_IF_FALSE(!mNeedsCleanup, "cleanup stream set unexpectedly");
-    mNeedsCleanup = nsnull;                     /* just in case */
+    mNeedsCleanup = nullptr;                     /* just in case */
 
     mSegmentWriter = writer;
     rv = mInputFrameDataStream->WriteSegments(this, count, countWritten);
-    mSegmentWriter = nsnull;
+    mSegmentWriter = nullptr;
 
     mLastDataReadEpoch = mLastReadEpoch;
 
@@ -1746,7 +1716,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
             mNeedsCleanup));
       CleanupStream(stream, NS_OK, RST_CANCEL);
       NS_ABORT_IF_FALSE(!mNeedsCleanup, "double cleanup out of data frame");
-      mNeedsCleanup = nsnull;                     /* just in case */
+      mNeedsCleanup = nullptr;                     /* just in case */
       return NS_OK;
     }
     
@@ -1755,7 +1725,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
             "cleanup stream based on mNeedsCleanup.\n",
             this, mNeedsCleanup, mNeedsCleanup ? mNeedsCleanup->StreamID() : 0));
       CleanupStream(mNeedsCleanup, NS_OK, RST_CANCEL);
-      mNeedsCleanup = nsnull;
+      mNeedsCleanup = nullptr;
     }
 
     // In v3 this is where we would generate a window update
@@ -1765,7 +1735,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
 
   if (mDownstreamState == DISCARDING_DATA_FRAME) {
     char trash[4096];
-    PRUint32 count = NS_MIN(4096U, mInputFrameDataSize - mInputFrameDataRead);
+    uint32_t count = NS_MIN(4096U, mInputFrameDataSize - mInputFrameDataRead);
 
     if (!count) {
       ResetDownstreamState();
@@ -1783,7 +1753,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
       return rv;
     }
 
-    LogIO(this, nsnull, "Discarding Frame", trash, *countWritten);
+    LogIO(this, nullptr, "Discarding Frame", trash, *countWritten);
 
     mInputFrameDataRead += *countWritten;
 
@@ -1813,7 +1783,7 @@ SpdySession2::WriteSegments(nsAHttpSegmentWriter *writer,
     return rv;
   }
 
-  LogIO(this, nsnull, "Reading Control Frame",
+  LogIO(this, nullptr, "Reading Control Frame",
         mInputFrameBuffer + 8 + mInputFrameDataRead, *countWritten);
 
   mInputFrameDataRead += *countWritten;
@@ -1862,9 +1832,9 @@ SpdySession2::Close(nsresult aReason)
 
   if (NS_SUCCEEDED(aReason))
     GenerateGoAway();
-  mConnection = nsnull;
-  mSegmentReader = nsnull;
-  mSegmentWriter = nsnull;
+  mConnection = nullptr;
+  mSegmentReader = nullptr;
+  mSegmentWriter = nullptr;
 }
 
 void
@@ -1897,8 +1867,8 @@ SpdySession2::CloseTransaction(nsAHttpTransaction *aTransaction,
 
 nsresult
 SpdySession2::OnReadSegment(const char *buf,
-                           PRUint32 count,
-                           PRUint32 *countRead)
+                           uint32_t count,
+                           uint32_t *countRead)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   
@@ -1919,7 +1889,7 @@ SpdySession2::OnReadSegment(const char *buf,
       return rv;
     
     if (*countRead < count) {
-      PRUint32 required = count - *countRead;
+      uint32_t required = count - *countRead;
       // assuming a commitment() happened, this ensurebuffer is a nop
       // but just in case the queuesize is too small for the required data
       // call ensurebuffer().
@@ -1954,7 +1924,7 @@ SpdySession2::OnReadSegment(const char *buf,
 }
 
 nsresult
-SpdySession2::CommitToSegmentSize(PRUint32 count)
+SpdySession2::CommitToSegmentSize(uint32_t count, bool forceCommitment)
 {
   if (mOutputQueueUsed)
     FlushOutputQueue();
@@ -1962,19 +1932,25 @@ SpdySession2::CommitToSegmentSize(PRUint32 count)
   // would there be enough room to buffer this if needed?
   if ((mOutputQueueUsed + count) <= (mOutputQueueSize - kQueueReserved))
     return NS_OK;
-  
-  // if we are using part of our buffers already, try again later
-  if (mOutputQueueUsed)
+
+  // if we are using part of our buffers already, try again later unless
+  // forceCommitment is set.
+  if (mOutputQueueUsed && !forceCommitment)
     return NS_BASE_STREAM_WOULD_BLOCK;
 
-  // not enough room to buffer even with completely empty buffers.
-  // normal frames are max 4kb, so the only case this can really happen
-  // is a SYN_STREAM with technically unbounded headers. That is highly
-  // unlikely, but possible. Create enough room for it because the buffers
-  // will be necessary - SSL does not absorb writes of very large sizes
-  // in single sends.
+  if (mOutputQueueUsed) {
+    // normally we avoid the memmove of RealignOutputQueue, but we'll try
+    // it if forceCommitment is set before growing the buffer.
+    RealignOutputQueue();
 
-  EnsureBuffer(mOutputQueueBuffer, count + kQueueReserved, 0, mOutputQueueSize);
+    // is there enough room now?
+    if ((mOutputQueueUsed + count) <= (mOutputQueueSize - kQueueReserved))
+      return NS_OK;
+  }
+
+  // resize the buffers as needed
+  EnsureBuffer(mOutputQueueBuffer, mOutputQueueUsed + count + kQueueReserved,
+               mOutputQueueUsed, mOutputQueueSize);
 
   NS_ABORT_IF_FALSE((mOutputQueueUsed + count) <=
                     (mOutputQueueSize - kQueueReserved),
@@ -1989,8 +1965,8 @@ SpdySession2::CommitToSegmentSize(PRUint32 count)
 
 nsresult
 SpdySession2::OnWriteSegment(char *buf,
-                            PRUint32 count,
-                            PRUint32 *countWritten)
+                            uint32_t count,
+                            uint32_t *countWritten)
 {
   NS_ABORT_IF_FALSE(PR_GetCurrentThread() == gSocketThread, "wrong thread");
   nsresult rv;
@@ -2123,10 +2099,10 @@ nsHttpConnection *
 SpdySession2::TakeHttpConnection()
 {
   NS_ABORT_IF_FALSE(false, "TakeHttpConnection of SpdySession2");
-  return nsnull;
+  return nullptr;
 }
 
-PRUint32
+uint32_t
 SpdySession2::CancelPipeline(nsresult reason)
 {
   // we don't pipeline inside spdy, so this isn't an issue
@@ -2155,17 +2131,16 @@ SpdySession2::SetConnection(nsAHttpConnection *)
 }
 
 void
-SpdySession2::GetSecurityCallbacks(nsIInterfaceRequestor **,
-                                  nsIEventTarget **)
+SpdySession2::GetSecurityCallbacks(nsIInterfaceRequestor **)
 {
   // This is unexpected
   NS_ABORT_IF_FALSE(false, "SpdySession2::GetSecurityCallbacks()");
 }
 
 void
-SpdySession2::SetSSLConnectFailed()
+SpdySession2::SetProxyConnectFailed()
 {
-  NS_ABORT_IF_FALSE(false, "SpdySession2::SetSSLConnectFailed()");
+  NS_ABORT_IF_FALSE(false, "SpdySession2::SetProxyConnectFailed()");
 }
 
 bool
@@ -2181,14 +2156,14 @@ SpdySession2::Status()
   return NS_ERROR_UNEXPECTED;
 }
 
-PRUint8
+uint32_t
 SpdySession2::Caps()
 {
   NS_ABORT_IF_FALSE(false, "SpdySession2::Caps()");
   return 0;
 }
 
-PRUint32
+uint64_t
 SpdySession2::Available()
 {
   NS_ABORT_IF_FALSE(false, "SpdySession2::Available()");
@@ -2205,7 +2180,7 @@ SpdySession2::RequestHead()
   return NULL;
 }
 
-PRUint32
+uint32_t
 SpdySession2::Http1xTransactionCount()
 {
   return 0;
@@ -2257,14 +2232,14 @@ SpdySession2::AddTransaction(nsAHttpTransaction *)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-PRUint32
+uint32_t
 SpdySession2::PipelineDepth()
 {
   return IsDone() ? 0 : 1;
 }
 
 nsresult
-SpdySession2::SetPipelinePosition(PRInt32 position)
+SpdySession2::SetPipelinePosition(int32_t position)
 {
   // This API is meant for pipelining, SpdySession2's should be
   // extended with AddStream()
@@ -2275,7 +2250,7 @@ SpdySession2::SetPipelinePosition(PRInt32 position)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-PRInt32
+int32_t
 SpdySession2::PipelinePosition()
 {
     return 0;
@@ -2311,7 +2286,7 @@ SpdySession2::IsReused()
 }
 
 nsresult
-SpdySession2::PushBack(const char *buf, PRUint32 len)
+SpdySession2::PushBack(const char *buf, uint32_t len)
 {
   return mConnection->PushBack(buf, len);
 }
