@@ -22,10 +22,10 @@
 // the next multipart token.  A token is usually preceded by a LF
 // or CRLF delimiter.
 // 
-static PRUint32
+static uint32_t
 LengthToToken(const char *cursor, const char *token)
 {
-    PRUint32 len = token - cursor;
+    uint32_t len = token - cursor;
     // Trim off any LF or CRLF preceding the token
     if (len && *(token-1) == '\n') {
         --len;
@@ -35,12 +35,12 @@ LengthToToken(const char *cursor, const char *token)
     return len;
 }
 
-nsPartChannel::nsPartChannel(nsIChannel *aMultipartChannel, PRUint32 aPartID,
+nsPartChannel::nsPartChannel(nsIChannel *aMultipartChannel, uint32_t aPartID,
                              nsIStreamListener* aListener) :
   mMultipartChannel(aMultipartChannel),
   mListener(aListener),
   mStatus(NS_OK),
-  mContentLength(LL_MAXUINT),
+  mContentLength(UINT64_MAX),
   mIsByteRangeRequest(false),
   mByteRangeStart(0),
   mByteRangeEnd(0),
@@ -59,7 +59,7 @@ nsPartChannel::~nsPartChannel()
 {
 }
 
-void nsPartChannel::InitializeByteRange(PRInt64 aStart, PRInt64 aEnd)
+void nsPartChannel::InitializeByteRange(int64_t aStart, int64_t aEnd)
 {
     mIsByteRangeRequest = true;
     
@@ -74,7 +74,7 @@ nsresult nsPartChannel::SendOnStartRequest(nsISupports* aContext)
 
 nsresult nsPartChannel::SendOnDataAvailable(nsISupports* aContext,
                                             nsIInputStream* aStream,
-                                            PRUint32 aOffset, PRUint32 aLen)
+                                            uint64_t aOffset, uint32_t aLen)
 {
     return mListener->OnDataAvailable(this, aContext, aStream, aOffset, aLen);
 }
@@ -301,21 +301,21 @@ nsPartChannel::SetContentCharset(const nsACString &aContentCharset)
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentLength(PRInt32 *aContentLength)
+nsPartChannel::GetContentLength(int64_t *aContentLength)
 {
-    *aContentLength = mContentLength; // XXX truncates 64-bit value
+    *aContentLength = mContentLength;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetContentLength(PRInt32 aContentLength)
+nsPartChannel::SetContentLength(int64_t aContentLength)
 {
     mContentLength = aContentLength;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentDisposition(PRUint32 *aContentDisposition)
+nsPartChannel::GetContentDisposition(uint32_t *aContentDisposition)
 {
     if (mContentDispositionHeader.IsEmpty())
         return NS_ERROR_NOT_AVAILABLE;
@@ -324,6 +324,12 @@ nsPartChannel::GetContentDisposition(PRUint32 *aContentDisposition)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPartChannel::SetContentDisposition(uint32_t aContentDisposition)
+{
+    return NS_ERROR_NOT_AVAILABLE;
+}
+
 NS_IMETHODIMP
 nsPartChannel::GetContentDispositionFilename(nsAString &aContentDispositionFilename)
 {
@@ -334,6 +340,13 @@ nsPartChannel::GetContentDispositionFilename(nsAString &aContentDispositionFilen
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPartChannel::SetContentDispositionFilename(const nsAString &aContentDispositionFilename)
+{
+    return NS_ERROR_NOT_AVAILABLE;
+}
+
+
 NS_IMETHODIMP
 nsPartChannel::GetContentDispositionHeader(nsACString &aContentDispositionHeader)
 {
@@ -345,7 +358,7 @@ nsPartChannel::GetContentDispositionHeader(nsACString &aContentDispositionHeader
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetPartID(PRUint32 *aPartID)
+nsPartChannel::GetPartID(uint32_t *aPartID)
 {
     *aPartID = mPartID;
     return NS_OK;
@@ -372,7 +385,7 @@ nsPartChannel::GetIsByteRangeRequest(bool *aIsByteRangeRequest)
 
 
 NS_IMETHODIMP 
-nsPartChannel::GetStartRange(PRInt64 *aStartRange)
+nsPartChannel::GetStartRange(int64_t *aStartRange)
 {
     *aStartRange = mByteRangeStart;
 
@@ -380,7 +393,7 @@ nsPartChannel::GetStartRange(PRInt64 *aStartRange)
 }
 
 NS_IMETHODIMP 
-nsPartChannel::GetEndRange(PRInt64 *aEndRange)
+nsPartChannel::GetEndRange(int64_t *aEndRange)
 {
     *aEndRange = mByteRangeEnd;
     return NS_OK;
@@ -458,14 +471,15 @@ private:
 // nsIStreamListener implementation
 NS_IMETHODIMP
 nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
-                                  nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count) {
+                                  nsIInputStream *inStr, uint64_t sourceOffset,
+                                  uint32_t count) {
 
     if (mToken.IsEmpty()) // no token, no love.
         return NS_ERROR_FAILURE;
 
     nsresult rv = NS_OK;
-    AutoFree buffer = nsnull;
-    PRUint32 bufLen = 0, read = 0;
+    AutoFree buffer = nullptr;
+    uint32_t bufLen = 0, read = 0;
 
     NS_ASSERTION(request, "multimixed converter needs a request");
 
@@ -530,7 +544,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         }
     }
 
-    char *token = nsnull;
+    char *token = nullptr;
 
     if (mProcessingHeaders) {
         // we were not able to process all the headers
@@ -547,7 +561,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         }
     }
 
-    PRInt32 tokenLinefeed = 1;
+    int32_t tokenLinefeed = 1;
     while ( (token = FindToken(cursor, bufLen)) ) {
 
         if (*(token+mTokenLen+1) == '-') {
@@ -591,7 +605,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             mNewPart = true;
             // Reset state so we don't carry it over from part to part
             mContentType.Truncate();
-            mContentLength = LL_MAXUINT;
+            mContentLength = UINT64_MAX;
             mContentDisposition.Truncate();
             mIsByteRangeRequest = false;
             mByteRangeStart = 0;
@@ -612,7 +626,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
     // we buffer enough data to handle a broken token.
 
     // carry over
-    PRUint32 bufAmt = 0;
+    uint32_t bufAmt = 0;
     if (mProcessingHeaders)
         bufAmt = bufLen;
     else if (bufLen) {
@@ -646,8 +660,8 @@ NS_IMETHODIMP
 nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
     // we're assuming the content-type is available at this stage
     NS_ASSERTION(mToken.IsEmpty(), "a second on start???");
-    const char *bndry = nsnull;
-    nsCAutoString delimiter;
+    const char *bndry = nullptr;
+    nsAutoCString delimiter;
     nsresult rv = NS_OK;
     mContext = ctxt;
 
@@ -679,7 +693,7 @@ nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
     char *attrib = (char *) strchr(bndry, ';');
     if (attrib) *attrib = '\0';
 
-    nsCAutoString boundaryString(bndry);
+    nsAutoCString boundaryString(bndry);
     if (attrib) *attrib = ';';
 
     boundaryString.Trim(" \"");
@@ -710,7 +724,7 @@ nsMultiMixedConv::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
             // don't bother checking the return value here, if the send failed
             // we're done anyway as we're in the OnStop() callback.
             free(mBuffer);
-            mBuffer = nsnull;
+            mBuffer = nullptr;
             mBufLen = 0;
         }
         (void) SendStop(aStatus);
@@ -737,8 +751,8 @@ nsMultiMixedConv::nsMultiMixedConv() :
 {
     mTokenLen           = 0;
     mNewPart            = true;
-    mContentLength      = LL_MAXUINT;
-    mBuffer             = nsnull;
+    mContentLength      = UINT64_MAX;
+    mBuffer             = nullptr;
     mBufLen             = 0;
     mProcessingHeaders  = false;
     mByteRangeStart     = 0;
@@ -751,12 +765,12 @@ nsMultiMixedConv::~nsMultiMixedConv() {
     NS_ASSERTION(!mBuffer, "all buffered data should be gone");
     if (mBuffer) {
         free(mBuffer);
-        mBuffer = nsnull;
+        mBuffer = nullptr;
     }
 }
 
 nsresult
-nsMultiMixedConv::BufferData(char *aData, PRUint32 aLen) {
+nsMultiMixedConv::BufferData(char *aData, uint32_t aLen) {
     NS_ASSERTION(!mBuffer, "trying to over-write buffer");
 
     char *buffer = (char *) malloc(aLen);
@@ -812,7 +826,7 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
     rv = mPartChannel->SetContentType(mContentType);
     if (NS_FAILED(rv)) return rv;
 
-    rv = mPartChannel->SetContentLength(mContentLength); // XXX Truncates 64-bit!
+    rv = mPartChannel->SetContentLength(mContentLength);
     if (NS_FAILED(rv)) return rv;
 
     mPartChannel->SetContentDisposition(mContentDisposition);
@@ -827,7 +841,7 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
 
     // Add the new channel to the load group (if any)
     if (loadGroup) {
-        rv = loadGroup->AddRequest(mPartChannel, nsnull);
+        rv = loadGroup->AddRequest(mPartChannel, nullptr);
         if (NS_FAILED(rv)) return rv;
     }
 
@@ -858,23 +872,23 @@ nsMultiMixedConv::SendStop(nsresult aStatus) {
 }
 
 nsresult
-nsMultiMixedConv::SendData(char *aBuffer, PRUint32 aLen) {
+nsMultiMixedConv::SendData(char *aBuffer, uint32_t aLen) {
 
     nsresult rv = NS_OK;
     
     if (!mPartChannel) return NS_ERROR_FAILURE; // something went wrong w/ processing
 
-    if (mContentLength != LL_MAXUINT) {
+    if (mContentLength != UINT64_MAX) {
         // make sure that we don't send more than the mContentLength
         // XXX why? perhaps the Content-Length header was actually wrong!!
-        if ((PRUint64(aLen) + mTotalSent) > mContentLength)
+        if ((uint64_t(aLen) + mTotalSent) > mContentLength)
             aLen = mContentLength - mTotalSent;
 
         if (aLen == 0)
             return NS_OK;
     }
 
-    PRUint32 offset = mTotalSent;
+    uint64_t offset = mTotalSent;
     mTotalSent += aLen;
 
     nsCOMPtr<nsIStringInputStream> ss(
@@ -892,9 +906,9 @@ nsMultiMixedConv::SendData(char *aBuffer, PRUint32 aLen) {
     return mPartChannel->SendOnDataAvailable(mContext, inStream, offset, aLen);
 }
 
-PRInt32
-nsMultiMixedConv::PushOverLine(char *&aPtr, PRUint32 &aLen) {
-    PRInt32 chars = 0;
+int32_t
+nsMultiMixedConv::PushOverLine(char *&aPtr, uint32_t &aLen) {
+    int32_t chars = 0;
     if ((aLen > 0) && (*aPtr == nsCRT::CR || *aPtr == nsCRT::LF)) {
         if ((aLen > 1) && (aPtr[1] == nsCRT::LF))
             chars++;
@@ -907,16 +921,16 @@ nsMultiMixedConv::PushOverLine(char *&aPtr, PRUint32 &aLen) {
 
 nsresult
 nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr, 
-                               PRUint32 &aLen, bool *_retval) {
+                               uint32_t &aLen, bool *_retval) {
     // NOTE: this data must be ascii.
     // NOTE: aPtr is NOT null terminated!
     nsresult rv = NS_OK;
-    char *cursor = aPtr, *newLine = nsnull;
-    PRUint32 cursorLen = aLen;
+    char *cursor = aPtr, *newLine = nullptr;
+    uint32_t cursorLen = aLen;
     bool done = false;
-    PRUint32 lineFeedIncrement = 1;
+    uint32_t lineFeedIncrement = 1;
     
-    mContentLength = LL_MAXUINT; // XXX what if we were already called?
+    mContentLength = UINT64_MAX; // XXX what if we were already called?
     while (cursorLen && (newLine = (char *) memchr(cursor, nsCRT::LF, cursorLen))) {
         // adjust for linefeeds
         if ((newLine > cursor) && (newLine[-1] == nsCRT::CR) ) { // CRLF
@@ -942,18 +956,18 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
         char *colon = (char *) strchr(cursor, ':');
         if (colon) {
             *colon = '\0';
-            nsCAutoString headerStr(cursor);
+            nsAutoCString headerStr(cursor);
             headerStr.CompressWhitespace();
             *colon = ':';
 
-            nsCAutoString headerVal(colon + 1);
+            nsAutoCString headerVal(colon + 1);
             headerVal.CompressWhitespace();
 
             // examine header
             if (headerStr.LowerCaseEqualsLiteral("content-type")) {
                 mContentType = headerVal;
             } else if (headerStr.LowerCaseEqualsLiteral("content-length")) {
-                mContentLength = atoi(headerVal.get()); // XXX 64-bit math?
+                mContentLength = nsCRT::atoll(headerVal.get());
             } else if (headerStr.LowerCaseEqualsLiteral("content-disposition")) {
                 mContentDisposition = headerVal;
             } else if (headerStr.LowerCaseEqualsLiteral("set-cookie")) {
@@ -973,10 +987,13 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
 
                 // pass the bytes-unit and the SP
                 char *range = (char *) strchr(colon + 2, ' ');
-
                 if (!range)
                     return NS_ERROR_FAILURE;
 
+                do {
+                    range++;
+                } while (*range == ' ');
+
                 if (range[0] == '*'){
                     mByteRangeStart = mByteRangeEnd = 0;
                 }
@@ -987,14 +1004,14 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
                     
                     tmpPtr[0] = '\0';
                     
-                    mByteRangeStart = atoi(range); // XXX want 64-bit conv
+                    mByteRangeStart = nsCRT::atoll(range);
                     tmpPtr++;
-                    mByteRangeEnd = atoi(tmpPtr);
+                    mByteRangeEnd = nsCRT::atoll(tmpPtr);
                 }
 
                 mIsByteRangeRequest = true;
-                if (mContentLength == LL_MAXUINT)
-                    mContentLength = PRUint64(PRInt64(mByteRangeEnd - mByteRangeStart + PRInt64(1)));
+                if (mContentLength == UINT64_MAX)
+                    mContentLength = uint64_t(mByteRangeEnd - mByteRangeStart + 1);
             }
         }
         *newLine = tmpChar;
@@ -1011,14 +1028,14 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
 }
 
 char *
-nsMultiMixedConv::FindToken(char *aCursor, PRUint32 aLen) {
+nsMultiMixedConv::FindToken(char *aCursor, uint32_t aLen) {
     // strnstr without looking for null termination
     const char *token = mToken.get();
     char *cur = aCursor;
 
     if (!(token && aCursor && *token)) {
         NS_WARNING("bad data");
-        return nsnull;
+        return nullptr;
     }
 
     for (; aLen >= mTokenLen; aCursor++, aLen--) {
@@ -1038,13 +1055,13 @@ nsMultiMixedConv::FindToken(char *aCursor, PRUint32 aLen) {
         }
     }
 
-    return nsnull;
+    return nullptr;
 }
 
 nsresult
 NS_NewMultiMixedConv(nsMultiMixedConv** aMultiMixedConv)
 {
-    NS_PRECONDITION(aMultiMixedConv != nsnull, "null ptr");
+    NS_PRECONDITION(aMultiMixedConv != nullptr, "null ptr");
     if (! aMultiMixedConv)
         return NS_ERROR_NULL_POINTER;
 
