@@ -45,11 +45,11 @@
 
 #include "nsEnumeratorUtils.h"
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 #include "prlog.h"
 #include "prprf.h"
 #include <stdio.h>
-PRLogModuleInfo* nsRDFLog = nsnull;
+PRLogModuleInfo* nsRDFLog = nullptr;
 #endif
 
 static NS_DEFINE_IID(kISupportsIID,           NS_ISUPPORTS_IID);
@@ -95,7 +95,7 @@ protected:
 
 	bool        mAllowNegativeAssertions;
 	bool        mCoalesceDuplicateArcs;
-    PRInt32     mUpdateBatchNest;
+    int32_t     mUpdateBatchNest;
 
     nsFixedSizeAllocator mAllocator;
 
@@ -139,7 +139,7 @@ protected:
 
     nsISimpleEnumerator* mCurrent;
     nsIRDFNode*  mResult;
-    PRInt32      mNext;
+    int32_t      mNext;
     nsAutoTArray<nsCOMPtr<nsIRDFNode>, 8>  mAlreadyReturned;
     bool mAllowNegativeAssertions;
     bool mCoalesceDuplicateArcs;
@@ -150,8 +150,8 @@ CompositeEnumeratorImpl::CompositeEnumeratorImpl(CompositeDataSourceImpl* aCompo
                                                  bool aAllowNegativeAssertions,
                                                  bool aCoalesceDuplicateArcs)
     : mCompositeDataSource(aCompositeDataSource),
-      mCurrent(nsnull),
-      mResult(nsnull),
+      mCurrent(nullptr),
+      mResult(nullptr),
 	  mNext(0),
       mAllowNegativeAssertions(aAllowNegativeAssertions),
       mCoalesceDuplicateArcs(aCoalesceDuplicateArcs)
@@ -174,7 +174,7 @@ NS_IMPL_QUERY_INTERFACE1(CompositeEnumeratorImpl, nsISimpleEnumerator)
 NS_IMETHODIMP
 CompositeEnumeratorImpl::HasMoreElements(bool* aResult)
 {
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -201,13 +201,13 @@ CompositeEnumeratorImpl::HasMoreElements(bool* aResult)
             if (rv == NS_RDF_NO_VALUE)
                 continue;
 
-            NS_ASSERTION(mCurrent != nsnull, "you're always supposed to return an enumerator from GetEnumerator, punk.");
+            NS_ASSERTION(mCurrent != nullptr, "you're always supposed to return an enumerator from GetEnumerator, punk.");
             if (! mCurrent)
                 continue;
         }
 
         do {
-            PRInt32 i;
+            int32_t i;
 
             bool hasMore;
             rv = mCurrent->HasMoreElements(&hasMore);
@@ -314,7 +314,7 @@ CompositeEnumeratorImpl::GetNext(nsISupports** aResult)
 
     // Don't AddRef: we "transfer" ownership to the caller
     *aResult = mResult;
-    mResult = nsnull;
+    mResult = nullptr;
 
     return NS_OK;
 }
@@ -343,7 +343,7 @@ public:
                                                              aNode, aType,
                                                              aAllowNegativeAssertions,
                                                              aCoalesceDuplicateArcs)
-            : nsnull; }
+            : nullptr; }
 
     virtual ~CompositeArcsInOutEnumeratorImpl();
 
@@ -365,8 +365,6 @@ protected:
 private:
     nsIRDFNode* mNode;
     Type        mType;
-    bool	    mAllowNegativeAssertions;
-    bool        mCoalesceDuplicateArcs;
 
     // Hide so that only Create() and Destroy() can be used to
     // allocate and deallocate from the heap
@@ -383,9 +381,7 @@ CompositeArcsInOutEnumeratorImpl::CompositeArcsInOutEnumeratorImpl(
                 bool aCoalesceDuplicateArcs)
     : CompositeEnumeratorImpl(aCompositeDataSource, aAllowNegativeAssertions, aCoalesceDuplicateArcs),
       mNode(aNode),
-      mType(aType),
-      mAllowNegativeAssertions(aAllowNegativeAssertions),
-      mCoalesceDuplicateArcs(aCoalesceDuplicateArcs)
+      mType(aType)
 {
     NS_ADDREF(mNode);
 }
@@ -457,7 +453,7 @@ public:
                                                              aTruthValue,
                                                              aAllowNegativeAssertions,
                                                              aCoalesceDuplicateArcs)
-            : nsnull; }
+            : nullptr; }
 
     virtual nsresult
     GetEnumerator(nsIRDFDataSource* aDataSource, nsISimpleEnumerator** aResult);
@@ -483,8 +479,6 @@ private:
     nsIRDFResource* mProperty;
     nsIRDFNode*     mTarget;
     bool            mTruthValue;
-    bool            mAllowNegativeAssertions;
-    bool            mCoalesceDuplicateArcs;
 
     // Hide so that only Create() and Destroy() can be used to
     // allocate and deallocate from the heap
@@ -505,9 +499,7 @@ CompositeAssertionEnumeratorImpl::CompositeAssertionEnumeratorImpl(
       mSource(aSource),
       mProperty(aProperty),
       mTarget(aTarget),
-      mTruthValue(aTruthValue),
-      mAllowNegativeAssertions(aAllowNegativeAssertions),
-      mCoalesceDuplicateArcs(aCoalesceDuplicateArcs)
+      mTruthValue(aTruthValue)
 {
     NS_IF_ADDREF(mSource);
     NS_ADDREF(mProperty); // always must be specified
@@ -586,15 +578,15 @@ CompositeDataSourceImpl::CompositeDataSourceImpl(void)
         sizeof(CompositeAssertionEnumeratorImpl),
         sizeof(CompositeArcsInOutEnumeratorImpl) };
 
-    static const PRInt32 kNumBuckets = sizeof(kBucketSizes) / sizeof(size_t);
+    static const int32_t kNumBuckets = sizeof(kBucketSizes) / sizeof(size_t);
 
     // Per news://news.mozilla.org/39BEC105.5090206%40netscape.com
-    static const PRInt32 kInitialSize = 256;
+    static const int32_t kInitialSize = 256;
 
     mAllocator.Init("nsCompositeDataSource", kBucketSizes, kNumBuckets, kInitialSize);
 
 #ifdef PR_LOGGING
-    if (nsRDFLog == nsnull) 
+    if (nsRDFLog == nullptr) 
         nsRDFLog = PR_NewLogModule("RDF");
 #endif
 }
@@ -606,16 +598,16 @@ CompositeDataSourceImpl::CompositeDataSourceImpl(void)
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(CompositeDataSourceImpl)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(CompositeDataSourceImpl)
-    PRUint32 i, count = tmp->mDataSources.Count();
+    uint32_t i, count = tmp->mDataSources.Count();
     for (i = count; i > 0; --i) {
         tmp->mDataSources[i - 1]->RemoveObserver(tmp);
         tmp->mDataSources.RemoveObjectAt(i - 1);
     }
-    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMARRAY(mObservers);
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mObservers);
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(CompositeDataSourceImpl)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMARRAY(mObservers)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMARRAY(mDataSources)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObservers)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDataSources)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 
@@ -639,7 +631,7 @@ NS_INTERFACE_MAP_END
 NS_IMETHODIMP
 CompositeDataSourceImpl::GetURI(char* *uri)
 {
-    *uri = nsnull;
+    *uri = nullptr;
     return NS_OK;
 }
 
@@ -652,8 +644,8 @@ CompositeDataSourceImpl::GetSource(nsIRDFResource* property,
 	if (!mAllowNegativeAssertions && !tv)
 		return(NS_RDF_NO_VALUE);
 
-    PRInt32 count = mDataSources.Count();
-    for (PRInt32 i = 0; i < count; ++i) {
+    int32_t count = mDataSources.Count();
+    for (int32_t i = 0; i < count; ++i) {
         nsresult rv;
         rv = mDataSources[i]->GetSource(property, target, tv, source);
         if (NS_FAILED(rv)) return rv;
@@ -680,15 +672,15 @@ CompositeDataSourceImpl::GetSources(nsIRDFResource* aProperty,
                                     bool aTruthValue,
                                     nsISimpleEnumerator** aResult)
 {
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aTarget != nullptr, "null ptr");
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -696,7 +688,7 @@ CompositeDataSourceImpl::GetSources(nsIRDFResource* aProperty,
         return(NS_RDF_NO_VALUE);
 
     *aResult = CompositeAssertionEnumeratorImpl::Create(mAllocator,
-                                                        this, nsnull, aProperty,
+                                                        this, nullptr, aProperty,
                                                         aTarget, aTruthValue,
                                                         mAllowNegativeAssertions,
                                                         mCoalesceDuplicateArcs);
@@ -714,23 +706,23 @@ CompositeDataSourceImpl::GetTarget(nsIRDFResource* aSource,
                                    bool aTruthValue,
                                    nsIRDFNode** aResult)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
     if (! mAllowNegativeAssertions && ! aTruthValue)
         return(NS_RDF_NO_VALUE);
 
-    PRInt32 count = mDataSources.Count();
-    for (PRInt32 i = 0; i < count; ++i) {
+    int32_t count = mDataSources.Count();
+    for (int32_t i = 0; i < count; ++i) {
         nsresult rv;
         rv = mDataSources[i]->GetTarget(aSource, aProperty, aTruthValue,
                                         aResult);
@@ -764,7 +756,7 @@ CompositeDataSourceImpl::HasAssertionN(int n,
                                        bool aTruthValue)
 {
     nsresult rv;
-    for (PRInt32 m = 0; m < n; ++m) {
+    for (int32_t m = 0; m < n; ++m) {
         bool result;
         rv = mDataSources[m]->HasAssertion(aSource, aProperty, aTarget,
                                            aTruthValue, &result);
@@ -786,15 +778,15 @@ CompositeDataSourceImpl::GetTargets(nsIRDFResource* aSource,
                                     bool aTruthValue,
                                     nsISimpleEnumerator** aResult)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -803,7 +795,7 @@ CompositeDataSourceImpl::GetTargets(nsIRDFResource* aSource,
 
     *aResult =
         CompositeAssertionEnumeratorImpl::Create(mAllocator, this,
-                                                 aSource, aProperty, nsnull,
+                                                 aSource, aProperty, nullptr,
                                                  aTruthValue,
                                                  mAllowNegativeAssertions,
                                                  mCoalesceDuplicateArcs);
@@ -821,15 +813,15 @@ CompositeDataSourceImpl::Assert(nsIRDFResource* aSource,
                                 nsIRDFNode* aTarget,
                                 bool aTruthValue)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aTarget != nullptr, "null ptr");
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
@@ -843,7 +835,7 @@ CompositeDataSourceImpl::Assert(nsIRDFResource* aSource,
     // We iterate backwards from the last data source which was added
     // ("the most remote") to the first ("the most local"), trying to
     // apply the assertion in each.
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         rv = mDataSources[i]->Assert(aSource, aProperty, aTarget, aTruthValue);
         if (NS_RDF_ASSERTION_ACCEPTED == rv)
             return rv;
@@ -861,15 +853,15 @@ CompositeDataSourceImpl::Unassert(nsIRDFResource* aSource,
                                   nsIRDFResource* aProperty,
                                   nsIRDFNode* aTarget)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aTarget != nullptr, "null ptr");
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
@@ -879,8 +871,8 @@ CompositeDataSourceImpl::Unassert(nsIRDFResource* aSource,
     // most local" and moving to "the most remote". If _any_ of the
     // datasources have the assertion, attempt to unassert it.
     bool unasserted = true;
-    PRInt32 i;
-    PRInt32 count = mDataSources.Count();
+    int32_t i;
+    int32_t count = mDataSources.Count();
     for (i = 0; i < count; ++i) {
         nsIRDFDataSource* ds = mDataSources[i];
 
@@ -927,19 +919,19 @@ CompositeDataSourceImpl::Change(nsIRDFResource* aSource,
                                 nsIRDFNode* aOldTarget,
                                 nsIRDFNode* aNewTarget)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aOldTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aOldTarget != nullptr, "null ptr");
     if (! aOldTarget)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aNewTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aNewTarget != nullptr, "null ptr");
     if (! aNewTarget)
         return NS_ERROR_NULL_POINTER;
 
@@ -952,7 +944,7 @@ CompositeDataSourceImpl::Change(nsIRDFResource* aSource,
     // We iterate backwards from the last data source which was added
     // ("the most remote") to the first ("the most local"), trying to
     // apply the change in each.
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         rv = mDataSources[i]->Change(aSource, aProperty, aOldTarget, aNewTarget);
         if (NS_RDF_ASSERTION_ACCEPTED == rv)
             return rv;
@@ -971,19 +963,19 @@ CompositeDataSourceImpl::Move(nsIRDFResource* aOldSource,
                               nsIRDFResource* aProperty,
                               nsIRDFNode* aTarget)
 {
-    NS_PRECONDITION(aOldSource != nsnull, "null ptr");
+    NS_PRECONDITION(aOldSource != nullptr, "null ptr");
     if (! aOldSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aNewSource != nsnull, "null ptr");
+    NS_PRECONDITION(aNewSource != nullptr, "null ptr");
     if (! aNewSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aTarget != nullptr, "null ptr");
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
@@ -996,7 +988,7 @@ CompositeDataSourceImpl::Move(nsIRDFResource* aOldSource,
     // We iterate backwards from the last data source which was added
     // ("the most remote") to the first ("the most local"), trying to
     // apply the assertion in each.
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         rv = mDataSources[i]->Move(aOldSource, aNewSource, aProperty, aTarget);
         if (NS_RDF_ASSERTION_ACCEPTED == rv)
             return rv;
@@ -1017,15 +1009,15 @@ CompositeDataSourceImpl::HasAssertion(nsIRDFResource* aSource,
                                       bool aTruthValue,
                                       bool* aResult)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aProperty != nsnull, "null ptr");
+    NS_PRECONDITION(aProperty != nullptr, "null ptr");
     if (! aProperty)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -1039,8 +1031,8 @@ CompositeDataSourceImpl::HasAssertion(nsIRDFResource* aSource,
 
     // Otherwise, look through all the data sources to see if anyone
     // has the positive...
-    PRInt32 count = mDataSources.Count();
-    for (PRInt32 i = 0; i < count; ++i) {
+    int32_t count = mDataSources.Count();
+    for (int32_t i = 0; i < count; ++i) {
         nsIRDFDataSource* datasource = mDataSources[i];
         rv = datasource->HasAssertion(aSource, aProperty, aTarget, aTruthValue, aResult);
         if (NS_FAILED(rv)) return rv;
@@ -1070,7 +1062,7 @@ CompositeDataSourceImpl::HasAssertion(nsIRDFResource* aSource,
 NS_IMETHODIMP
 CompositeDataSourceImpl::AddObserver(nsIRDFObserver* aObserver)
 {
-    NS_PRECONDITION(aObserver != nsnull, "null ptr");
+    NS_PRECONDITION(aObserver != nullptr, "null ptr");
     if (! aObserver)
         return NS_ERROR_NULL_POINTER;
 
@@ -1083,7 +1075,7 @@ CompositeDataSourceImpl::AddObserver(nsIRDFObserver* aObserver)
 NS_IMETHODIMP
 CompositeDataSourceImpl::RemoveObserver(nsIRDFObserver* aObserver)
 {
-    NS_PRECONDITION(aObserver != nsnull, "null ptr");
+    NS_PRECONDITION(aObserver != nullptr, "null ptr");
     if (! aObserver)
         return NS_ERROR_NULL_POINTER;
 
@@ -1097,8 +1089,8 @@ CompositeDataSourceImpl::HasArcIn(nsIRDFNode *aNode, nsIRDFResource *aArc, bool
 {
     nsresult rv;
     *result = false;
-    PRInt32 count = mDataSources.Count();
-    for (PRInt32 i = 0; i < count; ++i) {
+    int32_t count = mDataSources.Count();
+    for (int32_t i = 0; i < count; ++i) {
         rv = mDataSources[i]->HasArcIn(aNode, aArc, result);
         if (NS_FAILED(rv)) return rv;
         if (*result)
@@ -1112,8 +1104,8 @@ CompositeDataSourceImpl::HasArcOut(nsIRDFResource *aSource, nsIRDFResource *aArc
 {
     nsresult rv;
     *result = false;
-    PRInt32 count = mDataSources.Count();
-    for (PRInt32 i = 0; i < count; ++i) {
+    int32_t count = mDataSources.Count();
+    for (int32_t i = 0; i < count; ++i) {
         rv = mDataSources[i]->HasArcOut(aSource, aArc, result);
         if (NS_FAILED(rv)) return rv;
         if (*result)
@@ -1125,11 +1117,11 @@ CompositeDataSourceImpl::HasArcOut(nsIRDFResource *aSource, nsIRDFResource *aArc
 NS_IMETHODIMP
 CompositeDataSourceImpl::ArcLabelsIn(nsIRDFNode* aTarget, nsISimpleEnumerator** aResult)
 {
-    NS_PRECONDITION(aTarget != nsnull, "null ptr");
+    NS_PRECONDITION(aTarget != nullptr, "null ptr");
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -1151,11 +1143,11 @@ NS_IMETHODIMP
 CompositeDataSourceImpl::ArcLabelsOut(nsIRDFResource* aSource,
                                       nsISimpleEnumerator** aResult)
 {
-    NS_PRECONDITION(aSource != nsnull, "null ptr");
+    NS_PRECONDITION(aSource != nullptr, "null ptr");
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -1184,36 +1176,25 @@ NS_IMETHODIMP
 CompositeDataSourceImpl::GetAllCmds(nsIRDFResource* source,
                                     nsISimpleEnumerator/*<nsIRDFResource>*/** result)
 {
-    nsCOMPtr<nsISupportsArray> cmdArray;
     nsresult rv;
+    nsCOMPtr<nsISimpleEnumerator> set;
 
-    rv = NS_NewISupportsArray(getter_AddRefs(cmdArray));
-    if (NS_FAILED(rv)) return(rv);
-
-    for (PRInt32 i = 0; i < mDataSources.Count(); i++)
+    for (int32_t i = 0; i < mDataSources.Count(); i++)
     {
         nsCOMPtr<nsISimpleEnumerator> dsCmds;
 
         rv = mDataSources[i]->GetAllCmds(source, getter_AddRefs(dsCmds));
         if (NS_SUCCEEDED(rv))
         {
-            bool	hasMore = false;
-            while(NS_SUCCEEDED(rv = dsCmds->HasMoreElements(&hasMore)) &&
-                  hasMore)
-            {
-                nsCOMPtr<nsISupports>	item;
-                if (NS_SUCCEEDED(rv = dsCmds->GetNext(getter_AddRefs(item))))
-                {
-                    // rjc: do NOT strip out duplicate commands here
-                    // (due to items such as separators, it is done at a higher level)
-                    cmdArray->AppendElement(item);
-                }
-            }
+            nsCOMPtr<nsISimpleEnumerator> tmp;
+            rv = NS_NewUnionEnumerator(getter_AddRefs(tmp), set, dsCmds);
+            set.swap(tmp);
             if (NS_FAILED(rv)) return(rv);
         }
     }
 
-    return NS_NewArrayEnumerator(result, cmdArray);
+    set.forget(result);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1223,7 +1204,7 @@ CompositeDataSourceImpl::IsCommandEnabled(nsISupportsArray/*<nsIRDFResource>*/*
                                           bool* aResult)
 {
     nsresult rv;
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         bool enabled = true;
         rv = mDataSources[i]->IsCommandEnabled(aSources, aCommand, aArguments, &enabled);
         if (NS_FAILED(rv) && (rv != NS_ERROR_NOT_IMPLEMENTED))
@@ -1245,7 +1226,7 @@ CompositeDataSourceImpl::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSource
                                    nsIRDFResource*   aCommand,
                                    nsISupportsArray/*<nsIRDFResource>*/* aArguments)
 {
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         nsresult rv = mDataSources[i]->DoCommand(aSources, aCommand, aArguments);
         if (NS_FAILED(rv) && (rv != NS_ERROR_NOT_IMPLEMENTED))
         {
@@ -1258,7 +1239,7 @@ CompositeDataSourceImpl::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSource
 NS_IMETHODIMP
 CompositeDataSourceImpl::BeginUpdateBatch()
 {
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         mDataSources[i]->BeginUpdateBatch();
     }
     return NS_OK;
@@ -1267,7 +1248,7 @@ CompositeDataSourceImpl::BeginUpdateBatch()
 NS_IMETHODIMP
 CompositeDataSourceImpl::EndUpdateBatch()
 {
-    for (PRInt32 i = mDataSources.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mDataSources.Count() - 1; i >= 0; --i) {
         mDataSources[i]->EndUpdateBatch();
     }
     return NS_OK;
@@ -1311,7 +1292,7 @@ CompositeDataSourceImpl::SetCoalesceDuplicateArcs(bool aCoalesceDuplicateArcs)
 NS_IMETHODIMP
 CompositeDataSourceImpl::AddDataSource(nsIRDFDataSource* aDataSource)
 {
-    NS_ASSERTION(aDataSource != nsnull, "null ptr");
+    NS_ASSERTION(aDataSource != nullptr, "null ptr");
     if (! aDataSource)
         return NS_ERROR_NULL_POINTER;
 
@@ -1325,7 +1306,7 @@ CompositeDataSourceImpl::AddDataSource(nsIRDFDataSource* aDataSource)
 NS_IMETHODIMP
 CompositeDataSourceImpl::RemoveDataSource(nsIRDFDataSource* aDataSource)
 {
-    NS_ASSERTION(aDataSource != nsnull, "null ptr");
+    NS_ASSERTION(aDataSource != nullptr, "null ptr");
     if (! aDataSource)
         return NS_ERROR_NULL_POINTER;
 
@@ -1371,7 +1352,7 @@ CompositeDataSourceImpl::OnAssert(nsIRDFDataSource* aDataSource,
 			return(NS_OK);
 	}
 
-    for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
         mObservers[i]->OnAssert(this, aSource, aProperty, aTarget);
     }
     return NS_OK;
@@ -1402,7 +1383,7 @@ CompositeDataSourceImpl::OnUnassert(nsIRDFDataSource* aDataSource,
 			return NS_OK;
 	}
 
-    for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
         mObservers[i]->OnUnassert(this, aSource, aProperty, aTarget);
     }
     return NS_OK;
@@ -1422,7 +1403,7 @@ CompositeDataSourceImpl::OnChange(nsIRDFDataSource* aDataSource,
     // XXX Because of aggregation, this could actually mutate into a
     // variety of OnAssert or OnChange notifications, which we'll
     // ignore for now :-/.
-    for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
         mObservers[i]->OnChange(this, aSource, aProperty,
                                 aOldTarget, aNewTarget);
     }
@@ -1443,7 +1424,7 @@ CompositeDataSourceImpl::OnMove(nsIRDFDataSource* aDataSource,
     // XXX Because of aggregation, this could actually mutate into a
     // variety of OnAssert or OnMove notifications, which we'll
     // ignore for now :-/.
-    for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+    for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
         mObservers[i]->OnMove(this, aOldSource, aNewSource,
                               aProperty, aTarget);
     }
@@ -1455,7 +1436,7 @@ NS_IMETHODIMP
 CompositeDataSourceImpl::OnBeginUpdateBatch(nsIRDFDataSource* aDataSource)
 {
     if (mUpdateBatchNest++ == 0) {
-        for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+        for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
             mObservers[i]->OnBeginUpdateBatch(this);
         }
     }
@@ -1468,7 +1449,7 @@ CompositeDataSourceImpl::OnEndUpdateBatch(nsIRDFDataSource* aDataSource)
 {
     NS_ASSERTION(mUpdateBatchNest > 0, "badly nested update batch");
     if (--mUpdateBatchNest == 0) {
-        for (PRInt32 i = mObservers.Count() - 1; i >= 0; --i) {
+        for (int32_t i = mObservers.Count() - 1; i >= 0; --i) {
             mObservers[i]->OnEndUpdateBatch(this);
         }
     }
