@@ -48,7 +48,6 @@ static const char kURINC_BrowserMore2CharsetMenuRoot[] = "NC:BrowserMore2Charset
 static const char kURINC_BrowserMore3CharsetMenuRoot[] = "NC:BrowserMore3CharsetMenuRoot";
 static const char kURINC_BrowserMore4CharsetMenuRoot[] = "NC:BrowserMore4CharsetMenuRoot";
 static const char kURINC_BrowserMore5CharsetMenuRoot[] = "NC:BrowserMore5CharsetMenuRoot";
-static const char kURINC_BrowserUnicodeCharsetMenuRoot[] = "NC:BrowserUnicodeCharsetMenuRoot";
 static const char kURINC_MaileditCharsetMenuRoot[] = "NC:MaileditCharsetMenuRoot";
 static const char kURINC_MailviewCharsetMenuRoot[] = "NC:MailviewCharsetMenuRoot";
 static const char kURINC_ComposerCharsetMenuRoot[] = "NC:ComposerCharsetMenuRoot";
@@ -92,7 +91,7 @@ public:
   nsMenuEntry() { MOZ_COUNT_CTOR(nsMenuEntry); }
   ~nsMenuEntry() { MOZ_COUNT_DTOR(nsMenuEntry); }
 
-  nsCAutoString mCharset;
+  nsAutoCString mCharset;
   nsAutoString      mTitle;
 };
 
@@ -121,7 +120,6 @@ private:
   static nsIRDFResource * kNC_BrowserMore3CharsetMenuRoot;
   static nsIRDFResource * kNC_BrowserMore4CharsetMenuRoot;
   static nsIRDFResource * kNC_BrowserMore5CharsetMenuRoot;
-  static nsIRDFResource * kNC_BrowserUnicodeCharsetMenuRoot;
   static nsIRDFResource * kNC_MaileditCharsetMenuRoot;
   static nsIRDFResource * kNC_MailviewCharsetMenuRoot;
   static nsIRDFResource * kNC_ComposerCharsetMenuRoot;
@@ -144,19 +142,19 @@ private:
   bool mOthersInitialized;
 
   nsTArray<nsMenuEntry*> mBrowserMenu;
-  PRInt32                mBrowserCacheStart;
-  PRInt32                mBrowserCacheSize;
-  PRInt32                mBrowserMenuRDFPosition;
+  int32_t                mBrowserCacheStart;
+  int32_t                mBrowserCacheSize;
+  int32_t                mBrowserMenuRDFPosition;
 
   nsTArray<nsMenuEntry*> mMailviewMenu;
-  PRInt32                mMailviewCacheStart;
-  PRInt32                mMailviewCacheSize;
-  PRInt32                mMailviewMenuRDFPosition;
+  int32_t                mMailviewCacheStart;
+  int32_t                mMailviewCacheSize;
+  int32_t                mMailviewMenuRDFPosition;
 
   nsTArray<nsMenuEntry*> mComposerMenu;
-  PRInt32                mComposerCacheStart;
-  PRInt32                mComposerCacheSize;
-  PRInt32                mComposerMenuRDFPosition;
+  int32_t                mComposerCacheStart;
+  int32_t                mComposerCacheSize;
+  int32_t                mComposerMenuRDFPosition;
 
   nsCOMPtr<nsIRDFService>               mRDFService;
   nsCOMPtr<nsICharsetConverterManager> mCCManager;
@@ -189,19 +187,19 @@ private:
   nsresult AddCharsetToItemArray(nsTArray<nsMenuEntry*>* aArray,
                                  const nsAFlatCString& aCharset,
                                  nsMenuEntry ** aResult,
-                                 PRInt32 aPlace);
+                                 int32_t aPlace);
   nsresult AddCharsetArrayToItemArray(nsTArray<nsMenuEntry*> &aArray,
                                       const nsTArray<nsCString>& aCharsets);
   nsresult AddMenuItemToContainer(nsIRDFContainer * aContainer,
     nsMenuEntry * aItem, nsIRDFResource * aType, const char * aIDPrefix,
-    PRInt32 aPlace);
+    int32_t aPlace);
   nsresult AddMenuItemArrayToContainer(nsIRDFContainer * aContainer,
     nsTArray<nsMenuEntry*> * aArray, nsIRDFResource * aType);
   nsresult AddCharsetToContainer(nsTArray<nsMenuEntry*> * aArray,
                                  nsIRDFContainer * aContainer,
                                  const nsAFlatCString& aCharset,
                                  const char * aIDPrefix,
-    PRInt32 aPlace, PRInt32 aRDFPlace);
+    int32_t aPlace, int32_t aRDFPlace);
 
   nsresult AddFromPrefsToMenu(nsTArray<nsMenuEntry*> * aArray,
                               nsIRDFContainer * aContainer,
@@ -223,10 +221,10 @@ private:
   nsresult AddCharsetToCache(const nsAFlatCString& aCharset,
                              nsTArray<nsMenuEntry*> * aArray,
                              nsIRDFResource * aRDFResource,
-                             PRUint32 aCacheStart, PRUint32 aCacheSize,
-                             PRInt32 aRDFPlace);
+                             uint32_t aCacheStart, uint32_t aCacheSize,
+                             int32_t aRDFPlace);
 
-  nsresult WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray, PRInt32 aCacheStart,
+  nsresult WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray, int32_t aCacheStart,
     const char * aKey);
   nsresult UpdateCachePrefs(const char * aCacheKey, const char * aCacheSizeKey,
     const char * aStaticKey, const PRUnichar * aCharset);
@@ -239,7 +237,7 @@ private:
   nsresult NewRDFContainer(nsIRDFDataSource * aDataSource,
     nsIRDFResource * aResource, nsIRDFContainer ** aResult);
   void FreeMenuItemArray(nsTArray<nsMenuEntry*> * aArray);
-  PRInt32 FindMenuItemInArray(const nsTArray<nsMenuEntry*>* aArray,
+  int32_t FindMenuItemInArray(const nsTArray<nsMenuEntry*>* aArray,
                               const nsAFlatCString& aCharset,
                               nsMenuEntry ** aResult);
   nsresult ReorderMenuItemArray(nsTArray<nsMenuEntry*> * aArray);
@@ -286,17 +284,17 @@ NS_NewCharsetMenu(nsISupports * aOuter, const nsIID & aIID,
     return NS_ERROR_NULL_POINTER;
   }
   if (aOuter) {
-    *aResult = nsnull;
+    *aResult = nullptr;
     return NS_ERROR_NO_AGGREGATION;
   }
   nsCharsetMenu* inst = new nsCharsetMenu();
   if (!inst) {
-    *aResult = nsnull;
+    *aResult = nullptr;
     return NS_ERROR_OUT_OF_MEMORY;
   }
   nsresult res = inst->QueryInterface(aIID, aResult);
   if (NS_FAILED(res)) {
-    *aResult = nsnull;
+    *aResult = nullptr;
     delete inst;
   }
   return res;
@@ -304,14 +302,14 @@ NS_NewCharsetMenu(nsISupports * aOuter, const nsIID & aIID,
 
 struct charsetMenuSortRecord {
   nsMenuEntry* item;
-  PRUint8*     key;
-  PRUint32     len;
+  uint8_t*     key;
+  uint32_t     len;
 
 };
 
 static int CompareMenuItems(const void* aArg1, const void* aArg2, void *data)
 {
-  PRInt32 res; 
+  int32_t res; 
   nsICollation * collation = (nsICollation *) data;
   charsetMenuSortRecord *rec1 = (charsetMenuSortRecord *) aArg1;
   charsetMenuSortRecord *rec2 = (charsetMenuSortRecord *) aArg2;
@@ -330,7 +328,7 @@ nsCharsetMenu::SetArrayFromEnumerator(nsIUTF8StringEnumerator* aEnumerator,
   bool hasMore;
   rv = aEnumerator->HasMore(&hasMore);
   
-  nsCAutoString value;
+  nsAutoCString value;
   while (NS_SUCCEEDED(rv) && hasMore) {
     rv = aEnumerator->GetNext(value);
     if (NS_SUCCEEDED(rv))
@@ -456,7 +454,6 @@ nsIRDFResource * nsCharsetMenu::kNC_BrowserMore2CharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_BrowserMore3CharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_BrowserMore4CharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_BrowserMore5CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserUnicodeCharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_MaileditCharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_MailviewCharsetMenuRoot = NULL;
 nsIRDFResource * nsCharsetMenu::kNC_ComposerCharsetMenuRoot = NULL;
@@ -497,7 +494,7 @@ nsCharsetMenu::nsCharsetMenu()
   //get pref service
   nsCOMPtr<nsIPrefService> PrefService = do_GetService(NS_PREFSERVICE_CONTRACTID, &res);
   if (NS_SUCCEEDED(res))
-    res = PrefService->GetBranch(nsnull, getter_AddRefs(mPrefs));
+    res = PrefService->GetBranch(nullptr, getter_AddRefs(mPrefs));
 
   //register event listener
   mCharsetMenuObserver = new nsCharsetMenuObserver(this);
@@ -698,8 +695,6 @@ nsresult nsCharsetMenu::Init()
                              &kNC_BrowserMore4CharsetMenuRoot);
     mRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_BrowserMore5CharsetMenuRoot),
                              &kNC_BrowserMore5CharsetMenuRoot);
-    mRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_BrowserUnicodeCharsetMenuRoot),
-                             &kNC_BrowserUnicodeCharsetMenuRoot);
     mRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_MaileditCharsetMenuRoot),
                              &kNC_MaileditCharsetMenuRoot);
     mRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_MailviewCharsetMenuRoot),
@@ -738,8 +733,6 @@ nsresult nsCharsetMenu::Init()
     if (NS_FAILED(res)) goto done;
     res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore5CharsetMenuRoot, NULL);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserUnicodeCharsetMenuRoot, NULL);
-    if (NS_FAILED(res)) goto done;
     res = rdfUtil->MakeSeq(mInner, kNC_MaileditCharsetMenuRoot, NULL);
     if (NS_FAILED(res)) goto done;
     res = rdfUtil->MakeSeq(mInner, kNC_MailviewCharsetMenuRoot, NULL);
@@ -772,7 +765,6 @@ nsresult nsCharsetMenu::Done()
   NS_IF_RELEASE(kNC_BrowserMore3CharsetMenuRoot);
   NS_IF_RELEASE(kNC_BrowserMore4CharsetMenuRoot);
   NS_IF_RELEASE(kNC_BrowserMore5CharsetMenuRoot);
-  NS_IF_RELEASE(kNC_BrowserUnicodeCharsetMenuRoot);
   NS_IF_RELEASE(kNC_MaileditCharsetMenuRoot);
   NS_IF_RELEASE(kNC_MailviewCharsetMenuRoot);
   NS_IF_RELEASE(kNC_ComposerCharsetMenuRoot);
@@ -816,9 +808,7 @@ nsresult nsCharsetMenu::InitBrowserMenu()
     res = NewRDFContainer(mInner, kNC_BrowserCharsetMenuRoot, getter_AddRefs(container));
     if (NS_FAILED(res)) return res;
 
-
-    // how to clone mDecoderList??
-    nsTArray<nsCString> browserDecoderList = mDecoderList;
+    nsTArray<nsCString> browserDecoderList(mDecoderList);
 
     res = InitStaticMenu(browserDecoderList, kNC_BrowserCharsetMenuRoot, 
                          kBrowserStaticPrefKey, &mBrowserMenu);
@@ -894,7 +884,7 @@ nsresult nsCharsetMenu::InitMailviewMenu()
     res = NewRDFContainer(mInner, kNC_MailviewCharsetMenuRoot, getter_AddRefs(container));
     if (NS_FAILED(res)) return res;
 
-    nsTArray<nsCString> mailviewDecoderList = mDecoderList;
+    nsTArray<nsCString> mailviewDecoderList(mDecoderList);
 
     res = InitStaticMenu(mailviewDecoderList, kNC_MailviewCharsetMenuRoot, 
                          kMailviewStaticPrefKey, &mMailviewMenu);
@@ -930,7 +920,7 @@ nsresult nsCharsetMenu::InitComposerMenu()
     res = NewRDFContainer(mInner, kNC_ComposerCharsetMenuRoot, getter_AddRefs(container));
     if (NS_FAILED(res)) return res;
 
-    nsTArray<nsCString> composerDecoderList = mDecoderList;
+    nsTArray<nsCString> composerDecoderList(mDecoderList);
 
     // even if we fail, the show must go on
     res = InitStaticMenu(composerDecoderList, kNC_ComposerCharsetMenuRoot, 
@@ -963,7 +953,7 @@ nsresult nsCharsetMenu::InitOthers()
   nsresult res = NS_OK;
 
   if (!mOthersInitialized) {
-    nsTArray<nsCString> othersDecoderList = mDecoderList;
+    nsTArray<nsCString> othersDecoderList(mDecoderList);
 
     res = InitMoreMenu(othersDecoderList, kNC_DecodersRoot, ".notForBrowser");                 
     if (NS_FAILED(res))  return res;
@@ -971,7 +961,7 @@ nsresult nsCharsetMenu::InitOthers()
     // Using mDecoderList instead of GetEncoderList(), we can avoid having to
     // tag a whole bunch of 'font encoders' with '.notForOutgoing' in 
     // charsetData.properties file. 
-    nsTArray<nsCString> othersEncoderList = mDecoderList;
+    nsTArray<nsCString> othersEncoderList(mDecoderList);
 
     res = InitMoreMenu(othersEncoderList, kNC_EncodersRoot, ".notForOutgoing");                 
     if (NS_FAILED(res)) return res;
@@ -992,7 +982,7 @@ nsresult nsCharsetMenu::InitSecondaryTiers()
   nsresult res = NS_OK;
 
   if (!mSecondaryTiersInitialized)  {
-    nsTArray<nsCString> secondaryTiersDecoderList = mDecoderList;
+    nsTArray<nsCString> secondaryTiersDecoderList(mDecoderList);
 
     res = InitMoreSubmenus(secondaryTiersDecoderList);
     NS_ASSERTION(NS_SUCCEEDED(res), "err init browser charset more submenus");
@@ -1127,13 +1117,11 @@ nsresult nsCharsetMenu::InitMoreSubmenus(nsTArray<nsCString>& aDecs)
   nsCOMPtr<nsIRDFContainer> container3;
   nsCOMPtr<nsIRDFContainer> container4;
   nsCOMPtr<nsIRDFContainer> container5;
-  nsCOMPtr<nsIRDFContainer> containerU;
   const char key1[] = "intl.charsetmenu.browser.more1";
   const char key2[] = "intl.charsetmenu.browser.more2";
   const char key3[] = "intl.charsetmenu.browser.more3";
   const char key4[] = "intl.charsetmenu.browser.more4";
   const char key5[] = "intl.charsetmenu.browser.more5";
-  const char keyU[] = "intl.charsetmenu.browser.unicode";
 
   res = NewRDFContainer(mInner, kNC_BrowserMore1CharsetMenuRoot, 
     getter_AddRefs(container1));
@@ -1160,18 +1148,13 @@ nsresult nsCharsetMenu::InitMoreSubmenus(nsTArray<nsCString>& aDecs)
   if (NS_FAILED(res)) return res;
   AddFromNolocPrefsToMenu(NULL, container5, key5, aDecs, NULL);
 
-  res = NewRDFContainer(mInner, kNC_BrowserUnicodeCharsetMenuRoot, 
-    getter_AddRefs(containerU));
-  if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, containerU, keyU, aDecs, NULL);
-
   return res;
 }
 
 nsresult nsCharsetMenu::AddCharsetToItemArray(nsTArray<nsMenuEntry*> *aArray,
                                               const nsAFlatCString& aCharset,
                                               nsMenuEntry ** aResult,
-                                              PRInt32 aPlace)
+                                              int32_t aPlace)
 {
   nsresult res = NS_OK;
   nsMenuEntry * item = NULL; 
@@ -1186,18 +1169,15 @@ nsresult nsCharsetMenu::AddCharsetToItemArray(nsTArray<nsMenuEntry*> *aArray,
 
   item->mCharset = aCharset;
 
-  res = mCCManager->GetCharsetTitle(aCharset.get(), item->mTitle);
-  if (NS_FAILED(res)) {
+  if (NS_FAILED(mCCManager->GetCharsetTitle(aCharset.get(), item->mTitle))) {
     item->mTitle.AssignWithConversion(aCharset.get());
   }
 
   if (aArray != NULL) {
     if (aPlace < 0) {
-      res = aArray->AppendElement(item) != nsnull;
-      if (NS_FAILED(res)) goto done;
+      aArray->AppendElement(item);
     } else {
-      res = aArray->InsertElementsAt(aPlace, 1, item) != nsnull;
-      if (NS_FAILED(res)) goto done;
+      aArray->InsertElementsAt(aPlace, 1, item);
     }
   }
 
@@ -1216,9 +1196,9 @@ nsresult
 nsCharsetMenu::AddCharsetArrayToItemArray(nsTArray<nsMenuEntry*>& aArray,
                                           const nsTArray<nsCString>& aCharsets)
 {
-  PRUint32 count = aCharsets.Length();
+  uint32_t count = aCharsets.Length();
 
-  for (PRUint32 i = 0; i < count; i++) {
+  for (uint32_t i = 0; i < count; i++) {
 
     const nsCString& str = aCharsets[i];
     nsresult res = AddCharsetToItemArray(&aArray, str, NULL, -1);
@@ -1237,12 +1217,12 @@ nsresult nsCharsetMenu::AddMenuItemToContainer(
                         nsMenuEntry * aItem,
                         nsIRDFResource * aType,
                         const char * aIDPrefix,
-                        PRInt32 aPlace) 
+                        int32_t aPlace) 
 {
   nsresult res = NS_OK;
   nsCOMPtr<nsIRDFResource> node;
 
-  nsCAutoString id;
+  nsAutoCString id;
   if (aIDPrefix != NULL) id.Assign(aIDPrefix);
   id.Append(aItem->mCharset);
 
@@ -1295,10 +1275,10 @@ nsresult nsCharsetMenu::AddMenuItemArrayToContainer(
                         nsTArray<nsMenuEntry*> * aArray,
                         nsIRDFResource * aType) 
 {
-  PRUint32 count = aArray->Length();
+  uint32_t count = aArray->Length();
   nsresult res = NS_OK;
 
-  for (PRUint32 i = 0; i < count; i++) {
+  for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item == NULL) return NS_ERROR_UNEXPECTED;
 
@@ -1313,8 +1293,8 @@ nsresult nsCharsetMenu::AddCharsetToContainer(nsTArray<nsMenuEntry*> *aArray,
                                               nsIRDFContainer * aContainer,
                                               const nsAFlatCString& aCharset,
                                               const char * aIDPrefix,
-                                              PRInt32 aPlace,
-                                              PRInt32 aRDFPlace)
+                                              int32_t aPlace,
+                                              int32_t aRDFPlace)
 {
   nsresult res = NS_OK;
   nsMenuEntry * item = NULL; 
@@ -1397,8 +1377,8 @@ nsresult nsCharsetMenu::AddFromStringToMenu(
     *q = 0;
 
     // if this charset is not on the accepted list of charsets, ignore it
-    PRInt32 index;
-    index = aDecs.IndexOf(nsCAutoString(p), 0, nsIgnoreCaseCStringComparator());
+    int32_t index;
+    index = aDecs.IndexOf(nsAutoCString(p), 0, nsIgnoreCaseCStringComparator());
     if (index >= 0) {
 
       // else, add it to the menu
@@ -1420,11 +1400,11 @@ nsresult nsCharsetMenu::AddFromStringToMenu(
 
 nsresult nsCharsetMenu::AddSeparatorToContainer(nsIRDFContainer * aContainer)
 {
-  nsCAutoString str;
+  nsAutoCString str;
   str.AssignLiteral("----");
 
   // hack to generate unique id's for separators
-  static PRInt32 u = 0;
+  static int32_t u = 0;
   u++;
   str.AppendInt(u);
 
@@ -1440,11 +1420,11 @@ nsresult
 nsCharsetMenu::AddCharsetToCache(const nsAFlatCString& aCharset,
                                  nsTArray<nsMenuEntry*> * aArray,
                                  nsIRDFResource * aRDFResource,
-                                 PRUint32 aCacheStart,
-                                 PRUint32 aCacheSize,
-                                 PRInt32 aRDFPlace)
+                                 uint32_t aCacheStart,
+                                 uint32_t aCacheSize,
+                                 int32_t aRDFPlace)
 {
-  PRInt32 i;
+  int32_t i;
   nsresult res = NS_OK;
 
   i = FindMenuItemInArray(aArray, aCharset, NULL);
@@ -1467,17 +1447,17 @@ nsCharsetMenu::AddCharsetToCache(const nsAFlatCString& aCharset,
 }
 
 nsresult nsCharsetMenu::WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray,
-                                          PRInt32 aCacheStart,
+                                          int32_t aCacheStart,
                                           const char * aKey)
 {
   nsresult res = NS_OK;
 
   // create together the cache string
-  nsCAutoString cache;
-  nsCAutoString sep(NS_LITERAL_CSTRING(", "));
-  PRUint32 count = aArray->Length();
+  nsAutoCString cache;
+  nsAutoCString sep(NS_LITERAL_CSTRING(", "));
+  uint32_t count = aArray->Length();
 
-  for (PRUint32 i = aCacheStart; i < count; i++) {
+  for (uint32_t i = aCacheStart; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item != NULL) {    
       cache.Append(item->mCharset);
@@ -1502,7 +1482,7 @@ nsresult nsCharsetMenu::UpdateCachePrefs(const char * aCacheKey,
   nsXPIDLCString cachePrefValue;
   nsXPIDLCString staticPrefValue;
   NS_LossyConvertUTF16toASCII currentCharset(aCharset);
-  PRInt32 cacheSize = 0;
+  int32_t cacheSize = 0;
 
   mPrefs->GetCharPref(aCacheKey, getter_Copies(cachePrefValue));
   mPrefs->GetCharPref(aStaticKey, getter_Copies(staticPrefValue));
@@ -1518,7 +1498,7 @@ nsresult nsCharsetMenu::UpdateCachePrefs(const char * aCacheKey,
       cachePrefValue.Insert(", ", 0);
 
     cachePrefValue.Insert(currentCharset, 0);
-    if (cacheSize < (PRInt32) cachePrefValue.CountChar(',') + 1)
+    if (cacheSize < (int32_t) cachePrefValue.CountChar(',') + 1)
       cachePrefValue.Truncate(cachePrefValue.RFindChar(','));
 
     rv = mPrefs->SetCharPref(aCacheKey, cachePrefValue);
@@ -1533,8 +1513,8 @@ nsresult nsCharsetMenu::ClearMenu(nsIRDFContainer        * aContainer,
   nsresult res = NS_OK;
 
   // clean the RDF data source
-  PRUint32 count = aArray->Length();
-  for (PRUint32 i = 0; i < count; i++) {
+  uint32_t count = aArray->Length();
+  for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item != NULL) {    
       res = AddMenuItemToContainer(aContainer, item, NULL, "charset.", -2);
@@ -1553,7 +1533,7 @@ nsresult nsCharsetMenu::RemoveLastMenuItem(nsIRDFContainer * aContainer,
 {
   nsresult res = NS_OK;
 
-  PRInt32 last = aArray->Length() - 1;
+  int32_t last = aArray->Length() - 1;
   if (last >= 0) {
     nsMenuEntry * item = aArray->ElementAt(last);
     if (item != NULL) {    
@@ -1571,13 +1551,13 @@ nsresult nsCharsetMenu::RemoveFlaggedCharsets(nsTArray<nsCString>& aList,
                                               const nsString& aProp)
 {
   nsresult res = NS_OK;
-  PRUint32 count;
+  uint32_t count;
 
   count = aList.Length();
   if (NS_FAILED(res)) return res;
 
   nsAutoString str;
-  for (PRUint32 i = 0; i < count; i++) {
+  for (uint32_t i = 0; i < count; i++) {
 
     res = mCCManager->GetCharsetData(aList[i].get(), aProp.get(), str);
     if (NS_FAILED(res)) continue;
@@ -1606,8 +1586,8 @@ nsresult nsCharsetMenu::NewRDFContainer(nsIRDFDataSource * aDataSource,
 
 void nsCharsetMenu::FreeMenuItemArray(nsTArray<nsMenuEntry*> * aArray)
 {
-  PRUint32 count = aArray->Length();
-  for (PRUint32 i = 0; i < count; i++) {
+  uint32_t count = aArray->Length();
+  for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item != NULL) {
       delete item;
@@ -1616,13 +1596,13 @@ void nsCharsetMenu::FreeMenuItemArray(nsTArray<nsMenuEntry*> * aArray)
   aArray->Clear();
 }
 
-PRInt32 nsCharsetMenu::FindMenuItemInArray(const nsTArray<nsMenuEntry*>* aArray,
+int32_t nsCharsetMenu::FindMenuItemInArray(const nsTArray<nsMenuEntry*>* aArray,
                                            const nsAFlatCString& aCharset,
                                            nsMenuEntry ** aResult)
 {
-  PRUint32 count = aArray->Length();
+  uint32_t count = aArray->Length();
 
-  for (PRUint32 i=0; i < count; i++) {
+  for (uint32_t i=0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item->mCharset == aCharset) {
       if (aResult != NULL) *aResult = item;
@@ -1638,14 +1618,14 @@ nsresult nsCharsetMenu::ReorderMenuItemArray(nsTArray<nsMenuEntry*> * aArray)
 {
   nsresult res = NS_OK;
   nsCOMPtr<nsICollation> collation;
-  PRUint32 count = aArray->Length();
-  PRUint32 i;
+  uint32_t count = aArray->Length();
+  uint32_t i;
 
   // we need to use a temporary array
   charsetMenuSortRecord *array = new charsetMenuSortRecord [count];
   NS_ENSURE_TRUE(array, NS_ERROR_OUT_OF_MEMORY);
   for (i = 0; i < count; i++)
-    array[i].key = nsnull;
+    array[i].key = nullptr;
 
   res = GetCollation(getter_AddRefs(collation));
   if (NS_FAILED(res))
@@ -1680,8 +1660,8 @@ done:
 nsresult nsCharsetMenu::GetCollation(nsICollation ** aCollation)
 {
   nsresult res = NS_OK;
-  nsCOMPtr<nsILocale> locale = nsnull;
-  nsICollationFactory * collationFactory = nsnull;
+  nsCOMPtr<nsILocale> locale = nullptr;
+  nsICollationFactory * collationFactory = nullptr;
   
   nsCOMPtr<nsILocaleService> localeServ = 
            do_GetService(NS_LOCALESERVICE_CONTRACTID, &res);
