@@ -1,6 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim:expandtab:shiftwidth=4:tabstop=4:
- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=4 et sw=4 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -25,10 +24,6 @@
 #include "nsCRT.h"
 #include "mozilla/Services.h"
 
-#if defined(MOZ_WIDGET_GTK2)
-#include "gtk2compat.h"
-#endif
-
 #include "gfxASurface.h"
 #include "gfxXlibSurface.h"
 #include "gfxContext.h"
@@ -36,7 +31,7 @@
 #include "nsPresContext.h"
 #include "nsIDocument.h"
 #include "nsISelection.h"
-#include "nsIViewManager.h"
+#include "nsViewManager.h"
 #include "nsIFrame.h"
 
 // This sets how opaque the drag image is
@@ -229,7 +224,6 @@ OnSourceGrabEventAfter(GtkWidget *widget, GdkEvent *event, gpointer user_data)
     if (!gtk_widget_has_grab(sGrabWidget))
         return;
 
-    GdkModifierType state;
     if (event->type == GDK_MOTION_NOTIFY) {
         if (sMotionEvent) {
             gdk_event_free(sMotionEvent);
@@ -275,7 +269,7 @@ GetGtkWindow(nsIDOMDocument *aDocument)
     if (!presShell)
         return NULL;
 
-    nsCOMPtr<nsIViewManager> vm = presShell->GetViewManager();
+    nsRefPtr<nsViewManager> vm = presShell->GetViewManager();
     if (!vm)
         return NULL;
 
@@ -303,7 +297,7 @@ NS_IMETHODIMP
 nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
                                  nsISupportsArray * aArrayTransferables,
                                  nsIScriptableRegion * aRegion,
-                                 PRUint32 aActionType)
+                                 uint32_t aActionType)
 {
     PR_LOG(sDragLm, PR_LOG_DEBUG, ("nsDragService::InvokeDragSession"));
 
@@ -352,7 +346,7 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
     GdkEvent event;
     memset(&event, 0, sizeof(GdkEvent));
     event.type = GDK_BUTTON_PRESS;
-    event.button.window = mHiddenWidget->window;
+    event.button.window = gtk_widget_get_window(mHiddenWidget);
     event.button.time = nsWindow::GetLastUserInputTime();
 
     // Put the drag widget in the window group of the source node so that the
@@ -363,6 +357,13 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
     gtk_window_group_add_window(window_group,
                                 GTK_WINDOW(mHiddenWidget));
 
+#if (MOZ_WIDGET_GTK == 3)
+    // Get device for event source
+    GdkDisplay *display = gdk_display_get_default();
+    GdkDeviceManager *device_manager = gdk_display_get_device_manager(display);
+    event.button.device = gdk_device_manager_get_client_pointer(device_manager);
+#endif
+  
     // start our drag.
     GdkDragContext *context = gtk_drag_begin(mHiddenWidget,
                                              sourceList,
@@ -370,7 +371,7 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
                                              1,
                                              &event);
 
-    mSourceRegion = nsnull;
+    mSourceRegion = nullptr;
 
     if (context) {
         StartDragSession();
@@ -399,10 +400,11 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
 bool
 nsDragService::SetAlphaPixmap(gfxASurface *aSurface,
                                  GdkDragContext *aContext,
-                                 PRInt32 aXOffset,
-                                 PRInt32 aYOffset,
+                                 int32_t aXOffset,
+                                 int32_t aYOffset,
                                  const nsIntRect& dragRect)
 {
+#if (MOZ_WIDGET_GTK == 2)
     GdkScreen* screen = gtk_widget_get_screen(mHiddenWidget);
 
     // Transparent drag icons need, like a lot of transparency-related things,
@@ -444,6 +446,10 @@ nsDragService::SetAlphaPixmap(gfxASurface *aSurface,
                              aXOffset, aYOffset);
     g_object_unref(pixmap);
     return true;
+#else
+    // TODO GTK3
+    return false;
+#endif
 }
 
 NS_IMETHODIMP
@@ -499,13 +505,13 @@ nsDragService::GetCanDrop(bool *aCanDrop)
 }
 
 // count the number of URIs in some text/uri-list format data.
-static PRUint32
+static uint32_t
 CountTextUriListItems(const char *data,
-                      PRUint32 datalen)
+                      uint32_t datalen)
 {
     const char *p = data;
     const char *endPtr = p + datalen;
-    PRUint32 count = 0;
+    uint32_t count = 0;
 
     while (p < endPtr) {
         // skip whitespace (if any)
@@ -526,16 +532,16 @@ CountTextUriListItems(const char *data,
 // unicode.
 static void
 GetTextUriListItem(const char *data,
-                   PRUint32 datalen,
-                   PRUint32 aItemIndex,
+                   uint32_t datalen,
+                   uint32_t aItemIndex,
                    PRUnichar **convertedText,
-                   PRInt32 *convertedTextLen)
+                   int32_t *convertedTextLen)
 {
     const char *p = data;
     const char *endPtr = p + datalen;
     unsigned int count = 0;
 
-    *convertedText = nsnull;
+    *convertedText = nullptr;
     while (p < endPtr) {
         // skip whitespace (if any)
         while (p < endPtr && *p != '\0' && isspace(*p))
@@ -566,7 +572,7 @@ GetTextUriListItem(const char *data,
 }
 
 NS_IMETHODIMP
-nsDragService::GetNumDropItems(PRUint32 * aNumItems)
+nsDragService::GetNumDropItems(uint32_t * aNumItems)
 {
     PR_LOG(sDragLm, PR_LOG_DEBUG, ("nsDragService::GetNumDropItems"));
 
@@ -597,7 +603,7 @@ nsDragService::GetNumDropItems(PRUint32 * aNumItems)
 
 NS_IMETHODIMP
 nsDragService::GetData(nsITransferable * aTransferable,
-                       PRUint32 aItemIndex)
+                       uint32_t aItemIndex)
 {
     PR_LOG(sDragLm, PR_LOG_DEBUG, ("nsDragService::GetData %d", aItemIndex));
 
@@ -622,7 +628,7 @@ nsDragService::GetData(nsITransferable * aTransferable,
         return rv;
 
     // count the number of flavors
-    PRUint32 cnt;
+    uint32_t cnt;
     flavorList->Count(&cnt);
     unsigned int i;
 
@@ -654,7 +660,7 @@ nsDragService::GetData(nsITransferable * aTransferable,
                 continue;
 
             nsCOMPtr<nsISupports> data;
-            PRUint32 tmpDataLen = 0;
+            uint32_t tmpDataLen = 0;
             PR_LOG(sDragLm, PR_LOG_DEBUG,
                    ("trying to get transfer data for %s\n",
                    (const char *)flavorStr));
@@ -708,14 +714,14 @@ nsDragService::GetData(nsITransferable * aTransferable,
                 PR_LOG(sDragLm, PR_LOG_DEBUG, ("dataFound = false\n"));
 
                 // Dragging and dropping from the file manager would cause us 
-                // to parse the source text as a nsILocalFile URL.
+                // to parse the source text as a nsIFile URL.
                 if ( strcmp(flavorStr, kFileMime) == 0 ) {
                     gdkFlavor = gdk_atom_intern(kTextMime, FALSE);
                     GetTargetDragData(gdkFlavor);
                     if (mTargetDragData) {
                         const char* text = static_cast<char*>(mTargetDragData);
-                        PRUnichar* convertedText = nsnull;
-                        PRInt32 convertedTextLen = 0;
+                        PRUnichar* convertedText = nullptr;
+                        int32_t convertedTextLen = 0;
 
                         GetTextUriListItem(text, mTargetDragDataLen, aItemIndex,
                                            &convertedText, &convertedTextLen);
@@ -724,7 +730,7 @@ nsDragService::GetData(nsITransferable * aTransferable,
                             nsCOMPtr<nsIIOService> ioService = do_GetIOService(&rv);
                             nsCOMPtr<nsIURI> fileURI;
                             nsresult rv = ioService->NewURI(NS_ConvertUTF16toUTF8(convertedText),
-                                                            nsnull, nsnull, getter_AddRefs(fileURI));
+                                                            nullptr, nullptr, getter_AddRefs(fileURI));
                             if (NS_SUCCEEDED(rv)) {
                                 nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(fileURI, &rv);
                                 if (NS_SUCCEEDED(rv)) {
@@ -734,7 +740,7 @@ nsDragService::GetData(nsITransferable * aTransferable,
                                         // The common wrapping code at the end of 
                                         // this function assumes the data is text
                                         // and calls text-specific operations.
-                                        // Make a secret hideout here for nsILocalFile
+                                        // Make a secret hideout here for nsIFile
                                         // objects and return early.
                                         aTransferable->SetTransferData(flavorStr, file,
                                                                        convertedTextLen);
@@ -762,7 +768,7 @@ nsDragService::GetData(nsITransferable * aTransferable,
                         PR_LOG(sDragLm, PR_LOG_DEBUG, ("Got textplain data\n"));
                         const char* castedText =
                                     reinterpret_cast<char*>(mTargetDragData);
-                        PRUnichar* convertedText = nsnull;
+                        PRUnichar* convertedText = nullptr;
                         NS_ConvertUTF8toUTF16 ucs2string(castedText,
                                                          mTargetDragDataLen);
                         convertedText = ToNewUnicode(ucs2string);
@@ -786,8 +792,8 @@ nsDragService::GetData(nsITransferable * aTransferable,
                             PR_LOG(sDragLm, PR_LOG_DEBUG, ("Got textplain data\n"));
                             const char* castedText =
                                         reinterpret_cast<char*>(mTargetDragData);
-                            PRUnichar* convertedText = nsnull;
-                            PRInt32 convertedTextLen = 0;
+                            PRUnichar* convertedText = nullptr;
+                            int32_t convertedTextLen = 0;
                             nsPrimitiveHelpers::ConvertPlatformPlainTextToUnicode(
                                                 castedText, mTargetDragDataLen,
                                                 &convertedText, &convertedTextLen);
@@ -819,8 +825,8 @@ nsDragService::GetData(nsITransferable * aTransferable,
                                ("Got text/uri-list data\n"));
                         const char *data =
                                    reinterpret_cast<char*>(mTargetDragData);
-                        PRUnichar* convertedText = nsnull;
-                        PRInt32 convertedTextLen = 0;
+                        PRUnichar* convertedText = nullptr;
+                        int32_t convertedTextLen = 0;
 
                         GetTextUriListItem(data, mTargetDragDataLen, aItemIndex,
                                            &convertedText, &convertedTextLen);
@@ -851,8 +857,8 @@ nsDragService::GetData(nsITransferable * aTransferable,
                                    ("Got _NETSCAPE_URL data\n"));
                             const char* castedText =
                                   reinterpret_cast<char*>(mTargetDragData);
-                            PRUnichar* convertedText = nsnull;
-                            PRInt32 convertedTextLen = 0;
+                            PRUnichar* convertedText = nullptr;
+                            int32_t convertedTextLen = 0;
                             nsPrimitiveHelpers::ConvertPlatformPlainTextToUnicode(castedText, mTargetDragDataLen, &convertedText, &convertedTextLen);
                             if ( convertedText ) {
                                 PR_LOG(sDragLm,
@@ -928,13 +934,13 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor,
     // for it.
     if (isList) {
         PR_LOG(sDragLm, PR_LOG_DEBUG, ("It's a list.."));
-        PRUint32 numDragItems = 0;
+        uint32_t numDragItems = 0;
         // if we don't have mDataItems we didn't start this drag so it's
         // an external client trying to fool us.
         if (!mSourceDataItems)
             return NS_OK;
         mSourceDataItems->Count(&numDragItems);
-        for (PRUint32 itemIndex = 0; itemIndex < numDragItems; ++itemIndex) {
+        for (uint32_t itemIndex = 0; itemIndex < numDragItems; ++itemIndex) {
             nsCOMPtr<nsISupports> genericItem;
             mSourceDataItems->GetElementAt(itemIndex,
                                            getter_AddRefs(genericItem));
@@ -944,9 +950,9 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor,
                 currItem->FlavorsTransferableCanExport(
                           getter_AddRefs(flavorList));
                 if (flavorList) {
-                    PRUint32 numFlavors;
+                    uint32_t numFlavors;
                     flavorList->Count( &numFlavors );
-                    for ( PRUint32 flavorIndex = 0;
+                    for ( uint32_t flavorIndex = 0;
                           flavorIndex < numFlavors ;
                           ++flavorIndex ) {
                         nsCOMPtr<nsISupports> genericWrapper;
@@ -975,7 +981,8 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor,
 
     // check the target context vs. this flavor, one at a time
     GList *tmp;
-    for (tmp = mTargetDragContext->targets; tmp; tmp = tmp->next) {
+    for (tmp = gdk_drag_context_list_targets(mTargetDragContext); 
+         tmp; tmp = tmp->next) {
         /* Bug 331198 */
         GdkAtom atom = GDK_POINTER_TO_ATOM(tmp->data);
         gchar *name = NULL;
@@ -1059,15 +1066,17 @@ nsDragService::TargetDataReceived(GtkWidget         *aWidget,
     PR_LOG(sDragLm, PR_LOG_DEBUG, ("nsDragService::TargetDataReceived"));
     TargetResetData();
     mTargetDragDataReceived = true;
-    if (aSelectionData->length > 0) {
-        mTargetDragDataLen = aSelectionData->length;
+    gint len = gtk_selection_data_get_length(aSelectionData);
+    const guchar* data = gtk_selection_data_get_data(aSelectionData);
+    if (len > 0 && data) {
+        mTargetDragDataLen = len;
         mTargetDragData = g_malloc(mTargetDragDataLen);
-        memcpy(mTargetDragData, aSelectionData->data, mTargetDragDataLen);
+        memcpy(mTargetDragData, data, mTargetDragDataLen);
     }
     else {
         PR_LOG(sDragLm, PR_LOG_DEBUG,
                ("Failed to get data.  selection data len was %d\n",
-                aSelectionData->length));
+                mTargetDragDataLen));
     }
 }
 
@@ -1087,7 +1096,8 @@ nsDragService::IsTargetContextList(void)
 
     // walk the list of context targets and see if one of them is a list
     // of items.
-    for (tmp = mTargetDragContext->targets; tmp; tmp = tmp->next) {
+    for (tmp = gdk_drag_context_list_targets(mTargetDragContext); 
+         tmp; tmp = tmp->next) {
         /* Bug 331198 */
         GdkAtom atom = GDK_POINTER_TO_ATOM(tmp->data);
         gchar *name = NULL;
@@ -1145,7 +1155,7 @@ nsDragService::GetSourceList(void)
     nsTArray<GtkTargetEntry*> targetArray;
     GtkTargetEntry *targets;
     GtkTargetList  *targetList = 0;
-    PRUint32 targetCount = 0;
+    uint32_t targetCount = 0;
     unsigned int numDragItems = 0;
 
     mSourceDataItems->Count(&numDragItems);
@@ -1180,9 +1190,9 @@ nsDragService::GetSourceList(void)
             nsCOMPtr <nsISupportsArray> flavorList;
             currItem->FlavorsTransferableCanExport(getter_AddRefs(flavorList));
             if (flavorList) {
-                PRUint32 numFlavors;
+                uint32_t numFlavors;
                 flavorList->Count( &numFlavors );
-                for (PRUint32 flavorIndex = 0;
+                for (uint32_t flavorIndex = 0;
                      flavorIndex < numFlavors ;
                      ++flavorIndex ) {
                     nsCOMPtr<nsISupports> genericWrapper;
@@ -1222,9 +1232,9 @@ nsDragService::GetSourceList(void)
             nsCOMPtr <nsISupportsArray> flavorList;
             currItem->FlavorsTransferableCanExport(getter_AddRefs(flavorList));
             if (flavorList) {
-                PRUint32 numFlavors;
+                uint32_t numFlavors;
                 flavorList->Count( &numFlavors );
-                for (PRUint32 flavorIndex = 0;
+                for (uint32_t flavorIndex = 0;
                      flavorIndex < numFlavors ;
                      ++flavorIndex ) {
                     nsCOMPtr<nsISupports> genericWrapper;
@@ -1310,7 +1320,7 @@ nsDragService::GetSourceList(void)
         // allocate space to create the list of valid targets
         targets =
           (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry) * targetCount);
-        PRUint32 targetIndex;
+        uint32_t targetIndex;
         for ( targetIndex = 0; targetIndex < targetCount; ++targetIndex) {
             GtkTargetEntry *disEntry = targetArray.ElementAt(targetIndex);
             // this is a string reference but it will be freed later.
@@ -1320,7 +1330,7 @@ nsDragService::GetSourceList(void)
         }
         targetList = gtk_target_list_new(targets, targetCount);
         // clean up the target list
-        for (PRUint32 cleanIndex = 0; cleanIndex < targetCount; ++cleanIndex) {
+        for (uint32_t cleanIndex = 0; cleanIndex < targetCount; ++cleanIndex) {
             GtkTargetEntry *thisTarget = targetArray.ElementAt(cleanIndex);
             g_free(thisTarget->target);
             g_free(thisTarget);
@@ -1335,7 +1345,7 @@ nsDragService::SourceEndDragSession(GdkDragContext *aContext,
                                     gint            aResult)
 {
     // this just releases the list of data items that we provide
-    mSourceDataItems = nsnull;
+    mSourceDataItems = nullptr;
 
     if (!mDoingDrag || mScheduledTask == eDragTaskSourceEnd)
         // EndDragSession() was already called on drop
@@ -1356,7 +1366,7 @@ nsDragService::SourceEndDragSession(GdkDragContext *aContext,
     // The dropEffect of mDataTransfer is not updated for motion outside the
     // app, but is needed for the dragend event, so set it now.
 
-    PRUint32 dropEffect;
+    uint32_t dropEffect;
 
     if (aResult == MOZ_GTK_DRAG_RESULT_SUCCESS) {
 
@@ -1364,7 +1374,8 @@ nsDragService::SourceEndDragSession(GdkDragContext *aContext,
         // cancelled (but no drag-failed signal would have been sent).
         // aContext->dest_window will be non-NULL only if the drop was sent.
         GdkDragAction action =
-            aContext->dest_window ? aContext->action : (GdkDragAction)0;
+            gdk_drag_context_get_dest_window(aContext) ? 
+                gdk_drag_context_get_actions(aContext) : (GdkDragAction)0;
 
         // Only one bit of action should be set, but, just in case someone
         // does something funny, erring away from MOVE, and not recording
@@ -1394,13 +1405,13 @@ nsDragService::SourceEndDragSession(GdkDragContext *aContext,
     }
 
     // Schedule the appropriate drag end dom events.
-    Schedule(eDragTaskSourceEnd, nsnull, NULL, nsIntPoint(), 0);
+    Schedule(eDragTaskSourceEnd, nullptr, NULL, nsIntPoint(), 0);
 }
 
 static void
 CreateUriList(nsISupportsArray *items, gchar **text, gint *length)
 {
-    PRUint32 i, count;
+    uint32_t i, count;
     GString *uriList = g_string_new(NULL);
 
     items->Count(&count);
@@ -1411,9 +1422,9 @@ CreateUriList(nsISupportsArray *items, gchar **text, gint *length)
         item = do_QueryInterface(genericItem);
 
         if (item) {
-            PRUint32 tmpDataLen = 0;
+            uint32_t tmpDataLen = 0;
             void    *tmpData = NULL;
-            nsresult rv = 0;
+            nsresult rv = NS_OK;
             nsCOMPtr<nsISupports> data;
             rv = item->GetTransferData(kURLMime,
                                        getter_AddRefs(data),
@@ -1424,17 +1435,17 @@ CreateUriList(nsISupportsArray *items, gchar **text, gint *length)
                                                             data,
                                                             &tmpData,
                                                             tmpDataLen);
-                char* plainTextData = nsnull;
+                char* plainTextData = nullptr;
                 PRUnichar* castedUnicode = reinterpret_cast<PRUnichar*>
                                                            (tmpData);
-                PRInt32 plainTextLen = 0;
+                int32_t plainTextLen = 0;
                 nsPrimitiveHelpers::ConvertUnicodeToPlatformPlainText(
                                     castedUnicode,
                                     tmpDataLen / 2,
                                     &plainTextData,
                                     &plainTextLen);
                 if (plainTextData) {
-                    PRInt32 j;
+                    int32_t j;
 
                     // text/x-moz-url is of form url + "\n" + title.
                     // We just want the url.
@@ -1518,7 +1529,7 @@ nsDragService::SourceDataGet(GtkWidget        *aWidget,
         else
             actualFlavor = mimeFlavor;
 
-        PRUint32 tmpDataLen = 0;
+        uint32_t tmpDataLen = 0;
         void    *tmpData = NULL;
         nsresult rv;
         nsCOMPtr<nsISupports> data;
@@ -1531,15 +1542,15 @@ nsDragService::SourceDataGet(GtkWidget        *aWidget,
             // if required, do the extra work to convert unicode to plain
             // text and replace the output values with the plain text.
             if (needToDoConversionToPlainText) {
-                char* plainTextData = nsnull;
+                char* plainTextData = nullptr;
                 PRUnichar* castedUnicode = reinterpret_cast<PRUnichar*>
                                                            (tmpData);
-                PRInt32 plainTextLen = 0;
+                int32_t plainTextLen = 0;
                 if (strcmp(mimeFlavor, gTextPlainUTF8Type) == 0) {
                     plainTextData =
                         ToNewUTF8String(
                             nsDependentString(castedUnicode, tmpDataLen / 2),
-                            (PRUint32*)&plainTextLen);
+                            (uint32_t*)&plainTextLen);
                 } else {
                     nsPrimitiveHelpers::ConvertUnicodeToPlatformPlainText(
                                         castedUnicode,
@@ -1557,7 +1568,7 @@ nsDragService::SourceDataGet(GtkWidget        *aWidget,
             if (tmpData) {
                 // this copies the data
                 gtk_selection_data_set(aSelectionData,
-                                       aSelectionData->target,
+                                       gtk_selection_data_get_target(aSelectionData),
                                        8,
                                        (guchar *)tmpData, tmpDataLen);
                 // this wasn't allocated with glib
@@ -1570,7 +1581,7 @@ nsDragService::SourceDataGet(GtkWidget        *aWidget,
                 gint length;
                 CreateUriList(mSourceDataItems, &uriList, &length);
                 gtk_selection_data_set(aSelectionData,
-                                       aSelectionData->target,
+                                       gtk_selection_data_get_target(aSelectionData),
                                        8, (guchar *)uriList, length);
                 g_free(uriList);
                 return;
@@ -1592,16 +1603,16 @@ void nsDragService::SetDragIcon(GdkDragContext* aContext)
     if (!pc)
         return;
 
-    PRInt32 sx = mScreenX, sy = mScreenY;
+    int32_t sx = mScreenX, sy = mScreenY;
     ConvertToUnscaledDevPixels(pc, &sx, &sy);
 
-    PRInt32 offsetX = sx - dragRect.x;
-    PRInt32 offsetY = sy - dragRect.y;
+    int32_t offsetX = sx - dragRect.x;
+    int32_t offsetY = sy - dragRect.y;
 
     // If a popup is set as the drag image, use its widget. Otherwise, use
     // the surface that DrawDrag created.
     if (mDragPopup) {
-        GtkWidget* gtkWidget = nsnull;
+        GtkWidget* gtkWidget = nullptr;
         nsIFrame* frame = mDragPopup->GetPrimaryFrame();
         if (frame) {
             // DrawDrag ensured that this is a popup frame.
@@ -1750,7 +1761,7 @@ nsDragService::ScheduleLeaveEvent()
     // We don't know at this stage whether a drop signal will immediately
     // follow.  If the drop signal gets sent it will happen before we return
     // to the main loop and the scheduled leave task will be replaced.
-    if (!Schedule(eDragTaskLeave, nsnull, NULL, nsIntPoint(), 0)) {
+    if (!Schedule(eDragTaskLeave, nullptr, NULL, nsIntPoint(), 0)) {
         NS_WARNING("Drag leave after drop");
     }        
 }
@@ -1916,7 +1927,7 @@ nsDragService::RunScheduledTask()
 
         // This drag is over, so clear out our reference to the previous
         // window.
-        mTargetWindow = nsnull;
+        mTargetWindow = nullptr;
         // Make sure to end the drag session. If this drag started in a
         // different app, we won't get a drag_end signal to end it from.
         EndDragSession(true);
@@ -1997,7 +2008,7 @@ nsDragService::DispatchDropEvent()
     if (mTargetWindow->IsDestroyed())
         return FALSE;
 
-    PRUint32 msg = mCanDrop ? NS_DRAGDROP_DROP : NS_DRAGDROP_EXIT;
+    uint32_t msg = mCanDrop ? NS_DRAGDROP_DROP : NS_DRAGDROP_EXIT;
 
     mTargetWindow->DispatchDragEvent(msg, mTargetWindowPoint, mTargetTime);
 
