@@ -1,44 +1,7 @@
 /* -*- Mode: Javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-2007
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   smorrison@gte.com
- *   Terry Hayes <thayes@netscape.com>
- *   Daniel Brooks <db48x@yahoo.com>
- *   Florian QUEZE <f.qu@queze.net>
- *   Erik Fabert <jerfa@yahoo.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //******** define a js object to implement nsITreeView
 function pageInfoTreeView(copycol)
@@ -53,8 +16,8 @@ function pageInfoTreeView(copycol)
   this.tree = null;
   this.data = [ ];
   this.selection = null;
-  this.sortcol = null;
-  this.sortdir = 0;
+  this.sortcol = -1;
+  this.sortdir = false;
 }
 
 pageInfoTreeView.prototype = {
@@ -119,13 +82,51 @@ pageInfoTreeView.prototype = {
     }
   },
 
+  cycleHeader: function cycleHeader(col)
+  {
+    this.doSort(col);
+  },
+
+  doSort: function doSort(col, comparator)
+  {
+    var ascending = col.index != this.sortcol || !this.sortdir;
+    this.sortdir = ascending;
+    this.sortcol = col.index;
+
+    Array.forEach(this.tree.columns, function(treecol) {
+      treecol.element.removeAttribute("sortActive");
+      treecol.element.removeAttribute("sortDirection");
+    });
+    col.element.setAttribute("sortActive", true);
+    col.element.setAttribute("sortDirection", ascending ?
+                                              "ascending" : "descending");
+
+    var index = col.index;
+    if (!comparator) {
+      comparator = function comparator(a, b) {
+        return a[index].toLowerCase().localeCompare(b[index].toLowerCase());
+      };
+    }
+
+    this.data.sort(comparator);
+    if (!ascending)
+      this.data.reverse();
+
+    this.tree.invalidate();
+    // Note: we need to deselect before reselecting in order to trigger
+    // onselect handlers.
+    this.tree.view.selection.clearSelection();
+    this.tree.view.selection.select(0);
+    this.tree.ensureRowIsVisible(0);
+  },
+
   getRowProperties: function(row, prop) { },
   getCellProperties: function(row, column, prop) { },
   getColumnProperties: function(column, prop) { },
   isContainer: function(index) { return false; },
   isContainerOpen: function(index) { return false; },
   isSeparator: function(index) { return false; },
-  isSorted: function() { },
+  isSorted: function() { return this.sortcol > -1 },
   canDrop: function(index, orientation) { return false; },
   drop: function(row, orientation) { return false; },
   getParentIndex: function(index) { return -1; },
@@ -135,7 +136,6 @@ pageInfoTreeView.prototype = {
   getProgressMode: function(row, column) { },
   getCellValue: function(row, column) { },
   toggleOpenState: function(index) { },
-  cycleHeader: function(col) { },
   selectionChanged: function() { },
   cycleCell: function(row, column) { },
   isEditable: function(row, column) { return false; },
@@ -156,6 +156,9 @@ const COL_IMAGE_ALT     = 3;
 const COL_IMAGE_COUNT   = 4;
 const COL_IMAGE_NODE    = 5;
 const COL_IMAGE_BG      = 6;
+const COL_IMAGE_SIZENUM = 7;
+const COL_IMAGE_DEVICE  = 8;
+const COL_IMAGE_MIME    = 9;
 
 // column number to copy from, second argument to pageInfoTreeView's constructor
 const COPYCOL_NONE = -1;
@@ -172,15 +175,43 @@ var gFieldView = new pageInfoTreeView(COPYCOL_FIELD_VALUE);
 var gLinkView = new pageInfoTreeView(COPYCOL_LINK_ADDRESS);
 var gImageView = new pageInfoTreeView(COPYCOL_IMAGE);
 
-const ATOM_CONTRACTID = "@mozilla.org/atom-service;1";
-var gBrokenAtom = Components.classes[ATOM_CONTRACTID]
-                            .getService(Components.interfaces.nsIAtomService)
-                            .getAtom("broken");
+var gAtomSvc = Components.classes["@mozilla.org/atom-service;1"]
+                         .getService(Components.interfaces.nsIAtomService);
+var gBrokenAtom = gAtomSvc.getAtom("broken");
+var gLtrAtom = gAtomSvc.getAtom("ltr");
 
 gImageView.getCellProperties = function(row, col, props) {
   if (gImageView.data[row][COL_IMAGE_SIZE] == gStrings.unknown &&
       !/^https:/.test(gImageView.data[row][COL_IMAGE_ADDRESS]))
     props.AppendElement(gBrokenAtom);
+
+  if (col.id == "image-address")
+    props.AppendElement(gLtrAtom);
+};
+
+gFormView.getCellProperties = function(row, col, props) {
+  if (col.id == "form-action")
+    props.AppendElement(gLtrAtom);
+};
+
+gLinkView.getCellProperties = function(row, col, props) {
+  if (col.id == "link-address")
+    props.AppendElement(gLtrAtom);
+};
+
+gImageView.cycleHeader = function(col)
+{
+  var index = col.index;
+  var comparator;
+  switch (col.index) {
+    case COL_IMAGE_SIZE:
+      index = COL_IMAGE_SIZENUM;
+    case COL_IMAGE_COUNT:
+      comparator = function numComparator(a, b) { return a[index] - b[index]; };
+      break;
+  }
+
+  this.doSort(col, comparator);
 };
 
 var gImageHash = { };
@@ -425,6 +456,19 @@ function onClickMore()
   showTab("securityTab");
 }
 
+var cacheListener = {
+  onCacheEntryAvailable: function onCacheEntryAvailable(descriptor) {
+    if (descriptor) {
+      var pageSize = descriptor.dataSize;
+      var kbSize = Math.round(pageSize / 1024 * 100) / 100;
+      var sizeText = gBundle.getFormattedString("generalSize",
+                                                [formatNumber(kbSize),
+                                                 formatNumber(pageSize)]);
+      setItemValue("sizetext", sizeText);
+    }
+  }
+};
+
 function makeGeneralTab()
 {
   var title = (gDocument.title) ? gBundle.getFormattedString("pageTitle", [gDocument.title]) : gBundle.getString("noPageTitle");
@@ -467,26 +511,16 @@ function makeGeneralTab()
   document.getElementById("modifiedtext").value = modifiedText;
 
   // get cache info
+  setItemValue("sizetext", null);
   var cacheKey = url.replace(/#.*$/, "");
   try {
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
+    httpCacheSession.asyncOpenCacheEntry(cacheKey, ACCESS_READ, cacheListener);
   }
-  catch(ex) {
-    try {
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-    }
-    catch(ex2) { }
+  catch(ex) { }
+  try {
+    ftpCacheSession.asyncOpenCacheEntry(cacheKey, ACCESS_READ, cacheListener);
   }
-
-  var sizeText;
-  if (cacheEntryDescriptor) {
-    var pageSize = cacheEntryDescriptor.dataSize;
-    var kbSize = Math.round(pageSize / 1024 * 100) / 100;
-    sizeText = gBundle.getFormattedString("generalSize",
-                                          [formatNumber(kbSize),
-                                           formatNumber(pageSize)]);
-  }
-  setItemValue("sizetext", sizeText);
+  catch(ex) { }
 
   securityOnLoad();
 }
@@ -522,7 +556,7 @@ function processFrames()
     onProcessFrame.forEach(function(func) { func(doc); });
     var iterator = doc.createTreeWalker(doc, NodeFilter.SHOW_ELEMENT, grabAll, true);
     gFrameList.shift();
-    setTimeout(doGrab, 16, iterator);
+    setTimeout(doGrab, 10, iterator);
   }
   else
     onFinished.forEach(function(func) { func(); });
@@ -530,13 +564,13 @@ function processFrames()
 
 function doGrab(iterator)
 {
-  for (var i = 0; i < 50; ++i)
+  for (var i = 0; i < 500; ++i)
     if (!iterator.nextNode()) {
       processFrames();
       return;
     }
 
-  setTimeout(doGrab, 16, iterator);
+  setTimeout(doGrab, 10, iterator);
 }
 
 function ensureSelection(view)
@@ -547,10 +581,22 @@ function ensureSelection(view)
     view.selection.select(0);
 }
 
-function addImage(url, type, alt, elem, isBg)
+function imgCacheListener(url, type, alt, elem, isBg)
 {
-  if (!url)
-    return;
+  this.url = url;
+  this.type = type;
+  this.alt = alt;
+  this.elem = elem;
+  this.isBg = isBg;
+}
+
+imgCacheListener.prototype.onCacheEntryAvailable =
+function onCacheEntryAvailable(cacheEntryDescriptor) {
+  var url = this.url;
+  var type = this.type;
+  var alt = this.alt;
+  var elem = this.elem;
+  var isBg = this.isBg;
 
   if (!gImageHash.hasOwnProperty(url))
     gImageHash[url] = { };
@@ -558,27 +604,21 @@ function addImage(url, type, alt, elem, isBg)
     gImageHash[url][type] = { };
   if (!gImageHash[url][type].hasOwnProperty(alt)) {
     gImageHash[url][type][alt] = gImageView.data.length;
-    try {
-      // open for READ, in non-blocking mode
-      var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, ACCESS_READ, false);
-    }
-    catch(ex) {
-      try {
-        // open for READ, in non-blocking mode
-        cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, ACCESS_READ, false);
-      }
-      catch(ex2) { }
-    }
 
     var sizeText;
+    var pageSize;
+    var deviceID;
+    var mimeType;
     if (cacheEntryDescriptor) {
-      var pageSize = cacheEntryDescriptor.dataSize;
+      mimeType = getContentTypeFromHeaders(cacheEntryDescriptor);
+      deviceID = cacheEntryDescriptor.deviceID;
+      pageSize = cacheEntryDescriptor.dataSize;
       var kbSize = Math.round(pageSize / 1024 * 100) / 100;
       sizeText = gBundle.getFormattedString("mediaFileSize", [formatNumber(kbSize)]);
     }
     else
       sizeText = gStrings.unknown;
-    gImageView.addRow([url, type, sizeText, alt, 1, elem, isBg]);
+    gImageView.addRow([url, type, sizeText, alt, 1, elem, isBg, pageSize, deviceID, mimeType]);
 
     // Add the observer, only once.
     if (gImageView.data.length == 1) {
@@ -591,6 +631,15 @@ function addImage(url, type, alt, elem, isBg)
   }
 }
 
+function addImage(url, type, alt, elem, isBg)
+{
+  if (url) try {
+    var listener = new imgCacheListener(url, type, alt, elem, isBg);
+    httpCacheSession.asyncOpenCacheEntry(url, ACCESS_READ, listener);
+  }
+  catch (ex) { }
+}
+
 function grabAll(elem)
 {
   // check for background images, any node may have multiple
@@ -609,7 +658,7 @@ function grabAll(elem)
     addImage(elem.src, gStrings.mediaImg,
              (elem.hasAttribute("alt")) ? elem.alt : gStrings.notSet, elem, false);
   else if (elem instanceof HTMLAreaElement)
-    gLinkView.addRow([elem.alt, elem.href, gStrings.linkArea, elem.target]);
+    gLinkView.addRow([elem.alt, elem.href, gStrings.linkArea, elem.target, ""]);
   else if (elem instanceof HTMLVideoElement)
     addImage(elem.currentSrc, gStrings.mediaVideo, "", elem, false);
   else if (elem instanceof HTMLAudioElement)
@@ -622,12 +671,12 @@ function grabAll(elem)
       if (/(?:^|\s)icon(?:\s|$)/i.test(rel))
         addImage(elem.href, gStrings.mediaLink, "", elem, false);
       else if (/(?:^|\s)stylesheet(?:\s|$)/i.test(rel))
-        gLinkView.addRow([elem.rel, elem.href, gStrings.linkStylesheet, elem.target]);
+        gLinkView.addRow([elem.rel, elem.href, gStrings.linkStylesheet, elem.target, ""]);
       else
-        gLinkView.addRow([elem.rel, elem.href, gStrings.linkRel, elem.target]);
+        gLinkView.addRow([elem.rel, elem.href, gStrings.linkRel, elem.target, ""]);
     }
     else
-      gLinkView.addRow([elem.rev, elem.href, gStrings.linkRev, elem.target]);
+      gLinkView.addRow([elem.rev, elem.href, gStrings.linkRev, elem.target, ""]);
   }
   else if (elem instanceof HTMLInputElement || elem instanceof HTMLButtonElement)
   {
@@ -640,11 +689,11 @@ function grabAll(elem)
         if ("form" in elem && elem.form)
         {
           gLinkView.addRow([elem.value || getValueText(elem) || gStrings.linkSubmit,
-                            elem.form.action, gStrings.linkSubmission, elem.form.target]);
+                            elem.form.action, gStrings.linkSubmission, elem.form.target, ""]);
         }
         else
-          gLinkView.addRow([elem.value || getValueText(elem) || gStrings.linkSubmit, '',
-                            gStrings.linkSubmission, '']);
+          gLinkView.addRow([elem.value || getValueText(elem) || gStrings.linkSubmit, "",
+                            gStrings.linkSubmission, "", ""]);
     }
   }
   else if (elem instanceof HTMLFormElement)
@@ -659,7 +708,7 @@ function grabAll(elem)
     try {
       url = makeURLAbsolute(elem.baseURI, url, elem.ownerDocument.characterSet);
     } catch (e) {}
-    gLinkView.addRow([getValueText(elem), url, gStrings.linkX, ""]);
+    gLinkView.addRow([getValueText(elem), url, gStrings.linkX, "", ""]);
   }
   else if (elem.hasAttributeNS(XLinkNS, "href"))
   {
@@ -671,10 +720,12 @@ function grabAll(elem)
     if (elem instanceof SVGImageElement)
       addImage(url, gStrings.mediaImg, "", elem, false);
     else
-      gLinkView.addRow([getValueText(elem), url, gStrings.linkX, ""]);
+      gLinkView.addRow([getValueText(elem), url, gStrings.linkX, "", ""]);
   }
   else if (elem instanceof HTMLScriptElement)
-    gLinkView.addRow([elem.type || elem.language, elem.src || gStrings.linkScriptInline, gStrings.linkScript]);
+    gLinkView.addRow([elem.type || elem.getAttribute("language") || gStrings.notSet,
+                      elem.src || gStrings.linkScriptInline,
+                      gStrings.linkScript, "", "", ""]);
 
   onProcessElement.forEach(function(func) { func(elem); });
 
@@ -840,8 +891,17 @@ function saveMedia()
     var item = getSelectedImage(tree);
     var url = gImageView.data[tree.currentIndex][COL_IMAGE_ADDRESS];
 
-    if (url)
-      saveURL(url, null, "SaveImageTitle", false, true, makeURI(item.baseURI));
+    if (url) {
+      let titleKey = "SaveImageTitle";
+
+      if (item instanceof HTMLVideoElement)
+        titleKey = "SaveVideoTitle";
+      else if (item instanceof HTMLAudioElement)
+        titleKey = "SaveAudioTitle";
+
+      saveURL(url, null, titleKey, false, true, makeURI(item.baseURI),
+              gDocument);
+    }
   }
   else {
     var odir  = selectSaveFolder();
@@ -858,7 +918,7 @@ function saveMedia()
 
     var saveAnImage = function(aURIString, aChosenData, aBaseURI) {
       internalSave(aURIString, null, null, null, null, false, "SaveImageTitle",
-                   aChosenData, aBaseURI);
+                   aChosenData, aBaseURI, gDocument);
     }
 
     for (var i = 0; i < rowArray.length; i++) {
@@ -867,7 +927,7 @@ function saveMedia()
       var item = gImageView.data[v][COL_IMAGE_NODE];
       var uriString = gImageView.data[v][COL_IMAGE_ADDRESS];
       var uri = makeURI(uriString);
- 
+
       try {
         uri.QueryInterface(Components.interfaces.nsIURL);
         dir.append(decodeURIComponent(uri.fileName));
@@ -933,12 +993,7 @@ function onImageSelect()
 
 function makePreview(row)
 {
-  var imageTree = document.getElementById("imagetree");
-  var item = getSelectedImage(imageTree);
-  var col = imageTree.columns["image-address"];
-  var url = gImageView.getCellText(row, col);
-  // image-bg
-  var isBG = gImageView.data[row][COL_IMAGE_BG];
+  var [url, type, sizeText, alt, count, item, isBG, pageSize, deviceID, cachedType] = gImageView.data[row];
   var isAudio = false;
 
   setItemValue("imageurltext", url);
@@ -966,48 +1021,23 @@ function makePreview(row)
 
   // get cache info
   var sourceText = gBundle.getString("generalNotCached");
-  var cacheKey = url.replace(/#.*$/, "");
-  try {
-    // open for READ, in non-blocking mode
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-    if (cacheEntryDescriptor)
-      switch (cacheEntryDescriptor.deviceID) {
-        case "disk":
-          sourceText = gBundle.getString("generalDiskCache");
-          break;
-        case "memory":
-          sourceText = gBundle.getString("generalMemoryCache");
-          break;
-        default:
-          sourceText = cacheEntryDescriptor.deviceID;
-          break;
-      }
-  }
-  catch(ex) {
-    try {
-      // open for READ, in non-blocking mode
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-      if (cacheEntryDescriptor)
-        switch (cacheEntryDescriptor.deviceID) {
-          case "disk":
-            sourceText = gBundle.getString("generalDiskCache");
-            break;
-          case "memory":
-            sourceText = gBundle.getString("generalMemoryCache");
-            break;
-          default:
-            sourceText = cacheEntryDescriptor.deviceID;
-            break;
-        }
+  if (deviceID) {
+    switch (deviceID) {
+      case "disk":
+        sourceText = gBundle.getString("generalDiskCache");
+        break;
+      case "memory":
+        sourceText = gBundle.getString("generalMemoryCache");
+        break;
+      default:
+        sourceText = cacheEntryDescriptor.deviceID;
+        break;
     }
-    catch(ex2) { }
   }
   setItemValue("imagesourcetext", sourceText);
 
   // find out the file size
-  var sizeText;
-  if (cacheEntryDescriptor) {
-    var pageSize = cacheEntryDescriptor.dataSize;
+  if (pageSize) {
     var kbSize = Math.round(pageSize / 1024 * 100) / 100;
     sizeText = gBundle.getFormattedString("generalSize",
                                           [formatNumber(kbSize),
@@ -1016,15 +1046,25 @@ function makePreview(row)
   setItemValue("imagesizetext", sizeText);
 
   var mimeType;
-  var numFrames = 0;
-  if (item instanceof HTMLObjectElement ||
-      item instanceof HTMLEmbedElement ||
-      item instanceof HTMLLinkElement)
-    mimeType = item.type;
-  if (!mimeType && item instanceof nsIImageLoadingContent)
-    [mimeType, numFrames] = getContentTypeFromImgRequest(item);
+  var typeString = "mediaImageType";
+  if (!isBG) {
+    if (item instanceof nsIImageLoadingContent) {
+      var imageRequest = item.getRequest(nsIImageLoadingContent.CURRENT_REQUEST);
+      if (imageRequest) {
+        mimeType = imageRequest.mimeType;
+        if (imageRequest.imageStatus & imageRequest.STATUS_DECODE_COMPLETE &&
+            imageRequest.image.animated)
+          typeString = "mediaAnimatedType";
+      }
+    }
+    if (!mimeType &&
+        (item instanceof HTMLObjectElement ||
+         item instanceof HTMLEmbedElement ||
+         item instanceof HTMLLinkElement))
+      mimeType = item.type;
+  }
   if (!mimeType)
-    mimeType = getContentTypeFromHeaders(cacheEntryDescriptor);
+    mimeType = cachedType;
 
   // if we have a data url, get the MIME type from the url
   if (!mimeType) {
@@ -1039,11 +1079,7 @@ function makePreview(row)
     let imageMimeType = /^image\/(.*)/.exec(mimeType);
     if (imageMimeType) {
       imageType = imageMimeType[1].toUpperCase();
-      if (numFrames > 1)
-        imageType = gBundle.getFormattedString("mediaAnimatedImageType",
-                                               [imageType, numFrames]);
-      else
-        imageType = gBundle.getFormattedString("mediaImageType", [imageType]);
+      imageType = gBundle.getFormattedString(typeString, [imageType]);
     }
     else {
       // the MIME type doesn't begin with image/, display the raw type
@@ -1205,21 +1241,6 @@ function getContentTypeFromHeaders(cacheEntryDescriptor)
           .exec(cacheEntryDescriptor.getMetaDataElement("response-head")))[1];
 }
 
-function getContentTypeFromImgRequest(item)
-{
-  var httpRequest;
-  var numFrames = 0;
-
-  var imageRequest = item.getRequest(nsIImageLoadingContent.CURRENT_REQUEST);
-  if (imageRequest) {
-    httpRequest = imageRequest.mimeType;
-    let image = imageRequest.image;
-    if (image)
-      numFrames = image.numFrames;
-  }
-  return [httpRequest, numFrames];
-}
-
 //******** Other Misc Stuff
 // Modified from the Links Panel v2.3, http://segment7.net/mozilla/links/links.html
 // parse a node to extract the contents of the node
@@ -1311,35 +1332,53 @@ function formatDate(datestr, unknown)
                                     date.getHours(), date.getMinutes(), date.getSeconds());
 }
 
-function doCopy()
+function getSelectedItems(linksMode)
+{
+  // linksMode is a boolean that is used to determine 
+  // whether the getSelectedItems() function needs to
+  // run with urlSecurityCheck() or not.  
+
+  var elem = document.commandDispatcher.focusedElement;
+
+  var view = elem.view;
+  var selection = view.selection;
+  var text = [], tmp = '';
+  var min = {}, max = {};
+
+  var count = selection.getRangeCount();
+
+  for (var i = 0; i < count; i++) {
+    selection.getRangeAt(i, min, max);
+
+    for (var row = min.value; row <= max.value; row++) {
+      view.performActionOnRow("copy", row);
+
+      tmp = elem.getAttribute("copybuffer");
+      if (tmp)
+      {
+        try {
+          if (linksMode)
+            urlSecurityCheck(tmp, gDocument.nodePrincipal);
+          text.push(tmp);
+        }
+        catch (e) {
+        }
+      }
+      elem.removeAttribute("copybuffer");
+    }
+  }
+  
+  return text;
+}
+
+function doCopy(isLinkMode)
 {
   if (!gClipboardHelper)
     return;
 
-  var elem = document.commandDispatcher.focusedElement;
+  var text = getSelectedItems(isLinkMode);
 
-  if (elem && "treeBoxObject" in elem) {
-    var view = elem.view;
-    var selection = view.selection;
-    var text = [], tmp = '';
-    var min = {}, max = {};
-
-    var count = selection.getRangeCount();
-
-    for (var i = 0; i < count; i++) {
-      selection.getRangeAt(i, min, max);
-
-      for (var row = min.value; row <= max.value; row++) {
-        view.performActionOnRow("copy", row);
-
-        tmp = elem.getAttribute("copybuffer");
-        if (tmp)
-          text.push(tmp);
-        elem.removeAttribute("copybuffer");
-      }
-    }
-    gClipboardHelper.copyString(text.join("\n"));
-  }
+  gClipboardHelper.copyString(text.join("\n"), gDocument);
 }
 
 function doSelectAll()
@@ -1349,3 +1388,11 @@ function doSelectAll()
   if (elem && "treeBoxObject" in elem)
     elem.view.selection.selectAll();
 }
+
+function onOpenIn(mode)
+{
+  var linkList = getSelectedItems(true);
+
+  if (linkList.length)
+    openUILinkArrayIn(linkList, mode);
+}
