@@ -45,7 +45,7 @@ nsContextMenu.prototype = {
     this.isTextSelected = this.isTextSelection();
     this.isContentSelected = this.isContentSelection();
 
-    this.initPopupURL();
+    this.initPopupPrincipal();
 
     // Initialize (disable/remove) menu items.
     this.initItems();
@@ -158,18 +158,25 @@ nsContextMenu.prototype = {
     this.showItem("context-sep-properties",
                   !(this.inDirList || this.isContentSelected || this.onTextInput ||
                     this.onCanvas || this.onVideo || this.onAudio));
-    // Set As Wallpaper depends on whether an image was clicked on,
+    // Set Desktop Background depends on whether an image was clicked on,
     // and requires the shell service.
-    var hasShell = "@mozilla.org/suite/shell-service;1" in Components.classes;
-    this.showItem("context-setWallpaper",
-                  hasShell && (this.onLoadedImage || this.onStandaloneImage));
+    var canSetDesktopBackground = false;
+    if ("@mozilla.org/suite/shell-service;1" in Components.classes) try {
+      canSetDesktopBackground =
+          Components.classes["@mozilla.org/suite/shell-service;1"]
+                    .getService(Components.interfaces.nsIShellService)
+                    .canSetDesktopBackground;
+    } catch (e) {
+    }
+    this.showItem("context-setDesktopBackground",
+                  canSetDesktopBackground && (this.onLoadedImage || this.onStandaloneImage));
 
     this.showItem("context-sep-image",
                   this.onLoadedImage || this.onStandaloneImage);
 
-    if (hasShell && this.onLoadedImage)
-      // Disable the Set As Wallpaper menu item if we're still trying to load the image
-      this.setItemAttr("context-setWallpaper", "disabled",
+    if (canSetDesktopBackground && this.onLoadedImage)
+      // Disable the Set Desktop Background menu item if we're still trying to load the image
+      this.setItemAttr("context-setDesktopBackground", "disabled",
                        (("complete" in this.target) && !this.target.complete) ? "true" : null);
 
     this.showItem("context-fitimage", this.onStandaloneImage &&
@@ -246,18 +253,18 @@ nsContextMenu.prototype = {
                      this.autoDownload ? "valueSave" : "valueSaveAs");
 
     var blocking = true;
-      if (this.popupURL)
+      if (this.popupPrincipal)
         try {
           const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
                      .getService(Components.interfaces.nsIPopupWindowManager);
-          blocking = PM.testPermission(this.popupURL) ==
-                     Components.interfaces.nsIPopupWindowManager.DENY_POPUP;
+          blocking = PM.testPermission(this.popupPrincipal) == PM.DENY_POPUP;
         } catch (e) {
+          Components.utils.reportError(e);
         }
 
-    this.showItem("popupwindow-reject", this.popupURL && !blocking);
-    this.showItem("popupwindow-allow", this.popupURL && blocking);
-    this.showItem("context-sep-popup", this.popupURL);
+    this.showItem("popupwindow-reject", this.popupPrincipal && !blocking);
+    this.showItem("popupwindow-allow", this.popupPrincipal && blocking);
+    this.showItem("context-sep-popup", this.popupPrincipal);
 
     // BiDi UI
     this.showItem("context-sep-bidi", gShowBiDi);
@@ -429,7 +436,7 @@ nsContextMenu.prototype = {
     this.inSyntheticDoc        = false;
     this.hasBGImage            = false;
     this.bgImageURL            = "";
-    this.popupURL              = null;
+    this.popupPrincipal        = null;
     this.autoDownload          = false;
     this.isTextSelected        = false;
     this.isContentSelected     = false;
@@ -438,7 +445,7 @@ nsContextMenu.prototype = {
     // Remember the node that was clicked.
     this.target = aNode;
 
-    if (aNode.namespaceURI == xulNS || this.isTargetAFormControl(aNode)) {
+    if (aNode.namespaceURI == xulNS) {
       this.shouldDisplay = false;
       return;
     }
@@ -491,7 +498,15 @@ nsContextMenu.prototype = {
         this.onCanvas = true;
       }
       else if (this.target instanceof HTMLVideoElement) {
-        this.onVideo = true;
+        // Gecko always creates a HTMLVideoElement when loading an ogg file
+        // directly. If the media is actually audio, be smarter and provide
+        // a context menu with audio operations.
+        if (this.target.readyState >= this.target.HAVE_METADATA &&
+            (this.target.videoWidth == 0 || this.target.videoHeight == 0))
+          this.onAudio = true;
+        else
+          this.onVideo = true;
+
         this.mediaURL = this.target.currentSrc || this.target.src;
       }
       else if (this.target instanceof HTMLAudioElement) {
@@ -501,7 +516,7 @@ nsContextMenu.prototype = {
       else if (this.target instanceof HTMLInputElement) {
         this.onTextInput = this.isTargetATextBox(this.target);
         // allow spellchecking UI on all writable text boxes except passwords
-        if (!this.target.readOnly && !this.target.disabled &&
+        if (this.onTextInput && !this.target.readOnly &&
             this.target.mozIsTextField(true)) {
           this.possibleSpellChecking = true;
           InlineSpellCheckerUI.init(this.target.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor);
@@ -510,8 +525,8 @@ nsContextMenu.prototype = {
         this.onKeywordField = this.isTargetAKeywordField(this.target);
       }
       else if (this.target instanceof HTMLTextAreaElement) {
-        this.onTextInput = true;
-        if (!this.target.readOnly && !this.target.disabled) {
+        this.onTextInput = this.isTextBoxEnabled(this.target);
+        if (this.onTextInput && !this.target.readOnly) {
           this.possibleSpellChecking = true;
           InlineSpellCheckerUI.init(this.target.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor);
           InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
@@ -643,7 +658,7 @@ nsContextMenu.prototype = {
     }
   },
 
-  initPopupURL: function() {
+  initPopupPrincipal: function() {
     // quick check: if no opener, it can't be a popup
     if (!window.content.opener)
       return;
@@ -661,13 +676,11 @@ nsContextMenu.prototype = {
       if (xulwin.contextFlags &
           CI.nsIWindowCreator2.PARENT_IS_LOADING_OR_RUNNING_TIMEOUT) {
         // do the pref settings allow site-by-site popup management?
-        const PB = Components.classes["@mozilla.org/preferences-service;1"]
-                   .getService(CI.nsIPrefBranch);
-        show = !PB.getBoolPref("dom.disable_open_during_load");
+        show = !Services.prefs.getBoolPref("dom.disable_open_during_load");
       }
       if (show) {
-        // initialize popupURL
-        this.popupURL = Services.io.newURI(window.content.opener.location.href, null, null);
+        // initialize popupPrincipal
+        this.popupPrincipal = window.content.opener.document.nodePrincipal;
       }
     } catch(e) {
     }
@@ -702,18 +715,17 @@ nsContextMenu.prototype = {
 
   // Block popup windows
   rejectPopupWindows: function(aAndClose) {
-    const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
-               .getService(Components.interfaces.nsIPopupWindowManager);
-    PM.add(this.popupURL, false);
+    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
+                                    Services.perms.DENY_ACTION);
     if (aAndClose)
-      Services.obs.notifyObservers(window, "popup-perm-close", this.popupURL.spec);
+      Services.obs.notifyObservers(window, "popup-perm-close",
+                                   this.popupPrincipal.URI.spec);
   },
 
   // Unblock popup windows
   allowPopupWindows: function() {
-    const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
-               .getService(Components.interfaces.nsIPopupWindowManager);
-    PM.add(this.popupURL, true);
+    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
+                                    Services.perms.ALLOW_ACTION);
   },
 
   // Block/Unblock image from loading in the future.
@@ -839,7 +851,7 @@ nsContextMenu.prototype = {
       let uri = makeURI(this.mediaURL);
       let url = uri.QueryInterface(Components.interfaces.nsIURL);
       if (url.fileBaseName)
-        name = url.fileBaseName + ".jpg";
+        name = decodeURI(url.fileBaseName) + ".jpg";
     } catch (e) { }
     var video = this.target;
     var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
@@ -849,7 +861,8 @@ nsContextMenu.prototype = {
     ctxDraw.drawImage(video, 0, 0);
     saveImageURL(canvas.toDataURL("image/jpeg", ""), name, "SaveImageTitle",
                                   true, true,
-                                  this.target.ownerDocument.documentURIObject);
+                                  this.target.ownerDocument.documentURIObject,
+                                  this.target.ownerDocument);
   },
 
   // Full screen video playback
@@ -873,28 +886,9 @@ nsContextMenu.prototype = {
       openUILinkIn(this.bgImageURL, where, null, null, doc.documentURIObject);
   },
 
-  setWallpaper: function() {
-    // Confirm since it's annoying if you hit this accidentally.
-    var navigatorBundle = document.getElementById("bundle_navigator");
-    var promptTitle = navigatorBundle.getString("wallpaperConfirmTitle");
-    var promptMsg = navigatorBundle.getString("wallpaperConfirmMsg");
-    var promptConfirmButton = navigatorBundle.getString("wallpaperConfirmButton");
-
-    if (Services.prompt.confirmEx(window, promptTitle, promptMsg,
-                                  (Services.prompt.BUTTON_TITLE_IS_STRING *
-                                   Services.prompt.BUTTON_POS_0) +
-                                  (Services.prompt.BUTTON_TITLE_CANCEL *
-                                   Services.prompt.BUTTON_POS_1),
-                                  promptConfirmButton, null, null, null,
-                                  {value: false}) != 0)
-      return;
-
-    const nsIShellService = Components.interfaces.nsIShellService;
-
-    Components.classes["@mozilla.org/suite/shell-service;1"]
-              .getService(nsIShellService)
-              .setDesktopBackground(this.target,
-                                    nsIShellService.BACKGROUND_STRETCH);
+  setDesktopBackground: function() {
+    openDialog("chrome://communicator/content/setDesktopBackground.xul",
+               "_blank", "chrome,modal,titlebar,centerscreen", this.target);
   },
 
   // Save URL of clicked-on frame.
@@ -959,7 +953,7 @@ nsContextMenu.prototype = {
         if (aStatusCode == NS_ERROR_SAVE_LINK_AS_TIMEOUT) {
           // Do it the old fashioned way, which will pick the best filename
           // it can without waiting.
-          saveURL(linkURL, linkText, dialogTitle, bypassCache, true, doc.documentURIObject);
+          saveURL(linkURL, linkText, dialogTitle, bypassCache, true, doc.documentURIObject, doc);
         }
         if (this.extListener)
           this.extListener.onStopRequest(aRequest, aContext, aStatusCode);
@@ -1028,21 +1022,21 @@ nsContextMenu.prototype = {
 
   // Save URL of clicked-on image, video, or audio.
   saveMedia: function() {
+    var doc = this.target.ownerDocument;
     if (this.onCanvas)
       // Bypass cache, since it's a data: URL.
       saveImageURL(this.target.toDataURL(), "canvas.png", "SaveImageTitle",
-                   true, true, null);
+                   true, true, null, doc);
     else if (this.onImage)
       saveImageURL(this.mediaURL, null, "SaveImageTitle", false, true,
-                   this.target.ownerDocument.documentURIObject);
+                   doc.documentURIObject, doc);
     else if (this.onVideo || this.onAudio) {
       var dialogTitle = this.onVideo ? "SaveVideoTitle" : "SaveAudioTitle";
-      this.saveHelper(this.mediaURL, null, dialogTitle, false,
-                      this.target.ownerDocument);
+      this.saveHelper(this.mediaURL, null, dialogTitle, false, doc);
     }
   },
 
-  // Backwards-compatability wrapper
+  // Backwards-compatibility wrapper
   saveImage: function() {
     if (this.onCanvas || this.onImage)
       this.saveMedia();
@@ -1072,7 +1066,7 @@ nsContextMenu.prototype = {
   copyEmail: function() {
     var clipboard = this.getService("@mozilla.org/widget/clipboardhelper;1",
                                     Components.interfaces.nsIClipboardHelper);
-    clipboard.copyString(this.getEmail());
+    clipboard.copyString(this.getEmail(), this.target.ownerDocument);
   },
 
   bookmarkThisPage : function() {
@@ -1321,23 +1315,18 @@ nsContextMenu.prototype = {
            "contextMenu.hasBGImage = " + this.hasBGImage + "\n";
   },
 
-  // Returns true if aNode is a form control (except text boxes and images).
-  // This is used to disable the context menu for form controls.
-  isTargetAFormControl: function(aNode) {
-    if (aNode instanceof HTMLInputElement)
-      return (!aNode.mozIsTextField(false) && aNode.type != "image");
-
-    return (aNode instanceof HTMLButtonElement) ||
-           (aNode instanceof HTMLSelectElement) ||
-           (aNode instanceof HTMLOptionElement) ||
-           (aNode instanceof HTMLOptGroupElement);
+  isTextBoxEnabled: function(aNode) {
+    return !aNode.ownerDocument.defaultView
+                 .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                 .getInterface(Components.interfaces.nsIDOMWindowUtils)
+                 .isNodeDisabledForEvents(aNode);
   },
 
   isTargetATextBox: function(aNode) {
     if (aNode instanceof HTMLInputElement)
-      return aNode.mozIsTextField(false);
+      return aNode.mozIsTextField(false) && this.isTextBoxEnabled(aNode);
 
-    return (aNode instanceof HTMLTextAreaElement);
+    return aNode instanceof HTMLTextAreaElement && this.isTextBoxEnabled(aNode);
   },
 
   isTargetAKeywordField: function(aNode) {
@@ -1391,14 +1380,12 @@ nsContextMenu.prototype = {
         media.setAttribute("controls", "true");
         break;
       case "showstats":
-        var event = document.createEvent("CustomEvent");
-        event.initCustomEvent("media-showStatistics", false, true, true);
-        media.dispatchEvent(event);
+        media.dispatchEvent(new CustomEvent("media-showStatistics",
+          { bubbles: false, cancelable: true, detail: true }));
         break;
       case "hidestats":
-        var event = document.createEvent("CustomEvent");
-        event.initCustomEvent("media-showStatistics", false, true, false);
-        media.dispatchEvent(event);
+        media.dispatchEvent(new CustomEvent("media-showStatistics",
+          { bubbles: false, cancelable: true, detail: false }));
         break;
     }
   },
@@ -1406,7 +1393,7 @@ nsContextMenu.prototype = {
   copyMediaLocation: function() {
     var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                     .getService(Components.interfaces.nsIClipboardHelper);
-    clipboard.copyString(this.mediaURL);
+    clipboard.copyString(this.mediaURL, this.target.ownerDocument);
   },
 
   get imageURL() {
