@@ -112,8 +112,9 @@ var DefaultController =
       case "cmd_archive":
 			case "cmd_reply":
 			case "button_reply":
-			case "cmd_replySender":
+			case "cmd_replyList":
 			case "cmd_replyGroup":
+			case "cmd_replySender":
 			case "cmd_replyall":
 			case "button_replyall":
       case "cmd_replySenderAndGroup":
@@ -208,7 +209,7 @@ var DefaultController =
       case "cmd_killThread":
       case "cmd_killSubthread":
       case "cmd_cancel":
-        return(isNewsURI(GetFirstSelectedMessage()));
+        return gFolderDisplay.selectedMessageIsNews;
 
 			default:
 				return false;
@@ -260,13 +261,15 @@ var DefaultController =
       case "cmd_saveAsFile":
         return GetNumSelectedMessages() > 0;
       case "cmd_saveAsTemplate":
-        var target = gMessageBrowser.contentPrincipal.URI.scheme;
+        var msgFolder = GetSelectedMsgFolders();
+        var target = msgFolder[0].server.localStoreType;
         if (GetNumSelectedMessages() > 1 || target == "news")
           return false;   // else fall thru
       case "cmd_reply":
       case "button_reply":
-      case "cmd_replySender":
+      case "cmd_replyList":
       case "cmd_replyGroup":
+      case "cmd_replySender":
       case "cmd_replyall":
       case "button_replyall":
       case "cmd_replySenderAndGroup":
@@ -337,7 +340,7 @@ var DefaultController =
       case "cmd_displayMsgFilters":
         let mgr = Components.classes["@mozilla.org/messenger/account-manager;1"]
                             .getService(Components.interfaces.nsIMsgAccountManager);
-        return mgr.accounts.Count() > 0;
+        return mgr.accounts.length > 0;
       case "cmd_applyFilters":
         if (gDBView)
           gDBView.getCommandStatus(nsMsgViewCommandType.applyFilters, enabled, checkStatus);
@@ -471,12 +474,15 @@ var DefaultController =
 			case "cmd_reply":
 				MsgReplyMessage(null);
 				break;
-			case "cmd_replySender":
-				MsgReplySender(null);
+			case "cmd_replyList":
+				MsgReplyList(null);
 				break;
 			case "cmd_replyGroup":
 				MsgReplyGroup(null);
 				break;
+			case "cmd_replySender":
+				MsgReplySender(null);
+				break;
 			case "cmd_replyall":
 				MsgReplyToAllMessage(null);
 				break;
@@ -1044,12 +1050,6 @@ function SwitchPaneFocus(event)
   }
 }
 
-function SetFocusFolderPane()
-{
-    var folderTree = GetFolderTree();
-    folderTree.focus();
-}
-
 function SetFocusThreadPane()
 {
     var threadTree = GetThreadTree();
@@ -1064,11 +1064,6 @@ function SetFocusMessagePane()
     GetMessagePaneFrame().focus();
 }
 
-function is_collapsed(element) 
-{
-  return (element.getAttribute('state') == 'collapsed');
-}
-
 function isCommandEnabled(cmd)
 {
   var selectedFolders = GetSelectedMsgFolders();
@@ -1097,9 +1092,10 @@ function CanRenameDeleteJunkMail(aFolderUri)
   {
     var allServers = accountManager.allServers;
 
-    for (var i=0;i<allServers.Count();i++)
+    for (var i = 0; i < allServers.length; i++)
     {
-      var currentServer = allServers.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
+      var currentServer =
+        allServers.queryElementAt(i, Components.interfaces.nsIMsgIncomingServer);
       var settings = currentServer.spamSettings;
       // If junk mail control or move junk mail to folder option is disabled then
       // allow the folder to be removed/renamed since the folder is not used in this case.
