@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ian Neal <iann_bugzilla@blueyonder.co.uk>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* This is where functions related to the standalone message window are kept */
 
@@ -461,26 +427,11 @@ function GetSelectedMsgFolders()
 	return folderArray;
 }
 
-function GetFirstSelectedMessage()
-{
-	return GetLoadedMessage();
-}
-
 function GetNumSelectedMessages()
 {
   return (gCurrentMessageUri) ? 1 : 0;
 }
 
-function GetSelectedMessages()
-{
-	var messageArray = new Array(1);
-	var message = GetLoadedMessage();
-	if (message)
-		messageArray[0] = message;	
-
-	return messageArray;
-}
-
 function GetSelectedIndices(dbView)
 {
   try {
@@ -652,8 +603,9 @@ var MessageWindowController =
         return GetNumSelectedMessages() > 0;
       case "cmd_reply":
       case "button_reply":
-      case "cmd_replySender":
+      case "cmd_replyList":
       case "cmd_replyGroup":
+      case "cmd_replySender":
       case "cmd_replyall":
       case "cmd_replySenderAndGroup":
       case "cmd_replyAllRecipients":
@@ -729,8 +681,9 @@ var MessageWindowController =
         return enabled.value;
 			case "cmd_reply":
 			case "button_reply":
-			case "cmd_replySender":
+			case "cmd_replyList":
 			case "cmd_replyGroup":
+			case "cmd_replySender":
 			case "cmd_replyall":
 			case "button_replyall":
       case "cmd_replySenderAndGroup":
@@ -847,12 +800,15 @@ var MessageWindowController =
 			case "cmd_reply":
 				MsgReplyMessage(null);
 				break;
-			case "cmd_replySender":
-				MsgReplySender(null);
+			case "cmd_replyList":
+				MsgReplyList(null);
 				break;
 			case "cmd_replyGroup":
 				MsgReplyGroup(null);
 				break;
+			case "cmd_replySender":
+				MsgReplySender(null);
+				break;
 			case "cmd_replyall":
 				MsgReplyToAllMessage(null);
 				break;
