@@ -399,7 +399,7 @@ var gMailNewsTabsType =
     // If we have an account, we also always have a "Local Folders" account,
     let accountCount = Components.classes["@mozilla.org/messenger/account-manager;1"]
                                  .getService(Components.interfaces.nsIMsgAccountManager)
-                                 .accounts.Count();
+                                 .accounts.length;
     let multipleRealAccounts = accountCount > 2;
 
     // clear out specific tab data now, because we might need to return early
@@ -836,11 +836,9 @@ function ChangeMessagePaneVisibility()
     gDBView.loadMessageByUrl("about:blank");
     gDBView.selectionChanged();
   }
-  var event = document.createEvent("Events");
-  if (hidden)
-    event.initEvent("messagepane-hide", false, true);
-  else
-    event.initEvent("messagepane-unhide", false, true);
+
+  var event = new Event( "messagepane-"+ (hidden ? "hide" : "unhide"),
+                          { bubbles: false, cancelable: true });
   document.getElementById("messengerWindow").dispatchEvent(event);
 }
 
